//=================================================
//
//	DemoAnalysis.cc
//         geometry: under construction
//
//         Version 0.6.0	15-Aug-2000	T.Matuo
//	   Version 0.6.1	 8-Sep-2000	Y. Shimizu
//
//=================================================

#include "DemoAnalysis.hh"
#include <system/KonoeFDClient.hh>
#include <analysis/KonoeDataRecord.hh>
#include <system/KonoeOutputObjFile.hh>
#include <system/KonoeNameSpaceTable.hh>
#include <system/KonoeNameSpaceValue.hh>
#include <system/KonoeUDPClientSocket.hh>
#include <system/KonoeNetworkNameSpaceManager.hh>

DemoAnalysis::DemoAnalysis(KChar* master)
{

   masterhost = master;
  /**************** handler*********************************/

//  handler = new KonoeSignalHandler( SIGINT , this );
//  handler->enable( );

  /**************** socket *********************************/
  d_cm	= new KonoeDataRecordCM();
  
  daq_server	= new KonoeServerSocket( DAQ_PORT );
  daq_socket	= new KonoeInputObjSocket( daq_server );
  daq_socket->installCM(d_cm);
 
  ui_server     = new KonoeServerSocket( 10916 );
  ui_socket     = new KonoeOutputObjSocket( ui_server );
  /*************** file ***********************************/
  ofile00 = new KonoeOutputObjFile("GENERAL_HISTOdat");
  ofile01 = new KonoeOutputObjFile("CSI_TDC_no1.dat");
  ofile02 = new KonoeOutputObjFile("CSI_TDC_no2.dat");
  ofile03 = new KonoeOutputObjFile("CSI_TDC_no3.dat");
  ofile04 = new KonoeOutputObjFile("CSI_TDC_no4.dat");
  ofile11 = new KonoeOutputObjFile("CSI_ADC_no1.dat");
  ofile12 = new KonoeOutputObjFile("CSI_ADC_no2.dat");

  /************** analyzer setting***************************/
  analyzer = new KonoeDataAnalyzer( N_SECTION );
  general_analyzer	= new KonoeDefaultSectionAnalyzer( GENERAL_SECTION ,
							   N_MULTIPLET, 0 );
  csi_tdc_analyzer 	= new KonoeDefaultSectionAnalyzer( CSI_TDC_SECTION , 
							   N_MULTIPLET, 0);
  csi_adc_analyzer	= new KonoeDefaultSectionAnalyzer( CSI_ADC_SECTION ,
							   N_MULTIPLET, 0);
  s_tdc_analyzer        = new KonoeDefaultSectionAnalyzer( S_TDC_SECTION ,
							   N_MULTIPLET, 0);
  s_adc_analyzer	= new KonoeDefaultSectionAnalyzer( S_ADC_SECTION ,
							   N_MULTIPLET, 0 );
  intreg_analyzer	= new KonoeDefaultSectionAnalyzer( REG_SECTION ,
							   N_MULTIPLET, 0 );
  
  analyzer -> install( general_analyzer );
  analyzer -> install( csi_tdc_analyzer );
  analyzer -> install( csi_adc_analyzer );
  analyzer -> install( s_tdc_analyzer );
  analyzer -> install( s_adc_analyzer );
  analyzer -> install( intreg_analyzer );
	
  /************** data extract      **************************/
  
  KInt i ;
  
  for( i = 0 ; i < 1 ; i++){
    general_extractor =
      new KonoeFillDataExtractor(general_analyzer->getPhysicalValue());
    general_extractor->setIndex( 3 );
  }
  
  for( i = 0 ; i < CSI_TDC_HISTO_NUM ; i++){
    csi_tdc_1d_extractor[i] =
      new KonoeFillDataExtractor(csi_tdc_analyzer->getPhysicalValue());
    csi_tdc_1d_extractor[i]->setIndex( 3 );
  }
  
  for( i = 0 ; i < CSI_ADC_HISTO_NUM ; i++){
    csi_adc_1d_extractor[i] =
      new KonoeFillDataExtractor(csi_adc_analyzer->getPhysicalValue());
    csi_adc_1d_extractor[i]->setIndex( 3 );
  }
  
  for( i = 0 ; i < SCIN_TDC_HISTO_NUM ; i++){
    s_tdc_1d_extractor[i] =
      new KonoeFillDataExtractor(s_tdc_analyzer->getPhysicalValue());
    s_tdc_1d_extractor[i]->setIndex( 3 );
  }
  
  for( i = 0 ; i < SCIN_ADC_HISTO_NUM ; i++){
    s_adc_1d_extractor[i] =
      new KonoeFillDataExtractor(s_adc_analyzer->getPhysicalValue());
    s_adc_1d_extractor[i]->setIndex( 3 );
  }
  
  for( i = 0 ; i < INTREG_HISTO_NUM ; i++){
    intreg_extractor =
      new KonoeFillDataExtractor(intreg_analyzer->getPhysicalValue());
    intreg_extractor->setIndex( 3 );
  }
  
  
  /*
    csi_tdc_2d_extractor 	= new KonoeFillDataExtractor( 
    csi_tdc_analyzer->getPhysicalValue( ) );
    csi_tdc_2d_extractor->setIndex(
    CSI_DETECTOR_ID, KonoeRawDataset::DATA_INDEX);
    
    csi_adc_2d_extractor 	= new KonoeFillDataExtractor( 
    csi_adc_analyzer->getPhysicalValue( ) );
    csi_adc_2d_extractor->setIndex(
    CSI_DETECTOR_ID, KonoeRawDataset::DATA_INDEX);
  */
  
  /************** histogarm setting **************************/
  tdc_scale = new KonoeLinearDataScale( 0.0 , 5000 , 50 );
  adc_scale = new KonoeLinearDataScale( 0.0 , 5000 , 50 );
  
  /*
    c_tdc_x_count_histo 	= new Konoe2DHistogram( 
    new KonoeLinearDataScale( 0.0, 25.0, 25 ),
    tdc_scale		 );
    c_adc_x_count_histo 	= new Konoe2DHistogram( 
    new KonoeLinearDataScale( 0.0, 25.0, 25 ),
    adc_scale		 );
  */
  general_histogram = new Konoe1DHistogram( tdc_scale );	
  intreg_histogram = new Konoe1DHistogram( tdc_scale );	
  
  for( i = 0; i < CSI_TDC_HISTO_NUM ; i++ )
    csi_tdc_histogram[i] = new Konoe1DHistogram( tdc_scale);
  
  for( i = 0; i < CSI_ADC_HISTO_NUM ; i++ )
    csi_adc_histogram[i] = new Konoe1DHistogram( adc_scale);
  
  for( i = 0; i < SCIN_TDC_HISTO_NUM; i++ )
    s_tdc_histogram[i] = new Konoe1DHistogram( tdc_scale );
  
  for( i = 0; i < SCIN_ADC_HISTO_NUM; i++ )
    s_adc_histogram[i] = new Konoe1DHistogram( adc_scale );
  
  
  /************* add histogram to list ************************/

  filler_list = 
    new KonoeHistoFillerList( HISTO_TOTAL );
  
  general_hist_filler = 
    new KonoeHistogramFiller( general_extractor, general_histogram );
  intreg_hist_filler = 
    new KonoeHistogramFiller(  intreg_extractor, intreg_histogram );
  
  for( i = 0 ; i < CSI_TDC_HISTO_NUM ; i++){
    csi_tdc_1d_filler[i] = 
      new KonoeHistogramFiller( csi_tdc_1d_extractor[i],csi_tdc_histogram[i] );
  }
  for( i = 0 ; i < CSI_ADC_HISTO_NUM ; i++){
    csi_adc_1d_filler[i] = 
      new KonoeHistogramFiller( csi_adc_1d_extractor[i],csi_adc_histogram[i] );
  }
  for( i = 0 ; i < SCIN_TDC_HISTO_NUM ; i++){
    s_tdc_1d_filler[i] = 
      new KonoeHistogramFiller( s_tdc_1d_extractor[i],s_tdc_histogram[i] );
  }
  for( i = 0 ; i < SCIN_ADC_HISTO_NUM ; i++){
    s_adc_1d_filler[i] = 
      new KonoeHistogramFiller( s_adc_1d_extractor[i],s_adc_histogram[i] );
  }

  filler_list->add( general_hist_filler );

  for( i = 0 ; i < CSI_TDC_HISTO_NUM ; i++)
    filler_list->add( csi_tdc_1d_filler[i] ) ;

  for( i = 0 ; i < CSI_ADC_HISTO_NUM ; i++)
    filler_list->add( csi_adc_1d_filler[i] ) ;

  for( i = 0 ; i < SCIN_TDC_HISTO_NUM ; i++)
    filler_list->add( s_tdc_1d_filler[i] ) ;

  for( i = 0 ; i < SCIN_ADC_HISTO_NUM ; i++)
    filler_list->add( s_adc_1d_filler[i] ) ;

  filler_list->add( intreg_hist_filler );

///////////////////   logging   /////////////////////////

  KonoeNameSpaceTable * table;
  KonoeNameSpaceValue * value;

  table = new KonoeNameSpaceTable( );

  value = table->getValue("MASTER_HOST");
  if( value ){
	log = new KonoeLogMessenger( (KChar*)*value ); 	
  }
  else {
	cerr << "MASTER_HOST is not defined " << endl;
	exit(0);
  }

  delete table;

}

DemoAnalysis::~DemoAnalysis()
{
  delete daq_server;
  delete daq_socket;
}

KInt DemoAnalysis::go(KInt roop)
{

  while_loop = 1;
  KonoeDataRecord		* rec;
  KInt j;
  KInt i;
  KInt flag;
    
  daq_server -> waitConnection();

    for(  i = 0 ; i < roop ; i++){
//      while( while_loop ){

      /////////////////////////////////////////////////////////
      while( 1 ){
	rec = (KonoeDataRecord * ) daq_socket->get(0);
	if ( rec == 0 ) 
	  break;
//        rec->dump(cout);
	analyzer -> reset();
	analyzer -> analyze( rec );
	filler_list->fill();
      }
      //////////////////////////////////////////////////////////

	msg = new KonoeLogMessage("analysis data taking",1);
	log->send( msg->getLogMsg(),256);
	delete msg;
      
	
	////////////////////////  for ui /////////////////////////////// 
	KChar* ui_request ;
	static KInt ui_flag = 0;
	
	KonoeNameSpaceTable * table;
	KonoeNameSpaceValue * value;
	
	table = new KonoeNameSpaceTable( );

	value = table->getValue("UI_REQUEST");
	if( value ){
	  ui_request = (KChar*)*value;
	}
	delete table;

	if( strcmp( ui_request,"ON") == 0){
	  if( ui_flag == 0 )
	    ui_server -> waitConnection();
	  ui_flag = 1;
	  ui_socket->put( csi_tdc_histogram[3] );
	} 

      
      //////////////////// dump histogram /////////////////////////////////

	cout << " General " << endl;
	general_histogram->dump(cout);

	cout << " CSI TDC " << endl;
	for(j = 0 ; j < CSI_TDC_HISTO_NUM ; j++)
	  	csi_tdc_histogram[j]->dump(cout);

        cout << " CSI ADC " << endl;
        for(j = 0 ; j < CSI_ADC_HISTO_NUM ; j++)
          	csi_adc_histogram[j]->dump(cout);

       	cout << "  SCIN tdc " << endl;
	for(j = 0 ; j < SCIN_TDC_HISTO_NUM ; j++)
		s_tdc_histogram[j]->dump(cout);

	cout << "  SCIN adc " << endl;
	  for(j = 0 ; j < SCIN_ADC_HISTO_NUM ; j++)
		s_adc_histogram[j]->dump(cout);
      
    }
    // file pack
    ofile00->put( general_histogram );
    ofile01->put( csi_tdc_histogram[0]);
    ofile02->put( csi_tdc_histogram[1]);
    ofile03->put( csi_tdc_histogram[2]);
    ofile04->put( csi_tdc_histogram[3]);
    ofile11->put( csi_tdc_histogram[0]);
    ofile12->put( csi_tdc_histogram[1]);
    
    cout << " ok " << roop << " end " << endl;
    
    daq_server -> closeConnection();
    
    return 0;
}

void DemoAnalysis::catchSignal( KInt signal ){

	KonoeUDPClientSocket *command_process;
	command_process = new KonoeUDPClientSocket( masterhost, 50005);

	while_loop = 0;
	
	KChar ans[256];

	command_process->send("ready",256);
	command_process->receive(ans,256);

	KonoeNetworkNameSpaceManager * mgr;
	mgr = new KonoeNetworkNameSpaceManager( masterhost );
	mgr->setValue("CMD_STATUS","true");
	delete mgr;

}
