//
//        DemoAnalysis.hh 
//
//         Version 0.6.0	22-Aug-2000	T.Matuo
//         Version 0.6.1	 8-Sep-2000	Y.Shimizu
//
//=================================================

#ifndef __DEMO_HH_
#define	__DEMO_HH_

#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/KonoeDataRecordCM.hh>
#include <analysis/KonoeDataAnalyzer.hh>
#include <analysis/KonoeDefaultSectionAnalyzer.hh>
#include <analysis/KonoeFillDataExtractor.hh>
#include <analysis/KonoeHistogramFiller.hh>
#include <analysis/KonoeHistoFillerList.hh>
#include <analysis/KonoeDefaultSectionAnalyzer.hh>

#include <system/KonoeSignalClient.hh>
#include <system/KonoeSignalHandler.hh>

#include <system/KonoeFDSet.hh>
#include <system/KonoeServerSocket.hh>
#include <system/KonoeOutputObjSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjFile.hh>

#include <system/KonoeLogMessenger.hh>
#include <system/KonoeLogMessage.hh>

#include <string.h>
#include <stdlib.h>

enum {
  GENERAL_SECTION,
  CSI_TDC_SECTION,
  CSI_ADC_SECTION,
  S_TDC_SECTION,
  S_ADC_SECTION,
  REG_SECTION,
  N_SECTION
};

enum {
  TDC_CHANNEL     = 8,
  ADC_CHANNEL     = 16,
  REG_CHANNEL     = 1,
  GENE_CHANNEL    = 2,
  CSI_TDC  = 4,
  CSI_ADC  = 2
};

enum DATA_INDEX {
  SECTION_ID_INDEX,
  SEGMENT_ID_INDEX,
  SUBADDRESS_INDEX,
  DATA_INDEX,
  CSI_DETECTOR_ID,
  SC_DETECTOR_ID,
  MULTIPLET_SIZE
}; 

enum {  CSI_TDC_HISTO_NUM = 4 , 	
	CSI_ADC_HISTO_NUM = 2 ,
 	SCIN_TDC_HISTO_NUM = 1 , 
	SCIN_ADC_HISTO_NUM = 1 ,
	GENERAL_HISTO_NUM = 1 ,
	INTREG_HISTO_NUM = 1 ,
        //HISTO_2D_NUM = 2, 
        HISTO_2D_NUM = 0, 

	HISTO_TOTAL =   CSI_TDC_HISTO_NUM +
	       	        CSI_ADC_HISTO_NUM + 
			SCIN_TDC_HISTO_NUM +
			SCIN_ADC_HISTO_NUM + 
			GENERAL_HISTO_NUM + 
			INTREG_HISTO_NUM +
			HISTO_2D_NUM 
      };
  
enum { N_MULTIPLET = 256 };

enum {
  CSI 	 	= 25,
  SCIN		= 8,
  NUM_2D	= 3 
};

enum {
  DAQ_PORT	= 12341,
  UI_PORT	= 12342,
  ED_PORT	= 12343
};

const KInt  BUFFER_SIZE = 256;

class DemoAnalysis : public KonoeSignalClient {
protected:

  KonoeDataAnalyzer		* analyzer;

  KonoeDefaultSectionAnalyzer		* general_analyzer;
  KonoeDefaultSectionAnalyzer		* csi_tdc_analyzer;
  KonoeDefaultSectionAnalyzer		* csi_adc_analyzer;
  KonoeDefaultSectionAnalyzer		* s_tdc_analyzer;
  KonoeDefaultSectionAnalyzer		* s_adc_analyzer;
  KonoeDefaultSectionAnalyzer		* intreg_analyzer;
  
  KonoeFillDataExtractor	* general_extractor;
  KonoeFillDataExtractor	* csi_tdc_1d_extractor[CSI_TDC_HISTO_NUM];
  KonoeFillDataExtractor	* csi_adc_1d_extractor[CSI_ADC_HISTO_NUM];
  KonoeFillDataExtractor	* s_tdc_1d_extractor[SCIN_TDC_HISTO_NUM];
  KonoeFillDataExtractor	* s_adc_1d_extractor[SCIN_ADC_HISTO_NUM];
  KonoeFillDataExtractor	* intreg_extractor;
  
  //	KonoeFillDataExtractor		* csi_tdc_2d_extractor;
//	KonoeFillDataExtractor		* csi_adc_2d_extractor;
  
  KonoeHistogramFiller		* general_hist_filler;
  KonoeHistogramFiller		* intreg_hist_filler;
  KonoeHistogramFiller		* csi_tdc_1d_filler[CSI_TDC_HISTO_NUM];
  KonoeHistogramFiller		* csi_adc_1d_filler[CSI_ADC_HISTO_NUM];
  KonoeHistogramFiller		* s_tdc_1d_filler[SCIN_TDC_HISTO_NUM];
  KonoeHistogramFiller		* s_adc_1d_filler[SCIN_ADC_HISTO_NUM];
  
  //	KonoeHistogramFiller		* csi_tdc_2d_filler;
  //	KonoeHistogramFiller		* csi_adc_2d_filler;
  
  //	Konoe2DHistogram		* c_tdc_x_count_histo;
  //	Konoe2DHistogram		* c_adc_x_count_histo;
  Konoe1DHistogram		* csi_tdc_histogram[CSI_TDC_HISTO_NUM];
  Konoe1DHistogram		* csi_adc_histogram[CSI_ADC_HISTO_NUM];
  Konoe1DHistogram		* s_tdc_histogram[SCIN_TDC_HISTO_NUM];
  Konoe1DHistogram		* s_adc_histogram[SCIN_ADC_HISTO_NUM];
  Konoe1DHistogram		* general_histogram;
  Konoe1DHistogram		* intreg_histogram;
  
  KonoeLinearDataScale 		* tdc_scale;
  KonoeLinearDataScale		* adc_scale;	
  
  KonoeHistoFillerList		* filler_list;
  
  KonoeServerSocket		* daq_server;
  KonoeServerSocket		* ui_server;
  KonoeInputObjSocket		* daq_socket;
  KonoeOutputObjSocket		* ui_socket;
  
  KonoeOutputObjFile * ofile00;
  KonoeOutputObjFile * ofile01;
  KonoeOutputObjFile * ofile02;
  KonoeOutputObjFile * ofile03;
  KonoeOutputObjFile * ofile04;
  KonoeOutputObjFile * ofile11;
  KonoeOutputObjFile * ofile12;
  
  KonoeDataRecordCM		* d_cm;

  KonoeLogMessage   * msg; 
  KonoeLogMessenger * log;

  KInt while_loop ;
  
  KonoeSignalHandler * handler;
  void catchSignal( KInt signal);

  KChar* masterhost;
public:

  DemoAnalysis(KChar* master);
  ~DemoAnalysis();
  KInt go(KInt roop);
  KInt outtoui(KChar* buffer );
};

#endif

