// $Id: DemoAction.cc,v 1.6 2000/11/14 11:54:18 iwai Exp $
// ==================================================================
//  DemoAction.cc
//  01-September-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================

// -----------------------------------------------
//  native header files
// -----------------------------------------------
#include <iostream.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

// -----------------------------------------------
//  KONOE header files
// -----------------------------------------------
#include <daq/KonoeReadoutList.hh>
#include <system/KonoeNameSpaceTable.hh>
#include <system/KonoeNameSpaceValue.hh>

// -----------------------------------------------
//  user header files
// -----------------------------------------------
#include "DemoAction.hh"
#include "DemoStatus.hh"
#include "DemoModule.hh"

DemoAction::DemoAction()
  : anaHost("ngtlx0"), anaPort(12341)
{
  module = new DemoModule();  
  module -> install();
  outputSocket = new KonoeOutputObjSocket( anaHost, anaPort );
}

DemoAction::DemoAction(KChar* host, KInt port)
  : anaHost(host), anaPort(port)
{
  csocket = new KonoeClientSocket( anaHost, anaPort );
  outputSocket = new KonoeOutputObjSocket( csocket );
  module = new DemoModule();  
  module -> install();
}

DemoAction::~DemoAction()
{
  delete module;
  delete outputSocket;
}

void DemoAction::executeCommand(KChar* str)
{
  if ( (strcmp(str, "begin") == 0) )
    this -> begin();
  else if ( (strcmp(str, "pause") == 0) )
    this -> pause();
  else if ( (strcmp(str, "resume") == 0) )
    this -> resume();
  else if ( (strcmp(str, "end") == 0) )
    this -> end();
  else if ( (strcmp(str, "shutdown") == 0) )
    this -> shutdown();
  else
    cerr << "[daq] " << str << " is unknown command" << endl;
  return;
}

void DemoAction::begin()
{
  status -> setStatus(DemoStatus::busy);
 
  sleep(2);

  module -> update();
  module -> getReadoutList() -> read() -> dump(cout);
  csocket->connect();
  outputSocket -> put( module -> getReadoutList() -> read() );

  return;
}

void DemoAction::pause()
{
  status -> setStatus(DemoStatus::idle);
  return;
}

void DemoAction::resume()
{
  status -> setStatus(DemoStatus::busy);

  module -> update();
  module -> getReadoutList() -> read() -> dump(cout);
  outputSocket -> put( module -> getReadoutList() -> read() );
  return;
}

void DemoAction::end()
{
  status -> setStatus(DemoStatus::init);
  return;
}

void DemoAction::shutdown()
{
  status -> setStatus(DemoStatus::dead);
  delete this;
  exit(0);
}
