// $Id: DemoModule.cc,v 1.6 2000/11/14 11:54:18 iwai Exp $
// ==================================================================
//  DemoModule.cc
//  03-September-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "DemoModule.hh"

// ==================================================================
//  constans
// ==================================================================
static const KInt N_MODULES(11);
static const KInt N_TDC_CSI(4);
static const KInt N_ADC_CSI(2);
static const KInt N_SECTIONS(6);
static const KInt N_CHANNELS_TDC_CSI(8);
static const KInt N_CHANNELS_ADC_CSI(16);
static const KInt N_CHANNELS_TDC_SCI(8);
static const KInt N_CHANNELS_ADC_SCI(8);
static const KInt N_CHANNELS_IR(8);
static const KInt N_SEGMENTS_GENERAL(1);
static const KInt N_ELEMENTS_GENERAL(2);
static const KInt N_SEGMENTS_TDC_CSI(4);
static const KInt N_SEGMENTS_ADC_CSI(2);
static const KInt N_SEGMENTS_TDC_SCI(1);
static const KInt N_SEGMENTS_ADC_SCI(1);
static const KInt N_SEGMENTS_IR(1);
static const KInt RECORD_TYPE(100);
static const KInt GENERAL_SEC_TYPE(0);
static const KInt TDCCSI_SEC_TYPE(1);
static const KInt ADCCSI_SEC_TYPE(2);
static const KInt TDCSCI_SEC_TYPE(3);
static const KInt ADCSCI_SEC_TYPE(4);
static const KInt IR_SEC_TYPE(5);
static const KInt SEGMENT_TYPE(0);
static const KInt N_AVERAGE_HIT_8CH(4);
static const KInt N_AVERAGE_HIT_16CH(8);

DemoModule::DemoModule()
{
  crate = new KonoeSoftwareCrate(N_MODULES);
  crate -> clear();
  readoutList = new KonoeReadoutList(N_SECTIONS, RECORD_TYPE);
}

DemoModule::~DemoModule()
{
  delete crate;
  delete readoutList;
}

DemoModule::DemoModule(const DemoModule& right)
{
  readoutList = right.readoutList;
}

KBool DemoModule::operator==(const DemoModule& right) const
{
  return (readoutList == right.readoutList) ? KTrue : KFalse;
}

KBool DemoModule::operator!=(const DemoModule& right) const
{
  return (readoutList != right.readoutList) ? KTrue : KFalse;
}

const DemoModule& DemoModule::operator=(const DemoModule& right)
{
  readoutList = right.readoutList;
  return (*this);
}

void DemoModule::generate()
{
  // ---------------------------------------------
  //  generate modules
  // ---------------------------------------------
  runNumber = new KonoeSoftCounter();
  eventNumber = new KonoeSoftCounter();
  for ( KInt i = 0; i < N_TDC_CSI; i++ )
    tdc8chCsI[i] = 
      new KonoeSimTDCModule( N_CHANNELS_TDC_CSI, N_AVERAGE_HIT_8CH );
  for ( KInt i = 0; i < N_ADC_CSI; i++ )
    adc16chCsI[i] = 
      new KonoeSimADCModule( N_CHANNELS_ADC_CSI, N_AVERAGE_HIT_16CH );
  tdc8chScinti = 
    new KonoeSimTDCModule( N_CHANNELS_TDC_SCI, N_AVERAGE_HIT_8CH );
  adc8chScinti = 
    new KonoeSimADCModule( N_CHANNELS_ADC_SCI, N_AVERAGE_HIT_8CH );
  interruptRegister = 
    new KonoeSimADCModule( N_CHANNELS_IR, N_AVERAGE_HIT_8CH );

  // ---------------------------------------------
  //  generate ReadoutList
  // ---------------------------------------------
  KonoeReadoutSection* section;
  KonoeReadoutSegment* segment;

  // GENERAL
  readoutList -> 
    add(section = new KonoeReadoutSection(N_SEGMENTS_GENERAL, GENERAL_SEC_TYPE));
  section -> 
    add(segment = new KonoeReadoutSegment(N_ELEMENTS_GENERAL, SEGMENT_TYPE));
  segment -> add( new KonoeReadoutItem( runNumber, SEGMENT_TYPE ) );
  segment -> add( new KonoeReadoutItem( eventNumber, SEGMENT_TYPE ) );

  // TDC for CsI
  section = new KonoeReadoutSection( N_SEGMENTS_TDC_CSI, TDCCSI_SEC_TYPE );
  for ( KInt i = 0; i < N_TDC_CSI; i++ ) {
    segment = new KonoeReadoutSegment( tdc8chCsI[i], i );
    section -> add(segment);
  }
  readoutList -> add(section);

  // ADC for CsI
  section = new KonoeReadoutSection( N_SEGMENTS_ADC_CSI, ADCCSI_SEC_TYPE );
  for ( KInt i = 0; i < N_ADC_CSI; i++ ) {
    segment = new KonoeReadoutSegment( adc16chCsI[i], i );
    section -> add(segment);
  }
  readoutList -> add(section);

  // TDC for Scintillator
  section = new KonoeReadoutSection( N_SEGMENTS_TDC_SCI, TDCSCI_SEC_TYPE );
  segment = new KonoeReadoutSegment( tdc8chScinti, SEGMENT_TYPE );
  section -> add(segment);
  readoutList -> add(section);

  // ADC for Scintillator
  section = new KonoeReadoutSection( N_SEGMENTS_ADC_SCI, ADCSCI_SEC_TYPE );
  segment = new KonoeReadoutSegment( adc8chScinti, SEGMENT_TYPE );
  section -> add(segment);
  readoutList -> add(section);

  // InterruptRegister
  section = new KonoeReadoutSection( N_SEGMENTS_IR, IR_SEC_TYPE );
  segment = new KonoeReadoutSegment( interruptRegister, SEGMENT_TYPE );
  section -> add(segment);
  readoutList -> add(section);

  return;
}

void DemoModule::install()
{
  this -> generate();

  crate -> install(runNumber);
  crate -> install(eventNumber);
  for ( KInt i = 0; i < N_TDC_CSI; i++ )
    crate -> install(tdc8chCsI[i]);
  for ( KInt i = 0; i < N_ADC_CSI; i++ )
    crate -> install(adc16chCsI[i]);
  crate -> install(tdc8chScinti);
  crate -> install(adc8chScinti);
  crate -> install(interruptRegister);

  crate -> initialize();

  return;
}
