// $Id: daq.cc,v 1.7 2000/11/14 11:54:19 iwai Exp $
// ============================================================
//  daq.cc
//  01-September-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ============================================================
#include <iostream.h>
#include <stdlib.h>
#include <unistd.h>

#include <KonoePrimitives.hh>
#include <system/KonoeNetworkNameSpaceManager.hh>
#include <system/KonoeNameSpaceTable.hh>
#include <system/KonoeNameSpaceValue.hh>
#include <system/KonoeLogMessenger.hh>
#include <system/KonoeLogMessage.hh>

// user defined header files
#include "DemoStatus.hh"
#include "DemoAction.hh"
#include "DemoModule.hh"
#include "DemoNameSpace.hh"

// button is flag, through the init to busy or not.
enum Button { OFF, ON };

int main(int argc, char** argv)
{
  KChar* anaHost;
  KInt anaPort;

  if ( argc != 2 ) {
    cerr << "[daq] usage : daq [analysis host] " << endl;
    return (-1);
  } else {
    anaHost = argv[1];
//    anaPort = atol(argv[2]);
    anaPort = 12341;
  }
  cout << "[daq] start ..." << endl;

  KChar* master_host;
  KonoeNameSpaceTable* table = new KonoeNameSpaceTable();
  KonoeNameSpaceValue* value = table -> getValue("MASTER_HOST");
  master_host = (KChar*)*value;
  KonoeNetworkNameSpaceManager* manager 
    = new KonoeNetworkNameSpaceManager(master_host);

  DemoStatus* status = new DemoStatus();
  DemoAction* action = new DemoAction(anaHost, anaPort);
  DemoNameSpace* signalHandler = new DemoNameSpace();

  KonoeLogMessenger* log = new KonoeLogMessenger(master_host);
  KonoeLogMessage* msg = new KonoeLogMessage("Daq start!!",1);
  KonoeNetworkNameSpaceManager* mgr1;
  mgr1 = new KonoeNetworkNameSpaceManager( master_host );
  mgr1->setValue("DAQ_PID",msg->getPID());
  delete mgr1;

  KonoeNetworkNameSpaceManager* mgr2;
  mgr2 = new KonoeNetworkNameSpaceManager( master_host );
  mgr2->setValue("DAQ_HOST",msg->getHostName());
  delete mgr2;

  log->send(msg->getLogMsg(),256);
  delete msg;

  while (1) {
    static Button INITtoBUSY(OFF);

    // begin or shutdown
    if ( status -> getStatus() == DemoStatus::init ) {
      if ( ( status -> getCommand() == DemoStatus::begin ) && 
	   ( status -> getCommandStatus() == KTrue ) && 
	   ( INITtoBUSY == OFF ) ) {
	INITtoBUSY = ON;
	action -> executeCommand("begin");
      } else if ( ( status -> getCommand() == DemoStatus::shutdown ) && 
		  ( status -> getCommandStatus() == KTrue ) ) {
	action -> executeCommand("shutdown");
      }
    } 

    // busy or begin
    else if ( status -> getStatus() == DemoStatus::busy ) {
      if ( ( ( status -> getCommand() == DemoStatus::begin ) ||
	     ( status -> getCommand() == DemoStatus::resume ) ) && 
	   ( status -> getCommandStatus() == KTrue ) &&
	   ( INITtoBUSY == ON ) ) {
	action -> executeCommand("begin");
      } else if ( ( status -> getCommand() == DemoStatus::pause ) && 
		  ( status -> getCommandStatus() == KTrue ) ) {
	action -> executeCommand("pause");
      } 
    }

    // end or resume
    else if ( status -> getStatus() == DemoStatus::idle ) {
      sleep(3);
      if ( ( status -> getCommand() == DemoStatus::resume ) && 
	   ( status -> getCommandStatus() == KTrue ) ) {
	action -> executeCommand("resume");
      } else if ( ( status -> getCommand() == DemoStatus::end ) && 
		  ( status -> getCommandStatus() == KTrue ) ) {
	INITtoBUSY = OFF;
	action -> executeCommand("end");
      } 
    }
    
  }

  cout << "[daq] end ..." << endl;

  delete status;
  delete action;
  delete signalHandler;

  return (0);
}
