//================================================================================
/*

	FakeDAQ.cc	fake DAQ process

	25-Aug-2000	Y. Shimizu
*/
//================================================================================
#include<iostream.h>
#include<string.h>
#include<unistd.h>
#include<stdlib.h>
#include<signal.h>
#include<KonoePrimitives.hh>
#include<system/KonoeSignalClient.hh>
#include<system/KonoeSignalHandler.hh>
#include<system/KonoeNameSpaceTable.hh>
#include<system/KonoeNameSpaceValue.hh>
#include<system/KonoeNetworkNameSpaceManager.hh>
#include<system/KonoeLogMessenger.hh>
#include<system/KonoeLogMessage.hh>
#include<system/KonoeHostSystem.hh>
#include<system/KonoeUDPClientSocket.hh>

class FakeDAQ : public KonoeSignalClient {
public:
	enum { INIT, GO, PAUSE , END , UNKNOWN };
protected:
	KChar* 	masterhost;
	KonoeLogMessage * msg;
	KonoeLogMessenger *log;
	KInt daq_mode;	
	KonoeSignalHandler * handler;
	KonoeUDPClientSocket * command_process;
	void catchSignal( KInt signal );
public:
	FakeDAQ( );
	~FakeDAQ( );
	KInt init( );
	KInt go( );
	KInt pause( );
	KInt end( );
};

FakeDAQ::FakeDAQ( ) {


	KonoeNameSpaceTable * table;
	KonoeNameSpaceValue * value;

	table = new KonoeNameSpaceTable( );

	value = table->getValue("MASTER_HOST");
	if( value ){
	   masterhost = (KChar*)*value;
	}
	else {
	   cerr << "MASTER_HOST is not defined " << endl;
	   KonoeHostSystem::exit(0);
	}

	command_process = new KonoeUDPClientSocket( masterhost , 50005 );

	log = new KonoeLogMessenger( masterhost );

	msg = new KonoeLogMessage(" FakeDaq constructor ",1);

	KonoeNetworkNameSpaceManager * mgr1;
	mgr1 = new KonoeNetworkNameSpaceManager( masterhost );
	mgr1->setValue("DAQ_PID",msg->getPID() );

	KonoeNetworkNameSpaceManager * mgr2;
	mgr2 = new KonoeNetworkNameSpaceManager( masterhost );
	mgr2->setValue("DAQ_HOST",msg->getHostName() );
	
	
	handler = new KonoeSignalHandler( SIGINT , this );
	
	handler->enable();

	delete mgr1;
	delete mgr2;
	delete msg;

}

FakeDAQ::~FakeDAQ( ) {
	delete command_process;
}

KInt FakeDAQ::init( ){
	KonoeNetworkNameSpaceManager * mgr;
	mgr = new KonoeNetworkNameSpaceManager( masterhost );
	mgr->setValue("CMD_STATUS","true");
	delete mgr;

	msg = new KonoeLogMessage(" FakeDAQ initialize ",1);
	log->send( msg->getLogMsg(),256);
	delete msg;

	cout << " int the INIT " << endl;

	handler->wait( ) ;

	if( daq_mode == INIT )
	    init();
	else if( daq_mode == GO )
	    go();
	else if( daq_mode  == END ){
	    cout << " OK! fakeDaq END " << endl;
	    end();
	}	
        else if( daq_mode == PAUSE ){
	     cout << " OK! fakeDaq PAUSE " << endl;
	     pause();
	     return (0);
	}	
	return -1;
}

KInt FakeDAQ::go( ){

	KonoeNetworkNameSpaceManager * mgr;
	mgr = new KonoeNetworkNameSpaceManager( masterhost );
	mgr->setValue("CME_STATUS","true");
	delete mgr;

	msg = new KonoeLogMessage(" FakeDAQ go ",0);
	log->send( msg->getLogMsg(),256);
	delete msg;
	
	cout << " in the go " << endl;
	daq_mode = GO ;
	while( !( (daq_mode == INIT ) || (daq_mode == PAUSE )  ) ){
	     sleep(2);
	     cout << " int the GO " << endl;	
	}
	if( daq_mode == INIT )
	     init();
        else if( daq_mode == PAUSE ){
	     cout << " OK! fakeDaq PAUSE " << endl;
	     pause();
	     return (0);
	}	
	return (-1);

}	

KInt FakeDAQ::pause( ){

	KonoeNetworkNameSpaceManager * mgr;
	mgr = new KonoeNetworkNameSpaceManager( masterhost );
	mgr->setValue("CMD_STATUS","true");
	delete mgr;

	msg = new KonoeLogMessage(" FakeDAQ pause ",1);
	log->send( msg->getLogMsg(),256);
	delete msg;

	handler->wait( ) ;
	if( daq_mode == INIT )
	    init();
	else if( daq_mode == GO )
	    go();
	else if( daq_mode  == END ){
	    cout << " OK! fakeDaq END " << endl;
	    end();
	}	
	return -1;

}

KInt FakeDAQ::end( ){

	KonoeNetworkNameSpaceManager * mgr;
	mgr = new KonoeNetworkNameSpaceManager( masterhost );
	mgr->setValue("CMD_STATUS","true");
	delete mgr;

	cout << " bye ! " << endl;

	msg = new KonoeLogMessage(" FakeDAQ end",1);
	log->send( msg->getLogMsg(),256);
	delete msg;

	return (0);
}

void FakeDAQ::catchSignal( KInt signal ){

 	cout << " ========  catchSignal !!!  ======== " << endl;
	KChar mode[256];


	command_process->send("ready",256);
	command_process->receive(mode,256);

	if( strcmp(mode,"init") == 0 )
		daq_mode = INIT;
	else if( strcmp(mode,"go") == 0 )
		daq_mode = GO;
	else if( strcmp(mode,"pause") == 0 )
		daq_mode = PAUSE;
	else if( strcmp(mode,"end")  == 0 )
		daq_mode = END;
	else 
		daq_mode = UNKNOWN;	
	
	cout << " daq_mode is " << daq_mode << endl;

}
