// $Id: lam.cc,v 1.4 2000/11/14 09:55:02 iwai Exp $
// ==================================================================
//  lam.cc
//  23-October-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "modules.h"
#include "Buffer.hh"

#include <daq/KonoeCAMACCrate.hh>
#include <daq/KonoeCAMACInterruptRegister.hh>
#include <daq/KonoeCAMACADC.hh>
#include <daq/KonoeSoftCounter.hh>
#include <daq/KonoeCAMACXaction.hh>
#include <daq/KonoeReadoutList.hh>
#include <system/KonoeOutputObjFile.hh>

int main(int argc, char** argv)
{
  // --------------------------------------------------------------
  //  constants for KonoeDataRecord 
  static const KInt N_SECTION(1);
  static const KInt RECORDTYPE(1000);
  static const KInt N_SEGMENT(2);
  static const KInt SECTIONTYPE(200);
  static const KInt SEGMENTTYPE(100);  
  static const KInt N_EVENT(100000);
  static const KInt N_ELEMENT(1);
  // --------------------------------------------------------------

  KonoeCAMACCrate* cc7700 = new KonoeCAMACCrate(NODENAME, CCID);
  KonoeCAMACADC* adc = new KonoeCAMACADC(cc7700, ADC);
  KonoeCAMACInterruptRegister* ir 
    = new KonoeCAMACInterruptRegister(cc7700, INTERRUPTREGISTER);
  KonoeSoftCounter* counter = new KonoeSoftCounter();

  cc7700 -> install(adc);
  cc7700 -> install(ir);
  cc7700 -> clear();

  KonoeReadoutList* readoutList 
    = new KonoeReadoutList( N_SECTION, RECORDTYPE );
  KonoeReadoutSection* section 
    = new KonoeReadoutSection( N_SEGMENT, SECTIONTYPE );

  KonoeReadoutSegment* segment;
  KonoeReadoutItem* element;

  segment = new KonoeReadoutSegment( N_ELEMENT, SEGMENTTYPE );
  section -> add( segment );
  element = new KonoeReadoutItem( counter, (KInt)0 );
  segment -> add( element );

  segment = new KonoeReadoutSegment( adc, SEGMENTTYPE );
  section -> add( segment );
  readoutList -> add( section );

  KonoeCAMACXaction* xaction 
    = new KonoeCAMACXaction(readoutList, cc7700, ir, 4100);

  KonoeOutputObjFile* file = new KonoeOutputObjFile("lam.dat");
  KonoeDataRecord* data;
  ir -> writeMask((KUShort)252);

  static KInt eventCounter(1);
  static KInt spillCounter(1);
  Buffer* buffer = new Buffer(2000);

  while (1) {
    xaction -> enable();
    data = xaction -> read();

    if ( data ) {
      cout << "##### EVENT NUMBER [" << eventCounter << "] #####" << endl;
      buffer -> add( data );
    }

    if ( ir -> read(2) ) {
      cout << "##### SPILL [" << spillCounter << "] END #####" << endl;

      buffer -> reset();
      while ( data = buffer -> next() ) {
#ifdef DEBUG	
	data -> dump( cout );
#endif
	file -> put( data );
      }
      buffer -> clear();
      spillCounter++;
    }

    cc7700 -> clear();
    counter -> update();
    eventCounter++;

    if ( eventCounter > N_EVENT )
      break;
  }

  delete buffer;
  delete readoutList;
  delete file;
  delete xaction;
  delete cc7700;

  return (0);
}

//   // ===================================================== IR test
//   // LAMäƤӥåȥѥɤǤΤ disable
//   ir -> readPattern();
//   // LAMäƤӥåȥѥɤ
//   ir -> read();
//   // LAM˥ޥ򤫤 1 ̵뤵롣 0 Ωġ
//   // 1, 2 ̵뤷ä 3 ɤ,
//   // դ1, 2 ä 252 Ȥʤ
//   ir -> write(3);
//   // LAMΥޥɤ
//   ir -> readMask();
//   // disableLAM ⶯ػ
//   ir -> disableLAMoutput();
//   // disableLAMoutput() 
//   ir -> enableLAMoutput();
//   // LAM 򥯥ꥢ disable
//   ir -> clearLAM();
//   // LAMޥ 0 ˤ롣ĤޤƤդ
//   ir -> clearMask();
//   // LAM 쥸򥯥ꥢ enable 
//   ir -> clearEnable();
