// $Id: socket.cc,v 1.5 2001/01/01 09:50:10 iwai Exp $
// ==================================================================
//  socket.cc
//  23-October-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "modules.h"
#include "constants.h"
#include "Buffer.hh"

#include <daq/KonoeCAMACCrate.hh>
#include <daq/KonoeCAMACInterruptRegister.hh>
#include <daq/KonoeCAMACADC.hh>
#include <daq/KonoeSoftCounter.hh>
#include <daq/KonoeCAMACXaction.hh>
#include <daq/KonoeReadoutList.hh>
#include <daq/KonoeSimTDCModule.hh>
#include <daq/KonoeSimADCModule.hh>
#include <daq/KonoeOptionalNumberModule.hh>
#include <system/KonoeOutputObjSocket.hh>

int main(int argc, char** argv)
{
  KonoeCAMACCrate* cc7700 = new KonoeCAMACCrate(NODENAME, CCID);
  KonoeCAMACADC* adc = new KonoeCAMACADC(cc7700, ADC);
  KonoeCAMACInterruptRegister* ir 
    = new KonoeCAMACInterruptRegister(cc7700, INTERRUPTREGISTER);
  KonoeSoftCounter* counter = new KonoeSoftCounter();
  KonoeOptionalNumberModule* tmp = new KonoeOptionalNumberModule(1);
  KonoeSimTDCModule* tdcCsI = new KonoeSimTDCModule(8,8);
  KonoeSimTDCModule* tdcSci = new KonoeSimTDCModule(8,8);
  KonoeSimADCModule* adcSci = new KonoeSimADCModule(8,8);

  cc7700 -> install(adc);
  cc7700 -> install(ir);
  cc7700 -> clear();

  KonoeReadoutSection* section;
  KonoeReadoutSegment* segment;
  KonoeReadoutItem* element;

  KonoeReadoutList* readoutList 
    = new KonoeReadoutList( N_SECTION, RECORD_TYPE );

  // GENERAL
  section = new KonoeReadoutSection(GENERAL_N_SEGMENT,GENERAL_SECTION_TYPE);
  segment = new KonoeReadoutSegment(COUNTER_N_ELEMENT, COUNTER_SEGMENT_TYPE);
  section -> add(segment);
  element = new KonoeReadoutItem(counter,(KInt)0);
  segment -> add(element);
  element = new KonoeReadoutItem(counter,(KInt)1);
  segment -> add(element);
  segment = new KonoeReadoutSegment(TRIGGER_N_ELEMENT, TRIGGER_SEGMENT_TYPE);
  section -> add( segment );
  for ( KInt i = 0; i < TRIGGER_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( tmp, i );
    segment -> add(element);
  }
  segment 
    = new KonoeReadoutSegment(ENVIRONMENT_N_ELEMENT,ENVIRONMENT_SEGMENT_TYPE);
  section -> add(segment);
  for ( KInt i = 0; i < ENVIRONMENT_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( tmp, i );
    segment -> add( element );
  }
  readoutList -> add( section );

  // TDC CSI
  section = new KonoeReadoutSection(TDC_CSI_N_SEGMENT,TDC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(tdcCsI,TDC_CSI_ID0_SEGMENT_TYPE);
  section -> add(segment);
  segment = new KonoeReadoutSegment(tdcCsI,TDC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(tdcCsI,TDC_CSI_ID2_SEGMENT_TYPE);
  section -> add( segment );
  segment 
    = new KonoeReadoutSegment(TDC_CSI_ID3_N_ELEMENT,TDC_CSI_ID3_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( tdcCsI, (KInt)0 );
  segment -> add( element );
  readoutList -> add( section );
  
  // ADC CSI
  section = new KonoeReadoutSection(ADC_CSI_N_SEGMENT,ADC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(adc,ADC_CSI_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment 
    = new KonoeReadoutSegment(ADC_CSI_ID1_N_ELEMENT,ADC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  for ( KInt i = 0; i < ADC_CSI_ID1_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( adc, i );
    segment -> add( element );
  }
  readoutList -> add( section );

  // TDC SCI
  section 
    = new KonoeReadoutSection(TDC_SCINTI_N_SEGMENT,TDC_SCINTI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(tdcSci,TDC_SCINTI_SEGMENT_TYPE);
  section -> add( segment );
  readoutList -> add( section );

  // ADC SCI
  section 
    = new KonoeReadoutSection(ADC_SCINTI_N_SEGMENT,ADC_SCINTI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(adcSci,ADC_SCINTI_SEGMENT_TYPE);
  section -> add( segment );
  readoutList -> add( section );

  // INTERRUPT REGISTER
  section = new KonoeReadoutSection(IR_N_SEGMENT,IR_SECTION_TYPE);
  segment = new KonoeReadoutSegment(IR_N_ELEMENT,IR_SEGMENT_TYPE);
  section -> add( segment );
  for ( KInt i = 0; i < IR_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( tmp, i );
    segment -> add( element );
  }
  readoutList -> add( section );

  
  KonoeCAMACXaction* xaction 
    = new KonoeCAMACXaction(readoutList, cc7700, ir, 4100);

  KonoeOutputObjSocket* socket 
    = new KonoeOutputObjSocket( HOST_NAME, PORT_NUMBER );

  ir -> writeMask((KUShort)252);

  static KInt eventCounter(1);
  static KInt spillCounter(1);

  Buffer* buffer = new Buffer( BUFFER_SIZE );
  KonoeDataRecord* data;
  while (1) {
    xaction -> enable();
    data = xaction -> read();

    if ( data ) {
      cout << "##### EVENT NUMBER [" << eventCounter << "] #####" << endl;
      buffer -> add( data );
    }

    if ( ir -> read(2) ) {
      cout << "##### SPILL [" << spillCounter << "] END #####" << endl;

      buffer -> reset();
      while ( data = buffer -> next() ) {
#ifdef DEBUG	
	data -> dump( cout );
#endif
	socket -> put( data );
      }
      buffer -> clear();
      spillCounter++;
    }

    cc7700 -> clear();
    counter -> update();
    eventCounter++;

    if ( eventCounter > N_EVENT )
      break;
  }

  delete buffer;
  delete readoutList;
  delete socket;
  delete xaction;
  delete cc7700;

  return (0);
}
