//===================================================================
/*

	demoread.cc at konoe/demo

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	15-Oct-1997	H. Sakamoto
	Version 0.4	30-Oct-1997	H. Sakamoto
	Version 0.2.0	18-May-1998	H. Sakamoto

*/
//===================================================================

#include	<analysis/KonoeDataAnalyzer.hh>
#include	<analysis/KonoeGeneralSectionAnalyzer.hh>
#include	<analysis/KonoeDefaultSectionAnalyzer.hh>
#include	<iostream.h>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<system/KonoeInputObjFile.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeHistoFillerList.hh>
#include	<analysis/KonoeFillDataExtractor.hh>
#include	<analysis/Konoe1DHistogram.hh>


//-------------------------------------------------------------------

//	actual application class starts here

//-------------------------------------------------------------------

class	Demo {
public :
	enum	{ RECORD_TYPE = 101 };
	enum	{ GENERAL_SECTION, TDC_SECTION,
		N_SECTIONS };
	enum	{ EVENT_INFO, N_GENERAL_SEGMENTS };
	enum	{ TDC_1, N_TDC_SEGMENTS };
	enum	{ N_TDC_CHANNELS = 16 };
	enum	{ N_MULTIPLETS = 256 };
protected :
	KonoeDataAnalyzer	* analyzer;
	KonoeInputObjFile	* file;
	KonoeDataRecordCM	* data_record_cm;
	KonoeLinearDataScale	* tdc_scale;
	Konoe1DHistogram	* tdc_histogram;
	KonoeHistoFillerList	* filler_list;
	KonoeFillDataExtractor	* fill_data_extractor;
	KonoeDefaultSectionAnalyzer	* default_section_analyzer;

public :
	Demo( int argc, char ** argv );
	~Demo( );
	int	go( );
};

Demo::Demo( int, char ** ) {
          
	analyzer = new KonoeDataAnalyzer( N_SECTIONS );
	analyzer -> install( default_section_analyzer = 
		new KonoeDefaultSectionAnalyzer( TDC_SECTION,
			N_MULTIPLETS, 0 ) );
	analyzer -> install( new KonoeGeneralSectionAnalyzer( ) );

	file = new KonoeInputObjFile( "test.dat" );
	data_record_cm = new KonoeDataRecordCM( );
	file -> installCM( data_record_cm );
	tdc_scale = new KonoeLinearDataScale( 0.0, 4096.0, 4096 );
	tdc_histogram = new Konoe1DHistogram( tdc_scale );
	fill_data_extractor = new KonoeFillDataExtractor(
		default_section_analyzer -> getPhysicalValue( ) );
	fill_data_extractor -> setIndex(
		KonoeRawDataset::DATA_INDEX );
	filler_list = new KonoeHistoFillerList( 1 );
	filler_list -> add( new KonoeHistogramFiller(
		fill_data_extractor, tdc_histogram ) );
}

Demo::~Demo( ) {

	delete file;
}

int	Demo::go( ) {
	KonoeDataRecord	* rec;

	tdc_histogram -> clear( );

	while( 1 ) {
	    rec = ( KonoeDataRecord * ) file -> get( 0 );
	    if( rec == 0 ) break;
	    rec -> dump( cout );
            cout<<"_________This is getType"<<rec->getType()<<endl;
            cout<<"_________This is rec value"<<rec<<endl;
            cout<<"_________This is getStreamSize"<<rec->getStreamSize()<<endl;
	    analyzer -> reset( );
	    analyzer -> analyze( rec );
	    //		analyzer -> dumpPhysicalValue( cout );
	    filler_list -> fill( );
	    delete rec;
	}
	tdc_histogram -> dump( cout );
	return 0;
}


//-------------------------------------------------------------------

//	entry point

//-------------------------------------------------------------------

main( int argc, char ** argv ) {
	Demo demo( argc, argv );
	return demo.go( );
}
