//========================================================================
/*

	sysinfod.cc

	Version 0.0.1	24-Nov-2000	H. Sakamoto

	Demo program to update system related name space entries
	!!! Linux only !!!

*/
//=======================================================================

#include	<KonoePrimitives.hh>
#include	<system/KonoeNetworkNameSpaceManager.hh>
#include	<system/KonoeSignalHandler.hh>
#include	<system/KonoeSignalClient.hh>
#include	<system/KonoeIntervalTimer.hh>
#include	<unistd.h>
#include	<stdio.h>
#include	<string.h>
#include	<system/KonoeHostSystem.hh>
#include	<linux/kernel.h>
#include	<linux/sys.h>

extern "C"	int	sysinfo_call( struct sysinfo * );

class	SysInfoDaemon	: public KonoeSignalClient	{

protected :

	enum	{	DEFAULT_INTERVAL = 1000,
			HOSTNAME_LENGTH = 64		};

	KonoeNetworkNameSpaceManager	* networknamespace;
	KonoeSignalHandler	* signalhandler;
	KonoeIntervalTimer	* intervaltimer;

	KInt	interval;
	KChar	hostname[ HOSTNAME_LENGTH ];

	struct sysinfo	sysinfo_table;

	KChar	* uptime_entry;
	KChar	* load_entry;
	KChar	* freeram_entry;
	KChar	* sharedram_entry;
	KChar	* bufferram_entry;
	KChar	* freeswap_entry;
	KChar	* procs_entry;

public :

	SysInfoDaemon( KInt argc, KChar ** argv );
	~SysInfoDaemon( );
	KInt	go( );
	void	catchSignal( KInt signal );

};

SysInfoDaemon::SysInfoDaemon( KInt argc, KChar ** argv ) {

	interval = DEFAULT_INTERVAL;

	networknamespace = new KonoeNetworkNameSpaceManager( );
	signalhandler = new KonoeSignalHandler( SIGALRM, this );
	signalhandler -> enable( );
	intervaltimer = new KonoeIntervalTimer( );
	intervaltimer -> setInterval( interval );

	KonoeHostSystem::getHostName( hostname, HOSTNAME_LENGTH );

	KInt l = strlen( hostname );
	uptime_entry = new KChar[ l + 8 ];
	strcpy( uptime_entry, hostname );
	strcat( uptime_entry, "_uptime" );
	load_entry = new KChar[ l + 6 ];
	strcpy( load_entry, hostname );
	strcat( load_entry, "_load" );
	freeram_entry = new KChar[ l + 9 ];
	strcpy( freeram_entry, hostname );
	strcat( freeram_entry, "_freeram" );
	sharedram_entry = new KChar[ l + 11 ];
	strcpy( sharedram_entry, hostname );
	strcat( sharedram_entry, "_sharedram" );
	bufferram_entry = new KChar[ l + 11 ];
	strcpy( bufferram_entry, hostname );
	strcat( bufferram_entry, "_bufferram" );
	freeswap_entry = new KChar[ l + 10 ];
	strcpy( freeswap_entry, hostname );
	strcat( freeswap_entry, "_freeswap" );
	procs_entry = new KChar[ l + 7 ];
	strcpy( procs_entry, hostname );
	strcat( procs_entry, "_procs" );

}

SysInfoDaemon::~SysInfoDaemon( ) {

	delete networknamespace;
	delete [] uptime_entry;
	delete [] load_entry;
	delete [] freeram_entry;
	delete [] sharedram_entry;
	delete [] bufferram_entry;
	delete [] freeswap_entry;
	delete [] procs_entry;

}

KInt	SysInfoDaemon::go( ) {

	while( 1 ) {
	    pause( );
	}
	return 0;

}

void	SysInfoDaemon::catchSignal( KInt signal ) {

	KInt	st;
	KChar	buffer[ 64 ];

	st = sysinfo_call( & sysinfo_table );
	if( st < 0 ) {
	    perror( 0 );
	    KonoeHostSystem::exit( 1 );
	}

	sprintf( buffer, "%ld", sysinfo_table.uptime );
	networknamespace -> setValue( uptime_entry, buffer );

	sprintf( buffer, "%lu", sysinfo_table.loads[ 0 ] );
	networknamespace -> setValue( load_entry, buffer );

	sprintf( buffer, "%lu", sysinfo_table.freeram );
	networknamespace -> setValue( freeram_entry, buffer );

	sprintf( buffer, "%lu", sysinfo_table.sharedram );
	networknamespace -> setValue( sharedram_entry, buffer );

	sprintf( buffer, "%lu", sysinfo_table.bufferram );
	networknamespace -> setValue( bufferram_entry, buffer );

	sprintf( buffer, "%lu", sysinfo_table.freeswap );
	networknamespace -> setValue( freeswap_entry, buffer );

	sprintf( buffer, "%u", sysinfo_table.procs );
	networknamespace -> setValue( procs_entry, buffer );


	signalhandler -> enable( );
	intervaltimer -> setInterval( interval );

}

int	main( int argc,  char ** argv ) {

	SysInfoDaemon	daemon( argc, argv );

	return daemon.go( );

}

