#include <unistd.h>
#include <stdlib.h>

#include <system/KonoeServerSocket.hh>
#include <system/KonoeOutputObjSocket.hh>
#include <system/KonoeOutputObjFile.hh>

#include <daq/KonoeReadoutList.hh>
#include <daq/KonoeReadoutSection.hh>
#include <daq/KonoeReadoutSegment.hh>
#include <daq/KonoeCAMACCrate.hh>
#include <daq/KonoeCAMACADC.hh>
#include <daq/KonoeCAMACSwitchRegister.hh>

#include <analysis/KonoeDataRecord.hh>
#include <analysis/KonoeDataRecordCM.hh>

#include "exConstants.hh"


int main( int argc, char** argv, char** envv )
{
  enum { exCommand, exPort, exNumberOfArguments };
  if ( argc != exNumberOfArguments ) {
    cerr << "usage: " << argv[ exCommand ];
    cerr << " " << "<port>";
    cerr << endl;
    return( -1 );
  }
  const KInt port = atoi( argv[ exPort ] );

  KonoeCAMACCrate* cc7700 = new KonoeCAMACCrate( kNodeName, kControllerID );
  KonoeCAMACADC* adc
    = new KonoeCAMACADC( cc7700, kAdcSlot, kNumberOfAdcChannels );

  KonoeCAMACSwitchRegister* swreg
    = new KonoeCAMACSwitchRegister( cc7700, kSwitchRegisterSlot );

  cc7700 -> install( adc );
  cc7700 -> install( swreg );


  KonoeReadoutList* rlist
    = new KonoeReadoutList( kNumberOfSections, kListID );

  KonoeReadoutSection* rsec
    = new KonoeReadoutSection( kNumberOfSegments, kSectionID );

  KonoeReadoutSegment* rseg0
    = new KonoeReadoutSegment( adc, kAdcID );

  KonoeReadoutSegment* rseg1 =
    new KonoeReadoutSegment( swreg, kSwitchRegisterID );

  rsec -> add( rseg0 );
  rsec -> add( rseg1 );
  rlist -> add( rsec );

  KonoeOutputObjFile* ofile =
    new KonoeOutputObjFile( kFileNameForDataRecord );


  KonoeServerSocket* socket = new KonoeServerSocket( port );
  socket -> setWaitInterval( 1000 );
  KonoeOutputObjSocket* osocket = new KonoeOutputObjSocket( socket );

  cerr << "waiting for connection ... ";
  socket -> waitConnection();
  cout << "established." << endl;


  KonoeDataRecord* datarecord = 0;

  for ( KInt i = 0; i < kNumberOfEvents; i ++ ) {
    datarecord = rlist -> read();
    datarecord -> dump( cout );
    osocket -> put( datarecord );
    ofile -> put( datarecord );

    cc7700 -> update();

    delete datarecord;
    sleep( kSamplingTimeInSecond );
  }

  delete rlist;
  delete ofile;

  delete cc7700;


  return( 0 );
}
