#include <unistd.h>
#include <vector>
#include <stdlib.h>

#include <system/KonoeClientSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjFile.hh>

#include <analysis/KonoeDataRecord.hh>
#include <analysis/KonoeDataRecordCM.hh>
#include <analysis/KonoeDataSection.hh>
#include <analysis/KonoeDataSegment.hh>
#include <analysis/KonoeDataElement.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/KonoeLinearDataScale.hh>

#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclMainWindow.hh>
#include <ui/KonoeTclFrame.hh>

#include "exConstants.hh"

typedef vector<KFloat> KFloatList;

int main( int argc, char** argv, char** envv )
{
  enum { exCommand, exHost, exPort, exNumberOfArguments };
  if ( argc != exNumberOfArguments ) {
    cerr << "usage: " << argv[ exCommand ];
    cerr << " " << "<host>";
    cerr << " " << "<port>";
    cerr << endl;
    return( -1 );
  }
  KChar* host = argv[ exHost ];
  const KInt port = atoi( argv[ exPort ] );


  KonoeTclMainWindow* mainwindow
    = new KonoeTclMainWindow( kWindowTitle );

  KChar tmp[ 256 ];
  KonoeTclFrame* frame[ kNumberOfFrames ];
  for ( KInt i = 0; i < kNumberOfFrames; i ++ )
    frame[ i ] = new KonoeTclFrame( mainwindow, tmpnam( tmp ) );

  Konoe1DHistogram* dummy[ kNumberOfHistograms ];
  KonoeTclDrawPanel* panel[ kNumberOfHistograms ];
  KonoeTclDrawCanvas* canvas[ kNumberOfHistograms ];
  for ( KInt i = 0; i < kNumberOfHistograms; i ++ ) {
    if ( i != 8 ) {
      dummy[ i ]
	= new Konoe1DHistogram( new KonoeLinearDataScale( 0.0, 4096.0, 4096 ) );
    } else {
      dummy[ i ]
	= new Konoe1DHistogram( new KonoeLinearDataScale( 0.0, 65536.0, 65536 ) );
    }
    dummy[ i ] -> clear();


    panel[ i ]
      = new KonoeTclDrawPanel( frame[ i / kNumberOfFrames ], tmpnam( tmp ) );

    canvas[ i ]
      = new KonoeTclDrawCanvas( panel[ i ],
				tmpnam( tmp ),
				dummy[ i ],
				KonoeTclDimension( kCanvasWidth,
						   kCanvasHeight ),
				kHistogramTitle[ i ] );

    panel[ i ] -> setCanvas( canvas[ i ], tmpnam( tmp ) );
    canvas[ i ] -> setBackGround( kBackgroundColor );
    canvas[ i ] -> setHistoColor( kHistogramColor[ i ] );

    if ( i != 8 ) {
      canvas[ i ] -> setHorizontalScale( 0, 4096, 50 );
    } else {
      canvas[ i ] -> setHorizontalScale( 0, 65536, 50 );
    }

  }

  for ( KInt i = 0; i < kNumberOfFrames; i ++ ) {
    frame[ i ] -> setLayoutType( "left" );
    frame[ i ] -> setFrame();
  }





  KonoeClientSocket* socket = new KonoeClientSocket( host, port );
  KonoeInputObjSocket* isocket = new KonoeInputObjSocket( socket );
  KonoeDataRecordCM* manager = new KonoeDataRecordCM();
  isocket -> installCM( manager );


  KonoeOutputObjFile* ofile =
    new KonoeOutputObjFile( kFileNameForHistogram );



  KonoeDataRecord* datarecord = 0;
  KonoeDataSection* datasection = 0;
  KonoeDataSegment* datasegment = 0;
  KonoeDataElement* dataelement = 0;


  Konoe1DHistogram* hist[ kNumberOfHistograms ];
  KFloatList databuffer[ kNumberOfHistograms ];
  while ( ( datarecord = (KonoeDataRecord*)( isocket -> get() ) ) ) {
    datarecord -> dump( cout );
    KInt histid = 0;


    for ( KInt i = 0; i < kNumberOfHistograms; i ++ ) {
      if ( i != 8 ) {
	hist[ i ] = new Konoe1DHistogram( new KonoeLinearDataScale( 0.0, 4096.0, 4096 ) );
      } else {
	hist[ i ] = new Konoe1DHistogram( new KonoeLinearDataScale( 0.0, 65536.0, 65536 ) );
      }
      hist[ i ] -> clear();
    }



    datarecord -> reset();
    while ( ( datasection = datarecord -> next() ) ) {
      datasection -> reset();
      while ( ( datasegment = datasection -> next() ) ) {
	datasegment -> reset();
	while ( ( dataelement = datasegment -> next() ) ) {
	  ( databuffer[ histid ] ).push_back( (KFloat)( *( (KInt*)( dataelement -> getStore() ) ) ) );

	  for ( KInt i = 0; i < (KInt)( ( databuffer[ histid ] ).size() ); i ++ )
	    hist[ histid ] -> fill( new KonoeFillData( ( databuffer[ histid ] )[ i ] ) );

	  histid ++;

	}
      }
    }
  


    for ( KInt i = 0; i < kNumberOfHistograms; i ++ ) {
      ofile -> put( hist[ i ] );
      canvas[ i ] -> setDataSource( hist[ i ] );
      canvas[ i ] -> draw();
    }

    canvas[ 0 ] -> update();
    delete datarecord;
    sleep( kSamplingTimeInSecond );
  }


  mainwindow -> mainLoop();
  delete isocket;
  delete manager;


  return( 0 );
}
