#include <system/KonoeOutputObjFile.hh>

#include <daq/KonoeReadoutList.hh>
#include <daq/KonoeReadoutSection.hh>
#include <daq/KonoeReadoutSegment.hh>
#include <daq/KonoeSimADCModule.hh>

#include <analysis/KonoeDataRecord.hh>

#include "exConstants.hh"

int main( int argc, char** argv, char** envv )
{
  KonoeSimADCModule* adc
    = new KonoeSimADCModule( kNumberOfAdcChannels, kNumberOfAdcChannels );

  KonoeSimADCModule* swreg
    = new KonoeSimADCModule( kNumberOfSwitchRegisterChannels,
			     kNumberOfSwitchRegisterChannels, 
			     kSwitchRegisterFullScale );

  KonoeReadoutList* rlist
    = new KonoeReadoutList( kNumberOfSections, kListID );

  KonoeReadoutSection* rsec
    = new KonoeReadoutSection( kNumberOfSegments, kSectionID );

  KonoeReadoutSegment* rseg0
    = new KonoeReadoutSegment( adc, kAdcID );

  KonoeReadoutSegment* rseg1 =
    new KonoeReadoutSegment( swreg, kSwitchRegisterID );

  rsec -> add( rseg0 );
  rsec -> add( rseg1 );
  rlist -> add( rsec );

  KonoeOutputObjFile* ofile =
    new KonoeOutputObjFile( kFileNameForDataRecord );
  KonoeDataRecord* datarecord = 0;

  for ( KInt i = 0; i < kNumberOfEvents; i ++ ) {
    datarecord = rlist -> read();
    datarecord -> dump( cout );
    ofile -> put( datarecord );
    adc -> update();
    swreg -> update();
    delete datarecord;
  }

  delete rlist;
  delete ofile;

  return( 0 );
}
