//===================================================================
/*

	demoread.cc at konoe/demo

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	15-Oct-1997	H. Sakamoto
	Version 0.4	30-Oct-1997	H. Sakamoto
	Version 0.2.0	18-May-1998	H. Sakamoto

*/
//===================================================================

#include	<analysis/KonoeDataAnalyzer.hh>
#include	<analysis/KonoeGeneralSectionAnalyzer.hh>
#include	<analysis/KonoeHitSectionAnalyzer.hh>
#include	<iostream.h>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<system/KonoeInputObjFile.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeHistoFillerList.hh>
#include	<analysis/KonoeFillDataExtractor.hh>
#include	<analysis/Konoe1DHistogram.hh>
#include	<analysis/KonoeDetectorList.hh>
#include	<analysis/KonoeBoxGeometry.hh>
#include	<analysis/KonoeDataAddressSet.hh>
#include	<analysis/KonoeTDCCalibrator.hh>
#include	<analysis/KonoeHitDataset.hh>


//-------------------------------------------------------------------

//	actual application class starts here

//-------------------------------------------------------------------

class	Demo {
public :
	enum	{ RECORD_TYPE = 101 };
	enum	{
			GENERAL_SECTION,
			COUNTER_ADC_SECTION,
			COUNTER_TDC_SECTION,
		N_SECTIONS };
	enum	{ EVENT_INFO, N_GENERAL_SEGMENTS };
	enum	{ TDC_1, N_TDC_SEGMENTS };
	enum	{ N_TDC_CHANNELS = 16 };
	enum	{ N_MULTIPLETS = 256 };
	enum	{
			N_COUNTERS = 32,
			DETECTOR_ID = 1
		};
protected :
	KonoeDataAnalyzer	* analyzer;
	KonoeInputObjFile	* file;
	KonoeDataRecordCM	* data_record_cm;
	KonoeLinearDataScale	* tdc_scale;
	Konoe1DHistogram	* tdc_histogram;
	KonoeHistoFillerList	* filler_list;
	KonoeFillDataExtractor	* fill_data_extractor;
	KonoeHitSectionAnalyzer	* hit_section_analyzer;
	KonoeDetectorList	* detector_list;
	KonoeDetector		* detector;
	KonoeDataAddressSet	* tdcaddress;
	KonoeTDCCalibrator	* calibrator;
	KonoeHitDataset		* hitset;

	KFloat	counter_width;
	KFloat	counter_height;
	KFloat	counter_thickness;
	KFloat	detector_distance;

public :
	Demo( int argc, char ** argv );
	~Demo( );
	int	go( );
};

Demo::Demo( int, char ** )
	: counter_width( 0.1 ), counter_height( 1.0 ),
	counter_thickness( 0.05 ), detector_distance( 5.0 ) {

	Konoe3Vector	p, dp;
	KonoeTranslation	t;
	KonoeBoxGeometry	* b;
	KonoeDetectorElement	* e;

	KInt	md, ch;

	tdcaddress = new KonoeDataAddressSet( COUNTER_TDC_SECTION, 2,
				N_COUNTERS / 2 );
	detector_list = new KonoeDetectorList( 1 );
	detector = new KonoeDetector( N_COUNTERS, DETECTOR_ID );
	detector_list -> add( detector );
	p.set( counter_width * 0.5 - counter_width * N_COUNTERS / 2.0,
		- counter_height * 0.5, detector_distance );
	dp.set( counter_width, 0, 0 );
	for( KInt i = 0; i < N_COUNTERS; i ++ ) {
	    b = new KonoeBoxGeometry( counter_width, counter_height,
			counter_thickness );
	    t.setDisplacement( p );
	    b -> translate( & t );
	    detector -> add( e = new KonoeDetectorElement( b,
		detector, i ) );
	    md = i / ( N_COUNTERS / 2 );
	    ch = i % ( N_COUNTERS / 2 );
	    e -> add( new KonoeDataAddress( COUNTER_ADC_SECTION, md, ch ) );
	    e -> add( tdcaddress -> getAddress( md, ch ) );
	    p += dp;
	}

	calibrator = new KonoeTDCCalibrator( 0.0, 1.0 );
	hitset = new KonoeHitDataset( N_COUNTERS, 0, 0 );

	analyzer = new KonoeDataAnalyzer( N_SECTIONS );
	analyzer -> install( hit_section_analyzer = 
		new KonoeHitSectionAnalyzer( COUNTER_TDC_SECTION,
		detector, calibrator, hitset ) );
	analyzer -> install( new KonoeGeneralSectionAnalyzer( ) );

	file = new KonoeInputObjFile( "test.dat" );
	data_record_cm = new KonoeDataRecordCM( );
	file -> installCM( data_record_cm );
	tdc_scale = new KonoeLinearDataScale( 0.0, 4096.0, 256 );
	tdc_histogram = new Konoe1DHistogram( tdc_scale );
	fill_data_extractor = new KonoeFillDataExtractor(
		hit_section_analyzer -> getPhysicalValue( ) );
	fill_data_extractor -> setIndex(
		KonoeHitDataset::HIT_TIME_INDEX );
	filler_list = new KonoeHistoFillerList( 1 );
	filler_list -> add( new KonoeHistogramFiller(
		fill_data_extractor, tdc_histogram ) );
}

Demo::~Demo( ) {

	delete file;
}

int	Demo::go( ) {
	KonoeDataRecord	* rec;

	tdc_histogram -> clear( );

	while( 1 ) {
	    rec = ( KonoeDataRecord * ) file -> get( 0 );
	    if( rec == 0 ) break;
//	    rec -> dump( cout );
	    analyzer -> reset( );
	    analyzer -> analyze( rec );
//		analyzer -> dumpPhysicalValue( cout );
	    filler_list -> fill( );
	    delete rec;
	}
	tdc_histogram -> dump( cout );
	return 0;
}


//-------------------------------------------------------------------

//	entry point

//-------------------------------------------------------------------

main( int argc, char ** argv ) {
	Demo demo( argc, argv );
	return demo.go( );
}
