//===================================================================
/*

	demoread.cc at konoe/demo/socket

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	15-Oct-1997	H. Sakamoto
	Version 0.4	25-Oct-1997	H. Sakamoto

*/
//===================================================================

#include	<analysis/KonoeDataAnalyzer.hh>
#include	<analysis/KonoeSectionAnalyzer.hh>
#include	<analysis/KonoePhysIntElement.hh>
#include	<iostream.h>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<system/KonoeInputObjSocket.hh>

//-------------------------------------------------------------------

//	private classes

//-------------------------------------------------------------------

class	EventInfoValue	: public KonoePhysicalValue {

protected :
	KonoePhysIntElement	event_number;
public :
	EventInfoValue( KInt ptag ) : KonoePhysicalValue( ptag ) {
	}
	KInt	getDataCount( )	{	return 1;	}
	void	set( KInt pevent_number ) {
				event_number.set( 0, pevent_number );
		}
	KonoePhysValueElement	* next( ) {
				return & event_number;	}
	void	dump( ostream & os );
};

class	GeneralSectionAnalyzer	: public KonoeSectionAnalyzer {

protected :
	EventInfoValue	* event_info;
public :
	GeneralSectionAnalyzer( KInt tag );
	KInt	analyze( KonoeDataSection * section );
	KonoePhysicalValue	* getPhysicalValue( ) {
		return event_info;
	}
};

class	TDCValue	: public KonoePhysicalValue {

protected :
	KInt	n_data;
	KInt	storagesize;
	KInt	counter;
	KonoePhysIntElement	* pointer;
	KonoePhysIntElement	* datalist;
public :
	TDCValue( KInt ptag, KInt max_channel );
	KInt	getDataCount( ) {	counter = 0;
					pointer = datalist;
					return n_data;
	}
	KonoePhysValueElement	* next( );
	void	clear( )	{	n_data = 0;	}
	KInt	add( KInt address, KInt data );
	void	dump( ostream & os );
};

class	TDCSectionAnalyzer	: public KonoeSectionAnalyzer {

protected :
	TDCValue	* tdcvalue;
public :
	TDCSectionAnalyzer( KInt tag );
	KInt	analyze( KonoeDataSection * section );
	KonoePhysicalValue	* getPhysicalValue( ) {
		return tdcvalue;
	}
};

//-------------------------------------------------------------------

//	actual application class starts here

//-------------------------------------------------------------------

class	Demo {
public :
	enum	{ RECORD_TYPE = 101 };
	enum	{ GENERAL_SECTION, TDC_SECTION,
		N_SECTIONS };
	enum	{ EVENT_INFO, N_GENERAL_SEGMENTS };
	enum	{ TDC_1, N_TDC_SEGMENTS };
	enum	{ N_TDC_CHANNELS = 16 };
protected :
	KonoeDataAnalyzer	* analyzer;
	KonoeInputObjSocket	* socket;
	KonoeDataRecordCM	* data_record_cm;
	

public :
	Demo( int argc, char ** argv );
	~Demo( );
	int	go( );
};

Demo::Demo( int, char ** ) {

	analyzer = new KonoeDataAnalyzer( N_SECTIONS );
	analyzer -> install( new TDCSectionAnalyzer( TDC_SECTION ) );
	analyzer -> install( new GeneralSectionAnalyzer( GENERAL_SECTION ) );

	socket = new KonoeInputObjSocket( "kuhes3.pn.scphys.kyoto-u.ac.jp", 10000 );
	data_record_cm = new KonoeDataRecordCM( );
	socket -> installCM( data_record_cm );
}

Demo::~Demo( ) {

	delete socket;
}

int	Demo::go( ) {
	KonoeDataRecord	* rec;
	KonoePhysicalValue	* pv;
	KonoePhysValueElement	* el;

	while( 1 ) {
	    rec = ( KonoeDataRecord * ) socket -> get( 0 );
	    if( rec == 0 ) break;
	    rec -> dump( cout );
	    analyzer -> analyze( rec );
//		analyzer -> dumpPhysicalValue( cout );
	    pv = analyzer -> find( GENERAL_SECTION );
	    if( pv )
		cout << "Event # = " << pv -> next( ) -> getInteger( ) << endl;
	    else
		cout << "Event Number not found" << endl;
	    pv = analyzer -> find( TDC_SECTION );
	    if( pv ) {
		KInt	n, j;
		n = pv -> getDataCount( );
		for( j = 0; j < n; j ++ ) {
		    el = pv -> next( );
		    cout << "Channel = " << el -> getAddress( ) <<
			" Data = " << el -> getInteger( ) << endl;
		}
	    }
	    else
		cout << "TDC Data not found" << endl;

	    delete rec;
	}
	return 0;
}

//-------------------------------------------------------------------

//	private class members

//-------------------------------------------------------------------

GeneralSectionAnalyzer::GeneralSectionAnalyzer( KInt ptag )
		: KonoeSectionAnalyzer( ptag ) {

	event_info = new EventInfoValue( ptag );
}

KInt	GeneralSectionAnalyzer::analyze( KonoeDataSection * section ) {

	KonoeDataSegment	* segment;
	KonoeDataElement	* element;
	KonoePhysIntElement	* value;
	section -> reset( );
	while( segment = section -> next( ) ) {
	    switch( segment -> getID( ) ) {
	    case Demo::EVENT_INFO :
		segment -> reset( );
		element = segment -> next( );
		if( element )
		    event_info -> set( * ( KInt * ) element -> getStore( ) );
		break;
	    default :
		break;
	    }
	}
	return 0;
}

void	EventInfoValue::dump( ostream & os )	{
		os << "EventInfoValue ";
		event_number.dump( os );
}

TDCValue::TDCValue( KInt ptag, KInt max_channel )
	: KonoePhysicalValue( ptag ) {

	datalist = new KonoePhysIntElement[ max_channel ];
	storagesize = max_channel;
}

KInt	TDCValue::add( KInt paddress, KInt pdata ) {
	if( n_data >= storagesize )
		return -1;
	( datalist + n_data ) -> set( paddress, pdata );
	return n_data ++;
}

void	TDCValue::dump( ostream & os ) {

	os << "TDCValue: n_data = " << n_data << endl;
	for( int i = 0; i < n_data; i ++ )
	    ( datalist + i ) -> dump( os );

}

KonoePhysValueElement	* TDCValue::next( ) {
			if( counter >= n_data )
				return 0;
			counter ++;
			return pointer ++;
}

TDCSectionAnalyzer::TDCSectionAnalyzer( KInt ptag )
		: KonoeSectionAnalyzer( ptag ) {

	tdcvalue = new TDCValue( ptag, Demo::N_TDC_CHANNELS );

}

KInt	TDCSectionAnalyzer::analyze( KonoeDataSection * section ) {

	KonoeDataSegment	* segment;
	KonoeDataElement	* element;
	section -> reset( );
	while( segment = section -> next( ) ) {
	    switch( segment -> getID( ) ) {
	    case Demo::TDC_1 :
		tdcvalue -> clear( );
		segment -> reset( );
		while( element = segment -> next( ) ) {
		    tdcvalue -> add( element -> getAddress( ),
			* ( KInt * ) element -> getStore( ) );
		}
		break;
	    default :
		break;
	    }
	}
	return 0;
}

//-------------------------------------------------------------------

//	entry point

//-------------------------------------------------------------------

main( int argc, char ** argv ) {
	Demo demo( argc, argv );
	return demo.go( );
}
