//===================================================================
/*

	demo.cc at konoe/demo/socket

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	15-Oct-1997	H. Sakamoto
	Version 0.4	25-Oct-1997	H. Sakamoto
	Version 0.5	05-Nov-1997	H. Sakamoto
	Version 0.2.0	24-Apr-1998	H. Sakamoto

*/
//===================================================================

#include	<daq/KonoeReadoutList.hh>
#include	<daq/KonoeSoftCounter.hh>
#include	<daq/KonoeSoftRegister.hh>
#include	<daq/KonoeSimTDCModule.hh>
#include	<daq/KonoeSoftwareCrate.hh>
#include	<daq/KonoeTimerXaction.hh>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<system/KonoeServerSocket.hh>
#include	<system/KonoeOutputObjSocket.hh>
#include	<iostream.h>

//-------------------------------------------------------------------

//	actual application class starts here

//-------------------------------------------------------------------

class	Demo {
public :
	enum	{ RECORD_TYPE = 101 };
	enum	{ GENERAL_SECTION, TDC_SECTION,
		N_SECTIONS };
	enum	{ EVENT_INFO, N_GENERAL_SEGMENTS };
	enum	{ TDC_1, N_TDC_SEGMENTS };
	enum	{ N_TDC_CHANNELS = 16 };
	enum	{ EVENT_NUMBER, N_EVENT_ITEMS };
protected :
	KonoeReadoutList	* list;
	KonoeSoftCounter	* event_number_module;
	KonoeSoftRegister	* run_number_module;
	KonoeSimTDCModule	* tdc_module;
	KonoeSoftwareCrate	* crate;
	KonoeTimerXaction	* xaction;
	KonoeServerSocket	* socket;
	KonoeOutputObjSocket	* osocket;
	KonoeDataRecordCM	* data_record_cm;
	

public :
	Demo( int argc, char ** argv );
	~Demo( );
	int	go( );
};

Demo::Demo( int, char ** ) {
	KonoeReadoutSection	* psec;
	KonoeReadoutSegment	* pseg;

	event_number_module = new KonoeSoftCounter( );
	tdc_module = new KonoeSimTDCModule( N_TDC_CHANNELS, 4 );

	list = new KonoeReadoutList( N_SECTIONS, RECORD_TYPE );
	list -> add( psec = new KonoeReadoutSection(
		N_GENERAL_SEGMENTS, GENERAL_SECTION ) );
	psec -> add( pseg = new KonoeReadoutSegment( 1, EVENT_INFO ) );
	pseg -> add( new KonoeReadoutItem(
			event_number_module, EVENT_NUMBER ) );
	list -> add( psec = new KonoeReadoutSection(
		N_TDC_SEGMENTS, TDC_SECTION ) );
	psec -> add( new KonoeReadoutSegment( tdc_module, TDC_1 ) );

	crate = new KonoeSoftwareCrate( 2 );
	crate -> install( event_number_module );
	crate -> install( tdc_module );
	crate -> initialize( );

	socket = new KonoeServerSocket( 10000 );
	osocket = new KonoeOutputObjSocket( socket );
	data_record_cm = new KonoeDataRecordCM( );
	osocket -> installCM( data_record_cm );
}

Demo::~Demo( ) {
	delete list;

	delete socket;
}

int	Demo::go( ) {
	KonoeDataRecord	* rec;
	KInt	s;

	list -> dump( cout );
	crate -> clear( );
	KonoeSignalXaction::initialize( );
	xaction = new KonoeTimerXaction( list );
	xaction -> setInterval( 2000 );
	xaction -> enable( );

	for( KInt i = 0; i < 1000; i ++ ) {
	    socket -> wait( 1000 );
	    rec = xaction -> read( );
	    rec -> dump( cout );
	    osocket -> put( rec, 'A', 0 );
			// must be const. defined elsewhere
	    crate -> update( );
	    delete rec;
	}
	return 0;
}


//-------------------------------------------------------------------

//	entry point

//-------------------------------------------------------------------

main( int argc, char ** argv ) {
	Demo demo( argc, argv );
	return demo.go( );
}
