//===================================================================
/*

	demoreceive.cc at konoe/demo2/socket

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	15-Oct-1997	H. Sakamoto
	Version 0.4	25-Oct-1997	H. Sakamoto
			19-Dec-1997	revised by H.Kamata
			23-Jan-1998	revised by K.Ohta (For 16channels)
	Version 0.2.0	19-May-1998	H. Sakamoto

*/
//===================================================================

#include	<analysis/KonoeDataAnalyzer.hh>
#include	<analysis/KonoeSectionAnalyzer.hh>
#include	<iostream.h>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<system/KonoeInputObjSocket.hh>

// revised by H.Kamata
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <system/KonoeOutputObjSocket.hh>

#include	<analysis/KonoeHistoFillerList.hh>
#include	<analysis/KonoeDefaultSectionAnalyzer.hh>
#include	<analysis/KonoeGeneralSectionAnalyzer.hh>

extern "C"	int	gethostname( char *, int );

//-------------------------------------------------------------------

//	actual application class starts here

//-------------------------------------------------------------------

class	Demo {
public :
	enum	{ RECORD_TYPE = 101 };
	enum	{ GENERAL_SECTION, TDC_SECTION,
		N_SECTIONS };
	enum	{ EVENT_INFO, N_GENERAL_SEGMENTS };
	enum	{ TDC_1, N_TDC_SEGMENTS };
	enum	{ N_TDC_CHANNELS = 16 };
	enum	{ N_MULTIPLETS = 256 };
protected :
	KonoeDataAnalyzer	* analyzer;
	KonoeInputObjSocket	* socket;
	KonoeDataRecordCM	* data_record_cm;
	
        // revised by H.Kamata
        //     for output Histogram to socket
        KonoeLinearDataScale    * tdc_scale;
	KonoeLinearDataScale	* subaddress_scale;
        Konoe2DHistogram	* tdc_histogram;
        KonoeOutputObjSocket    * histogram_socket;
	KonoeHistoFillerList	* filler_list;
	KonoeFillDataExtractor	* fill_data_extractor;
	KonoeDefaultSectionAnalyzer	* default_section_analyzer;

public :
	Demo( int argc, char ** argv );
	~Demo( );
	int	go( );
};

Demo::Demo( int, char ** ) {

	KChar	hostname[ 64 ];

	analyzer = new KonoeDataAnalyzer( N_SECTIONS );
	analyzer -> install( default_section_analyzer = 
		new KonoeDefaultSectionAnalyzer( TDC_SECTION,
			N_MULTIPLETS, 0 ) );
	analyzer -> install( new KonoeGeneralSectionAnalyzer(  ) );

	gethostname( hostname, 64 );
	socket = new KonoeInputObjSocket( hostname, 10000);
	data_record_cm = new KonoeDataRecordCM( );
	socket -> installCM( data_record_cm );

	// revised by H.Kamata
	//    for output Histogram to socket
	histogram_socket = new KonoeOutputObjSocket( 15000 ); 
	histogram_socket -> installCM( data_record_cm );

	tdc_scale = new KonoeLinearDataScale( 0.0, 4096.0, 4096 );
	subaddress_scale = new KonoeLinearDataScale( 0.0, 16.0, 16 );
	tdc_histogram = new Konoe2DHistogram(
		subaddress_scale, tdc_scale );

	fill_data_extractor = new KonoeFillDataExtractor(
		default_section_analyzer -> getPhysicalValue( ) );
	fill_data_extractor -> setIndex(
		KonoeRawDataset::SUBADDRESS_INDEX,
		KonoeRawDataset::DATA_INDEX );
	filler_list = new KonoeHistoFillerList( 1 );
	filler_list -> add( new KonoeHistogramFiller(
		fill_data_extractor, tdc_histogram ) );
}

Demo::~Demo( ) {

	delete socket;
}

int	Demo::go( ) {
	KonoeDataRecord	* rec;

	tdc_histogram -> clear( );
  	int private_counter = 0;

	while( 1 ) {
	    rec = ( KonoeDataRecord * ) socket -> get( 0 );
	    if( rec == 0 ) break;
//	    rec -> dump( cout );
	    analyzer -> reset( );
	    analyzer -> analyze( rec );
//		analyzer -> dumpPhysicalValue( cout );
	    filler_list -> fill( );

  		private_counter++;
//OOTA HAND
		// send histogram object to UI
  		if ( private_counter > 10 ) {
		    histogram_socket -> put ( tdc_histogram ); 
  		    private_counter = 0;
  		}
//OOTA HAND

	    delete rec;
	}
	return 0;
}

//-------------------------------------------------------------------

//	entry point

//-------------------------------------------------------------------

main( int argc, char ** argv ) {
	Demo demo( argc, argv );
	return demo.go( );
}
