/*==================================================================== 
   KONOE Analysis package demonstraition program   [DemoAnalysis]

   version      0.2.0   Takagi Daisuke, Sanshiro Enomoto  10-Mar-1998
  ====================================================================*/


#include <iostream.h>

#include <system/KonoeInputObjFile.hh>
#include <system/KonoeOutputObjFile.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjSocket.hh>


#include <analysis/KonoeDataRecordCM.hh>
#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoePhysicalValue.hh>
#include <analysis/KonoePhysIntElement.hh>
#include <analysis/KonoePhysicalValue.hh>
#include <analysis/KonoePhysLinearScale.hh>
#include <analysis/KonoePhysIntElement.hh>
#include <analysis/KonoeSectionAnalyzer.hh>
#include <analysis/KonoeDataAnalyzer.hh>
#include <analysis/KonoeDataRecord.hh>

#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeFillData.hh>
#include <analysis/KonoeFillDataExtractor.hh>
#include <analysis/KonoeHistogramFiller.hh>
#include <analysis/KonoeHistoFillerList.hh>

enum section_id {
    GENERAL_SECTION, 
    S1_SECTION,
    S2_SECTION,
    C1_SECTION,
    N_SECTIONS
};
enum extract_manner {
    MNR_C1ENERGY,
    MNR_C1POSITION
};

#include "EventInfoValue.hh"
#include "ADCValue.hh"
#include "GeneralSectionAnalyzer.hh"
#include "ScintilatorSectionAnalyzer.hh"
#include "CalorieMeterSectionAnalyzer.hh"
#include "DemoExtractor.hh"

#include "EventInfoValue.cc"
#include "ADCValue.cc"
#include "GeneralSectionAnalyzer.cc"
#include "ScintilatorSectionAnalyzer.cc"
#include "CalorieMeterSectionAnalyzer.cc"
#include "DemoExtractor.cc"


// ======================================================================

class DemoAnalysis {
  protected:
    KonoeDataAnalyzer *analyzer;

    Konoe1DHistogram *c1_energy_histogram;
    Konoe2DHistogram *c1_position_histogram;

    DemoC1Extractor *c1_energy_extractor;
    DemoC1Extractor *c1_position_extractor;

    KonoeHistogramFiller *c1_energy_histogram_filler;
    KonoeHistogramFiller *c1_position_histogram_filler;
  public:
    DemoAnalysis(KonoeInputObjectStream *is, KonoeOutputObjectStream *os);
    ~DemoAnalysis();

    KInt go();
  protected:
    KonoeInputObjectStream *input_stream;
    KonoeOutputObjectStream *output_stream;
    KonoeDataRecordCM *data_record_cm;
    KonoeHistogramCM *histogram_cm;
};

DemoAnalysis::DemoAnalysis(KonoeInputObjectStream *is, KonoeOutputObjectStream *os) 
: input_stream(is), output_stream(os)
{
    data_record_cm = new KonoeDataRecordCM();
    histogram_cm = new KonoeHistogramCM();
    input_stream->installCM(data_record_cm);
    output_stream->installCM(histogram_cm);

    analyzer = new KonoeDataAnalyzer(N_SECTIONS);
    analyzer->install(new GeneralSectionAnalyzer(GENERAL_SECTION));
    analyzer->install(new ScintilatorSectionAnalyzer(S1_SECTION));
    analyzer->install(new ScintilatorSectionAnalyzer(S2_SECTION));
    analyzer->install(new CalorieMeterSectionAnalyzer(C1_SECTION));

    KonoePhysValueScale *c1_energy_histo_scale = new KonoePhysLinearScale(0, 4095, 512);
    c1_energy_histogram = new Konoe1DHistogram(c1_energy_histo_scale);

    KonoePhysValueScale *c1_position_histo_xscale = new KonoePhysLinearScale(0, 31, 32);
    KonoePhysValueScale *c1_position_histo_yscale = new KonoePhysLinearScale(0, 31, 32);
    c1_position_histogram = new Konoe2DHistogram(
        c1_position_histo_xscale, 
        c1_position_histo_yscale
    );

    cout << (c1_energy_extractor = new DemoC1Extractor(MNR_C1ENERGY)) << endl;
    cout << (c1_position_extractor = new DemoC1Extractor(MNR_C1POSITION)) << endl;

    c1_energy_histogram_filler = new KonoeHistogramFiller(
	c1_energy_extractor,
	c1_energy_histogram
    );
    c1_position_histogram_filler = new KonoeHistogramFiller(
	c1_position_extractor,
	c1_position_histogram
    );

    c1_energy_histogram->clear();
    c1_position_histogram->clear();
}    

DemoAnalysis::~DemoAnalysis() 
{
    delete analyzer;
    delete c1_energy_histogram_filler;
    delete c1_position_histogram_filler;
}

KInt DemoAnalysis::go()
{
    KonoeDataRecord *record;
    while ((record = (KonoeDataRecord *) input_stream->get()) != 0) {
	record->reset();
	analyzer->analyze(record);

	KonoePhysicalValue *info_value = analyzer->find(GENERAL_SECTION);
	if (info_value != 0) {
	    ((EventInfoValue *)info_value)->reset();
	    KonoePhysValueElement *element = info_value->next();
	    cout << "Run #   = " << element->getInteger() << endl;
	    element = info_value->next();
	    cout << "Event # = " << element->getInteger() << endl;
	}
	else {
	    cerr << "Event number not found" << endl;
	}

	KonoePhysicalValue *value = analyzer->find(C1_SECTION);
	c1_energy_histogram_filler->fill(value);
	value->reset();
	c1_position_histogram_filler->fill(value);

	static int i = 0;
	if (++i % 10 == 0) {
	    output_stream->put(c1_energy_histogram, 'A', 0);
	    output_stream->put(c1_position_histogram, 'A', 0);
	    i = 0;
	}

	delete record;
    }

    c1_energy_histogram->dump(cout);
    c1_position_histogram->dump(cout);

    return 0;
}

//======================================================================

int main() 
{
#ifdef SOCKET
    KonoeOutputObjectStream *os = new KonoeOutputObjSocket(15000);
    cerr << "waiting for connection...";
    while (((KonoeOutputObjSocket *)os)->waitConnection() == -1)
        ;
    KonoeInputObjectStream *is = new KonoeInputObjSocket("jsdsn2.kek.jp", 10000);
#else
    KonoeOutputObjectStream *os = new KonoeOutputObjFile("DemoHisto.dat");
    KonoeInputObjectStream *is = new KonoeInputObjFile("DemoSource.dat");
#endif

    DemoAnalysis demo_analysis(is, os);
    int result = demo_analysis.go();

    delete os;
    delete is;

    return result;
}
