/*==================================================================== 
             KONOE demonstraition program   [demosend]

           original version   H.Sakamoto
           version      1.0   Takagi Daisuke    10-Mar-1998
  ====================================================================*/
#include        <iostream.h>
#include        <stdio.h>
#include        <string.h>

#include        <daq/KonoeReadoutList.hh>
#include        <daq/KonoeSoftCounter.hh>
#include        <daq/KonoeSimADCModule.hh>
#include        <daq/KonoeSoftwareCrate.hh>
#include        <daq/KonoeTimerXaction.hh>
#include        <analysis/KonoeDataRecordCM.hh>
#include        <system/KonoeOutputObjSocket.hh>
#include	<system/KonoeServerSocket.hh>
#include        <system/KonoeOutputObjFile.hh>
//----------------------------------------------------------------------
class   DemoSend {
  public:
    enum record_id  { DEMO_RECORD };
    enum section_id { GENERAL_SECTION, S1_SECTION, S2_SECTION, C1_SECTION };
    enum segment_id { EVENT_INFO, S1_SEGMENTS, S2_SEGMENTS, C1_SEGMENTS };
    enum element_id { RUN_NUMBER, EVENT_NUMBER };

    enum number_of_sections {
        N_SECTIONS = 4
    };

    enum number_of_segments {
        N_GENERAL_SEGMENTS = 1, 
	N_S1_SEGMENTS = 4,  
	N_S2_SEGMENTS = 4,  
	N_C1_SEGMENTS = 64
    };

    enum number_of_elements {
        N_EVENT_INFO_ELEMENTS = 2
    };

    enum number_of_adc_moodules {
        N_ADC_MODULES = N_S1_SEGMENTS + N_S2_SEGMENTS + N_C1_SEGMENTS
    };

    enum number_of_adc_channels {
        N_ADC_CHANNELS = 16
    };

  protected:
    // DataList
    KonoeReadoutList     *list;
    KonoeReadoutSection  *general_section;
    KonoeReadoutSection  *s1_section;
    KonoeReadoutSection  *s2_section;
    KonoeReadoutSection  *c1_section;

    KonoeReadoutSegment  *event_info_segment;
    KonoeReadoutSegment  *s1_segments[ N_S1_SEGMENTS ];
    KonoeReadoutSegment  *s2_segments[ N_S2_SEGMENTS ];
    KonoeReadoutSegment  *c1_segments[ N_C1_SEGMENTS ];

    KonoeReadoutItem     *run_number_item;
    KonoeReadoutItem     *event_number_item;

    // Module
    KonoeSoftCounter     *event_number_module;
    KonoeSoftCounter     *run_number_module;
    KonoeSimADCModule    *adc_modules[ N_ADC_MODULES ];

    // DataSource
    KonoeSoftwareCrate   *crate;
    
  public:
    DemoSend();
    ~DemoSend();

    KInt go(KonoeOutputObjectStream *out);
};
// ----------------------------------------------------------------------
DemoSend::DemoSend() {
    KInt i;

    run_number_module   = new KonoeSoftCounter( );
    event_number_module = new KonoeSoftCounter( );
    for ( i = 0; i < N_ADC_MODULES; i++ ) {
	adc_modules[ i ] = new KonoeSimADCModule( N_ADC_CHANNELS, 4 );
    }

    // LIST ------------------------------------------------------------
    general_section = new KonoeReadoutSection( 1, GENERAL_SECTION );
    s1_section      = new KonoeReadoutSection( N_S1_SEGMENTS, S1_SECTION );
    s2_section      = new KonoeReadoutSection( N_S2_SEGMENTS, S2_SECTION );
    c1_section      = new KonoeReadoutSection( N_C1_SEGMENTS, C1_SECTION );
    
    list = new KonoeReadoutList( N_SECTIONS, DEMO_RECORD );
    list -> add( general_section );
    list -> add( s1_section );
    list -> add( s2_section );
    list -> add( c1_section );

    // GENERAL SECTION --------------------------------------------------
    event_info_segment = new KonoeReadoutSegment( N_EVENT_INFO_ELEMENTS, 0 );
    run_number_item    = new KonoeReadoutItem( run_number_module,   RUN_NUMBER );
    event_number_item  = new KonoeReadoutItem( event_number_module, EVENT_NUMBER );

    general_section -> add( event_info_segment );
    event_info_segment -> add( run_number_item );
    event_info_segment -> add( event_number_item );

    // S1 SECTION --------------------------------------------------
    for ( i = 0; i < N_S1_SEGMENTS; i++ ) {
	s1_segments[ i ] = new KonoeReadoutSegment( adc_modules[ i ], i );
	s1_section -> add( s1_segments[ i ] );
    }

    // S2 SECTION --------------------------------------------------
    for ( i = 0; i < N_S1_SEGMENTS; i++ ) {
	s2_segments[ i ] = new KonoeReadoutSegment( adc_modules[ i ], i );
 	s2_section -> add( s2_segments[ i ] );	
    }

    // C1 SECTION --------------------------------------------------
    for ( i = 0; i < N_C1_SEGMENTS; i++ ) {
	c1_segments[ i ] = new KonoeReadoutSegment( adc_modules[ i ], i );
	c1_section -> add( c1_segments[ i ] );
    }
    // ----------------------------------------------------------------------

    crate = new KonoeSoftwareCrate( N_EVENT_INFO_ELEMENTS + N_ADC_MODULES );
    crate -> install( run_number_module );
    crate -> install( event_number_module );

    for ( i = 0; i < N_ADC_MODULES; i++ ) {
	crate -> install( adc_modules[ i ] );
    }

    crate -> initialize();
}

DemoSend::~DemoSend() {
    delete list;
    delete crate;

    delete run_number_module;
    delete event_number_module;
    for (KInt i = 0; i < N_ADC_MODULES; i++) {
	delete adc_modules[ i ];
    }
}

KInt DemoSend::go(KonoeOutputObjectStream *out) {
    KonoeDataRecordCM *data_record_cm = new KonoeDataRecordCM( );
    out -> installCM( data_record_cm );

    KonoeSignalXaction::initialize( );
    KonoeTimerXaction *xaction = new KonoeTimerXaction( list );
    xaction -> setInterval( 2000 );
    xaction -> enable( );
    
    crate -> clear( );

    while ( 1 ) {
	KonoeDataRecord *record = xaction -> read( );
	record->reset();

	out -> put( record, 'A', 0 );

	crate -> update( );
	delete record;
    }

    delete data_record_cm;
    delete xaction;

    return 0;
}
// ======================================================================
void output_usage() {
    cerr << "usage: demosend -file filename" << endl;
    cerr << "       demosend -socket port"   << endl;
}
//----------------------------------------------------------------------
int main( int argc, char** argv ) {
    if ( argc != 3 ) {
	output_usage();
	return -1;
    }
    
    KonoeOutputObjectStream *out = 0;
    if ( strcmp( argv[ 1 ], "-file" ) == 0 ) {
	out = new KonoeOutputObjFile( argv[ 2 ] );

	if ( out == 0 ) {
	    cerr << "error: wrong file name" << endl;
	    output_usage();
	    return -1;
	}
    } 
    else if ( strcmp( argv[ 1 ], "-socket" ) == 0) {
	int port = 0;
	sscanf( argv[ 2 ], "%d", &port );
	if ( port <= 0 ) {
	    cerr << "error: wrong port number" << endl;
	    output_usage();
	    return -1;
	}
	KonoeServerSocket * socket = new KonoeServerSocket( port );
	out = new KonoeOutputObjSocket( socket );

	cerr << "waiting for connection...";
	while( socket -> waitConnection( ) == -1 );
	cerr << " done." << endl;
    }
    else {
	output_usage( );
	return -1;
    }

    DemoSend demosend;
    int return_value = demosend.go( out );

    delete out;

    return return_value;
}
