//============================================================
/*

	HVControl.java
	
	Somewhat realistic HV control demonstration
	
	Version 0.0.1	22-Jun-1998	H. Sakamoto

*/
//============================================================

import	java.applet.*;
import	java.awt.*;
import	java.awt.event.*;
import	konoe.ui.*;
import	HVStatus;
import	HVSimulator;

public class HVControl extends Applet implements Runnable, ActionListener {

  static final int N_SUBSYSTEMS = 7;
  HVStatus [] statuspanel = new HVStatus[ N_SUBSYSTEMS ];
  HVSimulator [] simulator = new HVSimulator[ N_SUBSYSTEMS ];
  static final String [] subsystemnames = {
    "CDC", "KLM", "CAL", "ACC", "TOF", "SVD", "EFC" };
  static final String [] buttonlabels = {
    "Offline", "Standby", "Shoulder", "Peak", "PANIC",
    "Trip Reset", "Hold" };
  static final int OFFLINE = 0;
  static final int STANDBY = 1;
  static final int SHOULDER = 2;
  static final int PEAK = 3;
  static final int PANIC = 4;
  static final int TRIP_RESET = 5;
  static final int HOLD = 6;
  
  Thread	thread;
  int		delay;
  
  Panel		switchpanel;
  Button	offlinebutton;
  Button	standbybutton;
  Button	shoulderbutton;
  Button	peakbutton;
  Label		connectlabel;
  Label		ascendlabel;
  Label		attacklabel;
  Label		withdrawlabel;
  Label		descendlabel;
  Label		disconnectlabel;

  Button	panicbutton;
  Label		triplabel;
  Button	tripresetbutton;
  Label		busylabel;
  Button	holdbutton;

  
  int		currentstate;
  int		targetstate;
  boolean	hold;

  public HVControl( ) {
    delay = 500;	// millisec.
  }
  
  public void init( ) {
    int	i;
    
    for( i = 0; i < N_SUBSYSTEMS; i ++ ) {
      statuspanel[ i ] = new HVStatus( subsystemnames[ i ] );
      add( statuspanel[ i ] );
      simulator[ i ] = new HVSimulator( );
    }
    
    switchpanel = new Panel( new GridLayout( 1, 7 ) );
    
    offlinebutton = new Button( buttonlabels[ OFFLINE ] );
    offlinebutton.addActionListener( this );
    switchpanel.add( offlinebutton );
    
    Panel p = new Panel( new GridLayout( 2, 1 ) );
    connectlabel = new Label( "Connect ->" );
    connectlabel.setAlignment( Label.RIGHT );
    p.add( connectlabel );
    disconnectlabel = new Label( "<- Disconnect" );
    p.add( disconnectlabel );
    switchpanel.add( p );
    
    standbybutton = new Button( buttonlabels[ STANDBY ] );
    standbybutton.addActionListener( this );
    switchpanel.add( standbybutton );
    
    p = new Panel( new GridLayout( 2, 1 ) );
    ascendlabel = new Label( "Ascend ->" );
    ascendlabel.setAlignment( Label.RIGHT );
    p.add( ascendlabel );
    descendlabel = new Label( "<- Descend" );
    p.add( descendlabel );
    switchpanel.add( p );
    
    shoulderbutton = new Button( buttonlabels[ SHOULDER ] );
    shoulderbutton.setEnabled( false );
    shoulderbutton.addActionListener( this );
    switchpanel.add( shoulderbutton );
    
    p = new Panel( new GridLayout( 2, 1 ) );
    attacklabel = new Label( "Attack ->" );
    attacklabel.setAlignment( Label.RIGHT );
    p.add( attacklabel );
    withdrawlabel = new Label( "<- Withdraw" );
    p.add( withdrawlabel );
    switchpanel.add( p );
    
    peakbutton = new Button( buttonlabels[ PEAK ] );
    peakbutton.addActionListener( this );
    peakbutton.setEnabled( false );
    switchpanel.add( peakbutton );
    
    add( switchpanel );
    
    Panel pp = new Panel( new GridLayout( 1, 5 ) );
    panicbutton = new Button( buttonlabels[ PANIC ] );
    panicbutton.addActionListener( this );
    pp.add( panicbutton );
    p = new Panel( new GridLayout( 2, 1 ) );
    p.add( new Label( "Trip Status" ) );
    triplabel = new Label( "Safe" );
    p.add( triplabel );
    pp.add( p );
    tripresetbutton = new Button( buttonlabels[ TRIP_RESET ] );
    tripresetbutton.addActionListener( this );
    pp.add( tripresetbutton );
    p = new Panel( new GridLayout( 2, 1 ) );
    p.add( new Label( "System Status" ) );
    busylabel = new Label( "Ready" );
    p.add( busylabel );
    pp.add( p );
    holdbutton = new Button( buttonlabels[ HOLD ] );
    holdbutton.addActionListener( this );
    pp.add( holdbutton );
    
    add( pp );
    
    currentstate = HVStatus.OFFLINE;
    targetstate = HVStatus.OFFLINE;
    hold = false;

  }
  
  public void start( ) {
  
    if( thread == null ) {
      thread = new Thread( this );
    }
    thread.start( );
  }
  
  public void stop( ) {
  
    thread = null;

  }
  
  public void run( ) {
  
    Thread.currentThread( ).setPriority( Thread.MIN_PRIORITY );
    long	starttime = System.currentTimeMillis( );
    while( Thread.currentThread( ) == thread ) {
      updateDisplay( );
      try {
        starttime += delay;
        Thread.sleep( Math.max( 0, starttime
          - System.currentTimeMillis( ) ) );
      }
      catch( InterruptedException e ) {
        break;
      }
    }
  }
  
  public void actionPerformed( ActionEvent e ) {
  
    String	command = e.getActionCommand( );
    System.out.println( "Command = " + command );
    if( command.equals( buttonlabels[ OFFLINE ] ) ) {
      hold = false;
      if( currentstate == HVStatus.STANDBY ) {
        setState( HVStatus.DISCONNECTING );
      }
    }
    else if( command.equals( buttonlabels[ STANDBY ] ) ) {
      hold = false;
      switch( currentstate ) {
      case HVStatus.OFFLINE :
        setState( HVStatus.CONNECTING );
        break;
      case HVStatus.SHOULDER :
      case HVStatus.ASCENDING :
        setState( HVStatus.DESCENDING );
        break;
      }
    }
    else if( command.equals( buttonlabels[ SHOULDER ] ) ) {
      hold = false;
      switch( currentstate ) {
      case HVStatus.STANDBY :
      case HVStatus.DESCENDING :
        setState( HVStatus.ASCENDING );
        break;
      case HVStatus.PEAK :
      case HVStatus.ATTACKING :
        setState( HVStatus.WITHDRAWING );
        break;
      }
    }
    else if( command.equals( buttonlabels[ PEAK ] ) ) {
      hold = false;
      switch( currentstate ) {
      case HVStatus.SHOULDER :
      case HVStatus.WITHDRAWING :
        setState( HVStatus.ATTACKING );
        break;
      }
    }
    else if( command.equals( buttonlabels[ PANIC ] ) ) {
      hold = false;
      switch( currentstate ) {
      case HVStatus.OFFLINE :
      case HVStatus.DISCONNECTING :
      case HVStatus.CONNECTING :
      case HVStatus.STANDBY :
        break;
      default :
        setState( HVStatus.DESCENDING );
        break;
      }
    }
    else if( command.equals( buttonlabels[ TRIP_RESET ] ) ) {
    }
    else if( command.equals( buttonlabels[ HOLD ] ) ) {
      hold = ! hold;
    }
  }
  
  void updateDisplay( ) {
  
    int count, state;
    
    count = 0;
    for( int i = 0; i < N_SUBSYSTEMS; i ++ ) {
    
      if( ! hold )
        simulator[ i ].update( );
      state = simulator[ i ].getState( );
      statuspanel[ i ].setState( state );
      statuspanel[ i ].setValues(
      	simulator[ i ].getVoltage( ), simulator[ i ].getCurrent( ) );
      switch( currentstate ) {
      case HVStatus.DESCENDING :
      case HVStatus.CONNECTING :
        if( state == HVStatus.STANDBY )
          count ++;
        break;
      case HVStatus.DISCONNECTING :
        if( state == HVStatus.OFFLINE )
          count ++;
        break;
      case HVStatus.ASCENDING :
      case HVStatus.WITHDRAWING :
        if( state == HVStatus.SHOULDER )
          count ++;
        break;
      case HVStatus.ATTACKING :
        if( state == HVStatus.PEAK )
          count ++;
        break;
      }
      	
    }
    switch( currentstate ) {
    case HVStatus.DESCENDING :
    case HVStatus.CONNECTING :
      if( count == N_SUBSYSTEMS ) {
        setState( HVStatus.STANDBY );
      }
      break;
    case HVStatus.DISCONNECTING :
      if( count == N_SUBSYSTEMS ) {
        setState( HVStatus.OFFLINE );
      }
      break;
    case HVStatus.ASCENDING :
    case HVStatus.WITHDRAWING :
      if( count == N_SUBSYSTEMS )
        setState( HVStatus.SHOULDER );
      break;
    case HVStatus.ATTACKING :
      if( count == N_SUBSYSTEMS )
        setState( HVStatus.PEAK );
      break;
    }
  }
  
  void setState( int newstate ) {
  
    currentstate = newstate;
    for( int i = 0; i < N_SUBSYSTEMS; i ++ ) {
    
      simulator[ i ].setState( newstate );

    }
    switch( currentstate ) {
    case HVStatus.OFFLINE :
      shoulderbutton.setEnabled( false );
      break;
    case HVStatus.STANDBY :
      shoulderbutton.setEnabled( true );
      peakbutton.setEnabled( false );
      offlinebutton.setEnabled( true );
      break;
    case HVStatus.SHOULDER :
      peakbutton.setEnabled( true );
      offlinebutton.setEnabled( false );
      standbybutton.setEnabled( true );
      break;
    case HVStatus.PEAK :
      standbybutton.setEnabled( false );
    }
  }
}