//===========================================================
/*

	HVSimulator.java
	
	Version 0.0.1	23-Jun-1998	H. Sakamoto

*/
//===========================================================

import	HVStatus;

public class HVSimulator {

  double	voltage;
  double	current;
  double	peakvoltage;
  double	shouldervoltage;
  double	voltagenoise;
  double	currentnoise;
  double	ascendrate;
  double	attackrate;
  double	withdrawrate;
  double	descendrate;
  int		state;
  int		delay;
  int		connectiondelay;
  int		disconnectiondelay;

  public HVSimulator( ) {
  
    connectiondelay = 3;
    disconnectiondelay = 3;
    voltage = 0;
    current = 0;
    voltagenoise = 2;
    currentnoise = 10;
    peakvoltage = 100;
    shouldervoltage = 80;
    ascendrate = 5;
    attackrate = 1;
    withdrawrate = 5;
    descendrate = 10;
  }
  

  public int getState( ) {
  
    return state;

  }
  
  public double getCurrent( ) {
  
    return current + currentnoise * ( Math.random( ) - 0.5 );

  }
  
  public double getVoltage( ) {
  
    return voltage + voltagenoise * ( Math.random( ) - 0.5 );

  }
  
  public void update( ) {
  
    switch( state ) {
    case HVStatus.OFFLINE :
      break;
    case HVStatus.CONNECTING :
      delay --;
      if( delay == 0 ) {
        state = HVStatus.STANDBY;
      }
      break;
    case HVStatus.STANDBY :
      break;
    case HVStatus.ASCENDING :
      voltage += ascendrate;
      if( voltage > shouldervoltage ) {
        voltage = shouldervoltage;
        state = HVStatus.SHOULDER;
      }
      break;
    case HVStatus.SHOULDER :
      break;
    case HVStatus.ATTACKING :
      voltage += attackrate;
      if( voltage >= peakvoltage ) {
        voltage = peakvoltage;
        state = HVStatus.PEAK;
      }
      break;
    case HVStatus.PEAK :
      break;
    case HVStatus.WITHDRAWING :
      voltage -= withdrawrate;
      if( voltage <= shouldervoltage ) {
        voltage = shouldervoltage;
        state = HVStatus.SHOULDER;
      }
      break;
    case HVStatus.DESCENDING :
      voltage -= descendrate;
      if( voltage <= 0 ) {
        voltage = 0;
        state = HVStatus.STANDBY;
      }
      break;
    case HVStatus.DISCONNECTING :
      delay --;
      if( delay == 0 ) {
        state = HVStatus.OFFLINE;
      }
      break;
    case HVStatus.TRIPPED :
      break;
    }
    double	d;
    
    d = voltage / peakvoltage;
    current = d * d * d * d * 100;
    
  }

  public void setState( int newstate ) {
  
    state = newstate;
    switch( state ) {
    case HVStatus.CONNECTING :
      delay = connectiondelay;
      break;
    case HVStatus.DISCONNECTING :
      delay = disconnectiondelay;
      break;
    }
  }

}