//====================================================================
/*

	DemoAnalysis.cc

	Version 0.4.1	 02-Aug-1999	Sanshiro Enomoto

*/
//====================================================================

#include <iostream.h>
#include <stdlib.h>

#include <system/KonoeInputObjFile.hh>
#include <system/KonoeOutputObjFile.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjSocket.hh>


#include <analysis/KonoeDataRecordCM.hh>
#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoeDataRecord.hh>
#include <analysis/KonoeDataAnalyzer.hh>
#include <analysis/KonoeSectionAnalyzer.hh>
#include <analysis/KonoeGeneralSectionAnalyzer.hh>
#include <analysis/KonoeDefaultSectionAnalyzer.hh>
#include <analysis/KonoePhysicalValue.hh>
#include <analysis/KonoeRawDataset.hh>

#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeDataScale.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/KonoeFillData.hh>
#include <analysis/KonoeFillDataExtractor.hh>
#include <analysis/KonoeHistogramFiller.hh>
#include <analysis/KonoeHistoFillerList.hh>


#include "DemoAnalysis.hh"


enum section_id_t {
    GENERAL_SECTION, 
    SCIFI1_SECTION,
    SCIFI2_SECTION,
    SCIFI3_SECTION,
    SCIFI4_SECTION,
    CAL_T_SECTION,
    CAL_Q_SECTION,
    N_SECTIONS
};


DemoAnalysis::DemoAnalysis( KonoeInputObjectStream * is )
: input_stream(is) /*, output_stream(os) */
{

// setup ObjectStream //
    data_record_cm = new KonoeDataRecordCM();
    histogram_cm = new KonoeHistogramCM();
    input_stream->installCM(data_record_cm);
    //output_stream->installCM(histogram_cm);


// create and install SectionAnalyzers //
    KInt raw_dataset_size;  // number of Multiplets per PhysicalValue
    KInt tag;
    
    KonoeSectionAnalyzer * general_section_analyzer
        = new KonoeGeneralSectionAnalyzer();
    KonoeSectionAnalyzer * scifi1_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            SCIFI1_SECTION, raw_dataset_size = 1, tag = 1
        );
    KonoeSectionAnalyzer * scifi2_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            SCIFI2_SECTION, raw_dataset_size = 1, tag = 2
        );
    KonoeSectionAnalyzer * scifi3_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            SCIFI3_SECTION, raw_dataset_size = 1, tag = 3
        );
    KonoeSectionAnalyzer * scifi4_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            SCIFI4_SECTION, raw_dataset_size = 1, tag = 4
        );
    KonoeSectionAnalyzer * cal_t_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            CAL_T_SECTION, raw_dataset_size = 1, tag = 5
        );
    KonoeSectionAnalyzer * cal_q_section_analyzer
        = new KonoeDefaultSectionAnalyzer(
            CAL_Q_SECTION, raw_dataset_size = 1, tag = 6
        );

    analyzer = new KonoeDataAnalyzer(N_SECTIONS);
    analyzer->install(general_section_analyzer);
    analyzer->install(scifi1_section_analyzer);
    analyzer->install(scifi2_section_analyzer);
    analyzer->install(scifi3_section_analyzer);
    analyzer->install(scifi4_section_analyzer);
    analyzer->install(cal_t_section_analyzer);
    analyzer->install(cal_q_section_analyzer);
    

// create DataExtractors //
    KonoeFillDataExtractor *cal_q_energy_extractor;
    KonoeFillDataExtractor *cal_t_position_extractor;

    cal_q_energy_extractor = new KonoeFillDataExtractor(
        cal_q_section_analyzer->getPhysicalValue(),
        "Data"
    );

    cal_t_position_extractor = new KonoeFillDataExtractor(
        cal_t_section_analyzer->getPhysicalValue(),
        "Segment ID",
        "Subaddress"
    );


// create Histograms //
    KonoeDataScale * cal_q_energy_xscale;
    KonoeDataScale * cal_t_position_xscale, * cal_t_position_yscale;

    cal_q_energy_xscale = new KonoeLinearDataScale(0, 4095, 256);
    cal_t_position_xscale = new KonoeLinearDataScale(0, 31, 32);
    cal_t_position_yscale = new KonoeLinearDataScale(0, 31, 32);
    
    cal_q_energy_histogram = new Konoe1DHistogram(
        cal_q_energy_xscale
    );
    cal_t_position_histogram = new Konoe2DHistogram(
        cal_t_position_xscale, 
        cal_t_position_yscale
    );
    

// create Fillers and FillerList //
    KonoeHistogramFiller *cal_q_energy_histogram_filler;
    KonoeHistogramFiller *cal_t_position_histogram_filler;

    cal_q_energy_histogram_filler = new KonoeHistogramFiller(
	cal_q_energy_extractor,
	cal_q_energy_histogram
    );
    cal_t_position_histogram_filler = new KonoeHistogramFiller(
	cal_t_position_extractor,
	cal_t_position_histogram
    );

    KInt nfillers = 2;
    filler_list = new KonoeHistoFillerList(nfillers);
    filler_list->add(cal_q_energy_histogram_filler);
    filler_list->add(cal_t_position_histogram_filler);
}


DemoAnalysis::~DemoAnalysis( )
{
    delete filler_list;
    delete analyzer;
}


KInt DemoAnalysis::go()
{
    cal_q_energy_histogram->clear();
    cal_t_position_histogram->clear();

    KonoeDataRecord *record;
    while ((record = (KonoeDataRecord *) input_stream->get()) != 0) {
      // record->dump(cout);

	analyzer->reset();
	record->reset();
	analyzer->analyze(record);

	filler_list->fill();

    }
    //   cal_q_energy_histogram->dump(cout);
    cal_t_position_histogram->dump(cout);
    return 0;
}



#ifdef STANDALONE

int main(int argc, char** argv) 
{
#if 0
    KonoeOutputObjSocket os(15000);
    cerr << "waiting for connection...";
    while (os.waitConnection() == -1) {
        ;
    }
#endif

    char hostname[64];
    int port;

    if (argc >= 2) {
      strcpy( hostname, argv[1] );
      port = atoi(argv[2]);
    }
    else {
      strcpy( hostname, "kuhes3" );
      port = 10000;
    }

    
    KonoeInputObjSocket is(hostname, port);
    
    DemoAnalysis demo_analysis(&is);
    int result = demo_analysis.go();

    return result;
}

#endif
