//===================================================================
/*

	DemoAnalysisOrderly.cc

	Version 0.0.1	 4-Aug-1998	S. Nishida and S. Enomoto

*/
//===================================================================

#include	"DemoAnalysisOrderly.hh"
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeOutputObjectStream.hh>
#include	<system/KonoeInputObjectStream.hh>
#include	<system/KonoeInputObjSocket.hh>
#include	<analysis/KonoeHistogramBase.hh>

const KChar	* DemoAnalysisOrderly::classname = "DemoAnalysis";
#define VIEWER_DOESNT_EXIST 1


DemoAnalysisOrderly::DemoAnalysisOrderly( KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add( 
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add( new KonoeArgumentDescriptor(
				 "is_source", STRING, NECESSARY ) );
#if ! (VIEWER_DOESNT_EXIST)
	cmd_descripter-> add(new KonoeArgumentDescriptor(
#endif				 "os_viewer", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "go", N_ARG_GO );
	addCommand( new GoUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor(
		"installHistogram", N_ARG_INSTALLHISTOGRAM );
	cmd_descripter-> add( 
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add( new KonoeArgumentDescriptor(
				 "histid", INTEGER, NECESSARY ) );
	addCommand(  new InstallHistogramUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );
}

KInt	DemoAnalysisOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

DemoAnalysisOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	DemoAnalysisOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

DemoAnalysisOrderly::GoUnit::GoUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	DemoAnalysisOrderly::GoUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	DemoAnalysis * target = ( DemoAnalysis * ) owner -> getTarget( );
	target -> go( );

	return 0;
}


DemoAnalysisOrderly::InstallHistogramUnit::InstallHistogramUnit(
	 KonoeCommandDescriptor * cd , KonoeOrderly * ow ) 
	: KonoeCommandProcessor( cd, ow ) {
}

KInt	DemoAnalysisOrderly::InstallHistogramUnit
		::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeCommander *commander
	  = owner -> getOfficer( ) -> getCommander( );

	if( ! checkArgument( cmd ) ) return 0;

        const KChar *objectname = cmd -> getArgument(
		ARG_INSTALLHISTOGRAM_HISTBASE ) -> getValue( );
	KonoeOrderly * orderly = commander -> findObject(objectname);
	if( orderly == 0 ){
	  os << "object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeHistogramBase * histbase
		= (KonoeHistogramBase * ) orderly -> getTarget( );

	KInt histid = cmd -> getArgument( ARG_INSTALLHISTOGRAM_HISTID )
	  			-> toInteger( ) ;

	DemoAnalysis * target = ( DemoAnalysis * ) owner -> getTarget( );
	target -> installHistogram( histbase , histid );

	return 0;
}

KInt	DemoAnalysisOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeCommander *commander = getOfficer( ) -> getCommander( );

        const KChar *objectname 
		= cmd -> getArgument( ARG_CREATE_ISSOURCE ) -> getValue( );
	KonoeOrderly * orderly = commander -> findObject(objectname);
	if( orderly == 0 ){
	  os << "Input Stream object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeInputObjSocket * is_source
		= (KonoeInputObjSocket * ) orderly -> getTarget( );

#if VIEWER_DOESNT_EXIST
	target = new DemoAnalysis( is_source );
#else
        objectname 
		= cmd -> getArgument( ARG_CREATE_OSVIEWER ) -> getValue( );
	orderly = commander -> findObject(objectname);
	if( orderly == 0 ){
	  os << "Outstream object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeOutputObjectStream * os_viewer
		= (KonoeOutputObjectStream * ) orderly -> getTarget( );

	target = new DemoAnalysis( is_source , os_viewer );
#endif
        
	return 0;

}

KInt	DemoAnalysisOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* DemoAnalysisOrderly::clone( KChar * name ) {

	return new DemoAnalysisOrderly( name );

}








