//==================================================================
/*

	DemoSimulator.cc

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version 0.3.0    5-Jun-1998     D. Takagi
	Version 0.4.0	 4-Aug-1999	T. Yamashita
*/
//==================================================================

#include	<daq/KonoeSimCrate.hh>
#include	<daq/KonoeSimTOFTDC.hh>
#include        <daq/KonoeSimADCDigitizer.hh>
#include	<daq/KonoeSimTDCModule.hh>
#include	<daq/KonoeSimADCModule.hh>

#include	<daq/KonoeReadoutList.hh>
#include	<daq/KonoeSoftwareCrate.hh>
#include	<daq/KonoeSoftCounter.hh>
#include	<daq/KonoeCrateList.hh>
#include	<daq/KonoeTimerXaction.hh>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<analysis/KonoeBoxGeometry.hh>
#include	<system/KonoeOutputObjFile.hh>
#include        <system/KonoeServerSocket.hh>
#include        <system/KonoeOutputObjSocket.hh>

#include	<iostream.h>
#include 	<stdio.h>
#include        "DemoEventGenerator.hh"

#include	"DemoSimulator.hh"

DemoSimulator::DemoSimulator( KInt modulecount, 
      KonoeDetectorList * dl, 
      KonoeEventGenerator * g,
      KInt digitizercount ) :
      KonoeSimCrate( modulecount, dl, g, digitizercount ) {
}

KInt    DemoSimulator::processHit( KonoeSimDetectorHit * hit ) {

        KInt    naddress;
        KonoeDetectorElement    * detectorElement;
        KonoeDataAddress        * dataAdress;
        KonoeSimDigitizer       * simDigitizer;

        detectorElement = hit -> getDetectorElement();

        detectorElement -> reset();
        while( dataAdress = detectorElement -> next() ) {

            simDigitizer = digitizerlist -> 
		find( dataAdress -> getSectionID() );
            if( simDigitizer ) {
                simDigitizer ->
			digitize( hit, dataAdress, getModule( dataAdress ) );
            }
            else {
                cout << "no digitizer found" << endl;
            }

#if 0
	    cerr << "HIT at " << dataAdress->getSectionID() << ";" << 
	      dataAdress->getSegmentID() << ";"
		 << dataAdress->getSubaddress() << " ,E" 
		 << hit->getEnergy() << " , time" << hit->getEntryTime()
		 << endl;
#endif
        }

        return 0;
}

// Detector Setting
//
// ->z
// 0m   0.5    1.0    1.5    2.0    2.5    3.0    3.5m
//                                                  _
//              ||     ||            ||     ||     | |
//             H||H   H||H          H||H   H||H    |C|
//             X||Y   X||Y          X||Y   X||Y    |A|
//             1||1   2||2          3||3   4||4    |L|
//              ||     ||            ||     ||     |_|
//
// DetectorID  0  1   2  3          4  5   6  7     8
//

// DataRecord Format
//
//     Sec 0: General
//         Seg 0: Info
//             Ele 0: Event#
//
//     Sec 1-8: SciFi
//         Seg 0-3: TDC0 - TDC3
//             Add 0-31
//
//     Sec 9: Cal TDC0 - TDC3
//         Seg 0-3 TDC
//             Add 0-31
//
//     Sec 10: Cal ADC
//         Seg 0-3 ADC0 - ADC3
//             Add 0-31
