//==================================================================
/*

	DemoSource.cc

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version 0.3.0    5-Jun-1998     D. Takagi
	Version 0.4.0	 4-Aug-1999	T. Yamashita
*/
//==================================================================

#include	<daq/KonoeSimCrate.hh>
#include	<daq/KonoeSimTOFTDC.hh>
#include        <daq/KonoeSimADCDigitizer.hh>
#include	<daq/KonoeSimTDCModule.hh>
#include	<daq/KonoeSimADCModule.hh>

#include	<daq/KonoeReadoutList.hh>
#include	<daq/KonoeSoftwareCrate.hh>
#include	<daq/KonoeSoftCounter.hh>
#include	<daq/KonoeCrateList.hh>
#include	<daq/KonoeTimerXaction.hh>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<analysis/KonoeBoxGeometry.hh>
#include	<system/KonoeOutputObjFile.hh>
#include        <system/KonoeServerSocket.hh>
#include        <system/KonoeOutputObjSocket.hh>

#include	<iostream.h>
#include 	<stdio.h>
#include	<unistd.h>
#include        "DemoEventGenerator.hh"

#include	"DemoSimulator.hh"
#include	"DemoSource.hh"

// Detector Setting
//
// ->z
// 0m   0.5    1.0    1.5    2.0    2.5    3.0    3.5m
//                                                  _
//              ||     ||            ||     ||     | |
//             H||H   H||H          H||H   H||H    |C|
//             X||Y   X||Y          X||Y   X||Y    |A|
//             1||1   2||2          3||3   4||4    |L|
//              ||     ||            ||     ||     |_|
//
// DetectorID  0  1   2  3          4  5   6  7     8
//

// DataRecord Format
//
//     Sec 0: General
//         Seg 0: Info
//             Ele 0: Event#
//
//     Sec 1-8: SciFi
//         Seg 0-3: TDC0 - TDC3
//             Add 0-31
//
//     Sec 9: Cal TDC0 - TDC3
//         Seg 0-3 TDC
//             Add 0-31
//
//     Sec 10: Cal ADC
//         Seg 0-3 ADC0 - ADC3
//             Add 0-31

DemoSource::DemoSource( KonoeOutputObjectStream* pdataout )
{
//
	KInt elementID = 0;

	dataout = pdataout;
// set up a demo detecter
	detectorList = new KonoeDetectorList( 9 );
	for ( KInt i = 0; i < 8; i++ ) {
	  scifi[i] = new KonoeDetector( 32, i );
	  detectorList -> add( scifi[i] );
	}
	cal = new KonoeDetector( 256, 8 );
	detectorList -> add( cal );

// scintilation fibers
	double z[] = { 1.0, 1.1, 1.5, 1.6, 2.5, 2.6, 3.0, 3.1 };

	KonoeTranslation translation;
	KonoeDetectorElement * detectorElement;

// scifi ( HX & HY )
	for ( KInt i = 0; i < 4; i ++ ) {
	  for ( KInt j = 0; j < 2; j ++ ) {
	    for( KInt k = 0; k < 32; k++ ){
	        KonoeBoxGeometry * box;
	        Konoe3Vector p;
	    
		KInt secID, segID, subAdress;
		secID = i + 1;
		segID = j;
		subAdress = k; // channel

	        if( j == 0 ) {
	            // HX ( 0, 2, 4, 6 )
	            box = new KonoeBoxGeometry( 2.0, 2.0 / 32 , 0.01 );
	            p.set( -1.0, k * 2.0 / 32 - 1.0, z[i * 2 + j] );
	        } else{
	            // HY ( 1, 3, 5, 7 )
		    box = new KonoeBoxGeometry( 2.0 / 32, 2.0, 0.01 );
		    p.set( k * 2.0 / 32 - 1.0, -1.0, z[i * 2 + j] );
		}	    
		translation.setDisplacement( p );
		box -> translate( & translation );
	    
		scifi[i * 2 + j] -> add( detectorElement =
			new KonoeDetectorElement(box, scifi[i * 2 + j],
				elementID++) );
		detectorElement -> add(
			new KonoeDataAddress( secID, segID, subAdress ) );

#if 0
		cout	<< "i*2+j " << i * 2 + j
			<< "  count " << detectorElement->getCount()
			<< "  detctorElement " << endl;
		detectorElement->dump(cout);
#endif
	      }
	   }
	}

// cal
	for ( KInt i = 0; i < 16; i ++ ) {
 	    for ( KInt j =  0; j < 16; j ++ ) {
	        KonoeBoxGeometry * box
		  = new KonoeBoxGeometry( 2.0 / 16, 2.0 / 16, 2.0 / 16 );
		Konoe3Vector p( i * 2.0 / 16 - 1.0, j * 2.0 / 16 - 1.0, 3.5 );
		translation.setDisplacement( p );
		box -> translate( & translation );

		cal -> add( detectorElement =
			new KonoeDetectorElement( box, cal, elementID++ ));

		KInt secID, segID, subAdress;
		segID = i;
		subAdress = j; // channel

		detectorElement -> add(
			new KonoeDataAddress( 5, segID, subAdress ) );
		detectorElement -> add(
			new KonoeDataAddress( 6, segID, subAdress ) );
#if 0
		cout	<< "i*2+j " << i * 2 + j
			<< "  count " << detectorElement->getCount()
			<< "  detctorElement " << endl;
		detectorElement->dump(cout);
#endif
	    }
	}


// set up an event generator

	generator = new DemoEventGenerator( );

// software modules

	softcrate = new KonoeSoftwareCrate( 1 );
	eventnumber = new KonoeSoftCounter( );
	softcrate -> install( eventnumber );

// simulator

	KonoeSimModule *tdc[24];
	KonoeSimModule *adc[16];
	KInt secID, segID, storageSize;

	simulator = new DemoSimulator( 40, detectorList, generator, 7 );
	storageSize = 32;
	for ( KInt i = 0; i < 4; i ++ ) {
 	    for ( KInt j = 0; j < 2; j ++ ) {
		secID = i + 1;
		segID = j;
	        tdc[i * 2 + j] =
			new KonoeSimModule( secID, segID, storageSize );
	        simulator -> add( tdc[i * 2 + j] );
	    }
	}

	storageSize = 16;
	for ( KInt i = 0; i < 16; i ++ ) {
	    secID = 5;
	    segID = i;
	    tdc[8 + i] = new KonoeSimModule( secID, segID, storageSize );
	    simulator -> add( tdc[i + 8] );
	}
	for ( KInt i = 0; i < 16; i ++ ) {
	    secID = 6;
	    segID = i;
	    adc[i] = new KonoeSimModule( secID, segID, storageSize );
	    simulator -> add( adc[i] );
	}

// crate list

	cratelist = new KonoeCrateList( 2 );
	cratelist -> add( softcrate );
	cratelist -> add( simulator );

// digitizer

	for ( KInt i = 1; i < 6; i ++ ) {
	    simulator -> addDigitizer( new KonoeSimTOFTDC( i, 0, TDC_GAIN ) );
	}
	simulator -> addDigitizer( new KonoeSimADCDigitizer( 6, 0, ADC_GAIN ) );

// readout procedure
	
	readoutlist = new KonoeReadoutList( 7, 0 );

	KonoeReadoutSection * section;
	KonoeReadoutSegment * segment;

	// general section
	segment = new KonoeReadoutSegment( 1, 0 );
        segment -> add( new KonoeReadoutItem( eventnumber, (KInt)0 ) );
	section = new KonoeReadoutSection( 1, 0 );
        section -> add( segment );
	readoutlist -> add( section );

 	// scifi tdc
	for ( KInt i = 0; i < 4; i ++ ) {
	    section = new KonoeReadoutSection( 2, i + 1);
	    for ( KInt j = 0; j < 2; j ++ ) {
	        segment = new KonoeReadoutSegment( tdc[i * 2 + j], j );
		section -> add( segment );
	    }
	    readoutlist -> add( section );
	}

	// cal tdc
	section = new KonoeReadoutSection( 16, 5 );
	for ( KInt j = 0; j < 16; j ++ ) {
	    segment = new KonoeReadoutSegment( tdc[8 + j], j );
	    section -> add( segment );
	}
	readoutlist -> add( section );

	// cal adc
	section = new KonoeReadoutSection( 16, 6 );
	for ( KInt j = 0; j < 16; j ++ ) {
	    segment = new KonoeReadoutSegment( adc[j], j );
	    section -> add( segment );
	}
	readoutlist -> add( section );
	
// output
}

DemoSource::~DemoSource( ) {

	delete detectorList;
	delete readoutlist;
//	delete dataout;
}

KInt	DemoSource::go( ) {

// connection
        KonoeGeometryCM * geometrycm = new KonoeGeometryCM();

        detectorList -> reset();

        KonoeDataRecordCM * datarecordcm = new KonoeDataRecordCM();
	dataout -> installCM( datarecordcm );

	KonoeTimerXaction::initialize( );
	transaction = new KonoeTimerXaction( readoutlist );
	transaction -> setInterval( 2000 );
	transaction -> enable( );

	cratelist -> clear();

        Konoe3Vector * vector = new Konoe3Vector(0, 0, 0);  
	
        for (KInt i = 0; i < 10; i ++ ) {
	    cout << "sending Event# " << i + 1 << endl;
	    cratelist -> clear();
	    cratelist -> update( );

	    KonoeDataRecord * record = transaction -> read( );

	    cout<< "sending record"<<endl;
	    record -> reset();
	    dataout -> put( record, 'A', 0 );
	    //	    record -> dump(cout);
	    delete record;
	}

	vector->x = 999;
	dataout -> put( vector ); // event end 

	transaction -> setInterval( 0 );
	transaction -> disable();

	sleep(2);

	return 0;
}


#ifdef STANDALONE

int	main( int argc, char ** argv ) {

// port1
	KonoeOutputObjectStream * dataout;
	KonoeServerSocket	* datasocket;

// serversocket
	KInt port1 = 10000;
	
	if (argc > 1) sscanf(argv[1], "%d", &port1);

	datasocket = new KonoeServerSocket( port1 );
	dataout = new KonoeOutputObjSocket( datasocket );

	cerr << "waiting for connection...";
	while( datasocket -> waitConnection( ) == -1 ) {
		;
	}
	cerr << " done." << endl;
/*	
        cerr << "waiting for connection No2...";
	while(  detectorhitsocket -> waitConnection( ) == -1 );
	cerr << " done." << endl;
*/
	DemoSource demo( dataout );

	return demo.go();

}

#endif

