//===================================================================
/*

	DemoSourceOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Nishida and S. Enomoto

*/
//===================================================================

#include	"DemoSourceOrderly.hh"
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeOutputObjectStream.hh>
#include	<system/KonoeOutputObjSocket.hh>

const KChar	* DemoSourceOrderly::classname = "DemoSource";


DemoSourceOrderly::DemoSourceOrderly( KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add( 
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add( new KonoeArgumentDescriptor(
				 "os_analysis", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "go", N_ARG_GO );
	addCommand( new GoUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );
}

KInt	DemoSourceOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

DemoSourceOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	DemoSourceOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

DemoSourceOrderly::GoUnit::GoUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	DemoSourceOrderly::GoUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	DemoSource * target = ( DemoSource * ) owner -> getTarget( );
	target -> go( );

	return 0;
}

KInt	DemoSourceOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeCommander *commander = getOfficer( ) -> getCommander( );

        const KChar *objectname 
		= cmd -> getArgument( ARG_CREATE_OSANALYSIS ) -> getValue( );
	KonoeOrderly * orderly = commander -> findObject(objectname);
	if( orderly == 0 ){
	  os << "Outstream object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeOutputObjSocket * os_analysis
		= (KonoeOutputObjSocket * ) orderly -> getTarget( );
        
	target = new DemoSource( os_analysis );

	return 0;

}

KInt	DemoSourceOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* DemoSourceOrderly::clone( KChar * name ) {

	return new DemoSourceOrderly( name );

}


