/*
	CRDAQModuleConstruction.cc

	Version 0.0.0	19-Aug-2002	mor
*/

#include "CRDAQModuleConstruction.hh"
#include "CRDAQStatus.hh"
#include "CRNameSpaceExtractor.hh"
#include "CRNameSpaceSetter.hh"

CRDAQModuleConstruction::CRDAQModuleConstruction()
{
  eventList = new KonoeReadoutList( numSection, eventRecord );

  ns = new CRDAQNameSpace();

  this -> generate();
  this -> install();
  this -> construct();

  show();

}

CRDAQModuleConstruction::~CRDAQModuleConstruction()
{
/*
  delete eventNumber;
  delete theTimer;
  delete theClock;
*/
  delete thePlace;
  delete tm;

  delete theScaler;
  if( adcFlag ) {
    delete theADC;
  }
  if( tdcFlag ) {
    delete theTDC;
  }
  if( scalFlag ) {
    delete theScaler;
  }

  delete theIR;
  delete theOR;
  delete theCrate;

  delete eventList;
}

void CRDAQModuleConstruction::generate()
{
  theCrate = new KonoeCAMACCrate( nodeName, cc_Id );
  if( adcFlag ) {
    theADC = new KonoeCAMACADC* [adcNumSeg];
    for( int i=0; i<adcNumSeg; i++ ) {
      theADC[i] = new KonoeCAMACADC( this -> theCrate, adc[i], adcNumEle[i] );
    }
  }
  if( tdcFlag ) {
    theTDC = new KonoeCAMACTDC* [tdcNumSeg];
    for( int i=0; i<tdcNumSeg; i++ ) {
      theTDC[i] = new KonoeCAMACTDC( this -> theCrate, tdc[i], tdcNumEle[i] );
    }
  }
  if( scalFlag ) {
    theScaler = new KonoeCAMACScaler* [scalerNumSeg];
    for( int i=0; i<scalerNumSeg; i++ ) {
      theScaler[i] = new KonoeCAMACScaler( this -> theCrate, scaler[i], scalerNumEle[i] );
    }
  }
  theIR = new KonoeCAMACInterruptRegister( this -> theCrate, inr );
  theIR -> writeMask( (KUShort)254 );
  theOR = new KonoeCAMACOutputRegister( this -> theCrate, outr );
//  theTimer = new KonoeTimerModule();
//  theClock = new KonoeClockModule();
//  eventNumber = new KonoeSoftCounter();
  thePlace = new KonoeOptionalNumberModule();
  thePlace -> setNumber( place );
  tm = new KonoeOptionalNumberModule();

  return;
}

void CRDAQModuleConstruction::install() const
{
  if( adcFlag ) {
    for( int i=0; i<adcNumSeg; i++ ) {
      this -> theCrate -> install( this -> theADC[i] );
    }
  }
  if( tdcFlag ) {
    for( int i=0; i<tdcNumSeg; i++ ) {
      this -> theCrate -> install( this -> theTDC[i] );
    }
  }

  return;
}

void CRDAQModuleConstruction::add( KonoeReadoutSection* aSection )
{
  this -> eventList -> add( aSection );
  return;
}

void CRDAQModuleConstruction::construct()
{
  KonoeReadoutSection* section;
  KonoeReadoutSegment* segment;
  KonoeReadoutItem* element;

  // GENERAL
  section = new KonoeReadoutSection( generalNumSeg, generalSecType );
  segment = new KonoeReadoutSegment( timeInfoNumEle, timeInfoSegType );
  section -> add( segment );
  element = new KonoeReadoutItem( this -> tm, (KInt)0 );
  segment -> add( element );
  segment = new KonoeReadoutSegment( runInfoNumEle, runInfoSegType );
  section -> add( segment );
//  element = new KonoeReadoutItem( this -> eventNumber, (KInt)0 );
  element = new KonoeReadoutItem( this -> thePlace, (KInt)0 );
  segment -> add( element );
/*
  element = new KonoeReadoutItem( this -> theTimer, (KInt)1 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> theClock, (KInt)2 );
  segment -> add( element );
  segment = new KonoeReadoutSegment( irNumEle, irSegType );
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theIR, (KInt)1 );
  segment -> add( element );
*/
  this -> add( section );

  // TDC
  if( tdcFlag ) {
    section = new KonoeReadoutSection( tdcNumSeg, tdcSecType );
    for( int i=0; i<tdcNumSeg; i++ ) { 
      segment = new KonoeReadoutSegment( this -> theTDC[i], tdcSegType[i] );
      section -> add ( segment );
    }
    this -> add( section );
  }

  // ADC
  if( adcFlag ) {
    section = new KonoeReadoutSection( adcNumSeg, adcSecType );
    for( int i=0; i<adcNumSeg; i++ ) {
      segment = new KonoeReadoutSegment( this -> theADC[i], adcSegType[i] );
      section -> add ( segment );
    }
    this -> add( section );
  }

  // SCALER
  if( scalFlag ) {
    section = new KonoeReadoutSection( scalerNumSeg, scalerSecType );
    for( int i=0; i<scalerNumSeg; i++ ) {
      segment = new KonoeReadoutSegment( this -> theScaler[i], scalerSegType[i] );
      section -> add ( segment );
    }
    this -> add( section );
  }

  return;
}

KonoeDataRecord* CRDAQModuleConstruction::read()
{
  KonoeDataRecord* data;

//  this -> theCrate -> enableLAM();
  this -> theADC[0] -> enableLAM();

  if(this -> theCrate -> testLAM(this->theADC[0]) ){
    data = this -> eventList -> read();
    this -> update();
    return data;
  }
  return 0;

/*
  this -> theCrate -> waitLAM( timeout, this -> theADC );
  if ( this -> theCrate -> getStatus() == 0 ) {
    if ( this -> theCrate -> getLAM() == adc ) {
      data = this -> eventList -> read();
      this -> update();
      return ( data );
    }
  } else {
    return (0);
  }
*/

}

KonoeDataRecord* CRDAQModuleConstruction::read_ir()
{
  KonoeDataRecord* data;

//  this -> theCrate -> enableLAM();
  this -> theIR -> enableLAM();

  if( (this->theIR->read(1)) == 1 ) {
    data = this -> eventList -> read();
    this -> update();
    return data;
  } else {
    return 0;
  }
}

KonoeDataRecord* CRDAQModuleConstruction::read_test()
{
  KonoeDataRecord* data;

//  this -> theCrate -> enableLAM();

  data = this -> eventList -> read();
  this -> update();
  return data;
}

void CRDAQModuleConstruction::update() const
{
//  this -> theCrate -> update();

  if( tdcFlag ) {
    for( int i=0; i<tdcNumSeg; i++ ) {
      this -> theTDC[i] -> clear();
    }
  }
  if( adcFlag ) {
    for( int i=0; i<adcNumSeg; i++ ) {
      this -> theADC[i] -> clear();
    }
  }
  if( scalFlag ) {
    for( int i=0; i<scalerNumSeg; i++ ) {
      this -> theScaler[i] -> clear();
    }
  }
  this -> theIR -> update();
//  this -> eventNumber -> update();
//  this -> theTimer -> update();
//  this -> theClock -> update();
  this -> tm -> setNumber( ns -> getTime() );
  return;
}

void CRDAQModuleConstruction::initialize() const
{
//  this -> theCrate -> initialize();
  if( tdcFlag ) {
    for( int i=0; i<tdcNumSeg; i++ ) {
      this -> theTDC[i] -> initialize();
    }
  }
  if( adcFlag ) {
    for( int i=0; i<adcNumSeg; i++ ) {
      this -> theADC[i] -> initialize();
    }
  }
  if( scalFlag ) {
    for( int i=0; i<scalerNumSeg; i++ ) {
      this -> theScaler[i] -> initialize();
    }
  }
  this -> theIR -> initialize();
//  this -> eventNumber -> initialize();
//  this -> theTimer -> initialize();
//  this -> theClock -> initialize();
  return;
}
