/*
	CRDAQNameSpace.cc

	Version 0.0.0	19-Aug-2002	mor
*/

#include "CRDAQNameSpace.hh"

CRDAQNameSpace::CRDAQNameSpace()
{
  this -> extract();
  this -> update();
}

CRDAQNameSpace::~CRDAQNameSpace()
{
  this -> update();
}

void CRDAQNameSpace::extract()
{
  daqCommandPort = getKIntValue("DAQ_CMD_PORT");
  this -> daqPID = KonoeHostSystem::getPid();
  dataHubInputPort = getKIntValue("DATAHUB_INPUT_PORT");
  preDHInputPort = getKIntValue("PREDATAHUB_INPUT_PORT");

  strcpy( this -> masterHost, getKCharValue("MASTER_HOST") );
  strcpy( this -> dataHubHost, getKCharValue("DATAHUB_HOST") );
  strcpy( this -> preDHHost, getKCharValue("PREDATAHUB_HOST") );
  strcpy( this -> daqStatus, getKCharValue("DAQ_STATUS") );

// FLAG
  this -> adcFlag = getKIntValue("ADC_FLAG");
  this -> tdcFlag = getKIntValue("TDC_FLAG");
  this -> scalFlag = getKIntValue("SCALER_FLAG");

// daq_constant
  this -> numSection = getKIntValue("N_SECTION");
  this -> eventRecord = getKIntValue("EVENT_RECORD");
  this -> generalSecType = getKIntValue("GENERAL_SECTION_TYPE");
  this -> generalNumSeg = getKIntValue("GENERAL_N_SEGMENT");
  this -> runInfoSegType = getKIntValue("RUNINFO_SEGMENT_TYPE");
  this -> runInfoNumEle = getKIntValue("RUNINFO_N_ELEMENT");
  this -> timeInfoSegType = getKIntValue("TIMEINFO_SEGMENT_TYPE");
  this -> timeInfoNumEle = getKIntValue("TIMEINFO_N_ELEMENT");

  this -> place = getKIntValue("DATA_DISTINCTION");

  if( tdcFlag ) {
    this -> tdcSecType = getKIntValue("TDC_SECTION_TYPE");
    this -> tdcNumSeg = getKIntValue("TDC_N_SEGMENT");
    this -> tdcSegType = new KInt [tdcNumSeg];
    this -> tdcNumEle = new KInt [tdcNumSeg];
    for( int i=0; i<tdcNumSeg; i++ ) {
      this -> tdcSegType[i] = getKIntValue("TDC[i]_SEGMENT_TYPE", i);
      this -> tdcNumEle[i] = getKIntValue("TDC[i]_N_ELEMENT", i);
    }
  }


  if( adcFlag ) {
    this -> adcSecType = getKIntValue("ADC_SECTION_TYPE");
    this -> adcNumSeg = getKIntValue("ADC_N_SEGMENT");
    this -> adcSegType = new KInt [adcNumSeg];
    this -> adcNumEle = new KInt [adcNumSeg];
    for( int i=0; i<adcNumSeg; i++ ) {
      this -> adcSegType[i] = getKIntValue("ADC[i]_SEGMENT_TYPE", i);
      this -> adcNumEle[i] = getKIntValue("ADC[i]_N_ELEMENT", i);
    }
  }

  if( scalFlag ) {
    this -> scalerSecType = getKIntValue("SCALER_SECTION_TYPE");
    this -> scalerNumSeg = getKIntValue("SCALER_N_SEGMENT");
    this -> scalerSegType = new KInt [scalerNumSeg];
    this -> scalerNumEle = new KInt [scalerNumSeg];
    for( int i=0; i<scalerNumSeg; i++ ) {
      this -> scalerSegType[i] = getKIntValue("SCALER[i]_SEGMENT_TYPE", i);
      this -> scalerNumEle[i] = getKIntValue("SCALER[i]_N_ELEMENT", i);
    }
  }

  this -> irNumEle = getKIntValue("IR_N_ELEMENT");
  this -> irSegType = getKIntValue("IR_SEGMENT_TYPE");

  this -> daqBufferSize = getKIntValue("DAQ_BUFFER_SIZE");
  this -> timeout = getKIntValue("TIMEOUT");

// daq_module
  strcpy( this -> nodeName, getKCharValue("NODE_NAME") );
  this -> cc_Id = getKIntValue("CC_ID");

  if( adcFlag ) {
    this -> adc = new KInt [adcNumSeg];
    for( int i=0; i<adcNumSeg; i++ ) {
      this -> adc[i] = getKIntValue("ADC[i]_STATION_NUMBER", i);
    }
  }

  if( tdcFlag ) {
    this -> tdc = new KInt [tdcNumSeg];
    for( int i=0; i<tdcNumSeg; i++ ) {
      this -> tdc[i] = getKIntValue("TDC[i]_STATION_NUMBER", i);
    }
  }

  if( scalFlag ) {
    this -> scaler = new KInt [scalerNumSeg];
    for( int i=0; i<scalerNumSeg; i++ ) {
      this -> scaler[i] = getKIntValue("SCALER[i]_STATION_NUMBER", i);
    }
  }

  this -> inr = getKIntValue("IR");
  this -> outr = getKIntValue("OR");


  KChar tmp[CHAR_LENGTH];
  KonoeHostSystem::getHostName( tmp, CHAR_LENGTH );
  strcpy( this -> daqHost, tmp );

  return;
}

KInt CRDAQNameSpace::isValidItem( KChar* item )
{
  KInt a = strcmp( item, "DAQ_STATUS" );
  KInt b = strcmp( item, "DAQ_PID" );
  KInt c = strcmp( item, "DAQ_HOST" );

  return ( a==0 | b==0 | c==0 );
}

void CRDAQNameSpace::update()
{
  set( "DAQ_STATUS", this -> getDaqStatus() );
  set( "DAQ_PID", this -> getDaqPid() );
  set( "DAQ_HOST", this -> getDaqHost() );

  return;
}

void CRDAQNameSpace::timerUpdate()
{
	this -> timer = getKIntValue( "NOW_TIME" );
	return;
}

void CRDAQNameSpace::update( KChar* item, KInt value )
{
  if( (this -> isValidItem( item ) ) == KFalse )
    return;
  else if( strcmp( item, "DAQ_PID" ) == 0 )
    this -> setDaqPid( value );
  this -> update();
  return;
}

void CRDAQNameSpace::update( KChar* item, KChar* value )
{
  if( (this -> isValidItem( item ) ) == KFalse )
    return;
  else if(strcmp( item, "DAQ_STATUS" ) == 0 )
    this -> setDaqStatus( value );
  else if(strcmp( item, "DAQ_HOST" ) == 0 )
    this -> setDaqHost( value );
  this -> update();
  return;
}

void CRDAQNameSpace::show()
{
  this -> showMasterHost();
  this -> showDaqHost();
  this -> showDaqPort();
  this -> showDaqPid();
  this -> showDaqStatus();
  this -> showDataHubHost();
  this -> showDataHubPort();
  this -> showPreDHHost();
  this -> showPreDHPort();
  return;
}

void CRDAQNameSpace::showModuleConstruction()
{
  cout << "adc_Flag : " << adcFlag << endl;
  cout << "tdc_Flag : " << tdcFlag << endl;
  cout << "scaler_Flag : " << scalFlag << endl;
  if( adcFlag ) {
    cout << "adc Num : " << adcNumSeg << endl;
  }
  if( tdcFlag ) {
    cout << "tdc Num : " << tdcNumSeg << endl;
  }
  if( scalFlag ) {
    cout << "scaler Num : " << scalerNumSeg << endl;
  }

  for( int i=0; i<adcNumSeg; i++ ) {
    cout << "adcNumEle[" << i << "] ; " << adcNumEle[i] << endl;
    cout << "adc[" << i << "] : " << adc[i] << endl;
  }

  for( int i=0; i<tdcNumSeg; i++ ) {
    cout << "tdcNumEle[" << i << "] ; " << tdcNumEle[i] << endl;
    cout << "tdc[" << i << "] : " << tdc[i] << endl;
  }

  for( int i=0; i<scalerNumSeg; i++ ) {
    cout << "scalerNumEle[" << i << "] ; " << scalerNumEle[i] << endl;
    cout << "scaler[" << i << "] : " << scaler[i] << endl;
  }
}

void CRDAQNameSpace::showTime()
{
	cout << "Now time is " << getTime() << "." << endl;
}
