/*
	CRDAQProcess.cc

	Version 0.0.0	20-Aug-2002	mor
*/

#include <unistd.h>

#include <system/KonoeOutputObjFileBuffered.hh>
#include <system/KonoeOutputObjSocketBuffered.hh>

#include "CRDAQProcess_file.hh"

CRDAQProcess_file::CRDAQProcess_file( KInt commandPort ) : CRProcess( commandPort )
{
  daqNameSpace = new CRDAQNameSpace();
  daqStatus = new CRDAQStatus();
  daqAction = new CRDAQAction();
  daqNameSpace -> update( "DAQ_STATUS", "standby" );
  daqModules = new CRDAQModuleConstruction();
  this -> daqModules -> setI();
}

CRDAQProcess_file::~CRDAQProcess_file()
{
  delete daqNameSpace;
  delete daqStatus;
  delete daqAction;
  delete daqModules;
}

void CRDAQProcess_file::catchSignal( KInt signal )
{
  KChar cmd[256];
  cmd_process -> send( "accept", 256 );
  cmd_process -> receive( cmd, 256 );

  if( daqStatus -> isValidCommand(cmd) ) {
    if( strcmp( cmd, "begin" ) == 0 ) {
      daqStatus -> setCommand( CRDAQStatus::begin );
      daqAction -> begin();
    } else if( strcmp( cmd, "pause" ) == 0 ) {
      daqStatus -> setCommand( CRDAQStatus::pause );
      daqAction -> pause();
    } else if( strcmp( cmd, "resume" ) == 0 ) {
      daqStatus -> setCommand( CRDAQStatus::resume );
      daqAction -> resume();
    } else if( strcmp( cmd, "end" ) == 0 ) {
      daqStatus -> setCommand( CRDAQStatus::end );
      daqAction -> end();
    } else if( strcmp( cmd, "shutdown" ) == 0 ) {
      daqStatus -> setCommand( CRDAQStatus::shutdown );
      daqAction -> shutdown();
    }
    cmd_process -> send( "ture", 256 );
  } else
    cmd_process -> send( "false", 256 ); 

  return;
}

void CRDAQProcess_file::go()
{
  KBool isOpen(KFalse);
  KonoeDataRecord* data;
  CRDAQDataBuffer* daqDataBuffer;
  KInt memoryAddress;
  KonoeOutputObjFileBuffered* output;
//  KonoeOutputObjSocketBuffered* output;
  KChar* hubHost = this -> daqNameSpace -> getDataHubHost();
  KInt hubPort = this -> daqNameSpace -> getDataHubPort();
  KInt count_data = 0;

  while(1) {

    if( daqStatus -> getStatus() == CRDAQStatus::standby ) {
      if( daqStatus -> getCommand() == CRDAQStatus::unknown ) {
//	handler -> wait();
	daqStatus -> setCommand( CRDAQStatus::begin );
      } else if( daqStatus -> getCommand() == CRDAQStatus::begin ) {
	if( isOpen == KFalse ) {
	  output = new KonoeOutputObjFileBuffered( "neko.obj" );
//	  output = new KonoeOutputObjSocketBuffered( hubHost, hubPort );
	  daqDataBuffer = new CRDAQDataBuffer( (KInt)daqBufferSize );
	  isOpen = KTrue;
	  this -> daqModules -> removeI();
	}
	daqStatus -> setStatus( CRDAQStatus::busy );
      } else if( daqStatus -> getCommand() == CRDAQStatus::shutdown ) {
	this -> daqNameSpace -> extract();
	if( strcmp( daqNameSpace -> getDaqStatus(), "none" ) != 0 ) {
	  daqNameSpace -> update( "DAQ_STATUS", "none" );
	  daqNameSpace -> update( "DAQ_PID", -1 );
	  sleep(1);
	}
	break;
      }
    }


    else if( daqStatus -> getStatus() == CRDAQStatus::busy ) {
      if( daqStatus -> getCommand() == CRDAQStatus::begin ) {
	data = this -> daqModules -> read();
	memoryAddress = (KInt)data;
	if( data != 0 && memoryAddress != 0x10 ) {
	  daqDataBuffer -> add( data );
//	  data -> dump( cout );
	  count_data++;
	}
	if( count_data == 5 ) {
	  cout << "GET 5 DATA" << endl;
	  this -> daqModules -> setI();
	  daqDataBuffer -> reset();
	  while( data = daqDataBuffer -> next() ) {
	    output -> put( data );
//	    output -> flush();
	  }
	  count_data = 0;
	  daqDataBuffer -> clear();
	  this -> daqModules -> removeI();
	}
      } else if( daqStatus -> getCommand() == CRDAQStatus::pause ) {
	this -> daqModules -> setI();
	daqStatus -> setStatus( CRDAQStatus::idle );
      }
    }

    else if( daqStatus -> getStatus() == CRDAQStatus::idle ) {
      if( daqStatus -> getCommand() == CRDAQStatus::pause ) {
	handler -> wait();
      } else if( daqStatus -> getCommand() == CRDAQStatus::resume ) {
	this -> daqModules -> removeI();
	daqStatus -> setStatus( CRDAQStatus::busy );
	daqStatus -> setCommand( CRDAQStatus::begin );
      } else if( daqStatus -> getCommand() == CRDAQStatus::end ) {
	daqDataBuffer -> reset();
	while( data = daqDataBuffer -> next() ) {
	  output -> put( data );
//	  output -> flush();
	}
	delete daqDataBuffer;
	delete output;
	daqStatus -> setStatus( CRDAQStatus::standby );
	daqStatus -> setCommand( CRDAQStatus::unknown );
	isOpen = KFalse;
      }
    }
  }

  return;
}
