/*
	CRDAQStatus.cc

	Version 0.0.0 19-Aug-2002	mor
*/

#include "CRDAQStatus.hh"
#include "CRDAQNameSpace.hh"

CRDAQStatus::DAQStatus CRDAQStatus::DAQ_STATUS = CRDAQStatus::none;
CRDAQStatus::DAQCommand CRDAQStatus::DAQ_COMMAND = CRDAQStatus::unknown;

KInt CRDAQStatus::initialize( const DAQStatus status )
{
  if( DAQ_STATUS == none && status != none ) {
    DAQ_STATUS = status;
    return 1;
  } else
    return 0;
}

KInt CRDAQStatus::finalize()
{
  if( DAQ_STATUS != none ) {
    DAQ_STATUS = none;
    return 0;
  } else
    return 0;
}

void CRDAQStatus::showStatus()
{
  if( DAQ_STATUS == none )
    cout << "[DAQ] NONE" << endl;
  else if( DAQ_STATUS == standby )
    cout << "[DAQ] STANDBY" << endl;
  else if( DAQ_STATUS == busy )
    cout << "[DAQ] BUSY" << endl;
  else
    cout << "[DAQ] UNKNOWN STATUS" << endl;
  return;
}

void CRDAQStatus::showCommand()
{
  if( DAQ_COMMAND == unknown )
    cout << "[DAQ] UNKNOWN" << endl;
  else if( DAQ_COMMAND == begin )
    cout << "[DAQ] BEGIN" << endl;
  else if( DAQ_COMMAND == pause )
    cout << "[DAQ] PAUSE" << endl;
  else if( DAQ_COMMAND == resume )
    cout << "[DAQ] RESUME" << endl;
  else if( DAQ_COMMAND == end )
    cout << "[DAQ] END" << endl;
  else if( DAQ_COMMAND == shutdown )
    cout << "[DAQ] SHUTDOWN" << endl;
  else
    cout << "[DAQ] UNKNOWN COMMAND" << endl;
  return;
}

KBool CRDAQStatus::isInstanciate()
{
  return (DAQ_STATUS != none);
}

KBool CRDAQStatus::isValidCommand( KChar* daq_cmd )
{
  CRDAQNameSpace* ns = new CRDAQNameSpace();
  KChar* daq_status = ns -> getDaqStatus();

  delete ns;

  KBool STATUS_STANDBY = ( strcmp( daq_status, "standby" ) == 0 );
  KBool STATUS_BUSY = ( strcmp( daq_status, "busy" ) == 0 );
  KBool STATUS_IDLE = ( strcmp( daq_status, "idle" ) == 0 );
  KBool STANDBY_CMD = ( ( strcmp(daq_cmd,"begin")==0 ) ||
				( strcmp(daq_cmd,"shutdown")==0 ) );
  KBool BUSY_CMD = ( strcmp(daq_cmd,"pause")==0 );
  KBool IDLE_CMD = ( ( strcmp(daq_cmd,"resume")==0 ) || 
				( strcmp(daq_cmd,"end")==0 ) );

  switch(DAQ_STATUS) {
  case standby:
    if( STATUS_STANDBY && STANDBY_CMD ) return KTrue;
    else return KFalse;

  case busy:
    if( STATUS_BUSY && BUSY_CMD ) return KTrue;
    else return KFalse;

  case idle:
    if( STATUS_IDLE && IDLE_CMD ) return KTrue;
    else return KFalse;

  case none:
  default:
    return KFalse;
  }
}
