// $Id: OfflineDAQModuleConstruction.cc,v 1.9 2002/07/30 04:20:52 iwai Exp $
// ==================================================================
//  OfflineDAQModuleConstruction.cc
//  03-December-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "OfflineDAQModuleConstruction.hh"
#include "T473DAQStatus.hh"
#include "T473DAQmodules.h"
#include "T473DAQconstants.h"

OfflineDAQModuleConstruction::OfflineDAQModuleConstruction()
  : spillEnd(0), lastTime(0), currentTime(0)
{
  runBeginList = new KonoeReadoutList(N_SECTION, RUN_BEGIN_RECORD);
  runEndList = new KonoeReadoutList(N_SECTION, RUN_END_RECORD);
  eventList = new KonoeReadoutList(N_SECTION, EVENT_RECORD);
  spillEndList = new KonoeReadoutList(N_SECTION, SPILL_END_RECORD);

  this -> generate();
  this -> install();

  this -> construct( RUN_END_RECORD );
  this -> construct( EVENT_RECORD );
  this -> construct( SPILL_END_RECORD );
  this -> construct( RUN_BEGIN_RECORD );

  localTimer = new KonoeTimerModule(KonoeTimerModule::kMilliSecond);
}

OfflineDAQModuleConstruction::~OfflineDAQModuleConstruction()
{
  delete theTimer;
  delete theClock;
  delete runNumber;
  delete eventNumber;
  delete theAppend;

  delete theCrate;
  delete theScaler0;
  delete theScaler1;
  delete theIR0;
  delete theIR1;
  delete theCR0;
  delete theCR1;
  delete theCR2;

  delete runEndList;
  delete eventList;
  delete spillEndList;
  delete runBeginList;

  delete localTimer;
}

void OfflineDAQModuleConstruction::generate()
{
  theCrate = new KonoeSoftwareCrate( MAX_SLOT );

  theADC0 = new KonoeSimADCModule(16, 16);
  theADC1 = new KonoeSimADCModule(9, 9);
  theADC2 = new KonoeSimADCModule(14, 14);

  theTDC0 = new KonoeSimTDCModule(8, 8);
  theTDC1 = new KonoeSimTDCModule(8, 8);
  theTDC2 = new KonoeSimTDCModule(8, 8);
  theTDC3 = new KonoeSimTDCModule(1, 1);
  theTDC4 = new KonoeSimTDCModule(8, 8);
  theTDC5 = new KonoeSimTDCModule(7, 7);

  theIR0 = new KonoeOptionalNumberModule();
  theIR1 = new KonoeOptionalNumberModule();

  theCR0 = new KonoeOptionalNumberModule(30732);
  theCR1 = new KonoeOptionalNumberModule();
  theCR2 = new KonoeOptionalNumberModule();

  theScaler0 = new KonoeSimTDCModule(12,12);
  theScaler1 = new KonoeSimTDCModule(4,4);

  theTimer = new KonoeTimerModule();
  theClock = new KonoeClockModule();
  theAppend = new KonoeOptionalNumberModule();
  runNumber = new KonoeOptionalNumberModule();
  eventNumber = new KonoeSoftCounter();
  runType = new KonoeOptionalNumberModule();

  return;
}

void OfflineDAQModuleConstruction::install() const
{
  this -> theCrate -> install(this -> theADC0);
  this -> theCrate -> install(this -> theADC1);
  this -> theCrate -> install(this -> theADC2);
  this -> theCrate -> install(this -> theTDC0);
  this -> theCrate -> install(this -> theTDC1);
  this -> theCrate -> install(this -> theTDC2);
  this -> theCrate -> install(this -> theTDC3);
  this -> theCrate -> install(this -> theTDC4);
  this -> theCrate -> install(this -> theTDC5);

  return;
}

void OfflineDAQModuleConstruction::add( KInt recordType, 
					KonoeReadoutSection* aSection)
{
  if ( recordType == RUN_BEGIN_RECORD )
    this -> runBeginList -> add( aSection );
  else if ( recordType == RUN_END_RECORD )
    this -> runEndList -> add( aSection );
  else if ( recordType == EVENT_RECORD )
    this -> eventList -> add( aSection );
  else if ( recordType == SPILL_END_RECORD )
    this -> spillEndList -> add( aSection );
  return;
}

void OfflineDAQModuleConstruction::construct( KInt recordType )
{ 
  KonoeReadoutSection* section;
  KonoeReadoutSegment* segment;
  KonoeReadoutItem* element;

  // GENERAL
  section = new KonoeReadoutSection(GENERAL_N_SEGMENT, GENERAL_SECTION_TYPE);
  segment = new KonoeReadoutSegment(RUNINFO_N_ELEMENT, RUNINFO_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> runNumber, (KInt)0 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> eventNumber,(KInt)1 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> runType, (KInt)2 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(IR_ID0_N_ELEMENT, IR_ID0_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theIR0, (KInt)1 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(IR_ID1_N_ELEMENT, IR_ID1_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theIR1, (KInt)1 );
  segment -> add( element );
  segment = new KonoeReadoutSegment( 1, CR_ID0_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theCR0, (KInt)0 );
  segment -> add( element );
  segment = new KonoeReadoutSegment( 1, CR_ID1_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theCR1, (KInt)0 );
  segment -> add( element );
  segment = new KonoeReadoutSegment( 1, CR_ID2_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theCR2, (KInt)0 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(TIMEINFO_N_ELEMENT,TIMEINFO_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theTimer, (KInt)0 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> theClock, (KInt)1 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(APPEND_N_ELEMENT, APPEND_SEGMENT_TYPE);
  section -> add( segment );
  for ( KInt i = 0; i < APPEND_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( this -> theAppend, i );
    segment -> add( element );
  }
  this -> add( recordType, section );



  // TDC CSI
  section = new KonoeReadoutSection(TDC_CSI_N_SEGMENT,TDC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theTDC0,TDC_CSI_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC1,TDC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC2,TDC_CSI_ID2_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC3,TDC_CSI_ID3_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  
  // ADC CSI
  section = new KonoeReadoutSection(ADC_CSI_N_SEGMENT,ADC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theADC0,ADC_CSI_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theADC1,ADC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );



  // TDC SCINTI
  section = 
    new KonoeReadoutSection(TDC_SCINTI_N_SEGMENT, TDC_SCINTI_SECTION_TYPE);
  segment = 
    new KonoeReadoutSegment(this->theTDC4,TDC_SCINTI_ID4_SEGMENT_TYPE);
  section -> add( segment ); 
  segment = 
    new KonoeReadoutSegment(this->theTDC5,TDC_SCINTI_ID5_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  // ADC SCINTI
  section = 
    new KonoeReadoutSection(ADC_SCINTI_N_SEGMENT, ADC_SCINTI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theADC2,ADC_SCINTI_ID2_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  // SCALER
  section = new KonoeReadoutSection(SCALER_N_SEGMENT, SCALER_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theScaler0,SCALER_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theScaler1,SCALER_ID1_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );

  return;
}

KonoeDataRecord* OfflineDAQModuleConstruction::read( KInt recordType )
{
  this -> spillEnd = KFalse;
  KonoeDataRecord* data;
  T473DAQStatus* daqStatus;

  if ( recordType == RUN_BEGIN_RECORD ) {    
    this -> runNumber -> setNumber( daqStatus -> getRunNumber() );
    this -> runType -> setNumber( daqStatus -> getRunType() );

    cout << "### RUN " << this->runNumber->getNumber() << " BEGIN" << endl;
    theCrate -> clear();
    data = this -> runBeginList -> read();
    this -> update();
    return ( data );
  } else if ( recordType == RUN_END_RECORD ) {
    cout << "### RUN " << this->runNumber->getNumber() << " END" << endl;
    data = this -> runEndList -> read();
    this -> initialize();
    return ( data );
  } else {
    cerr << "### NO EVENT" << endl;
    return (0);
  }
}

KonoeDataRecord* OfflineDAQModuleConstruction::read()
{
  this -> spillEnd = KFalse;
  KonoeDataRecord* data;
  this -> localTimer -> update();  
  this -> currentTime = this -> localTimer -> getCurrent();

  if ( currentTime - lastTime < 2000 ) {
    cout << "### EVENT " << this->eventNumber->getCounter() << endl;
    this -> theIR0 -> setNumber(1);
    data = this -> eventList -> read();
    this -> theIR0 -> setNumber(0);
    this -> update();
    return ( data );
  } else if ( currentTime - lastTime >= 2000 ) {
    cout << "### SPILL END" << endl;
    this -> spillEnd = KTrue;
    this -> lastTime = this -> currentTime;
    this -> theIR1 -> setNumber(1);
    data = this -> spillEndList -> read();
    this -> theIR1 -> setNumber(0);
    this -> spillEndUpdate();
    return ( data );
  } else {
    cerr << "### NO EVENT" << endl;
    return (0);
  }
}

void OfflineDAQModuleConstruction::update() const
{ 
  this -> theCrate -> update();
  this -> eventNumber -> update();
  this -> theTimer -> update();
  this -> theClock -> update();
  return;
}

void OfflineDAQModuleConstruction::initialize() const
{ 
  this -> theCrate -> initialize();
  this -> theScaler0 -> initialize();
  this -> theScaler1 -> initialize();
  this -> theIR0 -> initialize();
  this -> theIR1 -> initialize();
  this -> eventNumber -> initialize();
  this -> theTimer -> initialize();
  this -> theClock -> initialize();
  this -> theAppend -> initialize();
  this -> runType -> initialize();
  return;
}

void OfflineDAQModuleConstruction::spillEndUpdate() const
{ 
  this -> theCrate -> update();
  this -> eventNumber -> update();
  this -> theTimer -> update();
  this -> theClock -> update();
  this -> theScaler0 -> update();
  this -> theScaler1 -> update();
  return;
}
