// $Id: OfflineDAQProcess.cc,v 1.9 2000/12/13 19:03:26 iwai Exp $
// ==================================================================
//  OfflineDAQProcess.cc
//  03-December-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include <unistd.h>

#include <system/KonoeOutputObjFileBuffered.hh>
#include <system/KonoeOutputObjSocketBuffered.hh>
#include <system/KonoeMicroSleep.hh>

#include "OfflineDAQProcess.hh"
#include "T473DAQconstants.h"

OfflineDAQProcess::OfflineDAQProcess( KInt commandPort )
  : T473Process( commandPort )
{
  daqNameSpace = new T473DAQNameSpace();
  daqStatus = new T473DAQStatus();
  daqAction = new T473DAQAction();
  daqNameSpace -> update("DAQ_STATUS", "standby");
  daqModules = new OfflineDAQModuleConstruction();
}

OfflineDAQProcess::~OfflineDAQProcess()
{
  delete daqAction;
  delete daqNameSpace;
  delete daqStatus;
  delete daqModules;  
}

void OfflineDAQProcess::catchSignal(KInt signal)
{
#ifdef VERBOSE1
  daqStatus -> showStatus();
  daqStatus -> showCommand();
#endif

  KChar cmd[256];
  cmd_process -> send( "accept", 256 );
  cmd_process -> receive( cmd, 256 );

  if ( daqStatus -> isValidCommand(cmd) ) {
    if ( strcmp( cmd, "begin" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::begin );
      daqAction -> begin();
    } else if ( strcmp( cmd, "pause" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::pause );
      daqAction -> pause();
    } else if ( strcmp( cmd, "resume" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::resume ); 
      daqAction -> resume();
    } else if ( strcmp( cmd, "end" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::end );
      daqAction -> end();
    } else if ( strcmp( cmd, "shutdown" ) == 0 ) {
      daqStatus -> setCommand(T473DAQStatus::shutdown);
      daqAction -> shutdown();
    }
    cmd_process -> send( "true", 256 );
  } else {
    cmd_process -> send( "false", 256 );
  }

#ifdef VERBOSE1
  daqStatus -> showStatus();
  daqStatus -> showCommand();
#endif

  return;
}

void OfflineDAQProcess::setDaqRate( KInt rate )
{
  this -> daqRate = rate;
  KFloat takeTime = 0.15;  // unit is milli second
  KInt offset = (KInt)( takeTime * (this->daqRate) );
  KInt totalSleepTimeInSpill = 1000 - offset;
  this -> sleepTime = totalSleepTimeInSpill / ( this -> daqRate );
  return;
}

void OfflineDAQProcess::go()
{
  KBool isOpen(KFalse);
  KonoeDataRecord* data;
  T473DAQDataBuffer* daqDataBuffer;
  KonoeMicroSleep* microSleep = new KonoeMicroSleep();
  //KonoeOutputObjFileBuffered* output;
  KonoeOutputObjSocketBuffered* output;
  KChar* hubHost = this -> daqNameSpace -> getDataHubHost();
  KInt hubPort = this -> daqNameSpace -> getDataHubPort();

  while (1) {





    if ( daqStatus -> getStatus() == T473DAQStatus::standby ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::unknown ) {
	handler -> wait();
      } else if ( daqStatus -> getCommand() == T473DAQStatus::begin ) {
	if ( isOpen == KFalse ) {
	  //output = new KonoeOutputObjFileBuffered( "temporary.dat" );
	  output = new KonoeOutputObjSocketBuffered( hubHost, hubPort );
	  daqDataBuffer = new T473DAQDataBuffer( BUFFER_SIZE );
	  isOpen = KTrue;
	}
	data = this -> daqModules -> read( RUN_BEGIN_RECORD );
	if ( data != 0 ) {
	  daqDataBuffer -> add( data );
	  daqStatus -> setStatus( T473DAQStatus::busy );
	}
      } else if ( daqStatus -> getCommand() == T473DAQStatus::shutdown ) {
        this -> daqNameSpace -> extract();
        if ( strcmp( daqNameSpace -> getDaqStatus(), "none" ) != 0 ) {
          daqNameSpace -> update( "DAQ_STATUS", "none" );
          daqNameSpace -> update( "DAQ_PID", -1 );
	  cerr << "[DAQ] shutdown after 1 second." << endl;
          sleep(1);
        }
        break;
      }
    }



    else if ( daqStatus -> getStatus() == T473DAQStatus::busy ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::begin ) {
        data = this -> daqModules -> read();
	microSleep -> sleep( this -> sleepTime );
        if ( data != 0 ) {
          daqDataBuffer -> add( data );
	}
        if ( this -> daqModules -> isSpillEnd() ) {
          daqDataBuffer -> reset();
          while ( data = daqDataBuffer -> next() ) {
            output -> put( data );
	    output -> flush();
          }
          daqDataBuffer -> clear();
        }
      } else if ( daqStatus -> getCommand() == T473DAQStatus::pause ) {
        daqStatus -> setStatus( T473DAQStatus::idle );
      }
    }



    else if ( daqStatus -> getStatus() == T473DAQStatus::idle ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::pause ) {
        handler -> wait();
      } else if ( daqStatus -> getCommand() == T473DAQStatus::resume ) {
        daqStatus -> setStatus( T473DAQStatus::busy );
        daqStatus -> setCommand( T473DAQStatus::begin );
      } else if ( daqStatus -> getCommand() == T473DAQStatus::end ) {
        data = this -> daqModules -> read( RUN_END_RECORD );
        daqDataBuffer -> add( data );
        daqDataBuffer -> reset();
        while ( data = daqDataBuffer -> next() ) {
          output -> put( data );
	  output -> flush();
        }
        delete daqDataBuffer;
	delete output;
        daqStatus -> setStatus( T473DAQStatus::standby );
        daqStatus -> setCommand( T473DAQStatus::unknown );
	isOpen = KFalse;
      }
    }

  }

  delete microSleep;

  return;
}
