// $Id: T473DAQModuleConstruction.cc,v 1.30 2000/12/28 13:09:04 iwai Exp $
// ==================================================================
//  T473DAQModuleConstruction.cc
//  05-November-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "T473DAQModuleConstruction.hh"
#include "T473DAQStatus.hh"
#include "T473DAQmodules.h"
#include "T473DAQconstants.h"

T473DAQModuleConstruction::T473DAQModuleConstruction()
  : spillEnd(0)
{
  runBeginList = new KonoeReadoutList(N_SECTION, RUN_BEGIN_RECORD);
  runEndList = new KonoeReadoutList(N_SECTION, RUN_END_RECORD);
  eventList = new KonoeReadoutList(N_SECTION, EVENT_RECORD);
  spillEndList = new KonoeReadoutList(N_SECTION, SPILL_END_RECORD);

  this -> generate();
  this -> install();

  this -> construct( RUN_END_RECORD );
  this -> construct( EVENT_RECORD );
  this -> construct( SPILL_END_RECORD );
  this -> construct( RUN_BEGIN_RECORD );
}

T473DAQModuleConstruction::~T473DAQModuleConstruction()
{
  delete theTimer;
  delete theClock;
  delete runNumber;
  delete eventNumber;
  delete theAppend;

  delete theScaler0;
  delete theScaler1;
  delete theIR0;
  delete theIR1;
  delete theOR0;
  delete theOR1;
  delete theCrate;

  delete runEndList;
  delete eventList;
  delete spillEndList;
  delete runBeginList;
}

void T473DAQModuleConstruction::generate()
{
  theCrate = new KonoeCAMACCrate(NODE_NAME, CC_ID);

  theADC0 = new KonoeCAMACADC(this -> theCrate, ADC0);
  theADC1 = new KonoeCAMACADC(this -> theCrate, ADC1, ADC_CSI_ID1_N_ELEMENT);
  theADC2 = new KonoeCAMACADC(this->theCrate, ADC2, ADC_SCINTI_ID2_N_ELEMENT);
  theTDC0 = new KonoeCAMACTDC(this -> theCrate, TDC0);
  theTDC1 = new KonoeCAMACTDC(this -> theCrate, TDC1);
  theTDC2 = new KonoeCAMACTDC(this -> theCrate, TDC2);
  theTDC3 = new KonoeCAMACTDC(this -> theCrate, TDC3, TDC_CSI_ID3_N_ELEMENT);
  theTDC4 = new KonoeCAMACTDC(this -> theCrate, TDC4);
  theTDC5 = new KonoeCAMACTDC(this -> theCrate, TDC5);
  theIR0 = new KonoeCAMACInterruptRegister(this -> theCrate, IR0);
  theIR1 = new KonoeCAMACInterruptRegister(this -> theCrate, IR1);
  theCR0 = new KonoeCAMACCoincidenceRegister(this -> theCrate, CR0);
  theCR1 = new KonoeCAMACCoincidenceRegister(this -> theCrate, CR1);
  theCR2 = new KonoeCAMACCoincidenceRegister(this -> theCrate, CR2);
  theScaler0 = 
    new KonoeCAMACScaler(this->theCrate,SCALER0,SCALER_ID0_N_ELEMENT);
  theScaler1 = 
    new KonoeCAMACScaler(this->theCrate,SCALER1,SCALER_ID1_N_ELEMENT);

  theOR0 = new KonoeCAMACOutputRegister(this -> theCrate, OR0);
  theOR1 = new KonoeCAMACOutputRegister(this -> theCrate, OR1);

  theTimer = new KonoeTimerModule();
  theClock = new KonoeClockModule();
  theAppend = new KonoeOptionalNumberModule();
  runNumber = new KonoeOptionalNumberModule();
  eventNumber = new KonoeSoftCounter();
  runType = new KonoeOptionalNumberModule();

  theIR0 -> writeMask((KUShort)254);
  theIR1 -> writeMask((KUShort)254);

  return;
}

void T473DAQModuleConstruction::install() const
{
  this -> theCrate -> install(this -> theADC0);
  this -> theCrate -> install(this -> theADC1);
  this -> theCrate -> install(this -> theADC2);
  this -> theCrate -> install(this -> theTDC0);
  this -> theCrate -> install(this -> theTDC1);
  this -> theCrate -> install(this -> theTDC2);
  this -> theCrate -> install(this -> theTDC3);
  this -> theCrate -> install(this -> theTDC4);
  this -> theCrate -> install(this -> theTDC5);
  this -> theCrate -> install(this -> theCR0);
  this -> theCrate -> install(this -> theCR1);
  this -> theCrate -> install(this -> theCR2);

  // -------------------------------------------------------
  //  <<< NOT-INSTALL MODULES >>>
  //  o SCALER
  //    It's updated on only Spill-End.
  //  o INTERRUPT REGISTER
  //    It's updated on Spill-End.
  //  o OUTPUT REGISTER
  //    It should not be updated.
  //  o SOFTWARE MODULES
  //    These are not CAMAC module.
  // -------------------------------------------------------

  return;
}

void T473DAQModuleConstruction::add( KInt recordType, 
				     KonoeReadoutSection* aSection)
{
  if ( recordType == RUN_BEGIN_RECORD )
    this -> runBeginList -> add( aSection );
  else if ( recordType == RUN_END_RECORD )
    this -> runEndList -> add( aSection );
  else if ( recordType == EVENT_RECORD )
    this -> eventList -> add( aSection );
  else if ( recordType == SPILL_END_RECORD )
    this -> spillEndList -> add( aSection );
  return;
}

void T473DAQModuleConstruction::construct( KInt recordType )
{ 
  KonoeReadoutSection* section;
  KonoeReadoutSegment* segment;
  KonoeReadoutItem* element;

  // GENERAL
  section = new KonoeReadoutSection(GENERAL_N_SEGMENT, GENERAL_SECTION_TYPE);
  segment = new KonoeReadoutSegment(RUNINFO_N_ELEMENT, RUNINFO_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> runNumber, (KInt)0 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> eventNumber,(KInt)1 );
  segment -> add( element );
  element = new KonoeReadoutItem( this -> runType, (KInt)2 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(IR_ID0_N_ELEMENT, IR_ID0_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theIR0, (KInt)1 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(IR_ID1_N_ELEMENT, IR_ID1_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theIR1, (KInt)1 );
  segment -> add( element );
  segment = new KonoeReadoutSegment(this -> theCR0, CR_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this -> theCR1, CR_ID1_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this -> theCR2, CR_ID2_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(TIMEINFO_N_ELEMENT,TIMEINFO_SEGMENT_TYPE);
  section -> add( segment );
  element = new KonoeReadoutItem( this -> theTimer, (KInt)0 );
  segment -> add(element);
  element = new KonoeReadoutItem( this -> theClock, (KInt)1 );
  segment -> add(element);
  segment = new KonoeReadoutSegment(APPEND_N_ELEMENT, APPEND_SEGMENT_TYPE);
  section -> add(segment);
  for ( KInt i = 0; i < APPEND_N_ELEMENT; i++ ) {
    element = new KonoeReadoutItem( this -> theAppend, i );
    segment -> add(element);
  }
  this -> add( recordType, section );


  // TDC CSI
  section = new KonoeReadoutSection(TDC_CSI_N_SEGMENT,TDC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theTDC0,TDC_CSI_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC1,TDC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC2,TDC_CSI_ID2_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theTDC3,TDC_CSI_ID3_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );

  
  // ADC CSI
  section = new KonoeReadoutSection(ADC_CSI_N_SEGMENT,ADC_CSI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theADC0,ADC_CSI_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theADC1,ADC_CSI_ID1_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  // TDC SCINTI
  section = 
    new KonoeReadoutSection(TDC_SCINTI_N_SEGMENT, TDC_SCINTI_SECTION_TYPE);
  segment = 
    new KonoeReadoutSegment(this->theTDC4,TDC_SCINTI_ID4_SEGMENT_TYPE);
  section -> add( segment ); 
  segment = 
    new KonoeReadoutSegment(this->theTDC5,TDC_SCINTI_ID5_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  // ADC SCINTI
  section = 
    new KonoeReadoutSection(ADC_SCINTI_N_SEGMENT, ADC_SCINTI_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theADC2,ADC_SCINTI_ID2_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  // SCALER
  section = new KonoeReadoutSection(SCALER_N_SEGMENT, SCALER_SECTION_TYPE);
  segment = new KonoeReadoutSegment(this->theScaler0,SCALER_ID0_SEGMENT_TYPE);
  section -> add( segment );
  segment = new KonoeReadoutSegment(this->theScaler1,SCALER_ID1_SEGMENT_TYPE);
  section -> add( segment );
  this -> add( recordType, section );


  return;
}

KonoeDataRecord* T473DAQModuleConstruction::read( KInt recordType )
{
  this -> spillEnd = KFalse;
  KonoeDataRecord* data;
  T473DAQStatus* daqStatus;

  if ( recordType == RUN_BEGIN_RECORD ) {    
    this -> runNumber -> setNumber( daqStatus -> getRunNumber() );
    this -> runType -> setNumber( daqStatus -> getRunType() );
    this -> theIR0 -> enableLAM();
    this -> theCrate -> waitLAM( TIMEOUT, this -> theIR0 );
    if ( this -> theCrate -> getStatus() == 0 ) {
      cout << "### RUN " << this->runNumber->getNumber() << " BEGIN" << endl;
      data = this -> runBeginList -> read();
      this -> update();
      return ( data );
    } else {
#ifdef VERBOSE1
      switch ( this -> theCrate -> getStatus() ) {
      case -5:
	cerr << "[DAQ] CC7X00 is offline !" << endl;
	break;
      case -11:
	cerr << "[DAQ] other interrupt !" << endl;
	break;
      case -13:
	cerr << "[DAQ] not support !" << endl;
	break;
      case -22:
	cerr << "[DAQ] time setting has probrem !" << endl;
	break;
      case -62:
	cerr << "[DAQ] time out, no interrupt !" << endl;
	break;
      default:
	cerr << "[DAQ] status error !" << endl;
	break;
      }
#endif
      return (0);
    }
  } else if ( recordType == RUN_END_RECORD ) {
    cout << "### RUN " << this->runNumber->getNumber() << " END" << endl;
    data = this -> runEndList -> read();
    this -> initialize();
    return ( data );
  }
}

KonoeDataRecord* T473DAQModuleConstruction::read()
{
  this -> spillEnd = KFalse;
  KonoeDataRecord* data;

  this -> theIR0 -> enableLAM();
  this -> theIR1 -> enableLAM();

  this -> theCrate -> waitLAM( TIMEOUT, this -> theIR0, this -> theIR1 );
  if ( this -> theCrate -> getStatus() == 0 ) {
    if ( this -> theCrate -> getLAM() == IR0 ) { // EVENT
#ifdef VERBOSE1
      cout << "### EVENT " << this->eventNumber->getCounter() << endl;
#endif
      data = this -> eventList -> read();
      this -> update();
      return ( data );
    } else if ( this -> theCrate -> getLAM() == IR1 ) { // SPILL-END
      cout << "### EVENT " << this->eventNumber->getCounter() << endl;
      cout << "### SPILL END" << endl;
      this -> spillEnd = KTrue;
      data = this -> spillEndList -> read();
      this -> spillEndUpdate();
      return ( data );
    }
  } else {
#ifdef VERBOSE1
    switch ( this -> theCrate -> getStatus() ) {
    case -5:
      cerr << "[DAQ] is CC7X00 offline ?" << endl;
      break;
    case -11:
      cerr << "[DAQ] other interrupt !" << endl;
      break;
    case -13:
      cerr << "[DAQ] not support !" << endl;
      break;
    case -22:
      cerr << "[DAQ] time setting has probrem !" << endl;
      break;
    case -62:
      cerr << "[DAQ] time out, no interrupt !" << endl;
      break;
    default:
      cerr << "[DAQ] status error !" << endl;
      break;
    }
#endif
    return (0);
  }
}

void T473DAQModuleConstruction::update() const
{ 
  this -> theCrate -> update();
  this -> eventNumber -> update();
  this -> theTimer -> update();
  this -> theClock -> update();
  this -> theIR0 -> update();
  return;
}

void T473DAQModuleConstruction::initialize() const
{ 
  this -> theCrate -> initialize();
  this -> theScaler0 -> initialize();
  this -> theScaler1 -> initialize();
  this -> theIR0 -> initialize();
  this -> theIR1 -> initialize();
  this -> eventNumber -> initialize();
  this -> theTimer -> initialize();
  this -> theClock -> initialize();
  this -> theAppend -> initialize();
  this -> runType -> initialize();
  return;
}

void T473DAQModuleConstruction::spillEndUpdate() const
{ 
  this -> theCrate -> update();
  this -> eventNumber -> update();
  this -> theTimer -> update();
  this -> theClock -> update();
  this -> theScaler0 -> clear();
  this -> theScaler1 -> clear();
  this -> theIR0 -> update();
  this -> theIR1 -> update();
  return;
}
