// $Id: T473DAQProcess.cc,v 1.30 2000/12/24 12:15:04 iwai Exp $
// ==================================================================
//  T473DAQProcess.cc
//  05-November-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include <unistd.h>

#include <system/KonoeOutputObjFileBuffered.hh>
#include <system/KonoeOutputObjSocketBuffered.hh>

#include "T473DAQProcess.hh"
#include "T473DAQconstants.h"

T473DAQProcess::T473DAQProcess( KInt commandPort )
  : T473Process( commandPort )
{
  daqNameSpace = new T473DAQNameSpace();
  daqStatus = new T473DAQStatus();
  daqAction = new T473DAQAction();
  daqNameSpace -> update("DAQ_STATUS", "standby");
  daqModules = new T473DAQModuleConstruction();
  this -> daqModules -> setI();
  this -> daqModules -> cancellLatch();
}

T473DAQProcess::~T473DAQProcess()
{
  delete daqAction;
  delete daqNameSpace;
  delete daqStatus;
  delete daqModules;  
}

void T473DAQProcess::catchSignal(KInt signal)
{
#ifdef VERBOSE1
  daqStatus -> showStatus();
  daqStatus -> showCommand();
#endif

  KChar cmd[256];
  cmd_process -> send( "accept", 256 );
  cmd_process -> receive( cmd, 256 );

  if ( daqStatus -> isValidCommand(cmd) ) {
    if ( strcmp( cmd, "begin" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::begin );
      daqAction -> begin();
    } else if ( strcmp( cmd, "pause" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::pause );
      daqAction -> pause();
    } else if ( strcmp( cmd, "resume" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::resume ); 
      daqAction -> resume();
    } else if ( strcmp( cmd, "end" ) == 0 ) {
      daqStatus -> setCommand( T473DAQStatus::end );
      daqAction -> end();
    } else if ( strcmp( cmd, "shutdown" ) == 0 ) {
      daqStatus -> setCommand(T473DAQStatus::shutdown);
      daqAction -> shutdown();
    }
    cmd_process -> send( "true", 256 );
  } else {
    cmd_process -> send( "false", 256 );
  }

#ifdef VERBOSE1
  daqStatus -> showStatus();
  daqStatus -> showCommand();
#endif

  return;
}

void T473DAQProcess::go()
{
  KBool isOpen(KFalse);
  KonoeDataRecord* data;
  T473DAQDataBuffer* daqDataBuffer;
  KInt memoryAddress;
  //KonoeOutputObjFileBuffered* output;
  KonoeOutputObjSocketBuffered* output;
  KChar* hubHost = this -> daqNameSpace -> getDataHubHost();
  KInt hubPort = this -> daqNameSpace -> getDataHubPort();

  // ------------- KONOE study
  //KonoeTimerModule localTime;
  //KInt interval;
  // ------------- KONOE study

  while (1) {

    if ( daqStatus -> getStatus() == T473DAQStatus::standby ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::unknown ) {
	handler -> wait();
      } else if ( daqStatus -> getCommand() == T473DAQStatus::begin ) {
	if ( isOpen == KFalse ) {
	  //output = new KonoeOutputObjFileBuffered( "temporary.dat" );
	  output = new KonoeOutputObjSocketBuffered( hubHost, hubPort );
	  daqDataBuffer = new T473DAQDataBuffer( BUFFER_SIZE );
	  isOpen = KTrue;
	  this -> daqModules -> removeI();
	}
	data = this -> daqModules -> read( RUN_BEGIN_RECORD );
	if ( data != 0 ) {
	  daqDataBuffer -> add( data );
	  this -> daqModules -> cancellLatch();
	  daqStatus -> setStatus( T473DAQStatus::busy );
	}
      } else if ( daqStatus -> getCommand() == T473DAQStatus::shutdown ) {
	this -> daqNameSpace -> extract();
	if ( strcmp( daqNameSpace -> getDaqStatus(), "none" ) != 0 ) {
	  daqNameSpace -> update( "DAQ_STATUS", "none" );
	  daqNameSpace -> update( "DAQ_PID", -1 );
	  sleep(1);
	}
	break;
      }
    }



    else if ( daqStatus -> getStatus() == T473DAQStatus::busy ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::begin ) {
	data = this -> daqModules -> read();
	memoryAddress = (KInt)data;
	if ( data != 0 && memoryAddress != 0x10 ) {
	  daqDataBuffer -> add( data );
	  this -> daqModules -> cancellLatch();
	}
	if ( this -> daqModules -> isSpillEnd() ) {

	  // ------------- KONOE study
	  //localTime.update();
	  // ------------- KONOE study

	  this -> daqModules -> setI();
	  daqDataBuffer -> reset();
	  while ( data = daqDataBuffer -> next() ) {
	    output -> put( data );
	    output -> flush();
	  }
	  daqDataBuffer -> clear();
	  this -> daqModules -> removeI();

	  // ------------- KONOE study
	  //localTime.update();
	  //interval = localTime.getInterval();
	  //cout << "HubäƥХåե򥯥ꥢΤˤä : " 
	  //     << interval << " [us]" << endl;
	  //if ( interval > 2000000 )
	  //  cerr << "2äۤƤޤ !!!!!" << endl;
	  // ------------- KONOE study

	}
      } else if ( daqStatus -> getCommand() == T473DAQStatus::pause ) {
	this -> daqModules -> setI();
	daqStatus -> setStatus( T473DAQStatus::idle );
      }
    }



    else if ( daqStatus -> getStatus() == T473DAQStatus::idle ) {
      if ( daqStatus -> getCommand() == T473DAQStatus::pause ) {
	handler -> wait();
      } else if ( daqStatus -> getCommand() == T473DAQStatus::resume ) {
	this -> daqModules -> removeI();
	daqStatus -> setStatus( T473DAQStatus::busy );
	daqStatus -> setCommand( T473DAQStatus::begin );
      } else if ( daqStatus -> getCommand() == T473DAQStatus::end ) {
	data = this -> daqModules -> read( RUN_END_RECORD );
	daqDataBuffer -> add( data );
	this -> daqModules -> cancellLatch();
	daqDataBuffer -> reset();
	while ( data = daqDataBuffer -> next() ) {
	  output -> put( data );
	  output -> flush();
	}
	delete daqDataBuffer;
	delete output;
	daqStatus -> setStatus( T473DAQStatus::standby );
	daqStatus -> setCommand( T473DAQStatus::unknown );
	isOpen = KFalse;
      }
    }

  }

  return;
}
