// $Id: T473DAQStatus.cc,v 1.8 2000/12/05 08:51:57 iwai Exp $
// ==================================================================
//  T473DAQStatus.cc
//  05-November-2000'
//  Go Iwai [iwai@hep.sc.niigata-u.ac.jp]
// ==================================================================
#include "T473DAQStatus.hh"
#include "T473DAQNameSpace.hh"

T473DAQStatus::DAQStatus T473DAQStatus::DAQ_STATUS = T473DAQStatus::none;
T473DAQStatus::DAQCommand T473DAQStatus::DAQ_COMMAND = T473DAQStatus::unknown;
KInt T473DAQStatus::RUN_NUMBER = 0;
KInt T473DAQStatus::RUN_TYPE = 0;

KInt T473DAQStatus::initialize( const DAQStatus status )
{
  if ( DAQ_STATUS == none && status != none ) {
    DAQ_STATUS = status;
    return (1);
  } else {
    return (0);
  }
}

KInt T473DAQStatus::finalize()
{
  if ( DAQ_STATUS != none ) {
    DAQ_STATUS = none;
    return (1);
  } else {
    return (0);
  }
}

void T473DAQStatus::showStatus()
{
  if ( DAQ_STATUS == none )
    cout << "[DAQ] NONE" << endl;
  else if ( DAQ_STATUS == standby )
    cout << "[DAQ] STANDBY" << endl;
  else if ( DAQ_STATUS == busy )
    cout << "[DAQ] BUSY" << endl;
  else if ( DAQ_STATUS == idle )
    cout << "[DAQ] IDLE" << endl;
  else
    cout << "[DAQ] UNKNOWN STATUS" << endl;
  return;
}

void T473DAQStatus::showCommand()
{
  if ( DAQ_COMMAND == unknown )
    cout << "[DAQ] UNKNOWN" << endl;
  else if ( DAQ_COMMAND == begin )
    cout << "[DAQ] BEGIN" << endl;
  else if ( DAQ_COMMAND == pause )
    cout << "[DAQ] PAUSE" << endl;
  else if ( DAQ_COMMAND == resume )
    cout << "[DAQ] RESUME" << endl;
  else if ( DAQ_COMMAND == end )
    cout << "[DAQ] END" << endl;
  else if ( DAQ_COMMAND == shutdown )
    cout << "[DAQ] SHUTDOWN" << endl;
  else
    cout << "[DAQ] UNKNOWN COMMAND" << endl;
  return;
}

KBool T473DAQStatus::isInstanciate() 
{ 
  return ( DAQ_STATUS != none ); 
}

KBool T473DAQStatus::isValidCommand( KChar* daq_cmd )
{
  T473DAQNameSpace* ns = new T473DAQNameSpace();
  KChar* daq_status = ns -> getDaqStatus();
#ifdef VERBOSE1
  cout << "T473DAQStatus::isValidCommand " << daq_status << endl;
#endif
  delete ns;

  KBool STATUS_STANDBY = ( strcmp(daq_status, "standby") == 0 );
  KBool STATUS_BUSY = ( strcmp(daq_status, "busy") == 0 );
  KBool STATUS_IDLE = ( strcmp(daq_status, "idle") == 0 );
  KBool STANDBY_CMD 
    = ( (strcmp(daq_cmd,"begin")==0) || (strcmp(daq_cmd,"shutdown")==0) );
  KBool BUSY_CMD = ( strcmp(daq_cmd, "pause") == 0 );
  KBool IDLE_CMD 
    = ( (strcmp(daq_cmd,"resume")==0) || (strcmp(daq_cmd,"end")==0) );

  switch ( DAQ_STATUS ) {
  case standby:
    if ( STATUS_STANDBY && STANDBY_CMD ) return KTrue;
    else return KFalse;
  case busy:
    if ( STATUS_BUSY && BUSY_CMD ) return KTrue;
    else return KFalse;
  case idle:
    if ( STATUS_IDLE && IDLE_CMD ) return KTrue;
    else return KFalse;
  case none:
  default:
    return KFalse;
  }
}
