#include	"T473DataHub.hh"

// name space entry string
const KChar* T473DataHub::DATAHUB_INPUT_PORT  = "_INPUT_PORT";
const KChar* T473DataHub::DATAHUB_OUTPUT_PORT = "_OUTPUT_PORT";
const KChar* T473DataHub::DATAHUB_STORAGE_FILE_NAME 
			= "_STORAGE_FILE_NAME";
const KChar* T473DataHub::DATAHUB_BUFFER_SIZE = "_BUFFER_SIZE";

// command strings
const KChar* T473DataHub::DATAHUB_CMD_FILE_CLOSE = "DATAHUB_CMD_FILE_CLOSE";
const KChar* T473DataHub::DATAHUB_CMD_NEW_FILE   = "DATAHUB_CMD_NEW_FILE";
const KChar* T473DataHub::DATAHUB_CMD_CHANGE_BUFFER_SIZE 
			= "DATAHUB_CMD_CHANGE_BUFFER_SIZE";	
const KChar* T473DataHub::DATAHUB_CMD_IGNORE_OUTPUT_PORT
			= "DATAHUB_CMD_IGNORE_OUTPUT_PORT";

// request strings
const KChar* T473DataHub::DATAHUB_REQ_HEAD    = "DATAHUB_REQ_HEAD";
const KChar* T473DataHub::DATAHUB_REQ_CURRENT = "DATAHUB_REQ_CURRENT";
const KChar* T473DataHub::DATAHUB_REQ_TAIL    = "DATAHUB_REQ_TAIL";

// default value
const KChar* T473DataHub::DEFAULT_STORAGE_FILE_NAME 
			= "t473_default_data_file.dat";

T473DataHub::T473DataHub( KChar* tag_name )
	{
	name = tag_name;	// Data Hub ID name

	KInt	i_port, o_port;

	KInt	MAX_ENTRY_NAME_LENGTH = 256;
	KChar	entry_name[MAX_ENTRY_NAME_LENGTH]; 

	cout << "Getting the parameter from the NameSpace..." 
		<< endl << endl;

	name_table = new KonoeNameSpaceTable();		
	KonoeNameSpaceValue * value;

	strcpy( entry_name, name );
	value = name_table -> getValue( strcat(entry_name, 
			DATAHUB_INPUT_PORT) );

	if ( value )
		{
		i_port = atoi ( ( KChar* ) * value ); 
		cout << "DATAHUB_INPUT_PORT: " << i_port << endl;
		}
	else
		{
		i_port = INPUT_PORT;
		cout << "DATAHUB_INPUT_PORT is not found." << endl
			<< "Use the default value: " << i_port 
			<< endl << endl;
		}
	in_server  = new KonoeObjectServerSocket( i_port );

	strcpy( entry_name, name );
	value = name_table -> getValue( strcat(entry_name, 
			DATAHUB_OUTPUT_PORT) );
	if ( value )
		{
		o_port = atoi( ( KChar * ) * value ); 
		cout << "DATAHUB_OUTPUT_PORT: " << o_port << endl;
		}
	else
		{
		o_port = OUTPUT_PORT;	
		cout << "DATAHUB_OUTPUT_PORT is not found." << endl
			<< "Use the default value: " << o_port
			<< endl << endl;
		}
	out_server = new KonoeObjectServerSocket( o_port );

	strcpy( entry_name, name );
	value = name_table -> getValue( strcat(entry_name, 
			DATAHUB_BUFFER_SIZE) );
	if ( value )
		{
		buffer_size = atoi( ( KChar * ) * value ); 
		cout << "DATAHUB_BUFFER_SIZE: " << buffer_size << endl;
		}
	else
		{
		buffer_size = BUFFER_SIZE;
		cout << "DATAHUB_BUFFER_SIZE is not found." << endl
			<< "Use the default value: " << buffer_size 
			<< endl << endl;
		}

	out_file = 0;
	cmd_new_file();	

        in_server -> installCM( new KonoeDataRecordCM( ) );

	write_address = 0;

	for ( KInt i = 0; i < KonoeObjectServerSocket::MAX_CLIENTS; i++ )
		{ data_length[i] = 0; }


	WRITE_TO_FILE = KTrue;
	IGNORE_OUTPUT_PORT = KFalse;

	}

T473DataHub::~T473DataHub()
	{
	delete  in_server;
	delete  out_server;
	delete  name_table;

	for ( KInt i = 0; i < buffer_size; i++ )
		{ delete buffer[i]; }

	delete [] buffer;		
	}

KInt  T473DataHub::go()
	{
	cout << "\nDataHub: start..." << endl;

	buffer = new void* [ buffer_size ];	

	for ( KInt i = 0; i < buffer_size; i++ )
		{ buffer[i] = 0;  }

	while ( true ) // probably state != END
		{
// from input port
		client = in_server -> wait(1);
		if ( client )
			{ get_data();}

		if ( ! IGNORE_OUTPUT_PORT )
			{
			client = out_server -> wait(1);	
			if ( client )
				{ request_analyzer();} 
			}

		}
	}

KInt  T473DataHub::ring_counter( KInt value )
	{
	while ( value < 0 )
		{ value += buffer_size; }

	while ( value >= buffer_size)
		{ value -= buffer_size; }

	return  value;
	}

KInt  T473DataHub::find_index( KonoeObjectSocket * client )
	{
	KonoeObjectSocket ** clients = out_server -> getClients();

	for ( KInt i = 0; i < KonoeObjectServerSocket::MAX_CLIENTS; i++ )
		if ( client == clients[i] )
			return i;

/*
	// error: client is not found. 
	exit(-1);
*/
	}
	
void  T473DataHub::request_analyzer()
	{
	const KInt MAX_STRING_LENGTH = 256;		
	KChar  socket_mess[ MAX_STRING_LENGTH ];

//	KonoeStreamableObject*  item;
	void * item;	

	client -> getSocket() -> receive( socket_mess, MAX_STRING_LENGTH );

	KInt index = find_index( client );

	if ( socket_mess[0] ==  '\0' )	// null check
		{
		out_server -> close( client );
		}

	if ( data_length[index] <= 0 ) 
		return;		// no data. 

	if ( ! strcmp(socket_mess, DATAHUB_REQ_HEAD) )
		{
		item = // ( KonoeStreamableObject * )
			 buffer[ ring_counter( write_address - 
				buffer_size + 1) ];
		put_data( item );
		}
	else if ( ! strcmp(socket_mess, DATAHUB_REQ_CURRENT) )
		{
		item =// ( KonoeStreamableObject * )
				buffer[ ring_counter( write_address -
					data_length[ index ] + 1 ) ];

		data_length[ index ]--;

		put_data( item );
		}

 	else if ( ! strcmp(socket_mess, DATAHUB_REQ_TAIL) )
		{
		item = // ( KonoeStreamableObject * )
				buffer[ write_address ];
		put_data( item ); 
		}

	else if ( ! strcmp(socket_mess, DATAHUB_CMD_FILE_CLOSE) )
		{
		cout	<< "Attention !:\tData file is closed." << endl;
			
		WRITE_TO_FILE = KFalse;
		delete	out_file;
		out_file = 0;
		}

	else if ( ! strcmp(socket_mess, DATAHUB_CMD_NEW_FILE) )
		{
		cmd_new_file();
		}

	else if ( ! strcmp(socket_mess, DATAHUB_CMD_CHANGE_BUFFER_SIZE) )
		{
		// not implemented yet
		}
	
	else if ( ! strcmp(socket_mess, DATAHUB_CMD_IGNORE_OUTPUT_PORT) )
		{
		IGNORE_OUTPUT_PORT = ! IGNORE_OUTPUT_PORT;
		cout << "From now, this program ignores the output port\n";  
		}
	}

void  T473DataHub::get_data()
	{
	KonoeStreamableObject *  data = in_server -> get();

	if ( ! data )	// null check
		{ 
		in_server -> close( client ); 
		return ;
		}

	write_address = ring_counter( write_address + 1 );

	// These two casts are very important.
	delete  ( KonoeDataRecord * ) buffer[ write_address ];
	buffer[ write_address ] = ( KonoeDataRecord * ) data;

	for ( KInt i = 0; i < KonoeObjectServerSocket::MAX_CLIENTS; i++ )
		{
		data_length[ i ]++;
		if ( data_length[ i ] > buffer_size )
			data_length[ i ] = buffer_size;
		}

	// write the object to file.
	if ( WRITE_TO_FILE ) 
		if ( data )  out_file -> put( data );

	}

void  T473DataHub::put_data( void * data )
	{
	if ( data )
		{ client -> put( ( KonoeDataRecord * ) data); }
	}

void  T473DataHub::cmd_new_file()
	{
	WRITE_TO_FILE = KTrue;
	delete	out_file;

	KInt	MAX_ENTRY_NAME_LENGTH = 256;
	KChar	entry_name[MAX_ENTRY_NAME_LENGTH]; 

	KonoeNameSpaceValue * value;
	strcpy( entry_name, name );
	value = name_table -> getValue( strcat(entry_name, 
				DATAHUB_STORAGE_FILE_NAME ) );

	const KInt FILE_NAME_LENGTH = 256;	
	KChar	file_name[FILE_NAME_LENGTH];

	if ( value )
		{
		strcpy( file_name,  ( KChar * ) * value ); 
		cout << "DATAHUB_STORAGE_FILE_NAME: " << file_name 
							<< endl;
		}
	else
		{
		strcpy( file_name, DEFAULT_STORAGE_FILE_NAME );
		cout << "DATAHUB_STORAGE_FILE_NAME is not found." << endl 
				<< "Use the default value: " 
				<< file_name << endl;
		}
	out_file = new KonoeOutputObjFile(file_name);
	
	}






