#include"T473DataHubForPS.hh"
#include<system/KonoeHostSystem.hh>
#include<system/KonoeFDClient.hh>
#include<unistd.h>
#include<stdlib.h>
#include<sys/wait.h>
#include<sys/types.h>
#include<sys/stat.h>


// name space entry string
const KChar* T473DataHubForPS::DATAHUB_INPUT_PORT  = "_INPUT_PORT";
const KChar* T473DataHubForPS::DATAHUB_OUTPUT_PORT = "_OUTPUT_PORT";
const KChar* T473DataHubForPS::DATAHUB_STORAGE_FILE_NAME 
			= "_STORAGE_FILE_NAME";
const KChar* T473DataHubForPS::DATAHUB_BUFFER_SIZE = "_BUFFER_SIZE";

// command strings
const KChar* T473DataHubForPS::DATAHUB_CMD_FILE_CLOSE 
= "DATAHUB_CMD_FILE_CLOSE";
const KChar* T473DataHubForPS::DATAHUB_CMD_NEW_FILE   = "DATAHUB_CMD_NEW_FILE";
const KChar* T473DataHubForPS::DATAHUB_CMD_CHANGE_BUFFER_SIZE 
			= "DATAHUB_CMD_CHANGE_BUFFER_SIZE";	
const KChar* T473DataHubForPS::DATAHUB_CMD_IGNORE_OUTPUT_PORT
			= "DATAHUB_CMD_IGNORE_OUTPUT_PORT";

// request strings
const KChar* T473DataHubForPS::DATAHUB_REQ_HEAD    = "DATAHUB_REQ_HEAD";
const KChar* T473DataHubForPS::DATAHUB_REQ_CURRENT = "DATAHUB_REQ_CURRENT";
const KChar* T473DataHubForPS::DATAHUB_REQ_TAIL    = "DATAHUB_REQ_TAIL";

// default value
const KChar* T473DataHubForPS::DEFAULT_STORAGE_FILE_NAME 
= "t473_default_data_file.dat";

T473DataHubForPS::T473DataHubForPS( KChar* tag_name ) {

  name = tag_name;	// Data Hub ID name
  
  KInt	i_port, o_port;

  KInt	MAX_ENTRY_NAME_LENGTH = 256;
  KChar	entry_name[MAX_ENTRY_NAME_LENGTH]; 
  
  cout << "Getting the parameter from the NameSpace..." 
       << endl << endl;
  
  name_table = new KonoeNameSpaceTable();		
  KonoeNameSpaceValue * value;
  
  strcpy( entry_name, name );
  value = name_table -> getValue( strcat(entry_name, DATAHUB_INPUT_PORT) );

  if ( value ){
    i_port = atoi ( ( KChar* ) * value ); 
    cout << "DATAHUB_INPUT_PORT: " << i_port << endl;
  } 
  else {
    i_port = INPUT_PORT;
    cout << "DATAHUB_INPUT_PORT is not found." << endl
	 << "Use the default value: " << i_port 
	 << endl << endl;
  }

  in_server  = new KonoeObjectServerSocketBuffered( i_port );

  strcpy( entry_name, name );
  value = name_table -> getValue( strcat(entry_name, DATAHUB_OUTPUT_PORT) );
  if ( value ){
    o_port = atoi( ( KChar * ) * value ); 
    cout << "DATAHUB_OUTPUT_PORT: " << o_port << endl;
  }
  else {
    o_port = OUTPUT_PORT;	
    cout << "DATAHUB_OUTPUT_PORT is not found." << endl
	 << "Use the default value: " << o_port
	 << endl << endl;
  }

  out_server = new KonoeObjectServerSocketBuffered( o_port );

  strcpy( entry_name, name );
  value = name_table -> getValue( strcat(entry_name, DATAHUB_BUFFER_SIZE) );

  if ( value ) {
    buffer_size = atoi( ( KChar * ) * value ); 
    cout << "DATAHUB_BUFFER_SIZE: " << buffer_size << endl;
  }
  else {
    buffer_size = BUFFER_SIZE;
    cout << "DATAHUB_BUFFER_SIZE is not found." << endl
	 << "Use the default value: " << buffer_size 
	 << endl << endl;
  }

  out_file = 0;
  cmd_new_file();	

  in_server -> installCM( new KonoeDataRecordCM( ) );
  
  write_address = 0;
  
  for ( KInt i = 0; i < KonoeObjectServerSocketBuffered::MAX_CLIENTS; i++ )
    { data_length[i] = 0; }
  
  WRITE_TO_FILE = KTrue;
  IGNORE_OUTPUT_PORT = KFalse;

  setter = new T473NameSpaceSetter();

  KInt pid;
  KChar host[256];
  pid = KonoeHostSystem::getPid();
  KonoeHostSystem::getHostName(host,256);
  setter->set("DATAHUB_HOST",host);
  setter->set("DATAHUB_PID",pid);

}

T473DataHubForPS::~T473DataHubForPS() {
  delete  in_server;
  delete  out_server;
  delete  name_table;

  setter->set("DATAHUB_STATUS","none");

  for ( KInt i = 0; i < buffer_size; i++ )
    { delete buffer[i]; }
  
  delete [] buffer;		
}

void  T473DataHubForPS::flush( ){

  cout <<  "T473DataHubForPS::flush()" << endl;

  for ( KInt i = 0; i < buffer_size; i++ )
    delete (KonoeDataRecord*)buffer[i];  
  
  delete [] buffer;		

  write_address = 0;
  
  for ( KInt i = 0; i < KonoeObjectServerSocketBuffered::MAX_CLIENTS; i++ )
    { data_length[i] = 0; }

  buffer = new void* [ buffer_size ];	
  
  for ( KInt i = 0; i < buffer_size; i++ ){ 
    buffer[i] = 0;  
  }

}

KInt  T473DataHubForPS::go()	{

  mode == READY;

  setter->set("DATAHUB_STATUS","busy");

  KonoeFDClient * cl;

  cout << "\nDataHub: start..." << endl;

  buffer = new void* [ buffer_size ];	
  
  for ( KInt i = 0; i < buffer_size; i++ ){ 
    buffer[i] = 0;  
  }

  spill_flag = KFalse;  

  while ( mode != NONE ) { 
    
    client = in_server -> wait(0);
    if( client ){
      get_data();
    }

    while( spill_flag ){
	get_data();
    }
    
    client = out_server -> wait(0);
    if( client ){
      request_analyzer();
    }
    
  }
  
}

void T473DataHubForPS::get_data( ) {

  KonoeDataRecord* rec;
  rec  = (KonoeDataRecord*)in_server -> get();

  spill_flag = KTrue;

  if ( ! rec ) {	// null check
    in_server -> closeConnection( ); 
    spill_flag = KFalse;
    return ;
  }

  if ( WRITE_TO_FILE )
    out_file -> put( rec );
  
  if( rec->getType() == SPILL_END ){
    spill_flag = KFalse;
    cout << "SPILL_END" << endl;
  }

  if( rec->getType() == RUN_END ){
    spill_flag = KFalse;
    cout << "RUN_END" << endl;
    this->flush( );
  }

  write_address = ring_counter( write_address + 1 );
  
  // These two casts are very important.
  delete  ( KonoeDataRecord * ) buffer[ write_address ];
  buffer[ write_address ] =  rec;
  
  for (KInt i = 0; i < KonoeObjectServerSocketBuffered::MAX_CLIENTS; i++ ){
    data_length[ i ]++;
    if ( data_length[ i ] > buffer_size )
      data_length[ i ] = buffer_size;
  }

}

KInt  T473DataHubForPS::ring_counter( KInt value ){

  while ( value < 0 ) { 
    value += buffer_size; 
  }

  while ( value >= buffer_size){ 
    value -= buffer_size; 
  }

  return  value;

}

KInt  T473DataHubForPS::find_index( KonoeObjectSocketBuffered * client ) {

  KonoeObjectSocketBuffered ** clients = out_server -> getClients();

  for ( KInt i = 0; i < KonoeObjectServerSocketBuffered::MAX_CLIENTS; i++ )
    if ( client == clients[i] )
      return i;
  
}
	
KInt  T473DataHubForPS::request_analyzer() {

  const KInt MAX_STRING_LENGTH = 256;		
  KChar  socket_mess[ MAX_STRING_LENGTH ];

  void * item;	

  KInt index = find_index( client );

  client -> getSocket() -> receive( socket_mess, MAX_STRING_LENGTH );
  
  if ( socket_mess[0] ==  '\0' ) {	// null check
    out_server -> close( client );
    return -1;
  }
  
  if ( data_length[index] <= 0 ) {
    return -1;		// no data. ///////////?????????????????
  }
  
  if ( ! strcmp(socket_mess, DATAHUB_REQ_HEAD) ){
    item = buffer[ ring_counter( write_address - buffer_size + 1) ];
    put_data( item );
  }
  else if ( ! strcmp(socket_mess, DATAHUB_REQ_CURRENT) ) {
    item = buffer[ ring_counter( write_address - data_length[ index ] + 1 ) ];
    
    data_length[ index ]--;
    
    return put_data( item );
  }
  
  else if ( ! strcmp(socket_mess, DATAHUB_REQ_TAIL) ){
    item = buffer[ write_address ];
    return put_data( item ); 
  }
    
  else if ( ! strcmp(socket_mess, DATAHUB_CMD_FILE_CLOSE) ) {
      
    cout << "Attention !:\tData file is closed." << endl;
    
    WRITE_TO_FILE = KFalse;
    delete	out_file;
    out_file = 0;
    return 0;
  }
  
  else if ( ! strcmp(socket_mess, DATAHUB_CMD_NEW_FILE) ) {
    cmd_new_file();
    return 0;
  }
  
  else if ( ! strcmp(socket_mess, DATAHUB_CMD_CHANGE_BUFFER_SIZE) ) {
    // not implemented yet
    return -1;
  }
  
  else if ( ! strcmp(socket_mess, DATAHUB_CMD_IGNORE_OUTPUT_PORT) ){
    IGNORE_OUTPUT_PORT = ! IGNORE_OUTPUT_PORT;
    cout << "From now, this program ignores the output port\n";  
    return 0;
  }

  else if ( ! strcmp(socket_mess,"shutdown") ){
    mode = NONE;
    cout << "From now, this program ignores the output port\n";  
    return 0;
  }
}

KInt  T473DataHubForPS::put_data( void * data ) {

  if ( data ){ 
    client -> put( ( KonoeDataRecord * ) data); 
    client -> flush();
  }
  return 0;

}

void  T473DataHubForPS::cmd_new_file() { 

  WRITE_TO_FILE = KTrue;
  delete	out_file;
  KInt st;

  st = chmod(filename_bak,(mode_t)0444);
  if( st == 0 ){
	cout << "\tchmod " << filename_bak << " read only " << endl;
  }
  
  KInt	MAX_ENTRY_NAME_LENGTH = 256;
  KChar	entry_name[MAX_ENTRY_NAME_LENGTH]; 


  KonoeNameSpaceValue * value;
  strcpy( entry_name, name );
  value = name_table->getValue(strcat(entry_name,DATAHUB_STORAGE_FILE_NAME ));

  const KInt FILE_NAME_LENGTH = 256;	
  KChar	file_name[FILE_NAME_LENGTH];

  strcpy(file_name,"");

  if( getenv("T473_DATA_DIR") ){
	strcpy(file_name,getenv("T473_DATA_DIR"));
	strcat(file_name,"/");
  }

  if ( value ){
    strcat( file_name,  ( KChar * ) * value ); 
    cout << "DATAHUB_STORAGE_FILE_NAME: " << file_name << endl;

  } else {
    strcat( file_name, DEFAULT_STORAGE_FILE_NAME );
    cout << "DATAHUB_STORAGE_FILE_NAME is not found." << endl 
         << "Use the default value: " 
         << file_name << endl;
  }

  out_file = new KonoeOutputObjFileBuffered(file_name);
  strcpy(filename_bak,file_name);
	
}

void T473DataHubForPS::lastAction( ){

  setter->set("DATAHUB_STATUS","none");
  setter->set("DATAHUB_PID",-1);

  exit(0);

}


