//=================================================================
/*

        T473DataHubPS.hh

        Version 0.6.0   04-Oct-2000     Kentaro Mizouchi
        Version 0.6.0   05-Nov-2000     Y. Shimizu

*/
//=================================================================
/*
   Dual port version.
   Input port is independent on output port.
*/

#ifndef  __T473DATAHUBCHILD_HH
#define  __T473DATAHUBCHILD_HH

#include <string.h>
#include <stdlib.h>
#include <KonoePrimitives.hh>
#include <system/KonoeObjectServerSocketBuffered.hh>
#include <system/KonoeOutputObjFileBuffered.hh>
#include <system/KonoeNameSpaceTable.hh>
#include <analysis/KonoeDataRecord.hh>

#include "../Namespace/T473NameSpaceSetter.hh"
#include "../RunControl/T473DyingMessage.hh"

class T473DataHubForPS : T473DyingMessage {

public:
  enum { SPILL_END = 4000 };
  enum { RUN_END = 2000 };
  enum { NONE , READY };
  // name space entry string
  static const KChar* DATAHUB_INPUT_PORT;
  static const KChar* DATAHUB_OUTPUT_PORT;
  static const KChar* DATAHUB_STORAGE_FILE_NAME;
  static const KChar* DATAHUB_BUFFER_SIZE;

  // command strings
  static const KChar* DATAHUB_CMD_FILE_CLOSE;
  static const KChar* DATAHUB_CMD_NEW_FILE;
  static const KChar* DATAHUB_CMD_CHANGE_BUFFER_SIZE;
  static const KChar* DATAHUB_CMD_IGNORE_OUTPUT_PORT;
  
  // request strings
  static const KChar* DATAHUB_REQ_HEAD;
  static const KChar* DATAHUB_REQ_CURRENT;
  static const KChar* DATAHUB_REQ_TAIL;
  
protected:		
  KChar *			name;	// ID
  KonoeObjectServerSocketBuffered  * in_server,* out_server; 	
  KonoeObjectSocketBuffered   *	client;
  KonoeOutputObjFileBuffered  *	out_file;	
  KonoeNameSpaceTable *	name_table;	

  void **	buffer;
  KInt	buffer_size;	
  
  KInt	write_address;
  KInt	data_length[ KonoeObjectServerSocketBuffered::MAX_CLIENTS ];
  
  KBool	WRITE_TO_FILE,  IGNORE_OUTPUT_PORT;

  T473NameSpaceSetter * setter;

  // default value 
  enum	{ INPUT_PORT = 81000, OUTPUT_PORT = 81001, BUFFER_SIZE = 1000 };

  static const KChar * DEFAULT_STORAGE_FILE_NAME;

public:
  T473DataHubForPS( KChar*  tag_name);
  ~T473DataHubForPS();
	
  KInt	go();

protected:
  KInt	ring_counter( KInt counter );
  KInt  find_index( KonoeObjectSocketBuffered * client );
  KInt  spill_flag ;
  void	get_data(void );
  KInt	put_data( void * obj );
  KInt	request_analyzer();
  void	cmd_new_file();
  void	flush();
  
  void lastAction();

  KInt mode ;
  KChar filename_bak[64];
};


#endif
