//=====================================================================
/*

    T473Tcl2DEventDisplay.cc

    Version 0.0.1   28-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473Tcl2DEventDisplay.hh>

#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTcl2DrawPanel.hh>
#include <ui/KonoeTcl2DrawCanvas.hh>
#include <ui/KonoeTcl2DrawCanvasController.hh>
#include <ui/KonoeTclDimension.hh>

#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeFillData.hh>
#include <analysis/KonoeLinearDataScale.hh>

T473Tcl2DEventDisplay::T473Tcl2DEventDisplay(KonoeTclWindow* pwin)
{
  subWindow = new KonoeTclSubWindow(pwin, "subWindow2D", "2D EventDisplay");
  subWindow->setEventProcessor(this);
  addWindowAction(subWindow);

  displayData = KHITENERGY;

  initPanel();

  controller = 0;

  drawCanvas->setEventProcessor(this);
  addMouseAction(drawCanvas);
}

T473Tcl2DEventDisplay::~T473Tcl2DEventDisplay()
{
  delete drawPanel;
  delete drawCanvas;
  subWindow->destroy();
  delete subWindow;
}

void T473Tcl2DEventDisplay::initPanel()
{
  Konoe2DHistogram*
    dummyHist = new Konoe2DHistogram( new KonoeLinearDataScale( 0, 5, 5),
                                      new KonoeLinearDataScale( 0, 5, 5) );
  dummyHist->clear();

  drawPanel = new KonoeTcl2DrawPanel( subWindow, "eventdisplay2DPanel" );
  drawCanvas = new KonoeTcl2DrawCanvas( drawPanel, "eventdisplay2DCanvas",
               dummyHist,
               KonoeTclDimension(300, 300), "Event Display 2D",
               new KonoeLinearDataScale(0.0, 4096.0, 4096) );

  drawPanel->setCanvas( drawCanvas, "" );
  drawPanel->setFrame();
  drawCanvas->setSize(500, 500);
  drawCanvas->draw();
}

void T473Tcl2DEventDisplay::setSize( KInt width, KInt height )
{
  drawCanvas->setSize( width, height );
}

void T473Tcl2DEventDisplay::setDataSource(
                     KInt pADCvalue[], KFloat pADCenergy[], KFloat pHitTime[] )
{
  KInt i;
  for(i=0; i<25; i++){
    adcValue[i] = pADCvalue[i];
    adcEnergy[i] = pADCenergy[i];
    hitTime[i] = pHitTime[i];
  }

  switch(displayData){
    case KHITENERGY:
    case KHITENERGY_HITTIME:
      drawADCenergy();
      break;
    case KADCVALUE:
    case KADCVALUE_HITTIME:
      drawADCvalue();
      break;
    default:
      ;
  }
}

void T473Tcl2DEventDisplay::setVerticalScale(KonoeDataScale* pScale)
{
  drawCanvas->setVerticalScale(pScale);
}

void T473Tcl2DEventDisplay::setDisplayData(KInt mode)
{
  displayData = mode;
}

void T473Tcl2DEventDisplay::drawADCenergy()
{
  Konoe2DHistogram* histogram = new Konoe2DHistogram(
                         new KonoeLinearDataScale(0.0, 5.0, 5),
                         new KonoeLinearDataScale(0.0, 5.0, 5) );
  histogram->clear();

  KInt i;
  KInt j;
  KInt v;
  KInt n = 0;

  for(i=0; i<5; i++){
    for(j=0; j<5; j++){
      for(v=0; v<adcEnergy[n]; v++){
        histogram->fill( new KonoeFillData(i, j) );
      }
      n++;
    }
  }
//  for(j=0; j<5; j++){
//    for(i=5; i>0; i--){
//      for(v=0; v<adcEnergy[n]; v++){
//        histogram->fill( new KonoeFillData(i-1, j) );
//      }
//      n++;
//    }
//  }

  drawCanvas->setDataSource(histogram);
  if(displayData == KHITENERGY_HITTIME){
    KFloat max = 10.;
    KFloat min = 4096.;
    for(i=0; i<25; i++){
      if(max < hitTime[i]){
        if(hitTime[i] <4096) max = hitTime[i];
        else                 hitTime[i] = 8192.;
      }

      if(hitTime[i] < min){
        min = hitTime[i];
      }
    }

    KFloat binWidth = 255 / ( max - min );

    KInt RR;
    KInt GG;
    KInt BB = 255;
    for(i=0;i<25;i++){
//      drawCanvas->setVisAtt(i,
//                     255-255*fabs((hitTime[i]/max)),
//                     255-255*fabs((hitTime[i]/max)),
//                     255);
      RR = GG = (KInt)(binWidth*(hitTime[i]-min));

      if( RR < 0 ) RR = GG = 0;
      if( 255 < RR ) RR = GG = 255;

      drawCanvas->setVisAtt(i, RR, GG, BB);
    }
  }
  drawCanvas->draw();
}

void T473Tcl2DEventDisplay::drawADCvalue()
{
  Konoe2DHistogram* histogram = new Konoe2DHistogram(
                         new KonoeLinearDataScale(0.0, 5.0, 5),
                         new KonoeLinearDataScale(0.0, 5.0, 5) );
  histogram->clear();

  KInt i;
  KInt j;
  KInt v;
  KInt n = 0;

  for(i=0; i<5; i++){
    for(j=0; j<5; j++){
      for(v=0; v<adcValue[n]; v++){
        histogram->fill( new KonoeFillData(i, j) );
      }
      n++;
    }
  }

//  for(j=0; j<5; j++){
//    for(i=5; i>0; i--){
//      for(v=0; v<adcValue[n]; v++){
//        histogram->fill( new KonoeFillData(i-1, j) );
//      }
//      n++;
//    }
//  }

  drawCanvas->setDataSource(histogram);
  if(displayData == KADCVALUE_HITTIME){
    KFloat max = 10.;
    KFloat min = 4096.;
    for(i=0; i<25; i++){
      if(max < hitTime[i]){
        if(hitTime[i] <4096) max = hitTime[i];
        else                 hitTime[i] = 8192.;
      }

      if(hitTime[i] < min){
        min = hitTime[i];
      }
    }

    KFloat binWidth = 255 / ( max - min );

    KInt RR;
    KInt GG;
    KInt BB = 255;
    for(i=0;i<25;i++){
//      drawCanvas->setVisAtt(i,
//                     255-255*fabs((hitTime[i]/max)),
//                     255-255*fabs((hitTime[i]/max)),
//                     255);
      RR = GG = (KInt)(binWidth*(hitTime[i]-min));

      if( RR < 0 ) RR = GG = 0;
      if( 255 < RR ) RR = GG = 255;

      drawCanvas->setVisAtt(i, RR, GG, BB);
    }
  }

  drawCanvas->draw();
}

void T473Tcl2DEventDisplay::draw()
{
  switch(displayData){
    case KHITENERGY:
    case KHITENERGY_HITTIME:
      drawADCenergy();
      break;

    case KADCVALUE:
    case KADCVALUE_HITTIME:
      drawADCvalue();
      break;

    default:
      ;
  } 
}

void T473Tcl2DEventDisplay::mouseAction(KonoeTclCanvas* bt, KInt type,
                                     KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == RButton){
  if( 0<=x && x<bt->getWidth() && 0<=y && y<=bt->getHeight()){
    if(!controller)
      controller = new KonoeTcl2DrawCanvasController(drawCanvas);
    else{
      delete controller;
      controller = new KonoeTcl2DrawCanvasController(drawCanvas);
    }
  }
  }
}

void T473Tcl2DEventDisplay::windowAction(KonoeTclWindow* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
    delete controller;
    subWindow->destroy();
  }
}
