//=====================================================================
/*

    T473TclEventDisplay.cc

    Version 0.0.1   09-Oct-2000 T. Hosoi

*/
//====================================================================

#include <T473TclEventDisplay.hh>
#include <T473TclEventDisplayData.hh>
#include <T473TclEventDisplayInfo.hh>
#include <T473Tcl2DEventDisplay.hh>
#include <T473TclSetup.hh>
#include <Reconstruct/T473ReconstructData.hh>
#include <Reconstruct/T473CsIdata.hh>
#include <Reconstruct/T473GeneralData.hh>
#include <Namespace/T473NameSpaceExtractor.hh>
#include <Logger/T473LogSender.hh>

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclEventViewCanvas.hh>
#include <ui/KonoeTclEventViewPanel.hh>
#include <ui/KonoeTclPhysPlacement.hh>
#include <ui/KonoeTclDrawableBox.hh>
#include <ui/KonoeTclDrawableHorizonLine.hh>
#include <ui/KonoeTclDrawableVerticalLine.hh>
#include <ui/KonoeTclRotationMatrix.hh>
#include <ui/KonoeTcl3DVector.hh>
#include <ui/KonoeTclEventHitPoint.hh>

#include <KonoeList.hh>
#include <system/KonoeClientSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <analysis/Konoe3Vector.hh>
#include <analysis/KonoeDetectorHit.hh>

T473TclEventDisplay::T473TclEventDisplay(KonoeTclWindow* pwin,
                                         KonoeDetectorList* dList)
{
  kwin = pwin;

  detectorList = dList;
  displayData = KHITENERGY;

  eventNumber = 0;
  runNumber = 0;
  hitPointX = 0.;
  hitPointY = 0.;
  hitPointZ = 0.;

  str = new KChar*[25];
  KInt i;
  for(i=0; i<25; i++)
    str[i] = new KChar[128];

  initWidget();

  setup = new T473TclSetup(eventCanvas, 210, 10, 400, 50);

  setViewLotation(45.*DEG);
  setViewAngle(45.*DEG);

  setCenterEye(450, 450);
  eventCanvas->fillBoth();

  this->draw();
  eventCanvas->update();

//dump(cout);

  T473LogSender logsend;
  logsend.send("T473eventdisplay is executed.", T473LogSender::SUCCESS);
}

T473TclEventDisplay::~T473TclEventDisplay()
{
  KInt i;
  for(i=0; i<25; i++)
    delete str[i];
  delete [] str;

  delete info;
  delete drawPanel;
  delete eventCanvas;
  delete leftFrame;
  delete rightFrame;
  subWindow->destroy();
//  delete subWindow;

  kwin->destroy();
//  delete kwin;
}

void T473TclEventDisplay::initWidget()
{
  KChar tmpNam[256];

  subWindow = new KonoeTclSubWindow(kwin, tmpnam(tmpNam), "EventDisplay");
  leftFrame = new KonoeTclFrame(subWindow, "leftFrame");
  rightFrame = new KonoeTclFrame(subWindow, "rightFrame");

  leftFrame->setLayoutType("left");
  leftFrame->setBackGround("white");
  leftFrame->fillBoth();
  rightFrame->setLayoutType("right");

  drawPanel = new KonoeTclEventViewPanel(leftFrame, tmpnam(tmpNam));
  drawPanel->setLayoutType("left");

  eventCanvas = new KonoeTclEventViewCanvas(
                        drawPanel, "eeeventcanvas",
                        700,700,Konoe3Vector(400,400,400),detectorList,"ED");

  drawPanel->setCanvas( eventCanvas, "CsI Cal");

  info = new T473TclEventDisplayInfo( rightFrame );

  leftFrame->setFrame();
  rightFrame->setFrame();


  drawPanel->setBackGround("white");
  eventCanvas->setBackGround("white");

  hitPoint = new KonoeTclEventHitPoint(eventCanvas);

  eventCanvas->setEventProcessor(this);
  addWindowAction(eventCanvas);
}

void T473TclEventDisplay::dump( ostream& os )
{
  detectorList->reset();
  os << "DetectorList has " << detectorList->getCount()
       << " Detectors." << endl;

  for(KInt i=0; i < detectorList->getCount(); i++){
    KonoeDetector* d = detectorList->next();
    d->reset();
    os << "Detector has " << d->getCount() << " Elements." << endl;
    int i=0;
    for(i=0;i<d->getCount();i++)
      d->next()->dump(cout);
    os << endl;
  }
}

void T473TclEventDisplay::setViewLotation(KFloat value){
  eventCanvas->getSpaceManager()->setViewLotation(value);
}

void T473TclEventDisplay::setViewAngle(KFloat value){
  eventCanvas->getSpaceManager()->setViewAngle(value);
}

void T473TclEventDisplay::saveToPostScript(KChar* fName)
{
  eventCanvas->toPostScript(fName);
}

void T473TclEventDisplay::saveToXWD(KChar* fName)
{
  eventCanvas->toXWD(fName);
}

void T473TclEventDisplay::setMagnification(KFloat value)
{
  eventCanvas->setMagnification(value);
}

void T473TclEventDisplay::setCenterEye(KInt eyeX, KInt eyeY)
{
  eventCanvas->setCenterEye(eyeX, eyeY);
}

void T473TclEventDisplay::showGeometry()
{
  eventCanvas->showGeometry();
}

void T473TclEventDisplay::hideGeometry()
{
  eventCanvas->hideGeometry();
}

void T473TclEventDisplay::setForceWireFrame()
{
  eventCanvas->setForceWireFrame();
}

void T473TclEventDisplay::unsetForceWireFrame()
{
  eventCanvas->unsetForceWireFrame();
}

void T473TclEventDisplay::setScale()
{
  eventCanvas->setScale();
}

void T473TclEventDisplay::unsetScale()
{
  eventCanvas->unsetScale();
}

void T473TclEventDisplay::rotateY(KFloat thetaY)
{
  //eventCanvas->rotateY( thetaY );
}

void T473TclEventDisplay::setSlowDraw()
{
  eventCanvas->setSlowDraw();
}

void T473TclEventDisplay::unsetSlowDraw()
{
  eventCanvas->unsetSlowDraw();
}

void T473TclEventDisplay::setDisplayData(KInt n)
{
  displayData = n;
}

void T473TclEventDisplay::drawCoincidence()
{
  if( coin[15] == '1' ) setup->hitS1L();
  if( coin[14] == '1' ) setup->hitS1R();
  if( coin[13] == '1' ) setup->hitS2();
  if( coin[12] == '1' ) setup->hitS3();
  if( coin[11] == '1' ) setup->hitS4();
  if( coin[10] == '1' ) setup->hitS5();
  if( coin[9] == '1' ) setup->hitS6();
  if( coin[8] == '1' ) setup->hitS7();
  if( coin[7] == '1' ) setup->hitGC1();
  if( coin[6] == '1' ) setup->hitGC2();
  if( coin[4] == '1' || coin[3] == '1' || coin[2] == '1' || coin[1] == '1')
    setup->hitXe();
  if( coin[0] == '1' ) setup->hitS1P();
}

void T473TclEventDisplay::draw()
{
  eventCanvas->draw();
  this->drawEventInfo();
  setup->draw();
  drawCoincidence();
  KChar theta_Y[32];
  T473NameSpaceExtractor nsExtractor;
  strcpy(theta_Y, nsExtractor.getKCharValue("THETA_Y"));

  if( atoi(theta_Y) )
    ;
  else
    hitPoint->draw(255,0,0);

  this->update();
}

void T473TclEventDisplay::drawEventInfo()
{
  eventCanvas->setColor("red");
  eventCanvas->drawEventNumber(eventNumber);
  eventCanvas->drawRunNumber(runNumber);

  T473NameSpaceExtractor nsExtractor;
  KChar theta_Y[256];
  strcpy(theta_Y, nsExtractor.getKCharValue("THETA_Y"));
  
  if( atoi(theta_Y) )
    ;
  else
    eventCanvas->drawHitPoint(hitPointX, hitPointY, hitPointZ);


  KInt i;
  switch(displayData){
    case KHITENERGY:
    case KHITENERGY_HITTIME:
      for(i=0; i<25; i++)
        sprintf(str[i], "%.2lf", hitEnergy[i]);
      break;

    case KADCVALUE:
    case KADCVALUE_HITTIME:
      for(i=0; i<25; i++)
        sprintf(str[i], "%d", adcValue[i]);
      break;

    default:
      for(i=0; i<25; i++)
        sprintf(str[i], "%d", i);
  }

  eventCanvas->setEventHitString(str, "blue", "times", 32);
}


void T473TclEventDisplay::setStatus( T473ReconstructData* rec_data )
{
  T473GeneralData* gd;
  T473CsIdata* csidata;
  Konoe3Vector* vec;
  Konoe3Vector* theta;
  KFloat x;
  KFloat y;
  KFloat z;
  KChar cap[64];

  if(rec_data){
    gd = rec_data->getGeneralData();
    csidata = rec_data->getCsIdata();
    vec = csidata->getHitPoint();
    theta = csidata->getTheta();

//  Run Number
    runNumber = gd->getRunNumber();
    if(runNumber == -1)
      strcpy(cap, "Invalid Trigger Type");
    else
      sprintf(cap, "%d", runNumber);
    info->runNumVLabel->setCaption(cap);

//  Event Numver
    eventNumber = gd->getEventNumber();
    sprintf(cap, "%d", eventNumber);
    info->eventNumVLabel->setCaption(cap);

//  Hit Point
      hitPointX = vec->x;
      hitPointY = vec->y;
      hitPointZ = vec->z;
      hitPoint->setPoint(KonoeTcl3DVector(x,y,z));
      //cout << vec->x << " , " << vec->y << " , " << vec->z  << endl;
      //cout << theta->x << " , " << theta->y << " , " << theta->z << endl;

      sprintf(cap, "%.02f , %.02f , %.02f", vec->x, vec->y, vec->z);
        info->hpValueLabel->setCaption(cap);
      hitPoint->setPoint( KonoeTcl3DVector(vec->x, vec->y, vec->z) );

//  Coincidence Register
    sprintf(cap, "%s", gd->getCoinReg1());
      info->coinRegVLabel1->setCaption(cap);
    sprintf(cap, "%s", gd->getCoinReg2());
      info->coinRegVLabel2->setCaption(cap);
    sprintf(cap, "%s", gd->getCoinReg0());
      info->coinRegVLabel0->setCaption(cap);

    strcpy(coin, cap);

//  Energy , Hit Time , ADC Value and TDC Value
    KonoeList<KonoeDetectorHit>* lst = csidata->getHitList();

    lst->reset();

    KInt i;
    for(i=0; i<lst->getCount(); i++){
      KonoeDetectorHit* dh = lst->next();

      hitEnergy[i] = dh->getHitEnergy();
      sprintf(cap, "%.2f", hitEnergy[i]);
        info->lbla[i]->setCaption(cap);

      hitTime[i] = dh->getHitTime();
      sprintf(cap, "%.2f", dh->getHitTime());
        info->lblb[i]->setCaption(cap);

      adcValue[i] = dh->getADCValue();
      sprintf(cap, "%d", adcValue[i]);
        info->lbld[i]->setCaption(cap);

      tdcValue[i] = dh->getTDCValue();
      sprintf(cap, "%d", tdcValue[i]);
        info->lble[i]->setCaption(cap);
    }
  }
  else{
    cout << "ReconstructData is empty" << endl;
  }
}

void T473TclEventDisplay::setHitColor()
{
  KInt i;
  KFloat max = 10.;

  switch(displayData){
    case KHITENERGY:
    case KHITENERGY_HITTIME:
     for(i=0; i<25; i++){
       if(max < hitEnergy[i]) max = hitEnergy[i];
       if(hitEnergy[i] < 0) hitEnergy[i] = 0.;
     }

      for(i=0; i<25; i++){
      eventCanvas->setEventHitVisAtt(i, 255,
                        255-255*abs((KInt)hitEnergy[i])/max, 
                        255-255*abs((KInt)hitEnergy[i])/max);
      }
      break;

    case KADCVALUE:
    case KADCVALUE_HITTIME:
     for(i=0; i<25; i++){
       if(max < (KFloat)adcValue[i]) max = (KFloat)adcValue[i];
       if(adcValue[i] < 0.) adcValue[i] = 0;
     }

      for(i=0; i<25; i++){
        eventCanvas->setEventHitVisAtt(i, 255,
                        255-255.*fabs((KFloat)adcValue[i])/(KFloat)max,
                        255-255.*fabs((KFloat)adcValue[i])/(KFloat)max);
      }
      break;

    default: ;
  }
      
}

void T473TclEventDisplay::setThetaY()
{
  //T473NameSpaceExtractor nsExtractor;
  //KChar theta_Y[256];
  //strcpy(theta_Y, nsExtractor.getKCharValue("THETA_Y"));

  //rotateY(atof(theta_Y));
}

void T473TclEventDisplay::windowAction(KonoeTclCanvas* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
    this->~T473TclEventDisplay();
  }
}

void T473TclEventDisplay::update()
{
  eventCanvas->update();
}
