//=====================================================================
/*

    T473TclEventDisplayControl.cc

    Version 0.0.1   04-Nov-2000 T. Hosoi

*/
//====================================================================

#include <iostream.h>
#include <stdlib.h>
#include <T473TclEventDisplayControl.hh>
#include <T473TclEventDisplayRequest.hh>
#include <T473TclEventDisplayControl.hh>
#include <Namespace/T473NameSpaceExtractor.hh>

#include <Reconstruct/T473DetectorMaker.hh>
#include <Reconstruct/T473ReconstructData.hh>
#include <analysis/KonoeDetector.hh>
#include <analysis/KonoeDetectorList.hh>
#include <system/KonoeNameSpaceTable.hh>

T473TclEventDisplayControl::T473TclEventDisplayControl(KonoeTclWindow* pwin)
{
  kwin = pwin;
  thetaY = 0.;
  reqtype = RANDOM_TRIGGER;
  state = KReqType;
  ed2d = 0;

  if(!getenv("NONAMESPACE"))
  data = new T473TclEventDisplayData;

  initWidget();
  createEventDisplay();
  //create2DEventDisplay();

  isDetail = KTrue;

  setButtonAction();
}

void T473TclEventDisplayControl::createEventDisplay()
{
  KonoeNameSpaceTable* table = new KonoeNameSpaceTable();
  T473DetectorMaker* dm = new T473DetectorMaker(table);
  d = dm->getDetector();
  dl = new KonoeDetectorList(1);

  dl->add(d);

  ed = new T473TclEventDisplay(kwin, dl);

  req->setEventDisplay(ed);
//  delete table;
//  delete dm;
}

void T473TclEventDisplayControl::create2DEventDisplay()
{
  delete ed2d;
  ed2d = new T473Tcl2DEventDisplay(kwin);
}

void T473TclEventDisplayControl::initWidget()
{
//==================== Global Frame  ====================
  globalFrame = new KonoeTclFrame(kwin, "globalFrame");

  globalFrame->setEventProcessor(this);
  addTimer(globalFrame, REQUEST_INTERVAL);

//==================== Frame 1  ====================
  frame1 = new KonoeTclFrame(globalFrame, "f1", 0, 0);
  viewLotationLabel =
      new KonoeTclLabel(frame1, "viewLotationLabel", "ViewLotation", 0, 0);
  viewLotationSlider =
        new KonoeTclSlider(frame1, "viewLotationSlider", "", 1, 0);
  viewAngleLabel =
      new KonoeTclLabel(frame1, "viewAngleLabel", "ViewAngle", 0, 1);
  viewAngleSlider =
        new KonoeTclSlider(frame1, "viewAngleSlider", "", 1, 1);

  viewLotationSlider->setMinMax(0, 180);
  viewAngleSlider->setMinMax(0, 90);
  viewLotationSlider->setBigIncrement(1);
  viewAngleSlider->setBigIncrement(1);
  viewLotationSlider->setShowValue(KTrue);
  viewAngleSlider->setShowValue(KTrue);
  viewAngleSlider->setValue(15);
  viewLotationSlider->setValue(80);


//==================== Frame 2  ====================
  frame2 = new KonoeTclFrame(globalFrame, "f2", 0, 1);
  viewingModeLabel =
      new KonoeTclLabel(frame2, "viewingModeLabel", "Viewing Mode", 0, 0);
  viewingModeLabel->setGridRowspan(2);
  viewingModeLabel->setGridSticky("news");
  wireFrameRadio =
      new KonoeTclRadioButton(frame2, "wireFrameRadio", "WireFrame", 1, 0);
  surfaceRadio=
      new KonoeTclRadioButton(frame2, "surfaceRadio", "Surface", 2, 0);

  wireFrameRadio->setGroup("view");
  surfaceRadio->setGroup("view");
  surfaceRadio->setSelect();


//==================== Frame 3  ====================
  frame3 = new KonoeTclFrame(globalFrame, "f3", 0, 2);
  targetPointLabel =
      new KonoeTclLabel(frame3, "targetPointLabel", "Targetting Point", 0, 0);
  targetPointLabel->setGridRowspan(3);
  targetPointLabel->setGridSticky("news");

  xCoordLabel =
      new KonoeTclLabel(frame3, "xCoordLabel", "X coord", 1, 0);
  xCoordEntry =
      new KonoeTclEntry(frame3, "xCoordEntry", 2, 0);
  yCoordLabel =
      new KonoeTclLabel(frame3, "yCoordLabel", "Y coord", 1, 1);
  yCoordEntry =
      new KonoeTclEntry(frame3, "yCoordEntry", 2, 1);

  xCoordEntry->setBackGround("white");
  yCoordEntry->setBackGround("white");
  xCoordEntry->setText("400");
  yCoordEntry->setText("400");

  xCoordEntry->setEventProcessor(this);
  addKeyAction(xCoordEntry);
  yCoordEntry->setEventProcessor(this);
  addKeyAction(yCoordEntry);


//==================== Frame 4  ====================
  frame4 = new KonoeTclFrame(globalFrame, "f4", 0, 3);
  magnificationLabel =
      new KonoeTclLabel(frame4, "magnificationLabel", "Magnification", 0, 0);
  magnificationEntry =
      new KonoeTclEntry(frame4, "magnificationEntry", 1, 0);

  magnificationEntry->setBackGround("white");
  magnificationEntry->setText("1.0");

  magnificationEntry->setEventProcessor(this);
  addKeyAction(magnificationEntry);


//==================== Frame 5  ====================
  frame5 = new KonoeTclFrame(globalFrame, "f5", 0, 4);
  displayAxesLabel =
      new KonoeTclLabel(frame5, "dispalyAxesLabel", "Display Axes", 0, 0);
  displayAxesYesRadio =
      new KonoeTclRadioButton(frame5, "displayAxesYesRadio", "Yes", 1, 0);
  displayAxesNoRadio =
      new KonoeTclRadioButton(frame5, "displayAxesNoLabel", "No", 2, 0);

  displayAxesYesRadio->setGroup("axes");
  displayAxesNoRadio->setGroup("axes");

  displayAxesNoRadio->setSelect();


//==================== Frame 6  ====================
  frame6 = new KonoeTclFrame(globalFrame, "f6", 0, 5);
  drawModeLabel = new KonoeTclLabel
                      (frame6, "drawModeLabel", "Drawing Mode", 0, 0);
  fastDrawRadio = new KonoeTclRadioButton
                      (frame6, "fastDrawRadio", "Fast", 1, 0);
  slowDrawRadio = new KonoeTclRadioButton
                      (frame6, "slowDrawRadio", "Slow", 2, 0);

  drawModeLabel->setGridRowspan(2);

  fastDrawRadio->setGroup("drawingMode");
  slowDrawRadio->setGroup("drawingMode");

  fastDrawRadio->setSelect();


//==================== Frame 7  ====================
  frame7 = new KonoeTclFrame(globalFrame, "f7", 0, 6);
  showGeometryRadio =
      new KonoeTclRadioButton(frame7,
           "showGeometryRadio", "Show Geometry", 1, 0);
  hideGeometryRadio =
      new KonoeTclRadioButton(frame7,
           "hideGeometryRadio", "Hide Geometry", 2, 0);

  showGeometryRadio->setGroup("enableGeometry");
  hideGeometryRadio->setGroup("enableGeometry");

  showGeometryRadio->setSelect();


//==================== Frame 8  ====================
  frame8 = new KonoeTclFrame(globalFrame, "f8", 0, 7);
  postscriptButton = new KonoeTclButton
                      (frame8, "postscriptButton", "Save to PS", 0, 0);
  postscriptEntry = new KonoeTclEntry
                      (frame8, "postscriptEntry", 1, 0);

  postscriptEntry->setBackGround("white");

  addMouseAction(postscriptButton);
  postscriptButton->setEventProcessor(this);

  postscriptEntry->setEventProcessor(this);
  addKeyAction(postscriptEntry);


//==================== Frame 9  ====================
  frame9 = new KonoeTclFrame(globalFrame, "f9", 0, 8);
  xwdButton = new KonoeTclButton
                   (frame9, "xwdButton", "Save to XWD", 0, 0);
  xwdEntry = new KonoeTclEntry
                   (frame9, "xwdEntry", 1, 0);

  xwdEntry->setBackGround("white");

  xwdButton->setEventProcessor(this);
  xwdEntry->setEventProcessor(this);
  addMouseAction(xwdButton);
  addKeyAction(xwdEntry);


//==================== Frame 10  ====================
  frame10 = new KonoeTclFrame(globalFrame, "f10", 0, 9);
  setupButton = new KonoeTclButton(frame10, "setupButton", "Setup", 0, 0);

  setupButton->setFont("lucida", 18, 1, 0, 0, 0);
  addMouseAction(setupButton);
  setupButton->setEventProcessor(this);
  
//==================== Frame 11  ====================
  frame11 = new KonoeTclFrame(kwin, "f11");//, 0, 10);
  frame11->setSize(299, 5);
  frame11->setBorderWidth(5);
  frame11->setRelief(KRaised);
  frame11->setBackGround("blue");
  frame11->setCursor("hand2");

  frame11->setEventProcessor(this);
  addMouseAction(frame11);

//==================== Frame 12  ====================
  frame12 = new KonoeTclFrame(kwin, "f12");//, 0, 11);

  req = new T473TclEventDisplayRequest(frame12, this); 

  frame1->setBorderWidth(1);
  frame1->setRelief(KGroove);
  frame1->setGridSticky("news");
  frame1->setGridIpady(2);

  frame2->setBorderWidth(1);
  frame2->setRelief(KGroove);
  frame2->setGridSticky("news");
  frame2->setGridIpady(2);

  frame3->setBorderWidth(1);
  frame3->setRelief(KGroove);
  frame3->setGridSticky("news");
  frame3->setGridIpady(2);

  frame4->setBorderWidth(1);
  frame4->setRelief(KGroove);
  frame4->setGridSticky("news");
  frame4->setGridIpady(2);

  frame5->setBorderWidth(1);
  frame5->setRelief(KGroove);
  frame5->setGridSticky("news");
  frame5->setGridIpady(2);

  frame6->setBorderWidth(1);
  frame6->setRelief(KGroove);
  frame6->setGridSticky("news");
  frame6->setGridIpady(2);

  frame7->setBorderWidth(1);
  frame7->setRelief(KGroove);
  frame7->setGridSticky("news");
  frame7->setGridIpady(2);

  frame8->setBorderWidth(1);
  frame8->setRelief(KGroove);
  frame8->setGridSticky("news");
  frame8->setGridIpady(2);

  frame9->setBorderWidth(1);
  frame9->setRelief(KGroove);
  frame9->setGridSticky("news");
  frame9->setGridIpady(2);

  frame10->setBorderWidth(1);
  frame10->setRelief(KGroove);
  frame10->setGridSticky("news");
  frame10->setGridIpady(2);

//  frame1->setFrame();
//  frame2->setFrame();
//  frame3->setFrame();
//  frame4->setFrame();
//  frame5->setFrame();
//  frame6->setFrame();
//  frame7->setFrame();
//  frame8->setFrame();
//  frame9->setFrame();
//  frame10->setFrame();

  frame12->setFrame();
  frame12->fillX();
  frame11->setFrame();
  globalFrame->setFrame();

  postscriptEntry->setText("eventdisplay.ps");
  xwdEntry->setText("eventdisplay.xwd");
}

T473TclEventDisplayControl::~T473TclEventDisplayControl()
{
  ;
}

void T473TclEventDisplayControl::setThetaY()
{
  //T473NameSpaceExtractor nsExtractor;
  //KChar theta_Y[256];
  //strcpy(theta_Y, nsExtractor.getKCharValue("THETA_Y"));

  //thetaY = atof(theta_Y);
  //if(0 < thetaY)
  //  viewLotationSlider->setMinMax(0, 180 - thetaY*180./M_PI);
  //if(thetaY < 0)
  //  viewLotationSlider->setMinMax(0 + thetaY*180./M_PI, 180);

  //ed->rotateY(thetaY);
}

void T473TclEventDisplayControl::setRequestType(KInt type)
{
  reqtype = type;
}

void T473TclEventDisplayControl::changeState(KInt st)
{
  state = st;
}

void T473TclEventDisplayControl::setDisplayData(KInt mode)
{
  ed->setDisplayData( mode );
  if(ed2d) ed2d->setDisplayData( mode );
}

void T473TclEventDisplayControl::mouseAction
       (KonoeTclButton* bt, KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
  if( 0<=x && x<bt->getWidth() && 0<=y && y<=bt->getHeight()){
    if(strcmp(bt->getName(), "setupButton") == 0)
      setButtonAction();

    else if(strcmp(bt->getName(), "postscriptButton") == 0)
      postscriptAction();

    else if(strcmp(bt->getName(), "xwdButton") == 0)
      xwdAction();
  }
  }
}

void T473TclEventDisplayControl::setButtonAction()
{
  KChar viewLotation[16];
  KChar viewAngle[16];
  KChar magnification[16];
  KChar eyeX[16];
  KChar eyeY[16];

  sprintf(viewLotation, "%d", viewLotationSlider->getIndex());
  sprintf(viewAngle, "%d", viewAngleSlider->getIndex());
  strcpy(magnification, magnificationEntry->getText());
  strcpy(eyeX, xCoordEntry->getText());
  strcpy(eyeY, yCoordEntry->getText());

  ed->setViewLotation(atof(viewLotation)*DEG);
  ed->setViewAngle(atof(viewAngle)*DEG);

  if(strcmp("showGeometryRadio",
             showGeometryRadio->getValue()) == 0) ed->showGeometry();
  else{
    KFloat v;
    if(0 < thetaY) v = 90 - thetaY*180./M_PI;
    else if(thetaY < 0) v = 90 + thetaY*180./M_PI;
    else if(thetaY == 0) v = 90;

    ed->setViewLotation(v*DEG);
    ed->setViewAngle(-90.*DEG);
    ed->hideGeometry();
  }

  KFloat mag = atof(magnification);
  if( mag < 0. ) mag = 1.;

  ed->setMagnification( mag );

  KInt x = atoi(eyeX);
  KInt y = atoi(eyeY);
  if( x < 0 ) x = 0;
  if( y < 0 ) y = 0;
  ed->setCenterEye(x, y);

  if(strcmp("wireFrameRadio", 
             wireFrameRadio->getValue()) == 0) ed->setForceWireFrame();
  else ed->unsetForceWireFrame();

  if(strcmp("displayAxesYesRadio",
             displayAxesYesRadio->getValue()) == 0) ed->setScale();
  else ed->unsetScale();

  ed->draw();
  ed->update();

  if(strcmp("slowDrawRadio",
             slowDrawRadio->getValue()) == 0) ed->setSlowDraw();
  else ed->unsetSlowDraw();
}

void T473TclEventDisplayControl::postscriptAction()
{
  KChar dirName[256];
  KChar fName[256];
  if( getenv("T473_EVENTDISPLAY_DIR") )
    strcpy(dirName, getenv("T473_EVENTDISPLAY_DIR"));

  if(strlen(dirName) > 0)
    sprintf(fName, "%s/%s", dirName, postscriptEntry->getText());
  else
    strcpy(fName, postscriptEntry->getText());
  
  ed->saveToPostScript( fName );
}

void T473TclEventDisplayControl::xwdAction()
{
  KChar dirName[256];
  KChar fName[256];
  if( getenv("T473_EVENTDISPLAY_DIR") )
    strcpy(dirName, getenv("T473_EVENTDISPLAY_DIR"));

  if(strlen(dirName) > 0)
    sprintf(fName, "%s/%s", dirName, xwdEntry->getText());
  else
    strcpy(fName, xwdEntry->getText());

  
  kwin->hide(kwin);
  kwin->after(1000);
  ed->saveToXWD( fName );
  kwin->show(kwin);
}

void T473TclEventDisplayControl::mouseAction
       (KonoeTclWindow* fr, KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
  if( 0<=x && x<fr->getWidth() && 0<=y && y<=fr->getHeight()){
    if(strcmp(fr->getName(), "f11") == 0){
      if(isDetail){
        globalFrame->setForget();
        isDetail = KFalse;
      }
      else{
        globalFrame->setFrame();
        isDetail = KTrue;
      }
    }

  }
  }
}

void T473TclEventDisplayControl::keyAction(KonoeTclEntry* kbt, KInt type,
                                                    KeySym keysym, KInt state)
{
  if(type == KKeyRelease){
  if(keysym == XK_KP_Enter || keysym == XK_Return){
    if(strcmp(kbt->getName(), "postscriptEntry") == 0)
      postscriptAction();
    else if(strcmp(kbt->getName(), "xwdEntry") == 0)
      xwdAction();
    else
      setButtonAction();
  }
  }
}

void T473TclEventDisplayControl::timer()
{
static KInt n = 0;
cout << "============  No. " << n++  << " ==================" << endl;

//  hitPoint->setPoint( KonoeTcl3DVector(0, 0, 0) );

  if(!getenv("NONAMESPACE")){
  //setThetaY();
  T473ReconstructData* rec_data = 0;

  switch(state){
    case KReqType:
cout << "KReqType : " << reqtype << endl;
      data->requestType(reqtype);
      state = KGet;
      break;
    case KGet:
cout << "KGet" << endl;
      rec_data =  data->getReconstructData();
      state = KGet;
      break;
    case KInfoChange:
cout << "KInfoChange" << endl;
      data->infoChange();
      state = KGet;
      break;
    case KReqChange:
cout << "KReqChange" << endl;
      data->requestChange();
      state = KReqType;
      break;
    default : ;
  }

  ed->setStatus( rec_data );
  if(ed2d)
  ed2d->setDataSource(ed->getADCValue(), ed->getHitEnergy(), ed->getHitTime());

  delete rec_data;
  }

  ed->setHitColor();
  ed->draw();
}
