//=====================================================================
/*

    T473TclEventDisplayRequest.cc

    Version 0.0.1   23-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclEventDisplayRequest.hh>
#include <T473TclEventDisplay.hh>
#include <T473TclEventDisplayControl.hh>

#include <stdlib.h>

T473TclEventDisplayRequest::T473TclEventDisplayRequest
                 (KonoeTclWindow* pwin, T473TclEventDisplayControl* cont)
{
  kwin = pwin;
  control = cont;

  strcpy(fontName, "courier");
  fontSize = 14;

  strcpy(backColor, "#ffffcc");

  initWidget();
}

T473TclEventDisplayRequest::~T473TclEventDisplayRequest()
{
  delete requestTypeLabel;
  delete eventButton;
  delete xenonButton;
  delete randomButton;
  delete infoChange;
  delete displayDataButton;
  delete modeHitEnergy;
  delete modeADCValue;
  delete modeHitTime;
  delete requestButton;

  delete requestTypeFrame;
  delete dummyFrame;
  delete displayDataFrame;
}

void T473TclEventDisplayRequest::setEventDisplay
                               (T473TclEventDisplay* eventDisplay)
{
  ed = eventDisplay;
}

void T473TclEventDisplayRequest::initWidget()
{

//==================== Request Type Frame  ====================
  requestTypeFrame = new KonoeTclFrame( kwin, "requestTypeFrame" );

  requestTypeLabel = new KonoeTclLabel( requestTypeFrame, "requestTypeLabel",
                                        "Trigger Type", 0, 0);

  eventButton = new KonoeTclButton( requestTypeFrame, "eventButton",
                                           "Event  ", 1, 0);
  xenonButton = new KonoeTclButton( requestTypeFrame, "xenonButton",
                                         "Xenon  ", 1, 1);
  randomButton = new KonoeTclButton( requestTypeFrame, "randomButton",
                                         "Random ", 1, 2);
  infoChange = new KonoeTclButton( requestTypeFrame, "infoChange",
                                        "Information Change", 0, 3); 

  requestTypeLabel->setGridRowspan(3);
  infoChange->setGridColumnspan(3);

  requestTypeLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  eventButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  xenonButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  randomButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  infoChange->setFont(fontName, fontSize, 1, 0, 0, 0);

  requestTypeLabel->setBackGround(backColor);
  requestTypeLabel->setGridSticky("news");
  requestTypeLabel->setRelief(KRaised);

  eventButton->setGridSticky("w");
  xenonButton->setGridSticky("w");
  randomButton->setGridSticky("w");

  randomButton->setActiveBack("tomato");
  randomButton->setBackGround("tomato");

  requestTypeLabel->setGridIpadx(10);
  infoChange->setGridPady(5);

  requestTypeFrame->setFrame();
  requestTypeFrame->fillX();

  requestTypeFrame->setBackGround(backColor);


//==================== Dummy Frame  ====================
  dummyFrame = new KonoeTclFrame( kwin, "dummyFrame" );
  dummyFrame->setSize(0, 10);
  dummyFrame->setBackGround( backColor );

  dummyFrame->setFrame();
  dummyFrame->fillX();


//==================== Display Data Frame  ====================

  displayDataFrame = new KonoeTclFrame( kwin, "displayDataFrame" );

  displayDataButton = new KonoeTclButton( displayDataFrame, "displayDataButton",
                                        "Display Data", 0, 0);

  modeHitEnergy = new KonoeTclRadioButton( displayDataFrame, "modeHitEnergy",
                                           "Hit Energy", 1, 0);
  modeADCValue = new KonoeTclRadioButton( displayDataFrame, "modeADCValue",
                                           "ADC Value ", 1, 1);
  modeHitTime = new KonoeTclCheckButton( displayDataFrame, "modeHitTime",
                                           "Hit Time  ", 1, 2);

  displayDataButton->setGridRowspan(3);
  displayDataButton->setRelief(KRaised);
  displayDataButton->setGridSticky("news");
  displayDataButton->setGridIpadx(3);
  displayDataButton->setActiveBack("#fcf180");

  displayDataButton->setBackGround(backColor);
  modeHitEnergy->setBackGround(backColor);
  modeADCValue->setBackGround(backColor);
  modeHitTime->setBackGround(backColor);

  displayDataButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  modeHitEnergy->setFont(fontName, fontSize, 1, 0, 0, 0);
  modeADCValue->setFont(fontName, fontSize, 1, 0, 0, 0);
  modeHitTime->setFont(fontName, fontSize, 1, 0, 0, 0);

  modeHitEnergy->setActiveBack("blue");
  modeADCValue->setActiveBack("blue");
  modeHitTime->setActiveBack("blue");
  modeHitEnergy->setActiveFore("white");
  modeADCValue->setActiveFore("white");
  modeHitTime->setActiveFore("white");

  modeHitEnergy->setGroup("displayDataGroup");
  modeADCValue->setGroup("displayDataGroup");

  modeHitEnergy->setSelect();

  displayDataFrame->setFrame();
  displayDataFrame->fillX();
  displayDataFrame->setBackGround(backColor);

  requestButton =
       new KonoeTclButton( displayDataFrame, "requestButton", "Request", 0, 4);

  requestButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  requestButton->setGridPady(5);
  requestButton->setGridColumnspan(3);


//==================== Set Button Action  ====================

  eventButton->setEventProcessor(this);
  xenonButton->setEventProcessor(this);
  randomButton->setEventProcessor(this);
  infoChange->setEventProcessor(this);
  displayDataButton->setEventProcessor(this);
  requestButton->setEventProcessor(this);

  addMouseAction( eventButton );
  addMouseAction( xenonButton );
  addMouseAction( randomButton );
  addMouseAction( infoChange );
  addMouseAction( displayDataButton );
  addMouseAction( requestButton );
}

void T473TclEventDisplayRequest::mouseAction(KonoeTclButton* kbtn, KInt type,
                                      KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if(strcmp(kbtn->getName(), "requestButton") == 0)
        request();

      else if(strcmp(kbtn->getName(), "eventButton") == 0)
        eventButtonAction();

      else if(strcmp(kbtn->getName(), "xenonButton") == 0) 
        xenonButtonAction();

      else if(strcmp(kbtn->getName(), "randomButton") == 0) 
        randomButtonAction();

      else if(strcmp(kbtn->getName(), "infoChange") == 0)
        control->changeState( T473TclEventDisplayControl::KInfoChange );

      else if(strcmp(kbtn->getName(), "displayDataButton") == 0)
        control->create2DEventDisplay();
    }
  }
}

void T473TclEventDisplayRequest::eventButtonAction()
{
        control->setRequestType( T473TclEventDisplayControl::EVENT_TRIGGER );
        control->changeState( T473TclEventDisplayControl::KReqChange );
        eventButton->setBackGround("tomato");
        xenonButton->setBackGround("#d9d9d9");
        randomButton->setBackGround("#d9d9d9");
        eventButton->setActiveBack("tomato");
        xenonButton->setActiveBack("#e8e8e8");
        randomButton->setActiveBack("#e8e8e8");
}

void T473TclEventDisplayRequest::xenonButtonAction()
{
        control->setRequestType( T473TclEventDisplayControl::XENON_TRIGGER );
        control->changeState( T473TclEventDisplayControl::KReqChange );
        eventButton->setBackGround("#d9d9d9");
        xenonButton->setBackGround("tomato");
        randomButton->setBackGround("#d9d9d9");
        eventButton->setActiveBack("#e8e8e8");
        xenonButton->setActiveBack("tomato");
        randomButton->setActiveBack("#e8e8e8");
}

void T473TclEventDisplayRequest::randomButtonAction()
{
        control->setRequestType( T473TclEventDisplayControl::RANDOM_TRIGGER );
        control->changeState( T473TclEventDisplayControl::KReqChange );
        eventButton->setBackGround("#d9d9d9");
        xenonButton->setBackGround("#d9d9d9");
        randomButton->setBackGround("tomato");
        eventButton->setActiveBack("#e8e8e8");
        xenonButton->setActiveBack("#e8e8e8");
        randomButton->setActiveBack("tomato");
}

void T473TclEventDisplayRequest::request()
{
  enum { KHITENERGY, KADCVALUE, KHITENERGY_HITTIME, KADCVALUE_HITTIME };

  KChar value[256];
  sprintf( value, "%s", modeHitEnergy->getValue() );

  if( strcmp( value, "modeHitEnergy" ) == 0 ){
    if( !atoi(modeHitTime->getValue()) )
      control->setDisplayData( KHITENERGY );
    else
      control->setDisplayData( KHITENERGY_HITTIME );
  }
  else{
    if( !atoi(modeHitTime->getValue()) )
      control->setDisplayData( KADCVALUE );
    else
      control->setDisplayData( KADCVALUE_HITTIME );
  }
}
