//====================================================================
/*
        T473CsIADCSumExtractor.cc

	20-Nov-2000	Y. Shimizu
*/
//====================================================================


#include <analysis/KonoePhysicalValue.hh>
#include <analysis/KonoeFillData.hh>
#include <analysis/KonoeFillDataExtractor.hh>
#include "T473CsIADCSumExtractor.hh"
#include <stdio.h>
#include <unistd.h>

T473CsIADCSumExtractor::T473CsIADCSumExtractor(KonoePhysicalValue * pvalue,
					       KChar * name1, 
					       KChar * name2, 
					       KChar * name3 )
{
  
  KonoeMultipletDescriptor	* d;
  KInt	i, j;
  
  for( i = 0; i < 25 ; i++)
    pedestal[i] = 0;
  
  for( i = 0; i < FILL_DATA_DIMENSION; i ++ )
    indices[ i ] = -1;
  physicalvalue = pvalue;
  d = pvalue -> getDescriptor( );
  if( name1 )
    indices[ 0 ] = d -> getIndex( name1 );
  if( name2 )
    indices[ 1 ] = d -> getIndex( name2 );
  if( name3 )
    indices[ 2 ] = d -> getIndex( name3 );
  j = 0;
  for( i = 0; i < FILL_DATA_DIMENSION; i ++ )
    if( indices[ i ] == -1 )
      indices[ i ] = j ++;
  
}

void T473CsIADCSumExtractor::setPedestal(KInt index , KInt pede){
  
  pedestal[index] = pede;
  
}

KonoeFillData *T473CsIADCSumExtractor::next( )
{
  
  KonoeDataMultiplet	* multiplet;
  static KFloat	fill_data = 0;
  
  multiplet = physicalvalue -> next( );
  if( multiplet == 0 )
    return 0;
  
  //#define EXT_DEBUG
#ifdef EXT_DEBUG
  cout <<  multiplet->getValue( indices[ 0 ] )  << " - "  ;
  cout <<  multiplet->getValue( indices[ 1 ] )  << " - "  ;
  cout <<  multiplet->getValue( indices[ 2 ] )  << endl   ;
#endif
  
  KInt seg    = (KInt)multiplet->getValue( indices[ 1 ] );
  KInt subadd = (KInt)multiplet->getValue( indices[ 2 ] );
  
  if( ( seg == 0 ) && ( subadd == 0 ) ){
    fill_data = 0;
    fill_data += multiplet->getValue( indices[ 0 ] );
    fill_data -= pedestal[ address_to_index( seg, subadd )];
    this->next();
  } else if( ( seg == 0 ) && ( subadd != 0 ) ) {
    fill_data += multiplet->getValue( indices[ 0 ] );
    fill_data -= pedestal[ address_to_index( seg, subadd )];
    this->next();
  } else if ( ( seg == 1 ) && ( subadd <= 7 ) ) {
    fill_data += multiplet->getValue( indices[ 0 ] );
    fill_data -= pedestal[ address_to_index( seg, subadd )];
    this->next();
  } else if ( ( seg == 1 ) && ( subadd == 8 ) ) {
    
    fill_data += multiplet->getValue( indices[ 0 ] );
    fill_data -= pedestal[ address_to_index( seg, subadd )];
    return new KonoeFillData( fill_data ,
			      multiplet -> getValue( indices[ 1 ] ),
			      multiplet -> getValue( indices[ 2 ] ) );
  }
  else {
    this->next();
  }
  
}

KInt 	T473CsIADCSumExtractor::address_to_index( KInt seg, KInt subadd ){

	KInt i ;
	KInt j ;

	if( seg == 0 ){
		return subadd;
	}
		
	if( seg == 1 ){
		return subadd + 16 ;
	}

}
