//===================================================
/*
	T473Histogram.cc

	 4-Oct-2000	Y. Shimizu
	31-Oct-2000	Y. Shimizu
	 8-Nov-2000	Y. Shimizu
	10-Dec-2000	Y. Shimizu
				Xe event
*/
//==================================================	

#include	"T473Histogram.hh"
#include	"T473HistoInfo.hh"
#include        "T473XenonChecker.hh"
#include        "T473Process1DHistogram.hh"
#include	<analysis/KonoeDataRecord.hh>
#include	<system/KonoeHostSystem.hh>
#include	<system/KonoeMicroSleep.hh>
#include	<system/KonoeSocketIO.hh>
#include	<stdio.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/wait.h>
#include        <string.h>
#include        <system/KonoeHostSystem.hh>
//#include	<system/KonoeBench.hh>

const KChar * T473Histogram::DATAHUB_REQ_CURRENT = "DATAHUB_REQ_CURRENT";
const KChar * T473Histogram::DATAHUB_REQ_HEAD = "DATAHUB_REQ_HEAD";
const KChar * T473Histogram::DATAHUB_REQ_TAIL = "DATAHUB_REQ_TAIL";

T473Histogram::T473Histogram(KInt cmd_port ):T473Process( cmd_port ){

  setter  = new T473NameSpaceSetter();

  mode = NONE;

  log = new T473LogSender;
  log->send("T473Histogram Start",1);
  check = new T473StatusChecker("HISTOGRAM") ;

  cout <<"T473Histogram::informationSet()"<<endl;
  this->informationSet();
  cout <<"T473Histogram::histogramSet()"<<endl;
  this->histogramSet();
  cout <<"T473Histogram::datasocketSet()"<<endl;
  this->datasocketSet();
  cout <<"T473Histogram::uisocketSet()"<<endl;
  this->uisocketSet();

  event = 0;
}

KInt T473Histogram::begin(){
  mode = READY;

  setter->set("HISTOGRAM_STATUS","busy");

  if( end_flag ){
    this->histogramReset();
    spill_end->clear( );
    ns_reset( );
  }
  end_flag = KFalse;

  check->setTrue();

  KonoeDataRecord * rec;
  KonoeFDClient * cl;
  KonoeFDClient * hub_cl;

  KonoeMicroSleep * ms = new KonoeMicroSleep;

  cout << "\n\tT473Histogram::begin\n" << endl;
  log->send("T473Histogram BEGIN",0);
  while( ! data_client->getStatus( ) == KonoeSocketIO::CONNECTED ){
     data_client->connect();
     sleep(1);
  }
	 
  

  while( 1 ){ 
    if( mode == IDLE )
      break;
    
    if( data_ext_mode == REQ_CURRENT )
       data_client->send(DATAHUB_REQ_CURRENT,strlen(DATAHUB_REQ_CURRENT)+1);
    if( data_ext_mode == REQ_TAIL)
       data_client->send(DATAHUB_REQ_TAIL,strlen(DATAHUB_REQ_TAIL)+1);
    if( data_ext_mode == REQ_HEAD )
       data_client->send(DATAHUB_REQ_HEAD,strlen(DATAHUB_REQ_HEAD)+1);

    rec = 0;

//    hub_cl = readfds_for_hub->wait( wait_interval );
    hub_cl = readfds_for_hub->wait( 0 );
    if( hub_cl ){
      if( hub_cl ==  (KonoeClientSocket*)data_client ){
        rec = (KonoeDataRecord*)data_socket->get( );
      }
      else 
	cerr << "unknown socket ?" << endl;
    }
    else {
	ms->sleep(1);
    }

    if( rec ){

      if( (rec->getType() == T473HistoInfo::RUN_END_TYPE) ) {
	cerr << "RUN_END_TYPE " << endl;
	cerr << "RUN_END_TYPE event -> " << event  << endl;
        delete rec ;
//	this->end( );
      }
      else if( (rec->getType() == T473HistoInfo::SPILL_END_TYPE) ) {
        spill_end->process( rec );
        delete rec ;
      }
      else {

	  if( xenon->isXenonEvent( rec ) ) {

	     analyzer->reset( );
	     analyzer->analyze( rec );
	     filler_list_xenon -> fill( );
	     this->histogramListFill_xenon();

	  }
	  else {

	     analyzer->reset( );
	     analyzer->analyze( rec );
	     filler_list -> fill( );
	     this->histogramListFill();

	  }

	event++;
	if( event % event_cycle  == 0 ){
	  cout << event << "events get!" << endl;
	}

        delete rec ;

      }


    } // if( rec )

//    cl = readfds->wait( wait_interval );
    cl = readfds->wait( 0 );
    if( cl ){
      if( fork() == 0 )
        this->processFDClient( cl );
      else {
        uisocketClose();
        wait(0);
      }
    }
    
  }  // while(1)

  this->pause();
  return (0);
}

KInt T473Histogram::pause(){
  setter->set("HISTOGRAM_STATUS","idle");

  end_flag = KFalse;

  check->setTrue();
  
  cout << "\n\tT473Histogram::pause\n" << endl;
  
  log->send("T473Histogram PAUSE",0);

  if( event )
    this->pause_process( );
  
  
  while( mode != READY && mode != STANDBY){
    handler->wait();
  }
  if( mode == READY)
    begin();
  if( mode == STANDBY)
    end();
  
  return -1;
}

KInt T473Histogram::resume(){
  setter->set("HISTOGRAM_STATUS","busy");

  end_flag = KFalse;

  check->setTrue();

  this->begin();
  
  return -1;
}

KInt T473Histogram::end(){
  setter->set("HISTOGRAM_STATUS","standby");
  check->setTrue();

  KonoeFDClient * hub_cl;
  KonoeDataRecord* rec;

  log->send("T473Histogram END",0);


///////////////////////////////////////////////////////////////////////
/*
  KChar *filename1 = extractor->getKCharValue("CSI_TDC_HISTONAME");
  if( filename1 != (KChar*)-1)
    file1 = new KonoeOutputObjFile(filename1);
  else
    file1 = new KonoeOutputObjFile("CSI_TDC_HISTO.OBJ");
  
  KChar *filename2 = extractor->getKCharValue("CSI_ADC_HISTONAME");
  if( filename2 != (KChar*)-1)
    file2 = new KonoeOutputObjFile(filename2);
  else
    file2 = new KonoeOutputObjFile("CSI_ADC_HISTO.OBJ");
  
  KChar *filename3 = extractor->getKCharValue("SCIN_TDC_HISTONAME");
  if( filename3 != (KChar*)-1)
    file3 = new KonoeOutputObjFile(filename3);
  else
    file3 = new KonoeOutputObjFile("SCIN_TDC_HISTO.OBJ");
  
  KChar *filename4 = extractor->getKCharValue("SCIN_ADC_HISTONAME");
  if( filename4 != (KChar*)-1)
    file4 = new KonoeOutputObjFile(filename4);
  else
    file4 = new KonoeOutputObjFile("SCIN_ADC_HISTO.OBJ");
  
  KChar *filename6 = extractor->getKCharValue("TOF_HISTONAME");
  if( filename6 != (KChar*)-1)
    file6 = new KonoeOutputObjFile(filename6);
  else
    file6 = new KonoeOutputObjFile("TOF_HISTO.OBJ");

  file1->put( csi_tdc_list );
  file2->put( csi_adc_list );
  file3->put( scin_tdc_list );
  file4->put( scin_adc_list );
  file6->put( tof_list );

  delete file1;
  delete file2;
  delete file3;
  delete file4;
  delete file6;

  file1 = 0; 
  file2 = 0; 
  file3 = 0; 
  file4 = 0; 
  file6 = 0; 
*/
/////////////////////////////////////////////////////////////////////
  hub_cl = readfds_for_hub->wait( 2000 );
  if( hub_cl ){
      if( hub_cl ==  (KonoeClientSocket*)data_client ){
        rec = (KonoeDataRecord*)data_socket->get( );
	delete rec;
      }
      else 
	cerr << "unknown socket ?" << endl;
  }

  cout << "\n\tT473Histogram::end\n" << endl;
  event = 0;

  end_flag = KTrue;

  while( mode != NONE && mode != READY){
    handler->wait();
  }
  if( mode == READY )
    begin();
  if( mode == NONE);
  shutdown();

  return -1;
}

KInt T473Histogram::shutdown(){
  setter->set("HISTOGRAM_STATUS","none");
  setter->set("HISTOGRAM_PID",-1);
  check->setTrue();

  cout << "\n\tT473Histogram::shutdown\n" << endl;

  log->send("T473Histogram shutdown",0);
  
  KonoeHostSystem::exit(0);
  return (0);
  
}

////////////////////////////////////////////////////////////////////////////

void T473Histogram::all_histo_dump(){
  KInt i;
  
  cout << " ******General****** " << endl; 
  general_list->dump(cout);	
  cout << " *******CsI TDC****** " << endl; 
  csi_tdc_list->dump(cout);
  cout << " ****CsI ADC****** " << endl;
  csi_adc_list->dump(cout);
  cout << " ****Scin TDC****** " << endl; 
  scin_tdc_list->dump(cout);
  cout << " ****Scin ADC****** " << endl;
  scin_adc_list->dump(cout);
  cout << " ****Scaler****** " << endl; 
  scaler_list->dump(cout);  
  cout << " ****TOF****** " << endl; 
  tof_list->dump(cout);  
}

KInt T473Histogram::histogramReset(){
cout << "\tT473Histogram::histogramReset()"<<endl;

  KInt i;
  
  for( i = 0; i < T473HistoInfo::GENERAL_HISTO ; i++)
    general_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    csi_tdc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_filler->getXenonHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    scin_tdc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    scin_adc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCALER_HISTO ; i++)
    scaler_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::TOF_HISTO ; i++)
    tof_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSIADCSUM_HISTO ; i++)
    csi_adc_sum_filler->getHisto( i )->clear();


  csi_tdc_filler->setPedestal();
  csi_adc_filler->setPedestal();
  tof_filler->setPedestal();
  csi_adc_sum_filler->setPedestal();
  scin_adc_filler->setPedestal();
  
  spill_end->clear( );

  return (0);
}


KInt T473Histogram::processFDClient(KonoeFDClient* pcl ){

  if( pcl == (KonoeServerSocket*)ui_server0 ){
    send_0_UI();	
    return (0);
  }
  else if( pcl == (KonoeServerSocket*)ui_server1 ){
    send_1_UI();	
    return (0);
  }
  else if( pcl == (KonoeServerSocket*)ui_server2 ){
    send_2_UI();	
    return (0);
  }
  else if( pcl == (KonoeServerSocket*)event_num_socket ){
    event_num_UI();	
    return (0);
  }
  else {
    cerr <<"T473Histogram  unknown FDClient"<<endl;
    return (-1);
  }
  
}
	      

void T473Histogram::send_0_UI(){
  
    KChar	buff[256];
    ui_server0->waitConnection(); 
    ui_server0->receive(buff,256);
    if( this->isValidReq( buff ))
      ui_socket0->put( this->strToHisto(buff));
    delete ui_server0;
    KonoeHostSystem::exit(0);

};

void T473Histogram::send_1_UI(){

    KChar	buff[256];
    ui_server1->waitConnection(); 
    ui_server1->receive(buff,256);
    if( this->isValidReq( buff ))
      ui_socket1->put( this->strToHisto(buff));
    delete ui_server1;
    KonoeHostSystem::exit(0);

};

void T473Histogram::send_2_UI(){


    KChar	buff[256];
    ui_server2->waitConnection(); 
    ui_server2->receive(buff,256);
    if( this->isValidReq( buff ))
      ui_socket2->put( this->strToHisto(buff));
    delete ui_server2;
    KonoeHostSystem::exit(0);

};


KInt T473Histogram::pause_process( ){
cout << "\tT473Histogram::pause_process( )"<<endl;
  KChar logtxt[32];
  sprintf(logtxt,"histogram %d events get!",event);
  log->send(logtxt,1);

  KInt i,j;
  KInt peak;
  Konoe1DHistogram * histogram;
  histogram = 0;
  
  KChar buff[32];
  KChar valuetxt[32];
  for( i = 0 ; i < T473SpillEndProcessor::SEG0_ELEMENTS ; i++) {
    sprintf(buff,"SINGLE_COUNTS_0_%d", i );
    setter->set(buff,(KInt)spill_end->getElement0( i ));
  }
  for( i = 0 ; i < T473SpillEndProcessor::SEG1_ELEMENTS ; i++) {
    sprintf(buff,"SINGLE_COUNTS_1_%d", i );
    setter->set(buff,(KInt)spill_end->getElement1( i ));
  }

  sprintf(buff,"HISTOGRAM_EVENT");
  setter->set(buff,event);
	
//////////////// PROCESS HISTOGRAM ///////////////// 
  // CsI ADC
  csi_adc_list->reset();
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++){
    T473Process1DHistogram * processor;
    histogram = csi_adc_list->next( );
    if( histogram ){
      processor = new T473Process1DHistogram( histogram );
      sprintf(buff,"CSI_ADC_NO:%d_PEAK",i + 1);
      sprintf(valuetxt,"%d",processor->getMaxBinNumber( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_NO:%d_MEAN",i + 1);
      sprintf(valuetxt,"%lf",processor->getMean( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_NO:%d_VARI",i + 1);
      sprintf(valuetxt,"%lf",processor->getVariance( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_NO:%d_STAN",i + 1);
      sprintf(valuetxt,"%lf",processor->getStandardDeviation( 0 , 4096 ));
      setter->set(buff,valuetxt);
      delete processor;
    }
  }
		
  // CsI ADC Xe
  csi_adc_list_xenon->reset();
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++){
    T473Process1DHistogram * processor;
    histogram = csi_adc_list_xenon->next( );
    if( histogram ){
      processor = new T473Process1DHistogram( histogram );
      sprintf(buff,"CSI_ADC_XENON_NO:%d_PEAK",i + 1);
      sprintf(valuetxt,"%d",processor->getMaxBinNumber( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_XENON_NO:%d_MEAN",i + 1);
      sprintf(valuetxt,"%lf",processor->getMean( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_XENON_NO:%d_VARI",i + 1);
      sprintf(valuetxt,"%lf",processor->getVariance( 0 , 4096 ));
      setter->set(buff,valuetxt);
      sprintf(buff,"CSI_ADC_XENON_NO:%d_STAN",i + 1);
      sprintf(valuetxt,"%lf",processor->getStandardDeviation( 0 , 4096 ));
      setter->set(buff,valuetxt);
      delete processor;
    }
  }

  // Scin
  scin_adc_list->reset( );
  for( i = 0 ; i < T473HistoInfo::SCIN_ADC_HISTO ; i++){
    T473Process1DHistogram * processor;
    histogram = scin_adc_list->next( );
    if( histogram ){
      processor = new T473Process1DHistogram( histogram );
      if( 0 <= i && i <= 7){
	sprintf(buff,"SCIN_ADC_NO:%d_PEAK",i);
	sprintf(valuetxt,"%d",processor->getMaxBinNumber( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"SCIN_ADC_NO:%d_MEAN",i);
	sprintf(valuetxt,"%lf",processor->getMean( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"SCIN_ADC_NO:%d_VARI",i);
	sprintf(valuetxt,"%lf",processor->getVariance( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"SCIN_ADC_NO:%d_STAN",i);
	sprintf(valuetxt,"%lf",processor->getStandardDeviation( 0 , 4096 ));
	setter->set(buff,valuetxt);
      }
      else if( 8 <= i && i <= 9 ){
	sprintf(buff,"CHERENKOV_ADC_NO:%d_PEAK",i - 7 );
	sprintf(valuetxt,"%d",processor->getMaxBinNumber( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"CHERENKOV_ADC_NO:%d_MEAN",i - 7 );
	sprintf(valuetxt,"%lf",processor->getMean( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"CHERENKOV_ADC_NO:%d_VARI",i - 7 );
	sprintf(valuetxt,"%lf",processor->getVariance( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"CHERENKOV_ADC_NO:%d_STAN",i - 7 );
	sprintf(valuetxt,"%lf",processor->getStandardDeviation( 0 , 4096 ));
	setter->set(buff,valuetxt);
      }
      else if( 10 <= i && i <= 13 ){
	sprintf(buff,"XENON_SCIN_ADC_NO:%d_PEAK",i - 9 );
	sprintf(valuetxt,"%d",processor->getMaxBinNumber( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"XENON_SCIN_ADC_NO:%d_MEAN",i - 9 );
	sprintf(valuetxt,"%lf",processor->getMean( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"XENON_SCIN_ADC_NO:%d_VARI",i - 9 );
	sprintf(valuetxt,"%lf",processor->getVariance( 0 , 4096 ));
	setter->set(buff,valuetxt);
	sprintf(buff,"XENON_SCIN_ADC_NO:%d_STAN",i - 9 );
	sprintf(valuetxt,"%lf",processor->getStandardDeviation( 0 , 4096 ));
	setter->set(buff,valuetxt);
      }
      delete processor;
    }
  }
  
////////// PEDESTAL RUN ///////////////////////
  if( ! strcmp(ext->getKCharValue("RUN_TYPE"),"pedestal_run")){
    cout << "\tT473Histogram::Pedestal change!! "<<endl;

    //  CSI ADC PEDESTAL SET 
    j = -1;
    csi_adc_list->reset( );
    for( i = 0 ; i < 25 ; i++){
      T473Process1DHistogram * processor;
      if( ( i / 16 ) * 16  == i )
	j++;
      histogram = csi_adc_list->next( );
      processor = new T473Process1DHistogram( histogram );
      peak =  processor->getMaxBinNumber( 0 , 4096 );
      sprintf(buff,"CSI_ADC_%d_%d_PED",j, i - ( j * 16 ));
      setter->set( buff,peak );
      
      delete processor;
    }
    
    // SCIN ADC PEDESTAL SET
    j = -1;
    j++;
    scin_adc_list->reset( );
    for( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO ; i++){
      T473Process1DHistogram * processor;
      histogram = scin_adc_list->next( );
      processor = new T473Process1DHistogram( histogram );
      peak =  processor->getMaxBinNumber( 0 , 4096 );
      sprintf(buff,"SCIN_ADC_%d_%d_PED",j, i - ( j * 16 ));
      setter->set( buff,peak );
    }
  }
  return (0);
}

KInt T473Histogram::ns_reset( ){
  
  KInt i;
  KChar buff[32];
  KChar valuetxt[32];
  
  for( i = 0 ; i < T473SpillEndProcessor::SEG0_ELEMENTS ; i++) {
    sprintf(buff,"SINGLE_COUNTS_0_%d",i);
    setter->set(buff,(KInt)0);
  }
  
  for( i = 0 ; i < T473SpillEndProcessor::SEG1_ELEMENTS ; i++) {
    sprintf(buff,"SINGLE_COUNTS_1_%d", i );
    setter->set(buff,(KInt)0);
  }
  
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++){
    sprintf(buff,"CSI_ADC_NO:%d_PEAK",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_NO:%d_MEAN",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_NO:%d_VARI",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_NO:%d_STAN",i + 1);
    setter->set(buff,(KInt)0);
    
  }
  
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++){
    sprintf(buff,"CSI_ADC_XENON_NO:%d_PEAK",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_XENON_NO:%d_MEAN",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_XENON_NO:%d_VARI",i + 1);
    setter->set(buff,(KInt)0);
    sprintf(buff,"CSI_ADC_XENON_NO:%d_STAN",i + 1);
    setter->set(buff,(KInt)0);
    
  }
  
  // Scin
  for( i = 0 ; i < T473HistoInfo::SCIN_ADC_HISTO ; i++){
    if( 0 <= i && i <= 7){
      sprintf(buff,"SCIN_ADC_NO:%d_PEAK",i);
      setter->set(buff,(KInt)0);
      sprintf(buff,"SCIN_ADC_NO:%d_MEAN",i);
      setter->set(buff,(KInt)0);
      sprintf(buff,"SCIN_ADC_NO:%d_VARI",i);
      setter->set(buff,(KInt)0);
      sprintf(buff,"SCIN_ADC_NO:%d_STAN",i);
      setter->set(buff,(KInt)0);
    }
    else if( 8 <= i && i <= 9 ){
      sprintf(buff,"CHERENKOV_ADC_NO:%d_PEAK",i - 7 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"CHERENKOV_ADC_NO:%d_MEAN",i - 7 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"CHERENKOV_ADC_NO:%d_VARI",i - 7 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"CHERENKOV_ADC_NO:%d_STAN",i - 7 );
      setter->set(buff,(KInt)0);
    }
    else if( 10 <= i && i <= 13 ){
      sprintf(buff,"XENON_SCIN_ADC_NO:%d_PEAK",i - 9 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"XENON_SCIN_ADC_NO:%d_MEAN",i - 9 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"XENON_SCIN_ADC_NO:%d_VARI",i - 9 );
      setter->set(buff,(KInt)0);
      sprintf(buff,"XENON_SCIN_ADC_NO:%d_STAN",i - 9 );
      setter->set(buff,(KInt)0);
    }
    
  }
  
  return (0);
  
} 


void T473Histogram::event_num_UI(){

    KChar	buff[256];
    KChar	eventmsg[256];
    sprintf(eventmsg,"%d",event);
    event_num_socket->waitConnection(); 
    event_num_socket->receive(buff,strlen(buff));
    if( !strcmp(buff,"event"))
      event_num_socket->send(eventmsg,256);
    delete event_num_socket;
    KonoeHostSystem::exit(0);
  
};

KInt T473Histogram::isValidReq(KChar* request ){

  if( !strcmp(request,"general"))
    return 1;
  if( !strcmp(request,"csi_tdc"))
    return 1;
  if( !strcmp(request,"csi_adc"))
    return 1;
  if( !strcmp(request,"csi_adc_xenon"))
    return 1;
  if( !strcmp(request,"scin_tdc"))
    return 1;
  if( !strcmp(request,"scin_adc"))
    return 1;
  if( !strcmp(request,"scaler"))
    return 1;
  if( !strcmp(request,"tof"))
    return 1;
  if( !strcmp(request,"csi_adc_sum"))
    return 1;
  
  return (0);
  
}

KInt T473Histogram::histogramSet(){

  KInt i;

  general_list = new Konoe1DHistogramList( T473HistoInfo::GENERAL_HISTO); 
  csi_tdc_list = new Konoe1DHistogramList( T473HistoInfo::CSI_TDC_HISTO);
  csi_adc_list = new Konoe1DHistogramList( T473HistoInfo::CSI_ADC_HISTO);
  csi_adc_list_xenon = new Konoe1DHistogramList( T473HistoInfo::CSI_ADC_HISTO);
  scin_tdc_list = new Konoe1DHistogramList( T473HistoInfo::SCIN_TDC_HISTO);
  scin_adc_list = new Konoe1DHistogramList( T473HistoInfo::SCIN_ADC_HISTO);
  scaler_list	= new Konoe1DHistogramList( T473HistoInfo::SCALER_HISTO);  
  tof_list	= new Konoe1DHistogramList( T473HistoInfo::TOF_HISTO );
  csi_adc_sum_list = new Konoe1DHistogramList( T473HistoInfo::CSIADCSUM_HISTO );  

//..........................................................................
	
  general_analyzer = new KonoeDefaultSectionAnalyzer(GENERAL,N_MULTIPLET,0);
  csi_tdc_analyzer = new KonoeDefaultSectionAnalyzer(CSI_TDC,N_MULTIPLET,0);
  csi_adc_analyzer = new KonoeDefaultSectionAnalyzer(CSI_ADC,N_MULTIPLET,0);
  scin_tdc_analyzer = new KonoeDefaultSectionAnalyzer(SCIN_TDC,N_MULTIPLET,0);
  scin_adc_analyzer = new KonoeDefaultSectionAnalyzer(SCIN_ADC,N_MULTIPLET,0);
  scaler_analyzer = new KonoeDefaultSectionAnalyzer(SCALER,N_MULTIPLET,0);

  analyzer = new KonoeDataAnalyzer( T473HistoInfo::N_SECTION );
  analyzer->install( general_analyzer );
  analyzer->install( csi_tdc_analyzer );
  analyzer->install( csi_adc_analyzer );
  analyzer->install( scin_tdc_analyzer );
  analyzer->install( scin_adc_analyzer );
  analyzer->install( scaler_analyzer );

  general_filler = new T473GeneralHistoFiller( general_analyzer );
  csi_tdc_filler = new T473CsITDCHistoFiller( csi_tdc_analyzer );
  csi_adc_filler = new T473CsIADCHistoFiller( csi_adc_analyzer );
  scin_tdc_filler = new T473ScinTDCHistoFiller( scin_tdc_analyzer);
  scin_adc_filler = new T473ScinADCHistoFiller( scin_adc_analyzer);
  scaler_filler   = new T473ScalerHistoFiller( scaler_analyzer );
  tof_filler   = new T473TOFFiller( scin_tdc_analyzer );
  csi_adc_sum_filler   = new T473CsIADCSumFiller( csi_adc_analyzer );

//oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

  filler_list = new KonoeHistoFillerList( T473HistoInfo::HISTO_TOTAL );

  for( i = 0 ; i < T473HistoInfo::GENERAL_HISTO ; i++)
    filler_list->add( general_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    filler_list->add( csi_tdc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    filler_list->add( csi_adc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::CSIADCSUM_HISTO; i++)
    filler_list->add( csi_adc_sum_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    filler_list->add( scin_tdc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::TOF_HISTO ; i++)
    filler_list->add( tof_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    filler_list->add( scin_adc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCALER_HISTO ; i++)
    filler_list->add( scaler_filler->getFiller( i ) );

  filler_list_xenon = new KonoeHistoFillerList( T473HistoInfo::HISTO_TOTAL -1);

  for( i = 0 ; i < T473HistoInfo::GENERAL_HISTO ; i++)
    filler_list_xenon->add( general_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    filler_list_xenon->add( csi_tdc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    filler_list_xenon->add( csi_adc_filler->getXenonFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    filler_list_xenon->add( scin_tdc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::TOF_HISTO ; i++)
    filler_list_xenon->add( tof_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    filler_list_xenon->add( scin_adc_filler->getFiller( i ) );
  for( i = 0 ; i < T473HistoInfo::SCALER_HISTO ; i++)
    filler_list_xenon->add( scaler_filler->getFiller( i ) );


  for( i = 0; i < T473HistoInfo::GENERAL_HISTO ; i++)
    general_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    csi_tdc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_filler->getXenonHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    scin_tdc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    scin_adc_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::SCALER_HISTO ; i++)
    scaler_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::TOF_HISTO ; i++)
    tof_filler->getHisto( i )->clear();
  for( i = 0; i < T473HistoInfo::CSIADCSUM_HISTO ; i++)
    csi_adc_sum_filler->getHisto( i )->clear();

  xenon = new T473XenonChecker( analyzer , general_analyzer );
  spill_end = new T473SpillEndProcessor( analyzer , scaler_analyzer );
}

KInt T473Histogram::informationSet(){

  KInt pid;
  KChar host[256];

  pid = KonoeHostSystem::getPid();
  KonoeHostSystem::getHostName(host,256) ;
  extractor = new T473NameSpaceExtractor();
  setter->set("HISTOGRAM_HOST",host);
  setter->set("HISTOGRAM_PID",pid);

  event_cycle = T473HistoInfo::EVENT_CYCLE;
  wait_interval = T473HistoInfo::WAIT_INTERVAL;

  KChar* ext_mode = extractor->getKCharValue("HISTO_REQ_MODE");
  if( !strcmp(ext_mode,"DATAHUB_REQ_HEAD"))
	data_ext_mode =  REQ_HEAD;
  else if( !strcmp(ext_mode,"DATAHUB_REQ_CURRENT"))
	data_ext_mode =  REQ_CURRENT;
  else if( !strcmp(ext_mode,"DATAHUB_REQ_TAIL"))
	data_ext_mode =  REQ_TAIL;
  else 
	data_ext_mode =  REQ_CURRENT;
	
  ns_reset( );
//  delete setter;
  return (0);
}

KInt T473Histogram::datasocketSet(){

  readfds_for_hub = new KonoeFDSet;

  KInt dport = extractor->getKIntValue("DATAHUB_OUTPUT_PORT");
  KChar* dhost = extractor->getKCharValue("DATAHUB_HOST");
  data_client = new KonoeClientSocket( dhost,dport);


  d_cm = new KonoeDataRecordCM();	
  
  cout << "DATAHUB_HOST = " << dhost << endl;
  cout << "DATAHUB_OUTPUT_PORT = " << dport << endl;
  if( dport == -1 ){
    cout <<"DATAHUB_TO_HISTO_PORT is not defined "<<endl;  
    KonoeHostSystem::exit(0);
  }
  if( dhost == (KChar*)-1){
    cout <<"DATAHUB_HOST is not defined "<<endl;  
    KonoeHostSystem::exit(0);
  }
  
  data_socket = new KonoeInputObjSocketBuffered( data_client );
  data_socket->installCM( d_cm );

  readfds_for_hub->install( data_client );
  
  return (0);
}

KInt T473Histogram::uisocketSet(){

  readfds = new KonoeFDSet;

  KInt ui_port0 = extractor->getKIntValue("HISTO_TO_UI_PORT0");
  KInt ui_port1 = extractor->getKIntValue("HISTO_TO_UI_PORT1");
  KInt ui_port2 = extractor->getKIntValue("HISTO_TO_UI_PORT2");
  KInt event_num_port = extractor->getKIntValue("EVENT_NUM_PORT");
  if( (ui_port0 == -1) ){
	cout <<"HISTO_TO_UI_PORT0 is not defined "<<endl;  
	KonoeHostSystem::exit(0);
  }
  if( (ui_port1 == -1) ){
	cout <<"HISTO_TO_UI_PORT1 is not defined "<<endl;  
	KonoeHostSystem::exit(0);
  }
  if( (ui_port2 == -1) ){
	cout <<"HISTO_TO_UI_PORT2 is not defined "<<endl;  
	KonoeHostSystem::exit(0);
  }
  if( (event_num_port == -1) ){
	cout <<"EVENT_NUM_PORT is not defined "<<endl;  
	KonoeHostSystem::exit(0);
  }
	
  ui_server0 = new KonoeServerSocket( ui_port0,readfds);
  ui_socket0 = new KonoeOutputObjSocket( ui_server0 );
  ui_server1 = new KonoeServerSocket( ui_port1,readfds);
  ui_socket1 = new KonoeOutputObjSocket( ui_server1 );
  ui_server2 = new KonoeServerSocket( ui_port2,readfds);
  ui_socket2 = new KonoeOutputObjSocket( ui_server2 );
  event_num_socket = new KonoeServerSocket( event_num_port,readfds);

  
  return (0);
}

KInt T473Histogram::uisocketClose(){

  ui_server0->closeConnection();
  ui_server1->closeConnection();
  ui_server2->closeConnection();

  return (0);
}

KInt T473Histogram::histogramListFill_xenon(){

  KInt i;

  general_list->reset( );
  csi_tdc_list->reset( );
//  csi_adc_list->reset( );
  csi_adc_list_xenon->reset( );
  scin_tdc_list->reset( );
  scin_adc_list->reset( );
  scaler_list->reset( );
  tof_list->reset( );
  csi_adc_sum_list->reset( );
  
  for( i = 0; i < T473HistoInfo::GENERAL_HISTO ; i++)
    general_list->add( general_filler->getHisto( i ) );

  for( i = 0; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    csi_tdc_list->add( csi_tdc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_list_xenon->add( csi_adc_filler->getXenonHisto( i ) );

  for( i = 0; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    scin_tdc_list->add( scin_tdc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    scin_adc_list->add( scin_adc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::SCALER_HISTO ; i++)
    scaler_list->add( scaler_filler->getHisto( i ));
  for( i = 0; i < T473HistoInfo::TOF_HISTO ; i++)
    tof_list->add( tof_filler->getHisto( i ));
  
  return (0);
}

KInt T473Histogram::histogramListFill(){

  KInt i;

  general_list->reset( );
  csi_tdc_list->reset( );
  csi_adc_list->reset( );
//  csi_adc_list_xenon->reset( );
  scin_tdc_list->reset( );
  scin_adc_list->reset( );
  scaler_list->reset( );
  tof_list->reset( );
  csi_adc_sum_list->reset( );
  
  for( i = 0; i < T473HistoInfo::GENERAL_HISTO ; i++)
    general_list->add( general_filler->getHisto( i ) );

  for( i = 0; i < T473HistoInfo::CSI_TDC_HISTO ; i++)
    csi_tdc_list->add( csi_tdc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::CSI_ADC_HISTO ; i++)
    csi_adc_list->add( csi_adc_filler->getHisto( i ) );

  for( i = 0; i < T473HistoInfo::SCIN_TDC_HISTO ; i++)
    scin_tdc_list->add( scin_tdc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO ; i++)
    scin_adc_list->add( scin_adc_filler->getHisto( i ) );
  for( i = 0; i < T473HistoInfo::SCALER_HISTO ; i++)
    scaler_list->add( scaler_filler->getHisto( i ));
  for( i = 0; i < T473HistoInfo::TOF_HISTO ; i++)
    tof_list->add( tof_filler->getHisto( i ));
  for( i = 0; i < T473HistoInfo::CSIADCSUM_HISTO ; i++)
    csi_adc_sum_list->add( csi_adc_sum_filler->getHisto( i ));
  
  return (0);
}

Konoe1DHistogramList* T473Histogram::strToHisto(KChar *strings){
  
  if( strcmp(strings,"general") == 0){
    return general_list;
  }
  else if( strcmp(strings,"csi_tdc") == 0){
    return csi_tdc_list;
  }
  else if( strcmp(strings,"csi_adc") == 0){
    return csi_adc_list;
  }
  else if( strcmp(strings,"csi_adc_xenon") == 0){
    return csi_adc_list_xenon;
  }
  else if( strcmp(strings,"scin_tdc") == 0){
    return scin_tdc_list;
  }
  else if( strcmp(strings,"scin_adc") == 0){
    return scin_adc_list;
  }
  else if( strcmp(strings,"scaler") == 0){
    return scaler_list;
  }
  else if( strcmp(strings,"tof") == 0){
    return tof_list;
  }
  else if( strcmp(strings,"csi_adc_sum") == 0){
    return csi_adc_sum_list;
  }
  else {
    cout << "\tT473Histogram::strToHisot return 0" << endl;
    return (Konoe1DHistogramList*)0;
  }
  
}

T473Histogram::~T473Histogram(){

  delete ui_server0;
  delete ui_socket0;
  delete ui_server1;
  delete ui_socket1;
  delete ui_server2;
  delete ui_socket2;

  delete general_filler ;
  delete csi_tdc_filler ;
  delete csi_adc_filler ;
  delete scin_tdc_filler ;
  delete scin_adc_filler ;
  delete scaler_filler ; 

  delete check; 
  delete setter; 
  
  delete readfds;
  delete log;

  delete xenon;
  delete spill_end;
}

void T473Histogram::lastAction( ){

  delete ui_server0;
  delete ui_socket0;
  delete ui_server1;
  delete ui_socket1;
  delete ui_server2;
  delete ui_socket2;

  delete data_socket;
  log->send("T473Histogram Dump!!!!!!",3);
  cout << "\tT473Histogram Dump!!!!!!";
  setter->set("HISTOGRAM_STATUS","none");
  setter->set("HISTOGRAM_PID",-1);
  KonoeHostSystem::exit(0);

}

