//===================================================
/*
	T473Histogram.hh

	 4-Oct-2000	Y. Shimizu
	 8-Nov-2000	Y. Shimizu
	10-Dec-2000	Y. Shimizu
				Xe event 
*/
//==================================================	

#ifndef __T473HISTORGRAM_HH
#define __T473HISTORGRAM_HH

#include 	"../RunControl/T473Process.hh"
#include        "../RunControl/T473DyingMessage.hh"
#include 	"../Logger/T473LogSender.hh"

#include 	"T473CsIADCHistoFiller.hh"
#include 	"T473CsITDCHistoFiller.hh"
#include 	"T473ScinADCHistoFiller.hh"
#include 	"T473ScinTDCHistoFiller.hh"
#include 	"T473GeneralHistoFiller.hh"
#include 	"T473ScalerHistoFiller.hh"
#include 	"T473TOFExtractor.hh"
#include 	"T473TOFFiller.hh"
#include 	"T473CsIADCSumExtractor.hh"
#include 	"T473CsIADCSumFiller.hh"
#include 	"T473XenonChecker.hh"
#include	"T473SpillEndProcessor.hh"

#include	"../Namespace/T473NameSpaceSetter.hh"
#include	"../Namespace/T473NameSpaceExtractor.hh"
#include	"../RunControl/T473StatusChecker.hh"

#include	<analysis/KonoeDataAnalyzer.hh>
#include	<analysis/KonoeHistoFillerList.hh>
#include	<analysis/KonoeDataRecord.hh>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<analysis/KonoeDefaultSectionAnalyzer.hh>
#include	<analysis/Konoe1DHistogram.hh>
#include	<analysis/Konoe1DHistogramList.hh>

#include	<system/KonoeServerSocket.hh>
#include	<system/KonoeClientSocket.hh>
#include	<system/KonoeInputObjSocketBuffered.hh>
#include	<system/KonoeFDClient.hh>
#include	<system/KonoeFDSet.hh>

#include	<system/KonoeOutputObjFile.hh>
#include	<system/KonoeOutputObjSocket.hh>

class T473Histogram :  T473Process , T473DyingMessage {
public:
  enum {	N_MULTIPLET = T473HistoInfo::N_MULTIPLET };
  enum {	GENERAL = T473HistoInfo::GENERAL_SECTION };
  enum {	CSI_TDC = T473HistoInfo::CSI_TDC_SECTION };
  enum {	CSI_ADC = T473HistoInfo::CSI_ADC_SECTION };
  enum {	SCIN_TDC = T473HistoInfo::SCIN_TDC_SECTION };
  enum {	SCIN_ADC = T473HistoInfo::SCIN_ADC_SECTION };
  enum {	SCALER = T473HistoInfo::SCALER_SECTION };
  enum {        REQ_HEAD = 0 , 
		REQ_CURRENT = 1 ,
		REQ_TAIL = 2 };
protected:
  KonoeDefaultSectionAnalyzer * general_analyzer;
  KonoeDefaultSectionAnalyzer * csi_tdc_analyzer;
  KonoeDefaultSectionAnalyzer * csi_adc_analyzer;
  KonoeDefaultSectionAnalyzer * scin_tdc_analyzer;
  KonoeDefaultSectionAnalyzer * scin_adc_analyzer;
  KonoeDefaultSectionAnalyzer * scaler_analyzer;

  T473GeneralHistoFiller * general_filler;
  T473CsIADCHistoFiller	* csi_adc_filler;
  T473CsITDCHistoFiller	* csi_tdc_filler;
  T473ScinADCHistoFiller	* scin_adc_filler;
  T473ScinTDCHistoFiller	* scin_tdc_filler;
  T473ScalerHistoFiller	* scaler_filler;
  T473TOFFiller		* tof_filler;
  T473CsIADCSumFiller		* csi_adc_sum_filler;

  T473NameSpaceSetter	* setter;
  
  KonoeDataAnalyzer	* analyzer;
  KonoeHistoFillerList	* filler_list;
  KonoeHistoFillerList	* filler_list_xenon;
	
  Konoe1DHistogramList	* general_list;
  Konoe1DHistogramList	* csi_tdc_list;
  Konoe1DHistogramList	* csi_adc_list;
  Konoe1DHistogramList	* csi_adc_list_xenon;
  Konoe1DHistogramList	* scin_tdc_list;
  Konoe1DHistogramList	* scin_adc_list;
  Konoe1DHistogramList	* scaler_list;
  Konoe1DHistogramList	* tof_list;
  Konoe1DHistogramList	* csi_adc_sum_list;

  KonoeDataRecordCM	* d_cm ;
  
  KonoeClientSocket	* data_client;
  KonoeInputObjSocketBuffered	* data_socket;
  
  KonoeServerSocket	* ui_server0;
  KonoeOutputObjSocket	* ui_socket0;
  KonoeServerSocket	* ui_server1;
  KonoeOutputObjSocket	* ui_socket1;
  KonoeServerSocket	* ui_server2;
  KonoeOutputObjSocket	* ui_socket2;
  KonoeServerSocket	* event_num_socket;

  KonoeFDSet		* readfds;
  KonoeFDSet		* readfds_for_hub;
  
  KonoeOutputObjFile	* file1;
  KonoeOutputObjFile	* file2;
  KonoeOutputObjFile	* file3;
  KonoeOutputObjFile	* file4;
  KonoeOutputObjFile	* file6;
  
  T473NameSpaceExtractor	* extractor;
  T473XenonChecker		* xenon;
  T473SpillEndProcessor		* spill_end;
  
  T473StatusChecker	* check;
  T473LogSender		* log;
  
  KInt histogramSet();
  KInt histogramReset();
  KInt informationSet();
  KInt datasocketSet();
  KInt uisocketSet();
  KInt uisocketClose();
  KInt histogramListFill();
  KInt histogramListFill_xenon();
  KInt processFDClient(KonoeFDClient * client);
  KInt pause_process( );
  KInt isValidReq(KChar* req);
  void all_histo_dump();
  void send_0_UI();
  void send_1_UI();
  void send_2_UI();
  void event_num_UI();
  KInt ns_reset();
  Konoe1DHistogramList* strToHisto( KChar* str);

  static const KChar * DATAHUB_REQ_CURRENT;
  static const KChar * DATAHUB_REQ_TAIL;
  static const KChar * DATAHUB_REQ_HEAD;
  
  KInt event_cycle;
  KInt wait_interval;
  KInt data_ext_mode;
  KInt event;
  KBool end_flag;

public:

  T473Histogram(KInt cmd_port = 50005);
  ~T473Histogram();
  
  KInt begin();
  KInt pause();
  KInt resume();
  KInt end();
  KInt shutdown();

  void lastAction();
  
};

#endif
