//====================================================================
/*

        T473SpillEndProcessor.cc

	10-Dec-2000	Y. Shimizu

*/
//====================================================================

#include <analysis/KonoeDataSegment.hh>
#include <analysis/KonoeDataElement.hh>
#include "T473SpillEndProcessor.hh"
#include "T473HistoInfo.hh"
#include "T473Scaler.hh"
#include <stdio.h>
#include <unistd.h>

T473SpillEndProcessor::T473SpillEndProcessor(KonoeDataAnalyzer *pana ,KonoeDefaultSectionAnalyzer* psec_ana)
{
	ana     = pana;
	sec_ana = psec_ana; 
	setter = new T473NameSpaceSetter;

	KInt i;

	for( i = 0 ;  i < SEG0_ELEMENTS ; i++){
		element0[i] = 0;
	}
	for( i = 0 ;  i < SEG1_ELEMENTS ; i++){
		element1[i] = 0;
	}
	
}

T473SpillEndProcessor::~T473SpillEndProcessor( ){

	delete setter;

}

KInt T473SpillEndProcessor::process( KonoeDataRecord* rec )
{
	KInt element0_tmp[SEG0_ELEMENTS];
	KInt element1_tmp[SEG1_ELEMENTS];
	KChar buff[16];
	KInt 	i;

	rec->reset( );
	ana->reset( );
	ana->analyze( rec ); 
	physicalvalue = sec_ana->getPhysicalValue( );

	T473Scaler  * scaler = new T473Scaler( physicalvalue );
	scaler->setIndex(3 ,1 ,2 );
	for( i = 0 ;  i < SEG0_ELEMENTS ; i++){
		element0_tmp[i] = scaler->getCount( 0 , i );
		element0[i] += element0_tmp[i];
	} 
	for( i = 0 ;  i < SEG1_ELEMENTS ; i++){
		element1_tmp[i] = scaler->getCount( 1 , i );
		element1[i] += element1_tmp[i];
	}
	cout << "\n\tprocess spill end"<<endl;
	cout << "element0[11]        (veto) = "<< element0_tmp[11] << endl;
	cout << "element0[10]     (no veto) = "<< element0_tmp[10] << endl;
	sprintf(buff,"%lf",element0_tmp[11]/(KFloat)element0_tmp[10]);
	setter->set("DAQ_EFFICIENCY",buff);

	cout << "element0[11]    (veto sum) = " << element0[11] << endl;
	cout << "element0[10] (no_veto sum) = " << element0[10] << endl;
	sprintf(buff,"%lf",element0[11]/(KFloat)element0[10]);
	setter->set("DAQ_EFFICIENCY_SUM",buff);

	cout << "element0[11]        (veto) = " << element0_tmp[11] << endl;
	cout << "element1[ 2]        (clk2) = " << element1_tmp[2] << endl;
	sprintf(buff,"%lf",1000000.0*element0_tmp[11]/(KFloat)element1_tmp[2]);
	setter->set("DAQ_SPEED",buff);

	cout << "element0[11]    (veto sum) = " << element0[11] << endl;
	cout << "element1[ 2]    (clk2 sum) = " << element1[2] << endl;
	sprintf(buff,"%lf",1000000.0*element0[11]/(KFloat)element1[2]);
	setter->set("DAQ_SPEED_SUM",buff);
	
	delete scaler;


	return (0);
}


KInt T473SpillEndProcessor::clear( ){

	KInt i;
	for( i = 0 ;  i < SEG0_ELEMENTS ; i++){
		element0[i] = 0;
	}
	for( i = 0 ;  i < SEG1_ELEMENTS ; i++){
		element1[i] = 0;
	}

	return (0);
}

