//====================================================================
/*

        T473TOFExtractor.cc

	 6-Oct-2000	Y. Shimizu
	15-Nov-2000	Y. Shimizu

*/
//====================================================================


#include <analysis/KonoePhysicalValue.hh>
#include <analysis/KonoeFillData.hh>
#include <analysis/KonoeFillDataExtractor.hh>
#include "../Namespace/T473NameSpaceExtractor.hh"
#include "T473TOFExtractor.hh"

T473TOFExtractor::T473TOFExtractor(KonoePhysicalValue * pvalue,
						 KChar * name1, 
						 KChar * name2, 
						 KChar * name3 )
{
  KonoeMultipletDescriptor	* d;
  T473NameSpaceExtractor * ext = new T473NameSpaceExtractor;
  KInt	i, j;
  
  for( i = 0; i < FILL_DATA_DIMENSION; i ++ )
    indices[ i ] = -1;
  physicalvalue = pvalue;
  d = pvalue -> getDescriptor( );
  if( name1 )
    indices[ 0 ] = d -> getIndex( name1 );
  if( name2 )
    indices[ 1 ] = d -> getIndex( name2 );
  if( name3 )
    indices[ 2 ] = d -> getIndex( name3 );
  j = 0;
  for( i = 0; i < FILL_DATA_DIMENSION; i ++ )
    if( indices[ i ] == -1 )
      indices[ i ] = j ++;
  
  s1leftped = ext->getKIntValue("SCIN_TDC_0_0_PED");
  if( s1leftped < 0 )
	s1leftped = 0;
  s1rightped = ext->getKIntValue("SCIN_TDC_0_1_PED");
  if( s1rightped < 0 )
	s1rightped = 0;
  s3ped = ext->getKIntValue("SCIN_TDC_0_3_PED");
  if( s3ped < 0 )
	s3ped = 0;
  s1primeped = ext->getKIntValue("SCIN_TDC_1_7_PED");
  if( s1primeped < 0 )
	s1primeped = 0;

  delete ext;
  delete d;
}

KonoeFillData *T473TOFExtractor::next( )
{
  KonoeDataMultiplet	* multiplet;
  
  multiplet = physicalvalue -> next( );
  if( multiplet == 0 )
    return 0;

//#define EXT_DEBUG
#ifdef EXT_DEBUG
  	cout <<  multiplet->getValue( indices[ 0 ] )  << " - "  ;
  	cout <<  multiplet->getValue( indices[ 1 ] )  << " - "  ;
  	cout <<  multiplet->getValue( indices[ 2 ] )  << endl;
#endif

	if(      ( multiplet->getValue( indices[2] ) == S1_LEFT ) 
			&& 
	         ( multiplet->getValue( indices[1] ) == SEGMENT0 ) ){
		s1left = multiplet->getValue( indices[0] ) - s1leftped;
	}	
	else if( ( multiplet->getValue( indices[2] ) == S1_RIGHT ) 
			&& 
	       ( multiplet->getValue( indices[1] ) == SEGMENT0 ) ){
		s1right = multiplet->getValue( indices[0] ) - s1rightped;
	}	
	else if( ( multiplet->getValue( indices[2] ) == S3 ) 
			&& 
	       ( multiplet->getValue( indices[1] ) == SEGMENT0 ) ){
		s3 = multiplet->getValue( indices[0] ) - s3ped;
	}	
	else if( ( multiplet->getValue( indices[2] ) == S1PRIME ) 
			&& 
		 ( multiplet->getValue( indices[1] ) == SEGMENT1 ) ){
		s1prime = multiplet->getValue( indices[0] ) - s1primeped;

 	        return new KonoeFillData( this->calculate(),0,0);

	}	
  
			         
	this->next();

}

KFloat T473TOFExtractor::calculate(){

#ifdef EXT_DEBUG
	cout << " s3 " << s3 << " s1left " << s1left << " s1right " << s1right
		<< endl;
#endif
//	return (   s3 -  (s1left + s1right) / 2.0  );
	return (   s3 -  s1prime  );

}

KInt T473TOFExtractor::setPedestal( ){
	
  T473NameSpaceExtractor * ext = new T473NameSpaceExtractor;

  s1leftped = ext->getKIntValue("SCIN_TDC_0_0_PED");
  if( s1leftped < 0 )
	s1leftped = 0;
  cout << "SCIN_TDC_0_0_PED = " << s1leftped << endl;
  s1rightped = ext->getKIntValue("SCIN_TDC_0_1_PED");
  if( s1rightped < 0 )
	s1rightped = 0;
  cout << "SCIN_TDC_0_1_PED = " << s1rightped << endl;
  s3ped = ext->getKIntValue("SCIN_TDC_0_3_PED");
  if( s3ped < 0 )
	s3ped = 0;
  cout << "SCIN_TDC_0_3_PED = " << s3ped << endl;
  s1primeped = ext->getKIntValue("SCIN_TDC_1_7_PED");
  if( s1primeped < 0 )
	s1primeped = 0;
  cout << "SCIN_TDC_1_7_PED = " << s1primeped << endl;

  delete ext;

  return (0);
}
