// $Id: t473KtoP.cc,v 1.5 2000/12/23 21:33:34 iwai Exp $
//=================================================================
/*
	t473KtoP.cc

	16-Nov-2000	Y. Shimizu
	24-Dec-2000	G. Iwai
*/
//=================================================================
#include "T473Histogram.hh"
#include "T473OfflineFiller.hh"
#include <analysis/KonoeDataRecordCM.hh>
#include <analysis/KonoeDataRecord.hh>
#include <system/KonoeInputObjFileBuffered.hh>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <sys/types.h>

int main(int argc, char** argv)
{
  KBool ALL_EVENT_FILE;
  enum { N_MULTIPLET = T473HistoInfo::N_MULTIPLET };
  enum { GENERAL = T473HistoInfo::GENERAL_SECTION };
  enum { CSI_TDC = T473HistoInfo::CSI_TDC_SECTION };
  enum { CSI_ADC = T473HistoInfo::CSI_ADC_SECTION };
  enum { SCIN_TDC = T473HistoInfo::SCIN_TDC_SECTION };
  enum { SCIN_ADC = T473HistoInfo::SCIN_ADC_SECTION };
  enum { SCALER = T473HistoInfo::SCALER_SECTION };
  enum { DEFAULT, BEGIN_END };
  enum { RUN_INFORMATION = 0, EVENT_NUMBER = 1 };

  KonoeInputObjFileBuffered* ofile;
  KonoeDataRecord* rec;
  
  if ( argc < 2 ) {
    cerr << "usage: t473KtoP <inputfile> [Begin of Event] [End of Event]" 
	 << endl
	 << "       1KonoeDataRecordΥե̾ǤɬɬפǤ" 
	 << endl
	 << "       23ϤäƤʤƤɤǤ" 
	 << endl
	 << "       ׵ˤ椬ɲäΤǤ"
	 << endl;
    exit(0);
  }

  KChar* obj_file_name = *(++argv);

  KInt beginOfEvent;
  KInt endOfEvent;
  KInt writeMode = DEFAULT;
  if ( argc == 4 ) {
    writeMode = BEGIN_END;
    beginOfEvent = atoi( *(++argv) );
    endOfEvent = atoi( *(++argv) );
  }

  cout << "target file = " << obj_file_name << endl;
  
  KonoeDefaultSectionAnalyzer* general_analyzer 
    = new KonoeDefaultSectionAnalyzer(GENERAL,N_MULTIPLET,0);
  KonoeDefaultSectionAnalyzer* csi_tdc_analyzer 
    = new KonoeDefaultSectionAnalyzer(CSI_TDC,N_MULTIPLET,0);
  KonoeDefaultSectionAnalyzer* csi_adc_analyzer 
    = new KonoeDefaultSectionAnalyzer(CSI_ADC,N_MULTIPLET,0);
  KonoeDefaultSectionAnalyzer* scin_tdc_analyzer 
    = new KonoeDefaultSectionAnalyzer(SCIN_TDC,N_MULTIPLET,0);
  KonoeDefaultSectionAnalyzer* scin_adc_analyzer 
    = new KonoeDefaultSectionAnalyzer(SCIN_ADC,N_MULTIPLET,0);
  KonoeDefaultSectionAnalyzer* scaler_analyzer 
    = new KonoeDefaultSectionAnalyzer(SCALER,N_MULTIPLET,0);

  KonoeDataAnalyzer* analyzer 
    = new KonoeDataAnalyzer( T473HistoInfo::N_SECTION );

  analyzer->install( general_analyzer );
  analyzer->install( csi_tdc_analyzer );
  analyzer->install( csi_adc_analyzer );
  analyzer->install( scin_tdc_analyzer );
  analyzer->install( scin_adc_analyzer );
  analyzer->install( scaler_analyzer );
  
  T473GeneralHistoFiller* general_filler 
    = new T473GeneralHistoFiller( general_analyzer );
  T473CsITDCHistoFiller* csi_tdc_filler 
    = new T473CsITDCHistoFiller( csi_tdc_analyzer );
  T473CsIADCHistoFiller* csi_adc_filler 
    = new T473CsIADCHistoFiller( csi_adc_analyzer );
  T473ScinTDCHistoFiller* scin_tdc_filler 
    = new T473ScinTDCHistoFiller( scin_tdc_analyzer );
  T473ScinADCHistoFiller* scin_adc_filler 
    = new T473ScinADCHistoFiller( scin_adc_analyzer );
  T473ScalerHistoFiller* scaler_filler 
    = new T473ScalerHistoFiller( scaler_analyzer );

  KonoeHistoFillerList* filler_list 
    = new KonoeHistoFillerList( T473HistoInfo::HISTO_TOTAL );


  //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  T473OfflineFiller* general[T473HistoInfo::GENERAL_HISTO];
  T473OfflineFiller* csi_tdc[T473HistoInfo::CSI_TDC_HISTO];
  T473OfflineFiller* csi_adc[T473HistoInfo::CSI_ADC_HISTO];
  T473OfflineFiller* scin_tdc[T473HistoInfo::SCIN_TDC_HISTO];
  T473OfflineFiller* scin_adc[T473HistoInfo::SCIN_ADC_HISTO];
  T473OfflineFiller* scaler[T473HistoInfo::SCALER_HISTO];

  //_______________________________________________________________________  

  KonoeOutputFile* event_file;
  KChar event_filename[256];
  strcpy(event_filename, "for_PAW_");
  strcat(event_filename, obj_file_name);
  cout << "output file = " << event_filename <<endl;
  event_file = new KonoeOutputFile(event_filename);

  //-----------------------------------------------------------------------

  KInt i(0);
  KInt j(0);

  /////////////////////////////////////////////////////////////////////////


  for ( i = 0; i < T473HistoInfo::GENERAL_HISTO; i++ ) {
    general_filler -> getExt(i) -> setPedestal( 0 );
    general[i] = new T473OfflineFiller(general_filler->getExt(i),event_file);
    filler_list->add( general[i] );
  }

  for ( i = 0; i < T473HistoInfo::CSI_TDC_HISTO; i++ ) {
    csi_tdc_filler->getExt(i)->setPedestal( 0 );
    csi_tdc[i] = new T473OfflineFiller(csi_tdc_filler->getExt(i),event_file);
    filler_list->add( csi_tdc[i]);
  }

  for ( i = 0; i < T473HistoInfo::CSI_ADC_HISTO; i++ ) {
    csi_adc_filler->getExt(i)->setPedestal( 0 );
    csi_adc[i] = new T473OfflineFiller(csi_adc_filler->getExt(i),event_file);
    filler_list->add( csi_adc[i]);
  }
  
  for ( i = 0; i < T473HistoInfo::SCIN_TDC_HISTO; i++ ) {
    scin_tdc_filler->getExt(i)->setPedestal( 0 );
    scin_tdc[i] = new T473OfflineFiller(scin_tdc_filler->getExt(i),event_file);
    filler_list->add( scin_tdc[i]);
  }
  
  for ( i = 0; i < T473HistoInfo::SCIN_ADC_HISTO; i++ ) {
    scin_adc_filler->getExt(i)->setPedestal( 0 );
    scin_adc[i] = new T473OfflineFiller(scin_adc_filler->getExt(i),event_file);
    filler_list->add( scin_adc[i]);
  }

  for ( i = 0; i < T473HistoInfo::SCALER_HISTO; i++ ) {
    scaler_filler->getExt(i)->setPedestal( 0 );
    scaler[i] = new T473OfflineFiller(scaler_filler->getExt(i),event_file);
    filler_list->add( scaler[i] );
  }

  ofile = new KonoeInputObjFileBuffered( obj_file_name );
  KonoeDataRecordCM* d_cm = new KonoeDataRecordCM;
  ofile -> installCM( d_cm );
  
  KInt event(0);
  KInt eventNumber;

  switch ( writeMode ) {

  case BEGIN_END:
    while (1) {
      rec = (KonoeDataRecord*) ofile -> get();
      eventNumber = *( (KInt*)( rec -> get(GENERAL) -> get(RUN_INFORMATION) 
				-> get(EVENT_NUMBER) -> getStore() ) );
      if ( rec && eventNumber >= beginOfEvent ) {
	if ( rec -> getType() != 4000 ) {

	  analyzer->reset();
	  analyzer->analyze( rec );
	  filler_list -> fill( );
	  
	  event_file -> writeToStream( "\n", strlen("\n") );
	  ++event;
	}
      } else {
	continue;
      }
      if ( eventNumber >= endOfEvent )
	break;
      delete rec;
    }
    break;

    // ============================================================

  case DEFAULT:
  default:
    while (1) {
      rec = (KonoeDataRecord*) ofile -> get();
      if ( rec ) {
	if ( rec->getType() == 1000 || rec->getType() == 3000 ) {
	  //rec -> dump(cout);
	  analyzer->reset();
	  analyzer->analyze( rec );
	  filler_list -> fill( );
	  
	  event_file -> writeToStream( "\n", strlen("\n") );
	  ++event;
	}
      } else {
	break;
      }
      delete rec ;
    }
    break;
  }

  cout << "number of event ( in " << event_filename << " ) = " 
       << event << endl;
 
  delete event_file;
  
  return (0);
  
}
