//=====================================================================
/*

    T473TclHistogramDisplay.cc

    Version 0.0.1   05-Oct-2000 T. Hosoi

*/
//====================================================================

#include <T473TclHistogramDisplay.hh>
#include <Histogram/T4731DHistoCatcher.hh>

#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawCanvasController.hh>
#include <ui/KonoeTclDrawCanvasMouseListener.hh>
#include <ui/KonoeTcl2DrawPanel.hh>
#include <ui/KonoeTcl2DrawCanvas.hh>
#include <ui/KonoeTclScale.hh>
#include <ui/KonoeTclDimension.hh>

#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe1DHistogramList.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoePhysicalValueCM.hh>

#include <system/KonoeClientSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeTime.hh>

#include <stdlib.h>

T473TclHistogramDisplay::T473TclHistogramDisplay(KonoeTclWindow* kwin,
                                        KChar* aHost, KInt aPort,
                                        KChar* hType)
{
  strcpy(histType, hType);
  strcpy(host, aHost);
  port = aPort;
  autoVertical = KTrue;
  repaint = KFalse;

  hist1d = 0;

  catcher = new T4731DHistoCatcher(aHost, aPort);

  balloon = new KonoeTclBalloon();

  selectHistogram();
  this->create1DHistogramPanel(kwin);

  socket = 0;
}

T473TclHistogramDisplay::~T473TclHistogramDisplay()
{
  delete socket;
  delete in;
  subWindow->destroy();
}

void T473TclHistogramDisplay::create1DHistogramPanel(KonoeTclWindow* kwin)
{
  KChar aName[256];
//  subWindow = new KonoeTclSubWindow(kwin, tmpnam(aName), "Histogram Canvas");
  subWindow = kwin;

  Konoe1DHistogram* hist1D = new Konoe1DHistogram(new KonoeLinearDataScale(0,
                                                  100.0, 100));
  hist1D->clear();
  //hist1D->fill(new KonoeFillData(50));
//hist1D->dump(cout);
  drawPanel = new KonoeTclDrawPanel(subWindow, tmpnam(aName));
  drawCanvas = new KonoeTclDrawCanvas(drawPanel, tmpnam(aName),
                                     hist1D,
                                     KonoeTclDimension(400, 400),
                                     "1D Histogram Panel",
                                     new KonoeLinearDataScale(0.0, 100.0, 100));
  drawPanel->setCanvas(drawCanvas, "");

  drawCanvas->setBackGround("white");

  mouseListener = new KonoeTclDrawCanvasMouseListener;
  drawCanvas->setEventProcessor(mouseListener);
  drawPanel->setEventProcessor(this);
  addTimer(drawPanel, 900);
  //addWindowAction(drawPanel);
  mouseListener->addMouseAction(drawCanvas);
  mouseListener->addMouseMotion(drawCanvas);

  if( getenv("T473_HISTOGRAM_DIR") ){
    KChar fullPath[256];
    strcpy(fullPath, getenv("T473_HISTOGRAM_DIR"));
    mouseListener->setSaveFileFullPath(fullPath);
cout << "############## " << fullPath << endl;
  }

  addWindowAction(drawCanvas);

  subWindow->setEventProcessor(this);
  addKeyAction(subWindow);
}

/***********************************************************
//void T473TclHistogramDisplay::create2DHistogramPanel(KonoeTclWindow* kwin)
//{
//  KChar aName[256];
//  subWindow = new KonoeTclSubWindow(kwin, tmpnam(aName), "Histogram Canvas");
//
//  Konoe2DHistogram* hist2D = new Konoe2DHistogram(
//                                   new KonoeLinearDataScale(0.0, 100.0, 100),
//                                   new KonoeLinearDataScale(0.0, 100.0, 100));
//  hist2D->clear();
//
//  draw2DPanel = new KonoeTcl2DrawPanel(subWindow, tmpnam(aName));
//  draw2DCanvas = new KonoeTcl2DrawCanvas(draw2DPanel, tmpnam(aName),
//                                     hist2D,
//                                     KonoeTclDimension(300, 300),
//                                     "2D Histogram Panel",
//                                    new KonoeLinearDataScale(0.0, 50.0, 100));
//  drawPanel->setCanvas(drawCanvas, "");
//  delete hist2D;
//
//  drawCanvas->setEventProcessor(this);
//  addTimer(draw2DCanvas, 1000);
//  addWindowAction(draw2DCanvas);
//  is2DHistogram = 1;
//}
*****************************************************************/

KInt T473TclHistogramDisplay::draw1DHistogram()
{
  hist1d = get1DHistogramFromSocket();

  if(hist1d == 0){
    cerr << "invalid histogram" << endl;
    return 0;
  }

  drawCanvas->setDataSource( hist1d );
  drawCanvas->draw();
  drawCanvas->update();

  //delete hist;
  return 1;
}

Konoe1DHistogram* T473TclHistogramDisplay::get1DHistogramFromSocket()
{
  if(!strlen(histType)){
    if(!socket){
      socket = new KonoeClientSocket(host, port);
      in = new KonoeInputObjSocket(socket);
      in->installCM( new KonoeHistogramCM() );
      in->installCM( new KonoePhysicalValueCM() );
    }
    KChar* command = "ready";
    socket->send(command, strlen(command));
    return (Konoe1DHistogram*) in->get();
  }

  KonoeTime ktime;
  KChar* timetxt = ktime.getTimeInfo();
  KChar cmd[256];
  sprintf(cmd, "%s (%s)", histType, timetxt);
  drawCanvas->setHistogramName(cmd);
  histogramList = catcher->getHistoList(histName);

  if(histogramList){
    histogramList->reset();
    KInt i;
    histCount = histogramList->getCount();

    if(histCount == 0) return 0;

    for(i=0; i<histCount; i++){
      histogram[i] = histogramList->next();
      if(histNum != i && histogram)
        delete histogram[i];
    }

    return histogram[ histNum ];
  }

  return 0;
}

void T473TclHistogramDisplay::selectHistogram()
{
  KInt i;
  if(strstr(histType, "GENERAL")){
    strcpy(histName, "general");
    KChar s[16];
    for(i=1; i<9; i++){
      sprintf(s, "%d", i);
      if(strstr(histType, s)){
        histNum = i-1;
        break;
      }
    }
  }
  else if(strstr(histType, "CsI_ADC")){
    strcpy(histName, "csi_adc");
    KChar s[16];
    for(i=25; i>0; i--){
      sprintf(s, "%d", i);
      if(strstr(histType, s)){
        histNum = i-1;
        break;
      }
    }
    if(strstr(histType, "CsI_ADC_SUM")){
      strcpy(histName, "csi_adc_sum");
      histNum = 0;
    }
  }
  else if(strstr(histType, "CsI_TDC")){
    strcpy(histName, "csi_tdc");
    KChar s[16];
    for(i=25; i>0; i--){
      sprintf(s, "%d", i);
      if(strstr(histType, s)){
        histNum = i-1;
        break;
      }
    }
  }
  else if(strstr(histType, "Scin_ADC")){
    strcpy(histName, "scin_adc");
    KChar s[16];
    for(i=7; i>1; i--){
      sprintf(s, "%d", i);
      if(strstr(histType, s)){
        histNum = i;
        break;
      }
    }
    if(strstr(histType, "L1"))
      histNum = 0;
    if(strstr(histType, "R1"))
      histNum = 1;
    if(strstr(histType, "1P"))
      histNum = 14;
  }
  else if(strstr(histType, "Scin_TDC")){
    strcpy(histName, "scin_tdc");
    KChar s[16];
    for(i=7; i>1; i--){
      sprintf(s, "%d", i);
      if(strstr(histType, s)){
        histNum = i;
        break;
      }
    }
    if(strstr(histType, "L1"))
      histNum = 0;
    if(strstr(histType, "R1"))
      histNum = 1;
    if(strstr(histType, "1P"))
      histNum = 15;
  }
  else if(strstr(histType, "GC_ADC")){
    strcpy(histName, "scin_adc");
    if(strstr(histType, "1"))
      histNum = 8;
    else if(strstr(histType, "2"))
      histNum = 9;
  }
  else if(strstr(histType, "GC_TDC")){
    strcpy(histName, "scin_tdc");
    if(strstr(histType, "1"))
      histNum = 8;
    else if(strstr(histType, "2"))
      histNum = 9;
  }
  else if(strstr(histType, "XENON_ADC")){
    strcpy(histName, "scin_adc");
    if(strstr(histType, "1"))
      histNum = 10;
    else if(strstr(histType, "2"))
      histNum = 11;
    else if(strstr(histType, "3"))
      histNum = 12;
    else if(strstr(histType, "4"))
      histNum = 13;
  }
  else if(strstr(histType, "XENON_TDC")){
    strcpy(histName, "scin_tdc");
    if(strstr(histType, "1"))
      histNum = 11;
    else if(strstr(histType, "2"))
      histNum = 12;
    else if(strstr(histType, "3"))
      histNum = 13;
    else if(strstr(histType, "4"))
      histNum = 14;
  }  
  else if(strstr(histType, "TOF")){
    strcpy(histName, "tof");
    histNum = 0;
  }
//cout << "HistName = " << histName << "   HistNum = " << histNum << endl;
}

KInt T473TclHistogramDisplay::draw2DHistogram()
{
  Konoe2DHistogram* histogram;

  histogram = (Konoe2DHistogram*) in->get();
  if(histogram == 0){
    cerr << "invalid histogram" << endl;
    return 0;
  }

  draw2DCanvas->setDataSource( histogram );
  draw2DCanvas->clearAll();
  draw2DCanvas->draw();
  draw2DCanvas->update();

  delete histogram;
  return 1;
}

void T473TclHistogramDisplay::setAutoVertical()
{
  if( autoVertical ) autoVertical = KFalse;
  else               autoVertical = KTrue;

  KInt i;
  for(i=0; i<25; i++){
    drawCanvas->setAutoVertical();
  }
}

void T473TclHistogramDisplay::setDefault()
{
  KInt i;
  for(i=0; i<25; i++){
    drawCanvas->setHorizontalScale(0, 4096, 4096);
    autoVertical = KTrue;
    drawCanvas->setMark(20);
    drawCanvas->setBigMark(5);
    drawCanvas->getScale()->setMarkTotal(20);
    drawCanvas->getScale()->setBigMarkTotal(5);
  }
}

void T473TclHistogramDisplay::windowAction(KonoeTclCanvas* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
    cout << "DestroyWindow Action called" << endl;
    this->~T473TclHistogramDisplay();
  }
  else{
    if(repaint){
      drawCanvas->setSize( subWindow->getWidth(), subWindow->getHeight() );
      repaint = KFalse;
    }
    else{
      repaint = KTrue;
    }
  }
}

void T473TclHistogramDisplay::keyAction(KonoeTclWindow* kcv,
                                         KInt type, KeySym keysym, KInt state)
{
  if( type == KKeyRelease ){
    if( keysym == XK_v )
      setAutoVertical();
    else if( keysym == XK_Q )
      this->~T473TclHistogramDisplay();
    else if( keysym == XK_d )
      setDefault();
    else if( keysym == XK_s )
      drawCanvas->drawStatus();
  }
}

void T473TclHistogramDisplay::timer()
{
  draw1DHistogram();
}
