//=====================================================================
/*

    T473TclHistogramDisplayController.cc

    Version 0.0.1   05-Oct-2000 T. Hosoi

*/
//====================================================================

#include <T473TclHistogramDisplayController.hh>
#include <T473TclHistogramDisplayControllerAction.hh>
#include <T473TclHistogramDisplay.hh>
#include <T473TclHistogramDisplayCsI25.hh>
#include <T473TclHistogramDisplayScin.hh>
#include <T473TclHistogramDisplayExec.hh>
#include <Logger/T473LogSender.hh>

#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclFrame.hh>
#include <ui/KonoeTclListBox.hh>
#include <ui/KonoeTclButton.hh>

#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <string.h>

T473TclHistogramDisplayController::T473TclHistogramDisplayController
                                       (KonoeTclWindow* kwin)
{
  controlWindow = kwin;


  initWidget();
  insertItem();

  controllerAction = new T473TclHistogramDisplayControllerAction(this);

  requestButton->setEventProcessor(controllerAction);
  controllerAction->addMouseAction(requestButton);

  if(!getenv("NONAMESPACE")){
    connect0->setEventProcessor(controllerAction);
    controllerAction->addMouseAction(connect0);
    connect1->setEventProcessor(controllerAction);
    controllerAction->addMouseAction(connect1);
    connect2->setEventProcessor(controllerAction);
    controllerAction->addMouseAction(connect2);
  }

  T473LogSender* logsend  = new T473LogSender;
  logsend->send("T473histcontroller is executed.", T473LogSender::SUCCESS);
  delete logsend;
}

T473TclHistogramDisplayController::~T473TclHistogramDisplayController()
{ ; }

void T473TclHistogramDisplayController::initWidget()
{
  if(!getenv("NONAMESPACE")){
    KChar connectHist[256];

    strcpy(histHost, nsExtractor.getKCharValue("HISTOGRAM_HOST"));
  
    selectFrame = new KonoeTclFrame(controlWindow, "selectFrame");

    strcpy(histPort[0], nsExtractor.getKCharValue("HISTO_TO_UI_PORT0"));
    sprintf(connectHist, "%s : %s", histHost, histPort[0]);
    connect0 = new KonoeTclRadioButton(selectFrame, "connect0", connectHist);

    strcpy(histPort[1], nsExtractor.getKCharValue("HISTO_TO_UI_PORT1"));
    sprintf(connectHist, "%s : %s", histHost, histPort[1]);
    connect1 = new KonoeTclRadioButton(selectFrame, "connect1", connectHist);

    strcpy(histPort[2], nsExtractor.getKCharValue("HISTO_TO_UI_PORT2"));
    sprintf(connectHist, "%s : %s", histHost, histPort[2]);
    connect2 = new KonoeTclRadioButton(selectFrame, "connect2", connectHist);
    selectFrame->setFrame();
  }

  hostLabel = new KonoeTclLabel(controlWindow, "hostLabel", "Host");
  hostEntry = new KonoeTclEntry(controlWindow, "hostEntry");
  portLabel = new KonoeTclLabel(controlWindow, "portLabel", "Port");
  portEntry = new KonoeTclEntry(controlWindow, "portEntry");

  listFrame = new KonoeTclFrame(controlWindow, "listFrame");
  histList = new KonoeTclListBox(listFrame, "histList", KVertical);
  histList->setMode("browse");


//  listFrame->setLayoutType("left");
  listFrame->setFrame();

  requestButton = new KonoeTclButton(controlWindow, "requestButton", "Request");

  hostEntry->setBackGround("white");
  portEntry->setBackGround("white");

  listFrame->setExpand();
  listFrame->fillBoth();
  histList->setExpand();
  histList->fillBoth();
  histList->setBackGround("white");
  histList->setSelectBackGround("blue");
  histList->setSelectForeGround("white");

  //requestButton->setExpand();
}

KInt T473TclHistogramDisplayController::getSelectedIndex()
{
  return atoi(histList->getSelectedIndex());
}

KChar* T473TclHistogramDisplayController::getHostName()
{
  KChar host[256];
  if(strlen(hostEntry->getText())) strcpy(host, hostEntry->getText());
  else strcpy(host, "konoe3");
  return strdup(host);
}

KInt T473TclHistogramDisplayController::getPortNumber()
{
  KInt port;

  if(strlen(portEntry->getText())) port = atoi(portEntry->getText());
  else 12000;
  return port;
}

void T473TclHistogramDisplayController::insertItem()
{
  histList->insert("CsI TDC");
  histList->insert("CsI ADC");
  histList->insert("Scintillator TDC");
  histList->insert("Scintillator ADC");
  histList->insert("TOF");
  histList->insert("CsI ADC SUM");
  histList->insert("CsI ADC XENON");
  histList->insert("--------------------------------------");
  histList->insert("CsI TDC 1");
  histList->insert("CsI TDC 2");
  histList->insert("CsI TDC 3");
  histList->insert("CsI TDC 4");
  histList->insert("CsI TDC 5");
  histList->insert("CsI TDC 6");
  histList->insert("CsI TDC 7");
  histList->insert("CsI TDC 8");
  histList->insert("CsI TDC 9");
  histList->insert("CsI TDC 10");
  histList->insert("CsI TDC 11");
  histList->insert("CsI TDC 12");
  histList->insert("CsI TDC 13");
  histList->insert("CsI TDC 14");
  histList->insert("CsI TDC 15");
  histList->insert("CsI TDC 16");
  histList->insert("CsI TDC 17");
  histList->insert("CsI TDC 18");
  histList->insert("CsI TDC 19");
  histList->insert("CsI TDC 20");
  histList->insert("CsI TDC 21");
  histList->insert("CsI TDC 22");
  histList->insert("CsI TDC 23");
  histList->insert("CsI TDC 24");
  histList->insert("CsI TDC 25");
  histList->insert("CsI ADC 1");
  histList->insert("CsI ADC 2");
  histList->insert("CsI ADC 3");
  histList->insert("CsI ADC 4");
  histList->insert("CsI ADC 5");
  histList->insert("CsI ADC 6");
  histList->insert("CsI ADC 7");
  histList->insert("CsI ADC 8");
  histList->insert("CsI ADC 9");
  histList->insert("CsI ADC 10");
  histList->insert("CsI ADC 11");
  histList->insert("CsI ADC 12");
  histList->insert("CsI ADC 13");
  histList->insert("CsI ADC 14");
  histList->insert("CsI ADC 15");
  histList->insert("CsI ADC 16");
  histList->insert("CsI ADC 17");
  histList->insert("CsI ADC 18");
  histList->insert("CsI ADC 19");
  histList->insert("CsI ADC 20");
  histList->insert("CsI ADC 21");
  histList->insert("CsI ADC 22");
  histList->insert("CsI ADC 23");
  histList->insert("CsI ADC 24");
  histList->insert("CsI ADC 25");
  histList->insert("Scintillator TDC S1 L");
  histList->insert("Scintillator TDC S1 R");
  histList->insert("Scintillator TDC S1 P");
  histList->insert("Scintillator TDC S2");
  histList->insert("Scintillator TDC S3");
  histList->insert("Scintillator TDC S4");
  histList->insert("Scintillator TDC S5");
  histList->insert("Scintillator TDC S6");
  histList->insert("Scintillator TDC S7");
  histList->insert("Scintillator ADC S1 L");
  histList->insert("Scintillator ADC S1 R");
  histList->insert("Scintillator ADC S1 P");
  histList->insert("Scintillator ADC S2");
  histList->insert("Scintillator ADC S3");
  histList->insert("Scintillator ADC S4");
  histList->insert("Scintillator ADC S5");
  histList->insert("Scintillator ADC S6");
  histList->insert("Scintillator ADC S7");
  histList->insert("Gas Cherenkov TDC 1");
  histList->insert("Gas Cherenkov TDC 2");
  histList->insert("Gas Cherenkov ADC 1");
  histList->insert("Gas Cherenkov ADC 2");
  histList->insert("Xenon ADC 1");
  histList->insert("Xenon ADC 2");
  histList->insert("Xenon ADC 3");
  histList->insert("Xenon ADC 4");
  histList->insert("Xenon TDC 1");
  histList->insert("Xenon TDC 2");
  histList->insert("Xenon TDC 3");
  histList->insert("Xenon TDC 4");
}

void T473TclHistogramDisplayController::setConnect(KInt n)
{
  enum { CONNECT0, CONNECT1, CONNECT2 };
  switch(n){
    case CONNECT0:
      setEntry(histHost, histPort[0]);
      break;
    case CONNECT1:
      setEntry(histHost, histPort[1]);
      break;
    case CONNECT2:
      setEntry(histHost, histPort[2]);
      break;
    default : ;
  }
}

void T473TclHistogramDisplayController::setEntry(KChar* h, KChar* p)
{
  hostEntry->setText(h);
  portEntry->setText(p);
}
