//=====================================================================
/*

    T473TclHistogramDisplayCsI25.cc

    Version 0.0.1   21-Oct-2000 T. Hosoi

*/
//====================================================================

#include <T473TclHistogramDisplayCsI25.hh>
#include <T473TclHistogramDisplayOption.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawCanvasMouseListener.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclScale.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe1DHistogramBand.hh>
#include <analysis/Konoe1DHistogramList.hh>
#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoePhysicalValueCM.hh>
#include <analysis/KonoeLinearDataScale.hh>

#include <system/KonoeClientSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjFile.hh>
#include <stdlib.h>
#include <unistd.h>

#include <Histogram/T473HistoInfo.hh>
#include <Histogram/T4731DHistoCatcher.hh>

#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclScaleVertical.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/Konoe1DHistogramBand.hh>

T473TclHistogramDisplayCsI25::T473TclHistogramDisplayCsI25(KonoeTclWindow* kwin,
                                   KChar* pHost, KInt pPort, KChar* phistType)
{
  strcpy(aHost, pHost);
  aPort = pPort;
  autoVertical = KTrue;
  option = 0;
  histogramList = 0;
  strcpy(histType, phistType);
  strcpy(backColor, "white");
  drawFrame = new KonoeTclFrame*[5];
  drawPanel = new KonoeTclDrawPanel*[25];
  drawCanvas = new KonoeTclDrawCanvas*[25];

  KInt i;
  for(i=0; i<25; i++){
    dummyHisto[i]
       = new Konoe1DHistogram(new KonoeLinearDataScale(0.0, 100.0, 100));
    dummyHisto[i]->clear();
  }

  //for(i = 0; i < 1; i++ )
  //dummyHisto[i]->fill( new KonoeFillData( 50.0 ) );
  drawPanel[0] = 0;
  drawCanvas[0] = 0;

  catcher = new T4731DHistoCatcher(aHost, aPort);

  KInt j;
  KInt n;
  KChar wname[256];
  KChar wtitle[256];

  if(strcmp(histType, "csi_tdc") == 0){
    sprintf(wtitle, "CsI TDC (%s : %d)", aHost, aPort);
    strcpy(histTypeName, "TDC");
    //subWindow = new KonoeTclSubWindow(kwin, tmpnam(wname), wtitle);
    subWindow = kwin;
  }
  else if(strcmp(histType, "csi_adc") == 0){
    sprintf(wtitle, "CsI ADC (%s : %d)", aHost, aPort);
    strcpy(histTypeName, "ADC");
    //subWindow = new KonoeTclSubWindow(kwin, tmpnam(wname), wtitle);
    subWindow = kwin;
  }
  else if(strcmp(histType, "csi_adc_xenon") == 0){
    sprintf(wtitle, "CsI ADC XENON (%s : %d)", aHost, aPort);
    strcpy(histTypeName, "ADC XENON");
    subWindow = kwin;
  }

  for(j=5; j>0; j--){
    drawFrame[j-1] = new KonoeTclFrame(subWindow, tmpnam(wname));

    for(i=1; i<=5; i++){
      n = 5*(j-1)+i;
      drawPanel[n-1] = new KonoeTclDrawPanel(drawFrame[j-1], tmpnam(wname));
      KChar tname[256];
      sprintf(tname, "CsI %s %d", histTypeName, n);
      drawCanvas[n-1] = new KonoeTclDrawCanvas(drawPanel[n-1], tmpnam(wname),
                                   dummyHisto[n-1],
                                   KonoeTclDimension(70, 70),
                                   tname,
                                   new KonoeLinearDataScale(0.0, 100.0, 100));
      drawCanvas[n-1]->setBackGround(backColor);
      drawCanvas[n-1]->setRelief(KRidge);
      drawCanvas[n-1]->setBorderWidth(2);
      drawCanvas[n-1]->getScale()->setMarkTotal(10);
      drawCanvas[n-1]->getScale()->setBigMarkTotal(2);
      drawPanel[n-1]->setLayoutType("left");
      drawPanel[n-1]->setCanvas(drawCanvas[n-1], tname);
    }
    drawFrame[j-1]->setFrame();
  }

  cMouseListener= new KonoeTclDrawCanvasMouseListener;
  for(i=0; i<25; i++){
    drawCanvas[i]->setEventProcessor(cMouseListener);
    cMouseListener->addMouseAction(drawCanvas[i]);
    cMouseListener->addMouseMotion(drawCanvas[i]);
  }

  if( getenv("T473_HISTOGRAM_DIR") ){
    strcpy(fullPath, getenv("T473_HISTOGRAM_DIR"));
    cMouseListener->setSaveFileFullPath(fullPath);
  }

  drawPanel[0]->setEventProcessor(this);
  addTimer(drawPanel[0], 1000);

  subWindow->setEventProcessor(this);
  addWindowAction(subWindow);
  addKeyAction(subWindow);
  drawHistogram();
}

T473TclHistogramDisplayCsI25::~T473TclHistogramDisplayCsI25()
{
  KInt i;
  for(i=0; i<25; i++){
    delete drawCanvas[i];
    delete drawPanel[i];
  }
  for(i=0; i<5; i++){
    delete drawFrame[i];
  }
  delete [] drawCanvas;
  delete [] drawPanel;
  delete [] drawFrame;

  delete option;

  delete cMouseListener;
  subWindow->destroy();
  delete subWindow;
  removeTimer();
}

KInt T473TclHistogramDisplayCsI25::drawHistogram()
{
  Konoe1DHistogramList* h;

  if( h = catcher->getHistoList(histType) ){
    delete histogramList;
    histogramList = h;
    histogramList->reset();

    KInt i;
    for( i = 0; i < histogramList->getCount(); i++){
      Konoe1DHistogram* hist;
      hist = histogramList->next();
      KonoeLinearDataScale* scale =
           new KonoeLinearDataScale( hist->getScale()->getMinimum(),
                                     hist->getScale()->getMaximum(),
                                     hist->getScale()->getBinCount() );
      Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
      histogram[i] = band->getNewHisto( hist );

      if( !histogram[i] ) return 0;

      drawCanvas[i]->setDataSource(histogram[i]);
      drawCanvas[i]->setMark(10);
      drawCanvas[i]->setBigMark(2);
      drawCanvas[i]->getScale()->setMarkTotal(10);
      drawCanvas[i]->getScale()->setBigMarkTotal(2);
      drawCanvas[i]->draw();
    }
    if(drawCanvas[1])
    drawCanvas[1]->update();
    return 1;
  }
  else{
    cout << "Histogram is empty" << endl;
    return 0;
  }

  //delete histogramList;
}

void T473TclHistogramDisplayCsI25::setVerticalScale()
{
  KInt i;
  KFloat max = 0.;
  for(i=0; i<25; i++){
    if(max < drawCanvas[i]->getDataScale()->getMaximum())
      max = drawCanvas[i]->getDataScale()->getMaximum();
  }

  KFloat min = drawCanvas[0]->getDataScale()->getMinimum();
  KInt   bin = drawCanvas[0]->getDataScale()->getBinCount();

  if(autoVertical)
  for(i=0; i<25; i++){
    drawCanvas[i]->setVerticalScale( new KonoeLinearDataScale(min, max, bin) );
  }
}

void T473TclHistogramDisplayCsI25::setAutoVertical()
{
  if( autoVertical ) autoVertical = KFalse;
  else               autoVertical = KTrue;

  KInt i;
  for(i=0; i<25; i++){
    drawCanvas[i]->setAutoVertical();
  }
}

void T473TclHistogramDisplayCsI25::setDefault()
{
  KInt i;
  for(i=0; i<25; i++){
    drawCanvas[i]->setHorizontalScale(0, 4096, 4096);
    autoVertical = KTrue;
    drawCanvas[i]->getScale()->setMarkTotal(10);
    drawCanvas[i]->getScale()->setBigMarkTotal(2);
  }
}

void T473TclHistogramDisplayCsI25::setOption( KInt x, KInt y, KInt vmax,
                                               KInt hmin, KInt hmax, KInt hbin,
                                               KInt mark, KInt bigMark,
                                               KInt histType, KChar* histColor )
{
  KInt i;
  for(i=0; i<25; i++){
    autoVertical = KFalse;
    drawCanvas[i]->setDimension( KonoeTclDimension(x-70,y-70) );
    drawCanvas[i]->setSize( x,y );

    drawCanvas[i]->setVerticalScale(new KonoeLinearDataScale(0, vmax, vmax));

    drawCanvas[i]->setHorizontalScale( hmin, hmax, hbin );

    drawCanvas[i]->setMark( mark );
    drawCanvas[i]->setBigMark( bigMark );

    switch(histType){
      case BOX:
        drawCanvas[i]->enableBox();
        break;
      case FILL:
        drawCanvas[i]->enableFill();
        break;
      case LINE:
        drawCanvas[i]->enableLine();
        break;
    }

    drawCanvas[i]->setHistoColor( histColor );

    drawCanvas[i]->draw();
  }
}

void T473TclHistogramDisplayCsI25::saveToFile(KChar* fName)
{
  KChar value[256];
  if(strlen(fullPath) > 0)
    sprintf(value, "%s/%s", fullPath, fName);
  else
    sprintf(value, "%s", fName);

  KonoeOutputObjFile* oFile = new KonoeOutputObjFile( value );

  if( histogramList ){
    histogramList->reset();
    oFile->put( histogramList );
  }
  else
    cout << "Could not save to " << value << "." << endl;

  delete oFile;
}

KInt T473TclHistogramDisplayCsI25::saveAsXWDButtonAction(KChar* pfName)
{
  if( drawCanvas[0] ){
    option->getWindow()->hide();
    option->getWindow()->after(1000);
    drawCanvas[0]->toXWD(pfName);
    option->getWindow()->show();
  }
  return 0;
}

void T473TclHistogramDisplayCsI25::windowAction(
                                   KonoeTclWindow* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
    this->~T473TclHistogramDisplayCsI25();
  }
}

void T473TclHistogramDisplayCsI25::keyAction(KonoeTclWindow* kcv,
                                         KInt type, KeySym keysym, KInt state)
{
  if( type == KKeyRelease ){
    if( keysym == XK_v )
      setAutoVertical();
    else if( keysym == XK_Q )
      this->~T473TclHistogramDisplayCsI25();
    else if( keysym == XK_d )
      setDefault();
    else if( keysym == XK_O ){
      option = new T473TclHistogramDisplayOption(subWindow, this);
    }
  }
}

void T473TclHistogramDisplayCsI25::timer()
{
static KInt i=0;
i++;
//cout << "#########  " << i << endl;
  setVerticalScale();
  drawHistogram();
}
