//=====================================================================
/*

    T473TclHistogramDisplayScin.cc

    Version 0.0.1   15-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclHistogramDisplayScin.hh>
#include <T473TclHistogramDisplayOption.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawCanvasMouseListener.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclScale.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe1DHistogramBand.hh>
#include <analysis/Konoe1DHistogramList.hh>
#include <analysis/KonoeLinearDataScale.hh>

#include <Histogram/T473HistoInfo.hh>
#include <Histogram/T4731DHistoCatcher.hh>

#include <system/KonoeOutputObjFile.hh>

#include <stdlib.h>

T473TclHistogramDisplayScin::T473TclHistogramDisplayScin(KonoeTclWindow* kwin,
                                 KChar* pHost, KInt pPort, KChar* phistType)
{
  strcpy(aHost, pHost);
  aPort = pPort;
  autoVertical = KTrue;
  option = 0;
  histogramList = 0;
  strcpy(histType, phistType);
  strcpy(backColor, "white");
  drawFrame = new KonoeTclFrame*[3];
  drawPanel = new KonoeTclDrawPanel*[11];
  drawCanvas = new KonoeTclDrawCanvas*[11];

  KInt i;
  for(i=0; i<11; i++){
    dummyHisto[i]
       = new Konoe1DHistogram(new KonoeLinearDataScale(0.0, 100.0, 100));
    dummyHisto[i]->clear();
  }

  //for(i = 0; i < 1; i++ )
  //dummyHisto[i]->fill( new KonoeFillData( 50.0 ) );
  drawPanel[0] = 0;
  drawCanvas[0] = 0;

  catcher = new T4731DHistoCatcher(aHost, aPort);

  KInt j = 0;
  KInt n;
  KChar wname[256];
  KChar wtitle[256];

  if(strcmp(histType, "scin_tdc") == 0){
    sprintf(wtitle, "Scintillator TDC (%s : %d)", aHost, aPort);
    strcpy(histTypeName, "TDC");
    //subWindow = new KonoeTclSubWindow(kwin, tmpnam(wname), wtitle);
    subWindow = kwin;
  }
  else{
    sprintf(wtitle, "Scintillator ADC (%s : %d)", aHost, aPort);
    strcpy(histTypeName, "ADC");
    //subWindow = new KonoeTclSubWindow(kwin, tmpnam(wname), wtitle);
    subWindow = kwin;
  }

  for(n=0; n<3; n++)
    drawFrame[n] = new KonoeTclFrame(subWindow, tmpnam(wname));

  for(i=0; i<11; i++){
    drawPanel[i] = new KonoeTclDrawPanel(drawFrame[j], tmpnam(wname));
    KChar tname[256];
    if(i==0)sprintf(tname, "Scin L1 %s", histTypeName );
    else if(i==1)sprintf(tname, "Scin R1 %s", histTypeName);
    else if(i==8) sprintf(tname, "GC1 %s", histTypeName);
    else if(i==9) sprintf(tname, "GC2 %s", histTypeName);
    else if(i==10) sprintf(tname, "Scin 1\' %s", histTypeName);
    else sprintf(tname, "Scin %d %s", i, histTypeName);
    drawCanvas[i] = new KonoeTclDrawCanvas(drawPanel[i], tmpnam(wname),
                                   dummyHisto[i],
                                   KonoeTclDimension(70, 70),
                                   tname,
                                   new KonoeLinearDataScale(0.0, 100.0, 100));
    drawCanvas[i]->setBackGround(backColor);
    drawCanvas[i]->setBorderWidth(2);
    drawCanvas[i]->setRelief(KRidge);
    drawCanvas[i]->getScale()->setMarkTotal(10);
    drawCanvas[i]->getScale()->setBigMarkTotal(2);
    drawPanel[i]->setLayoutType("left");
    drawPanel[i]->setCanvas(drawCanvas[i], tname);
    if(i==3 || i==7) j++;
  }

  for(n=0; n<3; n++)
    drawFrame[n]->setFrame();

  cMouseListener= new KonoeTclDrawCanvasMouseListener;
  for(i=0; i<11; i++){
    drawCanvas[i]->setEventProcessor(cMouseListener);
    cMouseListener->addMouseAction(drawCanvas[i]);
    cMouseListener->addMouseMotion(drawCanvas[i]);
  }

  if( getenv("T473_HISTOGRAM_DIR") ){
    strcpy(fullPath, getenv("T473_HISTOGRAM_DIR"));
    cMouseListener->setSaveFileFullPath(fullPath);
  }

  drawPanel[0]->setEventProcessor(this);
  addTimer(drawPanel[0], 2000);

  subWindow->setEventProcessor(this);
  addWindowAction(subWindow);
  addKeyAction(subWindow);

  drawHistogram();
}

T473TclHistogramDisplayScin::~T473TclHistogramDisplayScin()
{
  KInt i;
  for(i=0; i<11; i++){
    delete drawCanvas[i];
    delete drawPanel[i];
  }
  for(i=0; i<3; i++){
    delete drawFrame[i];
  }
  delete [] drawCanvas;
  delete [] drawPanel;
  delete [] drawFrame;

  delete option;

  delete cMouseListener;
  subWindow->destroy();
  delete subWindow;
  removeTimer();
}

KInt T473TclHistogramDisplayScin::drawHistogram()
{
  Konoe1DHistogramList* h;

  if( h = catcher->getHistoList(histType) ){

    //if( histogramList ){
    //  for( i=0; i < histogramList->getCount(); i++)
    //    if(9<i && i!=histogramList->getCount()-1) delete histogram[i];
    //}
    delete histogramList;
    histogramList = h;
    histogramList->reset();

    KInt i;
    for( i = 0; i < histogramList->getCount(); i++){
      Konoe1DHistogram* hist;
      hist = histogramList->next();
      KonoeLinearDataScale* scale =
           new KonoeLinearDataScale( hist->getScale()->getMinimum(),
                                     hist->getScale()->getMaximum(),
                                     hist->getScale()->getBinCount() );
      Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
      histogram[i] = band->getNewHisto( hist );

      if( !histogram[i] ) return 0;
    //histogram[i]->dump(cout);

      if(i>9 && i!=histogramList->getCount()-1) delete histogram[i];
      else if(i==histogramList->getCount()-1){
        drawCanvas[10]->setDataSource(histogram[i]);
        drawCanvas[10]->getScale()->setMarkTotal(10);
        drawCanvas[10]->getScale()->setBigMarkTotal(2);
        drawCanvas[10]->draw();
      }
      else{
        drawCanvas[i]->setDataSource(histogram[i]);
        drawCanvas[i]->getScale()->setMarkTotal(10);
        drawCanvas[i]->getScale()->setBigMarkTotal(2);
        drawCanvas[i]->draw();
      }
    }
    if(drawCanvas[1])
    drawCanvas[1]->update();
    return 1;
  }
  else {
    cout << "Histogram is empty" << endl;
    return 0;
  }
}

void T473TclHistogramDisplayScin::setVerticalScale()
{
  KInt i;
  KFloat max = 0.;
  for(i=0; i<11; i++){
    if(max < drawCanvas[i]->getDataScale()->getMaximum())
      max = drawCanvas[i]->getDataScale()->getMaximum();
  }

  KFloat min = drawCanvas[0]->getDataScale()->getMinimum();
  KInt   bin = drawCanvas[0]->getDataScale()->getBinCount();

  if(autoVertical)
  for(i=0; i<11; i++){
    drawCanvas[i]->setVerticalScale( new KonoeLinearDataScale(min, max, bin));
  }
}

void T473TclHistogramDisplayScin::setAutoVertical()
{
  if( autoVertical ) autoVertical = KFalse;
  else               autoVertical = KTrue;

  KInt i;
  for(i=0; i<11; i++){
    drawCanvas[i]->setAutoVertical();
  }
}

void T473TclHistogramDisplayScin::setDefault()
{
  KInt i;
  for(i=0; i<11; i++){
    drawCanvas[i]->setHorizontalScale(0, 4096, 4096);
    autoVertical = KTrue;
    drawCanvas[i]->setMark(10);
    drawCanvas[i]->setBigMark(2);
    drawCanvas[i]->getScale()->setMarkTotal(10);
    drawCanvas[i]->getScale()->setBigMarkTotal(2);
  }
}

void T473TclHistogramDisplayScin::setOption( KInt x, KInt y, KInt vmax,
                                             KInt hmin, KInt hmax, KInt hbin,
                                             KInt mark, KInt bigMark,
                                             KInt histType, KChar* histColor )
{
  KInt i;
  for(i=0; i<11; i++){
    autoVertical = KTrue;
    drawCanvas[i]->setDimension( KonoeTclDimension(x-70,y-70) );
    drawCanvas[i]->setSize( x,y );

    drawCanvas[i]->setVerticalScale(new KonoeLinearDataScale(0, vmax, vmax));

    drawCanvas[i]->setHorizontalScale( hmin, hmax, hbin );

    drawCanvas[i]->setMark( mark );
    drawCanvas[i]->setBigMark( bigMark );

    switch(histType){
      case BOX:
        drawCanvas[i]->enableBox();
        break;
      case FILL:
        drawCanvas[i]->enableFill();
        break;
      case LINE:
        drawCanvas[i]->enableLine();
        break;
    }

    drawCanvas[i]->setHistoColor( histColor );

    drawCanvas[i]->draw();
  }
}

void T473TclHistogramDisplayScin::saveToFile(KChar* fName)
{
  KChar value[256];
  if( strlen(fullPath) > 0)
    sprintf(value, "%s/%s", fullPath, fName);
  else
    sprintf(value, "%s", fName);

  KonoeOutputObjFile* oFile = new KonoeOutputObjFile( value );

  if(histogramList)
    oFile->put( histogramList );
  else
    cout << "Could not save to " << value << "." << endl;

  delete oFile;
}

KInt T473TclHistogramDisplayScin::saveAsXWDButtonAction(KChar* pfName)
{
  KChar fName[256];
  if( drawCanvas[0] ){
    option->getWindow()->hide();
    option->getWindow()->after(1000);
    drawCanvas[0]->toXWD(pfName);
    option->getWindow()->show();
  }
  return 0;
}

void T473TclHistogramDisplayScin::windowAction(
                                   KonoeTclWindow* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
    this->~T473TclHistogramDisplayScin();
  }
}

void T473TclHistogramDisplayScin::keyAction(KonoeTclWindow* kcv,
                                         KInt type, KeySym keysym, KInt state)
{
  if( type == KKeyRelease ){
    if( keysym == XK_v )
      setAutoVertical();
    else if( keysym == XK_Q )
      this->~T473TclHistogramDisplayScin();
    else if( keysym == XK_d )
      setDefault();
    else if( keysym == XK_O ){
      option = new T473TclHistogramDisplayOption(subWindow, this);
    }
  }
}

void T473TclHistogramDisplayScin::timer()
{
static KInt i=0;
i++;
//cout << "$$$$$$$$$$  " << i << endl;
  drawHistogram();
  setVerticalScale();
}
