//===========================================================
/*
	T473Logger.cc

	 3-Oct-2000	Y. Shimizu
	12-Oct-2000	Y. Shimizu

		text mode only
*/
//===========================================================

#include	"T473Logger.hh"
#include	"../Namespace/T473NameSpaceSetter.hh"
#include	"../Namespace/T473NameSpaceExtractor.hh"
#include	<system/KonoeHostSystem.hh>
#include	<system/KonoeLoggerCM.hh>
#include	<system/KonoeLogMessage.hh>
#include	<system/KonoeFDClient.hh>
#include	<system/KonoeLock.hh>
#include	<stdio.h>
#include	<string.h>
#include	<stdlib.h>

T473Logger::T473Logger( KChar* filename , KInt pport ,KInt cmd_port) :
		KonoeLogger( filename , pport ) , 
		T473Process( cmd_port ) {

	KInt pid;
	KChar host[256];

	checker = new T473StatusChecker("LOGGER");
 
	pid = KonoeHostSystem::getPid();
	KonoeHostSystem::getHostName(host,256) ;
	ext = new T473NameSpaceExtractor();
	setter  = new T473NameSpaceSetter();
	setter->set("LOGGER_HOST",host);
	setter->set("LOGGER_PID",pid);

	fdset = new KonoeFDSet;
	fdset->install(this);

	NEW_FILE = KTrue;
	mode = IDLE ;
	file = 0;
}

T473Logger::~T473Logger( ){

	if( file ) 
	  delete file;
	delete ext;
	delete checker;

}
	
KInt T473Logger::begin( ){

	mode = READY;

	setter->set("LOGGER_STATUS","busy");


	checker->setTrue();
	cout << "\tT473Logger::begin( )"<<endl;

	KonoeLock lock("/tmp/lock.key");

	KChar logfilename[256];
	KChar logtxt[256];
	KonoeFDClient	* cl;

	if( NEW_FILE ){
	   if( !(ext->getKCharValue("LOGFILE_NAME") == (KChar*)-1)){
		if( getenv("T473_LOGGER_DIR") ){
		  strcpy(logfilename,getenv("T473_LOGGER_DIR"));
		  strcat(logfilename,"/");
		  strcat(logfilename,ext->getKCharValue("LOGFILE_NAME"));
		}
		else {
		  strcpy(logfilename,ext->getKCharValue("LOGFILE_NAME"));
		}
			
	   }
	   else {
		strcpy(logfilename,filename);
	   }

        }

	cout << "\tT473LOGGER::begin LOGFILE_NAME = " << logfilename << endl;
	file = new KonoeOutputFile( logfilename );

	while( mode != IDLE && mode != STANDBY){
 
          cl = fdset->wait(10);
	  if( cl ){
	    lock.lock();
	    receive( logtxt , 256 );
	    cout << logtxt << endl;
	    file->writeToStream(logtxt,strlen( logtxt ));
	    lock.unLock();
	  }	


        }
	if( mode == IDLE )
	   pause();
	if( mode == STANDBY )
	   end();
	return (0);
}

KInt T473Logger::pause( ){

	setter->set("LOGGER_STATUS","idle");
	checker->setTrue();

	NEW_FILE = KFalse;

	cout << "\tT473Logger::pause( )" << endl;
	while( mode != READY && mode != STANDBY ){
		handler->wait();
	}
	if( mode == READY )
		begin();
	if( mode == STANDBY )
		end();
 
	return (0);

}

KInt T473Logger::resume( ){

	setter->set("LOGGER_STATUS","busy");
	checker->setTrue();
cout << "\tT473Logger::resume( )"<<endl;

	while( mode != IDLE ){
		handler->wait();
	}
	begin();
	
	return (0);
}

KInt T473Logger::end( ){

	setter->set("LOGGER_STATUS","standby");
	checker->setTrue();

	NEW_FILE = KTrue;

cout << "\tT473Logger::end( )" << endl;

	if( file )
	  delete file;
	file = 0;
	

	while( mode != NONE && mode != READY ){
		handler->wait();
	}
	if( mode == NONE )
		shutdown();
	if( mode == READY )
		begin();

	return (0);
}

KInt T473Logger::shutdown( ){

	setter->set("LOGGER_STATUS","none");
	setter->set("LOGGER_PID",-1);
	checker->setTrue();

	cout << "\tT473Logger::shutdown( )"<<endl;
	cout << "\tbye!"<<endl;

	sleep(1);
	return (0);
}

void T473Logger::lastAction( ){

	setter->set("LOGGER_STATUS","none");
	setter->set("LOGGER_PID",-1);
	cout << "LOGGER broken !!! " << endl;
	sleep(1);
	exit(0);

}
