//======================================================================
/*
               T473AnalyzerMaker.cc
	       23-Oct-2000  Kirika Uchida
*/
//======================================================================
#include "T473AnalyzerMaker.hh"
#include <analysis/KonoeDetector.hh>
#include "T473CalibrationInfoMaker.hh"
#include "T473GeneralData.hh"
T473AnalyzerMaker::T473AnalyzerMaker
( KonoeDetector * detector, 
  T473GeneralData * pgeneral_data,  
  KonoeHitDataset * phitset, 
  KonoeList<T473CalibrationInfo> * ptdc_info_list, 
  KonoeList<T473CalibrationInfo> * padc_info_list, 
  KChar * preq_type,  
  KonoeNameSpaceTable * table ){
  hitset = phitset;
  general_data = pgeneral_data;
 //  TDC ADC calibrator
  T473CalibrationInfo * info;
  KonoeDataAddress    * address;
  KInt                 test = 0;
  tdc_info_list = ptdc_info_list;
  adc_info_list = padc_info_list;
  adc_info_list -> reset();
  
  // detector -> dump( cout );
  tdc_cal = new T473TDCCalibrator( tdc_info_list );

  adc_cal = new T473ADCCalibrator( adc_info_list );
  
  analyzer =         new KonoeDataAnalyzer( N_SECTION );
  
  general_analyzer = new T473GeneralSectionAnalyzer
    ( GENERAL, general_data, preq_type );
  csi_tdc_analyzer = new KonoeHitSectionAnalyzer
    ( CSI_TDC, detector, tdc_cal, hitset );
  csi_adc_analyzer = new KonoeHitSectionAnalyzer
    ( CSI_ADC, detector, adc_cal, hitset );
  a_analyzer =       new T473DummySectionAnalyzer( TDC );
  b_analyzer =       new T473DummySectionAnalyzer( ADC );
  c_analyzer =       new T473DummySectionAnalyzer( SCALER );
  analyzer -> install( general_analyzer );
  analyzer -> install( csi_tdc_analyzer );
  analyzer -> install( csi_adc_analyzer );
  analyzer -> install( a_analyzer );
  analyzer -> install( b_analyzer );
  analyzer -> install( c_analyzer );
}

T473AnalyzerMaker::~T473AnalyzerMaker(){
  delete tdc_cal;
  delete adc_cal;
  delete analyzer;
  delete general_analyzer;
  delete csi_tdc_analyzer;
  delete csi_adc_analyzer;
  delete a_analyzer;
  delete b_analyzer;
  delete c_analyzer;
}
