//======================================================================
/*
               T473CalibrationInfoMaker.cc
*/
//======================================================================
#include "T473CalibrationInfoMaker.hh"
#include  "T473.hh"
#include <analysis/KonoeDataAddress.hh>
#include <stdio.h>
#include <unistd.h>
T473CalibrationInfoMaker::T473CalibrationInfoMaker
( KonoeNameSpaceTable * ptable ){
  table = ptable;
  cal_tdc_info_list = new KonoeList<T473CalibrationInfo>( N_CSI );
  cal_adc_info_list = new KonoeList<T473CalibrationInfo>( N_CSI );
}

T473CalibrationInfoMaker::~T473CalibrationInfoMaker(){
  delete cal_tdc_info_list; 
  delete cal_adc_info_list;
}

KInt T473CalibrationInfoMaker::make(){
  T473CalibrationInfo * info;
  KonoeDataAddress    * address;
  KonoeNameSpaceValue * ns_value, *ns_value_ped, *ns_value_gain;
  KInt id = 0, sa, seg, offset, i;
  KFloat gain;
  KChar entry_name[256];
  //   CsI TDC Calibration Info  

  cal_tdc_info_list -> allocate( N_CSI );
  cal_adc_info_list -> allocate( N_CSI );

  for(seg = 0; seg < 4; seg ++ ){
    for( sa = 0; sa < 8 && 8 * seg + sa < 25  ; sa ++ ){
      //get TDC offset from NameSpace
      sprintf( entry_name, "CSI_TDC_%d_%d_PED",seg, sa );  
      while( !( ns_value_ped = table -> getValue( entry_name ) ) ){
	cout << "please set: " << entry_name << endl;
      }
      sscanf( ( KChar * ) ns_value_ped -> get(), "%d", &offset );
      delete ns_value_ped;
      
      //get TDC gain from NameSpace
      sprintf( entry_name, "CSI_TDC_%d_%d_GAIN",seg, sa );  
      while( !( ns_value_gain = table -> getValue( entry_name ) ) ){
	cout << "please set: " << entry_name << endl;
      }
      sscanf( ( KChar * ) ns_value_gain -> get(), "%lf", &gain );
      //      cout << "gain: " << gain << endl;
      delete ns_value_gain;
      
      info    = new T473CalibrationInfo( offset, gain );
      address = new KonoeDataAddress( CSI_TDC, seg, sa );
      
      info -> add( address );
      cal_tdc_info_list -> add( info );
      id ++;
    }
  }
  
  //   CsI ADC Calibration Info  
  id = 0;
  for(seg = 0; seg < 2; seg ++ ){
    for( sa = 0; sa < 16 && 16 * seg + sa < 25 ; sa ++ ){      
      
      //get ADC offset from NameSpace
      sprintf( entry_name, "CSI_ADC_%d_%d_PED",seg, sa );  
      
      while( !( ns_value_ped = table -> getValue( entry_name ) ) ){
	cout << "please set: " << entry_name << endl;
      }	
      sscanf( ( KChar * ) ns_value_ped -> get(), "%d", &offset );
      delete ns_value_ped;
            
      //get ADC gain from NameSpace
      sprintf( entry_name, "CSI_ADC_%d_%d_GAIN",seg, sa );  
      
      while( !( ns_value_gain = table -> getValue( entry_name ) ) ){
	cout << "please set: " << entry_name << endl;
      	return -1;
      }
      sscanf( ( KChar * ) ns_value_gain -> get(), "%lf", &gain );
      delete ns_value_gain;
      
      info =    new T473CalibrationInfo( offset, gain );
      address = new KonoeDataAddress( CSI_ADC, seg, sa );
      info -> add( address );
      
      cal_adc_info_list -> add( info );
      id ++;
    }
  }
  return 0;
}


