//===================================================================
/*
         T473CsIdata.cc
         10-Oct-2000  K.Uchida

*/
//===================================================================
#include  "T473CsIdata.hh"
#include  "T473Categories.hh"
#include  <iomanip.h>

const KChar     * T473CsIdata::className = "T473CsIdata";

T473CsIdata::T473CsIdata():
                    KonoeStreamableObject(
                         T473Categories::RECONSTRUCT,
			 T473ReconstructCM::CID_T473CSIDATA){
		      hitlist_tag = 0;
		      hit_point_tag = 0;
		      theta_tag = 0;
}
 
T473CsIdata::T473CsIdata( KonoeList<KonoeDetectorHit> * phitlist,
			 Konoe3Vector * p, Konoe3Vector * t )
: KonoeStreamableObject(
			  T473Categories::RECONSTRUCT,
			  T473ReconstructCM::CID_T473CSIDATA){
  hitlist = phitlist;
  hit_point = p;
  theta = t;
  
  hitlist_tag = 0;
  hit_point_tag = 0;
  theta_tag = 0;
}

T473CsIdata::~T473CsIdata(){
  if( hitlist_tag )
    delete hitlist;
  if( hit_point_tag )
    delete hit_point;
  if( theta_tag )
    delete theta;
}

void T473CsIdata::dump( ostream & os ) {
  KonoeDetectorHit * hit;
  hitlist -> reset();
  while( hit = hitlist -> next() )
    hit -> dump(os);
  os << "hit point " ; hit_point -> dump(os); os << endl;
  os << "CsI angle " ; theta -> dump( os ); os << endl;
}

KInt T473CsIdata::put( KonoeOutputObjectStream * os, KChar level, 
                       KInt memberid ){
  //  cout << "put header" << endl;
     KChar l = level + 1;
     KInt count = hitlist -> getCount();
     os -> putHeader( this, level, memberid, N_MEMBERS );
     //     cout << "put count " << endl;
     os -> put( l, MID_HIT_LIST_N_ENTRIES, count );
     KInt size = 0;
     hitlist -> reset( );
     KInt i;
     //     cout << "put hitlist" << endl;
     for( i = 0; i < count; i ++ ) {
       size += HEADER_SIZE + hitlist -> next( ) -> getStreamSize( );
     }
     os -> putObjectArrayHeader( l, MID_HIT_LIST_ENTRIES, count, size );
     hitlist -> reset( );
     for( i = 0; i < count; i ++ )
       hitlist -> next( ) -> put( os, l + 1, i );
     //     cout << "put hit point" << endl;
     hit_point -> put( os, l, MID_HITPOINT );
     //     hit_point.dump(cout); cout << endl;
     theta -> put( os, l, MID_THETA );
     return 0;
}

KInt T473CsIdata::getStreamSize(){
  //  cout << "getStreamSize" << endl;
  KInt   s = 0;
  s  = HEADER_SIZE + 1;    // n entries
  s += HEADER_SIZE;        // object array
  //  cout << "hitlist reset " << endl;
  hitlist -> reset( );
  for( KInt i = 0; i < hitlist -> getCount( ); i ++ ){
    s += HEADER_SIZE + hitlist -> next( ) -> getStreamSize( );
  }
  //  cout << "hit point getStreamSize" << endl;
  //  cout << " hit point: " << hit_point << endl;
  s += HEADER_SIZE + hit_point -> getStreamSize();
  //  cout << "theta: " << theta << endl; 
  //  cout << "theta getStreamSize" << endl;
  s += HEADER_SIZE + theta -> getStreamSize();
  return s;
   }

KInt    T473CsIdata::fillPrimitiveMember( KInt memberid, KInt datatype,
					  KInt datacount, void * data ) {
  switch( datatype ) {
  case KINT_TYPE :
    switch( memberid ) {
    case MID_HIT_LIST_N_ENTRIES :
      //allocate as many entries for the list
      //              cout << "allocate " << * ( KInt * ) data << endl;
      hitlist = new KonoeList<KonoeDetectorHit>( * ( KInt * ) data );
      hitlist_tag = 1;
      delete [] data;
      break;
    default :
      delete [] data;
      return -1;      // invalid member id
    }
    return 0;
  default :
    delete [] data;
    return -1;
  }
  return 0;
}


KInt T473CsIdata::fillObjectMember (KInt memberid,
				    KonoeStreamableObject * data ){
  
  //  cout << "fill object" << endl;
  switch( memberid ) {
  case MID_HITPOINT :
    hit_point = ( Konoe3Vector * ) data;
    hit_point_tag = 1;
    break;
  case MID_THETA :
    theta = ( Konoe3Vector * ) data;
    theta_tag = 1;
    break;
  case MID_HIT_LIST_ENTRIES :
    KonoeDetectorHit * hit;
    hit = ( KonoeDetectorHit * ) data ;
    hitlist -> add( hit );
    break;
  default :
    delete data;
    return -1;
  }
  return 0;
}
    
