//======================================================================
/*
               T473DetectorMaker.cc
	       20-Oct-2000  Kirika Uchida
*/
//======================================================================
#include "T473DetectorMaker.hh"
#include "T473.hh"
#include <stdio.h>
T473DetectorMaker::T473DetectorMaker( 
		     KonoeNameSpaceTable * ptable ){
  //  cout << "DetectorMaker constructor" << endl;
  //******************** set oa, AD, theta ************************//
  table = ptable;
  getGeometryInfoFromTable( );
  makeDetector( );
  setAddress( );
}

KInt  T473DetectorMaker::getGeometryInfoFromTable( ){
  KFloat x, y, z;
  KonoeNameSpaceValue * ns_value;
  //************************  set AD ******************************//
  ns_value = table -> getValue( "AD_X" ); 
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &x );
  delete ns_value;

  ns_value = table -> getValue( "AD_Y" );
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &y );
  delete ns_value;

  ns_value = table -> getValue( "AD_Z" );
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &z );
  delete ns_value;

  AD.set( x, y, z );
  //  cout << "AD has set" << endl;
  if( !( ns_value = table -> getValue( "OA_X" ) ) ){
    cout << "please set OA_X" << endl;
    return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &x );
  delete ns_value;

  if( !( ns_value = table -> getValue( "OA_Y" ) ) ){
    cout << "please set OA_Y" << endl;
    return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &y );
  delete ns_value;

  if( !( ns_value = table -> getValue( "OA_Z" ) ) ){
    cout << "please set OA_Z" << endl; return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &z );
  delete ns_value;

  oa.set( x, y, z );
  
  if( !( ns_value = table -> getValue( "THETA_X" ) ) ){
    cout << "please set THETA_X" << endl; return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &theta_x );
  delete ns_value;

  if( !( ns_value = table -> getValue( "THETA_Y" ) ) ){
    cout << "please set THETA_Y" << endl; return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &theta_y );
  delete ns_value;

  if( !( ns_value = table -> getValue( "THETA_Z" ) ) ){
    cout << "please set THETA_Z" << endl; return -1;
  }
  sscanf( ( KChar * ) ns_value -> get(), "%lf", &theta_z );
  delete ns_value;
  return 0;
}

Konoe3Vector  T473DetectorMaker::setRotation( Konoe3Vector di ){
  translation.addRotationX( theta_x );
  //translation.addRotationY( theta_y );
  translation.addRotationZ( theta_z );
  Konoe3Vector r;
  r = AD + di;
  return translation.rotate( r );           
}


KInt T473DetectorMaker::makeDetector(){
  KInt id, j, k;  
  Konoe3Vector p;
  KonoeBoxGeometry     * geometry;
  KonoeDetectorElement * element;
  
  detector = new KonoeDetector( N_CSI, CSI );  
  detector -> reset();
  id = 1;
  for( j = 0; j < 5; j ++ ) {
    for( k = 0 ; k < 5; k ++ ) {
      DI.set( - 70 * k, 70 * j, 0 );
      ai = setRotation(DI);
      p = oa + ai; 
      translation.setDisplacement( p );
      geometry = new KonoeBoxGeometry( CSI_WIDTH, CSI_HEIGHT, CSI_DEPTH );
      geometry -> translate( & translation ); 
      element = new KonoeDetectorElement( geometry, detector, id );
      //element -> dump( cout );
      detector -> add( element );
      id ++;
    }
  }   
  return 0;
}

KInt  T473DetectorMaker::setAddress( ){
  int id, seg, sa, l;
  KChar entry_name[256];
  KonoeNameSpaceValue * ns_value;
  KonoeDetectorElement * element;
  // TDC addresses set
  detector -> reset();
  id = 1;
  for(seg = 0; seg < 4; seg ++) {
    for(sa = 0; sa < 8 &&  8 * seg + sa < 25; sa ++) {
      sprintf( entry_name, "CSI_%d_TDC_SEG", id ); 
      if( !( ns_value = table -> getValue( entry_name ) ) ){
	cout << "please set " << entry_name << endl; return -1;
      }
      sscanf( ( KChar * ) ns_value -> get(), "%d", &seg );
      sprintf( entry_name, "CSI_%d_TDC_SA", id ); 
      if( !( ns_value = table -> getValue( entry_name ) ) ){
	cout << "please set " << entry_name << endl; return -1;
      }
      sscanf( ( KChar * ) ns_value -> get(), "%d", &sa );
      element = detector -> next();
      element -> add( new KonoeDataAddress( CSI_TDC, seg, sa ) );
      //      element -> dump(cout);
      id ++;
    }
  }
  //  cout << "TDC addresses have set" << endl;
  detector -> reset();
  id = 1;
  for(seg = 0; seg < 2; seg ++) {
    for(sa = 0; sa < 16 && 16 * seg + sa < 25; sa ++) {
      sprintf( entry_name, "CSI_%d_ADC_SEG", id ); 
      if( !( ns_value = table -> getValue( entry_name ) ) ){
	cout << "please set " << entry_name << endl; return -1;
      }
      sscanf( ( KChar * ) ns_value -> get(), "%d", &seg );
      sprintf( entry_name, "CSI_%d_ADC_SA", id ); 
      if( !( ns_value = table -> getValue( entry_name ) ) ){
	cout << "please set " << entry_name << endl; return -1;
      }
      sscanf( ( KChar * ) ns_value -> get(), "%d", &sa );
      element = detector -> next();
      element -> add( new KonoeDataAddress( CSI_ADC, seg, sa ) );
      id ++;
    }
  }  
  //  cout << "TDC addresses have set" << endl;
  return 0;
}

T473DetectorMaker::~T473DetectorMaker(){
  delete detector;
}

Konoe3Vector  T473DetectorMaker::getTheta(){
  theta.set( theta_x, theta_y, theta_z );
  return  theta;
}
