//======================================================================
/*
               T473GeneralData.cc
	       Kirika Uchida
*/
//======================================================================
#include "T473GeneralData.hh"
#include "T473Categories.hh"
#include "T473ReconstructCM.hh"
const KChar     * T473GeneralData::className = "T473GeneralData";

T473GeneralData::T473GeneralData(): 
  KonoeStreamableObject (T473Categories::RECONSTRUCT,
				T473ReconstructCM::CID_T473GENERALDATA){
    Time = 0;
    Clock = 0;
    KInt i;
    for( i = 0; i < MAX_BIT; i ++ )
      CoinReg0[i]= '1'; 
    CoinReg0[MAX_BIT]= '\0'; 
    for( i = 0; i < MAX_BIT; i ++ )
      CoinReg1[i]= '1'; 
    CoinReg1[MAX_BIT]= '\0'; 
    for( i = 0; i < MAX_BIT; i ++ )
      CoinReg2[i]= '1'; 
    CoinReg2[MAX_BIT]= '\0'; 

}

void T473GeneralData::dump( ostream & os ){
  os << "RunNumber: " << RunNumber << endl;
  os << "EventNumber: " << EventNumber << endl;
  os << "RunType: " << RunType << endl;
  os << "S1S4: " << S1S4 << endl;
  os << "GC: " << GC << endl;
  os << "S5: " << S5 << endl;
  os << "Time: " << Time << endl;
  os << "Clock: " << Clock << endl;
  os << "CoinReg0: " << CoinReg0 << endl;
  os << "CoinReg1: " << CoinReg1 << endl;
  os << "CoinReg2: " << CoinReg2 << endl;
}

KInt T473GeneralData::put( KonoeOutputObjectStream * os,
                        KChar level, KInt memberid ) {
  KInt    l = level + 1;
  os -> putHeader( this, level, memberid, N_MEMBERS );
  os -> put( l, MID_RUN_NUMBER, RunNumber );
  os -> put( l, MID_EVENT_NUMBER, EventNumber );
  os -> put( l, MID_RUN_TYPE,RunType );
  os -> put( l, MID_S1S4, S1S4 );
  os -> put( l, MID_GC, GC );
  os -> put( l, MID_S5, S5 );
  os -> put( l, MID_TIME, Time );
  os -> put( l, MID_CLOCK, Clock );
  os -> put( l, MID_COIN_REG_0, CoinReg0 );
  os -> put( l, MID_COIN_REG_1, CoinReg1 );
  os -> put( l, MID_COIN_REG_2, CoinReg2 );

  return 0;

}

KInt T473GeneralData::getStreamSize(){
  //  cout << "getStreamSize" << endl;
  KInt s;
  s  = HEADER_SIZE + 1; // RunNumber
  s += HEADER_SIZE + 1; // EventNumber
  s += HEADER_SIZE + RUNTYPE_LENGTH * sizeof( KChar )/ sizeof( KInt );
  // RunType
  s += HEADER_SIZE + 1; // S1S4
  s += HEADER_SIZE + 1; // GC
  s += HEADER_SIZE + 1; // S5
  s += HEADER_SIZE + 1; // Time
  s += HEADER_SIZE + 1; // Clock
  s += HEADER_SIZE + ( MAX_BIT + 1 ) * sizeof( KChar )/ sizeof( KInt ); 
  // CoinReg0
  s += HEADER_SIZE + ( MAX_BIT + 1 ) * sizeof( KChar )/ sizeof( KInt ); 
  // CoinReg1
  s += HEADER_SIZE + ( MAX_BIT + 1 ) * sizeof( KChar )/ sizeof( KInt ); 
  // CoinReg2
  return s;
}

KInt T473GeneralData::fillPrimitiveMember( KInt memberid,
	      KInt datatype, KInt datacount, void * data ){
  switch( datatype ){
  case KINT_TYPE:
    switch( memberid ){
    case MID_RUN_NUMBER:
      RunNumber = * ( KInt * ) data;
     delete [] data;
      break;
    case MID_EVENT_NUMBER:
      EventNumber = * ( KInt * ) data;
      delete [] data;
      break;
    case MID_S1S4:
      S1S4 = * ( KInt * ) data;
      delete[] data;
      break;
    case MID_GC:
      GC = * ( KInt * ) data;
      delete [] data;
      break;   
    case MID_S5:
      S5 = * ( KInt * ) data;
      delete[] data;
      break;   
    case MID_TIME:
      Time = * ( KInt * ) data;
     delete[] data;
      break;   
    case MID_CLOCK:
      Clock = * ( KInt * ) data;
      delete[] data;
      break;   
      
    default:
      delete[] data;
      return -1;
    }
    return 0;
  case KCHAR_TYPE:
    switch ( memberid ){
    case MID_RUN_TYPE:
      strcpy( RunType,( KChar * ) data );
      delete [] data;
      break;
    case MID_COIN_REG_0:
      strcpy( CoinReg0,( KChar * ) data );
      delete [] data;
      break;
    case MID_COIN_REG_1:
      strcpy( CoinReg1,( KChar * ) data );
      delete [] data;
      break;
    case MID_COIN_REG_2:
      strcpy( CoinReg2,( KChar * ) data );
      delete [] data;
      break;
    default:
      delete [] data;
      return -1;
    }
    return 0;
  }
}
