//======================================================================
/*
               T473GeneralSectionAnalyzer.cc
	       23-Oct-2000  Kirika Uchida
*/
//======================================================================
#include "T473GeneralSectionAnalyzer.hh"
#include "T473.hh"
#include "T473GeneralData.hh"

const KInt REQ_DATA = -2;

T473GeneralSectionAnalyzer::T473GeneralSectionAnalyzer
     ( KInt ptag, T473GeneralData * pdata, KChar * preq_type )
:KonoeSectionAnalyzer(ptag){
  data = pdata;
  req_type = preq_type;
}

KInt T473GeneralSectionAnalyzer::analyze( KonoeDataSection * section ){

  KonoeDataSegment        * segment;
  KonoeDataElement        * element;
  KInt  secid, segid, sa, run_number,event_number, run_type, s1s4, 
        gc, s5, time, clock, i;
  KChar coin_reg0[MAX_BIT + 1], coin_reg1[MAX_BIT + 1], 
        coin_reg2[MAX_BIT + 1]; 

  //  cout << "general section analyzer -> analyze " << endl;
  
  section -> reset();
  while(  segment = section -> next() ) {
    segid = segment -> getID( );
    
    /************************* COUNTER SEGMENT ****************************/
    if( segid == COUNTER ){
      //      cout << "counter segment" << endl;  
      segment -> reset( );
      //      cout << "segment reset" << endl;
      while( element = segment -> next( ) ) {
	sa =  element -> getAddress();
	//	cout << "subaddress in counter segment: " << sa << endl;
	switch ( sa ) {
	case RUN_NUMBER: 
	  run_number = * ( KInt * ) element -> getStore();
	  data -> setRunNumber( run_number );
	  //	  cout << "run_number  :  " << run_number << endl; 
	  break;
	case EVENT_NUMBER: 
	  event_number = * ( KInt * ) element -> getStore();
	  data -> setEventNumber( event_number );	
	  //	  cout << "event_number: " << event_number << endl; 
	  break;
	case RUN_TYPE_NUMBER:
	  break;
	default:
	  cout << "element address of general section counter segment invalid" 
	       << endl;
	  return -1;
	}
      }
    }
    
    /********************* INTERRAPT REGISTER SEGMENT ***********************/
    else if ( segid == INTERRUPT_REGISTER_0 );
    
    else if ( segid == INTERRUPT_REGISTER_1 );
    
    /******************** COINCIDENCE REGISTER SEGMENT **********************/

    else if( segid == COINCIDENCE_REGISTER_0 ) {
      segment -> reset( );
      element = segment -> next( );
      i = * ( KInt * ) element -> getStore();
      //  cout << i << endl;
      strcpy( coin_reg0, print_bit( i, MAX_BIT ) );
      //      cout << coin_reg0 << endl;
      if( coin_reg0[6] == 1 && coin_reg0[7] == 1 )
	gc = 1;
      else 
	gc = 0;
      
      if( coin_reg0[10] == 1 )
	s5 = 1;
      else 
	s5 = 0;
      
      if( !( strcmp( req_type, "XENON_TRIGGER" ) ) ){
	for( i = 11; i < 16; i ++ ){
	  if( coin_reg0[i] != '0'){
	    run_number = -1;
	    return -2;
	  } 
	}
	for( i = 1; i < 5; i ++ ){
	  if( coin_reg0[i] != '1'){
	    run_number = -1;
	    return -2;
	  }
	}
	if( run_number != -1 )
	  s1s4 = 0;
      }

      // ߤΥ٥ȥ쥯ϡS1',S2,S3,S4 Ǥ

      if( !( strcmp( req_type, "EVENT_TRIGGER" ) ) ){
	for( i = 11; i < 14; i ++ ){
	  if( coin_reg0[i] != '1'){
	    run_number = -1;
	    return -2;
	  }
	}
	if( coin_reg0[0] != '1' ){
	  run_number = -1;
	  return -2;
	}
	for( i = 1; i < 5; i ++ ){
	  if( coin_reg0[i] != '0' ){
	    run_number = -1;
	    return -2;
	  } 
	}
	if( run_number != -1 )
	  s1s4 = 1;
      }
    }
    
    else if( segid == COINCIDENCE_REGISTER_1 ) {
      segment -> reset( );
      element = segment -> next( );
      i = * ( KInt * ) element -> getStore();
      strcpy( coin_reg1, print_bit( i, MAX_BIT ) );
    }
    
    else if( segid == COINCIDENCE_REGISTER_2 ) {
      segment -> reset( );
      element = segment -> next( );
      i = * ( KInt * ) element -> getStore();
      strcpy( coin_reg2, print_bit( i, MAX_BIT ) );
    }
    
    /********************* TIME INFOMATION SEGMENT ************************/
    else if( segid == TIME_INFORMATION ) {
      segment -> reset( );
      while( element = segment -> next( ) ) {
	sa = element -> getAddress(); 
	switch ( sa ){
	case TIME:
	  time = * ( KInt * ) element -> getStore();
	  break;
	case CLOCK: // Υ٥ȤTIMEȸߤΥ٥ȤTIMEȤκʬ
	  clock = * ( KInt * ) element -> getStore();
	  break;
	default:
	  return -1;
	}
      }
    }
    else if( segid == APPEND );
    else return -1;
  }

  /****************** setValue of data ********************************/
  switch( run_type ){
  case 0:
    data -> setRunType( "Luminosity Run" );
    break;
  case 1:
    data -> setRunType( "Electron Run" );
    break;
  case 2:
    data -> setRunType( "Hadron Run" );
    break;
  case 3:
    data -> setRunType( "Cosmic Ray Run" );
    break;
  case 4:
    data -> setRunType( "Pedestal Run" );
    break;
  case 5:
    data -> setRunType( "Other Run" );
    break;
  }
  data -> setS1S4( s1s4 );
  data -> setGC( gc );
  data -> setS5( s5 );
  data -> setTime( time );
  data -> setClock( clock );
  data -> setCoinReg0( coin_reg0 );
  data -> setCoinReg1( coin_reg1 );
  data -> setCoinReg2( coin_reg2 );
  //  cout << "general section analyzed" << endl;
  return 0;
}

KChar *  T473GeneralSectionAnalyzer::print_bit( KInt value, KInt max_bit ) {
  KInt i, j, k;
  KInt bits[max_bit];
  KChar a[max_bit + 1];
  
  for( i = 0; i < max_bit; i ++) {
    k = max_bit - 1 - i;
    bits[k] = 1;
    for ( j = 0; j < i; j ++ ) 
      bits[k] *= 2;
  }
  for ( i = 0; i <= max_bit; i++ ) {
    k = max_bit - 1 - i;
    if ( ( value & bits[k] ) == bits[k] )
      a[k] = '1';
    else
      a[k] = '0';
  }
  a[max_bit] = '\0';
  return a;
}

