//=====================================================================
/*
             T473Reconstruct.cc
             12-Oct-2000   kirika Uchida
*/
//=====================================================================
#include "T473Reconstruct.hh"
#include <system/KonoeSocketIO.hh>
#include <system/KonoeServerSocket.hh>
#include <string.h>
KChar *  T473Reconstruct::PERMISSION = "GUI_REQUEST_PARMISSION";
const KChar *  T473Reconstruct::REC_REQUEST = "DATAHUB_REQ_CURRENT";
const KChar *  T473Reconstruct::CH_INFO_REQUEST = "CHANGE_INFO_TYPE";
const KChar *  T473Reconstruct::CH_TYPE_REQUEST = "CHANGE_REQUEST_TYPE";
const KChar *  T473Reconstruct::GUI_REQUEST = "GUI_REQ_CURRENT";
KChar *  T473Reconstruct::REC_STATUS = "RECONSTRUCT_STATUS";
KChar *  T473Reconstruct::STAND_BY = "standby";
KChar *  T473Reconstruct::BUSY = "busy";
KChar *  T473Reconstruct::NONE = "none";

T473Reconstruct::T473Reconstruct(int argc, char **argv ){
  cout << "(1)setting name space........................" << endl;
  set_name_space();
  cout << "---------------------------------------------" << endl;

  cout << "(2)setting client socket " << endl
       << "   for receive data record from HUB.........." << endl;
  set_socket_to_HUB();
  cout << "---------------------------------------------" << endl;

  ns_manager.setValue( REC_STATUS, STAND_BY );
  log_sender.send( "reconstruct: status = standby", T473LogSender::INFO );

  cout << "(3)setting server socket " << endl
       << "   for send CsI data to GUI.................." << endl;
  set_socket_to_GUI();
  cout << "---------------------------------------------" << endl;
  
  strcpy( req_type, "RANDOM_TRIGGER" );
  status = WAIT_REQ_TYPE;
  cout << "constructor end" << endl;
  spill_number = 0;
  receive_n = 0;
  send_n = 0;
  csi_data = 0;
  rec_data = 0;

} 

T473Reconstruct::~T473Reconstruct(){
  delete client_socket;
  delete recv_socket;
  delete server_socket;
  if( cal_info_maker )
    delete cal_info_maker;
  if( detector_maker )
    delete detector_maker;
  if( analyzer_maker )
    delete analyzer_maker;
  if( hitset )
    delete hitset;
}

KInt T473Reconstruct::set_name_space(){
  
  table = ns_manager.getTable();    
  
  // set reconstruct host name ----------------------------------
  host_system.getHostName( rec_host, TABLE_VALUE_SIZE ) ;
  ns_manager.setValue( "RECONSTRUCT_HOST", rec_host );
  cout << "   reconstruct host name: " << rec_host << endl;

  // set reconstruct pid ----------------------------------------
  sprintf( rec_pid, "%d", host_system.getPid() );
  ns_manager.setValue( "RECONSTRUCT_PID", rec_pid );
  cout << "   reconstruct pid: " << rec_pid << endl;


  return 0;
}

KInt T473Reconstruct::set_socket_to_HUB(){
  ns_value = table -> getValue( "DATAHUB_HOST" );
  pvalue = ( KChar * ) ns_value -> get();
  strcpy( hub_host, pvalue );
  delete ns_value;
  cout << "   hub host name: " << hub_host << endl;
  
  ns_value = table -> getValue( "DATAHUB_OUTPUT_PORT" );
  pvalue = ( KChar * ) ns_value -> get();
  hub_port = atoi( pvalue );
  delete ns_value;
  cout << "   hub output port: " << hub_port << endl;
  
  client_socket  = new KonoeClientSocket ( hub_host, hub_port ); 
 
  while( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ) {
    cout << "Hub is down. Try to connect again after 1 second." << endl;
    sleep( 1 );
    client_socket -> connect();
  }
  cout << "   connected to hub output port." << endl;
  
  fd_set.install( client_socket );
  
  recv_socket    = new KonoeInputObjSocketBuffered( client_socket ); 
  recv_socket    -> installCM( & rec_cm );
  return 0;
}

KInt T473Reconstruct::set_socket_to_GUI(){
  
  ns_value = table -> getValue( "REC_TO_GUI_PORT" );
  pvalue = ( KChar * ) ns_value -> get();
  rec_to_gui_port = atoi( pvalue );
  delete ns_value;
  cout << "   rec to GUI port: " << rec_to_gui_port << endl;  
  
  server_socket   = new KonoeObjectServerSocket( rec_to_gui_port );
  int adr = 0;
  while (server_socket ->getSocket() ->
	 getStatus() == KonoeServerSocket::OPENED ){
    delete server_socket;
    sleep(60);
    server_socket   = new KonoeObjectServerSocket( rec_to_gui_port );
    adr ++;
  }
  cout << "socket for GUI opened" << endl;
  server_socket -> setSendMode( KonoeServerSocket::SEND_TO_LAST );
  server_socket -> waitConnection( );
  
  
  cout << "   connected to GUI" << endl;
  sleep(1);
}

KInt T473Reconstruct::go(){
  rec = 0;
  analyzer_status = KFalse;
  KInt receive_record_st, analysis_st;
  cout << "receive request type from GUI" << endl;
  receive_request_type_from_GUI( );
  ns_manager.setValue( REC_STATUS, BUSY );   
  log_sender.send( "reconstruct: status = busy", T473LogSender::INFO );
  //  cout << "set analyzer" << endl;
  while( KTrue ) {  
    
    if( set_analyzer( ) == -1 ){
      cout << "failed to set analyzer" << endl;
      continue;
    }
    log_sender.send( "reconstruct: set analyzer", T473LogSender::INFO );
    analyzer_status = KTrue;
    
    while ( analyzer_status ){
      //###############################################################
      // receive datarecord and analyze the record
      //###############################################################
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "request and receive datarecord process" << endl;
      
      receive_record_st = request_and_receive_datarecord( );
      
      switch ( receive_record_st ){
      case RUN_START_RECORD:
	spill_number = 0; 
	delete record;	
	break;
      case RUN_END_RECORD:
	delete record;
	break;
      case EVENT_RECORD:
	cout << "received event datarecord" << endl;
	analysis_st = analyze( record );	
	delete record;
	break;
      case SPILL_END_RECORD:
	if ( mode = getenv ("RECONSTRUCT_TEST") )
	  cout << "request and receive spill end datarecord" << endl;
	spill_end_analysis();
	spill_number ++; 
	analysis_st = 1;
	delete record;
	break;
      default:
	cout << "no datarecord" << endl;
      }
      
      //###############################################################
      // send data to GUI
      //###############################################################
      
      if( (for_gui_socket = server_socket -> wait( 10 ) ) == 0 ){
	continue;
      }
      
      
      KInt receive_request_st;
      receive_request_st = receive_request_from_GUI( );
      
      if(  receive_request_st == 1 || receive_request_st == 2 
	   || receive_request_st == 3 || receive_request_st == -1 ){
      }
      else if( receive_request_st == 0 ){
	if( rec ){
	  send_rec_data();
	  cout << "send # = " << send_n << endl;
	  send_n ++;
	}
      }
    }
  }
  return 0;
}

KInt T473Reconstruct::receive_request_type_from_GUI( ){
  
  KChar buffer[MAX_LENGTH];
  KInt i, st;
  while( ( !( for_gui_socket = server_socket -> wait( 1 ) ) ) );
  
  if ( for_gui_socket -> readFromStream( buffer, MAX_LENGTH ) > 0 ) {
    
    if( !strcmp( buffer, "RANDOM_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received request_type from GUI RANDOM_TRIGGER"
	     << endl;
      strcpy( req_type, buffer );
    }
    else if( !strcmp( buffer, "EVENT_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received request_type from GUI EVENT_TRIGGER"
	     << endl;
      strcpy( req_type, buffer );
    }
    else if( !strcmp( buffer, "XENON_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received request_type from GUI XENON_TRIGGER"
	     << endl;
      strcpy( req_type, buffer );
    }
    else{
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received unknown request type" << endl;
      log_sender.send( "reconstruct received unknown request type",
		       T473LogSender::ERROR );
      return -1;
    }
  }
  
  else {
    
    if ( mode = getenv ("RECONSTRUCT_TEST") )
      cout << "cannot receive connection to gui." << endl;
    
    log_sender.send( "cannot receive gui request type",
		     T473LogSender::ERROR );
    
    server_socket -> close ( for_gui_socket );
    log_sender.send( "reconstruct closed connection to gui.", 
		     T473LogSender::INFO );
    return -1;
  }
  return 0;
}

KInt T473Reconstruct::set_analyzer( ){
  // calibrator ------------------------------------------------- 
  //  cout << "set calibration info" << endl;
  cal_info_maker = new T473CalibrationInfoMaker( table );
  if( cal_info_maker -> make() == -1 )    return -1;
  cal_tdc_info_list = cal_info_maker -> getTDCList();
  cal_adc_info_list = cal_info_maker -> getADCList();
  
  // detector ---------------------------------------------------
  //  cout << "set detector" << endl;
  detector_maker = new T473DetectorMaker( table );
  
  theta = detector_maker -> getTheta();  
  detector = detector_maker -> getDetector();
  //      detector -> dump( cout );
  
  // data set ---------------------------------------------------
  //  cout << "set hitset" << endl;
  hitset = new KonoeHitDataset( N_CSI, 0, 0 );
  
  // analyzer ---------------------------------------------------
  //  cout << "set analyzer" << endl;
  analyzer_maker = new T473AnalyzerMaker ( detector, &general_data, hitset, 
					   cal_tdc_info_list, cal_adc_info_list, 				          req_type, table );
  
  analyzer = analyzer_maker -> getAnalyzer();
  return 0;
}

KInt T473Reconstruct::spill_end_analysis(){
  spill_end_analyzer.analyze( record );
  spill_end_data = spill_end_analyzer.getData();
  spill_end_data -> reset();
  while( multiplet = spill_end_data -> nextT473DataMultiplet() ){
    //    cout << (KChar *)multiplet -> getValue( 1 ) << endl;
    ns_manager.setValue
      ( (KChar *)multiplet -> getValue( 0 ),
	(KChar*)multiplet -> getValue( 1 ) );
  }
  spill_end_data -> clear();
  return 0;
}
  
KInt  T473Reconstruct::receive_request_from_GUI( ){
  KInt i, st = 0;
  KChar buffer[MAX_LENGTH];
  
  if ( for_gui_socket -> readFromStream( buffer, MAX_LENGTH ) > 0 ) {
    //    cout << buffer << endl;
    
    if( !strcmp( buffer, GUI_REQUEST ) ){
      cout << "receive # = " << receive_n << endl;
      receive_n ++;      
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : GUI_REQUEST" << endl;
      analyzer_status = KTrue;
      return 0;
    }
    else if( !strcmp( buffer, CH_INFO_REQUEST ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : CHANGE_INFO_REQUEST"
	     << endl;
      log_sender.send( "reconstruct received GUI request: CHANGE_INFO_REQUEST",
		       T473LogSender::INFO );
      status = REQUESTED_TO_CHANGE_INFO;
      delete detector_maker; detector_maker = 0;
      delete cal_info_maker; cal_info_maker = 0;
      delete analyzer_maker; analyzer_maker = 0;
      delete hitset; hitset = 0;
      analyzer_status = KFalse;
      return 1;
    }
    else if( !strcmp( buffer, CH_TYPE_REQUEST ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : CHANGE_TYPE_REQUEST"
	     << endl;
      log_sender.send( "reconstruct received GUI request: CHANGE_TYPE_REQUEST",
		       T473LogSender::INFO );
      status = WAIT_REQ_TYPE;
      //delete detector_maker; detector_maker = 0;
      //delete cal_info_maker; cal_info_maker = 0;
      //delete analyzer_maker; analyzer_maker = 0;
      //delete hitset; hitset = 0;
      //      analyzer_status = KFalse;
      return 2;
    }
    else if( !strcmp( buffer, "RANDOM_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : RANDOM_TRIGGER" << endl;

      log_sender.send( "reconstruct received GUI request: RANDOM_TRIGGER",
		       T473LogSender::INFO );
      strcpy( req_type, buffer );
      status = REQUESTED_TO_CHANGE_INFO;
      delete detector_maker; detector_maker = 0;
      delete cal_info_maker; cal_info_maker = 0;
      delete analyzer_maker; analyzer_maker = 0;
      delete hitset; hitset = 0;
      analyzer_status = KFalse;
      return 3;
    }      
    else if( !strcmp( buffer, "EVENT_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : EVENT_TRIGGER" << endl;
      log_sender.send( "reconstruct received GUI request: EVENT_TRIGGER",
		       T473LogSender::INFO );
      strcpy( req_type, buffer );
      status = REQUESTED_TO_CHANGE_INFO;
      delete detector_maker; detector_maker = 0;
      delete cal_info_maker; cal_info_maker = 0;
      delete analyzer_maker; analyzer_maker = 0;
      delete hitset; hitset = 0;
      analyzer_status = KFalse;
      return 3;
    }
    else if( !strcmp( buffer, "XENON_TRIGGER" ) ){
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : XENON_TRIGGER" << endl;
      
      log_sender.send( "reconstruct received GUI request: XENON_TRIGGER",
		       T473LogSender::INFO );
      strcpy( req_type, buffer );
      status = REQUESTED_TO_CHANGE_INFO;
      delete detector_maker; detector_maker = 0;
      delete cal_info_maker; cal_info_maker = 0;
      delete analyzer_maker; analyzer_maker = 0;
      delete hitset; hitset = 0;
      analyzer_status = KFalse;
      return 3;
    }
    else {
      if ( mode = getenv ("RECONSTRUCT_TEST") )
	cout << "reconstruct received GUI request : unknown request" << endl;
      
      log_sender.send( "reconstruct received unknown GUI request", 
		       T473LogSender::ERROR );
      analyzer_status = KTrue;
      return -1;
    }
  }
  else {
    if ( mode = getenv ("RECONSTRUCT_TEST") )
      cout << "gui shutdown while receive request" << endl; 
    log_sender.send( "gui shutdown while receive request", 
		     T473LogSender::ERROR );
    server_socket -> close ( for_gui_socket );
    log_sender.send( "closed gui connection", T473LogSender::INFO );
    return -1;
  }
  return 0;
}

KInt T473Reconstruct::request_and_receive_datarecord( ){
  
  KInt record_type;
  
  while (  ( client_socket -> getStatus( ) ) != KonoeSocketIO::CONNECTED ) {
    if ( mode = getenv ("RECONSTRUCT_TEST") )
      cout << "HUB is down. reconstruct try " << endl;
    log_sender.send( "HUB is down. reconstruct try "
		     "to connect after 1 minute. " ,
		     T473LogSender::WARNING );
    sleep(60);
    client_socket -> connect();
  }
  
  client_socket -> send( REC_REQUEST, strlen( REC_REQUEST ) + 1 );
  
  if ( !( fd_client = fd_set.wait( 1 ) ) ){
    return NO_RECORD;
    
  }
  else {
    record = ( KonoeDataRecord * ) recv_socket -> get( 0 ) ; 	
    
    if( record == 0 )
      return NO_RECORD;
    
    record_type = record -> getType( ) ;
    return record_type;
  }
}



KInt  T473Reconstruct::analyze( KonoeDataRecord * record ){
  
  KInt st;
  KInt spill_event_number;
  KChar event_number[256];
  KChar EVENT_NUMBER[256] = "EVENT_NUMBER";
  if( ( st =  analyzer -> analyze( record ) ) < 0 ){
    spill_event_number = general_data.getEventNumber();
    sprintf( event_number,"%d", spill_event_number - spill_number );
    ns_manager.setValue( EVENT_NUMBER, event_number );
    cout << "analysis st: " << st << endl;
    return st;
  }
  if( csi_data )
    delete csi_data;
  if( rec_data )
    delete rec_data;
  hitlist = hitset -> getHitList();
  
  hit_point_maker.getHitPoint( hitset, hit_point );
  
  //general_data.setEventNumber( spill_event_number - spill_number );
  sprintf( event_number,"%d", general_data.getEventNumber() );
  ns_manager.setValue( EVENT_NUMBER, event_number );
  
  csi_data = new T473CsIdata( hitlist, &hit_point, &theta );  
  
  rec_data = new T473ReconstructData( &general_data, csi_data );
  
  rec = 1;
  
  if ( st == -1 ){
    if ( mode = getenv ("RECONSTRUCT_TEST") )
      cout << "reconstruct failed analysis." << endl;
    cout << "analysis st: " << st << endl;
  }
  cout << "analysis st: " << st << endl;
  return 0;
}


KInt T473Reconstruct::send_rec_data(){
  KInt st;
  st = for_gui_socket -> getSocket() -> getStatus();
  
  if ( st == KonoeSocketIO::CONNECTED ){
    
    for_gui_socket -> put( rec_data );
    
    if( for_gui_socket -> getSocket() -> getStatus()
	== KonoeSocketIO::CONNECTED ){
      analysis_status = KTrue;
      return 0;
    }
    else{
      log_sender.send( "gui shutdown during send reconstruct data.",
		       T473LogSender::ERROR );
      log_sender.send( "reconstruct close gui connection", 
		       T473LogSender::ERROR );
      server_socket -> close ( for_gui_socket );
      status = WAIT_REQ_TYPE;
      analyzer_status = KFalse;
      return -1;
    }
  } 
  else {
    log_sender.send( "gui shutdown befor send reconstruct data.",
		     T473LogSender::ERROR );
    log_sender.send( "reconstruct close gui connection", 
		     T473LogSender::ERROR );
    server_socket -> close ( for_gui_socket );
    status = WAIT_REQ_TYPE;
    analyzer_status = KFalse;
    return -1;
  } 
}
  
void T473Reconstruct::lastAction(){
  ns_manager.setValue( "RECONSTRUCT_STATUS", "none" ); 
  log_sender.send( "RECONSTRUCT DYING", T473LogSender::FATAL );
  exit(0);
}
