//======================================================================
/*
               T473ReconstructData.cc
	       Kirika Uchida
*/
//======================================================================
#include "T473ReconstructData.hh"
#include "T473Categories.hh"

const KChar   * T473ReconstructData::className = "T473ReconstructData";

T473ReconstructData::T473ReconstructData()
: KonoeStreamableObject(T473Categories::RECONSTRUCT,
			T473ReconstructCM::CID_T473RECONSTRUCTDATA){
  general_data_tag = 0;
  csi_data_tag = 0;
}

T473ReconstructData::T473ReconstructData( T473GeneralData * pgeneral,
					  T473CsIdata * pcsi )
:KonoeStreamableObject(T473Categories::RECONSTRUCT,
			T473ReconstructCM::CID_T473RECONSTRUCTDATA){
  general_data = pgeneral;
  csi_data = pcsi;
  general_data_tag = 0;
  csi_data_tag = 0;
}

T473ReconstructData::~T473ReconstructData(){
  if( general_data_tag )
    delete general_data;
  if( csi_data_tag )
    delete csi_data;
}

void T473ReconstructData::dump( ostream & os ){
  general_data -> dump( os );
  csi_data -> dump( os );
}

KInt T473ReconstructData::put( KonoeOutputObjectStream * os, KChar level,
			       KInt memberid ){
  KChar l = level + 1;
  //  cout << "put header" << endl;
  os -> putHeader( this, level, memberid, N_MEMBERS );
  //  cout << "put general data" << endl;
  general_data -> put( os, l, MID_GENERAL_DATA ); 
  //  cout << "put csi data" << endl;
  csi_data -> put( os, l,MID_CSI_DATA );
  return 0;
}

KInt T473ReconstructData::getStreamSize(){
  //  cout << "getStreamSize " << endl;
  KInt size;
  size  = HEADER_SIZE + general_data -> getStreamSize(); 
  size += HEADER_SIZE + csi_data -> getStreamSize();
  return size;
}

KInt    T473ReconstructData::fillObjectMember( KInt memberid,
	       KonoeStreamableObject * object ){
  switch( memberid ){
  case MID_GENERAL_DATA:
    general_data = ( T473GeneralData * ) object; 
    general_data_tag = 1;
    break;
  case MID_CSI_DATA:
    csi_data = ( T473CsIdata * ) object;
    csi_data_tag = 1;
    break;
  default:
    delete object;
    return -1;
  }
  return 0;
}


