//======================================================================
/*
               T473SpillEndAnalyzer.cc
	       07-Dec-2000 Kirika Uchida
*/
//======================================================================

#include "T473SpillEndAnalyzer.hh"
#include "T473.hh"
#include <stdio.h>

T473SpillEndAnalyzer::T473SpillEndAnalyzer( ){

  
  data_for_nms = new T473PhysicalValueForNameSpace(10, 0 );
  
  analyzer = new KonoeDataAnalyzer( N_SECTION );
  
  general_analyzer = new KonoeDefaultSectionAnalyzer
    ( GENERAL, 13, GENERAL );
  analyzer -> install( general_analyzer );
  
  d_1_analyzer = new T473DummySectionAnalyzer( CSI_TDC );
  analyzer -> install( d_1_analyzer );
  
  d_2_analyzer = new T473DummySectionAnalyzer( CSI_ADC );
  analyzer -> install( d_2_analyzer );

  d_3_analyzer = new T473DummySectionAnalyzer( TDC );
  analyzer -> install( d_3_analyzer );

  d_4_analyzer = new T473DummySectionAnalyzer( ADC );
  analyzer -> install( d_4_analyzer );

  scaler_section_analyzer = new KonoeDefaultSectionAnalyzer
    ( SCALER, 15, SCALER );
  analyzer -> install( scaler_section_analyzer );
}

T473SpillEndAnalyzer::~T473SpillEndAnalyzer( ){
  delete general_analyzer;
  delete d_1_analyzer;
  delete d_2_analyzer;
  delete d_3_analyzer;
  delete d_4_analyzer;
  delete scaler_section_analyzer;
}


KInt T473SpillEndAnalyzer::analyze( KonoeDataRecord * precord ){
  
  KonoePhysicalValue *general_physical_value;
  KonoePhysicalValue *scaler_physical_value;
  KonoeDataMultiplet *multiplet;
  KInt  analysis_process = 1;
  KFloat seg_id, sub_addr, trg_scl_no_veto, trg_scl_veto, efficiency_f;
  
  record = precord;
  
  analyzer -> analyze( record );
  
  general_physical_value =  general_analyzer -> getPhysicalValue();  
  scaler_physical_value  =  scaler_section_analyzer -> getPhysicalValue();
  scaler_physical_value -> reset();
  while( analysis_process ){
    multiplet = scaler_physical_value -> next();
    seg_id = multiplet -> getValue( SEGMENT_ID_INDEX );
    if ( ( KInt )seg_id == 0 ){
      sub_addr = multiplet -> getValue( SUBADDRESS_INDEX );
      
      if ( (KInt)sub_addr == 10 ){
	trg_scl_no_veto = multiplet ->  getValue( DATA_INDEX );
      }
      else if( (KInt)sub_addr == 11 ){
	trg_scl_veto = multiplet -> getValue( DATA_INDEX );
      }
    }
    else
      analysis_process = 0;
  }
  efficiency_f = trg_scl_veto / trg_scl_no_veto;
  //  cout << "efficiency_f: " << efficiency_f << endl;

  
  
  sprintf( efficiency, "%lf", efficiency_f );
  
  data_for_nms -> add( "EFFICIENCY", efficiency );
  analyzer -> reset();
  return 0;
}


