//======================================================================
/*
               DemoGUI.cc
*/
//======================================================================
#include        <iostream.h>
#include        <system/KonoeNameSpaceTable.hh>
#include        <system/KonoeClientSocket.hh>
#include        <system/KonoeInputObjSocket.hh>
#include        <analysis/KonoePhysicalValueCM.hh>
#include        <analysis/KonoeGeometryCM.hh>
#include        "T473CsIdata.hh"
#include        "T473ReconstructCM.hh"
#include         <unistd.h>
#include         <string.h>
#include         <stdlib.h>
#include         <analysis/Konoe3Vector.hh>
#include         "T473ReconstructData.hh"
#include         <analysis/KonoeBoxGeometry.hh>
#include         <analysis/KonoeDetector.hh>
#include         <system/KonoeFDClient.hh>
#include         <system/KonoeFDSet.hh>
#include          <sys/time.h>
#include          <sys/types.h>
#include           <unistd.h>
#include            <stdio.h>
#include            <fcntl.h>
#include            <sys/ioctl.h>

//const KChar *  request = "GUI_REQ_CURRENT";

class DemoGUI {
private:
        KonoeNameSpaceTable      table;
        KChar                   *rec_host;
        KChar                   *value;
        KChar                    request[256];
        KChar                    request_type[256];
        KonoeClientSocket       *client_socket;
	KonoeInputObjSocket     *recv_socket;

        KonoeFDSet                       kfd_set;
        KonoeFDClient                  * fd_client;
  
        KInt result;
        fd_set inputs, testfds;
        timeval timeout;
      
        static const KChar * permission;
	KonoePhysicalValueCM    *phys_cm;
	KonoeGeometryCM         *g_cm;
	T473ReconstructCM       * rec_cm;

	T473CsIdata             * csi_data;
        T473ReconstructData     * rec_data;
        T473GeneralData         * general_data;
        KonoeList<KonoeDetectorHit> * hit_list;
        KonoeDetectorHit            * hit;
        KonoeDetectorElement        * element;
        KonoeBoxGeometry            * geometry;
        Konoe3Vector                * hit_point;
        Konoe3Vector                * theta;
        KonoeDetector               * detector;
        KInt                         data_request_mode;
public:
	DemoGUI( int argc, char ** argv );
	~DemoGUI();
        void set_socket();
	int go();
        void set_request_type();
        KInt send_request_type_to_reconstruct();
        KInt receive_permission_from_reconstruct();
        KInt send_request_to_reconstruct();
        KInt receive_data_record_from_Reconstruct();   
};

const KChar * DemoGUI::permission = "GUI_REQUEST_PARMISSION";      

DemoGUI::DemoGUI( int argc, char ** argv ){
  
  phys_cm = new KonoePhysicalValueCM();
  g_cm = new KonoeGeometryCM();
  rec_cm = new T473ReconstructCM();
  
  set_socket( );
  // for select cin
  timeout.tv_sec = 0;
  timeout.tv_usec =500000;

  sleep( 1 );
}

DemoGUI::~DemoGUI(){
  client_socket -> ~KonoeClientSocket();
}       

void DemoGUI::set_socket( ){
  rec_host = ( KChar * )
    table.getValue( "RECONSTRUCT_HOST" ) -> get();
  cout << "have gotten reconstruct host name: " << rec_host << endl;
  
  value = ( KChar * )
    table.getValue( "REC_TO_GUI_PORT" ) -> get();
  cout << "have gotten rec to GUI port: " << value << endl;
  
  client_socket = new KonoeClientSocket( rec_host, atoi( value ) );
  
  while( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ) {
    cout << "reconstruct is down. Try to connect again after 1 minutes."
	 << endl;
    sleep( 60 );
    client_socket -> connect();
  }
  
  recv_socket = new KonoeInputObjSocket( client_socket );
  
  recv_socket -> installCM( phys_cm );
  recv_socket -> installCM( g_cm );
  recv_socket -> installCM( rec_cm );
  
  kfd_set.install( client_socket );
}  

int  DemoGUI::go(){
  KInt st, i = 0;

  while(1){
    set_request_type();
    
    if( client_socket == 0 )
      set_socket();
    
    while( client_socket -> getStatus() != KonoeSocketIO::CONNECTED ){
      cout << "reconstruct is down."
	   << "Try to connect again after 1 minutes."
	   << endl;
      sleep( 60 );
      client_socket -> connect();
    }
    /*    
	  if( fd_client = kfd_set.wait( 500 ) ) {
	  rec_data  = ( T473ReconstructData * ) recv_socket -> get( 0 );
	  delete rec_data;
	  }*/
    if( send_request_type_to_reconstruct() != 0 ) continue;
    
    if( receive_permission_from_reconstruct() != 0 ) continue;
    
    data_request_mode = 1;    
    while( data_request_mode ){
      FD_ZERO( &inputs );
      FD_SET( 0, &inputs );
      testfds = inputs;
      
      result = select( FD_SETSIZE, &testfds,
		       ( fd_set * ) 0, (fd_set * ) 0,
		       &timeout );
      switch( result ){
      case 0:
	strcpy( request, "GUI_REQ_CURRENT" ); 
	//cout << "request: " << request << endl;
	if( send_request_to_reconstruct() != 0 ){
	  cout << "failed send request" << endl;
	  data_request_mode = 0;
	  continue;
	}
	if( receive_data_record_from_Reconstruct() != 0 ){
	  cout << "failed receive data record" << endl;
	  data_request_mode = 0;
	  continue;
	}
	break;
      case -1:
	//      perror("select");
	return -1;
	
      default:
	if( FD_ISSET( 0, &testfds ) ) {
	  cin >> request;
	  data_request_mode = 0;
	  continue;
	}
      }// switch
      sleep(1);
    }// data_request_mode
  }
  return 0;
}

void DemoGUI::set_request_type(){
  KInt success = 0;
  
  while( !success ){
    cout << "enter request type << " ;
    cin >> request_type; 
    if( !strcmp( request_type, "RANDOM_TRIGGER" ) );
    
    else if( !strcmp( request_type, "EVENT_TRIGGER" ) );
    
    else if( !strcmp( request_type, "XENON_TRIGGER" ) );
    
    else{
      cout << "   unknown request type" << endl;
      continue;
    }
    success = 1;
  }
}


KInt DemoGUI::send_request_type_to_reconstruct(){
  KInt st;
  
  if( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ){
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  st = client_socket -> send( request_type, 256 );
  
  if ( st < 0 ){
    client_socket = 0;
    delete recv_socket;
    return -1;
  }
  return 0;
}

KInt DemoGUI::receive_permission_from_reconstruct(){
  KChar buffer[256];
  
  if( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ){
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  if( client_socket -> receive( buffer, 256 ) <= 0 ) {
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  cout << "message from reconstruct: " << buffer << endl;
  
  return 0;
}


KInt DemoGUI::send_request_to_reconstruct(){
  
  if( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ){
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  if(  client_socket -> send( request, 256 ) <= 0 ){
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  cout << "send message to reconstruct: " << request << endl;
  
  return 0;
}
 
KInt DemoGUI::receive_data_record_from_Reconstruct(){   
  
  while( !( fd_client = kfd_set.wait( 500 ) ) ){
    
    cout << "waiting reconstruct data....." << endl;
    
    if( client_socket -> getStatus() != KonoeClientSocket::CONNECTED ){
      cout << "connection closed" << endl;
      delete recv_socket;
      client_socket = 0;
      return -1;
    }
  }
  
  rec_data  = ( T473ReconstructData * ) recv_socket -> get( 0 );

  if( ! rec_data ){
    cout << "rec data is 0" << endl;
    delete recv_socket;
    client_socket = 0;
    return -1;
  }
  
  general_data = rec_data -> getGeneralData( );
  general_data -> dump( cout );
  // rec_data -> dump( cout );
  delete rec_data;

  return 0;
}

int main(int argc, char ** argv ){
  DemoGUI demo( argc, argv );

  sleep(1);
  
  while( 1 ){
  demo.go();
  }
  return 0;
}
       
