//====================================================
/*
	T473RunControl.cc

	26-Nov-2000	Y. Shimizu
*/
//====================================================

#include	"T473RunControl.hh"
#include        "../RunSummary/T473RunSummary.hh"
#include	<system/KonoeExecutor.hh>
#include	<system/KonoeTime.hh>
#include        <system/KonoeInputObjFile.hh>
#include        <analysis/Konoe1DHistogram.hh>
#include        <analysis/Konoe1DHistogramList.hh>
#include        <analysis/KonoeHistogramCM.hh>
#include        <analysis/KonoePhysicalValueCM.hh>
#include	<unistd.h>
#include	<string.h>
#include        <stdio.h>
#include	<stdlib.h>

T473RunControl::T473RunControl(){

  KChar	ans[256];

  time = new KonoeTime;

  status = STANDBY;
  answer = KFalse;
  timeout = new T473TimeoutHandler;
  histo_checker = new T473StatusChecker("HISTOGRAM");
  logger_checker = new T473StatusChecker("LOGGER");
  
  setter = new T473NameSpaceSetter;
  ext    = new T473NameSpaceExtractor;

  KInt daq_cmd_port = ext->getKIntValue("DAQ_CMD_PORT");
  if( daq_cmd_port == -1){
    cerr<<"DAQ_CMD_PORT is not defined : exit(0)"<<endl;
    exit(0);
  }
  cout << "DAQ_CMD_PORT = " << daq_cmd_port << endl;
  daq_control = new T473DAQControl(daq_cmd_port);

  KInt histo_cmd_port = ext->getKIntValue("HISTO_CMD_PORT");
  if( histo_cmd_port == -1 ){
    cerr << "HISTO_CMD_PORT is not defined : exit"<<endl;
    exit(0);
  }
  cout << "HISTO_CMD_PORT = " << histo_cmd_port << endl;
  histo_control = new T473HistoControl(histo_cmd_port);

  KInt hub_cmd_port = ext->getKIntValue("DATAHUB_OUTPUT_PORT");
  if( hub_cmd_port == -1 ){
    cerr << "DATAHUB_OUTPUT_PORT is not defined : exit"<<endl;
    exit(0);
  }
  cout << "DATAHUB_OUTPUT_PORT = " << hub_cmd_port << endl;
  hub_control = new T473DataHubControl(hub_cmd_port);

  KInt log_cmd_port = ext->getKIntValue("LOGGER_CMD_PORT");
  if( log_cmd_port == -1 ){
    cerr << "LOGGER_CMD_PORT is not defined : exit"<<endl;
    exit(0);
  }
  cout << "LOGGER_CMD_PORT = " << log_cmd_port << endl;
  logger_control = new T473LoggerControl(log_cmd_port);

  KonoeExecutor * hub_exe = new KonoeExecutor(ext->getKIntValue("EXE_PORT"));
  hub_exe->exec(ext->getKCharValue("DATAHUB_HOST"),"../DataHub/DataHub DATAHUB &");
  delete hub_exe;

  KonoeExecutor * recons_exe = new KonoeExecutor(ext->getKIntValue("EXE_PORT"));
  recons_exe->exec(ext->getKCharValue("RECONSTRUCT_HOST"),"../Reconstruct/reconstruct &");
  delete recons_exe;

  cout << endl << endl << endl;
  cout <<"\t========================================================"<<endl;
  cout <<"\t|               Welcome to  KONOE !!                   |"<<endl;
  cout <<"\t|                                                      |"<<endl;
  cout <<"\t|              (^o^)/  (*o*)/  (+o+)/                  |"<<endl;
  cout <<"\t|                                                      |"<<endl;
  cout <<"\t|                           T473Control                |"<<endl;
  cout <<"\t|                              CUI version(1.0)        |"<<endl;
  cout <<"\t|                                                      |"<<endl;
  //cout <<"\t|                           by  Y. Shimizu             |"<<endl;
  //cout <<"\t|                                 and                  |"<<endl;
  cout <<"\t|                               KONOE collaboration    |"<<endl;
  cout <<"\t========================================================"<<endl;
  cout << endl;
  cout <<"\n\tWhich controler MODE? ( safety = s , non-safety = n) \t:";
  cin.getline(ans,256);
  if( !strcmp(ans,"s")){
	cout << "\t\nSAFETY MODE \n\n";
  }
  else{
	cout << "\t\nNORMAL MODE \n\n";
#define NORMAL_MODE 
	status = IGNORE ;
  }
  
  
}

T473RunControl::~T473RunControl(){

  delete setter;
  delete ext;
  delete daq_control;
  delete histo_control;
  delete hub_control;
  delete logger_control;

}

KInt T473RunControl::askGo(){
  KChar buff[256];
  cout <<"\n\tT473Begin \t";
  cout <<"\n\tAre you Ready?( yes , no )\t:";
  cin.getline(buff,256);
  if( !strcmp(buff,"yes"))
	return 0;
  else if( !strcmp(buff,"no"))
	return 1;
  else {
	cout << "Hey! answer \"yes\" or \"no\" "<<endl;
	this->askGo();
  }
} 

KInt T473RunControl::setGeneralInfo(){

  KChar buffer[256];

  cout << endl;
  cout << endl;

  cout << "\tT473RunControl::setGeneralInfo\n\n"<<endl;
  
  cout << "\tRUN_NUMBER?\t:" ;
  cin.getline(buffer,256);
  setter->set("RUN_NUMBER",buffer);
  while(1){
    cout << "\tRUN_TYPE?\t\n\n" ;

    cout << "\tLuminosity Run\t		0\n";
    cout << "\tElectron Run\t		1\n";
    cout << "\tHadron Run\t		2\n";
    cout << "\tCosmic Ray Run\t		3\n";
    cout << "\tPedestal Run\t		4\n";
    cout << "\tOther\t			5\n";
    cout << "\n" ;
    cout << "\n\tplease select number:" ;
    cin.getline(buffer,256);
    if( !strcmp(buffer,"0") ){
      setter->set("RUN_TYPE","luminosity_run");
      break;
    }
    if( !strcmp(buffer,"1") ){
      setter->set("RUN_TYPE","electron_run");
      break;
    }
    if( !strcmp(buffer,"2") ){
      setter->set("RUN_TYPE","hadron_run");
      break;
    }
    if( !strcmp(buffer,"3") ){
      setter->set("RUN_TYPE","cosmic_ray_run");
      break;
    }
    if( !strcmp(buffer,"4") ){
      setter->set("RUN_TYPE","pedestal_run");
      break;
    }
    if( !strcmp(buffer,"5") ){
      setter->set("RUN_TYPE","other_run");
      break;
    }
    cout << "\tplease select Number again!\n";
  }
    
  cout << "\tPARTICLE_ENERGY\t:";
  cin.getline(buffer,256);
  setter->set("PARTICLE_ENERGY",buffer);
  cout << "\tPLYAER1\t:" ;
  cin.getline(buffer,256);
  setter->set("PLAYER1",buffer);
  cout << "\tPLYAER2\t:" ;
  cin.getline(buffer,256);
  setter->set("PLAYER2",buffer);
  strcpy(buffer,ext->getKCharValue("RUN_NUMBER"));
  strcat(buffer,"-logfile.dat"); 
  setter->set("LOGFILE_NAME",buffer);
  cout << "\tLOGFILE_NAME ( default = " << buffer <<") \t:" ;
  cin.getline(buffer,256);
  if( strcmp(buffer,""))
     setter->set("LOGFILE_NAME",buffer);
  cout << "\tTEMP\t:" ;
  cin.getline(buffer,256);
  setter->set("TEMP",buffer);

  cout << endl;
  cout << "please wait... " << endl;
  sleep(1);
  if( KTrue == this->isGeneralInfo())
    return 0;
  else
    this->setGeneralInfo();

  return (0);

}

KBool T473RunControl::isGeneralInfo(){
  
  KChar ans[256];

  cout << endl;
  cout << endl;
  cout << "== NameSpace Check == " << endl << endl;
  this->nsCheck("RUN_NUMBER");
  this->nsCheck("RUN_TYPE");
  this->nsCheck("PARTICLE_ENERGY");
  this->nsCheck("PLAYER1");
  this->nsCheck("PLAYER2");
  this->nsCheck("LOGFILE_NAME");
  this->nsCheck("TEMP");

  cout << endl;
  cout << "===================== " << endl << endl;

  while(1){
    cout << "\tOK? ( yes or no ) \t:" ;
    cin.getline(ans,256);
    if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
      break;
    cout << "\t please anser ( yes or no ) " << endl;
  }
 
  if( !strcmp(ans,"yes") )
    return KTrue;
  if( !strcmp(ans,"no") )
    return KFalse;
  

}

KInt T473RunControl::setCsITDCPedestalInfo(){

  cout << "\tT473RunControl::set CsI TDC Pedestal Info"<<endl;
  KChar buffer1[256];
  KChar buffer2[256];
  cout << endl;
  cout << endl;
  
  if( KTrue == this->isCsITDCPedestalInfo())
    return 0;
  
  cout << "\tPlease Input NameSpace Item\t:";
  cin.getline(buffer1,256);
  cout << "\tPlease Input NameSpace Value\t:";
  cin.getline(buffer2,256);
  setter->set(buffer1,buffer2);

  this->setCsITDCPedestalInfo();
  
  return (0);
}

KInt T473RunControl::setCsIADCPedestalInfo(){

  cout << endl;
  cout << endl;
  cout << "\tT473RunControl::set CsI ADC Pedestal Info"<<endl;
  KChar buffer1[256];
  KChar buffer2[256];
  cout << endl;
  
  if( KTrue == this->isCsIADCPedestalInfo())
    return 0;
  
  cout << "\tPlease Input NameSpace Item\t:";
  cin.getline(buffer1,256);
  cout << "\tPlease Input NameSpace Value\t:";
  cin.getline(buffer2,256);
  setter->set(buffer1,buffer2);

  this->setCsIADCPedestalInfo();
  
  return (0);
}

KInt T473RunControl::setScinTDCPedestalInfo(){

  cout << endl;
  cout << endl;
  cout << "\tT473RunControl::set Scin TDC Pedestal Info"<<endl;
  KChar buffer1[256];
  KChar buffer2[256];
  cout << endl;
  
  if( KTrue == this->isScinTDCPedestalInfo())
    return 0;
  
  cout << "\tPlease Input NameSpace Item\t:";
  cin.getline(buffer1,256);
  cout << "\tPlease Input NameSpace Value\t:";
  cin.getline(buffer2,256);
  setter->set(buffer1,buffer2);

  this->setScinTDCPedestalInfo();
  
  return (0);
}

KInt T473RunControl::setDataHubStrageName(){
  cout << endl;
  cout << endl;
  KChar com[256];  
  KChar ans[256];
  cout << "\n\tT473RunContorl::setDataHubStrageName\n\n" << endl;
  cout << "\tDATAHUB:CMD ( new_file or file_close or shutdown or continue) \n";
  cout << "\tDATAHUB:CMD>";
  cin.getline(com,256);
  timeout->setTime(120000);
  hub_control->control(com);
  timeout->setTime(0);
  if( strcmp(com,"file_close")){
    if( KTrue == isDataHubStrageName() )
      return 0;
    else 
      this->setDataHubStrageName();
  }
  else {
    cout << "\tData Hub Strage FILE <close> \n";
    while(1){
      cout << "\tOK? ( yes or no ) \t:" ;
      cin.getline(ans,256);
      if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
	break;
      cout << "\tHey!You! answer ( yes or no ) " << endl;
    }
 
    if( !strcmp(ans,"yes") )
      return 0;
    if( !strcmp(ans,"no") )
      this->setDataHubStrageName();
  }
    
  return -1;
}

KBool T473RunControl::isDataHubStrageName(){

  KChar ans[256];

  cout << endl;
  cout << "== NameSpace Check == " << endl << endl;
  this->nsCheck("DATAHUB_STORAGE_FILE_NAME");
  cout << endl;
  cout << "===================== " << endl << endl;

  while(1){
    cout << "\tOK? ( yes or no ) \t:" ;
    cin.getline(ans,256);
    if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
      break;
    cout << "\tHey!You! answer ( yes or no ) " << endl;
  }
 
  if( !strcmp(ans,"yes") )
    return KTrue;
  if( !strcmp(ans,"no") )
    return KFalse;
  
}


KBool T473RunControl::isCsIADCPedestalInfo(){
  KChar ans[256];
  
  cout << endl;
  cout << "== NameSpace Check == " << endl << endl;
  this->nsCheck("CSI_ADC_0_0_PED");
  this->nsCheck("CSI_ADC_0_1_PED");
  this->nsCheck("CSI_ADC_0_2_PED");
  this->nsCheck("CSI_ADC_0_3_PED");
  this->nsCheck("CSI_ADC_0_4_PED");
  this->nsCheck("CSI_ADC_0_5_PED");
  this->nsCheck("CSI_ADC_0_6_PED");
  this->nsCheck("CSI_ADC_0_7_PED");
  this->nsCheck("CSI_ADC_0_8_PED");
  this->nsCheck("CSI_ADC_0_9_PED");
  this->nsCheck("CSI_ADC_0_10_PED");
  this->nsCheck("CSI_ADC_0_11_PED");
  this->nsCheck("CSI_ADC_0_12_PED");
  this->nsCheck("CSI_ADC_0_13_PED");
  this->nsCheck("CSI_ADC_0_14_PED");
  this->nsCheck("CSI_ADC_0_15_PED");
  this->nsCheck("CSI_ADC_1_0_PED");
  this->nsCheck("CSI_ADC_1_1_PED");
  this->nsCheck("CSI_ADC_1_2_PED");
  this->nsCheck("CSI_ADC_1_3_PED");
  this->nsCheck("CSI_ADC_1_4_PED");
  this->nsCheck("CSI_ADC_1_5_PED");
  this->nsCheck("CSI_ADC_1_6_PED");
  this->nsCheck("CSI_ADC_1_7_PED");
  this->nsCheck("CSI_ADC_1_8_PED");

  cout << endl;
  cout << "===================== " << endl << endl;

  while(1){
    cout << "\tPedestal Change? ( yes or no ) \t:" ;
    cin.getline(ans,256);
    if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
      break;
    cout << "\tOops! please answer ( yes or no ) " << endl;
  }
 
  if( !strcmp(ans,"no") )
    return KTrue;
  if( !strcmp(ans,"yes") )
    return KFalse;
}

KBool T473RunControl::isCsITDCPedestalInfo(){
  KChar ans[256];
  
  cout << endl;
  cout << "== NameSpace Check == " << endl << endl;

  this->nsCheck("CSI_TDC_0_0_PED");
  this->nsCheck("CSI_TDC_0_1_PED");
  this->nsCheck("CSI_TDC_0_2_PED");
  this->nsCheck("CSI_TDC_0_3_PED");
  this->nsCheck("CSI_TDC_0_4_PED");
  this->nsCheck("CSI_TDC_0_5_PED");
  this->nsCheck("CSI_TDC_0_6_PED");
  this->nsCheck("CSI_TDC_0_7_PED");

  this->nsCheck("CSI_TDC_1_0_PED");
  this->nsCheck("CSI_TDC_1_1_PED");
  this->nsCheck("CSI_TDC_1_2_PED");
  this->nsCheck("CSI_TDC_1_3_PED");
  this->nsCheck("CSI_TDC_1_4_PED");
  this->nsCheck("CSI_TDC_1_5_PED");
  this->nsCheck("CSI_TDC_1_6_PED");
  this->nsCheck("CSI_TDC_1_7_PED");

  this->nsCheck("CSI_TDC_2_0_PED");
  this->nsCheck("CSI_TDC_2_1_PED");
  this->nsCheck("CSI_TDC_2_2_PED");
  this->nsCheck("CSI_TDC_2_3_PED");
  this->nsCheck("CSI_TDC_2_4_PED");
  this->nsCheck("CSI_TDC_2_5_PED");
  this->nsCheck("CSI_TDC_2_6_PED");
  this->nsCheck("CSI_TDC_2_7_PED");
  this->nsCheck("CSI_TDC_3_0_PED");

  cout << endl;
  cout << "===================== " << endl << endl;

  while(1){
    cout << "\tPedestal Change? ( yes or no ) \t:" ;
    cin.getline(ans,256);
    if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
      break;
    cout << "\tOops! please answer ( yes or no ) " << endl;
  }
 
  if( !strcmp(ans,"no") )
    return KTrue;
  if( !strcmp(ans,"yes") )
    return KFalse;
}


KBool T473RunControl::isScinTDCPedestalInfo(){
  KChar ans[256];
  
  cout << endl;
  cout << "== NameSpace Check == " << endl << endl;
  this->nsCheck("SCIN_TDC_0_0_PED");
  this->nsCheck("SCIN_TDC_0_1_PED");
  this->nsCheck("SCIN_TDC_0_3_PED");

  cout << endl;
  cout << "===================== " << endl << endl;

  while(1){
    cout << "\tPedestal Change? ( yes or no ) \t:" ;
    cin.getline(ans,256);
    if( !strcmp(ans,"yes") || !strcmp(ans,"no"))
      break;
    cout << "\tOops! please answer ( yes or no ) " << endl;
  }
 
  if( !strcmp(ans,"no") )
    return KTrue;
  if( !strcmp(ans,"yes") )
    return KFalse;
}
  

KInt T473RunControl::begin(){

  KInt count = 0;
  KInt check = KFalse;

  KChar* timetxt;
  timetxt = time->getTimeInfo();
  cout << "\tRUN_BEGIN_TIME = " << timetxt << endl;
  setter->set("RUN_BEGIN_TIME",timetxt);

  cout << "\tT473RunContorl::begin" << endl;
  timeout->setTime(2000);
  answer = daq_control->control("begin");
  timeout->setTime(0);
  if(answer){
    status = READY;
#ifdef NORMAL_MODE
    status = IGNORE;
#endif
    check = KTrue;
  }
  else {
    cout << "\tDAQ_CMD not REACED! \t TRY AGAIN." << endl;
//   return -1;
    check = KFalse;
  }
  if( check )
    cout << "\t####### DAQ_CMD BEGIN OK! #######" << endl;

  cout << "\tNow Histogram::begin() " << endl;
  check = KTrue;
  histo_checker->init();
  sleep(1);
  timeout->setTime( 1000 );
  histo_control->control("begin");
  timeout->setTime( 0 );
  while( histo_checker->check() ) {
    
    if( count == 20000 ){
      cout <<"\tHISTO:\tCMD NOT REACHED\n" ;
      check = KFalse;
//      return -1;
    }
    count++;
  }
  
  histo_checker->init();
  if( check )
    cout << "\t####### HISTO_CMD BEGIN OK! #######" << endl;

  cout << "\tNow Logger::begin() " << endl;
  check = KTrue;
  sleep(1);
  logger_checker->init();
  timeout->setTime( 1000 );
  logger_control->control("begin");
  timeout->setTime( 0 );
  while( logger_checker->check() ){
    
    if( count == 1000 ){
      cout << "\tLOGGER:\tCMD NOT REACHED\n";
      check  = KFalse;
//      return -1;
    }
    count++;
  }
  logger_checker->init();
  if( check )
    cout << "\t#######LOGGER_CMD BEGIN OK! #######" << endl;
  
  return (0);
}

KInt T473RunControl::pause(){
 
  KInt check = KTrue;

  cout << "\tT473RunControl::pause" << endl;
  timeout->setTime(2000);
  answer = daq_control->control("pause");
  timeout->setTime(0);
  if(answer){
    status = IDLE;
#ifdef 	NORMAL_MODE
    status = IGNORE;
#endif
  }
  else {
    cout << "\tDAQ_CMD not REACED! \t TRY AGAIN." << endl;
      check = KFalse;
//    return -1;
  }
  
  if( check )
    cout << "\t####### DAQ_CMD PAUSE OK! #######" << endl;
  return (0);

}

KInt T473RunControl::resume(){
  
  cout << "\tT473RunControl::resume" << endl;
  KInt check = KTrue;
  
  timeout->setTime(2000);
  answer = daq_control->control("resume");
  timeout->setTime(0);
  if(answer){
    status = READY;
#ifdef 	NORMAL_MODE
    status = IGNORE;
#endif
  }
  else {
    cout << "\tDAQ_CMD not REACED! \t TRY AGAIN." << endl;
    check = KFalse;
  }

  if( check )
    cout << "\t####### DAQ_CMD RESUME OK! #######" << endl;
  return (0);
}

KInt T473RunControl::end(){

  KInt  count = 0;
  KInt  check = KTrue;
  KInt  final_check = KTrue;
  KChar comment[256];
  KChar run_type[256];

  cout << "\tT473RunControl::end" << endl;
  timeout->setTime(2000);
  answer = daq_control->control("end");
  timeout->setTime(0);
  if(answer){
    status = STANDBY;
#ifdef 	NORMAL_MODE
    status = IGNORE;
#endif
  }
  else {
    cout << "\tDAQ_CMD not REACED! \t TRY AGAIN." << endl;
    check = KFalse;
    final_check = KFalse;
//    return -1;
  }
  if( check )
    cout << "\t####### DAQ_CMD END OK! #######" << endl;

  cout << "\tNow Histogram::pause() " << endl;
  check = KTrue;
  timeout->setTime( 1000 );
  histo_control->control("pause");
  timeout->setTime( 0 );
  while( histo_checker->check() ) {
                
    if( count == 20000 ){
      cout <<"\tHISTO:\tCMD NOT REACHED\n" ;
      check = KFalse;
      final_check = KFalse;
      break;
    }
    count++;
  }

  if( check )
    cout << "\tNow Histogram::end() " << endl;

  sleep(1);
  check = KTrue;
  timeout->setTime( 1000 );
  histo_control->control("end");
  timeout->setTime( 0 );
  while( histo_checker->check() ) {
                
    if( count == 20000 ){
      cout <<"\tHISTO:\tCMD NOT REACHED\n" ;
      check = KFalse;
      final_check = KFalse;
      break;
    }
    count++;
  }

  histo_checker->init();
  if( check )
    cout << "\t####### HISTO_CMD END OK! #######" << endl;

  cout << "\tNow Logger::end() " << endl;
  check = KTrue;
  sleep(1);
  logger_checker->init();
  timeout->setTime( 1000 );
  logger_control->control("end");
  timeout->setTime( 0 );
  while( logger_checker->check() ){
    
    if( count == 20000 ){
      cout << "\tLOGGER:\tCMD NOT REACHED\n";
      check = KFalse;
      final_check = KFalse;
      break;
//      return -1;
    }
    count++;
  }
  logger_checker->init();
  if( check )
    cout << "\t####### LOGGER_CMD END OK! #######" << endl;

  if( final_check ){

    KChar* timetxt;
    timetxt = time->getTimeInfo();
    cout << "\tRUN_END_TIME = " << timetxt << endl;
    setter->set("RUN_END_TIME",timetxt);
    cout << "\n\tRunSummary : comment ? \t:";
    cin.getline(comment,256);
    T473RunSummary * runsum;
    runsum = new T473RunSummary;
    runsum -> putRunInfo( );
    runsum -> putKonoeInfo( );
    runsum -> putNameInfo( );
    runsum -> putPedestalInfo( );
    runsum -> putLogInfo( );
    runsum -> putComment(comment);
    delete runsum;

  }

  strcpy(run_type, ext->getKCharValue("RUN_TYPE"));
  if( !strcmp(run_type,"Cosmic Ray Run") ){
    this->setADCPedestal_auto();
    //    this->setTDCPedestal_auto();
  }

  return (0);
  
}

KInt T473RunControl::shutdown(){

  KInt count = 0;
  KInt check = KTrue;
  KInt final_check = KTrue;
 

  cout << "\tT473RunControl::shutdown" << endl;
  timeout->setTime(2000);
  answer = daq_control->control("shutdown");
  timeout->setTime(0);
  if(answer){
    status = NONE;
#ifdef 	NORMAL_MODE
    status = IGNORE;
#endif
  }
  else {
    cout << "\tDAQ_CMD not REACED! \t TRY AGAIN." << endl;
    check = KFalse;
    final_check = KFalse;
//    return -1;
  }
  if( check )
    cout << "\t####### DAQ_CMD SHUTDOWN OK! #######" << endl;

  cout << "\tNow Histogram::shutdown() " << endl;
  check = KTrue;
  histo_checker->init();
  sleep(1);
  timeout->setTime( 1000 );
  histo_control->control("shutdown");
  timeout->setTime( 0 );
  while( histo_checker->check() ) {
    
    if( count == 10000 ){
      cout <<"\tHISTO:\tCMD NOT REACHED\n" ;
      check = KFalse;
      final_check = KFalse;
      break;
//      return -1;
    }
    count++;
  }

  if( check )
    cout << "\t####### HISTO_CMD SHUTDOWN OK! #######" << endl;

  cout << "\tNow Logger::shutdown() " << endl;
  check = KTrue;
  sleep(1);
  logger_checker->init();
  timeout->setTime( 1000 );
  logger_control->control("shutdown");
  timeout->setTime( 0 );
  while( logger_checker->check() ){
    
    if( count == 10000 ){
      cout << "\tLOGGER:\tCMD NOT REACHED\n";
      check = KFalse;
      final_check = KFalse;
      break;
//      return -1;
    }
    count++;
  }
  logger_checker->init();
  if( check )
    cout << "\t####### LOGGER_CMD SHUTDOWN OK! #######" << endl;

  if( final_check ){
  cout << "\n\t Goodby!! (^o^)|~  \n" << endl;
  
  exit(0);
  }
   
  return -1;
}

KInt T473RunControl::nsCheck(KChar* pitem){

  cout << pitem << " = " << ext->getKCharValue(pitem) << endl;

  return (0);

}

KInt T473RunControl::setADCPedestal_auto( ){

  KInt i,j;
  KInt peak;
  KChar pedtxt[256];
  KChar csi_adc_name[256];
  Konoe1DHistogramList * csi_adc_histo_list ;
  Konoe1DHistogram * csi_adc_histo ;

  KonoeHistogramCM * h_cm = new KonoeHistogramCM;
  KonoePhysicalValueCM * p_cm = new KonoePhysicalValueCM;

  strcpy(csi_adc_name,ext->getKCharValue("CSI_ADC_HISTONAME"));
  
  KonoeInputObjFile * file_csi_adc;
  file_csi_adc = new KonoeInputObjFile( csi_adc_name );
  file_csi_adc->installCM( h_cm );
  file_csi_adc->installCM( p_cm );
  csi_adc_histo_list = (Konoe1DHistogramList*)file_csi_adc->get( );
  
  csi_adc_histo_list->reset( );

  j = -1;

  for( i = 0 ; i < 25 ; i++){
    T473Process1DHistogram * processor;
    if( ( i / 16 ) * 16  == i )
      j++;
    
    csi_adc_histo = csi_adc_histo_list->next( );

    processor = new T473Process1DHistogram( csi_adc_histo );
    peak =  processor->getMaxBinNumber( 0 , 4096 );

    sprintf(pedtxt,"CSI_ADC_%d_%d_PED",j, i - ( j * 16 ));
    
    setter->set( pedtxt,peak );

    delete processor ;
  }

  delete h_cm ; 
  delete p_cm ;
  delete file_csi_adc;

}

KInt T473RunControl::setTDCPedestal_auto( ){

  KInt i,j;
  KInt peak;
  KChar pedtxt[256];
  KChar csi_tdc_name[256];
  Konoe1DHistogramList * csi_tdc_histo_list ;
  Konoe1DHistogram * csi_tdc_histo ;
  strcpy(csi_tdc_name,ext->getKCharValue("CSI_TDC_HISTONAME"));

  KonoeHistogramCM * h_cm = new KonoeHistogramCM;
  KonoePhysicalValueCM * p_cm = new KonoePhysicalValueCM;
  
  KonoeInputObjFile * file_csi_tdc;
  file_csi_tdc = new KonoeInputObjFile( csi_tdc_name );
  file_csi_tdc->installCM( h_cm );
  file_csi_tdc->installCM( p_cm );
  csi_tdc_histo_list = (Konoe1DHistogramList*)file_csi_tdc->get( );
  
  csi_tdc_histo_list->reset( );

  j = -1;

  for( i = 0 ; i < 25 ; i++){
    T473Process1DHistogram * processor;
    if( ( i / 8 ) * 8  == i )
      j++;
    
    csi_tdc_histo = csi_tdc_histo_list->next( );

    processor = new T473Process1DHistogram( csi_tdc_histo );
    peak =  processor->getMaxBinNumber( 0 , 4096 );

    sprintf(pedtxt,"CSI_TDC_%d_%d_PED",j, i - ( j * 8 ));
    
    setter->set( pedtxt,peak );

    delete processor ;
  }

  delete h_cm ; 
  delete p_cm ;
  delete file_csi_tdc;

}
