//=====================================================================
/*

    T473TclProcessControlAction.cc

    Version 0.0.1   13-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclProcessControlAction.hh>
#include <T473TclProcessControlPanel.hh>

#include <stdlib.h>


//==================== Constructor  ====================

T473TclProcessControlAction::T473TclProcessControlAction
                               (T473TclProcessControlPanel* pPanel)
{
  processControlPanel = pPanel;
  processController = 0;

  daqControl = 0;
  dataHubControl = 0;
  histControl = 0;
  reconstructControl = 0;
  loggerControl = 0;

  initProcessController();
}


//==================== Destructor  ====================

T473TclProcessControlAction::~T473TclProcessControlAction()
{ ; }


//==================== Initialize Process Controller  ====================
void T473TclProcessControlAction::initProcessController()
{
  daqControl = processControlPanel->getDAQControl();
  dataHubControl = processControlPanel->getDataHubControl();
  histControl = processControlPanel->getHistControl();
  reconstructControl = processControlPanel->getReconstructControl();
  loggerControl = processControlPanel->getLoggerControl();
}


//==================== Change Process  ====================

void T473TclProcessControlAction::changeProcessControl(KInt pProcess)
{
  selectedProcess = pProcess;

  switch(selectedProcess){
    case KDAQ:
      processControlPanel->createDAQControl();
      break;
    case KDATAHUB:
      processControlPanel->createDataHubControl();
      break;
    case KHISTOGRAM:
      processControlPanel->createHistogramControl();
      break;
    case KRECONSTRUCT:
      processControlPanel->createReconstructControl();
      break;
    case KLOGGER:
      processControlPanel->createLoggerControl();
      break;
    default:
      ;
  }
}


//==================== Remove Process Control Button  ====================

void T473TclProcessControlAction::forgetProcessControl()
{
  switch(selectedProcess){
    case KDAQ:
      processControlPanel->forgetDAQControl();
      break;
    case KDATAHUB:
      processControlPanel->forgetDataHubControl();
      break;
    case KHISTOGRAM:
      processControlPanel->forgetHistogramControl();
      break;
    case KRECONSTRUCT:
      processControlPanel->forgetReconstructControl();
      break;
    case KLOGGER:
      processControlPanel->forgetLoggerControl();
      break;
    default:
      ;
  }
}


//==================== Begin Button Action  ====================

void T473TclProcessControlAction::kBegin()
{
cout << "#########  kBegin : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("begin");
  }
  processControlPanel->kBegin();
}


//==================== End Button Action  ====================

void T473TclProcessControlAction::kEnd()
{
cout << "#########  kEnd : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("end");
  }
  processControlPanel->kEnd();
}


//==================== Pause Button Action  ====================

void T473TclProcessControlAction::kPause()
{
cout << "#########  kPause : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("pause");
  }
  processControlPanel->kPause();
}


//==================== Resume Button Action  ====================

void T473TclProcessControlAction::kResume()
{
cout << "#########  kResume : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("resume");
  }
  processControlPanel->kResume();
}


//==================== Shutdown Button Action  ====================

void T473TclProcessControlAction::kShutdown()
{
cout << "#########  kShutdown : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("shutdown");
  }
  processControlPanel->kShutdown();
}


//==================== Boot Button Action  ====================

void T473TclProcessControlAction::kBoot()
{
cout << "#########  kBoot : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("boot");
  }
  processControlPanel->kBoot();
}


//==================== NewFile Button Action  ====================

void T473TclProcessControlAction::kNewFile()
{
cout << "#########  kNewFile : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("new_file");
  }
  processControlPanel->kNewFile();
}


//==================== FileClose Button Action  ====================

void T473TclProcessControlAction::kFileClose()
{
cout << "#########  kFileClose : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("file_close");
  }
  processControlPanel->kFileClose();
}


//==================== Continue Button Action  ====================

void T473TclProcessControlAction::kContinue()
{
cout << "#########  kContinue : " << selectedProcess << endl;
  if(!getenv("NONAMESPACE")){
    processController->control("continue");
  }
  processControlPanel->kContinue();
}


//==================== Mouse Action by Button ====================

void T473TclProcessControlAction::mouseAction(KonoeTclButton* kbtn, KInt type, 
                                     KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
  if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
    if(strcmp(processControlPanel->getValue(), "daqProcess")==0)
        processController = daqControl;
    if(strcmp(processControlPanel->getValue(), "dataHubProcess")==0)
        processController = dataHubControl;
    if(strcmp(processControlPanel->getValue(), "histogramProcess")==0)
        processController = histControl;
    if(strcmp(processControlPanel->getValue(), "reconstructProcess")==0)
        processController = reconstructControl;
    if(strcmp(processControlPanel->getValue(), "loggerProcess")==0)
        processController = loggerControl;

    if(strcmp(kbtn->getName(), "beginProcessButton")==0) kBegin();
    else if(strcmp(kbtn->getName(), "endProcessButton")==0) kEnd();
    else if(strcmp(kbtn->getName(), "pauseProcessButton")==0) kPause();
    else if(strcmp(kbtn->getName(), "resumeProcessButton")==0) kResume();
    else if(strcmp(kbtn->getName(), "shutdownProcessButton")==0)kShutdown();
    else if(strcmp(kbtn->getName(), "bootProcessButton")==0) kBoot();
    else if(strcmp(kbtn->getName(), "newfileProcessButton")==0) kNewFile();
    else if(strcmp(kbtn->getName(), "filecloseProcessButton")==0) kFileClose();
    else if(strcmp(kbtn->getName(), "continueProcessButton")==0) kContinue();
    else if(strcmp(kbtn->getName(), "hubShutdownButton")==0) kShutdown();
    else if(strcmp(kbtn->getName(), "recShutdownButton")==0) kShutdown();
    else if(strcmp(kbtn->getName(), "hubBootButton")==0) kBoot();
    else if(strcmp(kbtn->getName(), "recBootButton")==0) kBoot();
  }
  }
}


//==================== Mouse Action by Radio Button ====================

void T473TclProcessControlAction::mouseAction(KonoeTclRadioButton* kbtn,
                            KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){

      forgetProcessControl();
      if(strcmp(kbtn->getName(), "daqProcess")==0)
        changeProcessControl( KDAQ );
      else if(strcmp(kbtn->getName(), "dataHubProcess")==0)
        changeProcessControl( KDATAHUB );
      else if(strcmp(kbtn->getName(), "histogramProcess")==0)
        changeProcessControl( KHISTOGRAM );
      else if(strcmp(kbtn->getName(), "reconstructProcess")==0)
        changeProcessControl( KRECONSTRUCT );
      else if(strcmp(kbtn->getName(), "loggerProcess")==0)
        changeProcessControl( KLOGGER );
    }
  }
}

//==================== Mouse Action by Check Button ====================

void T473TclProcessControlAction::mouseAction(KonoeTclCheckButton* kbtn,
                            KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if( atoi(kbtn->getValue()) )
        processControlPanel->expertMode();
      else
        processControlPanel->deSelect();
    }
  }
}
