//=====================================================================
/*

    T473TclProcessControlPanel.cc

    Version 0.0.1   12-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclProcessControlPanel.hh>
#include <T473TclRunControlPanel.hh>
#include <T473TclProcessControlAction.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>

#include <Namespace/T473NameSpaceExtractor.hh>

T473TclProcessControlPanel::T473TclProcessControlPanel(
                                      T473TclRunControlPanel* pRunControl)
{
  runControl = pRunControl;
  kwin = runControl->getWindow();
  strcpy(fontName, "courier");
  fontSize = 18;
  strcpy(backColor, "purple");

  processControlWindow = 0;
  beginProcessButton = 0;
  endProcessButton = 0;
  pauseProcessButton = 0;
  resumeProcessButton = 0;
  shutdownProcessButton = 0;
  bootProcessButton = 0;

  newfileProcessButton = 0;
  //newfileProcessEntry = 0;
  filecloseProcessButton = 0;
  continueProcessButton = 0;
  hubShutdownButton = 0;
  hubBootButton = 0;

  recShutdownButton = 0;
  recBootButton = 0;

  reconstructControlFrame = 0;
  dataHubControlFrame = 0;
  processControlFrame = 0;

  initProcessController();
}

T473TclProcessControlPanel::~T473TclProcessControlPanel()
{
}

void T473TclProcessControlPanel::initProcessController()
{
  daqControl = runControl->getDAQControl();
  dataHubControl = runControl->getDataHubControl();
  histControl = runControl->getHistControl();
  reconstructControl = runControl->getReconstructControl();
  loggerControl = runControl->getLoggerControl();
}

void T473TclProcessControlPanel::reConstruct()
{
  destroyWindow();
  initProcessControlFrame();
  initAction();
}

void T473TclProcessControlPanel::view()
{
  if(!processControlWindow){
    initProcessControlFrame();
    initAction();
  }
}


//====================   ====================

void T473TclProcessControlPanel::setDaq()
{
  initDefaultControl();
  createDAQControl();
  daqProcess->setSelect();
  processControlAction->setSelectedProcess(KDAQ);
}

void T473TclProcessControlPanel::setDataHub()
{
  initDefaultControl();
  createDataHubControl();
  dataHubProcess->setSelect();
  processControlAction->setSelectedProcess(KDATAHUB);
}

void T473TclProcessControlPanel::setHistogram()
{
  initDefaultControl();
  createHistogramControl();
  histogramProcess->setSelect();
  processControlAction->setSelectedProcess(KHISTOGRAM);
}

void T473TclProcessControlPanel::setReconstruct()
{
  initDefaultControl();
  createReconstructControl();
  reconstructProcess->setSelect();
  processControlAction->setSelectedProcess(KRECONSTRUCT);
}

void T473TclProcessControlPanel::setLogger()
{
  initDefaultControl();
  createLoggerControl();
  loggerProcess->setSelect();
  processControlAction->setSelectedProcess(KLOGGER);
}


//==================== Create Process Select Widget  ====================

void T473TclProcessControlPanel::initProcessControlFrame()
{
  processControlWindow = new KonoeTclSubWindow(kwin, "processControlWindow",
                                               "Process Control");


  processSelectFrame = new KonoeTclFrame(processControlWindow,
                                        "processSelectFrame");
  processSelectFrame->setBackGround(backColor);

  daqProcess = new KonoeTclRadioButton(processSelectFrame, "daqProcess",
                                       "DAQ     ",0,0);
  dataHubProcess = new KonoeTclRadioButton(processSelectFrame,
                                       "dataHubProcess", "Data Hub",0,1);
  histogramProcess = new KonoeTclRadioButton(processSelectFrame,
                                       "histogramProcess", "Histogram  ",1,0);
  reconstructProcess = new KonoeTclRadioButton(processSelectFrame,
                              "reconstructProcess", "Reconstruct",1,1);
  loggerProcess = new KonoeTclRadioButton(processSelectFrame,
                              "loggerProcess", "Logger  ",2,0);
  expertProcess = new KonoeTclCheckButton(processSelectFrame,
                              "expertProcess", "Expert  ",2,1);

  separateFrame = new KonoeTclFrame(processControlWindow, "separateFrame");



  daqProcess->setFont(fontName, fontSize, 1, 0, 0, 0);
  dataHubProcess->setFont(fontName, fontSize, 1, 0, 0, 0);
  histogramProcess->setFont(fontName, fontSize, 1, 0, 0, 0);
  reconstructProcess->setFont(fontName, fontSize, 1, 0, 0, 0);
  loggerProcess->setFont(fontName, fontSize, 1, 0, 0, 0);
  expertProcess->setFont(fontName, fontSize, 1, 0, 0, 0);

  daqProcess->setBackGround(backColor);
  dataHubProcess->setBackGround(backColor);
  histogramProcess->setBackGround(backColor);
  reconstructProcess->setBackGround(backColor);
  loggerProcess->setBackGround(backColor);
  expertProcess->setBackGround(backColor);

  daqProcess->setGroup("processControl");
  dataHubProcess->setGroup("processControl");
  histogramProcess->setGroup("processControl");
  reconstructProcess->setGroup("processControl");
  loggerProcess->setGroup("processControl");
  daqProcess->setSelect();

  daqProcess->setGridSticky("w");
  dataHubProcess->setGridSticky("w");
  histogramProcess->setGridSticky("w");
  reconstructProcess->setGridSticky("w");
  loggerProcess->setGridSticky("w");
  expertProcess->setGridSticky("w");

  separateFrame->setBorderWidth(3);
  separateFrame->setRelief(KRaised);
  separateFrame->setSize(0, 10);
  separateFrame->setBackGround("blue");
  
  processSelectFrame->setFrame();
  separateFrame->setFrame();

  separateFrame->fillX();
}


//==================== Definition of Process Widget Action  ====================

void T473TclProcessControlPanel::initAction()
{
  processControlAction = new T473TclProcessControlAction(this);

  daqProcess->setEventProcessor(processControlAction);
  dataHubProcess->setEventProcessor(processControlAction);
  histogramProcess->setEventProcessor(processControlAction);
  reconstructProcess->setEventProcessor(processControlAction);
  loggerProcess->setEventProcessor(processControlAction);
  expertProcess->setEventProcessor(processControlAction);

  processControlAction->addMouseAction(daqProcess);
  processControlAction->addMouseAction(dataHubProcess);
  processControlAction->addMouseAction(histogramProcess);
  processControlAction->addMouseAction(reconstructProcess);
  processControlAction->addMouseAction(loggerProcess);
  processControlAction->addMouseAction(expertProcess);

  processControlWindow->setEventProcessor(this);
  addWindowAction(processControlWindow);
}


//==================== Create Control Widget  ====================

void T473TclProcessControlPanel::initDefaultControl()
{
  processControlFrame
           = new KonoeTclFrame(processControlWindow, "processControlFrame");

  beginProcessButton = new KonoeTclButton(processControlFrame,
                              "beginProcessButton", "Begin");
  endProcessButton = new KonoeTclButton(processControlFrame,
                              "endProcessButton", "End");
  pauseProcessButton = new KonoeTclButton(processControlFrame,
                              "pauseProcessButton", "Pause");
  resumeProcessButton = new KonoeTclButton(processControlFrame,
                              "resumeProcessButton", "Resume");
  shutdownProcessButton = new KonoeTclButton(processControlFrame,
                              "shutdownProcessButton", "Shutdown");
  bootProcessButton = new KonoeTclButton(processControlFrame,
                          "bootProcessButton", "Boot");

  beginProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  endProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  pauseProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  resumeProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  shutdownProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  bootProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);

  beginProcessButton->fillX();
  endProcessButton->fillX();
  pauseProcessButton->fillX();
  resumeProcessButton->fillX();
  shutdownProcessButton->fillX();
  bootProcessButton->fillX();


  beginProcessButton->setEventProcessor(processControlAction);
  endProcessButton->setEventProcessor(processControlAction);
  pauseProcessButton->setEventProcessor(processControlAction);
  resumeProcessButton->setEventProcessor(processControlAction);
  shutdownProcessButton->setEventProcessor(processControlAction);
  bootProcessButton->setEventProcessor(processControlAction);

  processControlAction->addMouseAction(beginProcessButton);
  processControlAction->addMouseAction(endProcessButton);
  processControlAction->addMouseAction(pauseProcessButton);
  processControlAction->addMouseAction(resumeProcessButton);
  processControlAction->addMouseAction(shutdownProcessButton);
  processControlAction->addMouseAction(bootProcessButton);



  dataHubControlFrame
         = new KonoeTclFrame(processControlWindow, "dataHubControlFrame");

  newfileProcessButton = new KonoeTclButton(dataHubControlFrame,
                              "newfileProcessButton", "New File");
  //newfileProcessEntry = new KonoeTclEntry(dataHubControlFrame,
  //                            "newFileProcessEntry", 1, 0);
  filecloseProcessButton = new KonoeTclButton(dataHubControlFrame,
                              "filecloseProcessButton", "File Close");
  continueProcessButton = new KonoeTclButton(dataHubControlFrame,
                              "continueProcessButton", "Continue");
  hubShutdownButton = new KonoeTclButton(dataHubControlFrame,
                              "hubShutdownButton", "Shutdown");
  hubBootButton = new KonoeTclButton(dataHubControlFrame,
                              "hubBootButton", "Boot");
  
  newfileProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  //newfileProcessEntry->setFont(fontName, fontSize, 1, 0, 0, 0);
  filecloseProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  continueProcessButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  hubShutdownButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  hubBootButton->setFont(fontName, fontSize, 1, 0, 0, 0);

  //newfileProcessEntry->setBackGround("white");

  //filecloseProcessButton->setGridColumnspan(2);
  //continueProcessButton->setGridColumnspan(2);
  //hubShutdownButton->setGridColumnspan(2);
  //hubBootButton->setGridColumnspan(2);

  //newfileProcessButton->setGridSticky("news");
  //newfileProcessEntry->setGridSticky("news");
  //filecloseProcessButton->setGridSticky("news");
  //continueProcessButton->setGridSticky("news");
  //hubShutdownButton->setGridSticky("news");
  //hubBootButton->setGridSticky("news");

  newfileProcessButton->fillX();
  filecloseProcessButton->fillX();
  continueProcessButton->fillX();
  hubShutdownButton->fillX();
  hubBootButton->fillX();

  newfileProcessButton->setEventProcessor(processControlAction);
  filecloseProcessButton->setEventProcessor(processControlAction);
  continueProcessButton->setEventProcessor(processControlAction);
  hubShutdownButton->setEventProcessor(processControlAction);
  hubBootButton->setEventProcessor(processControlAction);

  processControlAction->addMouseAction(newfileProcessButton);
  processControlAction->addMouseAction(filecloseProcessButton);
  processControlAction->addMouseAction(continueProcessButton);
  processControlAction->addMouseAction(hubShutdownButton);
  processControlAction->addMouseAction(hubBootButton);



  reconstructControlFrame
         = new KonoeTclFrame(processControlWindow, "reconstructControlFrame");

  recShutdownButton
         = new KonoeTclButton(reconstructControlFrame, "recShutdownButton",
                              "Shutdown");
  recBootButton
         = new KonoeTclButton(reconstructControlFrame, "recBootButton",
                              "Boot");

  recShutdownButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  recShutdownButton->setGridSticky("news");
  recShutdownButton->fillX();

  recBootButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  recBootButton->setGridSticky("news");
  recBootButton->fillX();

  recShutdownButton->setEventProcessor(processControlAction);
  processControlAction->addMouseAction(recShutdownButton);
  recBootButton->setEventProcessor(processControlAction);
  processControlAction->addMouseAction(recBootButton);
}

//==================== Get Process Status  ====================
KInt T473TclProcessControlPanel::getStatus(KInt process)
{
  T473NameSpaceExtractor nsExtractor;
  KChar* charState;

  switch(process){
    case KDAQ:
      charState = nsExtractor.getKCharValue("DAQ_STATUS");
      break;
    case KDATAHUB:
      charState = nsExtractor.getKCharValue("DATAHUB_STATUS");
      break;
    case KHISTOGRAM:
      charState = nsExtractor.getKCharValue("HISTOGRAM_STATUS");
      break;
    case KRECONSTRUCT:
      charState = nsExtractor.getKCharValue("RECONSTRUCT_STATUS");
      break;
    case KLOGGER:
      charState = nsExtractor.getKCharValue("LOGGER_STATUS");
      break;
    default:
      ;
  }

  if(strcmp(charState, "standby") == 0)
    return KSTANDBY;
  else if(strcmp(charState, "busy") == 0)
    return KBUSY;
  else if(strcmp(charState, "idle") == 0)
    return KIDLE;
  else if(strcmp(charState, "none") == 0)
    return KNONE;
  else
    return KNONE;
}

//==================== Reconstruct Control Widget  ====================

void T473TclProcessControlPanel::expertMode()
{
  beginProcessButton->enable();
  endProcessButton->enable();
  resumeProcessButton->enable();
  shutdownProcessButton->enable();
  pauseProcessButton->enable();
  bootProcessButton->enable();
  hubBootButton->enable();
  hubShutdownButton->enable();
  recBootButton->enable();
  recShutdownButton->enable();
}

void T473TclProcessControlPanel::deSelect()
{
  beginProcessButton->disable();
  endProcessButton->disable();
  resumeProcessButton->disable();
  shutdownProcessButton->disable();
  pauseProcessButton->disable();
  bootProcessButton->disable();
  hubBootButton->disable();
  recBootButton->disable();
}

void T473TclProcessControlPanel::createDAQControl()
{
  processControlFrame->setFrame();
  processControlFrame->fillX();
//  cout << "###  createDAQControl" << endl;

  state = getStatus(KDAQ);

  switch(state){
    case KSTANDBY:
      kEnd();
      break;
    case KBUSY:
      kBegin();
      break;
    case KIDLE:
      kPause();
      break;
    case KNONE:
      kShutdown();
      break;
  }

  expertProcess->deSelect();
}

void T473TclProcessControlPanel::createDataHubControl()
{
  dataHubControlFrame->setFrame();
  dataHubControlFrame->fillX();
//  cout << "###  createDataHubControl" << endl;

  state = getStatus(KDATAHUB);

  switch(state){
    case KSTANDBY:
      hubShutdownButton->enable();
      hubBootButton->disable();
      break;
    case KBUSY:
      hubShutdownButton->enable();
      hubBootButton->disable();
      break;
    case KIDLE:
      hubShutdownButton->enable();
      hubBootButton->disable();
      break;
    case KNONE:
      hubShutdownButton->disable();
      hubBootButton->enable();
      break;
  }

  expertProcess->deSelect();
}

void T473TclProcessControlPanel::createHistogramControl()
{
  processControlFrame->setFrame();
  processControlFrame->fillX();
//  cout << "###  createHistogramControl" << endl;

  state = getStatus(KHISTOGRAM);

  switch(state){
    case KSTANDBY:
      kEnd();
      break;
    case KBUSY:
      kBegin();
      break;
    case KIDLE:
      kPause();
      break;
    case KNONE:
      kShutdown();
      break;
  }

  expertProcess->deSelect();
}

void T473TclProcessControlPanel::createReconstructControl()
{
  reconstructControlFrame->setFrame();
  reconstructControlFrame->fillX();
//  cout << "###  createReconstructControl" << endl;

  state = getStatus(KRECONSTRUCT);

  switch(state){
    case KSTANDBY:
      recShutdownButton->enable();
      recBootButton->disable();
      break;
    case KBUSY:
      recShutdownButton->enable();
      recBootButton->disable();
      break;
    case KIDLE:
      recShutdownButton->enable();
      recBootButton->disable();
      break;
    case KNONE:
      recShutdownButton->disable();
      recBootButton->enable();
      break;
  }

  expertProcess->deSelect();
}

void T473TclProcessControlPanel::createLoggerControl()
{
  processControlFrame->setFrame();
  processControlFrame->fillX();
//  cout << "###  createLoggerControl" << endl;

  state = getStatus(KLOGGER);

  switch(state){
    case KSTANDBY:
      kEnd();
      break;
    case KBUSY:
      kBegin();
      break;
    case KIDLE:
      kPause();
      break;
    case KNONE:
      kShutdown();
      break;
  }

  expertProcess->deSelect();
}


//==================== Remove Control Widget  ====================

void T473TclProcessControlPanel::forgetAll()
{
  forgetDAQControl();
  forgetDataHubControl();
  forgetHistogramControl();
  forgetReconstructControl();
  forgetLoggerControl();
}

void T473TclProcessControlPanel::forgetDAQControl()
{
  processControlFrame->setForget();
//  cout << "###  forgetDAQControl" << endl;
}

void T473TclProcessControlPanel::forgetDataHubControl()
{
  dataHubControlFrame->setForget();
//  cout << "###  forgetDataHubControl" << endl;
}

void T473TclProcessControlPanel::forgetHistogramControl()
{
  processControlFrame->setForget();
//  cout << "###  forgetHistogramControl" << endl;
}

void T473TclProcessControlPanel::forgetReconstructControl()
{
  reconstructControlFrame->setForget();
//  cout << "###  forgetReconstructControl" << endl;
}

void T473TclProcessControlPanel::forgetLoggerControl()
{
  processControlFrame->setForget();
//  cout << "###  forgetLoggerControl" << endl;
}


//==================== Destroy Window  ====================

void T473TclProcessControlPanel::destroyWindow()
{
  delete daqProcess;
  delete dataHubProcess;
  delete histogramProcess;
  delete reconstructProcess;
  delete loggerProcess;
  delete processSelectFrame;
  delete separateFrame;

  delete beginProcessButton;
  delete endProcessButton;
  delete pauseProcessButton;
  delete resumeProcessButton;
  delete shutdownProcessButton;
  delete bootProcessButton;

  delete newfileProcessButton;
  //delete newfileProcessEntry;
  delete filecloseProcessButton;
  delete continueProcessButton;
  delete hubShutdownButton;
  delete hubBootButton;

  delete recShutdownButton;
  delete recBootButton;

  delete reconstructControlFrame;
  delete dataHubControlFrame;
  delete processControlFrame;

  processControlWindow->destroy();
  delete processControlWindow;
  processControlWindow = 0;
}


//==================== Process Widget Action  ====================

void T473TclProcessControlPanel::kBegin()
{
  beginProcessButton->disable();
  endProcessButton->disable();
  resumeProcessButton->disable();
  shutdownProcessButton->disable();
//  beginProcessButton->after(2500);
  pauseProcessButton->enable();
  bootProcessButton->disable();
}

void T473TclProcessControlPanel::kEnd()
{
  endProcessButton->disable();
  resumeProcessButton->disable();
  pauseProcessButton->disable();
//  beginProcessButton->after(2500);
  beginProcessButton->enable();
  shutdownProcessButton->enable();
  bootProcessButton->disable();
}

void T473TclProcessControlPanel::kPause()
{
  beginProcessButton->disable();
  pauseProcessButton->disable();
  shutdownProcessButton->disable();
//  beginProcessButton->after(2500);
  endProcessButton->enable();
  resumeProcessButton->enable();
  bootProcessButton->disable();
}

void T473TclProcessControlPanel::kResume()
{
  beginProcessButton->disable();
  endProcessButton->disable();
  resumeProcessButton->disable();
  shutdownProcessButton->disable();
//  beginProcessButton->after(2500);
  pauseProcessButton->enable();
  bootProcessButton->disable();
}

void T473TclProcessControlPanel::kShutdown()
{
  beginProcessButton->disable();
  endProcessButton->disable();
  resumeProcessButton->disable();
  pauseProcessButton->disable();
  shutdownProcessButton->disable();
  bootProcessButton->enable();
}

void T473TclProcessControlPanel::kBoot()
{
  beginProcessButton->enable();
  endProcessButton->disable();
  resumeProcessButton->disable();
  pauseProcessButton->disable();
  shutdownProcessButton->disable();
  bootProcessButton->disable();
}

void T473TclProcessControlPanel::kNewFile()
{
}

void T473TclProcessControlPanel::kFileClose()
{
}

void T473TclProcessControlPanel::kContinue()
{
}


//==================== Window Action  ====================

void T473TclProcessControlPanel::windowAction(KonoeTclWindow* pwin, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
//    cout << "####  called windowAction ProcessControlPanel" << endl;
    destroyWindow();
  }
}
