//=====================================================================
/*

    T473TclProcessControlPanel.hh

    Version 0.0.1   12-Nov-2000 T. Hosoi

*/
//====================================================================

#ifndef __T473TCLPROCESSCONTROLPANEL_HH
#define __T473TCLPROCESSCONTROLPANEL_HH

#include <ui/KonoeTclWidget.hh>
#include <ui/KonoeTclAction.hh>
class T473TclRunControlPanel;
class T473TclProcessControlAction;
class KonoeTclWindow;
class KonoeTclSubWindow;
class T473DAQControl;
class T473DataHubControl;
class T473HistoControl;
class T473ReconstructControl;
class T473LoggerControl;

class T473TclProcessControlPanel : public KonoeTclEventProcessor,
                                   public KonoeTclWindowAction
{
  public:
    T473TclProcessControlPanel(T473TclRunControlPanel* prunControl);
    ~T473TclProcessControlPanel();

    void reConstruct();

// For mouseAction
    void kBegin();
    void kEnd();
    void kPause();
    void kResume();
    void kShutdown();
    void kBoot();
    void kNewFile();
    void kFileClose();
    void kContinue();

    void view();
    void setDaq();
    void setDataHub();
    void setHistogram();
    void setReconstruct();
    void setLogger();
    KChar* getValue() const { return daqProcess->getValue(); };

// Get Process Controller
    T473DAQControl* getDAQControl() { return daqControl; }
    T473DataHubControl* getDataHubControl() { return dataHubControl; }
    T473HistoControl* getHistControl() { return histControl; }
    T473ReconstructControl* getReconstructControl()
                                     { return reconstructControl; }
    T473LoggerControl* getLoggerControl() { return loggerControl; }


// Widget create and forget
  public:
    void createDAQControl();
    void createDataHubControl();
    void createHistogramControl();
    void createReconstructControl();
    void createLoggerControl();

    void forgetAll();
    void forgetDAQControl();
    void forgetDataHubControl();
    void forgetHistogramControl();
    void forgetReconstructControl();
    void forgetLoggerControl();


// Expert Mode
  public:
    void expertMode();
    void deSelect();

// Initialize Widget and Action
  private:
    void initProcessController();
    void initProcessControlFrame();
    void initAction();
    void initDefaultControl();
    void destroyWindow();
    void windowAction(KonoeTclWindow* kcv, XEvent* xev);

  private:
    enum { KSTANDBY, KBUSY, KIDLE, KNONE };
    KInt getStatus(KInt process);
    KInt state;

  private:
    KChar fontName[256];
    KInt fontSize;
    KChar backColor[256];

    KonoeTclWindow* kwin;
    T473TclRunControlPanel* runControl;
    T473TclProcessControlAction* processControlAction;
    KonoeTclSubWindow* processControlWindow;

    enum { KDAQ, KDATAHUB, KHISTOGRAM, KRECONSTRUCT, KLOGGER };

// Process select frame
    KonoeTclFrame*       processSelectFrame;
    KonoeTclRadioButton* daqProcess;
    KonoeTclRadioButton* dataHubProcess;
    KonoeTclRadioButton* histogramProcess;
    KonoeTclRadioButton* reconstructProcess;
    KonoeTclRadioButton* loggerProcess;
    KonoeTclCheckButton* expertProcess;
    KonoeTclFrame*       separateFrame;

// Process control frame;
    KonoeTclFrame*  processControlFrame;
    KonoeTclFrame*  dataHubControlFrame;
    KonoeTclFrame*  reconstructControlFrame;
    KonoeTclButton* beginProcessButton;
    KonoeTclButton* endProcessButton;
    KonoeTclButton* pauseProcessButton;
    KonoeTclButton* resumeProcessButton;
    KonoeTclButton* shutdownProcessButton;
    KonoeTclButton* bootProcessButton;

    KonoeTclButton* newfileProcessButton;
    //KonoeTclEntry*  newfileProcessEntry;
    KonoeTclButton* filecloseProcessButton;
    KonoeTclButton* continueProcessButton;
    KonoeTclButton* hubShutdownButton;
    KonoeTclButton* hubBootButton;

    KonoeTclButton* recShutdownButton;
    KonoeTclButton* recBootButton;

// T473 Process Control
  T473DAQControl* daqControl;
  T473DataHubControl* dataHubControl;
  T473HistoControl* histControl;
  T473ReconstructControl* reconstructControl;
  T473LoggerControl* loggerControl;
};

#endif
