//=====================================================================
/*

    T473TclRunControlAction.cc

    Version 0.0.1   12-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclRunControlAction.hh>
#include <Namespace/T473NameSpaceExtractor.hh>
#include <stdlib.h>

T473TclRunControlAction::T473TclRunControlAction(
                          T473TclRunControlPanel* prunControlPanel)
{
  runControlPanel = prunControlPanel;
  KonoeTclWindow* kwin = runControlPanel->getWindow();

  if( !getenv("NONAMESPACE") )
  initProcessController();
}

T473TclRunControlAction::~T473TclRunControlAction()
{ ; }

void T473TclRunControlAction::initProcessController()
{
  T473NameSpaceExtractor* nsExtractor = new T473NameSpaceExtractor;
  KInt cmd_port;

////////// Data Hub Controller //////////
  cmd_port = nsExtractor->getKIntValue("DATAHUB_OUTPUT_PORT");

  if( cmd_port == -1){
    cerr<<"DATAHUB_OUTPUT_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
  }
  else{
    cout << "DATAHUB_OUTPUT_PORT = " << cmd_port << endl;
    dataHubControl = new T473DataHubControl(cmd_port);
    //dataHubControl->control("boot");
  }

////////// DAQ Controller //////////
  cmd_port = nsExtractor->getKIntValue("DAQ_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"DAQ_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
  }
  else{
    cout << "DAQ_CMD_PORT = " << cmd_port << endl;
    daqControl = new T473DAQControl(cmd_port);
    //daqControl->control("boot");
  }


////////// Histogram Controller //////////
  cmd_port = nsExtractor->getKIntValue("HISTO_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"HISTO_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
  }
  else {
    cout << "HISTO_CMD_PORT = " << cmd_port << endl;
    histControl = new T473HistoControl(cmd_port);
    //histControl->control("boot");
  }

////////// Reconstruct Controller //////////
  cmd_port = nsExtractor->getKIntValue("RECONSTRUCT_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"RECONSTRUCT_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
  }
  else {
    cout << "RECONSTRUCT_CMD_PORT = " << cmd_port << endl;
    reconstructControl = new T473ReconstructControl(cmd_port);
    //reconstructControl->control("boot");
  }

////////// Logger Controller //////////
  cmd_port = nsExtractor->getKIntValue("LOGGER_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"LOGGER_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
  }
  else {
    cout << "LOGGER_CMD_PORT = " << cmd_port << endl;
    loggerControl = new T473LoggerControl(cmd_port);
    //loggerControl->control("boot");
  }

  delete nsExtractor;
}

void T473TclRunControlAction::kBegin()
{
  runControlPanel->kBegin();
}

void T473TclRunControlAction::kEnd()
{
  runControlPanel->kEnd();
}

void T473TclRunControlAction::kPause()
{
  runControlPanel->kPause();
}

void T473TclRunControlAction::kResume()
{
  runControlPanel->kResume();
}

void T473TclRunControlAction::kShutdown()
{
  runControlPanel->kShutdown();
}

void T473TclRunControlAction::mouseAction(KonoeTclButton* kbtn, KInt type,
                                      KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if(strcmp(kbtn->getName(), "beginButton")==0) kBegin();
      if(strcmp(kbtn->getName(), "endButton")==0) kEnd();
      if(strcmp(kbtn->getName(), "pauseButton")==0) kPause();
      if(strcmp(kbtn->getName(), "resumeButton")==0) kResume();
      if(strcmp(kbtn->getName(), "shutdownButton")==0) kShutdown();
    }
  }
}

void T473TclRunControlAction::mouseAction(KonoeTclLabel* klbl, KInt type,
                                     KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    static KInt clickedTime = 0;
    if( abs(clickedTime - time) < 300 ){ // Double Click
      if( strstr(klbl->getName(), "daq") )
                 runControlPanel->viewProcessControlPanel(KDaq);

      if( strstr(klbl->getName(), "dataHub") )
                 runControlPanel->viewProcessControlPanel(KDataHub);

      if( strstr(klbl->getName(), "histogram") )
                 runControlPanel->viewProcessControlPanel(KHistogram);

      if( strstr(klbl->getName(), "reconstruct") )
                 runControlPanel->viewProcessControlPanel(KReconstruct);

      if( strstr(klbl->getName(), "logger") )
                 runControlPanel->viewProcessControlPanel(KLogger);
    }
    clickedTime = time;
  }
}

void T473TclRunControlAction::mouseAction(KonoeTclCheckButton* kbtn,
                             KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      runControlPanel->readyNow();
    }
  }
}

void T473TclRunControlAction::mouseAction(KonoeTclCanvas* kbtn,
                             KInt type, KInt button, Time time, KInt x, KInt y)
{
  static KBool flag = KTrue;
  static KInt clickedTime2 = 0;

  if(type == KMouseRelease && button == RButton){
    if( abs(clickedTime2 - time) < 300 ){ // Double Click
      if(flag){
        runControlPanel->setImage();
        flag = KFalse;
      }
      else{
        runControlPanel->setImage2();
        flag = KTrue;
      }
    }
    clickedTime2 = time;
  }
}

void T473TclRunControlAction::keyAction(KonoeTclCheckButton* kcv, KInt type,
                                                     KeySym keysym, KInt state)
{
  if( type == KKeyRelease ){
    if(keysym == XK_KP_Enter || keysym == XK_Return){
      runControlPanel->readyNow();
    }
  }
}
