//=====================================================================
/*

    T473TclRunControlPanel.cc

    Version 0.0.1   11-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclRunControlPanel.hh>
#include <T473TclProcessControlPanel.hh>
#include <T473TclRunSummaryPanel.hh>
#include <T473TclRunControlAction.hh>
#include <Logger/T473LogSender.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <stdlib.h>
#include <ctype.h>


//#include <system/KonoeClientSocket.hh>
#include <system/KonoeTime.hh>
#include<system/KonoeMicroSleep.hh>

//////////////////// Constructor ////////////////////

T473TclRunControlPanel::T473TclRunControlPanel(KonoeTclWindow* pwin)
{
  strcpy(fontName, "courier");
  fontSize = 18;
  strcpy(backColor1, "#ffffcc");
  strcpy(backColor2, "#6688ee");
  strcpy(backColor3, "#ffbbbb");
  strcpy(backColor4, "#55ee99");


  kwin = pwin;
//  socket1 = 0;
//  socket1 = new KonoeClientSocket("konoe1", 30300);
//  socket1->connect();

  state = KSHUTDOWN;

  initGlobalFrame();
  initDateFrame();
  initRunStateFrame();
  initProcessStateFrame();
  initLogoFrame();
  initRunTypeFrame();
  initRunInfoFrame();
  initParticleEnergyFrame();
  initStorageFrame();
  initCherenkovFrame();
  initCsITempFrame();
  initPositionFrame();
  initAngleFrame();
  initShutterFrame();
  initNameRegisterFrame();
  initControlButtonFrame();
  initLoggerFrame();

  globalFrame->setFrame();
  leftFrame->setLayoutTypeWithPack("left");
  rightFrame->setLayoutTypeWithPack("rihgt");
//  loggerFrame->setLayoutTypeWithPack("bottom");

  initAction();
  initProcessController();

  processControlPanel = new T473TclProcessControlPanel(this);
  runSummaryPanel = new T473TclRunSummaryPanel(this);

  nsExtractor = new T473NameSpaceExtractor;
  nsSetter = new T473NameSpaceSetter;

  setNameSpaceItem();

  kwin->setEventProcessor(this);
  addKeyAction(kwin);
  kwin->setTakeFocus();

  T473LogSender logsend;
  logsend.send("T473controller is executed.", T473LogSender::SUCCESS);
}

//////////////////// Destructor //////////////////// 

T473TclRunControlPanel::~T473TclRunControlPanel()
{ ; }

//////////////////// initialize process controller //////////////////// 

void T473TclRunControlPanel::initProcessController()
{
  daqControl = runControlAction->getDAQControl();
  dataHubControl = runControlAction->getDataHubControl();
  histControl = runControlAction->getHistControl();
  reconstructControl = runControlAction->getReconstructControl();
  loggerControl = runControlAction->getLoggerControl();
}

//////////////////// initialize action method //////////////////// 

void T473TclRunControlPanel::initAction()
{
  runControlAction = new T473TclRunControlAction(this);

  globalFrame->setEventProcessor(this);
  addTimer(globalFrame, 1000);

  beginButton->setEventProcessor(runControlAction);
  endButton->setEventProcessor(runControlAction);
  pauseButton->setEventProcessor(runControlAction);
  resumeButton->setEventProcessor(runControlAction);
  shutdownButton->setEventProcessor(runControlAction);

  daqStateLabel->setEventProcessor(runControlAction);
  daqStateDrawLabel->setEventProcessor(runControlAction);
  dataHubStateLabel->setEventProcessor(runControlAction);
  dataHubStateDrawLabel->setEventProcessor(runControlAction);
  hisotgramStateLabel->setEventProcessor(runControlAction);
  hisotgramStateDrawLabel->setEventProcessor(runControlAction);
  reconstructStateLabel->setEventProcessor(runControlAction);
  reconstructStateDrawLabel->setEventProcessor(runControlAction);
  loggerStateLabel->setEventProcessor(runControlAction);
  loggerStateDrawLabel->setEventProcessor(runControlAction);

  runTypeCheckButton->setEventProcessor(runControlAction);
  runInfoCheckButton->setEventProcessor(runControlAction);
  particleEnergyCheckButton->setEventProcessor(runControlAction);
  storageCheckButton->setEventProcessor(runControlAction);
  cherenkovCheckButton->setEventProcessor(runControlAction);
  csiTempCheckButton->setEventProcessor(runControlAction);
  positionCheckButton->setEventProcessor(runControlAction);
  angleCheckButton->setEventProcessor(runControlAction);
  shutterCheckButton->setEventProcessor(runControlAction);

  konoeLogoCanvas->setEventProcessor(runControlAction);

  runControlAction->addMouseAction(beginButton);
  runControlAction->addMouseAction(endButton);
  runControlAction->addMouseAction(pauseButton);
  runControlAction->addMouseAction(resumeButton);
  runControlAction->addMouseAction(shutdownButton);

  runControlAction->addMouseAction(daqStateLabel);
  runControlAction->addMouseAction(daqStateDrawLabel);
  runControlAction->addMouseAction(dataHubStateLabel);
  runControlAction->addMouseAction(dataHubStateDrawLabel);
  runControlAction->addMouseAction(hisotgramStateLabel);
  runControlAction->addMouseAction(hisotgramStateDrawLabel);
  runControlAction->addMouseAction(reconstructStateLabel);
  runControlAction->addMouseAction(reconstructStateDrawLabel);
  runControlAction->addMouseAction(loggerStateLabel);
  runControlAction->addMouseAction(loggerStateDrawLabel);

  runControlAction->addMouseAction(runTypeCheckButton);
  runControlAction->addMouseAction(runInfoCheckButton);
  runControlAction->addMouseAction(particleEnergyCheckButton);
  runControlAction->addMouseAction(storageCheckButton);
  runControlAction->addMouseAction(cherenkovCheckButton);
  runControlAction->addMouseAction(csiTempCheckButton);
  runControlAction->addMouseAction(positionCheckButton);
  runControlAction->addMouseAction(angleCheckButton);
  runControlAction->addMouseAction(shutterCheckButton);

  runControlAction->addKeyAction(runTypeCheckButton);
  runControlAction->addKeyAction(runInfoCheckButton);
  runControlAction->addKeyAction(particleEnergyCheckButton);
  runControlAction->addKeyAction(storageCheckButton);
  runControlAction->addKeyAction(cherenkovCheckButton);
  runControlAction->addKeyAction(csiTempCheckButton);
  runControlAction->addKeyAction(positionCheckButton);
  runControlAction->addKeyAction(angleCheckButton);
  runControlAction->addKeyAction(shutterCheckButton);

  runControlAction->addMouseAction(konoeLogoCanvas);
}

//////////////////// initialize global frame //////////////////// 

void T473TclRunControlPanel::initGlobalFrame()
{
  globalFrame = new KonoeTclFrame(kwin, "globalFrame");
  globalFrame->setBackGround(backColor1);
  globalFrame->fillBoth();
  globalFrame->setExpand();
  globalFrame->setLayoutType("left");

  leftFrame = new KonoeTclFrame(globalFrame, "leftFrame");
  leftFrame->setBackGround(backColor1);
  leftFrame->fillBoth();
  leftFrame->setExpand();

  rightFrame = new KonoeTclFrame(globalFrame, "rightFrame");
  rightFrame->setBackGround(backColor2);
  rightFrame->fillBoth();
}

//////////////////// initialize date frame //////////////////// 

void T473TclRunControlPanel::initDateFrame()
{
  dateFrame = new KonoeTclFrame(leftFrame, "dateFrame",0,0);
  dateFrame->setBackGround(backColor1);
  dateFrame->setGridSticky("news");
  dateFrame->setGridPadx(5);
  dateFrame->setGridPady(5);

  dateLabel = new KonoeTclLabel(dateFrame, "dateLabel", "",0,0);
  dateLabel->setBackGround(backColor1);
  dateLabel->setGridSticky("news");
  dateLabel->setFont("charter", fontSize+4, 1, 1, 0, 0);

  getDate();
}

//////////////////// initialize run state frame //////////////////// 

void T473TclRunControlPanel::initRunStateFrame()
{
  runStateFrame = new KonoeTclFrame(leftFrame, "runStateFrame",0,1);
  runStateFrame->setBackGround(backColor3);
  runStateFrame->setBorderWidth(3);
  runStateFrame->setRelief(KRidge);
  runStateFrame->setGridPadx(30);
  runStateFrame->setGridPady(5);
  runStateFrame->setGridIpadx(30);
  runStateFrame->setGridIpady(15);
  runStateFrame->setGridSticky("we");

  runNumberLabel = new KonoeTclLabel(runStateFrame, "runNumberLabel",
                                     "Run Number",0,0);
  runNumberDrawLabel = new KonoeTclLabel(runStateFrame,
                                     "runNumberDrawLabel", "0", 1,0);
  eventNumberLabel = new KonoeTclLabel(runStateFrame, "eventNumberLabel",
                                     "Event Number",0,1);
  eventNumberDrawLabel = new KonoeTclLabel(runStateFrame,
                                     "eventNumberDrawLabel", "0", 1,1);
  startTimeLabel = new KonoeTclLabel(runStateFrame, "startTimeLabel",
                                     "Start Time",0,2);
  startTimeDrawLabel = new KonoeTclLabel(runStateFrame, "startTimeDrawLabel",
                                         "none",1,2);
  endTimeLabel = new KonoeTclLabel(runStateFrame, "endTimeLabel",
                                   "End Time",0,3);
  endTimeDrawLabel = new KonoeTclLabel(runStateFrame, "endTimeDrawLabel",
                                       "none",1,3);

  rateFrame = new KonoeTclFrame(runStateFrame, "rateFrame",0,4);
  rateFrame->setBackGround(backColor3);
  rateFrame->setBorderWidth(2);
  rateFrame->setRelief(KRidge);
  rateFrame->setGridSticky("news");
  rateFrame->setGridColumnspan(2);
  rateFrame->setGridIpady(10);

  lastSpillLabel = new KonoeTclLabel(rateFrame, "lastSpillLabel",
                                    "Last Spill",1,0);
  averageLabel = new KonoeTclLabel(rateFrame, "averageLabel",
                                   " Average ",2,0);
  deadTimeLabel = new KonoeTclLabel(rateFrame, "deadTimeLabel",
                                    "Dead Time",0,1);
  deadTimeSpillLabel = new KonoeTclLabel(rateFrame, "deadTimeSpillLabel",
                                        "  \\[%\\]",1,1);
  deadTimeTotalLabel = new KonoeTclLabel(rateFrame, "deadTimeTotalLabel",
                                         "  \\[%\\]", 2,1);
  daqSpeedLabel = new KonoeTclLabel(rateFrame, "daqSpeedLabel",
                                   "DAQ Speed",0,2);
  daqSpeedSpillLabel = new KonoeTclLabel(rateFrame, "daqSpeedSpillLabel",
                                       " \\[Hz\\]",1,2);
  daqSpeedTotalLabel = new KonoeTclLabel(rateFrame, "daqSpeedTotalLabel",
                                        " \\[Hz\\]",2,2);


  runNumberLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  runNumberDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  eventNumberLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  eventNumberDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  averageLabel->setFont(fontName, fontSize, 1, 1, 0, 0);
  lastSpillLabel->setFont(fontName, fontSize, 1, 1, 0, 0);
  deadTimeLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  deadTimeSpillLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  deadTimeTotalLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  daqSpeedLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  daqSpeedSpillLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  daqSpeedTotalLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  startTimeLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  startTimeDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  endTimeLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  endTimeDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);

  runNumberLabel->setBackGround(backColor3);
  runNumberDrawLabel->setBackGround(backColor3);
  eventNumberLabel->setBackGround(backColor3);
  eventNumberDrawLabel->setBackGround(backColor3);
  averageLabel->setBackGround(backColor3);
  lastSpillLabel->setBackGround(backColor3);
  deadTimeLabel->setBackGround(backColor3);
  deadTimeSpillLabel->setBackGround(backColor3);
  deadTimeTotalLabel->setBackGround(backColor3);
  daqSpeedLabel->setBackGround(backColor3);
  daqSpeedSpillLabel->setBackGround(backColor3);
  daqSpeedTotalLabel->setBackGround(backColor3);
  startTimeLabel->setBackGround(backColor3);
  startTimeDrawLabel->setBackGround(backColor3);
  endTimeLabel->setBackGround(backColor3);
  endTimeDrawLabel->setBackGround(backColor3);

  averageLabel->setForeGround("#0000ff");
  lastSpillLabel->setForeGround("#0000ff");

  runNumberLabel->setGridSticky("w");
  runNumberDrawLabel->setGridSticky("e");
  eventNumberLabel->setGridSticky("w");
  eventNumberDrawLabel->setGridSticky("e");
  deadTimeLabel->setGridSticky("w");
  deadTimeSpillLabel->setGridSticky("e");
  deadTimeTotalLabel->setGridSticky("e");
  daqSpeedLabel->setGridSticky("w");
  daqSpeedSpillLabel->setGridSticky("e");
  daqSpeedTotalLabel->setGridSticky("e");
  startTimeLabel->setGridSticky("w");
  startTimeDrawLabel->setGridSticky("e");
  endTimeLabel->setGridSticky("w");
  endTimeDrawLabel->setGridSticky("e");

  averageLabel->setGridPadx(10);
  lastSpillLabel->setGridPadx(10);
}

//////////////////// initialize process state frame //////////////////// 

void T473TclRunControlPanel::initProcessStateFrame()
{
  processStateFrame = new KonoeTclFrame(leftFrame, "processStateFrame",0,2);
  processStateFrame->setBackGround(backColor4);
  processStateFrame->setBorderWidth(3);
  processStateFrame->setRelief(KRidge);
  processStateFrame->setGridPadx(30);
  processStateFrame->setGridPady(15);
  processStateFrame->setGridIpadx(30);
  processStateFrame->setGridIpady(15);
  processStateFrame->setGridSticky("we");

  daqStateLabel = new KonoeTclLabel(processStateFrame, "daqStateLabel",
                                    "DAQ",0,0);
  daqStateDrawLabel = new KonoeTclLabel(processStateFrame, "daqStateDrawLabel",
                                        "none",1,0);
  dataHubStateLabel = new KonoeTclLabel(processStateFrame, "dataHubStateLabel",
                                        "Data Hub",0,1);
  dataHubStateDrawLabel = new KonoeTclLabel(processStateFrame,
                                        "dataHubStateDrawLabel", "none",1,1);
  hisotgramStateLabel = new KonoeTclLabel(processStateFrame,
                        "histogramStateLabel", "Histogram",0,2);
  hisotgramStateDrawLabel = new KonoeTclLabel(processStateFrame,
                        "histogramStateDrawLabel", "none",1,2);
  reconstructStateLabel = new KonoeTclLabel(processStateFrame,
                        "reconstructStateLabel", "Reconstruct",0,3);
  reconstructStateDrawLabel = new KonoeTclLabel(processStateFrame,
                        "reconstructStateDrawLabel", "none",1,3);
  loggerStateLabel = new KonoeTclLabel(processStateFrame, "loggerStateLabel",
                                       "Logger",0,4);
  loggerStateDrawLabel = new KonoeTclLabel(processStateFrame,
                             "loggerStateDrawLabel", "none",1,4);

  daqStateLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  daqStateDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  dataHubStateLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  dataHubStateDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  hisotgramStateLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  hisotgramStateDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  reconstructStateLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  reconstructStateDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  loggerStateLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  loggerStateDrawLabel->setFont(fontName, fontSize, 1, 0, 0, 0);

  daqStateLabel->setBackGround(backColor4);
  daqStateDrawLabel->setBackGround(backColor4);
  dataHubStateLabel->setBackGround(backColor4);
  dataHubStateDrawLabel->setBackGround(backColor4);
  hisotgramStateLabel->setBackGround(backColor4);
  hisotgramStateDrawLabel->setBackGround(backColor4);
  reconstructStateLabel->setBackGround(backColor4);
  reconstructStateDrawLabel->setBackGround(backColor4);
  loggerStateLabel->setBackGround(backColor4);
  loggerStateDrawLabel->setBackGround(backColor4);

  daqStateLabel->setGridSticky("w");
  daqStateDrawLabel->setGridSticky("w");
  dataHubStateLabel->setGridSticky("w");
  dataHubStateDrawLabel->setGridSticky("w");
  hisotgramStateLabel->setGridSticky("w");
  hisotgramStateDrawLabel->setGridSticky("w");
  reconstructStateLabel->setGridSticky("w");
  reconstructStateDrawLabel->setGridSticky("w");
  loggerStateLabel->setGridSticky("w");
  loggerStateDrawLabel->setGridSticky("w");
}

//////////////////// initialize logo frame //////////////////// 

void T473TclRunControlPanel::initLogoFrame()
{
  logoFrame = new KonoeTclFrame(leftFrame, "logoFrame",0,3);
  logoFrame->setBackGround(backColor1);
  logoFrame->setGridSticky("s");
  logoFrame->setGridPady(10);
  logoFrame->setBorderWidth(2);
  logoFrame->setRelief(KRaised);

  konoeLogoImage = new KonoeTclImage("konoeLogoImage");
  konoeLogoCanvas = new KonoeTclCanvas(logoFrame, "konoeLogoCanvas");
  setImage();
}

//////////////////// initialize run type frame //////////////////// 

void T473TclRunControlPanel::initRunTypeFrame()
{
  runTypeFrame = new KonoeTclFrame(rightFrame, "runTypeFrame",0,1);
  runTypeFrame->setBackGround(backColor2);
  runTypeFrame->setGridPadx(5);
  runTypeFrame->setGridSticky("w");

  runTypeCheckButton = new KonoeTclCheckButton(runTypeFrame,
                                "runTypeCheckButton", "Run Type",0,0);
  runTypeLuminosity = new KonoeTclRadioButton(runTypeFrame, "runTypeLuminosity",
                                              "Luminosity ",1,0);
  runTypeCosmic = new KonoeTclRadioButton(runTypeFrame, "runTypeCosmic",
                                          "Cosmic Ray ",2,0);
  runTypeHadron = new KonoeTclRadioButton(runTypeFrame, "runTypeHadron",
                                          "Hadron     ", 1,1);
  runTypeElectron = new KonoeTclRadioButton(runTypeFrame, "runTypeElectron",
                                            "Electron   ", 2,1);
  runTypePedestal = new KonoeTclRadioButton(runTypeFrame, "runTypePedestal",
                                            "Pedestal   ",1,2);
  runTypeOther = new KonoeTclRadioButton(runTypeFrame, "runTypeOther",
                                         "Other      ",2,2);

  runTypeCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypeLuminosity->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypeElectron->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypeHadron->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypeCosmic->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypePedestal->setFont(fontName, fontSize, 1, 0, 0, 0);
  runTypeOther->setFont(fontName, fontSize, 1, 0, 0, 0);

  runTypeCheckButton->setBackGround(backColor2);
  runTypeLuminosity->setBackGround(backColor2);
  runTypeElectron->setBackGround(backColor2);
  runTypeHadron->setBackGround(backColor2);
  runTypeCosmic->setBackGround(backColor2);
  runTypePedestal->setBackGround(backColor2);
  runTypeOther->setBackGround(backColor2);

  runTypeLuminosity->setGroup("runType");
  runTypeElectron->setGroup("runType");
  runTypeHadron->setGroup("runType");
  runTypeCosmic->setGroup("runType");
  runTypePedestal->setGroup("runType");
  runTypeOther->setGroup("runType");

  runTypeCheckButton->deSelect();
  runTypeCheckButton->setGridRowspan(3);

  runTypeLuminosity->setGridSticky("w");
  runTypeElectron->setGridSticky("w");
  runTypeHadron->setGridSticky("w");
  runTypeCosmic->setGridSticky("w");
  runTypePedestal->setGridSticky("w");
  runTypeOther->setGridSticky("w");

  runTypeCheckButton->setActiveBack("blue");
  runTypeLuminosity->setActiveBack("blue");
  runTypeElectron->setActiveBack("blue");
  runTypeHadron->setActiveBack("blue");
  runTypeCosmic->setActiveBack("blue");
  runTypePedestal->setActiveBack("blue");
  runTypeOther->setActiveBack("blue");

  runTypeCheckButton->setActiveFore("white");
  runTypeLuminosity->setActiveFore("white");
  runTypeElectron->setActiveFore("white");
  runTypeHadron->setActiveFore("white");
  runTypeCosmic->setActiveFore("white");
  runTypePedestal->setActiveFore("white");
  runTypeOther->setActiveFore("white");

  runTypeLuminosity->setGridPadx(20);
  runTypeElectron->setGridPadx(5);
  runTypeHadron->setGridPadx(20);
  runTypeCosmic->setGridPadx(5);
  runTypePedestal->setGridPadx(20);
  runTypeOther->setGridPadx(5);

  runTypeLuminosity->setSelect();
}

//////////////////// initialize run information //////////////////// 

void T473TclRunControlPanel::initRunInfoFrame()
{
  runInfoFrame = new KonoeTclFrame(rightFrame, "runInfoFrame",0,2);
  runInfoFrame->setBackGround(backColor2);
  runInfoFrame->setGridPadx(5);
  runInfoFrame->setGridPady(5);
  runInfoFrame->setGridSticky("w");

  runInfoCheckButton = new KonoeTclCheckButton(runInfoFrame,
                         "runInfoCheckButton", "Run Number              ",0,0);
  runInfoEntry = new KonoeTclEntry(runInfoFrame,
                                 "runInfoEntry",1,0);

  runInfoCheckButton->deSelect();

  runInfoCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  runInfoEntry->setFont(fontName, fontSize, 1, 0, 0, 0);
  
  runInfoCheckButton->setBackGround(backColor2);
  runInfoEntry->setBackGround("white");

  runInfoCheckButton->setGridSticky("w");
  runInfoEntry->setGridSticky("w");

  runInfoCheckButton->setActiveBack("blue");
  runInfoCheckButton->setActiveFore("white");

  runInfoEntry->setGridPadx(10);
  runInfoEntry->setSize(15);
}

//////////////////// initialize particle energy frame //////////////////// 
void T473TclRunControlPanel::initParticleEnergyFrame()
{
  particleEnergyFrame = new KonoeTclFrame(rightFrame,
                                          "particleEnergyFrame",0,3);
  particleEnergyFrame->setBackGround(backColor2);
  particleEnergyFrame->setGridPadx(5);
  particleEnergyFrame->setGridPady(5);
  particleEnergyFrame->setGridSticky("w");

  particleEnergyCheckButton = new KonoeTclCheckButton(particleEnergyFrame,
                          "particleEnergyCheckButton",
                          "Particle Energy  \\[GeV/c\\]",0,0);
  particleEnergyEntry = new KonoeTclEntry(particleEnergyFrame,
                                               "particleEnergyEntry",1,0);

  particleEnergyCheckButton->deSelect();

  particleEnergyCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  particleEnergyEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  particleEnergyCheckButton->setBackGround(backColor2);
  particleEnergyEntry->setBackGround("white");

  particleEnergyCheckButton->setGridSticky("w");
  particleEnergyEntry->setGridSticky("w");

  particleEnergyCheckButton->setActiveBack("blue");
  particleEnergyCheckButton->setActiveFore("white");

  particleEnergyEntry->setGridPadx(10);
  particleEnergyEntry->setSize(15);
}

//////////////////// initialize storage frame //////////////////// 

void T473TclRunControlPanel::initStorageFrame()
{
  storageFrame = new KonoeTclFrame(rightFrame, "storageFrame",0,4);
  storageFrame->setBackGround(backColor2);
  storageFrame->setGridPadx(5);
  storageFrame->setGridPady(5);
  storageFrame->setGridSticky("w");

  storageCheckButton = new KonoeTclCheckButton(storageFrame,
                          "storageCheckButton", "Storage ",0,0);
  storageLabel = new KonoeTclLabel(storageFrame, "storageLabel", "",1,0);
  storageValueLabel = new KonoeTclLabel(storageFrame,
                                        "storageValueLabel", "",1,1);

  storageCheckButton->setGridRowspan(2);
  storageCheckButton->deSelect();

  storageCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  storageLabel->setFont(fontName, 11, 0 ,0, 0, 0);
  storageValueLabel->setFont(fontName, 11, 0 ,0, 0, 0);

  storageCheckButton->setBackGround(backColor2);
  storageLabel->setBackGround(backColor2);
  storageValueLabel->setBackGround(backColor2);

  storageLabel->setGridSticky("w");
  storageValueLabel->setGridSticky("w");

  storageCheckButton->setActiveBack("blue");
  storageCheckButton->setActiveFore("white");
  getStorageStatus();
}

//==================== initialize cherenkov pressure frame ====================

void T473TclRunControlPanel::initCherenkovFrame()
{
  cherenkovFrame = new KonoeTclFrame(rightFrame, "cherenkovFrame",0,5);
  cherenkovFrame->setBackGround(backColor2);
  cherenkovFrame->setGridPadx(5);
  cherenkovFrame->setGridPady(5);
  cherenkovFrame->setGridSticky("w");

  cherenkovCheckButton = new KonoeTclCheckButton(cherenkovFrame,
                            "cherenkovFrame",
                            "Cherenkov Pressure \\[atm\\]",0,0);
  cherenkovEntry = new KonoeTclEntry(cherenkovFrame,
                            "cherenkovEntry",1,0);

  cherenkovCheckButton->deSelect();

  cherenkovCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  cherenkovEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  cherenkovCheckButton->setBackGround(backColor2);
  cherenkovEntry->setBackGround("white");

  cherenkovCheckButton->setGridSticky("w");
  cherenkovEntry->setGridSticky("w");

  cherenkovCheckButton->setActiveBack("blue");
  cherenkovCheckButton->setActiveFore("white");

  cherenkovEntry->setGridPadx(15);
  cherenkovEntry->setSize(15);
}

//==================== initialize CsI temperature frame  ====================

void T473TclRunControlPanel::initCsITempFrame()
{
  csiTempFrame = new KonoeTclFrame(rightFrame, "csiTempFrame",0,6);
  csiTempFrame->setBackGround(backColor2);
  csiTempFrame->setGridPadx(5);
  csiTempFrame->setGridPady(5);
  csiTempFrame->setGridSticky("w");

  csiTempCheckButton = new KonoeTclCheckButton(csiTempFrame,
                          "csiTempCheckButton",  
                          "CsI Temperature     ",0,0);
  csiTempEntry = new KonoeTclEntry(csiTempFrame,
                          "csiTempEntry",1,0);

  csiTempCheckButton->deSelect();

  csiTempCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  csiTempEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  csiTempCheckButton->setBackGround(backColor2);
  csiTempEntry->setBackGround("white");

  csiTempCheckButton->setGridSticky("w");
  csiTempEntry->setGridSticky("w");

  csiTempCheckButton->setActiveBack("blue");
  csiTempCheckButton->setActiveFore("white");

  csiTempEntry->setGridPadx(12);
  csiTempEntry->setSize(15);
}

//==================== initialize position frame  ====================

void T473TclRunControlPanel::initPositionFrame()
{
  positionFrame = new KonoeTclFrame(rightFrame, "positionFrame",0,7);
  positionFrame->setBackGround(backColor2);
  positionFrame->setGridPadx(5);
  positionFrame->setGridPady(5);
  positionFrame->setGridSticky("w");

  positionCheckButton  = new KonoeTclCheckButton(positionFrame,
                         "positionCheckButton", "Position                ",0,0);
  positionEntry = new KonoeTclEntry(positionFrame,
                            "positionEntry",1,0);

  positionCheckButton->deSelect();

  positionCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  positionEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  positionCheckButton->setBackGround(backColor2);
  positionEntry->setBackGround("white");

  positionCheckButton->setGridSticky("w");
  positionEntry->setGridSticky("w");

  positionCheckButton->setActiveBack("blue");
  positionCheckButton->setActiveFore("white");

  positionEntry->setGridPadx(15);
  positionEntry->setSize(15);
}

//==================== initialize angler frame  ====================

void T473TclRunControlPanel::initAngleFrame()
{
  angleFrame = new KonoeTclFrame(rightFrame, "angleFrame",0,8);
  angleFrame->setBackGround(backColor2);
  angleFrame->setGridPadx(5);
  angleFrame->setGridPady(5);
  angleFrame->setGridSticky("w");

  angleCheckButton = new KonoeTclCheckButton(angleFrame,
                        "angleCheckButton", "Angle                   ",0,0);
  angleEntry = new KonoeTclEntry(angleFrame,
                        "angleEntry",1,0);

  angleCheckButton->deSelect();

  angleCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  angleEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  angleCheckButton->setBackGround(backColor2);
  angleEntry->setBackGround("white");

  angleCheckButton->setGridSticky("w");
  angleEntry->setGridSticky("w");

  angleCheckButton->setActiveBack("blue");
  angleCheckButton->setActiveFore("white");

  angleEntry->setGridPadx(15);
  angleEntry->setSize(15);
}

//==================== initialize shutter frame  ====================

void T473TclRunControlPanel::initShutterFrame()
{
  shutterFrame = new KonoeTclFrame(rightFrame, "shutterFrame",0,9);
  shutterFrame->setBackGround(backColor2);
  shutterFrame->setGridPadx(5);
  shutterFrame->setGridPady(5);
  shutterFrame->setGridSticky("w");

  shutterCheckButton = new KonoeTclCheckButton(shutterFrame,
                        "shutterCheckButton", "Shutter     ",0,0);
  leftShutterLabel = new KonoeTclLabel(shutterFrame,
                        "leftShutterLabel", "Left  \\[mm\\]", 1,0);
  rightShutterLabel = new KonoeTclLabel(shutterFrame,
                        "rightShutterLabel", "Right \\[mm\\]", 1,1);
  leftShutterEntry = new KonoeTclEntry(shutterFrame,
                        "leftShutterEntry",2,0);
  rightShutterEntry = new KonoeTclEntry(shutterFrame,
                        "rightShutterEntry",2,1);

  shutterCheckButton->deSelect();

  shutterCheckButton->setFont(fontName, fontSize, 1, 0, 0, 0);
  leftShutterLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  rightShutterLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  leftShutterEntry->setFont(fontName, fontSize, 1, 0, 0, 0);
  rightShutterEntry->setFont(fontName, fontSize, 1, 0, 0, 0);

  shutterCheckButton->setBackGround(backColor2);
  leftShutterLabel->setBackGround(backColor2);
  rightShutterLabel->setBackGround(backColor2);
  leftShutterEntry->setBackGround("white");
  rightShutterEntry->setBackGround("white");

  shutterCheckButton->setGridSticky("w");
  leftShutterLabel->setGridSticky("w");
  rightShutterLabel->setGridSticky("w");
  leftShutterEntry->setGridSticky("w");
  rightShutterEntry->setGridSticky("w");

  shutterCheckButton->setActiveBack("blue");
  shutterCheckButton->setActiveFore("white");

  shutterCheckButton->setGridRowspan(2);

  leftShutterLabel->setGridPadx(10);
  rightShutterLabel->setGridPadx(10);
  leftShutterEntry->setGridPadx(10);
  rightShutterEntry->setGridPadx(10);

  leftShutterEntry->setSize(15);
  rightShutterEntry->setSize(15);
}

//////////////////// initialize shift frame //////////////////// 

void T473TclRunControlPanel::initNameRegisterFrame()
{
  nameRegisterFrame = new KonoeTclFrame(rightFrame, "nameRegisterFrame",0,10);
  nameRegisterFrame->setBackGround(backColor2);
  nameRegisterFrame->setGridPadx(15);
  nameRegisterFrame->setGridPady(2);

  registerNameLabel = new KonoeTclLabel(nameRegisterFrame, "registerNameLabel",
                                        "Shift",0,0);
  nameEntry1 = new KonoeTclEntry(nameRegisterFrame, "nameEntry1",1,0);
  //nameButton1 = new KonoeTclButton(nameRegisterFrame, "nameButton2",1,1);
  nameEntry2 = new KonoeTclEntry(nameRegisterFrame, "nameEntry2",1,1);
  //nameButton2 = new KonoeTclButton(nameRegisterFrame, "nameButton1",1,2);

  registerNameLabel->setFont(fontName, fontSize, 1, 0, 0, 0);
  nameEntry1->setFont(fontName, fontSize, 1, 0, 0, 0);
  nameEntry2->setFont(fontName, fontSize, 1, 0, 0, 0);
  registerNameLabel->setGridRowspan(2);
  registerNameLabel->setGridSticky("news");

  registerNameLabel->setBackGround(backColor2);
  nameEntry1->setBackGround("white");
  nameEntry2->setBackGround("white");

  registerNameLabel->setGridPadx(15);
  nameEntry1->setGridPady(5);
  nameEntry2->setGridPady(5);
}

//////////////////// initialize control button frame //////////////////// 

void T473TclRunControlPanel::initControlButtonFrame()
{
  controlButtonFrame = new KonoeTclFrame(rightFrame, "controlButtonFrame",0,11);
  controlButtonFrame->setBackGround(backColor2);
  controlButtonFrame->setGridPadx(15);
  controlButtonFrame->setGridPady(5);

  beginButton = new KonoeTclButton(controlButtonFrame, "beginButton",
                                   "Begin", 0,0);
  endButton = new KonoeTclButton(controlButtonFrame, "endButton",
                                 "End", 1,0);
  pauseButton = new KonoeTclButton(controlButtonFrame, "pauseButton",
                                   "Pause", 0,1);
  resumeButton = new KonoeTclButton(controlButtonFrame, "resumeButton",
                                    "Resume", 1,1);
  shutdownButton = new KonoeTclButton(controlButtonFrame, "shutdownButton",
                                      "Shutdown", 0,2);
  
  beginButton->setFont(fontName, fontSize+1, 1, 0, 0, 0);
  endButton->setFont(fontName, fontSize+1, 1, 0, 0, 0);
  pauseButton->setFont(fontName, fontSize+1, 1, 0, 0, 0);
  resumeButton->setFont(fontName, fontSize+1, 1, 0, 0, 0);
  shutdownButton->setFont(fontName, fontSize+1, 1, 0, 0, 0);

  beginButton->setSize(12, 2);
  endButton->setSize(12, 2);
  pauseButton->setSize(12, 2);
  resumeButton->setSize(12, 2);
  shutdownButton->setSize(12, 2);

  beginButton->setBorderWidth(3);
  endButton->setBorderWidth(3);
  pauseButton->setBorderWidth(3);
  resumeButton->setBorderWidth(3);
  shutdownButton->setBorderWidth(3);

  shutdownButton->setGridColumnspan(2);
  shutdownButton->setGridSticky("news");

  beginButton->disable();
  endButton->disable();
  pauseButton->disable();
  resumeButton->disable();
  shutdownButton->disable();

//dummy
//KonoeTclText* t = new KonoeTclText(globalFrame, "txtx", 0,2);
}

//////////////////// initialize logger frame //////////////////// 

void T473TclRunControlPanel::initLoggerFrame()
{
//  loggerFrame = new KonoeTclFrame(globalFrame, "loggerFrame");
//  loggerFrame->fillBoth();
//  loggerFrame->setExpand();

//  loggerText = new KonoeTclText(loggerFrame, "loggerText", KVertical);
//  loggerText->setBackGround("white");

//  loggerText->setSize(80, 16);
}

//////////////////// initialize logo image //////////////////// 

void T473TclRunControlPanel::setImage()
{
  konoeLogoImage->setData(
"R0lGODlhyABKAPcAAAAAADMzM2ZmZpmZmczMzP///5lmZmYzM8yZmZkzM8xmZswzMzMAAGYA"
"AJkAAMwAAP8AAMwzAMxmM5kzAJlmM8yZZmYzAMxmAMyZM5lmAMyZAJmZZmZmM8zMmZmZM8zM"
"ZszMMzMzAGZmAJmZAMzMAMz/AJnMAMz/M5nMM2aZAMz/Zpn/AGaZM5nMZjNmAGbMAJn/M8z/"
"mWb/AGbMMzOZAJn/ZjPMAGb/MzP/AGaZZjNmM5nMmTOZM2bMZjPMMwAzAABmAACZAADMAAD/"
"ADP/M2b/Zpn/mcz/zAD/MwDMMzP/ZjPMZgCZM2b/mQD/ZjOZZmbMmQBmMwDMZjP/mZn/zAD/"
"mTPMmQCZZmb/zADMmTP/zAD/zGaZmTNmZpnMzDOZmWbMzDPMzAAzMwBmZgCZmQDMzAD//zP/"
"/2b//5n//8z///D//wDM/wCZzDPM/zOZzABmmWbM/wCZ/zNmmWaZzAAzZgBmzDOZ/5nM/wBm"
"/zNmzAAzmWaZ/wAzzDNm/wAz/2ZmmTMzZpmZzDMzmWZmzDMzzAAAMwAAZgAAmQAAzAAA/zMz"
"/2Zm/5mZ/8zM/+jo//Dw/9jQ/zMA/+DY/zMAzGYz/9DA/+jg/8iw/9jI/7iY/2YzzDMAmYhI"
"/5hg/5lm/6h4/7CI/8Cg/9C4/+DQ//Do/4A4/4hA/5BQ/6Bo/7CA/7iQ/8io/2YA/4Aw/4g4"
"/5BI/5hY/6Bg/6hw/7iI/8CY/9jA/4Ao/4gw/5BA/5hQ/6ho/7B4/8ig/9Cw/+jY/4Ao8Igo"
"/5A4/5hI/6BY/7Bw/7iA/8CQ/9i4/+DI/5Aw/5hA/6BQ/6hg/7h4/8CI/8iY/9Co/7Bo/2Yz"
"mZlmzDMAZmYAzJkz/8yZ/9Cg/9iw/+DA/+jQ//Dg/5kA//DY/5kzzGYAmcxm/5kAzMwz/8wA"
"/5lmmWYzZsyZzJkzmcxmzMwzzDMAM2YAZpkAmcwAzP/o///w//8AzMwAmcwzmZkAZv8AmZkz"
"ZsxmmWYAM8wAZv8AZswzZpkAM8wAM/8AMyH5BAAAAAAALAAAAADIAEoAAAj/AAsIHEiwoMGD"
"CBMqXMiwocOHECNKnEixosWLGDNq3Mixo8ePIEOKHEmypMmTKFOqXMmypcuXMGPKnEmzps2b"
"OHPq3Mmzp8+fQIMKHUq0qNGjOyHBg1RKKVOnTZcKjPd0adSn8R4y9dWray9bWRnGa1pqLFOz"
"DaGqhbfG4VhkvVh1HVXqYamreNdW3ds0bMd4vGLpGoxqsK7ChmX5ikfLMOLDg4n1akiql6xh"
"uIC5uuVKWKxapBbWkmXYsa5hrBZe2lX6saxoDC9Fm6XrFTDOwV7pOoaM4ShixCATNj28uPBh"
"kz9eQibrNrBgzm/hihXtF6QCyXjZvuUc165Vv/wi//z1DFYwV69m8XImK3Nm1OILlvrVa1h0"
"zcCG1U0YjxStXN1R98t+CHXTiy6b4ZKKMbzssh0wr9RC4EHx/OILMa44B510r3To4SuZcccd"
"dK4kI1Ipu5wnHTC43CJLNwYVI510siAT30GXHAMLd7jMYqJApbAC4C3S0XJjQaQgeB50LabG"
"kC+vnMcLPAsFGYt7soziFzKwsCjdLhOO12WLmwEDyza/cJNmmrXwokuLmbnCzHUisfJchufl"
"8stBqrCYSzRHGmSLLOdBSMtBq8CpmZEL2eKel8DoEuZBpegCDDN7KkQKLyH2OCkyIbIIZpXD"
"EOneLbvQaVCOXQbj4kjxyP+CJ56qHJSMealw4xCULLJ46EG/dHnLebg4mRApuGRoqnSwLQTJ"
"m8YidOuIroxqUDyxzKiZMaTiyRkwySU0LS7NhvSsiiwWa9CtwMgyKaXaDcuiKoEWYEyGmQUT"
"S6YI0XJqqLHUC6QuevK3CoAqNvOuQL7c1mIwudiiUCn2xQlMwQuB6orEIgV7J3QR/yJymr6Y"
"R0xoQIqsppq/XNLNajxqlsslFF7yCy2FFloLf7K8YkyyH7tSCzc0IwQqKAWU0o3KIl9yyc+a"
"QfcKxwYp7cuKys6ikC8trhjMmWuyzHSatrwyTDIjp1n0RtF4vZkuvtgity29dLkvQZCQMrct"
"aF//x8uK0uHSW0HxTEIKKb/Ucudzt+iCskGkvEJMKc0Vyp0uoySz8DG47FwAJL/MPWAvnUrn"
"OUHxXIJ46NSeJwxY/fY64yzI7C334dzgToossiRj++MZxZOKw87R4rvcdeNCy6SXiL6fLcI4"
"zJ0s4lWYTDLcZNXLshnmwkr2gvZYANfEt/hMJvwSVCnGBfS3DSmF6zKssrDASNAvpCQDP3Zu"
"swjajRTDDWdwcQzb2SJ9AzHefI63jbVlBBLMUFZmIqY3usECF9w6SDf4trZfAKhF6fKF+ibB"
"t8dtr3TAMIYtgCcQXgCjN9hyBZM4owtkaG5Vw9CF/QTyi2QwhRi9KtQx/9RHim3YAnz82wye"
"2rXC+HisUPkxoA8rlJBScINvqspI2SCFi1xww3eswAwvFpYmVQGmTCAkhl96uI1M6GogvSDe"
"jFRhCzQR5BLDmBrD4kStAiKwANFwRQYHgr94iAIXZHoOLPjVQ99ZZyDJcBtnZFHH/cFRhvi6"
"BTGON7fQ1OJ0lNJfR6AxPy/NInS0uKCPEFKKS4gHGZmUjiua1R/RiQdnmWQRL+R2Qx7qZiCl"
"iIWrOlXDXk5lFq8YHCFFlouYnedXhZubDwnii1k9h4lyc+AxusOdof2OG7Gg2njeqBFs4WeY"
"QzOGbXLhiz9+rix3HIYMZSeMPeXNiHybUKWo5f8lOsqNX7UgIEFY0SmH1eKAeNOFMBxYgJbt"
"IpauwIUIC4dPOxJkF4tbEiVvF5ZKLWmAvaig6NxEpYktjCKPkKeruseNVdgmGJ2b1Fjis00V"
"rWgXAuFGJkQnnzGZylfS3A9GR6E+8+SSksZMxitShToL2QaE76nLJPDJt/gQyprVCqpAthgq"
"WORub8g4xiuolxLyzWoYq4gemXgRH6YcRBbfKhQuJlMKTk7zfqXC123+NDc7/uIViyzIM7wl"
"wz+hD47AGOL9JkELxs1TkEnjpC0YWgAE3WlYnetrpkj3U1wUI3S2QEY0GpOZaJnEhUvKEMFu"
"gydYTGimgnqpt14xoCv/ys2iA83XrNi5t9AoVRaEI0Ve7+SKXd4QW8JgYeiGqyKJQsK2vIwP"
"srjIoq90MiuzkF2LiMGLWcxiF6cIkRdTEg8EcSdDwqhFLuQKIaq59SB/0y2LZPHcnfJyUseI"
"a5l2MYq9mahP5UpZQBdnprjtqVKwYGji5AudYVxCp3v7Iyx5RCRXDCNu18XjsvAzomvKkKwo"
"IcUHh3WLXCSjcpuBTnKUdqRKsXZxn/1d1WKR0YjS4nfdIIbgCuKybuzIW7igow9hWQz5cGMW"
"P8XkyeoozRtlF12ZmYXtsocJi7UoF7RwBi8CA6LbhOskV8PkfElxr8XdghcPRhtCkLUiqPpC"
"/6R1/GMyhIFG7vB1b7nTTZjCU4CaJig/vtgGN35m2ksko2KOTaxkj1g15uLLutcVxUd9BVpb"
"aKMXGJIo4R4xksYySTPH+IVYKdxFDLMQkqXDJIalmUWB1ILCK6rR76IRZPlk6hIfvI10VJgJ"
"hSIwTTv6Vr5WQdVsCsqa56lhX1dYCiC6DRrQlZsvhOFa1E1ihx+pVCKDkVxS+CIXBRVkdOEb"
"UQ25Ihe1623VdByzWwgD0nPjhgtFeD9++auUjfPFt2Px2kuQLoicEQY0bJeMGxVjnl3DxSoW"
"nb9cWDMXhzNgLooMzMONxFHc9KrvsgvC8zQj3ZQtwJOPajvcDkTEXP9ElQHrCIvkFuTahASQ"
"XpXX2EEK5MG1qHOGhtFfPFfNUjJc0TCYrNm25cwVzTC07ZBxCkCBLn/YG0mYLccL0D7jqd/i"
"LaMNMosZxprgrS5AHDcsDF8s+nqSqx4Cy7zS/FxQmTy8hOISrshVy42ckAw6nggI53w2Npeh"
"XjQyiJEJm3ES2x9pT9Ru8YpegPbbzlTe7Q6C5Mf673fxabaYk0WLSnfSX6KwdUF+QWevcSaZ"
"o49HQJ1Z3HTPDfEFcKFuXUHfaDNbfm0nF5zRNJ/2gS4Z2/hRSPDo4YvhT27IyK7egeFPYw5k"
"5I6F9woNwrUlalJkJSfFLjR9R9hzSo7A2CT/gUBXgFeHiFYEl27p82XiutqOmY/CxSIlW/Cq"
"HU5gGBnF/Iapix7O7W85k29yA3v+Ij2ZAW8m9znN4CUy9AqwYV/SRArUhnh8hiR0Vj67xC8v"
"Mz6nUigqpG4FoQr5gh+H4nn5FEi3oSLG8AsQOH0HwQ1yIRLQIEG3YQzNMze9MCRx4j2ThySK"
"ghui4F9hojF30jkNVWwHVDY4pT7uFHuXhUhf0UsDkjTmEVe34E/xNnpDgknFkBWLFh7Dc3Sd"
"934I8W1EFRKWAlVTE22j8CZyVAy1U3+o0xzP1lvicQnEQGHAwG8F0HcF1yf0NhA2Iy2vsEQ8"
"yGxWNBBshy9YeHeo/wMK2oILtBVZQlhewlZi+uNfNxIPo5AL1QYSmSAMeqgncJYM29Q1edJz"
"lLWIl3ddBHEMmeQKuqAr8fCFl/Am4lQARJMQXacszeBfFicQBCVfH3hbDjQKupUbHGOCe4IM"
"L1Vh/SdZpxYPqpAhX+YRJ0Rhs9ANksUNnJUh+FJALng/zTQruwCCwoh1+YEyN3hb2ROKutBq"
"FWgrT0VAy4ZbzaYZI4I5TQZJIwYhgwMJ3ZgV5tcriXUJxYZE7WMJivcK2yASztY1KdSO/+Rj"
"1BULcXgQ5FMor8AKWcgwUYIvvXNy/nUdxRAp4tFKC7EL3PEKdodFBJEMIxZR4jiO04IvF/9G"
"SEJYAN1gKbghQ2MYYQNhCw7iMLqAfxYxCqIYfz5Skh4Ea4hUQHelPg/FOLgga9NXCrwCJ/QC"
"TJyEJqqXLA52P7BXEGUTDLLQd6d2b3eiC9HAS+1jCxfEOO4yEAKpWQKRjR7GC65XQqQADakg"
"W88BShzBiTLnWO5WC66HOFf1adwRC72QgEBSDK8gbLFwDL2gb4q3XaxQUjzUV6TQDbWwHT3C"
"SEiJUWwFgZIJCf9xVrSQmbaATHcCC9CAbb9QUfADCdMWi0wCC7zDO8QgDF0GZYElHxfRJqmQ"
"L1hzJ2O1C76ADLwARDQIJ8FwCs55JPEQRh6GSF0miZIRJu6HPIH/UVC3AAu8AD8hF1siVGnO"
"RxC2oGPAoBmSKIm3oRuhVhB3eXeZwAvNADSflmLWFypQRSS7kHkn5RC2EA2sEA0K2qAMygrH"
"YAzGQAu1Aw0LyqANygrPQAsSugrIkJ4F4G+qkAvD4Im5EAsWug1hR1F1pFMO6qALegkq2RDI"
"UBe1yFFVNFrDAAueCAvDQAvRMAqeeT9B1Q0vuqAXCqETOqGviaFJyqDpUzjz+BHx0IJTiSQy"
"phC/gAy+MAqBdqBFdF8MITISAVp4x0q2oG/9BaKUGGdiIY1ImTRFdGodgZB2iBC2156Qo1UU"
"MlW3U5b4qWYRUQp604SjZ1+SCUw6FXxs/0pILUintlZEmQCoG3F2YScQfYdQCuE+lXQk+fNP"
"SElRkzAR+DOk/BGm+UQpofM+l3otftiqSXN84wYS0WRA7kSoUnSg7eOn+pNFhOo7ydCoU8Gr"
"ukopruQWJPRP+tRIwfoQmSpoZbEG8UAV+JOJeDl8uZM7usMNsNcNpKCt3/qtU7gQ1Xo9upMm"
"ofkQD4Y7MioRxcpjX2Su36omh3OsDrGu4Xo43+o03+A06JqvufNIsDIVBMsfBRsRFZI29ioR"
"YRGn5WQzYuOwCNGwA0GxBWuxSJGxGruxHNuxHvuxIBuyN0EkuiYisqQMkiicubCysOCjO8oM"
"ghELxOAJv9kMu/9ws97VXVsmoaqwpBv6DLVwDEErCrXgFXHBoPrWpb5gCenWRtejP9zADZPA"
"MjbTDVZ7F/DwCDMFCVTBtZCgFHehNErzCEsjMogTtfmTCcBnX8gwCtrApfqGCaywoF0BDdDw"
"SdBwDKtwDM+wChxqDKqwHltWDA1SDLuQCrvAO4kLHLEQC8xgGDu6DLlwCsKQC8LZIYhEJCYr"
"EDPyL72iDK8gDMJAuTxKosOQQ48LHLJADLKQuIdruN5FuFvmDBIaChO6Cs+gCccgtLVwt0bb"
"oGqKDMLLZE6rP7gjNlYro2HrtdPKtQXAFs0LD2ShNFbbryoDrsmQCcFnRG3btl6qDZj/wKBG"
"Cw1Eu7tBewy0sKE86wyBywvFUAzfNQs22wyrCxz2Oxg7erqlS7mYiwvBAFOQIhBydH6I5ArK"
"wL+T27Itexowa78127rfVQygIMFbpgo9O6E/u7u8Cw1ekQ1zGw36pg3PWTvDyzf6Y7xqcgnf"
"8Asvc7VXwbVt8Tm+RxVjIb13kbxpgz+Hcz22MwpcOgqjEA3hywoe3Asc/ElCq7e0sMR/275b"
"5l3edbO7QL/0Cxy6EByncbq5ILmWCyLogUjJ6AqcWzqdG1EfsrKeeLqlIbMyy7qta7OzQLjw"
"274SyqRLrLfHoAnk+0lygaS+wAra4KU+LE1q+636M7Uj8zJh/1sWSkHDMjwV1zG9rWS1LSMy"
"U5s7T8vDoSW8SasNSCoXRtwLSKzBz6C+gMsgW+a+USzF9Fu/xCAY+KsLPsqjp5ALoQshtqGP"
"vcK51GUq3Bm6lbuyJYoKOaQLzPDKbpwKNou4uxDHshu4FsykGry7d1u0vSAKc8sKvgDCXiq8"
"g1y82ZuvVdsy3bDIX9s+FVsAbTGt07rIl0C2v7DCZos9T8tk3izCSSu3chENRWu3o7y3S8yz"
"gJvKsXu4iSsLzYDMugDLWVy6Xdy/7mE5nAs4ZBJRKSu6lovGalzM9ksM9KvMzRzBsxu4E9qz"
"tOC3eCy0dtsV2RAXCxrCw+t62Xs9Uf8bsFVbzo9wF8yLzvj5vF8btslrM2Gjr2prRNsgvMjw"
"tvqmzdl8zeXbuxq8xM8AuE5MuBEsxQgNnI8LyzuKCiY6uaLrHkBDYWPcy8mCC6aA0cHcssOA"
"GI2ruqvbujgbx+pRwSaNwc+gwXfru13x0iCstD5s1CZ8wmvSNOVcznfhyPEQw6iTNDN1w2Sr"
"wr9wyTusvW2EfECsbyCsoHRrxKJgvrvrtxvKobTLCxMswfE7xcosC7HA2sHxuGqMxisbuqGb"
"uTPCHQLcWfICxrT9Ciu7DGzt1lfsxq2rzKk9uKpAuxIq1bQA2p8kChx8tH/tC2+rDaE1N9q7"
"w+LaNOQctkv/Ma0UIq1fOxZK02NCjT+YfD3BR8JIndkN6hVEK8rny7emTNUE/b5SvNpx/cqP"
"exj6uwyUK7q24cUFhQtlrS0Jwp2Xy7KzXMwLbb/0a9CxS9ep3LMm3dx5vQpBS74s3QvAC9jD"
"uw3FC7CTIDLJ2w3SS8PO27Dr7LWLPMkqU+JQ97QQ6MNKm7RETMSivNehrcQ8m9ztC8Xfdbit"
"7MqC0db/jcbAPJyzcgu8PKAwZca4ULnB3KM5NAwxC9eKO9d1ncrsi8EnncSf9El9HQ3ZIMR/"
"7cOjgE/AGglQK7VN4zJiK73fTScsnhVNwRRi+zIis8K6w8Pae90+jM8YGhddUQvQ/53EfTva"
"f4vKExzHzYzVwCmzMIu/sADcttwhwgkMylBuI1jWUMVaKEvbk8uyp/uyV3zMrNsMiIu40hDH"
"qcwLVM3EYa63Gx7dHvzSSdu9Tau25hq1k73Cybu8db7Yxj7DfVEK8NANZNuvZYu216O2c0PC"
"IizCGFrEh867u9vcHGq70LwezvxdqfCb+23MsYDkJVq6lfshlkMmvLwscZUZl1vlLsvRrxzX"
"CC3FzgwK7ssgdSzVe2u+vsvBfSzEIezDJCxNv44/K+wyPYbYNAzeXNu8XavnZGu1fI6uX4TJ"
"O5UJyHDUma3ZTS0KRuzP883EPJvKEgy/Q36zwMm4g8EMSf8uufy76cqQcmPcZs9x1gMu2zy6"
"ozEfGfhuszj76nEMCloGuByavny74WRu6B+szfqW1CQs4uGMPQEL59/Qry/eF8zrtUtB5+Xt"
"NDYj41Jb2Qy0yW1r7XQbDU5dzburCaVMC7br7VrWXaCwyvo9s8HB0Dvq0AIOIf8iIk+ua51i"
"wMKJ0T9Poqfx4LqA73KNs+9b19EM5hoctL1rzUer6885yOkGrCdsOIjj7MOO4n0RD1r7tS6u"
"59S7NDbzDdkKdfZ1PLWj5prNFW5ft6JMyoxex+uR96d9s4gLnKvr97KsxSs7uqILIWXSuQfe"
"KecBuqJ7Cr6t0cWcuoyb1cZt9O90CwoXbMcobb5kTvBub/BT7823ZcL5c7wl3q8PXwo5rdM0"
"LL0/DdTzQckjc/YzPu1pChC+tPkaFQ1TtGisei2EVqvhsWOrntGiZcyiKmeqeG0sNmvXR1kh"
"mxGTRYyYLpSwYOVamSuXsFOvZOLCBazmrZoFAgIAOw==");

  konoeLogoCanvas->clearAll();
  konoeLogoCanvas->drawImagenw(0,0,konoeLogoImage);
  konoeLogoCanvas->setSize(200,74);
}

void T473TclRunControlPanel::setImage2()
{
  konoeLogoImage->setData(
"R0lGODdhmgGPAPcAAAAAAIB3d+Xp2qizqNLOzvPf1M7f68G4uufy6fHp6amkn8XBy+Xh8cq3"
"qNrb0r+0p+XK09TBtaPIyqzKv9Lp4bLBu9LBynNoZub/+7qhm6Clqcrb3Ofp6Of//4J7ccGg"
"m8GpkeHc2I6Idtrb2/7x2rq1tfj//9rS0dK4srKnmaXBt3VoccrKwXWAiJCQkv/99WZxbPH/"
"+6mZkPHS2mh5cf/p4dT18uG9vfHh5/bp2Km0tPH09JuZmZmMdrXS2oCKhXVwd/jy56m3wdLb"
"3P/o2qXBqZGMhYyRof///sHLy/b+/9bR49fS2srBwH2RkrbAysq4tuHz9P/g4Pv/6bupne7o"
"+und1JO0p+fy346jrLuptGZoccHBvff//+XayZmIhfvm5Y6Zh+XSzJmZqcqpodrawWhmZePO"
"4W6MhY6poY6doYN3aNfMwamQinVxbv///LrMzNHa0pmjm/j19b/c1K6ps3+Hkv///7SpkX2b"
"k//49bOdkKqZm9rl65e1s+TOwOn/6Wh5e4FxbJ2ZjvHy53eKg4WAff/77rKzqcnR2nd7eb26"
"wf/05J2osvjo8Y6blp2HkeX1/72ymdK7p5GAe/b//5F/ccHLweng5aypqf//9/34/wAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"
"AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACwAAAAAmgGPAAAI/gABCBxIsKDBgwgTKlzIsKHD"
"hxAjSpxIsaLFixgzatzIsaPHjyBDihxJsqTJkyhTqlzJsqXLlzBjypxJs6bNmzhz6tzJs6fP"
"nyrvCB0qcKhQg0bvID1KMOlSogWTSlUKtKrVq1gTUm26dSBUAF+9dvUalSlYs2QPjs3Ktq3b"
"mmPRFl17tuzarnLnRkVI963fv4BH4qXbF+zeu0217lUbuLHjxxq35q3LF2/RxZTVWk4LubPn"
"zwypTjascHNcznwTowbNOqKU17Bjy54tu/VJpWFVK+a8mfTuuUZtCwdAu7jx48iLD79oNm/h"
"y6glr0aq2/dywMmza9++/XrD05hT/kfnvbC3de9Xuatfzz45+urQxWo1T1pyYbR9n7+P2b6/"
"//+0eYdYeATGhRt88OW3n00ANuigg8IRVpZmBBo2GIUIxregSw926GGDoEloF1fUjcjVhRVq"
"uCFKH7boIoCOSXXiU84FRyKNd01l44omvejjj/3xKKRnQBZp5HpX6ajkkkw26eSTd+gBZZOa"
"TDEllJEw0AcHAnAwwgghjNDHEEOMMMQGcTCRSBw0Henmm939dOWcdNZpVAx25nllFAx0uSUm"
"AvTx5QhMjODAEEzEkQgBMsHp6KPH+aTnpJRKhUGlmE6lZQgccNklJmIOSmaicTDKIaSopjrb"
"TpkqWUmr/kzuACusVfTRpQCA9iGooEww4QATQ8QRbEuqFmvsaznNqiyTNgwl67KVMsCBrYHi"
"GoKugx7qwK9sqnTst8beBO245EI7Qqc4dOmlmIISaqiviaYE7rzh0lRuuR3cW+kSnQIaaJi8"
"ZktqtyTRa/CxM+mr8JQIXFKCBhqo4Ue5W3JwrQAChDkCqCH4SmYcDhAc0sEkIwzTwpjOoaQm"
"lyaVwCUnYBJDH3RcojLKdzAhQJ8Wh6nxoIMmeoJIJRddr0s4z5qvUJXkMUQUfVyyQBRJK8lE"
"p34KwC6YQBMa70dGh330SlW3uskmQ12xgVBKlG01JhZ36qWgXv48qqkdia13/rEsuU1nAncE"
"MRUHj9CwBRCBbEEDDEA8wkQfFPh9Bwdw49oHwHV/+Wuvaea99+epki05kzcrmUAdisCgOgxm"
"tO6664Ws7TYOnfaBCSaXXx4mqCOcgGgcIl8E+vCqpjT6VH3cgeeSm2zwCBCKLM7669SbAYMG"
"ZW9iMe6YhNC9oBqf8KWvJziQEfHoh37b8XRaoMgPLSji+vQXVN86DItUPQfWW4bAqZj+45rX"
"eiW89BkQUutLGgfyFAMXKCIAQJheANzQOjd4gBI9oIQHKPg6/QkqUCPQWt0w1i7OEbAiB0zh"
"o0zCvqGgrUk0kJ4ZDkdBD0jCCxFoABmoAAIZtKEH/j0wBOsUkbQ5UKBf38Pdxr50gl5tq1QU"
"UaEUVyiYFj6JelswgxvwkAIr3KEGf/hDBCZxg0mQQRJ4wAMVFPADQiSNAlvaWPduJ8DxcSt4"
"DZmiHh1VRSsy6XUwsOAgoCADMZAADAW4QQQicIMwLnISD2hABCyQiaQNwVpIXGLQEsVJ1+zx"
"k24SiR+ZxAXqKQIKZeiBFxhxiCDgcBJjhGUEyhDLSUDhBArAWaBwxyn//axXQeucQ0BJzFCC"
"ZJRLoh4MIkAFSUTACzkgwh8eIAlJwLKaknjAJBqwzQdAAXgLe8On4Ga7EApqCe7ymjAXUsx2"
"HumYyNSRKdtAhT94QQw5/rCCFdhATSrgAZsPIAMIyECGBzygCXJY2CZst73c+e8M4xNfr06o"
"EHdatEgfiedUNkG9LUYgjGwgRAjYEIEHlPQB/lQAFVC6w5VSoQKXINfy7jCHy3XPnGJol6F+"
"dYLyfQ0hFw0qkDyiUalcgnprmMQfoMCGEyBiEIPAgyRRSoWqptGlKfhnJnhALqoJBQG6k+O5"
"rlVCBzSRc0AVqlp91JGiJoV6F8DDR//ABkQYwQis84ALMqGAFPS1qmtUqQJkoIABNKtJL5yT"
"AIxiRFxlzHbeg6jGJjrRg6wVRtwhDjE5MrpnxYp6llDqH27wgAzwwBCUkJ8ZAiCDFFAhA4jI"
"/oBBZUDbFGTAD2hoUvLsNAKvcsl/GeOd5gbIhJ5SVCB60+xQY4PcVSnXPc1F1UBatBG3DoUQ"
"M3TdM73ATxloMAAB2KAbBEEJPiiAD1pIbwb4kALaZgAONGiSV+nEgQU8QSi/DQHGBIYoYHGr"
"uCJLbnTZ2lyGaGfAD3rugREMoYxYVyia+IHr8ECEAvDTCB64gBFKMAhDTDAAhoAED2SQCT6U"
"2K98aK0KzDAr7U1rAwQQis5wACqLDap8PKUsAZjA4KIRZKjKZeeCE1wQ9fQYRA62Lg0GmUUz"
"QLMATRiEIDjYOhCP1xBGGARtHRgARVBiEArAwwDMAAgmRSEOVWjb/lTe8KQQbkx3DjDUTikb"
"Mkb9aMBHNnKeOxTdPQ+ZuQpmLmzymFkFBzpIGHnwKd1AwR6QIJplgIICBOE6IFBwguS9qxF+"
"oAjVKsIFPHiEGSK3JA6coAlsKEMcTmAFHGMMCxnDAQ6wkANC6CEIerDSHWy1RK1p7lfc4paf"
"kVzgYSNnuhgljrIPrWdAHzmtP46UcxNCZItYFxBYkAEiRNCDHHiBCEQgwaPZkAJKuOGBWnRD"
"ALJMCUMowgPvlrAh+FAIM2hiSQ7wlBU4QAhCzIEQU7g3lOKQOVHFeQj/NXaDn8vsBTe82pZ1"
"uCenzfAgR0oiEJ+IdQnBgx+s4YYFKEAO/nLAiAL8gQyQ8IC7v7tuBQxCvG646w8oYQQZtI5J"
"AghCYvMUB1/aLc7bitebEKzwaQOZ2rV5uLOR3R4859kgyCo6e5hj3fH2gBHehqYXQi7GBwxC"
"BKilBCW+0AYyfIHRrdtCAMa+aTM4QV8E8NKhxCTcMwxN6R9i+qDxbnRj7h3pxy5y0xksdb1T"
"tyLWjUEYKEGFkHvBkZPY5hihQAUZDKINCnjAIaDQhjW4ThEWpDnruHqvONSxa1Kvdt8lznci"
"710KFZc47BH9d2Sx/s6Ity4LrLCHGoT8mpKo6geo8IEPtAGqD/DCHW4gAyOkdt1m0LALbk6u"
"TTAAAb7+Epm+/pR6PjO76FB/Z5CX/Xrowp78SCJ66yNOYI1bV5FWgGbX97BSKKCADWTIwAfw"
"8AAriOEOUIAHbQAJX3BXQWQIhmAG8aUnOzAmC1AqidArFmABi1ACAlAC2HI52td93vd9w4Zs"
"HOgffYZ+hhcgyFVs6RcgB1Z7uOd+blUDVvAHMThNldcGspUBBfVUPcAGIXcHzHR5bZBixxcG"
"CXgGUBICP/ADRmAILiAHClACBAA4TkIAYBImIeh9F3dxIzh0UIeCgadZW1hoIgiGV1hoEuE3"
"O7ckUGAF3eYFLEAFKdAGaiQDe0BYL0cFUaIH/NReMpBie1CHAyA/czAHJ5AJcvB2/tBChSNQ"
"hqpnHCV4fupnJBV1fpQYeOUHiWKIaILXghFRVJsAOGEAA5KQAyVVVQYVh21geZQgCA6wB20Q"
"hGBWW661UiVAA4EAByWwQOXyiFT0hSpogpIYbSPIi5dYicvWepWoicToIS54PJvgCEGAAJRD"
"CI6gB/ITAaTYTJFUg69IdubWOitgCD/gAlogAxmgBR9ABilQAjCgL8vIR3j3jkmXHcjoiIZW"
"cUp3gtQWbb6IZPRYhs1YLpvwBm+gB5qgB4M4BwkwiDTWB46QAAwwBHSAAi8QCHjgBSdQUouE"
"UnJ4fK31ADJgaerWOh6AeYigBQqQBSpwL/IIj/+Yif0Y/iTONpPKUWzQ1pKMGHXxyI/p14nl"
"ogc7sAm5tgOEsAN64AAJsAnp0gR8kIDWswIr4AY38AIy2FQGFQEl0ADrNVgPoAUPwAYHwANd"
"1mWKYASQkAF+dQWICC0t2Yv02Gz/WI+W+Hc99myHhoybOHV8N37K6BDjMgVupDyOMAebQAhx"
"UAIllgmZYAg8gEokMAVT0ICbMAc1IBReUAb8ZFIPgAgj1nxr5wJM6ECKgGVgFgY8kAkaIAeV"
"NCs8SS85aY9xIpc1CWgs+JIwkpd/5h7FqGcQMS4IEAeIAJqZUALnlQlN4ACBqQcEiZB6oAcz"
"cweOMBRlgJmIgAgGhQhiB16M/tZl7gZB69aHrygDcjAG0ZkpbdlO4PeaNpl0xvaaOlmP54mb"
"vPkQy0IIbjAIFigAhBAEBHkHb9Bvd6AJBDmg/XkHlXBYQiGDZfAAmZcCkLCE8EZlWsSEIGYI"
"g/AIYJYGeUAp8nlZgtYd/tiaUZebbhmb/3GGyqIJePIsbEagd0CUbPYkb8AG/DRYfGAEIuAB"
"Oqpy3clojGNpFnqaGuAAdtKhHgousgmMa4WiyiIAyvkGO1CQMWoniFAGkqAAg5WjPOpu8AZe"
"CAh6ZlCSYXAFGzolRnqkaLpZvbksCBBwmUJSV8oDUCWOhuABATBlE6RuAaCEqCUCj+AEUzoV"
"7Jem/oQaVD45K2/AATHQonTSMkmhAIiAB5YXBmDnbtBzp53WaViGZXnwCDE1GoUaqkd6qH60"
"NI+qg1l2VwjYpUAgCAgocx3mAmcqqrQqVKRqRaZqFDiQZQXYbjTXadu5dkbgApv2CIXwHLWa"
"rLa6pqO0CWqWFC4wCEv4pXV6bhR0bsPqBAnVJGmlrN6qQrfaQjFgAK+SFCOwbu3mAYIAc4xW"
"CDAQBj+gJ+H3rfS6N+HaQkIQCVORWu8GbzsaoWagCHEAK7NarwZbPMy6LOVaJ09QBVOBCQHL"
"o+02mu9jBLrIlud5sBrbfvQZTwvQB3FAB0bRNoawrjy6hEn4A+Xpjhm7/rEuS2wdi0w3EAIb"
"QGpGsQEwwJ2vKnZgUDYt+7JAO59+GU/BMrArOxRdZkF2Sgk/wAOl47PuGbRSC4kxi0wT0Acb"
"IAFT0ap3agh2QANOcAXs87NTW7bvObTx5AcMgAmJMBV1AAPu5m408Ahp4Edka7YvG5B+FAdn"
"0LdHOxRw+wMUpAhyoANPezwWYCp4u7g1ea9VswmRMF9CcQILwAQUMLBSIQeKAATW8wNysAAc"
"kIaS0wSLoriMe7pUW7VuQ5S3kxQEAFN0UARTMQcB0DqfpgGL0AeEmSd6MC2Z0gQEELwEkAhR"
"WzJJirqx6bgoswkxwAEJgAlJcQYlcAkVMDFJ/sECTWAEWfRpXLAADJCULVQCccACwTu+pqum"
"x/sid+t6JsOkbgM4VQA1NDUUmDC9cCAESaEBiACFMwRqSVACC5kAFys5FSC8LHAJSZC4LFC8"
"8/KBJ9qTKXii84pAFDE6aOMIVQC9pTMHGlABSYC/Q8EDA6CYI2AGTKgBC9AEc1AFYMI+00sA"
"LGABLDDDwTuBDHw0MZm8qReCANmtVKS3VeMID2mEz3oHcgAHcVABQrEAC6ABXPAIFpAIMFAC"
"PMAFl0Bwm3ACHHC4ZQNTlyC8MDzDE2gBTXDD6gMRq0KXgEeis2ebeulc0pV7KDNTQ/GQDHAC"
"QpFYWaAoiWAATLAI/osgBImAu4kABAtwAlZ8NXfQBCOglJKzCZcQw8Ebw5IMw2wwgQucvg2c"
"jw13k34WaEI2l7sJm3D8w0CsLxcrlHMQJmiTWI2QKP2VCIswAEMAyIngAkyQAKeWACFwByXQ"
"yAsUqDizA5H8xTBMADIswyyQwgtgAZdwCZrcvqJsgk43aFqIxtc8j9AFytHcl+6LMpjwQoQQ"
"A6vcyHfgsEIhBEvwOBEoy3XAAEywAJcwAGfTMQmQqE3ABPtzLm7DAQdMvsZ8CRNIAEycuM/c"
"BGXczWesbB+ajCRKjOGHk6H8gZEIj9b2uH0AOITJywSgMoTwQk+wzkywAX0QzxqQ0UKg/gGm"
"Rpi6cpSJq59MgANvILr6EgLLHLwLQADPTL6Y/MyX0ARc0AQlkNDDI4wRDZ+Eh9Trt4lhyJum"
"XMFlYwB9cMGdsgDRuQPLcwlwwARnQNIW8MeJwAE8oMELyQGOkAlz4AAEgAmOUCgKKcz6cgL/"
"HMkw3AQpzAJcgL3NjNBQgNBDrdBwYtTlV3jjB4a3t4LJOIxObdEXXTWRsANMAJ21UgJXvTyJ"
"sNUJsARnwARsmyZN8IlJqQUucAcnMAcjYAE44AgWMAIKOZA4wwQHrNN3XcxNQMYIfdt9jdCL"
"sL4u6YXGiJcVvc0iOs1Mt8OA3ZNU97g7MAKOYH0jUAJuhABD/oEAE8AEa7vOXhIHCyDAOFAH"
"FYCad2ABmxACcUBjhCLECYkESKAwTXDAUiM1cy3Pdi3PQY3QXBDUw+2aZDii/J2ev6ibOgmX"
"AT7gbsyFSVY2CZDg+8MCADw5d/BCOmDdCRA1ghJ3CDAAl7AIC6ADGrAJmUAICdAE6RICFsDW"
"YPAGczCYgVku7c3Tl2DFdt0EP73MQV3ffs0FE+xj+93fwD2GBF7gmPjj02xMB549Z+AIOGAB"
"UOBGbiQldzAAI8AALGzdA1AFiEC92g3Ii5AAJRACuowDUb4AOPBv/zmQvXwvJYC9Mw7fPn3X"
"3VvjNz6oxrvjvx0nY1iw2lybDn1s/k+d3HRCx3myCQJQBQkwgThQmP65AxzABWu9PwRXAYmQ"
"CFCYBAuQCEmwY1wQAo5Q2kywBE2ACbJ2ByCeM7pGLmk+wwswwwLtzAj90z8N1DZeAmt8MDYp"
"2GyM3Ol5jyLI2EWO4Myd09E5Am8QA5tQBYvwJfszBLrSB0uwAIiSu0lQX7WdAGdACIkwAows"
"awo5AnfgAM1ZLl4cw0z8z2Qs0FID1DE+1LI+ibRO5769zQ1m52285zrOWaMT2ZW7QGydADug"
"2meQCNqDtWfDAb8MPJiQBAmwCAU8AxYAskzAyJgQBNR4NizwnwA3LuFuAfIMw69+33jdBIuA"
"0AdwACUA/gUG1u78nedAPnhLfYwpbzQZ5TdspgeE8OmHzgCbEM5VEIELoD0hQAAIQJRwkAh9"
"oMVRXgGJ+/M/j9ddopCY8Aay9owJQAIrDitvAIUHXcyrbtu33b0iP/LDpN9D/oU9LO/v6ePi"
"R1R+s97dvgBenvMPfgcMsGOJUAVVwAFNQJRzUOkVMwJcngTFVT40jgPPq+mOEAKpPZj3LDiz"
"MgdWvMxa/+opXONcUAKLcABNAAUV+BBin4VCK5u5zp7KGIwxfzwlUFOOwADn/AYMEIG5LMAP"
"eDYbcO38xm8DsNpNhOlc4AhgMIFvcDlzkNpjHgT8OSs1H8lqXsyxLtQjH/L2/o3j2Gwy1Hy3"
"+d24MDnKAs6Jav/Id8AFAkzm4cwBE8UAjsABwDMHvbsIcJMA+YYIMX3IVHjfSI4IeuAIhIAD"
"ay3E1Yj+cJ0n643tAMGiSZNLl1gIFNhkwUAuUA6UaMJlERcuACxexJgRoxSOHT1+BBlS5EiQ"
"FkdeJJlSpUiTKzsCcBlT5syTGm3exInxzk6ePX3+BBpUaNAFCfokwLEpwSZMBEYQwESI0JA4"
"Fu5wWMCEAwcmAjIxeXqCgAMuLHYw4UJIgIUTHMQ4wqEHByE9hN4MxRtULIuCBQVeGhgxMBQo"
"AxdBKVEiZ06ajVfCdBxZ8mTKlUMuxpwZQF7OnTtj/hqBKUGVTY7ejEiUCKqjrQRc3HF0gkkf"
"Dg6YlDjxlEBbLhZ2EODApe2JEzkcwdUj981yz5wJEOC7wELBwFwYWm9CGHGJRYo127QcXvx4"
"8uUdf0dvs/l69nc23bEwAsGcO6QTMSEQH0eCBE0ewcYkvj5GOKGJE0KwwALQFrikDyYeJIAJ"
"TBwYwRFM9HBEDySQCEIPTe5q7ycCkphugQX4uoSLhSKCqInEEnMRovTAM69GG2/EcaYZd7Qo"
"xJ2U8HGoExKZIwH3NrEgkQUSWSIBMBI4QY4pEiCELQ5GGIgJQiLCQbc5TrAALPwEwIGDOUKo"
"ITlHgghiDj2C7ImL/Fas/g47F13k4sU8F9GCx4xyBDRQQc3zc0c4DwUqgQp2ECABDjbZLUwm"
"GEhgDi5CAI6Q2froozfgFuBiKxZOcISt54izYD9CcMAEhw71cLM5EHdCwqdFVgQMsMKaOKBX"
"X3t98cVC/xy0WGOPTWlYQxFd772dduCJPvcqQEuLTOaAron8QqCUg0sWGCHcEcBahAkxRrhE"
"BxxCAHcOhWRbgIAQcIhLDBwcWdWRWvPaEAnmeqr1kJ0qYNDOgRzy1SGHDosxMWWJRTZiiQN9"
"eEZmgwTR2TsyKUGDTB4RiwBtm+BAAA6G6ENbrTABSzh4S+CgD7ZwKEFkJhzYjRBMjoOLyjn+"
"/s0LxOWY83c5TSpQiKKBLOC1V+18DVYLhyveaGKrrw6Paosvzktjn7y+A5FMeChhgBIWYENO"
"LvoQQIARNMA5jtAWcCBe2S5BBKncBBhAbhYW4OCEM/CN6o05cKDP8Fl/Yg7oOxr/UA9EAqvz"
"AISdTmy7FxHRGmKsPwf9sc7RuxiDn3aIAoHONO5YgUweGEjkEw5g4gRWH1ygAqcWGCKr56Tj"
"ANMR4pCTheeYoJfe5A5/Q6nEecq4p6HvMvpx5hKQA5EXBzqgaWATk/rF7qYeHaXQz0efo/LT"
"u/iJnwRo1r0mXFegCQvkLMGCcplIAJMAEdEBIoJDgII4IEUWCEFT/ryVCbFYADhkCl4JOJaA"
"EPxsDm5anFysYD3rUe8NHxraHaxghB8YARKQ4AEfUpgJFrIQES+E4Qu1MMP1eS59N0RWDdl3"
"qJPp4FD0YcICeCA2FalIIeVawFz2c4nnyCsiSTjBJSpwgj5goi0VYAEiWMCGJhykBC5wgRH4"
"ELw5HGcTF1xVDnDABhwg4g56OMT0jKYJTXTwDVMIgReYJgNFGMKPRgBkIMMIRki4IIV84IMO"
"bYhDRuZIkaR7nI+iMIIEDeFQIeAACxWAiPvdL3cqcgoHGECbOOwGKypCSwlCwwHQKAAKIsuO"
"AvwogjBk4hKsOoFoOCAV+gggBSHggQJO/pCc6d2BjsYEYeTgeBctCOICZoCmGdzgBhi4IQBA"
"AIIhFBGAAPgxAI9cZCPFKR5womeXhDjnDhwxBQ5sYAE7IYQBqgCUM4xgARvQAKKYoAF+fuVs"
"FbjEIgQaH1724RJtWcAmBsAFJjSBBw4IwQnuwIVMvHBs2wRjJjzGRJPVkwGOwEIfThCANjQg"
"AieFAhusML1DxPENcExOTJNzByhQIgDT3EI0dbrTaJYznOMEamN8ip44xOEEDhCAA5RakCfo"
"rgSXaEQMNhGDnsgNPwUBW3OcdQIF8JOhWiAAIrhQAaU5pQ+OGIGcOBCfRWiABRV4Ib0cUQIe"
"aMCmLtCABBVS/lEWWKAwI0jAUxYABTPw4QYR+MMf2BCBByCCDR+8oyZaqollHoKOegjCG3IA"
"CUMEAAYrgAFPRWuGof40qKetSWm/Y7K2CSAEri1DE7SQp7dWgAFxwMR7NvEUB6RmAUmAw6GY"
"wIO6xouTk9MBReLFLUKACkwM0EIfEsGDHxjCAQ/gjwIUsd0AgLGum5TBGpvABoTIpgSKGEQT"
"ErveCEwCEQpwwBsOoYcPypSyQSAECXJgHC1wEwjTBO1odapajaDWwCwhMHrIysSxKDW2DPDY"
"5M62iNxuggMEoEpRCRAHP/REql2L1h0E8AgNgHWscLUAxxZhv3DNISsO0EAVxhBE/hfMwQVg"
"IsQcsOnH6oKxhIPAxCEKQAQisMkKCgCCIL7gKisUQAyLhcIDSoCIQUD2pTENQslCYAUreKFA"
"J1jEdhVBTRhcILQCTnCBD3zgNKdHDo+A8yCIK4M8VOAOj3DCI+RAMA7cgQFOYcIQHDCEUS3C"
"B8H181A0ZIUcBKEMXtjJk+WQ1zxVoAKZWEQmuDC5RSQgE3dggVKbEESKctKWEBVANKvpBkUY"
"gQceYLQX/lCAIJDAChFQoyO47AVZy/oGZHjAA/gwiA7NIQcOCNkJrBBRAgiAEDt5wwmMoE03"
"/Heazxxtm2m05kZqe0bEfYScFaDCPAwgCtRtgSII4Gfp/j5oXOPaDQGUdIeE+gQHVniADP5Q"
"A0YwggT9ZoSQaxABKvABdhLUgUC1h2lVZmITLCgBIS4hQQu4YAFGOIFw9JCJ6o45tDcOgheG"
"LHIi1GC/vP6DF066Xi43Id8KGMQXcvAGNtSciQs46qOgDdgTbKd+PFDEM6eZbW9vm9ugK/qO"
"vMsDORBXDj8YQyYMkQkdtIAnVKGKg/qw4USAZg6XYEJP5mAEM8i8Bmf3N8AZQQRGoN3WbDj4"
"iqUWPgHowGQVsEAcSjCGEhiBCT84iAUc4YJC3JTVHogAvbxABDEUIAcF8DUK2tADyvfgASoV"
"AwqiLIMwGEEEEi3DQS7x7J4w/qAIRUhofJtwFxxkYqfEDcBOk46To1tt9jzyoyJ+AMZHuMAO"
"u1dEIyaQBZ70AWVx4MrWOUBV90ygJw4YhBkkwWsiD5kIYCBB9nPg7+zz++wFuAFhZviiBSxi"
"CCMogRyi4IJFBCATLjDQqE6wUw8Y4gE5EAPb97tfIjy+AGt0A2kShBR4sj+IgCcLA0OANVCL"
"g6/hgBcqAvjZAViZAwugMkHgqR+4hNgbM9K6PdqrvUH5QD8ZLRj4gR+ggTFYAPfhCa1oJ6pg"
"gJ5IgDfZCS7wADeQhMSStZRDOccrAOvLPhIIgn6rNX+DvMV6AMJwAAg4AD4QgwywgO4SBLVg"
"ARyQ/oFo8oAyYLQf/MEc0C8vsAIceDwruIEcQARougBKGISai4Caa4JBCACxew89gIgSqJ+d"
"QAC02DSNGgRD2Ck3GIFFAAIzMIIoHEHMCEEbOcRCEbDtggE5WIBG8AnaYJsY5Aln2QTJCcAy"
"mAROPCnEQiwd5LL9I0UhZBMSqAFT7JAgAANG8EK2W7v9IoQcsIIgsIJZ9EL/48I/QAEyoAIZ"
"4AMqEMYbEIMGaIIAlCYPeIAIQIEG6DktsAJLnAOcsSUtuQNNwZPEWDgFMILY2ynuUAQYcIFr"
"WkTNSETKKMeHEbAtoIEtOIJEsAOf6AIGsMSe2IQF6AMGOEZoagLGegAW/jipMvjElVuvlCtI"
"kduv7hvC7fu3f6sBIgMDIqs1k+O/HOg//yOCPxCDk2uAyRMESpCBPZiECJifcTMhGUgBSviC"
"l7OCgYgDLIAnwEiMPHmhKeMDI1AEILgAZBwtIPgBRUhH9DjHmAhKqlnH0LKDJ9gCA8ALJbhH"
"tEAEsoMmI5ABRCiCSQi2B5CEYBvJBmgAKBjJgUQsNtDC/SMyL0TFs4vIAgCDVly7H0RIyEPI"
"IBhDLxCDu2QDKPAAMxAEGUCBZYQCGRABHqCENoAESvCAL6AEQYCCO5iDhgIVpJmyEtg0RBjH"
"agKCmzozaMopaDoz3QPKotyRoVQf0RwdAYsm/kU4gi1wgqFAnaKKgyQYABf4TEoQgUFQgNxU"
"ADzYTTxIAd/cJGGkAknYymCDgoDsNR5kNFKkxR88yy5DucQiy7x8ACpIgWWEO6kESSpogMZS"
"ABaoKTXUAiOwKUrIDsSogO3hAZzMzG0aOtQ0g9Cqppw0zYcJqvp8JNFShDj4AWgCAjswgy0I"
"CgsbAZwZAuDgAB4oBGgShDAYBA9QhPozBMSkBEowBBGwTQu1TUtwgUHw0EHAA2F8gO48zggo"
"gzIgSy840ZNiA14TyBRF0Qh4rwT0AA9QwzaQgSTMhAsQBCNIgUmww7YxIUP4ggw4TCNwAVeq"
"TB6gtu0yBGqCT53K/szu4gEXMAT8LB/bw9LSEi0X+IHNDK0tyKoYEIAhYIJE4BQzeRQeODOq"
"pASenCY32MkLoNNqQkM6naYI9QARMIIvUMxBkAEPNYJBqFA1lIQIiLLL6zIu+kVKCDrPBIIx"
"Qy/CUABp+gIZgDlEmANC6LtAUkNACgNH3K5BjNLRYjVDcIEK/aYtTTAdYdVDFK1BBNP4pAPm"
"2wkOSIIkiAM0ZYBIUAKq2oRLcAFostEnRU1siyY6NYOddAMPEIQ1EIRoFQQPWAOenMrqpIJl"
"FIM/gAItkAGbsladcgMxIgNE8EMzAMk18AA2eJxjdAND6I89zclSLVVruqabuoBX1dd9/k0w"
"0YJSnoKDPtiJOSAQJqgCaLkDJaiES9yEH0BGnkRWAaPTORUEN5DWAIjWMXMmZHymC4hQQxjU"
"JMw3xPSAzRStbiLPaJpQZzUDmHkDFlAE1ymTJJIDeo1SbOOmAOTXneVZRfLX0WoE5MsqoeCA"
"OPBGm0VDi52mil3aOL02NISmAKTTHj1MC8VA1ISBPlpMa3WDNVDALMqEOEgAiGMBASAAI0Fa"
"m5XTnmXbtq0YnjJZnRJHDYgDS/KMOYiDcE3bZU1WabLZ9exA+OwjEjJWLHQDSjCC+YGITAiC"
"V6u2cNzbztypuPVAt7Xcy/0OuBUwn9QAlFmdO2ACy9zb0SVd/rlF1Z8UVwg1AgWYzBJ4gNiD"
"ARMcJEA6Qdr10hPUPUM4wc6CgS2IXWrSW8wV3uG9Cc1dRzvgJziIg17Fi+BgIQ2Q29KFTxpw"
"gSnTgAHIBAXoPTsg1dG1pgtwAY4ZCHO9Qc9kXYOQoNyUgwGoUjDyMTvIWWraSWmK2Mol3vsV"
"3tGFgfY0gjGogA0wABPwiRhIBA1YT5BF1QCw0GuS3p2igaIlgN8aK4rIE45BBH7iA/f9vRPE"
"XSCgXHrV3RCeNjCSAxfInk0DowCA3zNzWmlFVvyFYcwt3f01hEfYAJ9QAgNoAg2Ytg9u4J3q"
"Xs7c3/0FAiQNX+wFnwEQmxayFgBqeyE+GJvsRSSOaSGNuuBNeioTARVrqdIR5oOF4IFCCoBt"
"6ibdDSRImLZBjOE1btsGhoEWWIQoYIAnEAIX+C/J/eGdMoLEyARFyGM33iZB2j0wIk9uuimd"
"0oB9tFk2ZuRGduRHhuRIluRJpuRKtuRLxuRM1uRN5uROppqAAAA7");

  konoeLogoCanvas->clearAll();
  konoeLogoCanvas->drawImagenw(0,0,konoeLogoImage);
  konoeLogoCanvas->setSize(410,143);
}

void T473TclRunControlPanel::setNameSpaceItem()
{
  KChar lbl[256];
  KChar s[64];
  KChar t[64];

  strcpy(s, nsExtractor->getKCharValue("DAQ_HOST"));
  strcpy(t, nsExtractor->getKCharValue("DAQ_STATUS"));
  if(s != (KChar*)-1 && t != (KChar*)-1){
    sprintf(lbl, "%-7s \\[%-6s\\]",t,s);
    daqStateDrawLabel->setCaption(lbl);
  }

  strcpy(s, nsExtractor->getKCharValue("DATAHUB_HOST"));
  strcpy(t, nsExtractor->getKCharValue("DATAHUB_STATUS"));
  if(s != (KChar*)-1 && t != (KChar*)-1){
    sprintf(lbl, "%-7s \\[%-6s\\]",t,s);
    dataHubStateDrawLabel->setCaption(lbl);
  }

  strcpy(s, nsExtractor->getKCharValue("HISTOGRAM_HOST"));
  strcpy(t, nsExtractor->getKCharValue("HISTOGRAM_STATUS"));
  if(s != (KChar*)-1 && t != (KChar*)-1){
    sprintf(lbl, "%-7s \\[%-6s\\]",t,s);
    hisotgramStateDrawLabel->setCaption(lbl);
  }

  strcpy(s, nsExtractor->getKCharValue("RECONSTRUCT_HOST"));
  strcpy(t, nsExtractor->getKCharValue("RECONSTRUCT_STATUS"));
  if(s != (KChar*)-1 && t != (KChar*)-1){
    sprintf(lbl, "%-7s \\[%-6s\\]",t,s);
    reconstructStateDrawLabel->setCaption(lbl);
  }

  strcpy(s, nsExtractor->getKCharValue("LOGGER_HOST"));
  strcpy(t, nsExtractor->getKCharValue("LOGGER_STATUS"));
  if(s != (KChar*)-1 && t != (KChar*)-1){
    sprintf(lbl, "%-7s \\[%-6s\\]",t,s);
    loggerStateDrawLabel->setCaption(lbl);
  }
}

////////////////////////////////////////

void T473TclRunControlPanel::getDate()
{
  KChar cmd[1024];
  KChar value[256];
  FILE* ofile;

  KInt i=0;
  sprintf(cmd, "/bin/date");
  if((ofile = popen(cmd, "r")) != NULL){
    while(fgets(cmd, 1024, ofile) != NULL){
      strcpy(value, cmd);
    }
  }
  pclose(ofile);

 dateLabel->setCaption(value);
}

////////////////////////////////////////

void T473TclRunControlPanel::getStorageStatus()
{
  KChar cmd[1024];
  KChar value1[256];
  KChar value2[256];
  FILE* ofile;

  KInt i=0;
  for(i=0; i<256; i++){
    value1[i] = '\0';
    value2[i] = '\0';
  }

  i = 0;
  sprintf(cmd, "/bin/df -h");
  if((ofile = popen(cmd, "r")) != NULL){
    while(fgets(cmd, 1024, ofile) != NULL){
      i++;
      if(i==1) strcpy(value1, cmd);
      if(strstr(cmd, "sdb1") != NULL) strcpy(value2, cmd);
    }
  }
  pclose(ofile);
  storageLabel->setCaption(value1);
  if(strlen(value2) > 0)
    storageValueLabel->setCaption(value2);
  else
    storageValueLabel->setCaption("No storage");
}

////////////////////////////////////////

void T473TclRunControlPanel::getDAQInfo()
{
  KChar v[32];
  KFloat n;
  n = strtod(nsExtractor->getKCharValue("DAQ_EFFICIENCY_SUM"), (KChar**)NULL);
  n *= 100.;
  n = 100. - n;
  sprintf(v, "%.2f \\[%\\]", n);
  deadTimeTotalLabel->setCaption( v );

  n = strtod(nsExtractor->getKCharValue("DAQ_EFFICIENCY"), (KChar**)NULL);
  n *= 100.;
  n = 100. - n;
  sprintf(v, "%.2f \\[%\\]", n);
  deadTimeSpillLabel->setCaption( v );

  n = strtod(nsExtractor->getKCharValue("DAQ_SPEED_SUM"), (KChar**)NULL);
  sprintf(v, "%.2f \\[Hz\\]", n);
  daqSpeedTotalLabel->setCaption( v );

  n = strtod(nsExtractor->getKCharValue("DAQ_SPEED"), (KChar**)NULL);
  sprintf(v, "%.2f \\[Hz\\]", n);
  daqSpeedSpillLabel->setCaption( v );
}

////////////////////////////////////////

KChar* T473TclRunControlPanel::getRunType()
{
  sprintf(value, runTypeElectron->getValue());

  if(strcmp(value, "runTypeLuminosity")==0){
    strcpy(value, "luminosity_run");
    return value;
  }
  if(strcmp(value, "runTypeElectron")==0){
    strcpy(value, "electron_run");
    return value;
  }
  if(strcmp(value, "runTypeHadron")==0){
    strcpy(value, "hadron_run");
    return value;
  }
  if(strcmp(value, "runTypeCosmic")==0){
    strcpy(value, "cosmic_ray_run");
    return value;
  }
  if(strcmp(value, "runTypePedestal")==0){
    strcpy(value, "pedestal_run");
    return value;
  }
  if(strcmp(value, "runTypeOther")==0){
    strcpy(value, "other_run");
    return value;
  }
}

void T473TclRunControlPanel::readyNow()
{
  KInt a = strcmp(nsExtractor->getKCharValue("DAQ_STATUS"), "none");
  KInt b = strcmp(nsExtractor->getKCharValue("DATAHUB_STATUS"), "none");
  KInt c = strcmp(nsExtractor->getKCharValue("HISTOGRAM_STATUS"), "none");
  KInt d = strcmp(nsExtractor->getKCharValue("RECONSTRUCT_STATUS"), "none");
  KInt e = strcmp(nsExtractor->getKCharValue("LOGGER_STATUS"), "none");

  a = abs(a);
  b = abs(b);
  c = abs(c);
  d = abs(d);
  e = abs(e);

  KInt i = runTypeCheckButton->isChecked();
  KInt j = runInfoCheckButton->isChecked();
  KInt k = particleEnergyCheckButton->isChecked();
  KInt l = storageCheckButton->isChecked();
  KInt m = cherenkovCheckButton->isChecked();
  KInt n = csiTempCheckButton->isChecked();
  KInt o = positionCheckButton->isChecked();
  KInt p = angleCheckButton->isChecked();
  KInt q = shutterCheckButton->isChecked();

  if(state == KSHUTDOWN || state == KEND){
    if(a & b & c & d & e){
      if(i & j & k & l & m & n & o & p & q){
        beginButton->enable();
        shutdownButton->enable();
      }
      else{
        beginButton->disable();
      }
    }
  }
}

void T473TclRunControlPanel::kBegin()
{
  KChar value[256];

  if(!getenv("NONAMESPACE")){
    nsSetter->set( "RUN_NUMBER", getRunNum() );
    nsSetter->set( "RUN_TYPE", getRunType() );
    nsSetter->set( "PARTICLE_ENERGY", getParticleEnergy() );
    nsSetter->set( "CSI_POSITION", getPosition() );
    nsSetter->set( "ANGLE", getAngle() );
    nsSetter->set( "CSI_TEMP", getTemperature() );
    nsSetter->set( "SHUTTER_LEFT", getLeftShutter() );
    nsSetter->set( "SHUTTER_RIGHT", getRightShutter() );
    nsSetter->set( "CHERENKOV_P", getPressure() );
    sprintf( value, "%s-logfile.dat", getRunNum() );
    nsSetter->set( "LOGFILE_NAME", value );
    sprintf( value, "%s.dat", getRunNum() );
    nsSetter->set( "DATAHUB_STORAGE_FILE_NAME", value);
    nsSetter->set( "PLAYER1", getShift1() );
    nsSetter->set( "PLAYER2", getShift2() );


    KonoeTime ktime;
    KChar* timetxt = ktime.getTimeInfo();
    nsSetter->set("RUN_BEGIN_TIME",timetxt);
    startTimeDrawLabel->setCaption(timetxt);

    runNumberDrawLabel->setCaption( getRunNum() );
    endTimeDrawLabel->setCaption( "still running" );

    KInt n;
    if( n = atoi(angleEntry->getText()) ) nsSetter->set( "THETA_Y", n );
    else                                  nsSetter->set( "THETA_Y", "0" );

//##################
    KInt daqAnswer;
    KInt hubAnswer;
    KInt histAnswer;
    KInt logAnswer;

    beginButton->after(1000);
    if( getRunNum() ){
cout << "%%%%%%%%%%%%%%%%%%   new_file command send." << endl;
      hubAnswer = dataHubControl->control("new_file");
    }
    else{
cout << "%%%%%%%%%%%%%%%%%%   file_close command send." << endl;
      hubAnswer = dataHubControl->control("file_close");
    }
    beginButton->after(1000);
    daqAnswer = daqControl->control("begin");
    beginButton->after(100);
    histAnswer = histControl->control("begin");
    beginButton->after(100);
    logAnswer = loggerControl->control("begin");
  }  

  beginButton->disable();
  shutdownButton->disable();
  resumeButton->disable();
  endButton->disable();
  pauseButton->enable();
  runTypeCheckButton->disable();
  runInfoCheckButton->disable();
  particleEnergyCheckButton->disable();
  storageCheckButton->disable();
  runTypeLuminosity->disable();
  runTypeElectron->disable();
  runTypeHadron->disable();
  runTypeCosmic->disable();
  runTypePedestal->disable();
  runTypeOther->disable();
  runInfoEntry->disable();
  particleEnergyEntry->disable();
  cherenkovCheckButton->disable();
  cherenkovEntry->disable();
  csiTempCheckButton->disable();
  csiTempEntry->disable();
  positionCheckButton->disable();
  positionEntry->disable();
  angleCheckButton->disable();
  angleEntry->disable();
  shutterCheckButton->disable();
  leftShutterEntry->disable();
  rightShutterEntry->disable();
  nameEntry1->disable();
  nameEntry2->disable();

  state = KBEGIN;
}

void T473TclRunControlPanel::kEnd()
{

  if(!getenv("NONAMESPACE")){
    KonoeTime ktime;
    KChar* timetxt = ktime.getTimeInfo();
    nsSetter->set("RUN_END_TIME",timetxt);
    endTimeDrawLabel->setCaption(timetxt);

    KInt n;
    KChar runNum[32];
    if( n = atoi(runInfoEntry->getText()) ){
      sprintf(runNum, "%d", ++n);
      runInfoEntry->setText( runNum );
    }

//###############
    histControl->control("pause");
    endButton->after(300);
    daqControl->control("end");
    histControl->control("end");
    loggerControl->control("end");
  }

  resumeButton->disable();
  pauseButton->disable();
  endButton->disable();
  beginButton->disable();
  shutdownButton->enable();

  if(!runSummaryPanel)
    runSummaryPanel = new T473TclRunSummaryPanel(this);
  runSummaryPanel->view();

  runTypeCheckButton->enable();
  runInfoCheckButton->enable();
  particleEnergyCheckButton->enable();
  storageCheckButton->enable();
  runTypeCheckButton->deSelect();
  runInfoCheckButton->deSelect();
  particleEnergyCheckButton->deSelect();
  storageCheckButton->deSelect();
  cherenkovCheckButton->deSelect();
  csiTempCheckButton->deSelect();
  positionCheckButton->deSelect();
  angleCheckButton->deSelect();
  shutterCheckButton->deSelect();

  runTypeLuminosity->enable();
  runTypeElectron->enable();
  runTypeHadron->enable();
  runTypeCosmic->enable();
  runTypePedestal->enable();
  runTypeOther->enable();
  runInfoEntry->enable();
  particleEnergyEntry->enable();
  cherenkovCheckButton->enable();
  cherenkovEntry->enable();
  csiTempCheckButton->enable();
  csiTempEntry->enable();
  positionCheckButton->enable();
  positionEntry->enable();
  angleCheckButton->enable();
  angleEntry->enable();
  shutterCheckButton->enable();
  leftShutterEntry->enable();
  rightShutterEntry->enable();
  nameEntry1->enable();
  nameEntry2->enable();


  //if(!getenv("NONAMESPACE")){
  //  nsSetter->set("EVENT_NUMBER", "0");
  //}

  state = KEND;
}

void T473TclRunControlPanel::kPause()
{
  if(!getenv("NONAMESPACE")){
//################
    daqControl->control("pause");
  }

  pauseButton->disable();
  beginButton->disable();
  shutdownButton->disable();
  resumeButton->enable();
  endButton->enable();

  state = KPAUSE;
}

void T473TclRunControlPanel::kResume()
{
  if(!getenv("NONAMESPACE")){
    daqControl->control("resume");
  }

  resumeButton->disable();
  endButton->disable();
  beginButton->disable();
  shutdownButton->disable();
  pauseButton->enable();

  state = KRESUME;
}

void T473TclRunControlPanel::kShutdown()
{
  if(!getenv("NONAMESPACE")){
//##############
    daqControl->control("shutdown");
    endButton->after(300);
    histControl->control("shutdown");
    endButton->after(300);
    reconstructControl->control("shutdown");
    endButton->after(300);
    loggerControl->control("shutdown");
    endButton->after(300);
    dataHubControl->control("shutdown");
  }

  beginButton->disable();
  endButton->disable();
  pauseButton->disable();
  resumeButton->disable();
  shutdownButton->disable();

  state = KSHUTDOWN;
}

KChar* T473TclRunControlPanel::getRunInformation()
{
  return runInfoEntry->getText();
}

KChar* T473TclRunControlPanel::getRunNum()
{
  return runInfoEntry->getText();
}

KChar* T473TclRunControlPanel::getRunNumber()
{
  return runNumberDrawLabel->getCaption();
}

KChar* T473TclRunControlPanel::getEventNumber()
{
  return eventNumberDrawLabel->getCaption();
}

KChar* T473TclRunControlPanel::getParticleEnergy()
{
  return particleEnergyEntry->getText();
}

KChar* T473TclRunControlPanel::getBeginTime()
{
  return startTimeDrawLabel->getCaption();
}

KChar* T473TclRunControlPanel::getEndTime()
{
  endTimeDrawLabel->getCaption();
}

KChar* T473TclRunControlPanel::getPressure()
{
  cherenkovEntry->getText();
}

KChar* T473TclRunControlPanel::getTemperature()
{
  csiTempEntry->getText();
}

KChar* T473TclRunControlPanel::getPosition()
{
  positionEntry->getText();
}

KChar* T473TclRunControlPanel::getAngle()
{
  angleEntry->getText();
}

KChar* T473TclRunControlPanel::getLeftShutter()
{
  leftShutterEntry->getText();
}

KChar* T473TclRunControlPanel::getRightShutter()
{
  rightShutterEntry->getText();
}

KChar* T473TclRunControlPanel::getShift1()
{
  nameEntry1->getText();
}

KChar* T473TclRunControlPanel::getShift2()
{
  nameEntry2->getText();
}

T473TclProcessControlPanel* T473TclRunControlPanel::getProcessControlPanel()
{
  return processControlPanel;
}

void T473TclRunControlPanel::viewProcessControlPanel(KInt process)
{
  processControlPanel->view();
  switch( process ){
    case KDaq:
      processControlPanel->setDaq();
      break;
    case KDataHub:
      processControlPanel->setDataHub(); 
      break;
    case KHistogram:
      processControlPanel->setHistogram();
      break;
    case KReconstruct:
      processControlPanel->setReconstruct();
      break;
    case KLogger:
      processControlPanel->setLogger();
      break;
    defaule :
      processControlPanel->setDaq();
  }
}

void T473TclRunControlPanel::timer()
{
  getDate();
  getStorageStatus();
//static KInt i=1;
//i*=2;
//KChar a[24];
//sprintf(a,"%d",i);
//eventNumberDrawLabel->setCaption(a);

  KChar value[256];

  if(!getenv("NONAMESPACE")){
    getDAQInfo();
    KChar* tmpValue = nsExtractor->getKCharValue("EVENT_NUMBER");
    if( tmpValue != (KChar*)-1 ){
      strcpy(value, tmpValue);
      if( strtod(value, (KChar**)NULL) )
        eventNumberDrawLabel->setCaption(value);
    }

  }
  setNameSpaceItem();
}

void T473TclRunControlPanel::keyAction(KonoeTclWindow* kcv, KInt type,
                                                   KeySym keysym, KInt state)
{
  if( type == KKeyRelease ){
    if( keysym == XK_R ){
      beginButton->enable();
      endButton->enable();
      pauseButton->enable();
      resumeButton->enable();
      shutdownButton->enable();

      KChar value[256];
      strcpy(value, nsExtractor->getKCharValue( "RUN_NUMBER" ));
      runInfoEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "PARTICLE_ENERGY" ));
      particleEnergyEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "CHERENKOV_P" ));
      cherenkovEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "CSI_TEMP" ));
      csiTempEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "CSI_POSITION" ));
      positionEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "ANGLE" ));
      angleEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "SHUTTER_LEFT" ));
      leftShutterEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "SHUTTER_RIGHT" ));
      rightShutterEntry->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "PLAYER1" ));
      nameEntry1->setText(value);
      strcpy(value, nsExtractor->getKCharValue( "PLAYER2" ));
      nameEntry2->setText(value);

      strcpy(value, nsExtractor->getKCharValue( "RUN_TYPE" ));
      if(strcmp(value, "Luminosity Run") == 0)
        runTypeLuminosity->setSelect();
      else if(strcmp(value, "Electron Run") == 0)
        runTypeElectron->setSelect();
      else if(strcmp(value, "Hadron Run") == 0)
        runTypeHadron->setSelect();
      else if(strcmp(value, "Cosmic Ray Run") == 0)
        runTypeCosmic->setSelect();
      else if(strcmp(value, "Pedestal Run") == 0)
        runTypePedestal->setSelect();
      else if(strcmp(value, "Other Run") == 0)
        runTypeOther->setSelect();
    }
  }
}

void T473TclRunControlPanel::windowAction(KonoeTclWindow* kfm, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
  }
}
