//=====================================================================
/*

    T473TclRunControlPanel.hh

    Version 0.0.1   11-Nov-2000 T. Hosoi

*/
//====================================================================

#ifndef __T473TCLRUNCONTROLPANEL_HH
#define __T473TCLRUNCONTROLPANEL_HH

#include <ui/KonoeTclWidget.hh>
#include <ui/KonoeTclAction.hh>
#include <Namespace/T473NameSpaceExtractor.hh>
#include <Namespace/T473NameSpaceSetter.hh>

class KonoeTclWindow;
class T473TclProcessControlPanel;
class T473TclRunSummaryPanel;
class T473TclRunControlAction;
class KonoeTclSubWindow;

//class KonoeTclClientSocket;

class T473DAQControl;
class T473DataHubControl;
class T473HistoControl;
class T473ReconstructControl;
class T473LoggerControl;


enum {KDaq, KDataHub, KHistogram, KReconstruct, KLogger};

class T473TclRunControlPanel : public KonoeTclEventProcessor,
                               public KonoeTclTimer, 
                               public KonoeTclKeyAction,
                               public KonoeTclWindowAction {

  public:
    T473TclRunControlPanel(KonoeTclWindow* pwin);
    ~T473TclRunControlPanel();

// Get Process Controller
  public:
    T473DAQControl* getDAQControl() { return daqControl; }
    T473DataHubControl* getDataHubControl() { return dataHubControl; }
    T473HistoControl* getHistControl() { return histControl; }
    T473ReconstructControl* getReconstructControl()
                                     { return reconstructControl; }
    T473LoggerControl* getLoggerControl() { return loggerControl; }

// Current State
  public:
    enum { KBEGIN, KEND, KPAUSE, KRESUME, KSHUTDOWN };
    KInt getCurrentState(){ return state; }
  private:
    KInt state;

// For mouseAction
  public:
    void readyNow();
    void kBegin();
    void kEnd();
    void kPause();
    void kResume();
    void kShutdown();

// For run summary
  public:
    KonoeTclWindow* getWindow(){ return kwin; };
    KChar* getRunType();
    KChar* getRunInformation();
    KChar* getParticleEnergy();
    KChar* getRunNumber();
    KChar* getEventNumber();
    KChar* getBeginTime();
    KChar* getEndTime();
    KChar* getPressure();
    KChar* getTemperature();
    KChar* getPosition();
    KChar* getAngle();
    KChar* getLeftShutter();
    KChar* getRightShutter();
    KChar* getShift1();
    KChar* getShift2();
    T473TclProcessControlPanel* getProcessControlPanel();
    void viewProcessControlPanel(KInt process=0);
    void setImage();
    void setImage2();

  private:
    void initProcessController();
    void timer();
    void keyAction(KonoeTclWindow* kcv, KInt type, KeySym keysym, KInt state);
    void windowAction(KonoeTclWindow* kcv, XEvent* xev);
    void setNameSpaceItem();
    void getDate();
    void getStorageStatus();
    void getDAQInfo();
    KChar* getRunNum();

    T473NameSpaceExtractor* nsExtractor;
    T473NameSpaceSetter* nsSetter;

//    KonoeClientSocket* socket1;

    KChar fontName[256];
    KInt fontSize;
    KChar backColor1[256];
    KChar backColor2[256];
    KChar backColor3[256];
    KChar backColor4[256];
    KChar value[256];
    void initGlobalFrame();
    void initDateFrame();
    void initRunStateFrame();
    void initProcessStateFrame();
    void initLogoFrame();
    void initRunTypeFrame();
    void initRunInfoFrame();
    void initParticleEnergyFrame();
    void initStorageFrame();
    void initCherenkovFrame();
    void initCsITempFrame();
    void initPositionFrame();
    void initAngleFrame();
    void initShutterFrame();
    void initNameRegisterFrame();
    void initControlButtonFrame();
    void initLoggerFrame();

    void initAction();

    KonoeTclWindow* kwin;

    KonoeTclFrame* globalFrame;
    KonoeTclFrame* leftFrame;
    KonoeTclFrame* rightFrame;
    KonoeTclFrame* dateFrame;
    KonoeTclFrame* runStateFrame;
    KonoeTclFrame* rateFrame;
    KonoeTclFrame* processStateFrame;
    KonoeTclFrame* logoFrame;
    KonoeTclFrame* runTypeFrame;
    KonoeTclFrame* runInfoFrame;
    KonoeTclFrame* particleEnergyFrame;
    KonoeTclFrame* storageFrame;
    KonoeTclFrame* cherenkovFrame;
    KonoeTclFrame* csiTempFrame;
    KonoeTclFrame* positionFrame;
    KonoeTclFrame* angleFrame;
    KonoeTclFrame* shutterFrame;
    KonoeTclFrame* nameRegisterFrame;
    KonoeTclFrame* controlButtonFrame;
    KonoeTclFrame* loggerFrame;

// Date frame
    KonoeTclLabel* dateLabel;

// Run state show frame
    KonoeTclLabel* deadTimeLabel;
    KonoeTclLabel* deadTimeSpillLabel;
    KonoeTclLabel* deadTimeTotalLabel;
    KonoeTclLabel* daqSpeedLabel;
    KonoeTclLabel* daqSpeedSpillLabel;
    KonoeTclLabel* daqSpeedTotalLabel;
    KonoeTclLabel* averageLabel;
    KonoeTclLabel* lastSpillLabel;
    KonoeTclLabel* startTimeLabel;
    KonoeTclLabel* startTimeDrawLabel;
    KonoeTclLabel* endTimeLabel;
    KonoeTclLabel* endTimeDrawLabel;
    KonoeTclLabel* runNumberLabel;
    KonoeTclLabel* runNumberDrawLabel;
    KonoeTclLabel* eventNumberLabel;
    KonoeTclLabel* eventNumberDrawLabel;

// Process state show frame
    KonoeTclLabel* daqStateLabel;
    KonoeTclLabel* daqStateDrawLabel;
    KonoeTclLabel* dataHubStateLabel;
    KonoeTclLabel* dataHubStateDrawLabel;
    KonoeTclLabel* hisotgramStateLabel;
    KonoeTclLabel* hisotgramStateDrawLabel;
    KonoeTclLabel* reconstructStateLabel;
    KonoeTclLabel* reconstructStateDrawLabel;
    KonoeTclLabel* loggerStateLabel;
    KonoeTclLabel* loggerStateDrawLabel;

// KONOE Logo
    KonoeTclImage* konoeLogoImage;
    KonoeTclCanvas* konoeLogoCanvas;

// Run Type select list
    KonoeTclCheckButton* runTypeCheckButton;
    KonoeTclRadioButton* runTypeLuminosity;
    KonoeTclRadioButton* runTypeElectron;
    KonoeTclRadioButton* runTypeHadron;
    KonoeTclRadioButton* runTypeCosmic;
    KonoeTclRadioButton* runTypePedestal;
    KonoeTclRadioButton* runTypeOther;

// Run Info
    KonoeTclCheckButton* runInfoCheckButton;
    KonoeTclEntry* runInfoEntry;

// Particle Energy
    KonoeTclCheckButton* particleEnergyCheckButton;
    KonoeTclEntry* particleEnergyEntry;

// Storage frame
    KonoeTclCheckButton* storageCheckButton;
    KonoeTclLabel* storageLabel;
    KonoeTclLabel* storageValueLabel;

// Cherenkov Pressure
  KonoeTclCheckButton* cherenkovCheckButton;
  KonoeTclEntry* cherenkovEntry;

// CsI temperature
   KonoeTclCheckButton* csiTempCheckButton;
   KonoeTclEntry* csiTempEntry;

// Position
   KonoeTclCheckButton* positionCheckButton;
   KonoeTclEntry* positionEntry;

// Angle
   KonoeTclCheckButton* angleCheckButton;
   KonoeTclEntry* angleEntry;

// Shutter
   KonoeTclCheckButton* shutterCheckButton;
   KonoeTclLabel* leftShutterLabel;
   KonoeTclLabel* rightShutterLabel;
   KonoeTclEntry* leftShutterEntry;
   KonoeTclEntry* rightShutterEntry;

// Shift member registry form
    KonoeTclLabel* registerNameLabel;
    KonoeTclEntry* nameEntry1;
    KonoeTclEntry* nameEntry2;

// Run control button
    KonoeTclButton* beginButton;
    KonoeTclButton* endButton;
    KonoeTclButton* pauseButton;
    KonoeTclButton* resumeButton;
    KonoeTclButton* shutdownButton;

// Logger frame
    KonoeTclText* loggerText;

// Run control action
    T473TclRunControlAction* runControlAction;

// Process control frame;
   T473TclProcessControlPanel* processControlPanel;

// Run summary frame
    T473TclRunSummaryPanel* runSummaryPanel;

// Process Controller
  T473DAQControl* daqControl;
  T473DataHubControl* dataHubControl;
  T473HistoControl* histControl;
  T473ReconstructControl* reconstructControl;
  T473LoggerControl* loggerControl;
};

#endif
