//=====================================================================
/*

    T473Boot.cc

    Version 0.0.1   07-Dec-2000 T. Hosoi

*/
//====================================================================

#include <T473Boot.hh>
#include <T473Progress.hh>
#include <T473MsgBox.hh>
#include <T473ConfirmDialog.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>

#include <RunControl/T473DAQControl.hh>
#include <RunControl/T473DataHubControl.hh>
#include <RunControl/T473HistoControl.hh>
#include <RunControl/T473ReconstructControl.hh>
#include <RunControl/T473LoggerControl.hh>

#include <Namespace/T473NameSpaceExtractor.hh>

#include <stdlib.h>

T473Boot::T473Boot( KonoeTclWindow* pwin )
{
  kwin = pwin;

  bar = 0;
  percent = 0;
  msgBox = 0;

  if( getenv("T473BOOT") )
  bootButton = new KonoeTclButton(kwin, "bootButton", "$B5/F0(B");
  stopButton = new KonoeTclButton(kwin, "stopButton", "$B6[5^Dd;_(B");

  stopButton->setBackGround("#ff0000");
  stopButton->setForeGround("#ffffff");
  stopButton->setActiveBack("#ff0000");
  stopButton->setActiveFore("#ffffff");


  KChar* fontName = "gothic";
  KInt   fontSize = 100;

  stopButton->setFont(fontName, fontSize, 0, 0, 0, 0);

  stopButton->fillBoth();

  stopButton->setExpand();

  stopButton->setIpadx(30);
  //bootButton->setIpady(20);
  stopButton->setIpady(20);

  stopButton->setEventProcessor(this);

  addMouseAction(stopButton);
  addTimer(stopButton, 500);

  if( getenv("T473BOOT") ){
    bootButton->setFont(fontName, fontSize, 0, 0, 0, 0);
    bootButton->fillBoth();
    bootButton->setExpand();
    bootButton->setIpadx(30);
    bootButton->setEventProcessor(this);
    addMouseAction(bootButton);
  }
}

T473Boot::~T473Boot()
{ ; }

void T473Boot::execute()
{
  sub = new KonoeTclSubWindow(kwin, "subwin");

  bar = new T473Progress(sub);

//  bootButton->disable();

  KChar mes[5][256];
  KInt bootedProcess = 0;

  if( loggerBoot() ){
    strcpy(mes[0], "Logger is Booted.");
    bootedProcess++;
  }
  else
    strcpy(mes[0], "Logger is NOT Booted.");

  if( dataHubBoot() ){
    strcpy(mes[1], "DataHub is Booted.");
    bootedProcess++;
  }
  else
    strcpy(mes[1], "DataHub is NOT Booted.");

  if( daqBoot() ){
    strcpy(mes[2], "DAQ is Booted.");
    bootedProcess++;
  }
  else
    strcpy(mes[2], "DAQ is NOT Booted.");

  if( histogramBoot() ){
    strcpy(mes[3], "Histogram is Booted.");
    bootedProcess++;
  }
  else
    strcpy(mes[3], "Histogram is NOT Booted.");

  if( reconstructBoot() ){
    strcpy(mes[4], "Reconstruct is Booted.");
    bootedProcess++;
  }
  else
    strcpy(mes[4], "Reconstruct is NOT Booted.");


  KChar message[512];

  sprintf(message, "%d process booted.", bootedProcess);
  bar->setText( message );
  bootButton->after(1000);

  sprintf(message, "%s\n%s\n%s\n%s\n%s", mes[0],mes[1],mes[2],mes[3],mes[4]);
  T473ConfirmDialog dialog(message);

  sub->hide();
  sub->destroy();
  delete bar;
  delete sub;
  percent = 0;
}

void T473Boot::shutdown()
{
  if( !msgBox )
    msgBox = new T473MsgBox(kwin);
  else{
    delete msgBox;
    msgBox = new T473MsgBox(kwin);
    msgBox->show();
  }
}

//==================== Logger Process  ====================
KInt T473Boot::loggerBoot()
{
  T473NameSpaceExtractor nsExtractor;
  cmd_port = nsExtractor.getKIntValue("LOGGER_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"LOGGER_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
    return 0;
  }
  else {
    T473LoggerControl loggerControl(cmd_port);

    bar->setText( "Logger process booting..." );
    bootButton->after(1000);
    value = nsExtractor.getKCharValue("LOGGER_PID");
    if( strcmp(value, "-1") != 0 ){
      percent += 20;
      sprintf(value, "%d %%", percent);
      bar->setValue( percent );
      bar->setProgressText( value );
      return 1;
    }
    return 0;
  }
}

//==================== DataHub Process  ====================
KInt T473Boot::dataHubBoot()
{
  T473NameSpaceExtractor nsExtractor;
  cmd_port = nsExtractor.getKIntValue("DATAHUB_OUTPUT_PORT");

  if( cmd_port == -1){
    cerr<<"DATAHUB_OUTPUT_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
    return 0;
  }
  else {
    T473DataHubControl dataHubControl(cmd_port);

    bar->setText( "Data Hub process booting..." );
    bootButton->after(1000);
    value = nsExtractor.getKCharValue("DATAHUB_PID");
    if( strcmp(value, "-1") != 0 ){
      percent += 20;
      sprintf(value, "%d %%", percent);
      bar->setValue( percent );
      bar->setProgressText( value );
      return 1;
    }
    return 0;
  }
}

//==================== DAQ Process  ====================
KInt T473Boot::daqBoot()
{
  T473NameSpaceExtractor nsExtractor;
  cmd_port = nsExtractor.getKIntValue("DAQ_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"DAQ_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
    return 0;
  }
  else {
    T473DAQControl daqControl(cmd_port);

    bar->setText( "DAQ process booting..." );
    value = nsExtractor.getKCharValue("DAQ_PID");
    bootButton->after(1000);
    if( strcmp(value, "-1") != 0 ){
      percent += 20;
      sprintf(value, "%d %%", percent);
      bar->setValue( percent );
      bar->setProgressText( value );
      return 1;
    }
    return 0;
  }
}

//==================== Histogram Process  ====================
KInt T473Boot::histogramBoot()
{
  T473NameSpaceExtractor nsExtractor;
  cmd_port = nsExtractor.getKIntValue("HISTO_CMD_PORT");
  if( cmd_port == -1){
    cerr<<"HISTO_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
    return 0;
  }
  else {
    T473HistoControl histControl(cmd_port);
    bar->setText( "Histogram process booting..." );
    bootButton->after(1000);
    value = nsExtractor.getKCharValue("HISTOGRAM_PID");
    if( strcmp(value, "-1") != 0 ){
      percent += 20;
      sprintf(value, "%d %%", percent);
      bar->setValue( percent );
      bar->setProgressText( value );
      return 1;
    }
    return 0;
  }
}

//==================== Reconstruct Process  ====================
KInt T473Boot::reconstructBoot()
{
  T473NameSpaceExtractor nsExtractor;
  cmd_port = nsExtractor.getKIntValue("RECONSTRUCT_CMD_PORT");

  if( cmd_port == -1){
    cerr<<"RECONSTRUCT_CMD_PORT is not defined : exit(0)"<<endl;
    cmd_port = -1;
    return 0;
  }
  else {
    T473ReconstructControl reconstructControl(cmd_port);

    bar->setText( "Reconstruct process booting..." );
    bootButton->after(1000);
    value = nsExtractor.getKCharValue("RECONSTRUCT_PID");
    if( strcmp(value, "-1") != 0 ){
      percent += 20;
      sprintf(value, "%d %%", percent);
      bar->setValue( percent );
      bar->setProgressText( value );
      return 1;
    }
    return 0;
  }
}

void T473Boot::mouseAction(KonoeTclButton* kbtn, KInt type, KInt button,
                                                 Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if( strcmp(kbtn->getName(), "bootButton" ) == 0 ) execute();
      if( strcmp(kbtn->getName(), "stopButton" ) == 0 ) shutdown();
    }
  }
}

void T473Boot::timer()
{
  KInt static i = 0;
  if( i ){
    stopButton->setBackGround("#ff0000");
    stopButton->setForeGround("#ffffff");
    stopButton->setActiveBack("#ff0000");
    stopButton->setActiveFore("#ffffff");
    i = 0;
  }
  else{
    stopButton->setBackGround("#ffffff");
    stopButton->setForeGround("#000000");
    stopButton->setActiveBack("#ffffff");
    stopButton->setActiveFore("#000000");
    i = 1;
  }
}
