//=====================================================================
/*

    T473MsgBox.cc

    Version 0.0.1   09-Dec-2000 T. Hosoi

*/
//====================================================================

#include <T473MsgBox.hh>
#include <T473Kill.hh>
#include <T473ConfirmDialog.hh>
#include <ui/KonoeTclSubWindow.hh>

T473MsgBox::T473MsgBox(KonoeTclWindow* pwin)
{
  kill = new T473Kill;

  kwin = pwin;

  strcpy(backColor, "white");

  initWidget();
}

T473MsgBox::~T473MsgBox()
{
  delete daqCheck;
  delete hubCheck;
  delete histCheck;
  delete recCheck;
  delete logCheck;

  delete shutdownButton;

  sub->destroy();
  delete sub;

  delete kill;
}

void T473MsgBox::show()
{
  sub->show();
}

void T473MsgBox::initWidget()
{
  KChar* fontName = "courier";
  KInt fontSize = 24;

  sub = new KonoeTclSubWindow(kwin, "subWindow", "T473 Shutdown Dialog");
  sub->show();
  sub->setBackGround(backColor);

  daqCheck = new KonoeTclCheckButton(sub, "daqCheck",
                                          "DAQ        ", 0,0);
  hubCheck = new KonoeTclCheckButton(sub, "hubCheck", 
                                          "Data Hub   ", 0,1);
  histCheck = new KonoeTclCheckButton(sub, "histCheck",
                                           "Histogram  ", 0,2);
  recCheck = new KonoeTclCheckButton(sub, "recCheck",
                                          "Reconstruct", 0,3);
  logCheck = new KonoeTclCheckButton(sub, "logCheck",
                                          "Logger     ", 0,4);

  shutdownButton = new KonoeTclButton(sub, "shutdownButton", "KILL", 0,5);

  daqCheck->setFont(fontName, fontSize, 1, 0, 0, 0);
  hubCheck->setFont(fontName, fontSize, 1, 0, 0, 0);
  histCheck->setFont(fontName, fontSize, 1, 0, 0, 0);
  recCheck->setFont(fontName, fontSize, 1, 0, 0, 0);
  logCheck->setFont(fontName, fontSize, 1, 0, 0, 0);
  shutdownButton->setFont(fontName, fontSize+8, 1, 0, 0, 0);

  daqCheck->setBackGround(backColor);
  hubCheck->setBackGround(backColor);
  histCheck->setBackGround(backColor);
  recCheck->setBackGround(backColor);
  logCheck->setBackGround(backColor);

  daqCheck->setActiveBack("blue");
  hubCheck->setActiveBack("blue");
  histCheck->setActiveBack("blue");
  recCheck->setActiveBack("blue");
  logCheck->setActiveBack("blue");

  daqCheck->setActiveFore("white");
  hubCheck->setActiveFore("white");
  histCheck->setActiveFore("white");
  recCheck->setActiveFore("white");
  logCheck->setActiveFore("white");

  daqCheck->setGridSticky("w");
  hubCheck->setGridSticky("w");
  histCheck->setGridSticky("w");
  recCheck->setGridSticky("w");
  logCheck->setGridSticky("w");

  shutdownButton->setGridPady(10);
  shutdownButton->setGridIpady(5);
  shutdownButton->setBorderWidth(5);

  daqCheck->setEventProcessor(this);
  hubCheck->setEventProcessor(this);
  histCheck->setEventProcessor(this);
  recCheck->setEventProcessor(this);
  logCheck->setEventProcessor(this);

  addMouseAction(daqCheck);
  addMouseAction(hubCheck);
  addMouseAction(histCheck);
  addMouseAction(recCheck);
  addMouseAction(logCheck);

  daqCheck->setGridPadx(20);
  hubCheck->setGridPadx(20);
  histCheck->setGridPadx(20);
  recCheck->setGridPadx(20);
  logCheck->setGridPadx(20);

  daqCheck->setGridPady(5);
  hubCheck->setGridPady(5);
  histCheck->setGridPady(5);
  recCheck->setGridPady(5);
  logCheck->setGridPady(5);

  daqCheck->setGridIpadx(5);
  hubCheck->setGridIpadx(5);
  histCheck->setGridIpadx(5);
  recCheck->setGridIpadx(5);
  logCheck->setGridIpadx(5);

  shutdownButton->setEventProcessor(this);
  addMouseAction(shutdownButton);
}

void T473MsgBox::mouseAction(KonoeTclButton* kbtn, KInt type, KInt button,
                                                   Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if( strcmp(kbtn->getName(), "shutdownButton" ) == 0 ) shutdown();
    }
  }
}

void T473MsgBox::mouseAction(KonoeTclCheckButton* kbtn, KInt type, KInt button,
                                                   Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if( kbtn->isChecked() ) kbtn->setBackGround("red");
      else kbtn->setBackGround("white");
    }
  }
}

void T473MsgBox::shutdown()
{
  KChar mes[5][256];
  KInt n = 0;

  KInt i;
  KInt j;
  for(i=0; i<256; i++){
    for(j=0; j<5; j++){
      mes[j][i] = '\0';
    }
  }

// DAQ Check
  if( daqCheck->isChecked() ){
    cout << "KILL DAQ" << endl;
    if( kill->killDAQ() )
      strcpy(mes[n], "DAQ is KILLED.");
    else
      strcpy(mes[n], "DAQ is NOT KILLED.");
    n++;
  }

// DataHub Check
  if( hubCheck->isChecked() ){
    cout << "KILL DATAHUB" << endl;
    if( kill->killDataHub() )
      strcpy(mes[n], "DataHub is KILLED.");
    else
      strcpy(mes[n], "DataHub is NOT KILLED.");
    n++;
  }

// Histogram Check
  if( histCheck->isChecked() ){
    cout << "KILL HISTOGRAM" << endl;
    if( kill->killHistogram() )
      strcpy(mes[n], "Histogram is KILLED.");
    else
      strcpy(mes[n], "Histogram is NOT KILLED.");
    n++;
  }

// Reconstruct Check
  if( recCheck->isChecked() ){
    cout << "KILL RECONSTRUCT" << endl;
    if( kill->killReconstruct() )
      strcpy(mes[n], "Reconstruct is KILLED.");
    else
      strcpy(mes[n], "Reconstruct is NOT KILLED.");
    n++;
  }

// Logger Check
  if( logCheck->isChecked() ){
    cout << "KILL LOGGER" << endl;
    if( kill->killLogger() )
      strcpy(mes[n], "Logger is KILLED.");
    else
      strcpy(mes[n], "Logger is NOT KILLED.");
  }

  KChar message[512];
  sprintf(message, "%s\n%s\n%s\n%s\n%s", mes[0],mes[1],mes[2],mes[3],mes[4]);
  T473ConfirmDialog dialog( message );

  sub->hide();
}

void T473MsgBox::timer()
{
}
