//=====================================================================
/*

    T473TclLogger.cc

    Version 0.0.1   14-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclLogger.hh>
#include <Namespace/T473NameSpaceExtractor.hh>

#include <stdlib.h>
#include <fstream.h>

T473TclLogger::T473TclLogger(KonoeTclWindow* pwin)
{
  kwin = pwin;
  strcpy(fontName, "courier");
  fontSize = 12;

  intervalTime = 2000;

  strcpy(successColor, "black");
  strcpy(fatalColor, "hotpink");
  strcpy(warningColor, "indianred");
  strcpy(errorColor, "red");
  strcpy(unknownColor, "black");
  strcpy(infoColor, "blue");

  initLoggerFrame();
  initAction();
}

T473TclLogger::~T473TclLogger()
{ ; }

void T473TclLogger::initLoggerFrame()
{
  loggerFrame = new KonoeTclFrame(kwin, "loggerFrame");
  loggerText = new KonoeTclText(loggerFrame, "loggerText", KVertical);

  loggerText->setBackGround("white");

  loggerFrame->setFrame();
  loggerFrame->fillBoth();
  loggerFrame->setExpand();
  loggerText->fillBoth();
  loggerText->setExpand();
  loggerText->setFont(fontName, fontSize, 0, 0, 0, 0);
}

void T473TclLogger::initAction()
{
  loggerFrame->setEventProcessor(this);
  addTimer(loggerFrame, intervalTime);
}

KInt T473TclLogger::getLogMessage()
{
  KChar fileName[256];
  T473NameSpaceExtractor nsExtractor;
  KChar message[256];

  if( getenv("T473_LOGGER_DIR") ){
    sprintf( fileName, "%s/%s",
                        getenv("T473_LOGGER_DIR"),
                        nsExtractor.getKCharValue("LOGFILE_NAME") );
  }
  else{
    cerr << "T473_LOGGER_DIR is not defined." << endl;
    exit(0);
  }
  ifstream ifile;

  ifile.open( fileName );

  if( !ifile.is_open() ){
    cerr << "File name "<< fileName << " is not found." << endl;
    return 0;
  }


  loggerText->clear();

  while( !ifile.eof() ){
    ifile.getline(message, 256);
    insertMessage( message );
  }

  ifile.close();

  return 0;
}

void T473TclLogger::insertMessage(KChar* Message)
{
  KBool isInsert = KFalse;
//cout << Message << endl;
  if(strstr(Message, "success")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "t5");
  loggerText->setForeGround("t5", successColor);   }

  else if(strstr(Message, "fatal")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "t6");
  loggerText->setForeGround("t6", fatalColor);   }

  else if(strstr(Message, "warning")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "tt");
  loggerText->setForeGround("tt", warningColor);   }

  else if(strstr(Message, "error")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "t2");
  loggerText->setForeGround("t2", errorColor); }

  else if(strstr(Message, "unknown")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "t3");
  loggerText->setForeGround("t3", unknownColor);  }

  else if(strstr(Message, "info")!=NULL){
  isInsert = KTrue;
  loggerText->insert(Message, "t4");
  loggerText->setForeGround("t4", infoColor); }

  if(strlen(Message) > 0 && isInsert){
  loggerText->insert("\n");}
}

void T473TclLogger::timer()
{
static KInt i = 0;
//cout << "No ." << i++ << endl;
  getLogMessage();
  loggerText->scrollYTo(1.0);
}
