//=====================================================================
/*

    T473TclNameSpaceViewer.cc

    Version 0.0.1   19-Nov-2000 T. Hosoi

*/
//====================================================================

#include <T473TclNameSpaceViewer.hh>
#include <T473TclNameSpaceViewerAction.hh>

T473TclNameSpaceViewer::T473TclNameSpaceViewer(KonoeTclWindow* pwin)
{
  strcpy(fontName, "courier");
  fontSize = 14;

  kwin = pwin;
//  kwin->setWindowMaxSize(569, 456);
  nsExtractor = new T473NameSpaceExtractor;

  initWidget();
  initNameSpaceItem();

  insertNameSpaceItem();

}

T473TclNameSpaceViewer::~T473TclNameSpaceViewer()
{
  delete nsExtractor;
}

void T473TclNameSpaceViewer::initWidget()
{
  nameSpaceList = new KonoeTclText(kwin, "nameSpaceList", KVertical);
  nameSpaceList->setBackGround("white");
  nameSpaceList->setFont(fontName, fontSize, 0, 0, 0, 0);
  nameSpaceList->setSize(60, 25);
  nameSpaceList->fillBoth();
  nameSpaceList->setExpand();

  buttonFrame = new KonoeTclFrame(kwin, "nameSpaceSetButtonFrame");

  itemNameLabel = new KonoeTclLabel(buttonFrame, "itemNameLabel", 
                                                      "Item name", 0,0);
  valueLabel = new KonoeTclLabel(buttonFrame, "valueLabel", "Value", 1,0);
  itemNameEntry = new KonoeTclEntry(buttonFrame, "itemNameEntry", 0,1);
  valueEntry = new KonoeTclEntry(buttonFrame, "valueEntry", 1,1);
  setButton = new KonoeTclButton(buttonFrame, "setButton", "Set Value", 0,2);

  nsViewAction = new T473TclNameSpaceViewerAction(this);

  itemNameEntry->setBackGround("white");
  valueEntry->setBackGround("white");
  setButton->setGridColumnspan(2);

  buttonFrame->setFrame();
  //buttonFrame->fillBoth();
  //buttonFrame->setExpand();

  itemNameEntry->setEventProcessor(nsViewAction);
  valueEntry->setEventProcessor(nsViewAction);
  nameSpaceList->setEventProcessor(nsViewAction);
  nsViewAction->addMouseAction(itemNameEntry);
  nsViewAction->addMouseAction(valueEntry);
  nsViewAction->addMouseAction(nameSpaceList);
  nsViewAction->addKeyAction(itemNameEntry);
  nsViewAction->addKeyAction(valueEntry);

  setButton->setEventProcessor(nsViewAction);
  nsViewAction->addMouseAction(setButton);

  buttonFrame->setEventProcessor(this);
  addTimer(buttonFrame, 1000);

}

KChar* T473TclNameSpaceViewer::getItem(KInt i)
{
  if( i > ItemCount) return 0;

  return nsItem[i-1];
}

KChar* T473TclNameSpaceViewer::getValue(KInt i)
{
  if( i > ItemCount) return 0;

  return nsValue[i-1];
}

void T473TclNameSpaceViewer::insertNameSpaceItem()
{
  KInt i;
  KChar nameSpaceItemValue[256];
  KChar tagName[16];
  nameSpaceList->clear();

  for(i=0; i<ItemCount; i++){
    //cout << "##############  " << i << endl;
    strcpy(nsValue[i], nsExtractor->getKCharValue(nsItem[i]));
    sprintf(nameSpaceItemValue, "%028s  :  ", nsItem[i]);
    sprintf(tagName, "line%d", i);
    strcat(nameSpaceItemValue, nsValue[i]);
    nameSpaceList->insert(nameSpaceItemValue, tagName);
    nameSpaceList->insert("\n");
  }
}

void T473TclNameSpaceViewer::initNameSpaceItem()
{
  static KChar* itemName[] = {
    "T473KNSINIT_TIME",
    "RUN_NUMBER",
    "EVENT_NUMBER",
    "RUN_TYPE",
    "PARTICLE_ENERGY",
    "PARTICLE_TYPE",
    "CSI_POSITION",
    "ANGLE",
    "CSI_TEMP",
    "SHUTTER_LEFT",
    "SHUTTER_RIGHT",
    "CHERENKOV_P",
    "PLAYER1",
    "PLAYER2",
//    "RUN_BEGIN_TIME",
//    "RUN_END_TIME",

    "MASTER_HOST",
    "DAQ_HOST",
    "DATAHUB_HOST",
    "HISTOGRAM_HOST",
    "RECONSTRUCT_HOST",
    "LOGGER_HOST",

    "DAQ_PID",
    "DATAHUB_PID",
    "HISTOGRAM_PID",
    "RECONSTRUCT_PID",
    "LOGGER_PID",

    "DAQ_STATUS",
    "DATAHUB_STATUS",
    "HISTOGRAM_STATUS",
    "RECONSTRUCT_STATUS",
    "LOGGER_STATUS",

    "SIGNAL_PORT",
    "DEFAULT_CMD_PORT",
    "EXE_PORT",
    "DAQ_CMD_PORT",
    "DATAHUB_INPUT_PORT",
    "DATAHUB_OUTPUT_PORT",
    "HISTO_CMD_PORT",
    "HISTO_TO_UI_PORT0",
    "HISTO_TO_UI_PORT1",
    "HISTO_TO_UI_PORT2",
    "RECONSTRUCT_CMD_PORT",
    "REC_TO_GUI_PORT",
    "LOGGER_CMD_PORT",
    "LOGGER_PORT",
    "EVENT_NUM_PORT",

    "DAQ_EFFICIENCY",
    "DAQ_EFFICIENCY_SUM",
    "DAQ_SPEED",
    "DAQ_SPEED_SUM",

    "DATAHUB_BUFFER_SIZE",
    "DATAHUB_STORAGE_FILE_NAME",

    "HISTOGRAM_CMD_STATUS",
    "HISTO_REQ_MODE",

    "RECONSTRUCT_RUNINFO",
    "RECONSTRUCT_DATA",

    "LOGGER_CMD_STATUS",
    "LOGFILE_NAME",

    "CSI_TDC_HISTONAME",
    "CSI_ADC_HISTONAME",
    "SCIN_TDC_HISTONAME",
    "SCIN_ADC_HISTONAME",
    "TOF_HISTONAME",
                         "CSI_ADC_0_0_PED",
                         "CSI_ADC_0_1_PED",
                         "CSI_ADC_0_2_PED",
                         "CSI_ADC_0_3_PED",
                         "CSI_ADC_0_4_PED",
                         "CSI_ADC_0_5_PED",
                         "CSI_ADC_0_6_PED",
                         "CSI_ADC_0_7_PED",
                         "CSI_ADC_0_8_PED",
                         "CSI_ADC_0_9_PED",
                         "CSI_ADC_0_10_PED",
                         "CSI_ADC_0_11_PED",
                         "CSI_ADC_0_12_PED",
                         "CSI_ADC_0_13_PED",
                         "CSI_ADC_0_14_PED",
                         "CSI_ADC_0_15_PED",
                         "CSI_ADC_1_0_PED",
                         "CSI_ADC_1_1_PED",
                         "CSI_ADC_1_2_PED",
                         "CSI_ADC_1_3_PED",
                         "CSI_ADC_1_4_PED",
                         "CSI_ADC_1_5_PED",
                         "CSI_ADC_1_6_PED",
                         "CSI_ADC_1_7_PED",
                         "CSI_ADC_1_8_PED",
                         "CSI_TDC_0_0_PED",
                         "CSI_TDC_0_1_PED",
                         "CSI_TDC_0_2_PED",
                         "CSI_TDC_0_3_PED",
                         "CSI_TDC_0_4_PED",
                         "CSI_TDC_0_5_PED",
                         "CSI_TDC_0_6_PED",
                         "CSI_TDC_0_7_PED",
                         "CSI_TDC_1_0_PED",
                         "CSI_TDC_1_1_PED",
                         "CSI_TDC_1_2_PED",
                         "CSI_TDC_1_3_PED",
                         "CSI_TDC_1_4_PED",
                         "CSI_TDC_1_5_PED",
                         "CSI_TDC_1_6_PED",
                         "CSI_TDC_1_7_PED",
                         "CSI_TDC_2_0_PED",
                         "CSI_TDC_2_1_PED",
                         "CSI_TDC_2_2_PED",
                         "CSI_TDC_2_3_PED",
                         "CSI_TDC_2_4_PED",
                         "CSI_TDC_2_5_PED",
                         "CSI_TDC_2_6_PED",
                         "CSI_TDC_2_7_PED",
                         "CSI_TDC_3_0_PED",
                         "SCIN_TDC_0_0_PED",
                         "SCIN_TDC_0_1_PED",
                         "SCIN_TDC_0_3_PED",
						 "AD_X",
						 "AD_Y",
						 "AD_Z",
						 "OA_X",
						 "OA_Y",
						 "OA_Z",
						 "THETA_X",
						 "THETA_Y",
						 "THETA_Z",
						 "CSI_ADC_0_0_GAIN",
						 "CSI_ADC_0_1_GAIN",
						 "CSI_ADC_0_2_GAIN",
						 "CSI_ADC_0_3_GAIN",
						 "CSI_ADC_0_4_GAIN",
						 "CSI_ADC_0_5_GAIN",
						 "CSI_ADC_0_6_GAIN",
						 "CSI_ADC_0_7_GAIN",
						 "CSI_ADC_0_8_GAIN",
						 "CSI_ADC_0_9_GAIN",
						 "CSI_ADC_0_10_GAIN",
						 "CSI_ADC_0_11_GAIN",
						 "CSI_ADC_0_12_GAIN",
						 "CSI_ADC_0_13_GAIN",
						 "CSI_ADC_0_14_GAIN",
						 "CSI_ADC_0_15_GAIN",
						 "CSI_ADC_1_0_GAIN",
						 "CSI_ADC_1_1_GAIN",
						 "CSI_ADC_1_2_GAIN",
						 "CSI_ADC_1_3_GAIN",
						 "CSI_ADC_1_4_GAIN",
						 "CSI_ADC_1_5_GAIN",
						 "CSI_ADC_1_6_GAIN",
						 "CSI_ADC_1_7_GAIN",
						 "CSI_ADC_1_8_GAIN",
						 "CSI_TDC_0_0_GAIN",
						 "CSI_TDC_0_1_GAIN",
						 "CSI_TDC_0_2_GAIN",
						 "CSI_TDC_0_3_GAIN",
						 "CSI_TDC_0_4_GAIN",
						 "CSI_TDC_0_5_GAIN",
						 "CSI_TDC_0_6_GAIN",
						 "CSI_TDC_0_7_GAIN",
						 "CSI_TDC_1_1_GAIN",
						 "CSI_TDC_1_2_GAIN",
						 "CSI_TDC_1_3_GAIN",
						 "CSI_TDC_1_4_GAIN",
						 "CSI_TDC_1_5_GAIN",
						 "CSI_TDC_1_6_GAIN",
						 "CSI_TDC_1_7_GAIN",
						 "CSI_TDC_2_0_GAIN",
						 "CSI_TDC_2_1_GAIN",
						 "CSI_TDC_2_2_GAIN",
						 "CSI_TDC_2_3_GAIN",
						 "CSI_TDC_2_4_GAIN",
						 "CSI_TDC_2_5_GAIN",
						 "CSI_TDC_2_6_GAIN",
						 "CSI_TDC_2_7_GAIN",
						 "CSI_TDC_3_0_GAIN"};
  nsItem = itemName;
}

void T473TclNameSpaceViewer::timer()
{
static KInt i = 0;
//cout << "No. " << i++ << endl;
  KFloat v;
  v = nameSpaceList->getCurrentYPosition();
  insertNameSpaceItem();
  nameSpaceList->scrollYTo( v );
//cout << nameSpaceList->search("PLAYER1") << endl;
}
