//===================================================================
/*

	Konoe2DHistogramOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Enomoto and S. Nishida
	Version 0.4.0	 7-Sep-1998	S. Nishida
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/Konoe2DHistogramOrderly.hh>
#include	<analysis/KonoeLinearDataScaleOrderly.hh>
#include	<analysis/KonoeFillDataOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeFillData.hh>

const KChar	* Konoe2DHistogramOrderly::classname = "2DHistogram";

Konoe2DHistogramOrderly::Konoe2DHistogramOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "xscale", STRING, NECESSARY ) );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "yscale", STRING, NECESSARY ) );
	addCommand( new KonoeCreateCommand( cmd_descripter, this ) );

	addCommand( new KonoeShowCommand( 0 , this ) );

	cmd_descripter = new KonoeCommandDescriptor( "clear", N_ARG_CLEAR );
	addCommand( new ClearUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "fill", N_ARG_FILL );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "filldata", STRING, NECESSARY ) );
	addCommand( new FillUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	Konoe2DHistogramOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

Konoe2DHistogramOrderly::ClearUnit::ClearUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	Konoe2DHistogramOrderly::ClearUnit::execute( KonoeCommand * cmd ) {

	Konoe2DHistogram	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( Konoe2DHistogram * )
		( ( Konoe2DHistogramOrderly * ) owner ) -> getTarget( );
	target -> clear( );

	return 0;
}

Konoe2DHistogramOrderly::FillUnit::FillUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	Konoe2DHistogramOrderly::FillUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	Konoe2DHistogram	* target;

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_FILL_OBJECT );
        const KChar *objectname = arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );

	KonoeFillDataOrderly * filldataorderly
	  = (KonoeFillDataOrderly *) commander -> findObject(objectname);

	if( filldataorderly == 0 ){
	  os << "FillData object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}

	KonoeFillData *fill_data
		= (KonoeFillData * ) filldataorderly -> getTarget( );

	target = ( Konoe2DHistogram * )
		( ( Konoe2DHistogramOrderly * ) owner ) -> getTarget( );
	target -> fill( fill_data );

	return 0;
}

KInt	Konoe2DHistogramOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeCommandArgument	* arg ;
	KonoeCommander *commander = getOfficer( ) -> getCommander( );
	KonoeLinearDataScaleOrderly * scaleorderly;

	arg = cmd -> getArgument( ARG_CREATE_XSCALE );
	const KChar *objectname_x = arg -> getValue( );
	scaleorderly = (KonoeLinearDataScaleOrderly *) commander 
	  	-> findObject(objectname_x);
	if( scaleorderly == 0 ){
	  os << "Scale object " << objectname_x << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeLinearDataScale *xscale 
	  	= (KonoeLinearDataScale * ) scaleorderly -> getTarget( );

	arg = cmd -> getArgument( ARG_CREATE_YSCALE );
	const KChar *objectname_y = arg -> getValue( );
	scaleorderly = (KonoeLinearDataScaleOrderly *) commander 
	  	-> findObject(objectname_y);
	if( scaleorderly == 0 ){
	  os << "Scale object " << objectname_y << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeLinearDataScale *yscale
	  	= (KonoeLinearDataScale * ) scaleorderly -> getTarget( );

	target = new Konoe2DHistogram( xscale , yscale );

	return 0;

}

KInt	Konoe2DHistogramOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* Konoe2DHistogramOrderly::clone( const KChar * name ) {

	return new Konoe2DHistogramOrderly( name );

}

KInt    Konoe2DHistogramOrderly::showTarget( KonoeCommand * cmd ){

	ostream & os = cmd -> getOutputStream( );
	target -> dump( os );
	return 0;
}

