//===================================================================
/*

	KonoeBoxGeometryOrderly.cc

	Version 0.0.1	 7-Jun-2001	T. Matuo
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeBoxGeometryOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<ui/KonoeCreateCommand.hh>

const KChar	* KonoeBoxGeometryOrderly::classname = "BoxGeometry";


KonoeBoxGeometryOrderly::KonoeBoxGeometryOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;
	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "width", FLOATING, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "height", FLOATING, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "depth", FLOATING, NECESSARY ) );
        addCommand( new KonoeCreateCommand( cmd_descripter, this ) );

	addCommand( new KonoeShowCommand( 0 , this ) );

	cmd_descripter = new KonoeCommandDescriptor( "translate", N_ARG_TRANSLATE );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "translation", STRING, NECESSARY ) );
	addCommand( new TranslateUnit( cmd_descripter, this ) );
	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeBoxGeometryOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;
}



KInt	KonoeBoxGeometryOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

        KInt n_arg = cmd -> getArgumentCount( );
        if( n_arg == N_ARG_CREATE ){
        	KFloat width = cmd -> getArgument( ARG_CREATE_WIDTH ) -> toFloating( );
        	KFloat height = cmd -> getArgument( ARG_CREATE_HEIGHT ) -> toFloating( );
        	KFloat depth = cmd -> getArgument( ARG_CREATE_DEPTH ) -> toFloating( );
        	target = new KonoeBoxGeometry( width , height , depth );
	}
	else {
		target = new KonoeBoxGeometry( );
	}

	return 0;

}

KInt	KonoeBoxGeometryOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KInt    KonoeBoxGeometryOrderly::showTarget( KonoeCommand * cmd ){

	ostream & os = cmd -> getOutputStream( );
	target -> dump( os );
	return 0;
}


KonoeOrderly	* KonoeBoxGeometryOrderly::clone( const KChar * name ) {

	return new KonoeBoxGeometryOrderly( name );

}

KonoeBoxGeometryOrderly::TranslateUnit::TranslateUnit(  KonoeCommandDescriptor * cd, 
KonoeOrderly * owner ): KonoeCommandProcessor( cd, owner ) {
}

KInt KonoeBoxGeometryOrderly::TranslateUnit::execute(KonoeCommand * cmd ) {
	ostream& os = cmd -> getOutputStream();

	KonoeCommandArgument* arg = cmd -> getArgument(ARG_TRANSLATE_TRANSLATION);
	const KChar * objectname = arg -> getValue();
	KonoeCommander * commander = owner -> getOfficer() -> getCommander();
	KonoeOrderly * orderly = commander -> findObject(objectname);

	if( orderly == 0 ){
		os << "Trajectory object " << objectname << "doesn't exist "
		   << endl;
		return 0;
	}
	
	KonoeTranslation * trajectory =
		(KonoeTranslation * ) orderly -> getTarget();

	KonoeBoxGeometry * target =(KonoeBoxGeometry * )owner -> getTarget();

	target -> translate( trajectory );
	return 0;
}

