//===================================================================
/*

	KonoeDataAnalyzerOrderly.cc

	Version 0.0.1	 4-Aug-1998	S. Nishida
	Version 0.4.0	 5-Sep-1998	S. Nishida
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDataAnalyzerOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<analysis/KonoeSectionAnalyzer.hh>
#include	<analysis/KonoeDataRecord.hh>

const KChar	* KonoeDataAnalyzerOrderly::classname = "DataAnalyzer";


KonoeDataAnalyzerOrderly::KonoeDataAnalyzerOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "maxsection", INTEGER, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "install", N_ARG_INSTALL );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "secana", STRING, NECESSARY ) );
        addCommand( new InstallUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "analyze", N_ARG_ANALYZE );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "record", STRING, NECESSARY ) );
        addCommand( new AnalyzeUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );
}

KInt	KonoeDataAnalyzerOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeDataAnalyzerOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeDataAnalyzerOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

KonoeDataAnalyzerOrderly::InstallUnit::InstallUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeDataAnalyzerOrderly::InstallUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeDataAnalyzer	* target;

	if( ! checkArgument( cmd ) ) return 0;

        const KChar *objectname = 
	  cmd -> getArgument( ARG_INSTALL_SECANA )-> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly   =  commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "Section Analyzer object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}

	KonoeSectionAnalyzer * secana
		= (KonoeSectionAnalyzer * ) orderly -> getTarget( );

	target = ( KonoeDataAnalyzer * ) owner  -> getTarget( );
	target -> install( secana );

	return 0;
}

KonoeDataAnalyzerOrderly::AnalyzeUnit::AnalyzeUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeDataAnalyzerOrderly::AnalyzeUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeDataAnalyzer	* target;

	if( ! checkArgument( cmd ) ) return 0;

        const KChar *objectname 
	  = cmd -> getArgument( ARG_ANALYZE_RECORD ) -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly   =  commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "Data Record object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}

	KonoeDataRecord *record
		= (KonoeDataRecord * ) orderly -> getTarget( );

	target = ( KonoeDataAnalyzer * ) owner  -> getTarget( );
	target -> analyze( record );

	return 0;
}

KInt	KonoeDataAnalyzerOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	target = new KonoeDataAnalyzer(
	  cmd -> getArgument( ARG_CREATE_MAXSECTION ) -> toInteger( ) );

	return 0;

}

KInt	KonoeDataAnalyzerOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeDataAnalyzerOrderly::clone( const KChar * name ) {

	return new KonoeDataAnalyzerOrderly( name );

}














