//===================================================================
/*

	KonoeDataRecordOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Enomoto and S. Nishida
	Version 0.4.0	 7-Sep-1998	S. Nishida
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDataRecordOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeFillData.hh>

const KChar	* KonoeDataRecordOrderly::classname = "DataRecord";


KonoeDataRecordOrderly::KonoeDataRecordOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "tablesize", INTEGER, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "type", INTEGER, OPTIONAL ) );
	addCommand( new KonoeCreateCommand( cmd_descripter, this ) );

	addCommand( new KonoeShowCommand( 0, this ) );
	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeDataRecordOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeDataRecordOrderly::DeleteUnit::DeleteUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeDataRecordOrderly::DeleteUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;
	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;
	
	KonoeOrderly * o = (KonoeOrderly *) owner -> getTarget( );
	o -> deleteObject( cmd );

	return 0;
}

KInt	KonoeDataRecordOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt n_arg = cmd -> getArgumentCount( );

	if( n_arg == N_ARG_CREATE ){
	  KInt tablesize 
	    = cmd -> getArgument( ARG_CREATE_TABLESIZE ) -> toInteger( );
	  KInt type 
	    = cmd -> getArgument( ARG_CREATE_TYPE ) -> toInteger( );
	  target = new KonoeDataRecord( tablesize , type  );
	}
	else  target = new KonoeDataRecord( );

	return 0;
}

KInt	KonoeDataRecordOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeDataRecordOrderly::clone( const KChar * name ) {

	return new KonoeDataRecordOrderly( name );

}

KInt    KonoeDataRecordOrderly::showTarget( KonoeCommand * cmd ){

	ostream & os = cmd -> getOutputStream( );
	target -> dump( os );
	return 0;
}




#if 0
	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OS );
        const KChar *objectname = arg -> getValue( );

	KonoeCommander *commander = getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly = commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "OutputObjStream object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeOutputObjectStream *stream
		= (KonoeOutputObjectStream * ) orderly -> getTarget( );

        KChar level 
	  = cmd -> getArgument( ARG_CREATE_LEVEL ) -> toInteger( );
        KInt memberid 
	  = cmd -> getArgument( ARG_CREATE_MEMBERID ) -> toInteger( );

	target = new KonoeDataRecord( stream ,level , memberid  );

	return 0;
#endif










