//===================================================================
/*

	KonoeDefaultSectionAnalyzerOrderly.cc

	Version 0.0.1	 4-Aug-1998	S. Nishida
	Version 0.4.0	 5-Sep-1998	S. Nishida
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDefaultSectionAnalyzerOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<analysis/KonoeDataSection.hh>
#include	<analysis/KonoePhysicalValue.hh>

const KChar	* KonoeDefaultSectionAnalyzerOrderly::classname = "DefaultSectionAnalyzer";


KonoeDefaultSectionAnalyzerOrderly::KonoeDefaultSectionAnalyzerOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "section", INTEGER, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "tablesize", INTEGER, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "tag", INTEGER, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "clear", N_ARG_CLEAR );
	addCommand( new ClearUnit( cmd_descripter, this ) );
#if 0
	cmd_descripter = new KonoeCommandDescriptor( "analize", N_ARG_FILL );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "section", STRING, NECESSARY ) );
        addCommand( new AnalyzerUnit( cmd_descripter, this ) );
#endif
	cmd_descripter = new KonoeCommandDescriptor(
		"getPhysicalValue", N_ARG_GETPHYSICALVALUE );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "value", STRING, NECESSARY ) );
        addCommand( new GetPhysicalValueUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeDefaultSectionAnalyzerOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeDefaultSectionAnalyzerOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeDefaultSectionAnalyzerOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

KonoeDefaultSectionAnalyzerOrderly::ClearUnit::ClearUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeDefaultSectionAnalyzerOrderly::ClearUnit::execute( KonoeCommand * cmd ) {

	KonoeDefaultSectionAnalyzer	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( KonoeDefaultSectionAnalyzer * ) owner -> getTarget( );
	target -> clear( );

	return 0;
}

KonoeDefaultSectionAnalyzerOrderly::GetPhysicalValueUnit::
		GetPhysicalValueUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeDefaultSectionAnalyzerOrderly::GetPhysicalValueUnit::
		execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeDefaultSectionAnalyzer	* target;

	if( ! checkArgument( cmd ) ) return 0;

        const KChar *objectname 
	  = cmd -> getArgument( ARG_GETPHYSICALVALUE_VALUE ) -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly   =  commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "Physical Value object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}

	KonoePhysicalValue * value
		= (KonoePhysicalValue * ) orderly -> getTarget( );

	target = ( KonoeDefaultSectionAnalyzer * ) owner -> getTarget( );
	value = target -> getPhysicalValue( );

	return 0;
}


KInt	KonoeDefaultSectionAnalyzerOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

        KInt section 
	  = cmd -> getArgument( ARG_CREATE_SECTION ) -> toInteger( );
        KInt tablesize 
	  = cmd -> getArgument( ARG_CREATE_TABLESIZE ) -> toInteger( );
        KInt tag 
	  = cmd -> getArgument( ARG_CREATE_TAG ) -> toInteger( );

	target = new KonoeDefaultSectionAnalyzer(
  			section, tablesize, tag);

	return 0;

}

KInt	KonoeDefaultSectionAnalyzerOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeDefaultSectionAnalyzerOrderly::clone(
	const KChar * name ) {

	return new KonoeDefaultSectionAnalyzerOrderly( name );

}
