//===================================================================
/*

	KonoeDetectorElementOrderly.cc

	Version 0.0.1	 7-Jun-2001	T. Matuo
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDetectorElementOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<ui/KonoeCreateCommand.hh>

const KChar	* KonoeDetectorElementOrderly::classname = "DetectorElement";


KonoeDetectorElementOrderly::KonoeDetectorElementOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;


	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE2 );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "first", STRING, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "second", STRING, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "third", INTEGER, OPTIONAL ) );
	addCommand(new KonoeCreateCommand( cmd_descripter, this ) );

        /***
	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "classid", INTEGER, OPTIONAL ) );
        addCommand( new KonoeCreateCommand( cmd_descripter, this ) );
        ***/

	addCommand( new KonoeShowCommand( 0 , this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "interact", N_ARG_INTERACT );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "segment", INTEGER, NECESSARY ) );
	addCommand( new InteractUnit( cmd_descripter, this ) );
}

KInt	KonoeDetectorElementOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;
}



KInt	KonoeDetectorElementOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

        KInt n_arg = cmd -> getArgumentCount( );
	cout << "n_arg(DetectorOrderly)"<< n_arg << endl;
	if( n_arg == N_ARG_CREATE2 ){ 
		KInt identifier = cmd -> getArgument( ARG_CREATE2_IDENTIFIER )->toInteger();

                KonoeCommandArgument    * arg = cmd -> getArgument( ARG_CREATE2_GEOMETRY );
        	const KChar *objectname = arg -> getValue( );

        	KonoeCommander * commander = getOfficer( ) -> getCommander( );
        	KonoeOrderly * orderly = commander -> findObject(objectname);

        	if( orderly == 0 ){
          		os << "Geometry object " << objectname << " doesn't exist."
             		<< endl;
          		return 0;
        	}
        	KonoeGeometry * geometry
                	= (KonoeGeometry * ) orderly -> getTarget( );

                arg = cmd -> getArgument( ARG_CREATE2_DETECTOR );
        	objectname = arg -> getValue( );

        	commander = getOfficer( ) -> getCommander( );
        	orderly = commander -> findObject(objectname);

        	if( orderly == 0 ){
          		os << "Detector object " << objectname << " doesn't exist."
             		<< endl;
          		return 0;
        	}
        	KonoeDetector * detector
                	= (KonoeDetector * ) orderly -> getTarget( );

		target = new KonoeDetectorElement( geometry, detector, identifier );

	}
	else if( n_arg == N_ARG_CREATE ){ 
		KInt classid = cmd -> getArgument( ARG_CREATE_CLASSID )-> toInteger();
		target = new KonoeDetectorElement( classid );
	}
	else if( n_arg == N_ARG_CREATE-1 ){
		target = new KonoeDetectorElement( );
	}
	
	return 0;
}

KInt	KonoeDetectorElementOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KInt    KonoeDetectorElementOrderly::showTarget( KonoeCommand * cmd ){

	ostream & os = cmd -> getOutputStream( );
	target -> dump( os );
	return 0;
}


KonoeOrderly	* KonoeDetectorElementOrderly::clone( const KChar * name ) {

	return new KonoeDetectorElementOrderly( name );

}

KonoeDetectorElementOrderly::InteractUnit::InteractUnit(KonoeCommandDescriptor * cd,
		KonoeOrderly *ow ) : KonoeCommandProcessor(cd, ow ){
}

KInt KonoeDetectorElementOrderly::InteractUnit::execute( KonoeCommand * cmd ){
	ostream & os = cmd -> getOutputStream();
	KonoeDetectorElement * target;

	if( ! checkArgument( cmd ) ) return 0;
	
	KonoeCommandArgument    * arg = cmd -> getArgument( ARG_INTERACT_SEGMENT );
        const KChar *objectname = arg -> getValue( );

        KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );

        KonoeOrderly * orderly   =  commander -> findObject(objectname);

        if( orderly == 0 ){
          os << "Tracksegment  object " << objectname << " doesn't exist."
             << endl;
          return 0;
        }
	
	KonoeTrackSegment * segment
			= (KonoeTrackSegment * )  orderly -> getTarget();

	target = ( KonoeDetectorElement * ) owner-> getTarget();
	target -> interact( segment );
	
	return 0;
}

