//===================================================================
/*

	KonoeDetectorOrderly.cc

	Version 0.0.1	 7-Jun-2001	T. Matuo
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDetectorOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<ui/KonoeCreateCommand.hh>

const KChar	* KonoeDetectorOrderly::classname = "Detector";


KonoeDetectorOrderly::KonoeDetectorOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "first", INTEGER, NECESSARY ) );
        cmd_descripter -> add(
                new KonoeArgumentDescriptor( "second", INTEGER, OPTIONAL ) );
        addCommand( new KonoeCreateCommand( cmd_descripter, this ) );

	addCommand( new KonoeShowCommand( 0 , this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "find", N_ARG_FIND );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "section", INTEGER, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "segment", INTEGER, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "subaddress", INTEGER, NECESSARY ) );
	addCommand( new FindUnit( cmd_descripter, this ) );
}

KInt	KonoeDetectorOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;
}



KInt	KonoeDetectorOrderly::createObject( KonoeCommand * cmd ) {
	ostream & os = cmd -> getOutputStream( );

        KInt n_arg = cmd -> getArgumentCount( );
	//cout << "n_arg(DetectorOrderly)"<< n_arg << endl;
	if( n_arg == 2 ){ // call KonoeDetector(KInt classid)
		KInt classid = cmd -> getArgument( ARG_CREATE_FIRST ) -> toInteger();
		target = new KonoeDetector( classid );
	}
	else if( n_arg == 3 ){ // call KonoeDetector(KInt ptablesizer, KInt pidentifier)
		KInt ptablesizer = cmd -> getArgument( ARG_CREATE_FIRST )-> toInteger();
		KInt pidentifier = cmd -> getArgument( ARG_CREATE_SECOND )-> toInteger();
		target = new KonoeDetector( ptablesizer, pidentifier );
	}
	return 0;
}

KInt	KonoeDetectorOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KInt    KonoeDetectorOrderly::showTarget( KonoeCommand * cmd ){

	ostream & os = cmd -> getOutputStream( );
	target -> dump( os );
	return 0;
}


KonoeOrderly	* KonoeDetectorOrderly::clone( const KChar * name ) {

	return new KonoeDetectorOrderly( name );

}

KonoeDetectorOrderly::FindUnit::FindUnit(KonoeCommandDescriptor * cd,
		KonoeOrderly *ow ) : KonoeCommandProcessor(cd, ow ){
}

KInt KonoeDetectorOrderly::FindUnit::execute( KonoeCommand * cmd ){
	ostream & os = cmd -> getOutputStream();
	KonoeDetector * target;

	if( ! checkArgument( cmd ) ) return 0;
	KInt n_arg = cmd-> getArgumentCount();
	//cout << " Count is " << n_arg <<endl;
	KInt section = cmd -> getArgument( ARG_FIND_SECTION ) -> toInteger();
	//cout << " section " << section << endl;
	KInt segment = cmd -> getArgument( ARG_FIND_SEGMENT ) -> toInteger();
	//cout << " segment " << segment << endl;
	KInt subaddress = cmd -> getArgument( ARG_FIND_SUBADDRESS ) -> toInteger();
	
	target = ( KonoeDetector * ) owner-> getTarget();
	target -> find( section, segment, subaddress );
	return 0;
}

