//===================================================================
/*

	KonoeFillDataExtractorOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 5-Sep-1998	S. Nishida
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeFillDataExtractorOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeShowCommand.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeFillData.hh>
#include	<string.h>

const KChar	* KonoeFillDataExtractorOrderly::classname = "FillDataExtractor";


KonoeFillDataExtractorOrderly::KonoeFillDataExtractorOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "value", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "name1", STRING, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "name2", STRING, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "name3", STRING, OPTIONAL ) );
	addCommand(  new KonoeCreateCommand( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "index1", INTEGER, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "index2", INTEGER, OPTIONAL ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "index3", INTEGER, OPTIONAL ) );
	addCommand(  new SetIndexUnit( cmd_descripter, this ) );

	addCommand( new KonoeShowCommand( 0, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "reset", N_ARG_RESET );
	addCommand( new ResetUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeFillDataExtractorOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}


KonoeFillDataExtractorOrderly::SetIndexUnit::SetIndexUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeFillDataExtractorOrderly::SetIndexUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KInt n_arg =  cmd -> getArgumentCount( );
	KonoeFillDataExtractor	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( KonoeFillDataExtractor * ) owner -> getTarget( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_SETINDEX_INDEX1 );
	KInt index1 = arg -> toInteger(   );
	if( n_arg == ARG_SETINDEX_INDEX1 + 1 ){
	  target -> setIndex( index1 );
	  return 0;
	}

	arg = cmd -> getArgument( ARG_SETINDEX_INDEX2 );
	KInt index2 = arg -> toInteger( );
	if( n_arg == ARG_SETINDEX_INDEX2 + 1 ){
	  target -> setIndex( index1 , index2);
	  return 0;
	}
	
	arg = cmd -> getArgument( ARG_SETINDEX_INDEX3 );
	KInt index3 = arg -> toInteger( );
	target -> setIndex( index1 , index2 , index3 );
	return 0;
	
}

KonoeFillDataExtractorOrderly::ResetUnit::ResetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeFillDataExtractorOrderly::ResetUnit::execute( KonoeCommand * cmd ) {

	KonoeFillDataExtractor	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( KonoeFillDataExtractor * ) owner -> getTarget( );
	target -> reset( );

	return 0;
}

KInt	KonoeFillDataExtractorOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KInt n_arg =  cmd -> getArgumentCount( );
 
	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_VALUE );
        const KChar *objectname = arg -> getValue( );

	KonoeCommander *commander = getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly = commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "PhysicalValue object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoePhysicalValue * value
		= (KonoePhysicalValue * ) orderly -> getTarget( );

	if( n_arg == ARG_CREATE_VALUE + 1 ){
	  target == new KonoeFillDataExtractor( value );
	  return 0;
	}
	 
	arg = cmd -> getArgument( ARG_CREATE_NAME1 );
 	const KChar * name1 = arg -> getValue( );
	KChar cs1[64]; strcpy(cs1,name1);
	if( n_arg == ARG_CREATE_NAME1 + 1 ){
	  target == new KonoeFillDataExtractor( value , cs1 );
	  return 0;
	}

	arg = cmd -> getArgument( ARG_CREATE_NAME2 );
	const KChar * name2 = arg -> getValue( );
	KChar cs2[64]; strcpy(cs2,name2);

	if( n_arg == ARG_CREATE_NAME2 + 1 ){
	  target == new KonoeFillDataExtractor( value , cs1 , cs2 );
	  return 0;
	}

	arg = cmd -> getArgument( ARG_CREATE_NAME3 );
	const KChar * name3 = arg -> getValue( );
	KChar cs3[64]; strcpy(cs3,name3);
	target == new KonoeFillDataExtractor( value , cs1 ,cs2 , cs3 );

	return 0;

}

KInt	KonoeFillDataExtractorOrderly::deleteObject( KonoeCommand * ) {

	delete target;
	return 0;

}

KonoeOrderly	* KonoeFillDataExtractorOrderly::clone( const KChar * name ) {

	return new KonoeFillDataExtractorOrderly( name );

}

KInt	KonoeFillDataExtractorOrderly::showTarget( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	os << "Default FillDataExtractor.show" << endl;
	return 0;
}

