//===================================================================
/*

	KonoeHistogramFillerOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 5-Sep-1998
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeHistogramFillerOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<analysis/KonoeFillDataExtractor.hh>
#include	<analysis/KonoeHistogramBase.hh>


const KChar	* KonoeHistogramFillerOrderly::classname = "HistogramFiller";


KonoeHistogramFillerOrderly::KonoeHistogramFillerOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "extractor", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "histogram", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "fill", N_ARG_FILL );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "filldata", STRING, NECESSARY ) );
        addCommand( new FillUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeHistogramFillerOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeHistogramFillerOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeHistogramFillerOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

KonoeHistogramFillerOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeHistogramFillerOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeHistogramFiller	* target;

	if( ! checkArgument( cmd ) ) return 0;

	os << "HistogramFiller" << endl;

	//target = ( KonoeHistogramFiller * ) owner -> getTarget( );
	//target -> dump( os );

	return 0;
}


KonoeHistogramFillerOrderly::FillUnit::FillUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeHistogramFillerOrderly::FillUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeHistogramFiller	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( KonoeHistogramFiller * ) owner -> getTarget( );
	target -> fill( );

	return 0;
}

KInt	KonoeHistogramFillerOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_EXTRACTOR );
        const KChar *objectname = arg -> getValue( );

	KonoeCommander * commander = getOfficer( ) -> getCommander( );
	KonoeOrderly * orderly = commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "Extractor object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeFillDataExtractor * extractor
		= (KonoeFillDataExtractor * ) orderly -> getTarget( );

	arg = cmd -> getArgument( ARG_CREATE_HISTOGRAM );
        objectname = arg -> getValue( );

	commander = getOfficer( ) -> getCommander( );
	orderly = commander -> findObject(objectname);

	if( orderly == 0 ){
	  os << "Histogram object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeHistogramBase * histogram
		= (KonoeHistogramBase * ) orderly -> getTarget( );

	target = new KonoeHistogramFiller( extractor , histogram );

	return 0;

}

KInt	KonoeHistogramFillerOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeHistogramFillerOrderly::clone( const KChar * name ) {

	return new KonoeHistogramFillerOrderly( name );

}
