//===================================================================
/*

	KonoeLinearDataScaleOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 5-Sep-1998
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeLinearDataScaleOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<string.h>
#include	<ui/KonoeHelpCommand.hh>

const KChar	* KonoeLinearDataScaleOrderly::classname = "LinearDataScale";


KonoeLinearDataScaleOrderly::KonoeLinearDataScaleOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	createcommand = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	createcommand -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor( "minimum", FLOATING, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor( "maxium", FLOATING, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor( "bincount", INTEGER, NECESSARY ) );
	createunit = new CreateUnit( createcommand, this );
	addCommand( createunit );

	showcommand = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	showunit = new ShowUnit( showcommand, this );
	addCommand( showunit );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeLinearDataScaleOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeLinearDataScaleOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeLinearDataScaleOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;
	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0; 

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> createObject( cmd );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	return 0;
}

KonoeLinearDataScaleOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeLinearDataScaleOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeLinearDataScale	* target;

	if( ! checkArgument( cmd ) ) return 0; 

	target = ( KonoeLinearDataScale * )
		( ( KonoeLinearDataScaleOrderly * ) owner ) -> getTarget( );
	target -> dump( os );

	return 0;
}

KInt	KonoeLinearDataScaleOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	
	KonoeCommandArgument * arg = cmd -> getArgument( ARG_CREATE_MINIMUM );
	KFloat minimum = arg -> toFloating( );

	arg = cmd -> getArgument( ARG_CREATE_MAXIMUM );
	KFloat maximum = arg -> toFloating( );

	arg = cmd -> getArgument( ARG_CREATE_BINCOUNT );
	KInt	bincount = arg -> toInteger( );

	target = new KonoeLinearDataScale( minimum, maximum, bincount );

	return 0;

}

KInt	KonoeLinearDataScaleOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeLinearDataScaleOrderly::clone( const KChar * name ) {

	return new KonoeLinearDataScaleOrderly( name );

}







