//===================================================================
/*

	Konoe1DHistogramOrderly.cc

	Version 0.0.1	 1-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 5-Sep-1998
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/Konoe1DHistogramOrderly.hh>
#include	<analysis/KonoeLinearDataScaleOrderly.hh>
#include	<analysis/KonoeFillDataOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoeFillData.hh>

const KChar	* Konoe1DHistogramOrderly::classname = "1DHistogram";


Konoe1DHistogramOrderly::Konoe1DHistogramOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "scale", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "clear", N_ARG_CLEAR );
	addCommand( new ClearUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "fill", N_ARG_FILL );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "filldata", STRING, NECESSARY ) );
        addCommand( new FillUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	Konoe1DHistogramOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

Konoe1DHistogramOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	Konoe1DHistogramOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );     // 98/8/1
	return 0;
}

Konoe1DHistogramOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	Konoe1DHistogramOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	Konoe1DHistogram	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( Konoe1DHistogram * )
		( ( Konoe1DHistogramOrderly * ) owner ) -> getTarget( );
	target -> dump( os );

	return 0;
}

Konoe1DHistogramOrderly::ClearUnit::ClearUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	Konoe1DHistogramOrderly::ClearUnit::execute( KonoeCommand * cmd ) {

	Konoe1DHistogram	* target;

	if( ! checkArgument( cmd ) ) return 0;

	target = ( Konoe1DHistogram * )
		( ( Konoe1DHistogramOrderly * ) owner ) -> getTarget( );
	target -> clear( );

	return 0;
}

Konoe1DHistogramOrderly::FillUnit::FillUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	Konoe1DHistogramOrderly::FillUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	Konoe1DHistogram	* target;

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_FILL_OBJECT );
        const KChar *objectname = arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );

	KonoeFillDataOrderly * filldataorderly
	  = (KonoeFillDataOrderly *) commander -> findObject(objectname);

	if( filldataorderly == 0 ){
	  os << "FillData object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}

	KonoeFillData *fill_data
		= (KonoeFillData * ) filldataorderly -> getTarget( );

	target = ( Konoe1DHistogram * )
		( ( Konoe1DHistogramOrderly * ) owner ) -> getTarget( );
	target -> fill( fill_data );

	return 0;
}

KInt	Konoe1DHistogramOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_SCALE );

        const KChar *objectname = arg -> getValue( );

	KonoeCommander *commander = getOfficer( ) -> getCommander( );

	KonoeLinearDataScaleOrderly * scaleorderly
	  = (KonoeLinearDataScaleOrderly *) commander -> findObject(objectname);

	if( scaleorderly == 0 ){
	  os << "Scale object " << objectname << " doesn't exist."
	     << endl;
	  return 0;
	}
	KonoeLinearDataScale *scale
		= (KonoeLinearDataScale * )scaleorderly -> getTarget( );

	target = new Konoe1DHistogram( scale );

	return 0;

}

KInt	Konoe1DHistogramOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* Konoe1DHistogramOrderly::clone( const KChar * name ) {

	return new Konoe1DHistogramOrderly( name );

}














