//==================================================================
/*

	KonoeDataElement.cc

	Version 0.1	18-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	16-Oct-1997	H. Sakamoto
		objectstream changed
	Version 0.4	12-Dec-1997	H. Sakamoto
		streamableobject changed

        Version 0.1.0                   H. Sakamoto
	Version 0.2.0	27-Apr-1998	H. Sakamoto

	Version 0.4.0	06-Aug-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//==================================================================

#include	<system/KonoeOutputObjectStream.hh>
#include	<analysis/KonoeDataElement.hh>
#include	<KonoeCategories.hh>

const KChar	* KonoeDataElement::className = "KonoeDataElement";

KonoeDataElement::KonoeDataElement( )
	: KonoeStreamableObject( 
		KonoeCategories::DATARECORD,
		KonoeDataRecordCM::CID_DATAELEMENT ) {

	store = 0;
	datasize = 0;
	type = -1;
}

KonoeDataElement::KonoeDataElement( KInt ptype, KInt pdatasize )
	: KonoeStreamableObject(
		KonoeCategories::DATARECORD,
		KonoeDataRecordCM::CID_DATAELEMENT ) {

	store = new KChar [ pdatasize * sizeof( KInt ) ];
	datasize = pdatasize;
	type = ptype;

}

KonoeDataElement::KonoeDataElement( KInt ptype, KInt paddress, KInt pvalue )
	: KonoeStreamableObject(
		KonoeCategories::DATARECORD,
		KonoeDataRecordCM::CID_DATAELEMENT ) {

	store = new KChar[ sizeof( KInt ) ];
	datasize = 1;
	type = ptype;
	address = paddress;
	* ( KInt * ) store = pvalue;

}

KonoeDataElement::~KonoeDataElement( ) {
	delete [] ( KChar * ) store;
}

void	KonoeDataElement::dump( ostream & os ) {

	KInt	i;
	os << "Element type = " << type
		<< "  Data size = " << datasize
		<< "  Address = " << address << endl;
	switch( type ) {
	default :
		os << "Unknown type = " << type << endl;
	case INT:
		KInt	* p;
		p = ( KInt * ) store;
		for( i = 0; i < datasize; i ++ )
		    os << * p ++ << " ";
		os << endl;
		break;
	}

}

KInt	KonoeDataElement::fill( KInt * pbuf, KInt psize ) {

	KInt	l;
	KInt	* p;
	if( psize == 1 ) {
	    * ( KInt * ) store = * pbuf;
	    return 1;
	}
	l = psize;
	if( l > datasize )
	    l = datasize;
	p = ( KInt * ) store;
	for( int i = 0; i < l; i ++ )
	    * p ++ = * pbuf ++;
	return l;
}

KInt	KonoeDataElement::put(
		KonoeOutputObjectStream * os,
		KChar level, KInt memberid ) {

	KChar	l = level + 1;
	KInt	st;

	st = os -> putHeader( this, level, memberid, N_MEMBERS );
	if( st < 0 )
	    return st;
	st = os -> put( l, MID_TYPE, type );
	if( st < 0 )
	    return st;
	st = os -> put( l, MID_ADDRESS, address );
	if( st < 0 )
	    return st;
	st = os -> put( l, MID_DATASIZE, datasize );
	if( st < 0 )
	    return st;
	switch( type ) {
	case INT :
		st = os -> put( l, MID_STORE, ( KInt * ) store, datasize );
		return st;
	case FLOAT :
		st = os -> put( l, MID_STORE, ( KFloat * ) store, datasize );
		return st;
	default :
		cerr << "Sorry. not yet implemented. type = "
			<< hex << type << dec << endl;
		return -1;
	}
}

KInt	KonoeDataElement::getStreamSize( ) {

	KInt	s;
	s = N_MEMBERS * ( HEADER_SIZE + 1 );
	switch( type ) {
	case INT :
		s += datasize - 1;
		return s;
	case FLOAT :
		s += datasize * sizeof( KFloat ) / sizeof( KInt ) - 1;
		return s;
	default :
		cerr << "Sorry. not yet implemented. type = " << type << endl;
		return s;
	}
}

KInt	KonoeDataElement::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_TYPE :
		type = * ( KInt * ) data;
		delete [] ( KChar * ) data;
		break;
	    case MID_ADDRESS :
		address = * ( KInt * ) data;
		delete [] ( KChar * ) data;
		break;
	    case MID_DATASIZE :
		datasize = * ( KInt * ) data;
		delete [] ( KChar * ) data;
		break;
	    case MID_STORE :
		store = data;
		break;
	    default :
		delete [] ( KChar * ) data;
		return -1;	// unknown / invalid memberid
	    }
	    return 0;
	case KFLOAT_TYPE :
	    if( memberid != MID_STORE ) {
		delete [] ( KChar * ) data;
		return -1;
	    }
	    store = data;
	    return 0;
	default :
	    delete [] ( KChar * ) data;
	    return -1;
	}
}

