//==================================================================
/*

	KonoeDataSection.cc

	Version 0.1	18-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.3	23-Oct-1997	H. Sakamoto
	Version 0.4	12-Dec-1997	H. Sakamoto
		streamable object changed

        Version 0.1.0                   H. Sakamoto
	Version 0.2.0	27-Apr-1998	H. Sakamoto

	Version 0.4.0	06-Aug-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	02-Dec-2000	H. Sakamoto
	Version 0.6.1	26-May-2001	H. Sakamoto
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//==================================================================

#include	<system/KonoeOutputObjectStream.hh>
#include	<analysis/KonoeDataSection.hh>
#include	<KonoeCategories.hh>

const KChar	* KonoeDataSection::className = "KonoeDataSection";

KonoeDataSection::KonoeDataSection( )
	: KonoeList<KonoeDataSegment>( 0 ),
	KonoeStreamableObject( KonoeCategories::DATARECORD,
		KonoeDataRecordCM::CID_DATASECTION ) {

	type = -1;

}

KonoeDataSection::KonoeDataSection( KInt ptablesize, KInt ptype )
	: KonoeList<KonoeDataSegment>( ptablesize ),
	KonoeStreamableObject( KonoeCategories::DATARECORD,
		KonoeDataRecordCM::CID_DATASECTION ) {

	type = ptype;

}

KonoeDataSection::~KonoeDataSection( ) {
}


void	KonoeDataSection::dump( ostream & os ) {
	KonoeDataSegment	* p;

	os << "Section type = " << type
		<< "  # segments = " << n_entries << endl;
	reset( );
	while( p = next( ) )
	    p -> dump( os );

}

KInt	KonoeDataSection::put(
		KonoeOutputObjectStream * os, KChar level,
		KInt memberid ) {

	KChar	l = level + 1;
	KInt	st, s, i, k;

	k = getCount( );
	st = os -> putHeader( this, level, memberid, N_MEMBERS );
	if( st < 0 )
	    return st;
	st = os -> put( l, MID_TYPE, type );
	if( st < 0 )
	    return st;
	st = os -> put( l, MID_N_ENTRIES, k );
	if( st < 0 )
	    return st;
	s = 0;
	reset( );
	for( i = 0; i < k; i ++ )
	    s += HEADER_SIZE + next( ) -> getStreamSize( );
	st = os -> putObjectArrayHeader( l, MID_ENTRIES, k, s );
	if( st < 0 )
	    return st;
	reset( );
	for( i = 0; i < k; i ++ ) {
	    st = next( ) -> put( os, l + 1, i );
	    if( st < 0 )
		return st;
	}
	return 0;

}

KInt	KonoeDataSection::getStreamSize( ) {

	KInt	s, i, k;
	
	s = HEADER_SIZE + 1;	// type
	s += HEADER_SIZE + 1;	// n_entries
	s += HEADER_SIZE;		// entries as an object array
	k = getCount( );
	reset( );
	for( i = 0; i < k; i ++ )
	    s += HEADER_SIZE + next( ) -> getStreamSize( );
	return s;
}

KInt	KonoeDataSection::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_TYPE :
		type = * ( KInt * ) data;
		delete [] ( KChar * ) data;
		break;
	    case MID_N_ENTRIES :
		// allocate as many entries for the list
		allocate( * ( KInt * ) data );
		delete [] ( KChar * ) data;
		break;
	    default :
		delete [] ( KChar * ) data;
		return -1;	// invalid member id
	    }
	    return 0;
	default :
	    delete [] ( KChar * ) data;
	    return -1;
	}

}

KInt	KonoeDataSection::fillObjectMember( KInt memberid,
				KonoeStreamableObject * data ) {
	if( memberid != MID_ENTRIES ) {
		delete data;
		return -1;	// invalid member id
	}
	add( ( KonoeDataSegment * ) data );
	return 0;

}
