//===================================================================
/*

	Konoe3Vector.cc

	Version 0.2.0	27-Apr-1998	H. Sakamoto
	Version 0.4.0	31-Jul-1998	S. Inaba
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/Konoe3Vector.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<KonoeCategories.hh>
#include	<math.h>

const KChar	* Konoe3Vector::className = "Konoe3Vector";
Konoe3Vector::Konoe3Vector( ) 
: KonoeStreamableObject( KonoeCategories::GEOMETRY, 
			KonoeGeometryCM::CID_3VECTOR ) {

	x = 0.0;
	y = 0.0;
	z = 0.0;

}
Konoe3Vector::Konoe3Vector( KFloat px, KFloat py, KFloat pz )
: KonoeStreamableObject( KonoeCategories::GEOMETRY, 
			KonoeGeometryCM::CID_3VECTOR ) {

	x = px;
	y = py;
	z = pz;

}
Konoe3Vector::Konoe3Vector( const Konoe3Vector & p )
: KonoeStreamableObject( KonoeCategories::GEOMETRY, 
			KonoeGeometryCM::CID_3VECTOR ) {

	x = p.x;
	y = p.y;
	z = p.z;

}

Konoe3Vector & Konoe3Vector::set( KFloat px, KFloat py, KFloat pz ) {

	x = px;
	y = py;
	z = pz;
	return * this;

}

Konoe3Vector & Konoe3Vector::operator = ( const Konoe3Vector & v ) {

	x = v.x;
	y = v.y;
	z = v.z;
	return * this;

}

Konoe3Vector	Konoe3Vector::operator + ( const Konoe3Vector & v ) const {

	Konoe3Vector	p;
	p.x = x + v.x;
	p.y = y + v.y;
	p.z = z + v.z;
	return p;

}

Konoe3Vector	Konoe3Vector::operator - ( const Konoe3Vector & v ) const {

	Konoe3Vector	p;
	p.x = x - v.x;
	p.y = y - v.y;
	p.z = z - v.z;
	return p;

}

Konoe3Vector	Konoe3Vector::operator * ( KFloat f ) const {

	Konoe3Vector	p;
	p.x = x * f;
	p.y = y * f;
	p.z = z * f;
	return p;
}

Konoe3Vector	Konoe3Vector::operator / ( KFloat f ) const {

	Konoe3Vector	p;
	p.x = x / f;
	p.y = y / f;
	p.z = z / f;
	return p;

}

Konoe3Vector	Konoe3Vector::operator * ( const Konoe3Vector & v ) const {

	Konoe3Vector	p;
	p.x = y * v.z - z * v.y;
	p.y = z * v.x - x * v.z;
	p.z = x * v.y - y * v.x;
	return p;

}

KFloat	Konoe3Vector::operator & ( const Konoe3Vector & v ) const {

	return x * v.x + y * v.y + z * v.z;

}

Konoe3Vector &	Konoe3Vector::operator += ( const Konoe3Vector & v ) {

	x += v.x;
	y += v.y;
	z += v.z;
	return * this;

}

Konoe3Vector &	Konoe3Vector::operator -= ( const Konoe3Vector & v ) {

	x -= v.x;
	y -= v.y;
	z -= v.z;
	return * this;

}

Konoe3Vector &	Konoe3Vector::operator *= ( KFloat f ) {

	x *= f;
	y *= f;
	z *= f;
	return * this;

}

Konoe3Vector &	Konoe3Vector::operator /= ( KFloat f ) {

	x /= f;
	y /= f;
	z /= f;
	return * this;

}

Konoe3Vector::operator KFloat( ) {

	return sqrt( x * x + y * y + z * z );

}

Konoe3Vector &	Konoe3Vector::normalize( ) {

	KFloat	f;
	f = sqrt( x * x + y * y + z * z );
	x /= f;
	y /= f;
	z /= f;
	return * this;

}

Konoe3Vector	operator * ( KFloat f, Konoe3Vector & v ) {

	return v * f;

}

void	Konoe3Vector::dump( ostream & os ) {

	os << "( " << x << ", " << y << ", " << z << " )";

}

ostream &	operator << ( ostream & os, Konoe3Vector & v ) {

	v.dump( os );
	return os;

}

KInt	Konoe3Vector::put( KonoeOutputObjectStream * os,
			KChar level, KInt memberid ) {

	KInt	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_X, x );
	os -> put( l, MID_Y, y );
	os -> put( l, MID_Z, z );

	return 0;
	}

KInt	Konoe3Vector::getStreamSize( ) {

	KInt s;
	s = HEADER_SIZE + sizeof(KFloat)/sizeof(KInt);
	s += HEADER_SIZE +  sizeof(KFloat)/sizeof(KInt);
	s += HEADER_SIZE +  sizeof(KFloat)/sizeof(KInt);
	return s;

	} 


KInt	Konoe3Vector::fillPrimitiveMember( KInt memberid, KInt datatype,
					KInt datacount, void * data ) {

	switch( datatype ){
	case KFLOAT_TYPE :
		switch( memberid ){
		case MID_X :
			x = * ( KFloat * ) data;
			delete [] data;
			break;
		case MID_Y :
			y = * ( KFloat * ) data;
			delete [] data;
			break;
		case MID_Z :
			z = * ( KFloat * ) data;
			delete [] data;
			break;
		default :
			delete [] data;
			return -1;
		}
	return 0;
	default :
		delete [] data;
		return -1;
	}
}
	
