//=================================================================
/*

	KonoeDetector.cc

	Version 0.2.0	28-Apr-1998	H. Sakamoto
	Version 0.4.0   31-Jul-1998	D. Takagi
	Version 0.5.0   21-Sep-1999	Y. Kouno
	Version 0.7.0   26-Aug-2001	T. SUmida

*/
//=================================================================

#include	<analysis/KonoeDetector.hh>
#include 	<analysis/KonoeGeometryCM.hh>
#include	<KonoeCategories.hh>

const KChar	* KonoeDetector::className = "KonoeDetector";


KonoeDetector::KonoeDetector( ) 
	: KonoeList<KonoeDetectorElement>( 0 ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, 
		KonoeGeometryCM::CID_DETECTOR ) {
}

KonoeDetector::KonoeDetector( KInt classid ) 
	: KonoeList<KonoeDetectorElement>( 0 ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, classid ) {
}

KonoeDetector::KonoeDetector( KInt ptablesize, KInt pidentifier )
	: KonoeList<KonoeDetectorElement>( ptablesize ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, 
		KonoeGeometryCM::CID_DETECTOR ) {

	identifier = pidentifier;

}

KonoeDetectorElement	* KonoeDetector::find( KInt psection, KInt psegment,
		KInt psubaddress ) {

	KonoeDetectorElement	* e;
	KonoeDataAddress	* a;

	reset( );
	while( e = next( ) ) {
	    e -> reset( );
	    while( a = e -> next( ) ) {
		if( a -> getSectionID( ) == psection &&
			a -> getSegmentID( ) == psegment &&
			a -> getSubaddress( ) == psubaddress ){
		  return e;
		}
	      }
	  }
	return 0;
	
      }

void 	KonoeDetectorElement::setDetector( KonoeDetector * det ) {
  if ( detector != 0 )
    delete detector;
//  cout << "det:" << det << endl;
//  cout << "detector: " << detector << endl;
  detector = det;
//  cout << "det': " << det << endl;
}

void    KonoeDetector::dump( ostream & os ) {
        os << "Detector ID: " << identifier << endl;
        reset();
	KInt i;
	for( i = 0; i < n_entries; i ++ )
	  next() -> dump( os ) ;
}

KInt	KonoeDetector::put(
		KonoeOutputObjectStream * os, KChar level,
		KInt memberid ) {
//  cout << "detector put" << endl;
	KChar	l = level + 1;

	KInt count = getCount( );
	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_IDENTIFIER, identifier );
	os -> put( l, MID_N_ENTRIES, count );
	KInt size = 0;
	reset( );
	KInt i;
	for( i = 0; i < count; i ++ ) {
	    size += HEADER_SIZE + next( ) -> getStreamSize( );
	}
//	cout << "count: " << count << " size: " << size << endl;
        os -> putObjectArrayHeader( l, MID_ENTRIES, count, size );
	reset( );
	for( i = 0; i < count; i ++ )
//	  cout << i << endl;  
	  next( ) -> put( os, l + 1, i );
	return 0;
}


KInt	KonoeDetector::getStreamSize( ) {
    
	KInt	size, count;
	
	size  = HEADER_SIZE + 1;  	// identifier
	size += HEADER_SIZE + 1;	// n_entries
	size += HEADER_SIZE;		// entries as an object array

	reset( );
	for( KInt i = 0; i < getCount( ); i ++ )
	    size += HEADER_SIZE + next( ) -> getStreamSize( );
	return size;
}

KInt	KonoeDetector::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {
//cout << "detector fill primitive member" << endl;
	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_IDENTIFIER :
		identifier = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_N_ENTRIES :
		//allocate as many entries for the list
//		cout << "allocate " << * ( KInt * ) data << endl;
		allocate( * ( KInt * ) data );
		delete [] data;
		break;
	    default :
		delete [] data;
		return -1;	// invalid member id
	    }
	    return 0;
	default :
	    delete [] data;
	    return -1;
	}

}

KInt	KonoeDetector::fillObjectMember( KInt memberid,
				KonoeStreamableObject * data ) {
//  cout << "detector fill object" << endl;
  
  if( memberid != MID_ENTRIES ) {
		delete data;
		return -1;	// invalid member id
	}
	KonoeDetectorElement * element = ( KonoeDetectorElement * ) data;
	element -> setDetector( this );
	add( element );
//        cout << "now count: " << getCount() << endl;
	return 0;

}
