//=================================================================
/*

	KonoeDetectorElement.cc

	Version 0.2.0	28-Apr-1998	H. Sakamoto
	Version 0.4.0   31-Jul-1998	D. Takagi
	Version 0.4.1	 5-Aug-1999	T. Yamashita
	  modified the method "interact"
	Version 0.5.0	21-Sep-1999	Y. Kouno
	23-Nov-2000      K.Uchida
	!!!!!!!  temporaly use : DetectorElement  delete Geometry
--

	Of course, not only the displacement but also the rotation
	of the shape should be included.

	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//=================================================================

#include	<analysis/KonoeDetectorElement.hh>
#include	<analysis/KonoeSimDetectorHit.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<KonoeCategories.hh>
#include        <iostream.h>
#include        <iomanip.h>

const KChar	* KonoeDetectorElement::className = "KonoeDetectorElement";

KonoeDetectorElement::KonoeDetectorElement( )
	: KonoeList<KonoeDataAddress>( MAX_READOUT ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, 
		KonoeGeometryCM::CID_DETECTORELEMENT ) {
	    detector = 0;
}


KonoeDetectorElement::KonoeDetectorElement( KInt classid )
	: KonoeList<KonoeDataAddress>( MAX_READOUT ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, classid ) {
}

KonoeDetectorElement::KonoeDetectorElement( 
                        KonoeGeometry * pgeometry,
			KonoeDetector * pdetector, KInt pidentifier )
	: KonoeList<KonoeDataAddress>( MAX_READOUT ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY, 
		KonoeGeometryCM::CID_DETECTORELEMENT ) {
   	geometry = pgeometry;
	detector = pdetector;
	identifier = pidentifier;
}

KonoeDetectorElement::~KonoeDetectorElement( ){
  delete geometry;
}

KonoeSimDetectorHit	* KonoeDetectorElement::interact(
		KonoeTrackSegment * segment ) {

	KonoeSimDetectorHit	* hit = 0;
	Konoe3Vector	entrypoint, exitpoint;
	KFloat	d, t;

	if( geometry -> isCrossing( segment -> getTrajectory( ),
		entrypoint, exitpoint ) ) {
	    hit = new KonoeSimDetectorHit( this, entrypoint, exitpoint );

	    d = entrypoint - segment -> getTrajectory( ) -> getStartPoint( );
	    t = segment->getStartTime() + d / segment -> getVelocity( );
            hit -> setEntryTime( t );

	    d = exitpoint - segment -> getTrajectory( ) -> getStartPoint( );
	    t = segment->getStartTime() + d / segment -> getVelocity( );
            hit -> setExitTime( t );
	}
	return hit;

}

void	KonoeDetectorElement::dump( ostream & os ) {
	KInt	i, n;
	os << "ID: " << identifier << endl;
	geometry -> dump( os );
	n = getCount( );
//	cout << "address count: " << n << endl;
	reset( );
	for( i = 0; i < n; i ++ )
	  next( ) -> dump( os );
	reset();
      }

KInt	KonoeDetectorElement::put(
		KonoeOutputObjectStream * os, KChar level,
		KInt memberid ) {
        KChar	l = level + 1;
        os -> putHeader( this, level, memberid, N_MEMBERS );
	
	KInt  count = getCount();
	os -> put( l, MID_N_ENTRIES, count );
	KInt i, size = 0;
	reset();
	for( i = 0; i < count; i ++ ) {
	  size += HEADER_SIZE + next( ) -> getStreamSize( );
	}
	os -> putObjectArrayHeader( l, MID_ENTRIES, count, size );
	reset();
	for( i = 0; i < count; i ++ )
	  next( ) -> put( os, l + 1, i );
	geometry -> put( os, l, MID_GEOMETRY );
	os -> put( l, MID_IDENTIFIER, identifier );
	return 0;
      }

KInt	KonoeDetectorElement::getStreamSize( ) {

	KInt size = 0, s, i;
	size  = HEADER_SIZE + 1;	// n_entries
	size += HEADER_SIZE;		// entries as an object array
	reset( );
	s = getCount();
	for(  i = 0; i < s; i ++ ) {
	  size += HEADER_SIZE + next( ) -> getStreamSize( );
	}
	size += HEADER_SIZE + geometry -> getStreamSize( );	// geometry
	size += HEADER_SIZE + 1;        // identifier
	return size;
      }

KInt	KonoeDetectorElement::fillPrimitiveMember( KInt memberid, 
                                KInt datatype,
				KInt datacount, void * data ) {
	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_N_ENTRIES :
	      allocate( * ( KInt * ) data );
	      delete [] data;
	      break;
	    case MID_IDENTIFIER :
	      identifier = * ( KInt * ) data;
	      delete [] data;
	      break;
	    default :
	      delete [] data;
	      return -1;	// invalid member id
	    }
        return 0;
        default :
	    delete [] data;
	    return -1;
	}
}

KInt	KonoeDetectorElement::fillObjectMember( KInt memberid,
				KonoeStreamableObject * data ) {
//  cout << "detector element fill object member" << endl;
    switch( memberid ) {
	case MID_ENTRIES :
		add( ( KonoeDataAddress * ) data );
		break;
	case MID_GEOMETRY :
		geometry = ( KonoeGeometry * ) data;
		break;
	default :
	         return -1;	// invalid member id
	}
	return 0;

}




