//=================================================================
/*

	KonoeDetectorElement.hh

	Version 0.2.0	28-Apr-1998	H. Sakamoto
	Version 0.4.0    3-Aug-1998	D. Takagi
	Version 0.5.0   21-Sep-1999	Y. Kouno
	13-Oct-2000  Kirika Uchida
--

	two usage. one for analysis and one for simulation.

	1) analysis case.
		this gives geometrical information.
		searched by Sec ID, Seg. ID and Element ID/Address.

	2) simulation case.
		from interaction with a track, this gives timing and
		response data which fills sim-module channels.
		this should be treated in DAQ package, by
		inheriting this class.


	Version 0.7.0   26-Aug-2001 T. Sumida


*/
//=================================================================

#ifndef	__KONOEDETECTORELEMENT_HH
#define	__KONOEDETECTORELEMENT_HH

#include	<KonoeList.hh>
#include	<analysis/KonoeGeometry.hh>
#include	<analysis/KonoeTrackSegment.hh>
#include	<analysis/KonoeDataAddress.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<iostream.h>
#include        <analysis/KonoeBoxGeometry.hh>

class	KonoeSimDetectorHit;
class	KonoeDetector;

class	KonoeDetectorElement	: public KonoeList<KonoeDataAddress>,
		public KonoeStreamableObject {

public :
        enum	{	MAX_READOUT = 8	};
	enum	MEMBER_ID	{   MID_N_ENTRIES, MID_ENTRIES, 
				    MID_GEOMETRY, 
				    MID_IDENTIFIER,
                                    N_MEMBERS }; 

	static const KChar	* className;

protected :
        KonoeGeometry	* geometry;
	KInt		identifier;
	KonoeDetector	* detector;
public :
	KonoeDetectorElement( );
	KonoeDetectorElement( KInt classid );
	KonoeDetectorElement( KonoeGeometry * geometry,
				KonoeDetector * detector, KInt identifier );
        ~KonoeDetectorElement();
	KonoeSimDetectorHit	* interact( KonoeTrackSegment * segment );
	KInt	getID( )	{	return identifier;	}
	KonoeDetector	* getDetector( )	{ return detector; }
	KonoeGeometry	* getGeometry( )	{ return geometry; }
	void 	setDetector( KonoeDetector * det );

	void	dump( ostream & os );

// KonoeStreamableObject members.

	KInt	put( KonoeOutputObjectStream * os, KChar level,
			KInt memberid );
	KInt	getStreamSize( );

	KInt	fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data );
	KInt	fillObjectMember( KInt memberid,
				KonoeStreamableObject * data );
	
};

#endif
