//==================================================================
/*

	KonoeDetectorHit.cc

	Version 0.2.0	29-Apr-1998	H. Sakamoto
	Version 0.4.0	31-Jul-1998	S. Inaba
	Version 0.5.0	21-Sep-1999	Y. Kouno
        16-Oct-2000   Kirika Uchida 
          KonoeSimDetectorHit::energy -> KonoeDetectorHit::hitenergy

	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//===================================================================

#include	<analysis/KonoeDetectorHit.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<KonoeCategories.hh>

const KChar	* KonoeDetectorHit::className = "KonoeDetectorHit";

KonoeDetectorHit::KonoeDetectorHit( ): KonoeStreamableObject( KonoeCategories::GEOMETRY, KonoeGeometryCM::CID_DETECTORHIT ) {
  detectorelement_tag = 0;
}

KonoeDetectorHit::KonoeDetectorHit( KInt classid )
            : KonoeStreamableObject( KonoeCategories::GEOMETRY, classid ) {
	      detectorelement_tag = 0;
}

KonoeDetectorHit::KonoeDetectorHit( KInt categoryid, KInt classid )
            : KonoeStreamableObject( categoryid, classid ) {
	      detectorelement_tag = 0;
}

KonoeDetectorHit::KonoeDetectorHit( KonoeDetectorElement * e )
		: KonoeStreamableObject( KonoeCategories::GEOMETRY,
			KonoeGeometryCM::CID_DETECTORHIT ) {
		  detectorelement_tag = 0;
		  detectorelement = e;

}

KonoeDetectorHit::~KonoeDetectorHit(){
  if ( detectorelement_tag )
    delete detectorelement;
}

void KonoeDetectorHit::dump( ostream& os ){
  os << "TDC value: " << TDCValue << endl;
  os << "hit time:"   << hittime << endl;
  os << "ADC value: " << ADCValue << endl;
  os << "hitenergy: " << hitenergy << endl;
}

KInt	KonoeDetectorHit::put(KonoeOutputObjectStream * os, KChar level, 
			      KInt memberid ) {
	KChar	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	
	detectorelement -> put( os, l, MID_DETECTOR_ELEMENT );
	os -> put( l, MID_HITTIME, hittime );
	os -> put( l, MID_TDC_VALUE, TDCValue );
	os -> put( l, MID_HITENERGY, hitenergy );
	os -> put( l, MID_ADC_VALUE, ADCValue );
	return 0;
	} 
	
KInt	KonoeDetectorHit::getStreamSize( ) {

	KInt s;
	s = HEADER_SIZE + detectorelement -> getStreamSize( );
	s += HEADER_SIZE + sizeof( KFloat ) / sizeof( KInt ) ;  
	s += HEADER_SIZE + 1;
	s += HEADER_SIZE + sizeof( KFloat ) / sizeof( KInt ) ;  
	s += HEADER_SIZE + 1;
	return s;
	}

KInt	KonoeDetectorHit::fillPrimitiveMember( KInt memberid, KInt datatype,
					       KInt datacount, void * data ) {
  switch( datatype ) {
  case KFLOAT_TYPE :
    switch( memberid ) {
    case MID_HITTIME:
      hittime = * ( KFloat * ) data;
      delete [] data;
      break;
    case MID_HITENERGY:
      hitenergy = * ( KFloat * ) data;
      delete [] data;
      break;
    default:
      delete [] data;
      return -1;
    }
    break;
  case KINT_TYPE:
    switch ( memberid ){
    case MID_TDC_VALUE:
      TDCValue = * ( KInt * ) data;
      delete [] data;
      break;
    case MID_ADC_VALUE:
      ADCValue = * ( KInt * ) data;
      delete [] data;
      break;
    default :
      delete [] data;
      return -1;
    }
    break;
  default :
    delete [] data;
    return -1;
  }
  return 0;
}

KInt	KonoeDetectorHit::fillObjectMember( KInt memberid, 
		KonoeStreamableObject * obj ) {
  
	switch( memberid ) {
	case MID_DETECTOR_ELEMENT :
	        detectorelement = ( KonoeDetectorElement * ) obj;
		detectorelement_tag = 1;
		break;
	default :
		delete obj;
		return -1; 
	}
	return 0;
}
