//==================================================================
/*

	KonoeDetectorList.cc

	Version 0.2.0	29-Apr-1998	H. Sakamoto
	Version 0.4.0	06-Aug-1998	D. Takagi
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//==================================================================

#include	<analysis/KonoeDetectorList.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<KonoeCategories.hh>

const KChar * KonoeDetectorList::className = "KonoeDetectorList";

KonoeDetectorList::KonoeDetectorList( ) 
	: KonoeList<KonoeDetector>( 0 ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY,
		KonoeGeometryCM::CID_DETECTORLIST ) {
}

KonoeDetectorList::KonoeDetectorList( KInt ptablesize )
	: KonoeList<KonoeDetector>( ptablesize ),
	  KonoeStreamableObject( KonoeCategories::GEOMETRY,
		KonoeGeometryCM::CID_DETECTORLIST ) {

}

KInt	KonoeDetectorList::put(
		KonoeOutputObjectStream * os, KChar level,
		KInt memberid ) {

	KChar	l = level + 1;

	KInt count = getCount( );
	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_N_ENTRIES, count );
	KInt size = 0;
	reset( );
	KInt i;
	for( i = 0; i < count; i ++ ) {
	    size += HEADER_SIZE + next( ) -> getStreamSize( );
	}
	os -> putObjectArrayHeader( l, MID_ENTRIES, count, size );
	reset( );
	for( i = 0; i < count; i ++ )
	    next( ) -> put( os, l + 1, i );
	return 0;
}


KInt	KonoeDetectorList::getStreamSize( ) {
    
	KInt	size, count;

	size  = HEADER_SIZE + 1;	// n_entries
	size += HEADER_SIZE;		// entries as an object array

	reset( );
	for( KInt i = 0; i < getCount( ); i ++ )
	    size += HEADER_SIZE + next( ) -> getStreamSize( );
	return size;
}

KInt	KonoeDetectorList::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_N_ENTRIES :
		//allocate as many entries for the list
		allocate( * ( KInt * ) data );
		delete [] data;
		break;
	    default :
		delete [] data;
		return -1;	// invalid member id
	    }
	    return 0;
	default :
	    delete [] data;
	    return -1;
	}

}

KInt	KonoeDetectorList::fillObjectMember( KInt memberid,
				KonoeStreamableObject * data ) {
	if( memberid != MID_ENTRIES ) {
		delete data;
		return -1;	// invalid member id
	}
	KonoeDetector * element = ( KonoeDetector * ) data;
	add( element );
	return 0;

}
