//==================================================================
/*

	KonoeHitDataset.cc

	Version 0.2.0	13-May-1998	H. Sakamoto
	Version 0.4.0	06-Aug-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//==================================================================

#include        <iomanip.h>
#include	<analysis/KonoeHitDataset.hh>

KChar	* KonoeHitDataset::dataNameList[ ] = {
	"Detector ID", "Hit Time", "Hit Energy" };

KonoeHitDataset::KonoeHitDataset( KInt ptablesize,
				  KonoeMultipletDescriptor * pspecifier, 
				  KInt ptag )
  : KonoePhysicalValue( ptag ) {
          multipletlist = 0;
	  hitlist = new KonoeList<KonoeDetectorHit>( ptablesize );
	  multipletlist = new KonoeList<KonoeDataMultiplet>( ptablesize );
	  specifier = pspecifier;
	  if( specifier == 0 ) {
	    specifier = new KonoeMultipletDescriptor( MULTIPLET_SIZE );
	    specifier -> add
	      ( new KonoeDataSpecifier( dataNameList[ DETECTOR_ID_INDEX ] ) );
	    detector_id_index = DETECTOR_ID_INDEX;
	    
	    specifier -> add( new KonoeDataSpecifier(
				dataNameList[ HIT_TIME_INDEX ] ) );
	    hit_time_index = HIT_TIME_INDEX;

	    specifier -> add( new KonoeDataSpecifier(
				dataNameList[ HIT_ENERGY_INDEX ] ) );
	    hit_energy_index = HIT_ENERGY_INDEX;
	}
	else {
	    detector_id_index = specifier -> getIndex(
				dataNameList[ DETECTOR_ID_INDEX ] );
	    hit_time_index = specifier -> getIndex(
				dataNameList[ HIT_TIME_INDEX ] );
	    hit_energy_index = specifier -> getIndex(
				dataNameList[ HIT_ENERGY_INDEX ] );

	}
}

KonoeHitDataset::~KonoeHitDataset( ) {

	delete hitlist;
	if(multipletlist)
	  delete multipletlist;
}

KInt	KonoeHitDataset::clear( ) {

	hitlist -> clear( );
	multipletlist -> clear( );
	return 0;
}

KInt	KonoeHitDataset::reset( ) {

	multipletlist -> reset( );
	return 0;
}

KInt	KonoeHitDataset::getDataCount( ) {

	multipletlist -> reset( );
	return multipletlist -> getCount( );

}

KonoeDataMultiplet	* KonoeHitDataset::next( ) {

	return multipletlist -> next( );

}

KInt	KonoeHitDataset::add( KonoeDetectorHit * hit ) {

	KInt	st;
	if( ( st = hitlist -> add( hit ) ) < 0 )
	    return st;	// table full
	return st;

}

void KonoeHitDataset::dump(ostream & os) {
  KonoeDataMultiplet * mul;
  KInt i = 0;
  reset();
  os << setw(10) << "Detector ID" << setw(12) << "hit time" 
    << setw(13) << "hit energy" << endl;
  while( mul = next()) {
    os << setw(8)<< mul -> getValue( detector_id_index ) 
       << setw(13) << mul -> getValue( hit_time_index ) 
       << setw(13) << mul -> getValue( hit_energy_index ) << endl;
  }
}

void  KonoeHitDataset::MakeMultipletList( ) {
  KonoeDetectorHit * hit;
  hitlist -> reset();
  while( hit = hitlist -> next()){
    KonoeDataMultiplet	* multiplet;
    multiplet = new KonoeDataMultiplet( specifier );
    if( detector_id_index >= 0 )
      multiplet -> setValue( detector_id_index,
			    hit -> getDetectorElement( ) -> getID( ) );
    if( hit_time_index >= 0 )
      multiplet -> setValue( hit_time_index,
			    hit -> getHitTime( ) );
    if( hit_energy_index >= 0 )
      multiplet -> setValue( hit_energy_index,
			    hit -> getHitEnergy( ) );
    multipletlist -> add(multiplet);
  }
}
