//==================================================================
/*

	KonoeTranslation.cc

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version 0.4.0	06-Aug-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//==================================================================

#include	<analysis/KonoeTranslation.hh>
#include	<math.h>

KonoeTranslation::KonoeTranslation( ) {

	KInt	i, j;

	for( i = 0; i < 3; i ++ )
	    for( j = 0; j < 3; j ++ )
		if( i == j )
		    t[ i ][ j ] = 1.0;
		else
		    t[ i ][ j ] = 0.0;
	displacement.set( 0.0, 0.0, 0.0 );

}

void	KonoeTranslation::setDisplacement( Konoe3Vector & d ) {

	displacement = d;

}

void	KonoeTranslation::addRotationX( KFloat theta ) {

	KFloat	s, c;
	KFloat	u[ 3 ][ 3 ];
	KInt	i, j;

	s = sin( theta );
	c = cos( theta );

	u[ 0 ][ 0 ] = t[ 0 ][ 0 ];
	u[ 0 ][ 1 ] = t[ 0 ][ 1 ] * c - t[ 0 ][ 2 ] * s;
	u[ 0 ][ 2 ] = t[ 0 ][ 1 ] * s + t[ 0 ][ 2 ] * c;
	u[ 1 ][ 0 ] = t[ 1 ][ 0 ];
	u[ 1 ][ 1 ] = t[ 1 ][ 1 ] * c - t[ 1 ][ 2 ] * s;
	u[ 1 ][ 2 ] = t[ 1 ][ 1 ] * s + t[ 1 ][ 2 ] * c;
	u[ 2 ][ 0 ] = t[ 2 ][ 0 ];
	u[ 2 ][ 1 ] = t[ 2 ][ 1 ] * c - t[ 2 ][ 2 ] * s;
	u[ 2 ][ 2 ] = t[ 2 ][ 1 ] * s + t[ 2 ][ 2 ] * c;
	for( i = 0; i < 3; i ++ )
	    for( j = 0; j < 3; j ++ )
		t[ i ][ j ] = u[ i ][ j ];
}
void	KonoeTranslation::addRotationY( KFloat theta ) {

	KFloat	s, c;
	KFloat	u[ 3 ][ 3 ];
	KInt	i, j;

	s = sin( theta );
	c = cos( theta );

	u[ 0 ][ 0 ] = t[ 0 ][ 0 ] * c + t[ 0 ][ 2 ] * s;
	u[ 0 ][ 1 ] = t[ 0 ][ 1 ];
	u[ 0 ][ 2 ] = - t[ 0 ][ 0 ] * s + t[ 0 ][ 2 ] * c;
	u[ 1 ][ 0 ] = t[ 1 ][ 0 ] * c + t[ 1 ][ 2 ] * s;
	u[ 1 ][ 1 ] = t[ 1 ][ 1 ];
	u[ 1 ][ 2 ] = - t[ 1 ][ 0 ] * s + t[ 1 ][ 2 ] * c;
	u[ 2 ][ 0 ] = t[ 2 ][ 0 ] * c + t[ 2 ][ 2 ] * s;
	u[ 2 ][ 1 ] = t[ 2 ][ 1 ];
	u[ 2 ][ 2 ] = - t[ 2 ][ 0 ] * s + t[ 2 ][ 2 ] * c;
	for( i = 0; i < 3; i ++ )
	    for( j = 0; j < 3; j ++ )
		t[ i ][ j ] = u[ i ][ j ];
}
void	KonoeTranslation::addRotationZ( KFloat theta ) {

	KFloat	s, c;
	KFloat	u[ 3 ][ 3 ];
	KInt	i, j;

	s = sin( theta );
	c = cos( theta );

	u[ 0 ][ 0 ] = t[ 0 ][ 0 ] * c - t[ 0 ][ 1 ] * s;
	u[ 0 ][ 1 ] = t[ 0 ][ 0 ] * s + t[ 0 ][ 1 ] * c;
	u[ 0 ][ 2 ] = t[ 0 ][ 2 ];
	u[ 1 ][ 0 ] = t[ 1 ][ 0 ] * c - t[ 1 ][ 1 ] * s;
	u[ 1 ][ 1 ] = t[ 1 ][ 0 ] * s + t[ 1 ][ 1 ] * c;
	u[ 1 ][ 2 ] = t[ 1 ][ 2 ];
	u[ 2 ][ 0 ] = t[ 2 ][ 0 ] * c - t[ 2 ][ 1 ] * s;
	u[ 2 ][ 1 ] = t[ 2 ][ 0 ] * s + t[ 2 ][ 1 ] * c;
	u[ 2 ][ 2 ] = t[ 2 ][ 2 ];
	for( i = 0; i < 3; i ++ )
	    for( j = 0; j < 3; j ++ )
		t[ i ][ j ] = u[ i ][ j ];
}

void	KonoeTranslation::resetRotation( ) {

	KInt	i, j;
	for( i = 0; i < 3; i ++ )
	    for( j = 0; j < 3; j ++ )
		if( i == j )
		    t[ i ][ j ] = 1.0;
		else
		    t[ i ][ j ] = 0.0;

}

Konoe3Vector	KonoeTranslation::rotate( Konoe3Vector & v ) {

	Konoe3Vector	r;

	r.x = t[ 0 ][ 0 ] * v.x + t[ 0 ][ 1 ] * v.y + t[ 0 ][ 2 ] * v.z;
	r.y = t[ 1 ][ 0 ] * v.x + t[ 1 ][ 1 ] * v.y + t[ 1 ][ 2 ] * v.z;
	r.z = t[ 2 ][ 0 ] * v.x + t[ 2 ][ 1 ] * v.y + t[ 2 ][ 2 ] * v.z;
	return r;

}
