//====================================================================
/*

	Konoe1DHistogram.cc

	Version 0.1	30-Oct-1997	H. Sakamoto
	Version 0.2	12-Dec-1997	H. Sakamoto
		streamable object changed
	Version 0.2.0	 9-Mar-1998	Sanshiro Enomoto
                modified for HistogramFiller
	Version 0.2.1	27-Apr-1998	H. Sakamoto
	Version 0.2.2	02-Jun-1998	H. Sakamoto
		scaling.
	Version 0.4.0	10-Sep-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//====================================================================

#include	<analysis/Konoe1DHistogram.hh>
#include	<analysis/KonoeHistogramCM.hh>
#include	<analysis/KonoePhysicalValueCM.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<KonoeCategories.hh>

const KChar	* Konoe1DHistogram::className = "Konoe1DHistogram";

Konoe1DHistogram::Konoe1DHistogram( KonoeDataScale * pscale )
		: KonoeHistogramBase( KonoeHistogramCM::CID_1DHISTOGRAM ) {

	scale = pscale;
	storage = new KonoeHistogramStorage( scale -> getBinCount( ) );

}

Konoe1DHistogram::Konoe1DHistogram( )
		: KonoeHistogramBase( KonoeHistogramCM::CID_1DHISTOGRAM ) {

	scale = 0;
	storage = 0;

}


Konoe1DHistogram::~Konoe1DHistogram( ) {

	delete scale;
	delete storage;

}

void	Konoe1DHistogram::clear( ) {

	if( storage )
	    storage -> clear( );
	//else... should be implemented.

}

KInt	Konoe1DHistogram::fill( KonoeFillData * fill_data )
{
    storage->addOne( scale -> getBinIndex( fill_data->getX() ) );
    delete fill_data;

    return 0;
}

void	Konoe1DHistogram::dump( ostream & os ) {

	os << "1D Histogram" << endl;
	scale -> dump( os );
	storage -> dump( os );
}


// to be streamable...

KInt	Konoe1DHistogram::put( KonoeOutputObjectStream * os,
		KChar level, KInt memberid ) {

	KChar	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	scale -> put( os, l, MID_SCALE );
	storage -> put( os, l, MID_STORAGE );
	return 0;

}

KInt	Konoe1DHistogram::getStreamSize( ) {

	KInt	s;

	s = HEADER_SIZE + scale -> getStreamSize( );
	s += HEADER_SIZE + storage -> getStreamSize( );
	return s;

}

KInt	Konoe1DHistogram::fillObjectMember( KInt memberid,
		KonoeStreamableObject * obj ) {

	    KInt	c = obj -> getCategoryID( );

	    if( c == category_id ) {	// this.
		switch( obj -> getClassID( ) ) {
		case KonoeHistogramCM::CID_HISTOGRAMSTORAGE :
		    storage = ( KonoeHistogramStorage * ) obj;
		    break;
		default :
		    delete obj;
		    return -1;
		}
	    }
	    else if( c == KonoeCategories::PHYSICALVALUE ) {
		switch( obj -> getClassID( ) ) {
		case KonoePhysicalValueCM::CID_LINEARDATASCALE :
		    scale = ( KonoeLinearDataScale * ) obj;
		    break;
		default :
		    delete obj;
		    return -1;	// invalid context
		}
	    }
	    else {
		delete obj;
		return -1;	// unknown object.
	    }
	    return 0;
}

KInt	Konoe1DHistogram::getBinCount( )
{
	return scale->getBinCount( );
}

KFloat	Konoe1DHistogram::getOne( KInt pindex )
{
	return storage->get( pindex );
}

KonoeDataScale* Konoe1DHistogram::getScale( )
{
	return scale;
}
