//==========================================================
/*
	Konoe1DHistogramList.cc

	Version 0.6.0	 5-Oct-2000	Y. Shimizu
	Version 0.7.0	26-Aug-2001	T. Sumida
*/
//==========================================================

#include	<analysis/Konoe1DHistogramList.hh>
#include	<analysis/Konoe1DHistogram.hh>
#include	<system/KonoeOutputObjectStream.hh>
#include	<KonoeCategories.hh>
#include	<analysis/KonoeHistogramCM.hh>

const KChar	* Konoe1DHistogramList::className = "Konoe1DHistogramList";

Konoe1DHistogramList::Konoe1DHistogramList( )
	: KonoeList< Konoe1DHistogram >( 0 ),
	KonoeStreamableObject( KonoeCategories::HISTOGRAM,
			KonoeHistogramCM::CID_1DHISTOGRAMLIST ){

}

Konoe1DHistogramList::Konoe1DHistogramList( KInt ptablesize ) :
	KonoeList< Konoe1DHistogram >( ptablesize ),
		KonoeStreamableObject( KonoeCategories::HISTOGRAM,
			KonoeHistogramCM::CID_1DHISTOGRAMLIST ){

}


void Konoe1DHistogramList::dump( ostream & os){

	Konoe1DHistogram * hist ;
	
	os << "1DHistogram List "  << endl;
	os << " this list hold : " << n_entries << endl; 
	reset( );
	while( hist = next() )
		hist->dump( os );

}

KInt Konoe1DHistogramList::put( KonoeOutputObjectStream * os , KChar level, 
					KInt memberid ){

	KChar 	l = level + 1 ;
	KInt	s , i , k;

	k = getCount( );
	os->putHeader( this, level , memberid , N_MEMBERS );
	os->put( l , MID_N_ENTRIES, k );
	s = 0 ;
	reset( );
	for( i = 0 ; i < k ; i ++ )
		s += HEADER_SIZE + next( )->getStreamSize( );
	os->putObjectArrayHeader( l , MID_ENTRIES, k, s );
	reset( );
	for( i = 0 ; i < k ; i ++ )
 		next( ) -> put( os , l + 1 , i );
	return 0;	
}

KInt 	Konoe1DHistogramList::getStreamSize( ){

	KInt	s, i , k;
	
	s = HEADER_SIZE + 1; // n_entries 
	s += HEADER_SIZE;    // entries as an object array
	k = getCount( );
	reset( );
	for( i = 0 ; i < k ; i ++ )
		s += HEADER_SIZE + next( )->getStreamSize( );
	
	return s ;
	
}

KInt	Konoe1DHistogramList::fillPrimitiveMember( KInt memberid,KInt datatype,
		KInt datacount , void * data ){
	switch( datatype ) {
	case KINT_TYPE :
		switch( memberid ){
		case MID_N_ENTRIES :
			allocate( * (KInt *) data );
			delete [] data;
			break;	
		default:
			delete [] data;
			return -1;	
		}
		return 0;
	default :
		delete [] data;
		return -1;	
	}

}

KInt Konoe1DHistogramList::fillObjectMember( KInt memberid , 
				KonoeStreamableObject * data ) {

	if( memberid != MID_ENTRIES ) {
		delete data;
		return -1;
	}
	
	add( ( Konoe1DHistogram * ) data );
	return 0;

}
	

