//====================================================================
/*

	Konoe2DHistogram.cc

	Version 0.1.1	18-Dec-1997	Sanshiro Enomoto
                based Konoe1DHistogram.cc Version 0.2
	Version 0.2.0	 9-Mar-1998	Sanshiro Enomoto
                modified for HistogramFiller
	Version 0.2.1	27-Apr-1998	H. Sakamoto
	Version 0.2.2	02-Jun-1998	H. Sakamoto
	Version 0.4.0	10-Sep-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.6.0	29-Aug-2000	T. Hosoi
		fixed fill method
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//====================================================================

#include	<analysis/Konoe2DHistogram.hh>
#include	<analysis/KonoeHistogramCM.hh>
#include	<analysis/KonoePhysicalValueCM.hh>
#include	<analysis/KonoeLinearDataScale.hh>
#include	<KonoeCategories.hh>

const KChar	* Konoe2DHistogram::className = "Konoe2DHistogram";

Konoe2DHistogram::Konoe2DHistogram( KonoeDataScale * pxscale, KonoeDataScale * pyscale )
		: KonoeHistogramBase( KonoeHistogramCM::CID_2DHISTOGRAM ) {

	xscale = pxscale;
	yscale = pyscale;
	storage = new KonoeHistogramStorage( xscale -> getBinCount( ), yscale -> getBinCount( ) );

}

Konoe2DHistogram::Konoe2DHistogram( )
		: KonoeHistogramBase( KonoeHistogramCM::CID_2DHISTOGRAM ) {

	xscale = 0;
	yscale = 0;
	storage = 0;

}


Konoe2DHistogram::~Konoe2DHistogram( ) {

	delete xscale;
	delete yscale;
	delete storage;

}

void	Konoe2DHistogram::clear( ) {

	if( storage )
	    storage -> clear( );
	//else... should be implemented.

}

KInt	Konoe2DHistogram::fill( KonoeFillData * fill_data )
{
    storage->addOne( xscale -> getBinIndex( fill_data->getX() ),
		yscale -> getBinIndex( fill_data->getY() ) );
    delete fill_data;
    return 0;
}

void	Konoe2DHistogram::dump( ostream & os ) 
{
	os << "2D Histogram" << endl;
	xscale -> dump( os );
	yscale -> dump( os );
	storage -> dump( os );
}

KInt    Konoe2DHistogram::getXscale()
{
    return (KInt) xscale -> getMaximum();
}

KInt    Konoe2DHistogram::getYscale()
{
    return (KInt) yscale -> getMaximum();
}

KInt    Konoe2DHistogram::getBinCountx()
{
    return xscale -> getBinCount();
}

KInt    Konoe2DHistogram::getBinCounty()
{
    return yscale -> getBinCount();
}

KFloat  Konoe2DHistogram::getOne( KInt pxindex, KInt pyindex )
{
    return storage -> get( pxindex, pyindex );
}

// to be streamable...

KInt	Konoe2DHistogram::put( KonoeOutputObjectStream * os,
		KChar level, KInt memberid ) {

	KChar	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	xscale -> put( os, l, MID_XSCALE );
	yscale -> put( os, l, MID_YSCALE );
	storage -> put( os, l, MID_STORAGE );
	return 0;

}

KInt	Konoe2DHistogram::getStreamSize( ) {

	KInt	s;

	s = HEADER_SIZE + xscale -> getStreamSize( );
	s += HEADER_SIZE + yscale -> getStreamSize( );
	s += HEADER_SIZE + storage -> getStreamSize( );
	return s;

}

KInt	Konoe2DHistogram::fillObjectMember( KInt memberid,
		KonoeStreamableObject * obj ) {

	    KInt	c = obj -> getCategoryID( );

	    if( c == category_id ) {	// this.
		switch( obj -> getClassID( ) ) {
		case KonoeHistogramCM::CID_HISTOGRAMSTORAGE :
		    storage = ( KonoeHistogramStorage * ) obj;
		    break;
		default :
		    delete obj;
		    return -1;
		}
	    }
	    else if( c == KonoeCategories::PHYSICALVALUE ) {
		switch( obj -> getClassID( ) ) {
		case KonoePhysicalValueCM::CID_LINEARDATASCALE :
		    if (memberid == MID_XSCALE)
			xscale = ( KonoeLinearDataScale * ) obj;
		    else if (memberid == MID_YSCALE)
			yscale = ( KonoeLinearDataScale * ) obj;
		    else
                        return -1; // invalid member ID
		    break;
		default :
		    delete obj;
		    return -1;	// invalid context
		}
	    }
	    else {
		delete obj;
		return -1;	// unknown object.
	    }
	    return 0;
}

