//=====================================================================
/*

	KonoeHistogramStorage.cc

	Version 0.1	30-Oct-1997	H. Sakamoto
	Version 0.2	12-Dec-1997	H. Sakamoto
		streamable object changed.
        Version 0.1.0                   H. Sakamoto
	Version 0.2.0	27-Apr-1998	H. Sakamoto
	Version 0.4.0	10-Sep-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//=====================================================================

/*

	The storage has 2 bins more than required bin count. They
	contain overflow and underflow values for each dimension.

*/

#include	<analysis/KonoeHistogramStorage.hh>
#include	<analysis/KonoeHistogramCM.hh>

const KChar	* KonoeHistogramStorage::className = "KonoeHistogramStorage";

// this is necessary to be storeamable.

KonoeHistogramStorage::KonoeHistogramStorage( )
	: KonoeHistoStorageBase( KonoeHistogramCM::CID_HISTOGRAMSTORAGE ) {

	store = 0;
	storage_size = 0;
}

KonoeHistogramStorage::KonoeHistogramStorage(
		KInt xsize, KInt ysize, KInt zsize )
 	: KonoeHistoStorageBase( KonoeHistogramCM::CID_HISTOGRAMSTORAGE ){

	x_size = xsize;
	y_size = ysize;
	z_size = zsize;
	x_bincount = xsize + 2;
	y_bincount = ysize + 2;
	z_bincount = zsize + 2;

	if( zsize != 0 ) {
	    type = TYPE_3D;
	    storage_size = x_bincount * y_bincount * z_bincount;
// never set y = 0 if z != 0.
	}
	else if( ysize != 0 ) {
	    type = TYPE_2D;
	    storage_size = x_bincount * y_bincount;
	}
	else {
	    type = TYPE_1D;
	    storage_size = x_bincount;
	}

	store = new KInt[ storage_size ];

}

KonoeHistogramStorage::~KonoeHistogramStorage( ) {

	delete [] store;

}

KInt	KonoeHistogramStorage::set( KFloat value,
		KInt xindex, KInt yindex, KInt zindex ) {

	KInt	k;

	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 )
		return k;
	store[ k ] = KInt( value );
	return k;
}

KInt	KonoeHistogramStorage::addOne(
		KInt xindex, KInt yindex, KInt zindex ) {

	KInt	k;

	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 )
		return k;
	store[ k ] ++;
	return k;

}

KFloat	KonoeHistogramStorage::get( KInt xindex, KInt yindex, KInt zindex ) {

	KInt	k;

	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 )
		return 0.0;
	return KFloat( store[ k ] );

}

void	KonoeHistogramStorage::clear( ) {

	KInt	* p, i;
	p = store;
	for( i = 0; i < storage_size; i ++ )
	    * p ++ = 0;

}

void	KonoeHistogramStorage::dump( ostream & os ) {

	KInt	i, j;
	KInt	* p;
	p = store;
	i = 0;
	while( i < storage_size ) {
	    os.width( 4 );
	    os << i << ": ";
	    j = i + 10;
	    if( j > storage_size )
		j = storage_size;
	    os.width( 6 );
	    for( ; i < j ; i ++ )
		os << * p ++ << " ";
	    os << endl;
	}

}

KInt	KonoeHistogramStorage::get_index(
		KInt xindex, KInt yindex, KInt zindex ) {

	KInt	k;

	k = 0;
	switch( type ) {
	case TYPE_3D :
	    if( zindex >=0 && zindex < z_size )
		k = zindex * x_bincount * y_bincount;
	    else if( zindex == -1 || zindex == -2 )
		k = ( zindex + z_bincount ) * x_bincount * y_bincount;
	    else
		return -1;	// failure
	case TYPE_2D :
	    if( yindex >= 0 && yindex < y_size )
		k += yindex * x_bincount;
	    else if( yindex == -1 || yindex == -2 )
		k += ( yindex + y_bincount ) * x_bincount;
	    else
		return -1;	// failure
	case TYPE_1D :
	    if( xindex >= 0 && xindex < x_size )
		k += xindex;
	    else if( xindex == -1 || xindex == -2 )
		k += xindex + x_bincount;
	    else
		return -1;
	}
	return k;

}

// StreamableObject members

KInt	KonoeHistogramStorage::put( KonoeOutputObjectStream * os,
			KChar level, KInt memberid ) {

	KInt	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_TYPE, type );
	os -> put( l, MID_X_SIZE, x_size );
	os -> put( l, MID_Y_SIZE, y_size );
	os -> put( l, MID_Z_SIZE, z_size );
	os -> put( l, MID_STORAGE_SIZE, storage_size );
	os -> put( l, MID_STORE, store, storage_size );

	return 0;

}

KInt	KonoeHistogramStorage::getStreamSize( ) {

	return ( N_MEMBERS - 1 ) * ( HEADER_SIZE + 1 )
		+ HEADER_SIZE + storage_size;

}


KInt	KonoeHistogramStorage::fillPrimitiveMember(
				KInt memberid, KInt datatype,
				KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_TYPE :
		type = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_X_SIZE :
		x_size = * ( KInt * ) data;
		x_bincount = x_size + 2;
		delete [] data;
		break;
	    case MID_Y_SIZE :
		y_size = * ( KInt * ) data;
		y_bincount = y_size + 2;
		delete [] data;
		break;
	    case MID_Z_SIZE :
		z_size = * ( KInt * ) data;
		z_bincount = z_size + 2;
		delete [] data;
		break;
	    case MID_STORAGE_SIZE :
		storage_size = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_STORE :
		store = ( KInt * ) data;
		// check storage_size
		break;
	    default :
		delete [] data;
		return -1;	// unknown 
	    }
	    return 0;
	default :
	    delete [] data;
	    return -1;
	}	
}
