//=================================================================
/*

	KonoeDataAddress.cc

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version 0.2.1   31-Jul-1998     D. Takagi
	Version 0.4.0	10-Sep-1998	D. Takagi	(Check only)
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//=================================================================

#include	<analysis/KonoeDataAddress.hh>
#include	<analysis/KonoePhysicalValueCM.hh>
#include	<KonoeCategories.hh>
#include         <iomanip.h>
const KChar	* KonoeDataAddress::className = "KonoeDataAddress";

KonoeDataAddress::KonoeDataAddress( ) 
	: KonoeStreamableObject( KonoeCategories::PHYSICALVALUE,
		KonoePhysicalValueCM::CID_DATAADDRESS ) {
}

KonoeDataAddress::KonoeDataAddress( KInt classid )
	: KonoeStreamableObject( KonoeCategories::PHYSICALVALUE, classid ) {
}

KonoeDataAddress::KonoeDataAddress( KInt psectionid, KInt psegmentid,
			KInt psubaddress, KInt ptype, KInt pbit )
	: KonoeStreamableObject( KonoeCategories::PHYSICALVALUE,
		KonoePhysicalValueCM::CID_DATAADDRESS ) {

	section_id = psectionid,
	segment_id = psegmentid,
	subaddress = psubaddress,
	address_type = ptype;
	bit_offset = pbit;
}

KInt	KonoeDataAddress::match( KonoeDataAddress & ref ) {

	if( ref.section_id != section_id )
	    return 0;
	if( ref.segment_id != segment_id )
	    return 0;
	if( ref.subaddress != subaddress )
	    return 0;
	if( ref.address_type != address_type )
	    return 0;
	if( address_type == NORMAL )
	    return 1;
	if( ref.bit_offset != bit_offset )
	    return 0;
	return 1;
}

void	KonoeDataAddress::dump( ostream & os ) {
        os << setw(12) << "section id" << setw(12) << "segment id"
	  << setw(12) << "address" << setw(15) << "bit offset"
	  << endl; 

 	os << setw(6) << section_id << setw(13) << segment_id
		<< setw(13) << subaddress;
	if( address_type == BITMAP )
	    os << setw(15) << bit_offset;
	os << endl;

}

KInt	KonoeDataAddress::put(
		KonoeOutputObjectStream * os, KChar level,
		KInt memberid ) {

	KChar	l = level + 1;
//cout << "put address" << endl;
//	cout << "address n members: " << N_MEMBERS << endl;
	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_ADDRESS_TYPE, address_type );
	os -> put( l, MID_SECTION_ID, section_id );
	os -> put( l, MID_SEGMENT_ID, segment_id );
	os -> put( l, MID_SUBADDRESS, subaddress );
	os -> put( l, MID_BIT_OFFSET, bit_offset );
	return 0;
}

KInt	KonoeDataAddress::getStreamSize( ) {

	KInt size = 0;
	size  = HEADER_SIZE + 1;	// address_type
	size += HEADER_SIZE + 1;	// section_id
	size += HEADER_SIZE + 1;	// segment_id
	size += HEADER_SIZE + 1;	// subaddress
	size += HEADER_SIZE + 1;	// bit_offset
	return size;
}

KInt	KonoeDataAddress::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {
//  cout << "address fill primitive member " << endl;
	switch( datatype ) {
	case KINT_TYPE :
	    switch( memberid ) {
	    case MID_ADDRESS_TYPE :
		address_type = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_SECTION_ID :
		section_id = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_SEGMENT_ID :
		segment_id = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_SUBADDRESS :
		subaddress = * ( KInt * ) data;
		delete [] data;
		break;
	    case MID_BIT_OFFSET :
		bit_offset = * ( KInt * ) data;
		delete [] data;
		break;
	    default :
		delete [] data;
		return -1;	// invalid member id
	      }
	    return 0;
	default :
	    delete [] data;
	    return -1;
	
	}
        return 0;
      }


KInt	KonoeDataAddress::fillObjectMember( KInt memberid,
				KonoeStreamableObject * data ) {

	// KonoeDataAddess has no object menbers.	
	return -1;

}

