//====================================================================
/*

	KonoeDataAnalyzer.cc

	Version 0.1	02-Aug-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version 0.1.0                   H. Sakamoto
	Version 0.4.0	10-Sep-1998	D. Takagi
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//====================================================================

#include	<analysis/KonoeDataAnalyzer.hh>

KonoeDataAnalyzer::KonoeDataAnalyzer( KInt pn_sections ) {

	section_analyzer_list = new KonoeList<KonoeSectionAnalyzer>
			( pn_sections );

}

KInt	KonoeDataAnalyzer::reset( ) {

	KonoeSectionAnalyzer	* a;

	section_analyzer_list -> reset( );
	while( a = section_analyzer_list -> next( ) ) {
	    a -> clear( );
	}
	return 0;

}

KInt	KonoeDataAnalyzer::analyze(
		KonoeDataRecord * precord ) {

	KonoeDataSection	* section;
	KInt	st;

	record = precord;

	record -> reset( );
	while( section = record -> next( ) ) {
	    if( ( st = process_section( section ) ) < 0 ) return st;
	}

	return 0;
}

KInt	KonoeDataAnalyzer::process_section( KonoeDataSection * section ) {

	KonoeSectionAnalyzer	* analyzer;
	if( analyzer = find_analyzer( section -> getID( ) ) ) {
	    return analyzer -> analyze( section );
	}
	else {
cout<<"KonoeDataAnalyzer Error. Analyzer for Section "
	<< section -> getID( ) << " not found" << endl;
	    return -1;		// analyzer not found
	}
}

KInt	KonoeDataAnalyzer::install( KonoeSectionAnalyzer * section_analyzer ) {

	return section_analyzer_list -> add( section_analyzer );
}

KonoeSectionAnalyzer	* KonoeDataAnalyzer::find_analyzer( KInt tag ) {

	KonoeSectionAnalyzer	* a;

	section_analyzer_list -> reset( );
	while( a = section_analyzer_list -> next( ) ) {
	    if( a -> getID( ) == tag )
		return a;
	}
	return 0;
}

