//====================================================================
/*

	KonoeLinearDataScale.cc

	Version 0.1	30-Oct-1997	H. Sakamoto
	Version 0.2	12-Dec-1997	H. Sakamoto
	Version 0.1.0                   H. Sakamoto
	Version 0.2.0	27-Apr-1998	H. Sakamoto
	Version 0.4.0	10-Sep-1998	D. Takagi
	Version 0.5.0	21-Sep-1999	Y. Kouno
	Version 0.7.0	26-Aug-2001	T. Sumida

*/
//====================================================================

#include	<analysis/KonoeLinearDataScale.hh>
#include	<analysis/KonoePhysicalValueCM.hh>

const KChar	* KonoeLinearDataScale::className = "KonoeLinearDataScale";

KonoeLinearDataScale::KonoeLinearDataScale(
	KFloat pminimum, KFloat pmaximum, KInt pbincount )
	: KonoeDataScale( KonoePhysicalValueCM::CID_LINEARDATASCALE ) {

	minimum = pminimum;
	maximum = pmaximum;
	bincount = pbincount;
	binwidth = ( pmaximum - pminimum ) / bincount;

}

KonoeLinearDataScale::KonoeLinearDataScale( )
	: KonoeDataScale( KonoePhysicalValueCM::CID_LINEARDATASCALE ) {
}


KInt	KonoeLinearDataScale::getBinIndex( KFloat value ) {

	KFloat	i;

	i = ( value - minimum ) / binwidth;
	if( i < 0. )
	    return UNDERFLOW;
	else if( (KInt)i >= bincount )
	    return OVERFLOW;
	return (KInt)i;
}

KFloat	KonoeLinearDataScale::getBinLower( KInt binnumber ) {

	return minimum + binwidth * binnumber;

}

void	KonoeLinearDataScale::dump( ostream & os ) {

	os << "Linear scale. from " << minimum << " to "
		<< maximum << " in " << bincount << " steps of "
		<< binwidth << " wide bins" << endl;

}

KInt	KonoeLinearDataScale::put( KonoeOutputObjectStream * os,
		KChar level, KInt memberid ) {

	KChar	l = level + 1;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_MINIMUM, minimum );
	os -> put( l, MID_MAXIMUM, maximum );
	os -> put( l, MID_BINWIDTH, binwidth );
	os -> put( l, MID_BINCOUNT, bincount );
	return 0;
}

KInt	KonoeLinearDataScale::getStreamSize( ) {

	return 3 * ( HEADER_SIZE + 2 ) + HEADER_SIZE + 1;

}

KInt	KonoeLinearDataScale::fillPrimitiveMember( KInt memberid, KInt datatype,
				KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
	    if( memberid != MID_BINCOUNT ) {
		delete [] data;
		return -1;	// error
	    }
	    bincount = * ( KInt * ) data;
	    return 0;
	case KFLOAT_TYPE :
	    switch( memberid ) {
	    case MID_MINIMUM :
		minimum = * ( KFloat * ) data;
		delete [] data;
		break;
	    case MID_MAXIMUM :
		maximum = * ( KFloat * ) data;
		delete [] data;
		break;
	    case MID_BINWIDTH :
		binwidth = * ( KFloat * ) data;
		delete [] data;
		break;
	    default :
		delete [] data;
		return -1;
	    }
	    return 0;
	default :
	    delete [] data;
	    return -1;
	}
}
