// $Id: KonoeCrateListOrderly.cc,v 1.7 2002/03/16 08:11:34 iwai Exp $
// $Name: KONOE-0-09-02 $
//====================================================================
/*

	KonoeCrateListOrderly.cc

	Version 0.0.1	24-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
        Version 0.6.0   04-Aug-2000     G. Iwai
	Version 0.6.1	24-Nov-2000	H. Sakamoto
	Version 0.6.2	11-May-2001	H. Sakamoto
	Version 0.6.3	21-May-2001	H. Sakamoto

*/
//====================================================================
#include <daq/KonoeCrateListOrderly.hh>
#include <ui/KonoeHelpCommand.hh>
#include <ui/KonoeOfficer.hh>
#include <daq/KonoeVMECrateOrderly.hh>
#include	<daq/KonoeSoftwareCrateOrderly.hh>

const KChar* KonoeCrateListOrderly::classname = "CrateList";

KonoeCrateListOrderly::KonoeCrateListOrderly( const KChar * objname )
  : KonoeOrderly(N_COMMANDS, objname)
{
  setClassID(CLASS_ID);

	KonoeArgumentDescriptor	* ad;

  createcommand = new KonoeCommandDescriptor("create", N_ARG_CREATE);
	ad = new KonoeArgumentDescriptor("object", STRING, NECESSARY);
	ad -> setDescription( "name of the crate list" );
	createcommand -> add( ad );
 	ad = new KonoeArgumentDescriptor("size", INTEGER, NECESSARY);
	ad -> setDescription( "size of the list" );
	createcommand -> add( ad );
	createcommand -> setDescription( "creates a container for crates" );
  createunit = new CreateUnit(createcommand, this);
  addCommand(createunit);

  showcommand = new KonoeCommandDescriptor("show", N_ARG_SHOW);
	ad = new KonoeArgumentDescriptor("crate", STRING, OPTIONAL);
	ad -> setDescription( "name of crate in the list" );
	showcommand -> add( ad );
	showcommand -> setDescription( "print contents of the crate list" );
  showunit = new ShowUnit(showcommand, this);
  addCommand(showunit);

  addcommand = new KonoeCommandDescriptor("add", N_ARG_ADD);
	ad = new KonoeArgumentDescriptor("crate", STRING, NECESSARY);
	ad -> setDescription( "name of crate to be added to the list" );
	addcommand -> add( ad );
	addcommand -> setDescription( "install a crate to the crate list" );
  addunit = new AddUnit(addcommand, this);
  addCommand(addunit);

  getcommand = new KonoeCommandDescriptor("get", N_ARG_GET);
	ad = new KonoeArgumentDescriptor("index", INTEGER, NECESSARY);
	ad -> setDescription( "index number of a crate in the list" );
	getcommand -> add( ad );
	ad = new KonoeArgumentDescriptor("object", STRING, NECESSARY);
	ad -> setDescription( "name of the crate object obtained from the list" );
	getcommand -> add( ad );
	getcommand -> setDescription( "get a crate as a named object" );
  getunit = new GetUnit(getcommand, this);
  addCommand(getunit);

  addCommand(new KonoeHelpCommand(0, this));

	initializecommand = new KonoeCommandDescriptor( "initialize",
		N_ARG_INITIALIZE );
	initializecommand -> setDescription(
		"activate 'initialize' method for all crates" );
	initializeunit = new InitializeUnit( initializecommand, this );
	addCommand( initializeunit );

	clearcommand = new KonoeCommandDescriptor( "clear",
		N_ARG_CLEAR );
	clearcommand -> setDescription(
		"activate 'clear' method for all crates" );
	clearunit = new ClearUnit( clearcommand, this );
	addCommand( clearunit );

	updatecommand = new KonoeCommandDescriptor( "update",
		N_ARG_UPDATE );
	updatecommand -> setDescription(
		"activate 'update' method for all crates" );
	updateunit = new UpdateUnit( updatecommand, this );
	addCommand( updateunit );

}

KInt KonoeCrateListOrderly::isOrderlyOf(void* address)
{
  return address == (void*) target;
}

KonoeCrateListOrderly::CreateUnit::
CreateUnit(KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  : KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::CreateUnit::execute(KonoeCommand* cmd)
{
  KonoeOfficer* officer;

  ostream &os = cmd -> getOutputStream();

  if (cmd -> getArgumentCount() < N_ARG_CREATE) {
    os << "Argument missing. Needs object name and table size" << endl;
    return 0;
  }

  KonoeOrderly* o = owner -> 
    clone((KChar*) cmd -> getArgument(ARG_CREATE_OBJECT) -> getValue());
  o -> createObject(cmd);
  o -> setOfficer(officer = owner -> getOfficer());
  officer -> add(o);
  return 0;
}


KonoeCrateListOrderly::AddUnit::
AddUnit(KonoeCommandDescriptor* cd, KonoeOrderly* ow)
  : KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::AddUnit::execute(KonoeCommand* cmd)
{
  ostream  &os = cmd -> getOutputStream();

  if (cmd -> getArgumentCount() < N_ARG_ADD) {
    os << "Argument missing. Needs crate" << endl;
    return 0;
  }

  KonoeOrderly* o;
  KonoeOfficer* officer = owner -> getOfficer();

  o = officer -> 
    findObject((KChar*) cmd -> getArgument(ARG_ADD_CRATE) -> getValue());

  if (o == 0) {
    os << "Object not found = " 
       << cmd -> getArgument(ARG_ADD_CRATE) -> getValue() << endl;
    return 0;
  }

  KInt cid = o -> getClassID();
	if (cid == KonoeVMECrateOrderly::CLASS_ID) {
	    ((KonoeCrateList*) owner -> getTarget()) -> 
		add((KonoeVMECrate*) o -> getTarget());
	}
	else if( cid == KonoeSoftwareCrateOrderly::CLASS_ID ) {
	    ( ( KonoeCrateList * ) owner -> getTarget( ) ) ->
		add( ( KonoeSoftwareCrate * ) o -> getTarget( ) );
	}
	else {
	    os << "Not a crate object" << endl;
	    return 0;
	}
  
  return 0;
}

KonoeCrateListOrderly::ShowUnit::
ShowUnit(KonoeCommandDescriptor* cd, KonoeOrderly* ow )
  : KonoeCommandProcessor(cd, ow) 
{;}

KInt KonoeCrateListOrderly::ShowUnit::execute(KonoeCommand* cmd)
{

	KonoeCrateList	* cratelist;
	ostream & os = cmd -> getOutputStream( );

	cratelist = ( KonoeCrateList * ) owner -> getTarget( );
	os << "CrateList : " << owner -> getObjectName( ) << endl;

  return 0;
}

KonoeCrateListOrderly::GetUnit::
GetUnit(KonoeCommandDescriptor* cd, KonoeOrderly* ow) 
  : KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::GetUnit::execute(KonoeCommand* cmd)
{
  return 0;
}

KonoeCrateListOrderly::InitializeUnit::InitializeUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::InitializeUnit::execute(KonoeCommand* cmd)
{
	KonoeCrateList * cl = ( KonoeCrateList * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeCrateListOrderly::InitializeUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> initialize( );
	return 0;
}

KonoeCrateListOrderly::ClearUnit::ClearUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::ClearUnit::execute(KonoeCommand* cmd)
{
	KonoeCrateList * cl = ( KonoeCrateList * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeCrateListOrderly::ClearUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> clear( );
	return 0;
}

KonoeCrateListOrderly::UpdateUnit::UpdateUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeCrateListOrderly::UpdateUnit::execute(KonoeCommand* cmd)
{
	KonoeCrateList * cl = ( KonoeCrateList * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeCrateListOrderly::UpdateUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> update( );
	return 0;
}


KInt KonoeCrateListOrderly::createObject(KonoeCommand* cmd)
{
  ostream &os = cmd -> getOutputStream();

  KonoeCommandArgument* arg = cmd -> getArgument(ARG_CREATE_SIZE);
  if (! arg -> checkInteger()) {
    os << "Invalid argument type. Integer required = "
       << arg -> getValue( ) << endl;
    return -1;
  }

  KInt size = arg -> toInteger();

  target = new KonoeCrateList(size);

  return 0;
}

KInt KonoeCrateListOrderly::deleteObject(KonoeCommand*)
{
  delete target;
  return 0;
}

KonoeOrderly* KonoeCrateListOrderly::clone( const KChar* name )
{
  return new KonoeCrateListOrderly(name);
}
