// $Id: KonoePT3ChipOrderly.cc,v 1.3 2002/03/16 08:11:36 iwai Exp $
// $Name: KONOE-0-09-02 $
//=================================================================
/*

	KonoePT3ChipOrderly.cc

	Version 0.0.1	21-May-2001	H. Sakamoto

*/
//=================================================================

#include	<daq/KonoePT3ChipOrderly.hh>
#include	<ui/KonoeHelpCommand.hh>

const KChar	* KonoePT3ChipOrderly::classname = "PT3Chip";

KonoePT3ChipOrderly::KonoePT3ChipOrderly( const KChar * objname )
		: KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;

// create command

	cd = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Name to be assigned to the created chip" );
	cd -> add( ad );
	cd -> setDescription( "Create a chip for PT3 module. Not for use." );
	addCommand( createunit = new KonoeCreateCommand( cd, this ) );
	createunit -> setClassName( "CreateUnit" );

// help command

	addCommand( new KonoeHelpCommand( 0, this ) );

// show command

	cd = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	cd -> setDescription( "Show FPGA chip status" );
	addCommand( new ShowUnit( cd, this ) );

// configure command

	cd = new KonoeCommandDescriptor( "configure", N_ARG_CONFIGURE );
	ad = new KonoeArgumentDescriptor( "file", STRING, NECESSARY );
	ad -> setDescription( "File name to be used for the configuration" );
	cd -> add( ad );
	cd -> setDescription( "Configure the FPGA chip" );
	addCommand( new ConfigureUnit( cd, this ) );

}

KInt	KonoePT3ChipOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	createunit -> printErrorHeader( os );
	os << "PT3Chip object can't be created directly."
		" Create a PT3Module and get from it." << endl;
	return -1;

}

KInt	KonoePT3ChipOrderly::deleteObject( KonoeCommand * cmd ) {

	return 0;

}

KInt	KonoePT3ChipOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOrderly	* KonoePT3ChipOrderly::clone( const KChar * name ) {

	return new KonoePT3ChipOrderly( name );

}

void	KonoePT3ChipOrderly::setTarget( void * ptarget ) {

	target = ( KonoePT3Chip * ) ptarget;

}


KonoePT3ChipOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ShowUnit" );

}

KInt	KonoePT3ChipOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoePT3Chip	* chip = ( KonoePT3Chip * ) owner -> getTarget( );

	chip -> showStatus( os );

	return 0;

}

KonoePT3ChipOrderly::ConfigureUnit::ConfigureUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "CongigureUnit" );

}

KInt	KonoePT3ChipOrderly::ConfigureUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg
		 = cmd -> getArgument( ARG_CONFIGURE_FILE );

	const KChar	* filename = arg -> getValue( );

	KonoePT3Chip	* chip = ( KonoePT3Chip * ) owner -> getTarget( );

	return chip -> configure( filename );

}

