// $Id: KonoePT3ModuleOrderly.cc,v 1.4 2002/03/16 08:11:36 iwai Exp $
// $Name: KONOE-0-09-02 $
//=================================================================
/*

	KonoePT3ModuleOrderly.cc

	Version 0.0.1	21-May-2001	H. Sakamoto
	Version 0.0.2	25-Jun-2001	H. Sakamoto


*/
//=================================================================

#include	<daq/KonoePT3ModuleOrderly.hh>
#include	<daq/KonoePT3ChipOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<string.h>

const KChar	* KonoePT3ModuleOrderly::classname = "PT3Module";

KonoePT3ModuleOrderly::KonoePT3ModuleOrderly( const KChar * objname )
		: KonoeVMEModuleOrderly( objname, N_COMMANDS ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;

// create command

	cd = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Name to be assigned to the module object" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "address", INTEGER, NECESSARY );
	ad -> setDescription( "VME address of the module" );
	cd -> add( ad );
	cd -> setDescription( "Create a PT3 module" );
	createunit = new KonoeCreateCommand( cd, this );
	createunit -> setClassName( "CreateUnit" );
	addCommand( createunit );

// set command

	cd = new KonoeCommandDescriptor( "set", N_ARG_SET );
	ad = new KonoeArgumentDescriptor( "item", STRING, NECESSARY );
	ad -> setDescription( "Item to be set. {LED}" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "value", INTEGER, NECESSARY );
	ad -> setDescription( "Value to be set." );
	cd -> add( ad );
	addCommand( new SetUnit( cd, this ) );

// get command

	cd = new KonoeCommandDescriptor( "get", N_ARG_GET );
	ad = new KonoeArgumentDescriptor( "index", INTEGER, NECESSARY );
	ad -> setDescription( "Index number for the chip {0-3}" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Object name assigned to the chip" );
	cd -> add( ad );
	cd -> setDescription( "Get an FPGA chip on the module" );
	addCommand( new GetUnit( cd, this ) );

}

KInt	KonoePT3ModuleOrderly::createObject( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream	& os = cmd -> getOutputStream( );

	if( n_arg < ARG_CREATE_ADDRESS + 1 ) {
	    createunit -> printError( os,
		KonoeCommandProcessor::E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg =
			cmd -> getArgument( ARG_CREATE_ADDRESS );

	if( ! arg -> isInteger( ) ) {
	    createunit -> printError( os,
		KonoeCommandProcessor::E_ARGUMENT_NOT_INTEGER,
		arg -> getValue( ) );
	    return -1;
	}

	KInt	vmeaddress = arg -> toInteger( );

	target = new KonoePT3Module( vmeaddress );

	return 0;

}

KInt	KonoePT3ModuleOrderly::deleteObject( KonoeCommand * cmd ) {

	return 0;

}

KonoeOrderly	* KonoePT3ModuleOrderly::clone( const KChar * name ) {

	return new KonoePT3ModuleOrderly( name );

}


//

KonoePT3ModuleOrderly::SetUnit::SetUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "SetUnit" );

}

KInt	KonoePT3ModuleOrderly::SetUnit::execute( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream	& os = cmd -> getOutputStream( );

	if( n_arg < N_ARG_SET ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_SET_ITEM );
	const KChar	* item = arg -> getValue( );

	if( strcmp( item, "LED" ) == 0 || strcmp( item, "led" ) == 0 ) {

	    arg = cmd -> getArgument( ARG_SET_VALUE );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
		return -1;
	    }

	    KInt flag = arg -> toInteger( );

	    KonoePT3Module	* module =
		( KonoePT3Module * ) owner -> getTarget( );

	    module -> setLED( flag );

	}

	return 0;

}

KonoePT3ModuleOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "GetUnit" );

}

KInt	KonoePT3ModuleOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream	& os = cmd -> getOutputStream( );

	if( n_arg < N_ARG_GET ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_GET_INDEX );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	chip = arg -> toInteger( );

	if( chip < 0 || chip >= KonoePT3Module::N_FPGA ) {
	    printError( os, E_ARGUMENT_OUT_OF_RANGE, arg -> getValue( ) );
	    return -1;
	}

	arg = cmd -> getArgument( ARG_GET_NAME );

	if( ! arg -> isAlphabetic( ) ) {
	    printError( os, E_ARGUMENT_NOT_ALPHA, arg -> getValue( ) );
	    return -1;
	}

	KChar	* name = ( KChar * ) arg -> getValue( );

	KonoePT3ChipOrderly	* orderly = new KonoePT3ChipOrderly( name );
	KonoeOfficer	* officer;

	KonoePT3Module	* target = ( KonoePT3Module * ) owner -> getTarget( );

	orderly -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( orderly );
	orderly -> setTarget( ( void * ) ( target -> getChip( chip ) ) );

	return 0;

}
