// $Id: KonoePT4ChipOrderly.cc,v 1.4 2002/03/16 08:11:36 iwai Exp $
// $Name: KONOE-0-09-02 $
//==================================================================
/*

	KonoePT4ChipOrderly.cc

	Version 0.0.1	18-May-2001	H. Sakamoto

*/
//==================================================================

#include	<daq/KonoePT4ChipOrderly.hh>

#include	<daq/KonoePT4Chip.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<ui/KonoeHelpCommand.hh>

const KChar	* KonoePT4ChipOrderly::classname = "PT4Chip";

KonoePT4ChipOrderly::KonoePT4ChipOrderly( const KChar * objname )
		: KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;

// create command

	cd = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Name to be assigned to the created chip" );
	cd -> add( ad );
	cd -> setDescription( "Create a chip for PT4 module. Not for use" );
	addCommand( createunit = new KonoeCreateCommand( cd, this ) );
	createunit -> setClassName( "CreateUnit" );

// help command

	addCommand( new KonoeHelpCommand( 0, this ) );

// show command

	cd = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	cd -> setDescription( "Show FPGA chip status" );
	addCommand( new ShowUnit( cd, this ) );

// configure command

	cd = new KonoeCommandDescriptor( "configure", N_ARG_CONFIGURE );
	ad = new KonoeArgumentDescriptor( "file", STRING, NECESSARY );
	ad -> setDescription( "File name to be used for the configuration" );
	cd -> add( ad );
	cd -> setDescription( "Configure the FPGA chip" );
	addCommand( new ConfigureUnit( cd, this ) );

}

KInt	KonoePT4ChipOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	createunit -> printErrorHeader( os );
	os << "PT4Chip object can't be created directly."
		" Create a PT4Module and get from it." << endl;
	return -1;

}

KInt	KonoePT4ChipOrderly::deleteObject( KonoeCommand * cmd ) {

	return 0;

}

KInt	KonoePT4ChipOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOrderly	* KonoePT4ChipOrderly::clone( const KChar * name ) {

	return new KonoePT4ChipOrderly( name );

}

void	KonoePT4ChipOrderly::setTarget( void * ptarget ) {

	target = ( KonoePT4Chip * ) ptarget;

}

// command processors.

KonoePT4ChipOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ShowUnit" );

}

KInt	KonoePT4ChipOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoePT4Chip	* chip = ( KonoePT4Chip * ) owner -> getTarget( );

	chip -> showStatus( os );

	return 0;

}

KonoePT4ChipOrderly::ConfigureUnit::ConfigureUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "CongigureUnit" );

}

KInt	KonoePT4ChipOrderly::ConfigureUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg
		 = cmd -> getArgument( ARG_CONFIGURE_FILE );

	const KChar	* filename = arg -> getValue( );

	KonoePT4Chip	* chip = ( KonoePT4Chip * ) owner -> getTarget( );

	return chip -> configure( filename );

}

