// $Id: KonoePT4ModuleOrderly.cc,v 1.5 2002/03/16 08:11:37 iwai Exp $
// $Name: KONOE-0-09-02 $
//==================================================================
/*

	KonoePT4ModuleOrderly.cc

	Version 0.0.1	18-May-2001	H. Sakamoto
	Version 0.0.2	23-May-2001	H. Sakamoto

*/
//==================================================================

#include	<daq/KonoePT4ModuleOrderly.hh>

#include	<daq/KonoePT4Module.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<daq/KonoePT4ChipOrderly.hh>
#include	<ui/KonoeOfficer.hh>

const	KChar	* KonoePT4ModuleOrderly::classname = "PT4Module";

KonoePT4ModuleOrderly::KonoePT4ModuleOrderly( const KChar * objname )
		: KonoeVMEModuleOrderly( objname, N_COMMANDS ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;
	KonoeCommandProcessor	* cp;

// create command

	cd = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Name to be assigned to the created module" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "offset", INTEGER, NECESSARY );
	ad -> setDescription( "Relative address offset in a crate" );
	cd -> add( ad );
	cd -> setDescription( "Create a PT4 module" );
	addCommand( createunit = new KonoeCreateCommand( cd, this ) );
	createunit -> setClassName( "CreateUnit" );

// show command

	cd = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	ad = new KonoeArgumentDescriptor( "item", STRING, OPTIONAL );
	ad -> setDescription( "Item to be shown. 'status' etc" );
	cd -> add( ad );
	cd -> setDescription( "Show module information" );
	addCommand( new ShowUnit( cd, this ) );

// get command

	cd = new KonoeCommandDescriptor( "get", N_ARG_GET );
	ad = new KonoeArgumentDescriptor( "chip", INTEGER, NECESSARY );
	ad -> setDescription( "FPGA chip number. 0 to 3" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	ad -> setDescription( "Object name assigned to the chip" );
	cd -> add( ad );
	cd -> setDescription( "Get an FPGA chip as an object" );
	addCommand( new GetUnit( cd, this ) );

}

KInt	KonoePT4ModuleOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KInt	n_arg = cmd -> getArgumentCount( );
	if( n_arg < 2 ) {
	    createunit -> printError(
		os, KonoeCommandProcessor::E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument * arg = cmd -> getArgument( ARG_CREATE_OFFSET );

	if( ! arg -> isInteger( ) ) {
	    createunit -> printError(
		os, KonoeCommandProcessor::E_ARGUMENT_NOT_INTEGER );
	    return -1;
	}

	KInt	offset = arg -> toInteger( );

	target = new KonoePT4Module( offset );

	return 0;

}

KInt	KonoePT4ModuleOrderly::deleteObject( KonoeCommand * cmd ) {

	return 0;

}

KonoeOrderly	* KonoePT4ModuleOrderly::clone( const KChar * name ) {

	return new KonoePT4ModuleOrderly( name );

}

KonoePT4ModuleOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ShowUnit" );

}

KInt	KonoePT4ModuleOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoePT4Module	* target = ( KonoePT4Module * ) owner -> getTarget( );
	if( target )
	    target -> showStatus( os );

	return 0;

}

KonoePT4ModuleOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "GetUnit" );

}

KInt	KonoePT4ModuleOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	if( n_arg < 2 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_GET_CHIP );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER,
			( KChar * ) arg -> getValue( ) );
	    return -1;
	}

	KInt	chip = arg -> toInteger( );

	if( chip < 0 || chip >= KonoePT4Module::N_FPGA ) {
	    printError( os, E_ARGUMENT_OUT_OF_RANGE,
			( KChar * ) arg -> getValue( ) );
	    return -1;
	}

	arg = cmd -> getArgument( ARG_GET_NAME );

	if( ! arg -> isAlphabetic( ) ) {
	    printError( os, E_ARGUMENT_NOT_ALPHA,
			( KChar * ) arg -> getValue( ) );
	    return -1;
	}

	KChar	* name = ( KChar * ) arg -> getValue( );

	KonoePT4ChipOrderly	* orderly = new KonoePT4ChipOrderly( name );
	KonoeOfficer	* officer;

	KonoePT4Module	* target = ( KonoePT4Module * ) owner -> getTarget( );

	orderly -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( orderly );
	orderly -> setTarget( ( void * ) ( target -> getChip( chip ) ) );

	return 0;

}


