// $Id: KonoeReadoutListOrderly.cc,v 1.4 2002/03/16 08:11:38 iwai Exp $
// $Name: KONOE-0-09-02 $
//===================================================================
/*

	KonoeReadoutListOrderly.cc

	Version 0.0.1	12-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno

*/
//===================================================================

#include	<daq/KonoeReadoutListOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<string.h>
#include	<ui/KonoeHelpCommand.hh>

const KChar	* KonoeReadoutListOrderly::classname = "ReadoutList";


KonoeReadoutListOrderly::KonoeReadoutListOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	createcommand = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	createcommand -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor( "size", INTEGER, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor(
					"recordtype", INTEGER, NECESSARY ) );
	createunit = new CreateUnit( createcommand, this );
	addCommand( createunit );


	showcommand = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	showcommand -> add(
		new KonoeArgumentDescriptor( "section", STRING, OPTIONAL ) );
	showunit = new ShowUnit( showcommand, this );
	addCommand( showunit );

	addcommand = new KonoeCommandDescriptor( "add", N_ARG_ADD );
	addcommand -> add(
		new KonoeArgumentDescriptor( "section", STRING, NECESSARY ) );
	addunit = new AddUnit( addcommand, this );
	addCommand( addunit );


	getcommand = new KonoeCommandDescriptor( "get", N_ARG_GET );
	getcommand -> add(
		new KonoeArgumentDescriptor( "index", INTEGER, NECESSARY ) );
	getcommand -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	getunit = new GetUnit( getcommand, this );
	addCommand( getunit );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeReadoutListOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeReadoutListOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutListOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );
	if( cmd -> getArgumentCount( ) < N_ARG_CREATE ) {
	  os << "Argument missing. Needs table size and record type" << endl;
	  return 0;
	}

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> createObject( cmd );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	return 0;
}

KonoeReadoutListOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeReadoutListOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeReadoutList	* l;

	l = ( KonoeReadoutList * )
		( ( KonoeReadoutListOrderly * ) owner ) -> getTarget( );
	l -> dump( os );

	return 0;
}

KonoeReadoutListOrderly::AddUnit::AddUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeReadoutListOrderly::AddUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	if( cmd -> getArgumentCount( ) < N_ARG_ADD ) {
	  os << "Argument missing. ReadoutSection object required" << endl;
	  return 0;
	}

	KonoeOrderly	* o;
	KonoeOfficer	* officer = owner -> getOfficer( );

	o = officer -> findObject( ( KChar * )
		cmd -> getArgument( ARG_ADD_SECTION ) -> getValue( ) );
	if( o == 0 ) {
	  os << "Object not found = " <<
		cmd -> getArgument( ARG_ADD_SECTION ) -> getValue( )
		<< endl;
	  return 0;
	}

	if( strcmp( o -> getClassName( ), "ReadoutSection" ) == 0 ) {
	  KonoeReadoutList	* list;
	  list = ( KonoeReadoutList * )
		( ( KonoeReadoutListOrderly * ) owner ) -> getTarget( );
	  list -> add( ( KonoeReadoutSection * ) o -> getTarget( ) );
	  return 0;
	}
	else {
	  os << "Object "
		<< cmd -> getArgument( ARG_ADD_SECTION ) -> getValue( )
		<< " is not an instance of ReadoutSection" << endl;
	  return 0;
	}
	return 0;

}

KonoeReadoutListOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutListOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	return 0;
}

KInt	KonoeReadoutListOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_SIZE );
	if( ! arg -> checkInteger( ) ) {
	  os << "Invalid argument type. Integer required" << endl;
	  return -1;
	}

	KInt	size = arg -> toInteger( );

	arg = cmd -> getArgument( ARG_CREATE_RECORDTYPE );
	if( ! arg -> checkInteger( ) ) {
	  os << "Invalid argument type. Integer required" << endl;
	  return -1;
	}

	KInt	recordtype = arg -> toInteger( );
	target = new KonoeReadoutList( size, recordtype );

	return 0;

}

KInt	KonoeReadoutListOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeReadoutListOrderly::clone( const KChar * name ) {

	return new KonoeReadoutListOrderly( name );

}
