// $Id: KonoeReadoutSectionOrderly.cc,v 1.4 2002/03/16 08:11:38 iwai Exp $
// $Name: KONOE-0-09-02 $
//====================================================================
/*

	KonoeReadoutSectionOrderly.cc

	Version 0.0.1	12-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//====================================================================

#include	<daq/KonoeReadoutSectionOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>

const KChar	* KonoeReadoutSectionOrderly::classname = "ReadoutSection";

KonoeReadoutSectionOrderly::KonoeReadoutSectionOrderly( const KChar * name )
	: KonoeOrderly( N_COMMANDS, name ) {

	setClassID( CLASS_ID );

	createcommand = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	createcommand -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor( "size", INTEGER, NECESSARY ) );
	createcommand -> add(
		new KonoeArgumentDescriptor(
					"sectiontype", INTEGER, NECESSARY ) );
	createunit = new CreateUnit( createcommand, this );
	addCommand( createunit );

	showcommand = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	showcommand -> add(
		new KonoeArgumentDescriptor( "segment", STRING, OPTIONAL ) );
	showunit = new ShowUnit( showcommand, this );
	addCommand( showunit );

	addcommand = new KonoeCommandDescriptor( "add", N_ARG_ADD );
	addcommand -> add(
		new KonoeArgumentDescriptor( "segment", STRING, NECESSARY ) );
	addunit = new AddUnit( addcommand, this );
	addCommand( addunit );

	getcommand = new KonoeCommandDescriptor( "get", N_ARG_GET );
	getcommand -> add(
		new KonoeArgumentDescriptor( "index", INTEGER, NECESSARY ) );
	getcommand -> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	getunit = new GetUnit( getcommand, this );
	addCommand( getunit );

	addCommand( new KonoeHelpCommand( 0, this ) );
}

KInt	KonoeReadoutSectionOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeReadoutSectionOrderly::CreateUnit::CreateUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow ) :
		KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutSectionOrderly::CreateUnit::execute(
		KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	if( cmd -> getArgumentCount( ) < N_ARG_CREATE ) {
	  os << "Argument missing. Needs table size and record type" << endl;
	  return 0;
	}
	KonoeOrderly	* o = owner -> clone( ( KChar * )
			cmd -> getArgument( ARG_CREATE_OBJECT )
			-> getValue( ) );
	KonoeOfficer	* officer = owner -> getOfficer( );

	o -> createObject( cmd );
	o -> setOfficer( officer );
	officer -> add( o );
	return 0;

}


KonoeReadoutSectionOrderly::ShowUnit::ShowUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow ) :
		KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutSectionOrderly::ShowUnit::execute(
		KonoeCommand * cmd ) {

	ostream	& os = cmd -> getOutputStream( );
	( ( KonoeReadoutSection * ) owner -> getTarget( ) ) -> dump( os );
	return 0;
}

KonoeReadoutSectionOrderly::AddUnit::AddUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow ) :
		KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutSectionOrderly::AddUnit::execute(
		KonoeCommand * cmd ) {

	return 0;
}

KonoeReadoutSectionOrderly::GetUnit::GetUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow ) :
		KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeReadoutSectionOrderly::GetUnit::execute(
		KonoeCommand * cmd ) {

	return 0;
}

KInt	KonoeReadoutSectionOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_SIZE );
	if( ! arg -> checkInteger( ) ) {
	  os << "Invalid argument type. Integer required" << endl;
	  return -1;
	}

	KInt	size = arg -> toInteger( );

	arg = cmd -> getArgument( ARG_CREATE_SECTIONTYPE );
	if( ! arg -> checkInteger( ) ) {
	  os << "Invalid argument type. Integer required" << endl;
	  return -1;
	}

	KInt	sectiontype = arg -> toInteger( );
	target = new KonoeReadoutSection( size, sectiontype );

	return 0;

}

KInt	KonoeReadoutSectionOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeReadoutSectionOrderly::clone( const KChar * name ) {

	return new KonoeReadoutSectionOrderly( name );

}

