// $Id: KonoeSoftwareCrateOrderly.cc,v 1.5 2002/03/16 08:11:39 iwai Exp $
// $Name: KONOE-0-09-02 $
//====================================================================
/*

	KonoeSoftwareCrateOrderly.cc

	Version 0.0.1	25-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	12-May-2001	H. Sakamoto
	Version 0.6.1	21-May-2001	H. Sakamoto

*/
//====================================================================

#include	<daq/KonoeSoftwareCrateOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<daq/KonoeSoftCounterOrderly.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<daq/KonoeSoftwareCrate.hh>

const KChar	* KonoeSoftwareCrateOrderly::classname = "SoftwareCrate";

KonoeSoftwareCrateOrderly::KonoeSoftwareCrateOrderly( const KChar * objname )
	: KonoeOrderly( N_COMMANDS, objname ) {

	KonoeArgumentDescriptor	* ad;

	setClassID( CLASS_ID );

	createcommand = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	createcommand -> add( ad = new KonoeArgumentDescriptor( "object",
			STRING, NECESSARY ) );
	ad -> setDescription( "Name to be assigned to the created crate" );
	createcommand -> add( ad = new KonoeArgumentDescriptor( "size",
			INTEGER, NECESSARY ) );
	ad -> setDescription( "Number of slots allocated to the crate" );
	createcommand -> setDescription(
		"Create a software crate. Software modules can be installed" );

	createunit = new KonoeCreateCommand( createcommand, this );
	addCommand( createunit );

	showcommand = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	showcommand -> setDescription(
		"Show crate status and installed modules" );
	showunit = new ShowUnit( showcommand, this );
	addCommand( showunit );

	addcommand = new KonoeCommandDescriptor( "add", N_ARG_ADD );
	addcommand -> add( ad = new KonoeArgumentDescriptor( "module",
			STRING, NECESSARY ) );
	ad -> setDescription( "Name of a software module to be installed" );
	addcommand -> setDescription(
		"Install a software module to the crate" );

	addunit = new AddUnit( addcommand, this );
	addCommand( addunit );

	addCommand( new KonoeHelpCommand( 0, this ) );

	initializecommand = new KonoeCommandDescriptor( "initialize",
		N_ARG_INITIALIZE );
	initializecommand -> setDescription(
		"activate 'initialize' method for the crate" );
	initializeunit = new InitializeUnit( initializecommand, this );
	addCommand( initializeunit );

	clearcommand = new KonoeCommandDescriptor( "clear",
		N_ARG_CLEAR );
	clearcommand -> setDescription(
		"activate 'clear' method for the crate" );
	clearunit = new ClearUnit( clearcommand, this );
	addCommand( clearunit );

	updatecommand = new KonoeCommandDescriptor( "update",
		N_ARG_UPDATE );
	updatecommand -> setDescription(
		"activate 'update' method for the crate" );
	updateunit = new UpdateUnit( updatecommand, this );
	addCommand( updateunit );


}

KonoeSoftwareCrateOrderly::AddUnit::AddUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeSoftwareCrateOrderly::AddUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < N_ARG_ADD ) {
	  os << "Argument missing" << endl;
	  return 0;
	}
	KonoeOrderly	* o;
	KonoeOfficer	* officer = owner -> getOfficer( );
	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_ADD_MODULE );

	o = officer -> findObject( ( KChar * ) arg -> getValue( ) );
	if( o == 0 ) {
	  os << "Object not found = " << arg -> getValue( ) << endl;
	  return 0;
	}

	if( o -> getClassID( ) == KonoeSoftCounterOrderly::CLASS_ID ) {
	    KonoeSoftwareCrate	* cr;
	    KonoeSoftCounter	* md;

	    cr = ( KonoeSoftwareCrate * ) owner -> getTarget( );
	    md = ( KonoeSoftCounter * ) o -> getTarget( );
	    cr -> install( md );
	    return 0;

	}
	else {
	  os << "Object is not a SoftwareModule instance" << endl;
	  return 0;
	}
	return 0;

}

KInt	KonoeSoftwareCrateOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_SIZE );
	if( ! arg -> checkInteger( ) ) {
	  os << "Invalid argument type. Integer required" << endl;
	  return -1;
	}

	KInt	size = arg -> toInteger( );

	target = new KonoeSoftwareCrate( size );
cout<<"SoftwareCrate created with size = "<<size<<" : "<<target<<endl;
	return 0;

}

KInt	KonoeSoftwareCrateOrderly::deleteObject( KonoeCommand * cmd ) {

	delete target;

	return 0;

}

KInt	KonoeSoftwareCrateOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOrderly	* KonoeSoftwareCrateOrderly::clone( const KChar * name ) {

	return new KonoeSoftwareCrateOrderly( name );

}

KonoeSoftwareCrateOrderly::ShowUnit::ShowUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow )
	: KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeSoftwareCrateOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	KonoeSoftwareCrate	* cr
		= ( KonoeSoftwareCrate * ) owner -> getTarget( );

	ostream & os = cmd -> getOutputStream( );

	os << "Software crate = " << owner -> getObjectName( ) << endl;
	os << "Number of modules installed = " << cr -> getModuleCount( )
		<< endl;
	
	return 0;

}


KonoeSoftwareCrateOrderly::InitializeUnit::InitializeUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeSoftwareCrateOrderly::InitializeUnit::execute(KonoeCommand* cmd)
{
	KonoeSoftwareCrate * cl = ( KonoeSoftwareCrate * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeSoftwareCrateOrderly::InitializeUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> initialize( );
	return 0;
}

KonoeSoftwareCrateOrderly::ClearUnit::ClearUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeSoftwareCrateOrderly::ClearUnit::execute(KonoeCommand* cmd)
{
	KonoeSoftwareCrate * cl = ( KonoeSoftwareCrate * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeSoftwareCrateOrderly::ClearUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> clear( );
	return 0;
}

KonoeSoftwareCrateOrderly::UpdateUnit::UpdateUnit(
	KonoeCommandDescriptor* cd, KonoeOrderly* ow ) 
  	: KonoeCommandProcessor(cd, ow)
{;}

KInt KonoeSoftwareCrateOrderly::UpdateUnit::execute(KonoeCommand* cmd)
{
	KonoeSoftwareCrate * cl = ( KonoeSoftwareCrate * ) owner -> getTarget( );
	if( cl == 0 ) {

	    ostream & os = cmd -> getOutputStream( );
	    os << "KonoeSoftwareCrateOrderly::UpdateUnit::execute: "
		<< "Null object." << endl;
	    return 0;
	}
	cl -> update( );
	return 0;
}

