// $Id: KonoeVMECrateOrderly.cc,v 1.5 2002/03/16 08:11:40 iwai Exp $
// $Name: KONOE-0-09-02 $
//====================================================================
/*

	KonoeVMECrateOrderly.cc

	Version 0.0.1	24-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	18-May-2001	H. Sakamoto

*/
//====================================================================

#include	<daq/KonoeVMECrateOrderly.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeOfficer.hh>
#include	<KonoeCategories.hh>
#include	<daq/KonoeDataSourceCM.hh>
#include	<daq/KonoePT3Module.hh>
#include	<daq/KonoePT4Module.hh>
#include	<string.h>

const KChar	* KonoeVMECrateOrderly::classname = "VMECrate";

KonoeVMECrateOrderly::KonoeVMECrateOrderly( const KChar * objname )
		: KonoeOrderly( N_COMMANDS, objname ) {

	KonoeArgumentDescriptor	* ad;

	setClassID( CLASS_ID );

// 'create' command

	createcommand = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	createcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"object", STRING, NECESSARY ) );
	ad -> setDescription( "Name of created crate" );
	createcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"mode", STRING, NECESSARY ) );
	ad -> setDescription( "VME access mode. ex., 'A24D16'" );
	createcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"base", INTEGER, NECESSARY ) );
	ad -> setDescription( "Base address of VME" );
	createcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"window", INTEGER, NECESSARY ) );
	ad -> setDescription( "Window size of VME memory" );
	createcommand -> add (
		ad = new KonoeArgumentDescriptor(
			"slots", INTEGER, OPTIONAL ) );
	ad -> setDescription( "Number of slots in the crate" );

	createcommand -> setDescription(
		"Create a VME crate" );

	createunit = new KonoeCreateCommand( createcommand, this );

	addCommand( createunit );

// 'show' command

	showcommand = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	showcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"module", STRING, OPTIONAL ) );
	ad -> setDescription( "Name of VME module to be displayed" );
	showcommand -> setDescription(
		"Show installed modules" );

	showunit = new ShowUnit( showcommand, this );
	addCommand( showunit );

// 'add' command

	addcommand = new KonoeCommandDescriptor( "add", N_ARG_ADD );
	addcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"module", STRING, NECESSARY ) );
	ad -> setDescription( "Name of module to be installed" );
	addcommand -> setDescription(
		"Install a VME module to the crate" );

	addunit = new AddUnit( addcommand, this );
	addCommand( addunit );

// 'get' command

	getcommand = new KonoeCommandDescriptor( "get", N_ARG_GET );
	getcommand -> add(
		ad = new KonoeArgumentDescriptor(
			"index", INTEGER, NECESSARY ) );
	ad -> setDescription( "Index number of required module" );

	getcommand -> add(
		ad = new KonoeArgumentDescriptor(
		"object", STRING, NECESSARY ) );
	ad -> setDescription( "Name to be assigned to the module" );
	getcommand -> setDescription(
		"Enabes to access an installed module" );

	getunit = new GetUnit( getcommand, this );
	addCommand( getunit );

// 'help' command

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeVMECrateOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeVMECrateOrderly::AddUnit::AddUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "AddUnit" );

}

KInt	KonoeVMECrateOrderly::AddUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg;

	if( cmd -> getArgumentCount( ) < 1 ) {
	    printErrorHeader( os );
	    os << "Argument missing" << endl;
	    return -1;
	}

// module argument. must be a VME module.

	arg = cmd -> getArgument( ARG_ADD_MODULE );
	if( ! arg -> isAlphabetic( ) ) {
	    printErrorHeader( os );
	    os << "Invalid argument type" << endl;
	    return -1;
	}

	const KChar * modulename = arg -> getValue( );

	KonoeOrderly	* moduleorderly;

	moduleorderly = owner -> getOfficer( ) -> findObject( modulename );
	if( moduleorderly == 0 ) {
	    printErrorHeader( os );
	    os << "Unknown module object = " << modulename << endl;
	    return -1;
	}

	KInt	cid;
	KonoeVMEModule	* module;

	cid = moduleorderly -> getClassID( );
	if( cid == KonoeDataSourceCM::CID_PT3MODULE +
		KonoeCategories::DATASOURCE ) {

	    module = ( KonoePT3Module * ) moduleorderly -> getTarget( );
	    module -> setCrate( ( KonoeVMECrate * ) owner -> getTarget( ) );
	}
	else if( cid == KonoeDataSourceCM::CID_PT4MODULE +
		KonoeCategories::DATASOURCE ) {
	    module = ( KonoePT4Module * ) moduleorderly -> getTarget( );
	    module -> setCrate( ( KonoeVMECrate * ) owner -> getTarget( ) );

	}
	else {
	    printErrorHeader( os );
	    os << "Not a VME module = " << modulename << endl;
	    return -1;
	}

	return 0;

}


KonoeVMECrateOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ShowUnit" );

}

KInt	KonoeVMECrateOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeVMECrate * target = ( KonoeVMECrate * ) owner -> getTarget( );

	if( target == 0 ) {
	    printErrorHeader( os );
	    os << "VME crate not implemented" << endl;
	    return -1;
	}

	target -> showStatus( os );

	return 0;

}


KonoeVMECrateOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "GetUnit" );

}

KInt	KonoeVMECrateOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	return 0;

}

KInt	KonoeVMECrateOrderly::createObject( KonoeCommand * cmd ) {

// orderly object is already created by using the first argument.

	KonoeCommandArgument	* arg;
	KInt	n_arg = cmd -> getArgumentCount( );
	ostream & os = cmd -> getOutputStream( );
	KInt	vmemode, baseaddress, windowsize, nslots;

	if( n_arg < 4 ) {
	    os << "KonoeVMECrateOrderly::createObject."
			" Argument missing." << endl;
	    return -1;
	}

// VME access mode.

	arg = cmd -> getArgument( ARG_CREATE_MODE );
	const KChar * vmedevname = arg -> getValue( );
	if( strcmp( vmedevname, "A16D16" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A16D16;
	else if( strcmp( vmedevname, "A16D32" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A16D32;
	else if( strcmp( vmedevname, "A24D16" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A24D16;
	else if( strcmp( vmedevname, "A24D32" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A24D32;
	else if( strcmp( vmedevname, "A32D16" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A32D16;
	else if( strcmp( vmedevname, "A32D32" ) == 0 )
	    vmemode = KonoeVMECrate::VME_A32D32;
	else {
	    os << "KonoeVMECrateOrderly::createObject. "
			"Unknown VME access mode = " << vmedevname << endl;
	    return -1;
	}

// Base Address

	arg = cmd -> getArgument( ARG_CREATE_BASE );

	if( arg -> isInteger( ) ) {
	    baseaddress = arg -> toInteger( );
	}
	else {
	    os << "KonoeVMECrateOrderly::createObject. "
			"Invalid argument type" << endl;
	    return -1;
	}


// Window size.

	arg = cmd -> getArgument( ARG_CREATE_WINDOW );

	if( arg -> isInteger( ) ) {
	    windowsize = arg -> toInteger( );
	}
	else {
	    os << "KonoeVMECrateOrderly::createObject. "
			"Invalid argument type" << endl;
	    return -1;
	}

// Slot count ( optional ).

	nslots = DEFAULT_SLOT_COUNT;

	if( n_arg == N_ARG_CREATE ) {

	    arg = cmd -> getArgument( ARG_CREATE_SLOTS );
	    if( arg -> isInteger( ) ) {
		nslots = arg -> toInteger( );
	    }
	    else {
		os << "KonoeVMECrateOrderly::createObject. "
			"Invalid argument type" << endl;
		return -1;
	    }

	}

// create the target

	target = new KonoeVMECrate( vmemode, windowsize, baseaddress, nslots );

	return 0;

}

KInt	KonoeVMECrateOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeVMECrateOrderly::clone( const KChar * name ) {

	return new KonoeVMECrateOrderly( name );

}


