// $Id: KonoeVMEModuleOrderly.cc,v 1.2 2002/03/16 08:11:40 iwai Exp $
// $Name: KONOE-0-09-02 $
//==================================================================
/*

	KonoeVMEModuleOrderly.cc

	Version 0.0.1	18-May-2001	H. Sakamoto

*/
//==================================================================

#include	<daq/KonoeVMEModuleOrderly.hh>

#include	<daq/KonoeVMEModule.hh>
#include	<ui/KonoeCreateCommand.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeCommander.hh>
#include	<ui/KonoeCommandInterpreter.hh>

KonoeVMEModuleOrderly::KonoeVMEModuleOrderly( const KChar * objname,
		KInt n_commands )
		: KonoeOrderly( n_commands, objname ) {

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;
	KonoeCommandProcessor	* cp;

// help command

	addCommand( new KonoeHelpCommand( 0, this ) );

// dump command

	cd = new KonoeCommandDescriptor( "dump", N_ARG_DUMP );
	ad = new KonoeArgumentDescriptor( "offset", INTEGER, OPTIONAL );
	ad -> setDescription( "VME address offset from where dump starts." );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "size", INTEGER, OPTIONAL );
	ad -> setDescription( "Window size to be dumped" );
	cd -> add( ad );
	cd -> setDescription( "Dump VME memory of the module" );
	addCommand( new DumpUnit( cd, this ) );

// write command

	cd = new KonoeCommandDescriptor( "write", N_ARG_WRITE );
	ad = new KonoeArgumentDescriptor( "offset", INTEGER, NECESSARY );
	ad -> setDescription( "Address offset from the module top" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "pattern", INTEGER, NECESSARY );
	ad -> setDescription( "Data pattern to be written" );
	cd -> add( ad );
	cd -> setDescription( "Write 16 bit pattern to the module" );
	addCommand( new WriteUnit( cd, this ) );

// read command

	cd = new KonoeCommandDescriptor( "read", N_ARG_READ );
	ad = new KonoeArgumentDescriptor( "offset", INTEGER, NECESSARY );
	ad -> setDescription( "Address offset from the module top" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "symbol", INTEGER, NECESSARY );
	ad -> setDescription( "Symbol name into where the result is written" );
	cd -> add( ad );
	cd -> setDescription( "Read 16 bit data from the module" );
	addCommand( new ReadUnit( cd, this ) );
	

}

KInt	KonoeVMEModuleOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

void	KonoeVMEModuleOrderly::setTarget( void * ptarget ) {

	target = ( KonoeVMEModule * ) ptarget;

}


KonoeVMEModuleOrderly::DumpUnit::DumpUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "DumpUnit" );

}

KInt	KonoeVMEModuleOrderly::DumpUnit::execute( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream &	os = cmd -> getOutputStream( );
	KonoeCommandArgument	* arg;

	KInt	offset, size;

	offset = 0;
	if( n_arg >= ARG_DUMP_OFFSET + 1 ) {
	    arg = cmd -> getArgument( ARG_DUMP_OFFSET );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER,
			( KChar * ) arg -> getValue( ) );
		return -1;
	    }
	    offset = arg -> toInteger( );
	}

	size = 0;
	if( n_arg >= ARG_DUMP_SIZE + 1 ) {
	    arg = cmd -> getArgument( ARG_DUMP_SIZE );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER,
			( KChar * ) arg -> getValue( ) );
		return -1;
	    }
	    size = arg -> toInteger( );
	}

	KonoeVMEModule	* module = ( KonoeVMEModule * ) owner -> getTarget( );
	module -> dump( os, offset, size );

	return 0;

}

KonoeVMEModuleOrderly::WriteUnit::WriteUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "WriteUnit" );

}

KInt	KonoeVMEModuleOrderly::WriteUnit::execute( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream & os = cmd -> getOutputStream( );

	if( n_arg < N_ARG_WRITE ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_WRITE_OFFSET );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	offset = arg -> toInteger( );

	arg = cmd -> getArgument( ARG_WRITE_PATTERN );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	pattern = arg -> toInteger( );

	KonoeVMEModule	* module = ( KonoeVMEModule * ) owner -> getTarget( );

	module -> write16( offset, pattern );

	return 0;

}

KonoeVMEModuleOrderly::ReadUnit::ReadUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ReadUnit" );

}

KInt	KonoeVMEModuleOrderly::ReadUnit::execute( KonoeCommand * cmd ) {

	KInt	n_arg = cmd -> getArgumentCount( );
	ostream & os = cmd -> getOutputStream( );

	if( n_arg < ARG_READ_OFFSET + 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_READ_OFFSET );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	offset = arg -> toInteger( );

	KonoeVMEModule	* module = ( KonoeVMEModule * ) owner -> getTarget( );

	KShort	pattern = module -> read16( offset );

	if( n_arg >= ARG_READ_SYMBOL + 1 ) {

	    KonoeSymbolResolverList	* resolverlist;

	    resolverlist = ( ( KonoeVMEModuleOrderly * ) owner )
	      -> getOfficer( ) -> getCommander( ) -> getInterpreter( ) -> getResolverList( );

	    if( resolverlist == 0 ) {
		printError( os, E_NO_SYMBOL_RESOLVER );
		return -1;
	    }
	    arg = cmd -> getArgument( ARG_READ_SYMBOL );

	    if( ! arg -> isAlphabetic( ) ) {
		printError( os, E_ARGUMENT_NOT_ALPHA, arg -> getValue( ) );
		return -1;
	    }

	    KonoeSymbol	* symbol = resolverlist -> findSymbol(
					arg -> getValue( ) );

	    if( symbol == 0 )
		symbol = resolverlist -> get( 0 )
			 -> addSymbol( arg -> getValue( ) );

	    symbol -> setIValue( pattern );

	}
	else {

	    os << " @ " << dec << offset << " ( 0x" << hex << offset
		<< " ) = " << dec << pattern << " ( 0x" << hex << pattern
		<< " )"<< endl;

	}

	return 0;

}
