// $Name: KONOE-0-09-02 $
// $Id: demo.cc,v 1.5 2002/03/16 08:11:43 iwai Exp $
// $Id: demo.cc,v 1.5 2002/03/16 08:11:43 iwai Exp $
//
// Hey, where is analysis/KonoePhysIntElement.hh !?
// Let me know, if you have anyway.
//
//===================================================================
/*

	demo.cc at konoe/src/daq/datalist

	Version 0.1	24-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version	0.4.0	11-Sep-1998
	Version 0.5.0   21-Sep-1999     Y. Kouno

*/
//===================================================================

#include <daq/KonoeReadoutList.hh>
#include <daq/KonoeSoftCounter.hh>
#include <daq/KonoeSoftRegister.hh>
#include <daq/KonoeSimTDCModule.hh>
#include <daq/KonoeSoftwareCrate.hh>
#include <daq/KonoeTimerXaction.hh>
#include <analysis/KonoeDataAnalyzer.hh>
#include <analysis/KonoeSectionAnalyzer.hh>
#include <analysis/KonoePhysIntElement.hh>
#include <iostream.h>

//-------------------------------------------------------------------
//	private classes
//-------------------------------------------------------------------

class EventInfoValue : public KonoePhysicalValue {

protected:
  KonoePhysIntElement event_number;

public:
  EventInfoValue( KInt ptag ) : KonoePhysicalValue( ptag ) {;}
  KInt getDataCount() { return 1; }
  void set( KInt pevent_number ) { event_number.set( 0, pevent_number ); }
  KonoePhysValueElement* next() { return & event_number; }
  void dump( ostream& os );
};

class GeneralSectionAnalyzer : public KonoeSectionAnalyzer {

protected:
  EventInfoValue* event_info;

public:
  GeneralSectionAnalyzer( KInt tag );
  KInt analyze( KonoeDataSection* section );
  KonoePhysicalValue* getPhysicalValue() { return event_info; }
};

class TDCValue : public KonoePhysicalValue {

protected:
  KIntn_data;
  KInt storagesize;
  KInt counter;
  KonoePhysIntElement* pointer;
  KonoePhysIntElement* datalist;

public:
  TDCValue( KInt ptag, KInt max_channel );
  KInt getDataCount() 
  { 
    counter = 0;
    pointer = datalist;
    return n_data;
  }
  KonoePhysValueElement* next();
  void clear() { n_data = 0; }
  KInt add( KInt address, KInt data );
  void dump( ostream& os );
};

class TDCSectionAnalyzer : public KonoeSectionAnalyzer {

protected:
  TDCValue* tdcvalue;

public:
  TDCSectionAnalyzer( KInt tag );
  KInt analyze( KonoeDataSection* section );
  KonoePhysicalValue* getPhysicalValue() { return tdcvalue; }
};

//-------------------------------------------------------------------
//	actual application class starts here
//-------------------------------------------------------------------

class Demo {

public:
  enum { RECORD_TYPE = 101 };
  enum { GENERAL_SECTION, TDC_SECTION, N_SECTIONS };
  enum { EVENT_INFO, N_GENERAL_SEGMENTS };
  enum { TDC_1, N_TDC_SEGMENTS };
  enum { N_TDC_CHANNELS = 16 };

protected:
  KonoeReadoutList* list;
  KonoeSoftCounter* event_number_module;
  KonoeSoftRegister* run_number_module;
  KonoeSimTDCModule* tdc_module;
  KonoeSoftwareCrate* crate;
  KonoeTimerXaction* xaction;
  KonoeDataAnalyzer* analyzer;

public:
  Demo( int argc, char** argv );
  ~Demo();
  int go();
};

Demo::Demo( int, char** )
{
  KonoeReadoutSection* psec;
  KonoeReadoutSegment* pseg;
  list = new KonoeReadoutList( N_SECTIONS, RECORD_TYPE );
  list -> 
    add( psec = 
	 new KonoeReadoutSection(N_GENERAL_SEGMENTS, GENERAL_SECTION) );
  psec -> add( pseg = new KonoeReadoutSegment( 1, EVENT_INFO ) );
  event_number_module = new KonoeSoftCounter();
  pseg -> add( new KonoeReadoutItem( event_number_module ) );
  tdc_module = new KonoeSimTDCModule( N_TDC_CHANNELS, 4 );
  list -> add( psec = new KonoeReadoutSection( N_TDC_SEGMENTS, TDC_SECTION ) );
  psec -> add( pseg = new KonoeReadoutSegment( tdc_module, TDC_1 ) );
  crate = new KonoeSoftwareCrate( 2 );
  crate -> install( event_number_module );
  crate -> install( tdc_module );
  crate -> initialize();
  KonoeSignalXaction::initialize();
  xaction = new KonoeTimerXaction( list );
  xaction -> setInterval( 2000 );

  analyzer = new KonoeDataAnalyzer( N_SECTIONS );
  analyzer -> install( new TDCSectionAnalyzer( TDC_SECTION ) );
  analyzer -> install( new GeneralSectionAnalyzer( GENERAL_SECTION ) );
}

Demo::~Demo() 
{
  delete list;
}

int Demo::go()
{
  KonoeDataRecord* rec;
  KonoePhysicalValue* pv;
  KonoePhysValueElement* el;

  list -> dump( cout );
  crate -> clear();
  xaction -> enable();
  for ( KInt i = 0; i < 10; i++ ) {
    rec = xaction -> read();
    rec -> dump( cout );
    analyzer -> analyze( rec );
    //analyzer -> dumpPhysicalValue( cout );
    pv = analyzer -> find( GENERAL_SECTION );
    if ( pv )
      cout << "Event # = " << pv -> next() -> getInteger() << endl;
    else
      cout << "Event Number not found" << endl;
    pv = analyzer -> find( TDC_SECTION );
    if ( pv ) {
      KInt n;
      KInt j;
      n = pv -> getDataCount();
      for ( j = 0; j < n; j++ ) {
	el = pv -> next();
	cout << "Channel = " << el -> getAddress() 
	     <<	" Data = " << el -> getInteger() << endl;
      }
    } else {
      cout << "TDC Data not found" << endl;
    }
    crate -> update( );
  }
  delete rec;
  return 0;
}

//-------------------------------------------------------------------
//	private class members
//-------------------------------------------------------------------

GeneralSectionAnalyzer::GeneralSectionAnalyzer( KInt ptag )
  : KonoeSectionAnalyzer( ptag )
{
  event_info = new EventInfoValue( ptag );
}

KInt GeneralSectionAnalyzer::analyze( KonoeDataSection* section )
{
  KonoeDataSegment* segment;
  KonoeDataElement* element;
  KonoePhysIntElement* value;
  section -> reset();
  while ( segment = section -> next() ) {
    switch ( segment -> getID() ) {
    case Demo::EVENT_INFO:
      segment -> reset();
      element = segment -> next();
      if ( element )
	event_info -> set( *(KInt*) element -> getStore() );
      break;
    default:
      break;
    }
  }
  return 0;
}

void EventInfoValue::dump( ostream& os )
{
  os << "EventInfoValue ";
  event_number.dump( os );
  return;
}

TDCValue::TDCValue( KInt ptag, KInt max_channel )
  : KonoePhysicalValue( ptag )
{
  datalist = new KonoePhysIntElement[ max_channel ];
  storagesize = max_channel;
}

KInt TDCValue::add( KInt paddress, KInt pdata )
{
  if ( n_data >= storagesize )
    return -1;
  ( datalist + n_data ) -> set( paddress, pdata );
  return n_data++;
}

void TDCValue::dump( ostream & os ) 
{
  os << "TDCValue: n_data = " << n_data << endl;
  for ( int i = 0; i < n_data; i++ )
    ( datalist + i ) -> dump( os );
  return;
}

KonoePhysValueElement* TDCValue::next() 
{
  if ( counter >= n_data )
    return 0;
  counter++;
  return pointer ++;
}

TDCSectionAnalyzer::TDCSectionAnalyzer( KInt ptag )
  : KonoeSectionAnalyzer( ptag )
{
  tdcvalue = new TDCValue( ptag, Demo::N_TDC_CHANNELS );
}

KInt TDCSectionAnalyzer::analyze( KonoeDataSection* section )
{
  KonoeDataSegment* segment;
  KonoeDataElement* element;
  section -> reset();
  while ( segment = section -> next() ) {
    switch( segment -> getID() ) {
    case Demo::TDC_1:
      tdcvalue -> clear();
      segment -> reset();
      while( element = segment -> next() ) {
	tdcvalue -> 
	  add( element -> getAddress(), *(KInt*) element -> getStore() );
      }
      break;
    default:
      break;
    }
  }
  return 0;
}

//-------------------------------------------------------------------
//	entry point
//-------------------------------------------------------------------

int main( int argc, char** argv )
{
  Demo demo( argc, argv );
  return demo.go( );
}
