// $Name: KONOE-0-09-02 $
// $Id: KonoeCAMACCrate.cc,v 1.21 2003/01/21 01:38:02 morinaka Exp $
//==================================================================
/*

	KonoeCAMACCrate.cc

	Version 0.0.1	09-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0   06-Mar-2000     G. Iwai
	Version 0.6.1	24-Nov-2000	H. Sakamoto
	add getLAM() member function
	Version 0.6.2   25-Mar-2000     G. Iwai
*/
//==================================================================
#include <daq/KonoeCAMACCrate.hh>
#include <system/KonoeHostSystem.hh>

KonoeCAMACCrate::KonoeCAMACCrate(const KChar* deviceNodeName, KInt idNumber)
  : slotSize(KonoeCAMACCrate::MAX_SLOT), numberOfModule(0)
{
  modules = new KonoeCAMACModule* [ slotSize ];
  card = COPEN(deviceNodeName);
  status = CRESET(card);
  this -> check();
  status = CSETCR(card, idNumber);
  this -> check();
  status = CGENZ(card);
  this -> check();  
}

KonoeCAMACCrate::~KonoeCAMACCrate()
{
  for ( KInt i = 0; i < slotSize; i++ ) {
    if ( !(modules[i]) )
      continue;
    delete modules[i];
  }
  delete [] modules;
  status = CCLOSE(card);
  this -> check();
}

void KonoeCAMACCrate::check() const
{
  if ( this -> status != 0 ) {
    cerr << "# CAMAC : Status Error !" << endl;
    KonoeHostSystem::exit (-1);
  }
  return;
}

KUInt KonoeCAMACCrate::bitPattern(KInt stationNumber)
{
  KUInt mask = getMask(stationNumber);
  mask = ~mask;
  return mask;
}

void KonoeCAMACCrate::waitLAM(KUInt milliSec, KonoeCAMACModule* aModule)
{ 
  KInt stNum = aModule -> getStationNumber();
  this -> status = CWAITLAM( card, milliSec, bitPattern(stNum) ); 
  return;
}

void KonoeCAMACCrate::waitLAM(KUInt milliSec, KonoeCAMACModule* module1, 
			      KonoeCAMACModule* module2)
{ 
  KUInt bit1 = bitPattern( module1 -> getStationNumber() );
  KUInt bit2 = bitPattern( module2 -> getStationNumber() );
  this -> status = CWAITLAM( card, milliSec, (bit1 & bit2) ); 
  return;
}

void KonoeCAMACCrate::waitLAM(KUInt milliSec, KUInt bit)
{ 
  bit = ~bit;
  this -> status = CWAITLAM( card, milliSec, bit ); 
  return;
}

KInt KonoeCAMACCrate::install(KonoeCAMACModule* aModule)
{
  if ( numberOfModule >= slotSize )
    return (-1);
  KInt stNum = aModule -> getStationNumber();
  if ( modules[stNum - 1] )
    return (-1);
  modules[stNum - 1] = aModule;
  numberOfModule++;
  return numberOfModule;
}

void KonoeCAMACCrate::initialize()
{
  for ( KInt i = 0; i < slotSize; i++ ) {
    if ( !(modules[i]) )
      continue;
    modules[i] -> initialize();
  }
  this -> status = CGENZ(this -> card);
  return;
}

void KonoeCAMACCrate::clear()
{ 
  for ( KInt i = 0; i < slotSize; i++ ) {
    if ( !(modules[i]) )
      continue;
    modules[i] -> clear();
  }
  return;
}

void KonoeCAMACCrate::update()
{
  for ( KInt i = 0; i < slotSize; i++ ) {
    if ( !(modules[i]) )
      continue;
    modules[i] -> update();
  }
  return;
}

KInt KonoeCAMACCrate::getLAM()
{
  KInt stationNumber(0);
  unsigned tmp(0);
  this -> status = CGETLAM( this -> card, &tmp );
  while ( stationNumber < slotSize ) {
    if ( tmp & 1 == 1 ) break;
    tmp = tmp >> 1;
    stationNumber++;
  }
  return ( stationNumber + 1 );
}

KInt KonoeCAMACCrate::testLAM( KonoeCAMACModule* aModule )
{
  KInt stNum = aModule -> getStationNumber();

  while(1) {
    this -> d.naf = NAF( this -> card, stNum, 0, 8 );
    CAMAC( card, &d );
    if( d.q ) {
      return 1;
    }
  }
  return 0;
}
