// $Name: KONOE-0-09-02 $
// $Id: KonoeCAMACCrate.hh,v 1.23 2003/01/21 01:38:01 morinaka Exp $
//==================================================================
/*

	KonoeCAMACCrate.hh

	Version 0.0.1	09-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0   06-Mar-2000     G. Iwai
	Version 0.6.1   24-Oct-2000     G. Iwai
	add getLAM() member function
	Version 0.6.2   25-Nov-2000     G. Iwai

*/
//==================================================================
#ifndef	__KONOECAMACCRATE_HH
#define	__KONOECAMACCRATE_HH

#include "camlib.h"

#include <iostream.h>
#include <iomanip.h>

#include <KonoeList.hh>
#include <daq/KonoeDAQCrateBase.hh>
#include <daq/KonoeCAMACModule.hh>

class KonoeCAMACModule;

class KonoeCAMACCrate : public KonoeDAQCrateBase {

private:
  enum { ITEM_WIDTH = 16, VALUE_WIDTH = 10 };

public:
  enum { MAX_SLOT = 24 };

private:
  struct CamacCardDescriptor* card;
  KInt status;
  void check() const;
  KUInt bitPattern(KInt stationNumber);

  KInt slotSize;
  KInt numberOfModule;
  KonoeCAMACModule** modules;
  
public:
  struct CamacData d;

  KonoeCAMACCrate();
  KonoeCAMACCrate(const KChar* deviceNodeName, KInt idNumber = 0);
  ~KonoeCAMACCrate();

  // get/show member functions for outside classes
  inline struct CamacCardDescriptor* getCardDescriptor() const
  { return ( this -> card ); }
  inline KInt getStatus() const
  { return ( this -> status ); }
  inline void showStatus() const
  { 
    cout << setw(ITEM_WIDTH) << "status :" 
	 << setw(VALUE_WIDTH) << this -> getStatus() << endl; 
    return;
  }

  // utilities for users
  inline struct CamacCardDescriptor* open(const KChar* nodeName)
  { return COPEN(nodeName); }
  inline void close()
  { this -> status = CCLOSE(card); }
  inline void setID(KInt idNumber)
  { this -> status = CSETCR(this -> card, idNumber); }
  inline void reset() 
  { this -> status = CRESET(this -> card); }
  inline void setI() 
  { this -> status = CSETI(this -> card); }
  inline void removeI() 
  { this -> status = CREMI(this -> card); }
  inline void enableLAM() 
  { this -> status = CENLAM(this -> card); }
  inline void disableLAM() 
  { this -> status = CDSLAM(this -> card); }

  //  0  : LAM
  // -5  : CC is OFFLINE  / EIO       5      /* I/O error */
  // -11 : other LAM      / EAGAIN   11      /* Try again */
  // -13 : not support    / EACCES   13      /* Permission denied */
  // -22 : time < 0       / EINVAL   22      /* Invalid argument */
  // -62 : time out       / ETIME    62      /* Timer expired */
  void waitLAM(KUInt milliSec, KonoeCAMACModule* aModule);
  void waitLAM(KUInt milliSec, KonoeCAMACModule* module1, 
	       KonoeCAMACModule* module2);
  
  // LAM enable  : 1 / disable : 0
  // If you want to wait LAM from station number 2 or 4 or 5,
  // the value of KInt bit should be set 26.
  void waitLAM(KUInt milliSec, KUInt bit);

  KInt install(KonoeCAMACModule* aModule);
  void update();
  void initialize();
  void clear();

  // return Station Number, its range is 1 to slotSize(<- member)
  KInt getLAM();

  // under construction 
  inline KInt isOnline() { return 0;}

  KInt testLAM( KonoeCAMACModule* aModule );

};

#endif
