// $Name: KONOE-0-09-02 $
// $Id: KonoeCAMACModule.hh,v 1.17 2002/03/16 08:11:47 iwai Exp $
//==================================================================
/*

	KonoeCAMACModule.hh

	Version 0.1	15-Jul-1997	H. Sakamoto
	Version 0.2	16-Sep-1997	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0   06-Mar-2000     G. Iwai

*/
//==================================================================
#ifndef	__KONOECAMACMODULE_HH
#define	__KONOECAMACMODULE_HH

#include "camlib.h"

#include <iostream.h>
#include <iomanip.h>

#include <daq/KonoeHardwareModule.hh>
#include <daq/KonoeCAMACCrate.hh>

class KonoeCAMACCrate;

class KonoeCAMACModule : public KonoeHardwareModule {

protected:
  enum { ITEM_WIDTH = 16, VALUE_WIDTH = 10 };

protected:
  struct CamacCardDescriptor* card;
  struct CamacData data;

  KInt stationNumber;
  KInt numberOfChannel;

protected:
  inline KUInt naf(KInt subAddress, KInt function)
  { return NAF(this -> card, this -> stationNumber, subAddress, function); }
  
public:
  KonoeCAMACModule() {;}
  KonoeCAMACModule(const KonoeCAMACCrate* crate, KInt aStationNumber, 
		   KInt aChannel);
  virtual ~KonoeCAMACModule() {;}

  // set/get/show member functions
  inline void setStationNumber(KInt aStationNumber)
  { this -> stationNumber = aStationNumber; }
  inline KInt getStationNumber() const
  { return ( this -> stationNumber ); }
  inline void showStationNumber() const 
  { 
    cout << setw(ITEM_WIDTH) << "stationNumber :" 
	 << setw(VALUE_WIDTH) << this -> getStationNumber() << endl;
    return;
  }
  inline void setChannel(KInt aChannel) 
  { this -> numberOfChannel = aChannel; }
  inline KInt getChannel() const 
  { return ( this -> numberOfChannel ); }

  inline KShort getQ() const
  { return ( this -> data.q ); }
  inline void showQ() const
  { 
    cout << setw(ITEM_WIDTH) << "data.q :" 
	 << setw(VALUE_WIDTH) << this->getQ() << endl;
    return;
  }

  inline KShort getX() const
  { return ( this -> data.x ); }
  inline void showX() const
  { 
    cout << setw(ITEM_WIDTH) << "data.x :" 
	 << setw(VALUE_WIDTH) << this->getX() << endl; 
    return;
  }

  KInt getDataSize() 
  { return ( this -> numberOfChannel ); }

  virtual KInt fillData( KonoeDataElement* anElement, 
			 KonoeModuleSubaddress* subAddress ) = 0;
  virtual KonoeDataSegment* getDataSegment(KInt segmentTag) = 0;
  virtual void initialize() = 0;
  virtual void clear() = 0;
  virtual void update() = 0;
  
  void dump(ostream& os) 
  { os << "  Module ID = " << this -> stationNumber << endl; }
  
};

#endif
