// $Name: KONOE-0-09-02 $
//=================================================================
/*

	KonoePT3Chip.cc

	Version 0.0.1	14-Apr-2001	H. Sakamoto

	-------------------------------------------

	Spartan chip class for PT3 module

*/
//=================================================================

#include	<daq/KonoePT3Chip.hh>

#include	<unistd.h>
#include	<stdio.h>
#include	<string.h>

KonoePT3Chip::KonoePT3Chip( KonoePT3Module * pmodule, KInt chip ) {

	module = pmodule;
	chip_number = chip;
	address_offset = KonoePT3Module::S_FPGA0_BASE
		+ chip_number * KonoePT3Module::W_FPGA_ADDRESS;
	bit_mask = 1 << chip_number;

}

void	KonoePT3Chip::showStatus( ostream & os ) {

	KShort	prog, init, done;
	prog = module -> read16( KonoePT3Module::S_PROGRAM );
	init = module -> read16( KonoePT3Module::S_INIT );
	done = module -> read16( KonoePT3Module::S_DONE_HDC );

	os << "PROGRAM_ = " << ( ( ( 0xff & prog ) >> chip_number ) % 2 )
	   << "  INIT_ = " << ( ( ( 0xff & init ) >> chip_number ) % 2 )
	   << "  HDC = " << ( ( ( 0xff & done ) >> chip_number ) % 2 )
	   << "  DONE = " << ( ( ( 0xff & done ) >> 4 + chip_number ) % 2 )
	   << endl;

}

KInt	KonoePT3Chip::getPinState( KInt pin_id ) {

	KShort	s;

	switch( pin_id ) {
	case PIN_PROGRAM :
	    s = ( ( 0xff & module -> read16( KonoePT3Module::S_PROGRAM ) )
		>> chip_number ) % 2;
	    break;
	case PIN_INIT :
	    s = ( ( 0xff & module -> read16( KonoePT3Module::S_INIT ) )
		>> chip_number ) % 2;
	    break;
	case PIN_HDC :
	    s = ( ( 0xff & module -> read16( KonoePT3Module::S_DONE_HDC ) )
		>> chip_number ) % 2;
	    break;
	case PIN_DONE :
	    s = ( ( 0xff & module -> read16( KonoePT3Module::S_DONE_HDC ) )
		>> chip_number + 4 ) % 2;
	    break;
	default :
	    s = 0;
	    break;
	}
	return s;
}

void	KonoePT3Chip::setPinState( KInt pin_id, KInt flag ) {

	switch( pin_id ) {
	case PIN_PROGRAM :
	    module -> write16( KonoePT3Module::S_PROGRAM,
			~( ( flag & 1 ) << chip_number ) );
	    break;
	default :
	    break;
	}

}

void	KonoePT3Chip::erase( ) {

	module -> write16( KonoePT3Module::S_PROGRAM,
			~( 0x01 << chip_number ) );
	usleep( 100 * 1000 );	// 100 msec
	module -> write16( KonoePT3Module::S_PROGRAM, 0xffff );

}

KInt	KonoePT3Chip::configure( const KChar * rbtfile ) {

	enum	{	BUFFERSIZE = 512	};
	FILE	* fp;
	KChar	buffer[ BUFFERSIZE ];
	KInt	l, n, k;
	KChar	* p, c;
	KShort	v;

	setPinState( PIN_PROGRAM, 1 );
	usleep( 1000 );
	setPinState( PIN_PROGRAM, 0 );

	while( getPinState( PIN_INIT ) == 0 ) {
	    cout << "INIT pin is low. waiting to become high" << endl;
	}

	fp = fopen( rbtfile, "r" );
	if( fp == 0 ) {
	    cerr << "File open failure = " << rbtfile << endl;
	    return -1;
	}

	n = 0;
	k = 0;
	while( p = fgets( buffer, BUFFERSIZE, fp ) ) {

	    l = strlen( buffer );
	    p = buffer;
	    if( l != 0 && ( * p == '0' || * p == '1' ) ) {

		while( c = * p ++ ) {
		    if( c == '0' || c == '1' ) {
			v = c - '0';
			module -> write16(
				KonoePT3Module::S_CFG_DATA
				+ chip_number * 2, v );
			k ++;
			if( k % CHECK_INTERVAL == 0 ) {
			    if( getPinState( PIN_INIT ) == 0 ) {
				cerr << "Configuration error at " << k << endl;
				fclose( fp );
				return -1;
			    }
			}
		    }
		    else if( c == ' ' || c == '\t' || c == '\n' ) {
		    }
		    else {
			cerr << "Invalid RBT format. Detected = "
				<< -- p << endl;
			fclose( fp );
			return -1;
		    }


		}
	    }
	    else
		cout << buffer;
	    n ++;
	}

	if( getPinState( PIN_INIT ) == 0 ) {
	    cerr << "Configuration error. INIT is low" << endl;
	    fclose( fp );
	    return -1;
	}
	fclose( fp );
	return 0;

}

void	KonoePT3Chip::setPCLKMode( KInt mode ) {

	module -> write16( KonoePT3Module::S_PCLK_SELECT
			+ chip_number * 2, mode );

}

KInt	KonoePT3Chip::getPCLKMode( ) {

	return module -> read16( KonoePT3Module::S_PCLK_SELECT
				+ chip_number * 2 );

}

void	KonoePT3Chip::setSCLKMode( KInt mode ) {

	module -> write16( KonoePT3Module::S_SCLK_SELECT
			+ chip_number * 2, mode );

}

KInt	KonoePT3Chip::getSCLKMode( ) {

	return module -> read16( KonoePT3Module::S_SCLK_SELECT
				+ chip_number * 2 );

}
