// $Name: KONOE-0-09-02 $
//=================================================================
/*

	KonoePT4Chip.cc

	Version 0.0.1	20-Apr-2001	H. Sakamoto
	Version 0.0.3	10-Jul-2001	H. Sakamoto

	-------------------------------------------

	PT4 FPGA chip class

*/
//=================================================================

#include	<daq/KonoePT4Chip.hh>
#include	<daq/KonoePT4Module.hh>

#include	<unistd.h>
#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<iostream.h>

KonoePT4Chip::KonoePT4Chip( KonoePT4Module * pmodule, KInt index ) {

	module = pmodule;
	chip_number = index;
	address_offset = KonoePT4Module::L_FPGA0 +
		chip_number * KonoePT4Module::WL_FPGA;
	config_port = 0;

}

void	KonoePT4Chip::set_address( ) {

	KInt	* p;
	p = ( KInt * ) module -> getPointer( );

	if( p ) {
	    config_port = p + ( KonoePT4Module::L_CFG0 + chip_number );
	    data_port = p + KonoePT4Module::L_CFGDATA;
	    base_port = p + address_offset;
	    clock_port = p + ( KonoePT4Module::L_CLK0 + chip_number );
	    jtag_port = p + ( KonoePT4Module::L_JTAG0 + chip_number );
	}
	else {
	    cerr << "KonoePT4Chip::set_address. "
		"VME address not available" << endl;
	}
}

KInt	KonoePT4Chip::configure( const KChar * filename ) {

	if( config_port == 0 )
	    set_address( );

// start programming

	* config_port = B_CFGCS | B_CFGWRITE;

// set the device ready to configure

	* config_port = B_CFGCS | B_CFGWRITE | B_CFGPRG;

// wait for INIT_ goes high. ( end of initialization )

	while( 1 ) {
	    if( * config_port & B_CFGINIT )
		break;
	    usleep( 1000 );	// 1 milisec
	}

// select the device

	* config_port = B_CFGPRG;

// download a file.

	KInt	fd, i, l, k, d, e;
	const KInt	BUFFERSIZE = 512;
	KChar	buffer[ BUFFERSIZE ];
	KChar	* p;

	fd = open( filename, O_RDONLY );
	if( fd == -1 ) {
	    perror( 0 );
	    return -1;
	}

	l = read( fd, buffer, BUFFERSIZE );
	if( l <= 0 ) {
	    cerr << "File read error" << endl;
	    close( fd );
	    return -1;
	}

	p = buffer;
	for( k = 0; k < l; k ++ ) {
	    if( * p == '\377' )
		break;
	    p ++;
	}
	if( k == l ) {
	    cerr << "Invalid file format" << endl;
	    close( fd );
	    return -1;
	}

	while( 1 ) {
	    d = 0;
	    e = 1;
	    for( i = 0; i < 8; i ++ ) {		// bit swap
		d |= ( * p & e ) ? 1 : 0;
		if( i != 7 ) {
		    e <<= 1;
		    d <<= 1;
		}
	    }
	    * ( data_port ) = d;
	    p ++;
	    k ++;
	    if( k == l ) {
		l = read( fd, buffer, BUFFERSIZE );
		if( l < 0 ) {
		    cerr << "Error while reading file" << endl;
		    close( fd );
		    return -1;
		}
		else if( l == 0 ) {
		    break;
		}
		k = 0;
		p = buffer;
	    }
	}

	close( fd );

	* config_port = B_CFGCS | B_CFGWRITE | B_CFGPRG;

	if( ( * config_port & B_CFGDONE ) == 0 ) {
	    cerr << "DONE not set" << endl;
	    return -1;
	}

	return 0;

}

KInt	KonoePT4Chip::verify( const KChar * filename ) {

	if( config_port == 0 )
	    set_address( );

// select the device

	* config_port = B_CFGPRG;

// download a file.

	KInt	fd, i, l, k, d, e, c;
	const KInt	BUFFERSIZE = 512;
	KChar	buffer[ BUFFERSIZE ];
	KChar	* p;

	fd = open( filename, O_RDONLY );
	if( fd == -1 ) {
	    perror( 0 );
	    return -1;
	}

	l = read( fd, buffer, BUFFERSIZE );
	if( l <= 0 ) {
	    cerr << "File read error" << endl;
	    close( fd );
	    return -1;
	}

// skip headers

	p = buffer;
	for( k = 0; k < l; k ++ ) {
	    if( * p == '\377' )
		break;
	    p ++;
	}
	if( k == l ) {
	    cerr << "Invalid file format" << endl;
	    close( fd );
	    return -1;
	}

// CLB readout

	KInt	commandout, bytecount, totalbytes, diffs;

	commandout = 32;
	totalbytes = 0;
	bytecount = 0;
	diffs = 0;

	while( 1 ) {

	  if( commandout > 0 ) {
	    d = 0;
	    e = 1;
	    for( i = 0; i < 8; i ++ ) {		// bit swap
		d |= ( * p & e ) ? 1 : 0;
		if( i != 7 ) {
		    e <<= 1;
		    d <<= 1;
		}
	    }
	    * ( data_port ) = d;
	    if( commandout < 4 ) {
		totalbytes <<= 8;
		totalbytes += ( ( KInt ) * p ) & 0xff;
	    }
	    commandout --;
	    if( commandout == 0 ) {
		cout << "Readout command sent. bytecount = "
		<< totalbytes << endl;
		* config_port = B_CFGWRITE | B_CFGPRG;
	    }
	  }
	  else {	// readback
	    c = * ( data_port ) & 0xff;
	    d = 0;
	    e = 1;
	    for( i = 0; i < 8; i ++ ) {
		d |= ( c & e ) ? 1 : 0;
		if( i != 7 ) {
		    e <<= 1;
		    d <<= 1;
		}
	    }
	    if( * p != d ) {
		cout << "Difference found at " << bytecount
		<< ": read = " << * p << " write = " << d << endl;
		diffs ++;
	    }
	    bytecount ++;
	    if( bytecount >= totalbytes )
		break;
	  }
	    p ++;
	    k ++;
	    if( k == l ) {
		l = read( fd, buffer, BUFFERSIZE );
		if( l < 0 ) {
		    cerr << "Error while reading file" << endl;
		    close( fd );
		    return -1;
		}
		else if( l == 0 ) {
		    break;
		}
		k = 0;
		p = buffer;
	    }
	}

	cout << "Total " << totalbytes << " bytes read-back and found "
		<< diffs << " differences." << endl;

	close( fd );

	* config_port = B_CFGCS | B_CFGWRITE | B_CFGPRG;

	return 0;

}

void	KonoePT4Chip::showStatus( ostream & os ) {

	if( config_port == 0 )
	    set_address( );

	KInt	r = 0xffL & * config_port;

	os << "PT4 Chip # " << chip_number << " CFG= " << hex << r;
	os << " CS_= " << ( ( r & B_CFGCS ) ? '1' : '0' );
	os << " WRITE_= " << ( ( r & B_CFGWRITE ) ? '1' : '0' );
	os << " PRG_= " << ( ( r & B_CFGPRG ) ? '1' : '0' );
	os << " DONE= " << ( ( r & B_CFGDONE ) ? '1' : '0' );
	os << " INIT_= " << ( ( r & B_CFGINIT ) ? '1' : '0' );
	os << " BUSY= " << ( ( r & B_CFGBUSY ) ? '1' : '0' );
	os << endl;

}

KInt	KonoePT4Chip::setClockMode( KInt clockline, KInt clockmode ) {

	if( clockline < 0 || clockline > 3 )
	    return -1;	// GCLK0-GCLK3 for Virtex FPGA
	KInt	mode = clockmode << ( clockline * 2 );
	KInt	mask = 3 << ( clockline * 2 );
	KInt	currentvalue;

	if( config_port == 0 )
	    set_address( );

	mask = ~mask;
	currentvalue = * clock_port;
	currentvalue &= mask;
	currentvalue |= mode;
	* clock_port = currentvalue;

	return 0;

}

void	KonoePT4Chip::resetTestLogic( ) {

	if( config_port == 0 )
	    set_address( );

	for( KInt i = 0; i < 5; i ++ ) {
	    * jtag_port = B_JTAG_TMS;
	    * jtag_port = B_JTAG_TMS | B_JTAG_TCK;
	}
	* jtag_port = B_JTAG_TMS;
	jtag_state = S_JTAG_TEST_LOGIC_RESET;

}

KInt	KonoePT4Chip::getIDCode( ) {

	if( config_port == 0 )
	    set_address( );

	resetTestLogic( );

	goto_capture_ir( );

	set_instruction( JTAG_IDCODE );

	goto_capture_dr( );

	* jtag_port = 0;
	* jtag_port = B_JTAG_TCK;

	KInt	id = 0, l = 1;
	for( KInt i = 0; i < 32; i ++ ) {
	    * jtag_port = 0;	// shift-dr
	    if( * jtag_port & B_JTAG_TDO )
		id |= l;
	    if( i < 31 ) {
		l <<= 1;
	    }
	    * jtag_port = B_JTAG_TCK;
	}

	resetTestLogic( );

	return id;

}

void	KonoePT4Chip::goto_capture_ir( ) {

	* jtag_port = 0;	// to run-test/idle
	* jtag_port = B_JTAG_TCK;

	* jtag_port = B_JTAG_TMS;	// to select dr
	* jtag_port = B_JTAG_TMS | B_JTAG_TCK;

	* jtag_port = B_JTAG_TMS;	// to select ir
	* jtag_port = B_JTAG_TMS | B_JTAG_TCK;

	* jtag_port = 0;	// to capture-ir
	* jtag_port = B_JTAG_TCK;

	jtag_state = S_JTAG_CAPTURE_IR;

}

void	KonoePT4Chip::goto_capture_dr( ) {

	switch( jtag_state ) {
	case S_JTAG_TEST_LOGIC_RESET :
	    * jtag_port = 0;
	    * jtag_port = B_JTAG_TCK;
	case S_JTAG_UPDATE_DR :
	case S_JTAG_UPDATE_IR :
	case S_JTAG_RUN_TEST_IDLE :
	    * jtag_port = B_JTAG_TMS;
	    * jtag_port = B_JTAG_TMS | B_JTAG_TCK;
	case S_JTAG_SELECT_DR_SCAN :
	    * jtag_port = 0;
	    * jtag_port = B_JTAG_TCK;
	}
	jtag_state = S_JTAG_CAPTURE_DR;

}

void	KonoePT4Chip::set_instruction( KInt ins ) {

	KInt	j = ins;

	for( KInt i = 0; i < JTAG_IR_LENGTH; i ++ ) {

	    if( j & 0x10 ) {
		* jtag_port = B_JTAG_TDI;	// 1.
		* jtag_port = B_JTAG_TDI | B_JTAG_TCK;
	    }
	    else {
		* jtag_port = 0;		// 0.
		* jtag_port = B_JTAG_TCK;
	    }
	    j <<= 1;
	}

	* jtag_port = B_JTAG_TMS;	// to exit1-ir
	* jtag_port = B_JTAG_TMS | B_JTAG_TCK;

	* jtag_port = B_JTAG_TMS;	// to update-ir
	* jtag_port = B_JTAG_TMS | B_JTAG_TCK;

	* jtag_port = B_JTAG_TMS;	// to select-dr-scan
	* jtag_port = B_JTAG_TMS | B_JTAG_TCK;

	jtag_state = S_JTAG_SELECT_DR_SCAN;

}

void	KonoePT4Chip::bypass( ) {

	resetTestLogic( );

	goto_capture_ir( );

	set_instruction( JTAG_BYPASS );

}

void	KonoePT4Chip::setInstruction( KInt instruction ) {

	resetTestLogic( );

	goto_capture_ir( );

	set_instruction( instruction );

}

void	KonoePT4Chip::shiftDR( const KChar * input, KChar * output,
		KInt nbits ) {

	goto_capture_dr( );

	KInt	i, k, nbytes = ( nbits - 1 ) / 8 + 1;
	KChar	ci, co, * q = output;
	const KChar	* p = input;

	for( i = 0; i < nbits; i ++ ) {
	    if( i % 8 == 0 ) {
		k = 1;
		ci = * p ++;
		if( i != 0 )
		    * q ++ = co;
		co = 0;
	    }
	    else
		k <<= 1;
	    if( ci & k ) {
		* jtag_port = B_JTAG_TDI;
		* jtag_port = B_JTAG_TDI | B_JTAG_TCK;
	    }
	    else {
		* jtag_port = 0;
		* jtag_port = B_JTAG_TCK;
	    }
	    * jtag_port = 0;
	    if( * jtag_port & B_JTAG_TDO )
		co |= k;
	}
	if( i % 8 ) {
	    * q = co;
	}
	
}
