// $Name: KONOE-0-09-02 $
// $Id: KonoeSimADCModule.cc,v 1.7 2002/03/16 08:11:55 iwai Exp $
//=========================================================================
/*

	KonoeSimADCModule.cc
	     
	Version 0.1  	12-Feb-1998	K.Ohnishi
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno

*/
//=========================================================================
#include <daq/KonoeSimADCModule.hh>
#include <stdlib.h>
#include <math.h>

KonoeSimADCModule::KonoeSimADCModule(KInt pnchannels, KInt pnaveragehits,
				     KInt pfullscale)
  : KonoeSoftwareModule(SIM_ADC_MODULE) 
{
  n_channels = pnchannels;
  n_average_hits = pnaveragehits;
  fullscale = pfullscale;
  mean = new KInt[n_channels];
  sigma = new KInt[n_channels];
  
  for ( KInt i = 0; i < n_channels; i++ ) {
    mean[i] = lrand48() % fullscale;
    sigma[i] = (lrand48() % fullscale) / 10;
  }
}

KonoeSimADCModule::KonoeSimADCModule(KInt pnchannels, KInt pnaveragehits,
				     KInt* pmean, KInt* psigma, 
				     KInt pfullscale)
  : KonoeSoftwareModule(SIM_ADC_MODULE) 
{
  n_channels = pnchannels;
  n_average_hits = pnaveragehits;
  
  mean = new KInt [n_channels];
  sigma = new KInt [n_channels];
    
  for ( KInt i = 0; i < n_channels; i++ ) {
    mean[i] = pmean[i];
    sigma[i] = psigma[i];
  }
    
  fullscale = pfullscale;
}

KonoeSimADCModule::~KonoeSimADCModule()
{
  delete [] mean;
  delete [] sigma;
}

KInt KonoeSimADCModule::getDataSize()
{
  return n_channels;
}

KInt KonoeSimADCModule::fillData(KonoeDataElement* pelement,
				 KonoeModuleSubaddress* ) 
{
  KInt data = 1024;
  pelement -> fill( &data, 1 );
  return (data);
}

void KonoeSimADCModule::dump( ostream& os ) 
{
  return;
}

void KonoeSimADCModule::initialize()
{
  return;
}

void KonoeSimADCModule::clear()
{
  return;
}

void KonoeSimADCModule::update()
{
  return;
}

KonoeDataSegment * KonoeSimADCModule::getDataSegment(KInt psegmenttag) 
{
  KonoeDataSegment* r;
  KonoeDataElement* e;
  KInt s, t, u;
  
  t = 0x10000 * n_average_hits / n_channels;
  r = new KonoeDataSegment(n_channels, psegmenttag);
  for ( KInt i = 0; i < n_channels; i++ ) {
    u = lrand48() & 0xffff;
    if ( u < t ) {
      s = (KInt) ( mean[i] + sigma[i] * gaussDev() );
      r -> add( new KonoeDataElement(KonoeDataElement::INT, i, s) );
    }
  }
  return r;
}

KFloat KonoeSimADCModule::gaussDev()
{
  static KInt has_random = 0;
  static KFloat ran1 = 0;
  KFloat ran2;
  KFloat a1;
  KFloat a2;

  if ( has_random ) {
    has_random = 0;
    return ran1;
  } else {
    a1 = drand48();
    a2 = drand48();
    ran1 = sqrt(-2.0 * log(a1)) * cos(a2);
    ran2 = sqrt(-2.0 * log(a1)) * sin(a2);
    return ran2;
  }
}
