// $Name: KONOE-0-09-02 $
//=====================================================================
/*

	KonoeSuehiroChip.cc

	Version 0.0.1	31-Aug-1998	H. Sakamoto

*/
//====================================================================

#include	<daq/KonoeSuehiroChip.hh>
#include	<iostream.h>
#include	<fstream.h>

KonoeSuehiroChip::KonoeSuehiroChip( KInt pid ) {

	id = pid;
	mask = 1 << id;
	address = 0;	// means not set.
}

void	KonoeSuehiroChip::setBase( void * baseaddress ) {

	modulebase = ( KShort * ) baseaddress;
	chipbase = modulebase + OFS_CHIP0 + id * SIZE_CHIP;

	lca_reset = modulebase + OFS_RESET;
	lca_program = modulebase + OFS_PROGRAM;
	lca_hdc = modulebase + OFS_HDC;
	lca_ldc = modulebase + OFS_LDC;
	lca_busy = modulebase + OFS_BUSY;
	lca_init = modulebase + OFS_INIT;

	lca_prgbyte = chipbase + OFS_PRGBYTE;

	clock_mask = modulebase + OFS_CLKMASK + id;
	clock_strobe = modulebase + OFS_CLKVME;
	clock_enable = modulebase + OFS_CLKENABLE;

	address = 1;

}

void	KonoeSuehiroChip::put_frame( KChar * p ) {

	KInt	i;

	while( * p ) {
	  if( * p == '1' )
	    ibyte |= ishift;
	  if( ishift == 0x80 ) {
	    * lca_prgbyte = KShort( ibyte & 0xff );
	    for( i = 0; i < 0xfff; i ++ ) { }
	    ibyte = 0;
	    ishift = 1;
	  }
	  else
	    ishift <<= 1;
	  p ++;
	}

}

KInt	KonoeSuehiroChip::download( KChar * filename ) {

	if( ! address ) {
	  cout << "No valid address set" << endl;
	  return -1;
	}


	ifstream ifs( filename );
	if( ifs.bad( ) ) {
	  cout << "File open failre = " << filename << endl;
	  return -1;
	}

	initialize( );

	KChar	buf[ 256 ];
	KChar	* p;
	KInt	i;

	ibyte = 0;
	ishift = 1;
	i = 0;
	while( 1 ) {
	  ifs.getline( buf, 256 );
	  if( ifs.eof( ) )
	    break;
	  p = buf;
	  if( * p == '0' ) {
	    put_frame( p );
	    cout << ".";
	    i ++;
	    if( i >= 75 ) {
	      cout << endl;
	      i = 0;
	    }
	  }
	  else if( * p == '1' ) {
	    put_frame( p );
	    cout << "*";
	    i ++;
	    if( i >= 75 ) {
	      cout << endl;
	      i = 0;
	    }
	  }
	  else {
	    cout << buf << endl;
	  }
	}
	put_frame( "11111111" );
	dumpCSR( cout );
	return 0;
}

KInt	KonoeSuehiroChip::initialize( ) {

	if( ! address ) {
	  cout << "No valid address set" << endl;
	  return -1;
	}

	* lca_reset = mask;
	* lca_program = mask;
	for( KInt i = 0; i < 1; i ++ ) { }	// no-op.
	* lca_reset = 0;
	* lca_program = 0;
	while( * lca_init & mask != mask ) {
	  cout << "Initializing..." << endl;
	}
	cout << "Initialization done" << endl;

	return 0;
}

void	KonoeSuehiroChip::dumpCSR( ostream & os ) {

	os << "Reset = " << ( ( * lca_reset & mask ) ? '1' : '0' );
	os << "  Dn/Prg = " << ( ( * lca_program & mask ) ? '1' : '0' );
	os << "  HDC = " << ( ( * lca_hdc & mask ) ? '1' : '0' );
	os << "  LDC = " << ( ( * lca_ldc & mask ) ? '1' : '0' );
	os << "  Busy = " << ( ( * lca_busy & mask ) ? '1' : '0' );
	os << "  Init = " << ( ( * lca_init & mask ) ? '1' : '0' );
	os << endl;
}

void	KonoeSuehiroChip::dump( ostream & os ) {

	KInt	i;
	KShort	* p;

	p = chipbase;
	for( i = 0; i < SIZE_CHIP; i ++ ) {
	  os.width( 4 );
	  os << ( ( * p ++ ) & 0xffff );
	  if( i == 7 )
	    os << endl;
	  else
	    os << ' ';
	}
}

void	KonoeSuehiroChip::setClockMode( KInt mode ) {

	if( mode ) {	// internal
	  * clock_mask = MASK_INTERNAL;
	}
	else {	// external
	  * clock_mask = MASK_EXTERNAL;
	}

}

void	KonoeSuehiroChip::enableClock( KInt flag ) {

	* clock_enable = flag;

}

void	KonoeSuehiroChip::strobeClock( ) {

	* clock_strobe = 0xff;
	* clock_strobe = 0x00;

}
