// $Id: KonoeTimerModule.cc,v 1.5 2002/04/09 22:47:48 iwai Exp $
// $Name: KONOE-0-09-02 $
//====================================================================
/*

	KonoeTimerModule.cc

        Version 0.6.0   09-Nov-2000     G. Iwai

*/
//====================================================================
#include <sys/time.h>
#include <sys/timeb.h>
#include <time.h>
#include <string.h>
#include <daq/KonoeTimerModule.hh>

KonoeTimerModule::KonoeTimerModule( KUnit_t unit )
  : KonoeSoftwareModule( KonoeTimerModule::MODULE_TYPE ), 
    theCurrent( 0 ), theInterval( 0 ), thePrevious( 0 ), theUnit( unit )
{
  setCurrent();
}

KInt KonoeTimerModule::fillData( KonoeDataElement* anElement,
				 KonoeModuleSubaddress* subAddress) 
{
  anElement -> fill( (KInt*)(&theInterval), 1 );
  return( 1 );
}

void KonoeTimerModule::initialize()
{
  theInterval = 0;
  theCurrent = 0;
  thePrevious = 0;
  return;
}

void KonoeTimerModule::clear()
{
  this -> theInterval = 0;
  return;
}

void KonoeTimerModule::update()
{
  thePrevious = theCurrent;
  setCurrent();
  theInterval = theCurrent - thePrevious;
  return;
}


void KonoeTimerModule::setCurrent()
{
  if ( theUnit == KonoeTimerModule::kSecond ) {
    time_t now;
    time( &now );
    theCurrent = ( KULong )( now );
  } else if ( theUnit == KonoeTimerModule::kMilliSecond ) {
    struct timeb now;
    ftime( &now );
    theCurrent = ( KULong )( now.time * 1000 + now.millitm );
  } else if ( theUnit == KonoeTimerModule::kMicroSecond ) {
    struct timeval now;
    gettimeofday( &now, 0 );
    theCurrent = ( KULong )( now.tv_sec * 1000000 + now.tv_usec );
  }

  return;
}

void KonoeTimerModule::dump( ostream& os ) const
{
  KChar* unit = NULL;
  switch ( theUnit ) {
  case kSecond:
    unit = new KChar[ strlen( "sec" ) + 1 ];
    strcpy( unit, "sec" );
  case kMilliSecond:
    unit = new KChar[ strlen( "msec" ) + 1 ];
    strcpy( unit, "msec" );
  case kMicroSecond:
    unit = new KChar[ strlen( "usec" ) + 1 ];
    strcpy( unit, "usec" );
  }

  os << "  KonoeTimerModule" << endl;
  os << "    curr. " << theCurrent << " [" << unit << "],";
  os << "  prev. " << thePrevious << " [" << unit << "],";
  os << "  intr. " << theInterval << " [" << unit << "]" << endl;

  delete [] unit;

  return;
}
