// $Name: KONOE-0-09-02 $
// $Id: KonoeVMECrate.cc,v 1.14 2002/03/16 08:12:01 iwai Exp $
//====================================================================
/*

	KonoeVMECrate.cc

	Version 0.0.1	09-Jul-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	13-Apr-2001	H. Sakamoto

*/
//====================================================================

#include <daq/KonoeVMECrate.hh>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/mman.h>

#include	<iostream.h>

#ifdef KONOE_SBS_617_KINOKO
KChar	* KonoeVMECrate::device[] =
{
	"/dev/vmedrv16d16", "/dev/vmedrv16d32", "/dev/vmedrv24d16",
	"/dev/vmedrv24d32", "/dev/vmedrv32d16", "/dev/vmedrv32d32"
};
#else
KChar	* KonoeVMECrate::device[] =
{
	"/dev/vme16d16", "/dev/vme16d32", "/dev/vme24d16",
	"/dev/vme24d32", "/dev/vme32d16", "/dev/vme32d32"
};
#endif

KonoeVMECrate::KonoeVMECrate( KInt pmode, KInt msize, 
			      KInt poffset, KInt ptablesize ) 
  : KonoeList<KonoeVMEModule>( ptablesize ) 
{
  mode = pmode;
  offset = poffset;

#ifdef	KONOE_SBS_617_KINOKO
	cout<<"SBS BIT3 617 KINOKO version"<<endl;
#elifdef	KONOE_SBS_616_VMEHB
	cout<<"SBS BIT3 616 NIKHEF VMEHB version"<<endl;
#endif

	cout << "Opening VME device = " << device[ mode ] << endl;

	if( device[ mode ] == 0 ) {
	    cout << "No real VME device. Mode set to OFFLINE" <<endl;
	    status = OFFLINE;
	    return;
	}

  filed = open( device[ mode ], O_RDWR | O_CREAT );
  if ( filed == -1 ) {
    cout << "Error opening VME device " << device[ mode ] << endl;
    status = OFFLINE;
    return;
  }

  size = ( unsigned long ) msize;
  pointer = ( caddr_t ) mmap( 0, size, ( PROT_READ | PROT_WRITE ), 
		  MAP_SHARED, filed, offset );
  if( pointer == ( caddr_t ) -1 ) {
    cout << "Error mapping to VME space" << endl;
    status = OFFLINE;
    close( filed );
	filed = -1;
    return;
  }
  status = ONLINE;
}

void	KonoeVMECrate::showStatus( ostream & os ) {

	if( device[ 0 ] == 0 ) {
	    os << "VME is not supported in this program" << endl;
	    return;
	}

	if( filed == -1 ) {
	    os << "VME device is not opened" << endl;
	    return;
	}

	os << "VME device : " << device[ mode ] << " is open on fd = "
		<< filed << endl;
	os << "    Base address offset = " << hex << offset
		<< " with memory window = " << size << dec << endl;
	os << "    " << getCount( ) << " modules installed" << endl;

}

KInt KonoeVMECrate::install( KonoeVMEModule* module ) 
{
  // this should check module type A24D16 etc.
  module -> setCrate( this );
  return add( module );
}

void KonoeVMECrate::initialize() 
{
  KonoeVMEModule* module;
  reset();
  while ( module = next() )
    module -> initialize();
  return;
}

void KonoeVMECrate::clear()
{
  KonoeVMEModule* module;
  reset();
  while ( module = next() )
    module -> clear();
  return;
}

void KonoeVMECrate::update()
{
  KonoeVMEModule* module;
  reset();
  while ( module = next() )
    module -> update( );
  return;
}
