// $Name: KONOE-0-09-02 $
// $Id: demo.cc,v 1.6 2002/03/16 08:12:02 iwai Exp $
//==================================================================
/*

	demo.cc at src/daq/datasource

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version	0.4.0	11-Sep-1998
	Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0   24-Dec-2000     G. Iwai

*/
//==================================================================

#include <daq/KonoeSimCrate.hh>
#include <analysis/KonoeBoxGeometry.hh>
#include <daq/KonoeSimTOFTDC.hh>
#include <daq/KonoeReadoutList.hh>
#include <daq/KonoeSoftwareCrate.hh>
#include <daq/KonoeSoftCounter.hh>
#include <analysis/KonoeDataRecordCM.hh>
#include <system/KonoeOutputObjFile.hh>
#include <daq/KonoeCrateList.hh>
#include <analysis/KonoeDataAddressSet.hh>
#include <iostream.h>

//------------------------------------------------------------------
// classes
//------------------------------------------------------------------

class DemoSimulator : public KonoeSimCrate {

public:
  DemoSimulator( KInt modulecount, KonoeDetectorList* dl, 
		 KonoeEventGenerator* g, KInt digitizercount );

};

class SimulationDemo {

public:
  enum { N_COUNTERS = 32,
	 N_MODULES = 2,
	 N_DIGITIZERS = 2,
	 DETECTOR_ID = 1 };

  enum { GENERAL_SECTION = 0,
	 COUNTER_ADC_SECTION = 1,
	 COUNTER_TDC_SECTION = 2,
	 N_SECTIONS = 2,
	 RECORD_TYPE = 102,
	 TDC_SEGMENT0 = 0,
	 TDC_SEGMENT1 = 1,
	 EVENT_INFO = 0 };

protected:
  KonoeDetectorList* detectorlist;
  KonoeDetector* detector;

  KFloat counter_width;
  KFloat counter_height;
  KFloat counter_thickness;
  KFloat detector_distance;

  KonoeEventGenerator* generator;

  KonoeSimCrate* simulator;
  KonoeSimTOFTDC* digitizer;
  KonoeSimModule* tdcmodule[ 2 ];
  KonoeDataAddressSet* tdcaddress;

  KonoeSoftwareCrate* softcrate;
  KonoeSoftCounter* eventnumber;

  KonoeCrateList* cratelist;

  KonoeReadoutList* readoutlist;

  KonoeDataRecordCM* datarecordcm;
  KonoeOutputObjFile* outputfile;

public:
  SimulationDemo( KInt argc, KChar** argv );
  ~SimulationDemo( );
  KInt go( );
};

//--------------------------------------------------------------------
// methods
//--------------------------------------------------------------------

DemoSimulator::DemoSimulator( KInt modulecount, KonoeDetectorList* dl, 
			      KonoeEventGenerator* g, KInt digitizercount )
  : KonoeSimCrate( modulecount, dl, g, digitizercount )
{;}

SimulationDemo::SimulationDemo( KInt, KChar** )
  : counter_width( 0.1 ), counter_height( 1.0 ), 
    counter_thickness( 0.05 ), detector_distance( 5.0 )
{
  Konoe3Vector p, dp;
  KonoeTranslation t;
  KonoeBoxGeometry* b;
  KonoeDetectorElement* e;

  KInt md, ch;

  // set up a demo detector

  tdcaddress = new KonoeDataAddressSet( COUNTER_TDC_SECTION, 2,
					N_COUNTERS / 2 );
  detectorlist = new KonoeDetectorList( 1 );
  detector = new KonoeDetector( N_COUNTERS, DETECTOR_ID );
  detectorlist -> add( detector );
  p.set( counter_width * 0.5 - counter_width * N_COUNTERS / 2.0,
	 - counter_height * 0.5, detector_distance );
  dp.set( counter_width, 0, 0 );
  for ( KInt i = 0; i < N_COUNTERS; i ++ ) {
    b = new KonoeBoxGeometry( counter_width, counter_height, 
			      counter_thickness );
    t.setDisplacement( p );
    b -> translate( &t );
    detector -> add( e = new KonoeDetectorElement( b, detector, i ) );
    md = i / ( N_COUNTERS / 2 );
    ch = i % ( N_COUNTERS / 2 );
    e -> add( new KonoeDataAddress( COUNTER_ADC_SECTION, md, ch ) );
    e -> add( tdcaddress -> getAddress( md, ch ) );
    p += dp;
  }

  // software modules

  softcrate = new KonoeSoftwareCrate( 1 );
  eventnumber = new KonoeSoftCounter( );
  softcrate -> install( eventnumber );

  // set up an event generator

  generator = new KonoeEventGenerator( );

  // simulator

  simulator = new DemoSimulator( N_MODULES, detectorlist, 
				 generator, N_DIGITIZERS );
  tdcmodule[ 0 ] 
    = new KonoeSimModule( tdcaddress->getSectionID(), 0, N_COUNTERS / 2 );
  tdcmodule[ 1 ] 
    = new KonoeSimModule( tdcaddress->getSectionID(), 1, N_COUNTERS / 2 );
  simulator -> add( tdcmodule[ 0 ] );
  simulator -> add( tdcmodule[ 1 ] );

  // digitizer

  digitizer = new KonoeSimTOFTDC( tdcaddress->getSectionID(), 0.0, 1.0 );
  simulator -> addDigitizer( digitizer );

  // crate lsit

  cratelist = new KonoeCrateList( 2 );
  cratelist -> add( softcrate );
  cratelist -> add( simulator );

  // readout procedure

  readoutlist = new KonoeReadoutList( N_SECTIONS, RECORD_TYPE );

  KonoeReadoutSection* psect;
  KonoeReadoutSegment* pseg;

  // -- general section

  psect = new KonoeReadoutSection( 1, GENERAL_SECTION );
  readoutlist -> add( psect );
  pseg = new KonoeReadoutSegment( 1, EVENT_INFO );
  psect -> add( pseg );
  pseg -> add( new KonoeReadoutItem( eventnumber, (KInt)0 ) );
  
  // -- tdc section

  psect = new KonoeReadoutSection( 1, tdcaddress -> getSectionID( ) );
  readoutlist -> add( psect );
  pseg = new KonoeReadoutSegment( tdcmodule[ 0 ] );
  psect -> add( pseg );
  pseg = new KonoeReadoutSegment( tdcmodule[ 1 ] );
  psect -> add( pseg );

  // output

  outputfile = new KonoeOutputObjFile( "test.dat" );
  datarecordcm = new KonoeDataRecordCM( );
  outputfile -> installCM( datarecordcm );
  
}

SimulationDemo::~SimulationDemo()
{
  delete detectorlist;
}

KInt SimulationDemo::go()
{
  KonoeDataRecord* r;

  for ( KInt i = 0; i < 10; i ++ ) {
    cratelist -> update( );
    r = readoutlist -> read( );
    r -> dump( cout );
    outputfile -> put( r );
    delete r;
  }
  return 0;
}

//-------------------------------------------------------------------
// entry point
//-------------------------------------------------------------------

int main( int argc, char** argv )
{
  SimulationDemo demo( argc, argv );
  return demo.go();
}
