// $Name: KONOE-0-09-02 $
// $Id: KonoeCAMACXaction.cc,v 1.4 2002/03/16 08:12:03 iwai Exp $
//====================================================================
/*

        KonoeCAMACXaction.cc

        Version 0.6.0   09-Nov-2000     G. Iwai

*/
//====================================================================
#include <daq/KonoeCAMACXaction.hh>

KonoeCAMACXaction::KonoeCAMACXaction(KonoeReadoutList* aList, 
				     KonoeCAMACCrate* aCrate,
				     KonoeCAMACInterruptRegister* aIR,
				     KUInt milliSec)
  : KonoeHardwareXaction(aList), interruptRegister(aIR), crate(aCrate),
    time(milliSec), mode(WAIT_ONE_IR)
{;}

KonoeCAMACXaction::KonoeCAMACXaction( KonoeReadoutList* aList, 
				      KonoeCAMACCrate* aCrate,
				      KonoeCAMACInterruptRegister* aIR1,
				      KonoeCAMACInterruptRegister* aIR2,
				      KUInt milliSec )
  : KonoeHardwareXaction(aList), interruptRegister(aIR1), 
    interruptRegister2(aIR2), crate(aCrate), time(milliSec), 
    mode(WAIT_TWO_IR)
{;}

// under construction ....
KonoeCAMACXaction::KonoeCAMACXaction( KonoeReadoutList* aList, 
				      KonoeCAMACCrate* aCrate,
				      KUInt bit, KUInt milliSec )
  : KonoeHardwareXaction(aList), crate(aCrate), time(milliSec), 
    mode(WAIT_BIT_PATTERN)
{;}

KInt KonoeCAMACXaction::enable()
{
  this -> crate -> enableLAM();
  interruptRegister -> enableLAM();
  if ( mode == WAIT_TWO_IR )
    interruptRegister2 -> enableLAM();
  return (0);
}

KInt KonoeCAMACXaction::disable()
{
  this -> crate -> disableLAM();
  interruptRegister -> disableLAM();
  if ( mode == WAIT_TWO_IR )
    interruptRegister2 -> disableLAM();
  return (0);
}

KonoeDataRecord* KonoeCAMACXaction::read()
{
  while (1) {
    if ( mode == WAIT_ONE_IR )
      this -> crate -> waitLAM(this -> time, this -> interruptRegister);
    else if ( mode == WAIT_TWO_IR )
      crate -> waitLAM(time, interruptRegister, interruptRegister2);
    KInt status = crate -> getStatus();
    if ( status == 0 ) {
      return ( list -> read() );
    } else {
      switch ( status ) {
      case -5:
	cerr << "OFFLLINE" << endl;
	break;
      case -11:
	cerr << "OTHER INTERRUPT" << endl;
	return NULL;
      case -13:
	cerr << "NOT SUPPORT" << endl;
	return NULL;
      case -22:
	cerr << "NOT GOOD TIME" << endl;
	return NULL;
      case -62:
	cerr << "TIME OUT" << endl;
	break;
      default:
	cerr << "STATUS ERROR " << status << endl;
	break;
      }
      continue;
    }
  }
}
