//==================================================================
/*

	KonoeRing.cc

	Version 0.0.1	18-Aug-1998	S. Nishida
	Version 0.4.0	 8-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno          
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//==================================================================

#include	<KonoeRing.hh>
#ifndef __KONOERING_CC
#define __KONOERING_CC

template<class T> KonoeRing<T>::KonoeRing( KInt ptablesize ) {

	if( ptablesize )
	    entries = new T * [ ptablesize ];
	else
	    entries = 0;
	tablesize = ptablesize;
	n_entries = 0;
	tail = 0;
	counter = 0;
}

template<class T> KonoeRing<T>::~KonoeRing( ) {

	for( KInt i = 0; i < n_entries; i ++ )
	    delete * ( entries + ( tail + i ) % tablesize );
	delete [] entries;	

}

template<class T> KInt KonoeRing<T>::add( T * pentry ) {

	if( n_entries >= tablesize ){
	  delete * ( entries + tail );
	  tail = ( tail + 1 ) % tablesize;
	} else
	  n_entries ++;
	* ( entries + ( tail + n_entries - 1 ) % tablesize ) = pentry;

	return n_entries - 1 ;
}

template<class T> KInt KonoeRing<T>::clear( ) {

	for( KInt i = 0; i < n_entries; i ++ )
	    delete * ( entries + ( tail + i ) % tablesize );
	n_entries = 0;
	tail  = 0;
	counter = 0;
	return 0;
}

template<class T> void KonoeRing<T>::reset( ) {

	counter = 0;

}

template<class T> T * KonoeRing<T>::next( ) {

	if( counter >= n_entries ) return 0;
	return * ( entries + ( tail + counter ++ ) % tablesize );

}

template<class T> T * KonoeRing<T>::get( KInt pindex ) {

	if( pindex < 0 || pindex >= n_entries )
	    return 0;
	return * ( entries + ( tail + pindex ) % tablesize );

}
#endif

