//=================================================================
/*

        KonoePath.cc

	Version 0.5.0   21-Sep-1999     Y. Kouno
*/
//=================================================================

#include	<system/KonoePath.hh>
#include        <KonoeCategories.hh>
#include	<system/KonoeFileManagerCM.hh>
#include	<system/KonoeLocalNameSpaceManager.hh>
#include	<string.h>
#include	<unistd.h>

KChar		* KonoePath::root = 0;
KonoePath	KonoePath::pwd("/");
const KChar	* KonoePath::className = "KonoePath";

KonoePath::KonoePath( const KChar * ppath, const KChar * phost ):
		KonoeStreamableObject( KonoeCategories::FILEMANAGER,
					KonoeFileManagerCM::CID_PATH ){

	if( root == 0 ){
	    KonoeLocalNameSpaceManager * manager;
	    KonoeNameSpaceValue	* value;
	    KInt	length;
	    manager = new KonoeLocalNameSpaceManager();
	    value = manager -> getValue("KONOE_FILESYSTEM_ROOT");
	    if( value ){
		length = strlen( (KChar*)( value -> get( ) ) );
		root = new KChar[ length + 1 ];
		strcpy( root, (KChar*)( value -> get( ) ) );
	    }
	    else {
		cerr<< "KonoePath::constructor. " <<
			"KONOE_FILESYSTEM_ROOT not defined" << endl;
	    }
	    //root = ".";
	    root = getcwd(0,256);
	}
	if( phost == 0 ){
	    host = "localhost";
	}
	else {
	    host = new KChar[ strlen( phost ) + 1 ];
	    strcpy( host, phost );
	}

	path = new KChar[ strlen(ppath) + 1 ];
	strcpy( path, ppath );
	//checkPath( ppath, path );
}

KonoePath::KonoePath( ): KonoeStreamableObject( KonoeCategories::FILEMANAGER,
                                        KonoeFileManagerCM::CID_PATH ){

	path = 0;
	root = 0;

	if( root == 0 ){
            //root = ".";
            root = getcwd(0,256);
        }
}

KonoePath::~KonoePath( ){

	delete [] path;
}

KChar * KonoePath::getUnixPath( ){

	KChar * unixpath, *p;
	if( *path == '/' ){
	    unixpath = new char[ strlen( root ) + strlen( path ) + 1 ];
	    strcpy( unixpath, root );
	    strcat( unixpath, path );
	}
	else {
	    p = new char[ strlen( pwd.path ) + 1 + strlen( path ) + 1 ];
	    strcpy( p, pwd.path );
	    strcat( p, "/" );
	    strcat( p, path );
	    checkPath( p, p );

	    unixpath = new char[ strlen( root ) + strlen( p ) + 1 ];
	    strcpy( unixpath, root );
	    strcat( unixpath, p );
	}
	return unixpath;
}

KInt KonoePath::changePath( KonoePath * ppath ){
	
	return changePath( ppath -> path );
}

KInt KonoePath::changePath( char * ppath ){

	KChar * newpath;
        if( ppath[0]=='/'){
            newpath = new KChar[ strlen( ppath ) + 1 ];
            strcpy( newpath, ppath );
        }
        else{
            newpath = new KChar[strlen(path)+ 1 + strlen( ppath ) + 1];
            strcpy( newpath, path );
            strcat( newpath, "/");
            strcat( newpath, ppath );
            checkPath( newpath, newpath);
        }
        delete [] path;
        path = newpath;

        return 0;
}


KInt KonoePath::changePwd( KonoePath * ppath ){

	return pwd.changePath( ppath );
}

KInt KonoePath::checkPath( KChar * ppath, KChar * buffer ){

	KInt	i;
	KChar	*p = ppath;

	i = 0;
	if( *p == '/' ){
	    buffer[i] = *p;
	}else{
	    strcpy( buffer, ppath );
	    return 0;
	}

	while( 1 ){
	    p++;

	    if( p[0] == '\0' ){ break; } 
	    else if( p[0] == '/' ){ continue; } 
	    else if( p[0] == '.' ){
		if( p[1] == '\0' ){ break; }
		else if( p[1] == '/' ){ p++; continue; }
	 	else if( p[1] == '.' ){ 
		    if( p[2] == '\0' ){
			if( i == 0 ){ break; }
			while( buffer[--i] != '/' ){ ; }
			break;
		    }else if( p[2] == '/' ){
			p += 2;
			if( i == 0 ){ continue; }
                	while( buffer[--i] != '/' ){ ; }
			continue;
		    }
		}
	    }

	    while( *p != '/' && *p != '\0' ){
		buffer[++i] = *(p++);
	    }
	    if( *p == '/' ){ buffer[++i] = *p; continue;}
	    else { i++; break; }

	}

	if( i == 0  ){
	    buffer[++i] = '\0';
	} else {
	    buffer[i] = '\0';
	}
		
	return 0;
}

KChar * KonoePath::getUpDir( ){

	if( strcmp( path, "/" ) == 0 ){

	}

	KChar *slash, *p;
	KInt len;

	len = strlen( path ); // length of path

	p = path + len - 1;   // point to end of path
	if( *p == '/' ){
	    p--;
	}
	while( *p !='/' ){
	    p--;
	}

	KChar * up = new KChar[ p - path + 1 ]; 
	strncpy( up, path, p - path );
	up[ p - path + 1 ] = '\0';

	return up;
}

KInt    KonoePath::getStreamSize( ){

        KInt    s;

        s  = HEADER_SIZE + ( strlen( path ) / sizeof( KInt ) ) + 1;
        s += HEADER_SIZE + ( strlen( host ) / sizeof( KInt ) ) + 1;
        return s;
}

KInt	KonoePath::put( KonoeOutputObjectStream * os,
                        KChar level, KInt memberid ){

        KChar   l = level + 1;

        os -> putHeader( this, level, memberid, N_MEMBERS );
        os -> put( l, MID_PATH, path );
        os -> put( l, MID_HOST, host );

        return 0;
}

KInt    KonoePath::fillPrimitiveMember( KInt memberid, KInt datatype,
                                KInt datacount, void * data ){

        switch( datatype ) {
        case KCHAR_TYPE :
            switch( memberid ){
            case MID_PATH :
                path = ( KChar * ) data;
                break;
            case MID_HOST :
		host = ( KChar * ) data;
		break;
            default :
                delete [] data;
                return -1;
            }
            return 0;
        default :
            delete [] data;
            return -1;
        }
}

