//==================================================================
/*
	KonoeLogMessage.cc

	Version 0.1.0   05-Mar-1998     H. Sakamoto
        Version 0.4.0   27-Aug-1998     S. Inaba
        Version 0.4.1   03-Aug-1999     Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	22-Jul-2000	Y. Shimizu
	Version 0.6.1	15-Aug-2000	Y. Shimizu
	Version 0.7.0	28-Aug-2001	S. Morinaka

	Package : system
	Category : logger
*/
//==================================================================

#include	<system/KonoeLogMessage.hh>
#include	<system/KonoeLoggerCM.hh>
#include	<system/KonoeHostSystem.hh>
#include	<KonoeCategories.hh>
#include	<string.h>
#include	<stdio.h>
#include	<unistd.h>

const KChar	* KonoeLogMessage::className = "KonoeLogMessage";

KonoeLogMessage::KonoeLogMessage( KChar * pmessage, KInt plevel ) :
        KonoeStreamableObject( KonoeCategories::SYSTEMLOGGER,
                               KonoeLoggerCM::CID_LOGMESSAGE ) {
        time = new KonoeTime( );
        message = new KChar[ strlen( pmessage ) + 1 ];
        strcpy( message, pmessage );
	level=plevel;

	hostname = new KChar[HOSTNAMELENGTH];

	if( KonoeHostSystem::getHostName( hostname, HOSTNAMELENGTH ) < 0 ){
		cerr << " can't get hostname " << endl;
		strcpy( hostname , "unknown host");
	}
	pid = getpid( );
}	

KonoeLogMessage::KonoeLogMessage( ) :
	KonoeStreamableObject( KonoeCategories::SYSTEMLOGGER,
			       KonoeLoggerCM::CID_LOGMESSAGE ) {
	time = 0;
	message = 0;
}

KonoeLogMessage::~KonoeLogMessage( ) {

	delete time;
	delete [] message;
   	  message = NULL;
	delete [] hostname;
	  hostname = NULL;
}

KChar * KonoeLogMessage::getLevelMsg( ){
	switch( level ){
	case SUCCESS :
		return " success " ;
	case INFO :
		return " info " ;
	case WARNING :
		return " warning " ;
	case ERROR :
		return " error " ;
	case FATAL :
		return " fatal ";
	default :
		return " unknown-level ";
	}
}

KChar *	KonoeLogMessage::getPIDMsg( ) {

	KChar pidMsg[30];
	sprintf(pidMsg," pid - %d ", pid );
	return (pidMsg);

}

KChar * KonoeLogMessage::getLogMsg( ){

	KChar  LogMsg[LOGMSGLENGTH];
	sprintf(LogMsg,"%-9s %-25s %s @%-8s ",getLevelMsg(),message,
				getTimeInfo(),hostname); 
	strcat(LogMsg,getPIDMsg( ));
	strcat(LogMsg,"\n");
/*
	strcpy(LogMsg,getLevelMsg( ) );
	strcat(LogMsg," ");
	strcat(LogMsg,message );
	strcat(LogMsg," ");
	strcat(LogMsg,"\n\t\t\t\t");
	strcat(LogMsg,getTimeInfo( ) );
	strcat(LogMsg,"\t");
	strcat(LogMsg,"@");
	strcat(LogMsg,hostname);
	strcat(LogMsg,"\t");
	strcat(LogMsg,getPIDMsg( ));
	strcat(LogMsg,"\n");
*/

	return LogMsg;

}

void	KonoeLogMessage::dump( ostream & os ){

	os << "LogMessage" << endl;
	os << "level = " << level << endl;
	os << "message = " << message << endl;
	time -> dump( os );
	os << "hostname = " << hostname << endl;
	os << "pid = " << pid << endl;

}

KInt	KonoeLogMessage::put(KonoeOutputObjectStream * os,
                        KChar plevel, KInt memberid ){

	KChar   l = plevel + 1;

        os -> putHeader( this, plevel, memberid, N_MEMBERS );
	os -> put( l, MID_LEVEL, level );
        os -> put( l, MID_MESSAGE, message );
	time -> put( os, l, MID_TIME );
	os -> put( l, MID_HOST,hostname );
	os -> put( l, MID_PID, pid );

	return 0;
}

KInt	KonoeLogMessage::getStreamSize( ){

	KInt    s;

	s = HEADER_SIZE + 1;
	s += HEADER_SIZE + ( strlen( message ) / sizeof( KInt ) ) + 1;
	s += HEADER_SIZE + time -> getStreamSize( );
	s += HEADER_SIZE + ( strlen( hostname )/ sizeof( KInt ) ) + 1;
	s += HEADER_SIZE + 1; return s; 
} 

KInt	KonoeLogMessage::fillPrimitiveMember( KInt memberid, KInt datatype,
                                KInt datacount, void * data ){

	switch( datatype ) {
	case KCHAR_TYPE :
	    switch( memberid ){
	    case MID_MESSAGE :
		message = ( KChar * ) data;
		break;
	    case MID_HOST :
		hostname = ( KChar * ) data;
		break;
	    default :
		delete [] data;
                return -1;
            }
	    return 0;
	case KINT_TYPE :
	    switch( memberid ){
	    case MID_LEVEL :	
	        level = * ( KInt * ) data;
                delete [] data;
	        break; 
	    case MID_PID :
		pid   = * ( KInt * ) data;
		break;
            default :
		delete [] data;
                return -1;
	    }
	    return 0;
        default :
            delete [] data;
            return -1;
        }
}

KInt	KonoeLogMessage::fillObjectMember( KInt memberid, 
					KonoeStreamableObject * obj ){

	if( memberid != MID_TIME ){
		delete obj;
		return -1;
	}

	time = ( KonoeTime * ) obj;

	return 0;
}


