//===================================================================
/*

	KonoeMessage.cc

	Version 0.1	02-Mar-1998	H. Kamata
        Version 0.2.0   26-Mar-1998     H. Kamata
	Version 0.4.0	27-Aug-1998	S. Inaba
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.7.0	28-Aug-2001	S. Morinaka
*/
//===================================================================

#include <system/KonoeMessage.hh>

const KChar	* KonoeMessage::className = "KonoeMessage";

KonoeMessage::KonoeMessage(void) :
    KonoeStreamableObject( KonoeCategories::MESSAGESYSTEM,
			   KonoeMessageCM::CID_MESSAGE ) {
}

KonoeMessage::~KonoeMessage(void) {

  delete hostname;
  delete processname;
  delete message;
  delete option;
}

KInt KonoeMessage::put( KonoeOutputObjectStream *os, \
			      KChar level, KInt memberid ) {

  KChar l = level + 1;

  os -> putHeader( this, level, memberid, N_MEMBERS );
  os -> put( l, MID_HOSTNAME, hostname );
  os -> put( l, MID_HOSTNAMELENGTH, hostnamelength ); 
  os -> put( l, MID_PROCESSNAME, processname );
  os -> put( l, MID_PROCESSNAMELENGTH, processnamelength );
  os -> put( l, MID_MESSAGE, message );
  os -> put( l, MID_MESSAGELENGTH, messagelength );
  os -> put( l, MID_OPTION, option );
  os -> put( l, MID_OPTIONLENGTH, optionlength );

  return 0;

}

KInt KonoeMessage::getStreamSize(void) {

    KInt s;

    s = N_MEMBERS * HEADER_SIZE;    // header
    s += 4;                         // four integer type members
    s += ( hostnamelength / sizeof( KInt ) ) + 1;
    s += ( processnamelength / sizeof( KInt ) ) + 1;
    s += ( messagelength / sizeof( KInt ) ) + 1;
    s += ( optionlength / sizeof( KInt ) ) + 1;
    return s;
    
}

KInt KonoeMessage::fillPrimitiveMember( KInt memberid, \
					      KInt datatype, KInt datacount, \
					      void *data ) {

  switch( datatype ) {
  case KINT_TYPE :
    switch( memberid ) {
    case MID_HOSTNAMELENGTH :
      hostnamelength = * ( KInt * ) data;
      delete [] data;
      break;
    case MID_PROCESSNAMELENGTH :
      processnamelength = * ( KInt * ) data;
      delete [] data;
      break;
    case MID_MESSAGELENGTH :
      messagelength = * ( KInt * ) data;
      delete [] data;
      break;
    case MID_OPTIONLENGTH :
      optionlength = * (KInt * ) data;
      delete [] data;
      break;
    default :
      delete [] data;
      return -1;
    }
    return 0;
  case KCHAR_TYPE :
    switch( memberid ) {
    case MID_HOSTNAME :
      hostname = ( KChar * ) data;
      break;
    case MID_PROCESSNAME :
      processname = ( KChar * ) data;
      break;
    case MID_MESSAGE :
      message = ( KChar * ) data;
      break;
    case MID_OPTION :
      option = ( KChar * ) data;
      break;
    default :
      delete [] data;
      return -1;
    }
    return 0;
  default :
    delete [] data;
    return -1;
  }
}

// set message object
KInt KonoeMessage::setMessage( KChar *phost, KChar *pprocess, \
				     KChar *pmessage ) {
  hostname = phost;
  processname = pprocess;
  message = pmessage;

  // default option
  KChar default_option[8] = {"default"};
  option = default_option;

  return( 0 );

}

// set message object (add option)
KInt KonoeMessage::setMessage( KChar *phost, KChar *pprocess, \
				     KChar *pmessage, KChar *poption) {

  hostname = phost;
  processname = pprocess;
  message = pmessage;
  option = poption;

  return( 0 );

}

// dump message object
void KonoeMessage::dump( ostream &os ) {

  os << "host name    : " << hostname << endl;
  os << "process name : " << processname << endl;
  os << "message      : " << message << endl;
  os << "option       : " << option << endl;

}

// get host name
KChar *KonoeMessage::getHostName( void ) {

    return( hostname );

}

// get process name
KChar *KonoeMessage::getProcessName( void ) {

    return( processname );
    
}

// get message
KChar *KonoeMessage::getMessage( void ) {

    return( message );

}

// get option
KChar *KonoeMessage::getOption( void ) {

    return( option );

}
