//==================================================================
/*

	KonoeNameSpaceItem.cc

	Version 0.1	11-Dec-1997	H. Sakamoto
	Version 0.2.0	25-Apr-1998	H. Sakamoto
	Version	0.4.0	27-Aug-1998	S. Inaba
        Version 0.5.0   21-Sep-1999     Y. Kouno

*/
//==================================================================

#include	<system/KonoeNameSpaceItem.hh>
#include	<system/KonoeNameSpaceValue.hh>
#include	<KonoeCategories.hh>
#include	<string.h>

const KChar	* KonoeNameSpaceItem::className = "KonoeNameSpaceItem";

KonoeNameSpaceItem::KonoeNameSpaceItem( ) :
	KonoeStreamableObject( KonoeCategories::NAMESPACE,
			KonoeNameSpaceCM::CID_NAMESPACEITEM ) {

	name = 0;
	namelength = 0;
	valuecount = 0;
}

KonoeNameSpaceItem::~KonoeNameSpaceItem( ) {

	delete name;
	for( KInt i = 0; i < valuecount; i ++ )
	    delete values[ i ];
	delete [] values;
}

KInt	KonoeNameSpaceItem::setName( const KChar * pname ) {

	KInt	l;

	if( namelength != 0 )
	    delete [] name;
	l = strlen( pname );
	namelength = l;
	name = new KChar[ l + 1 ];
	strcpy( name, pname );
	return 0;
}

KInt	KonoeNameSpaceItem::allocate( KInt pvaluecount ) {

	valuecount = pvaluecount;
	values = new KonoeNameSpaceValue * [ valuecount ];
	for( KInt i = 0; i < valuecount; i ++ )
	    values[ i ] = 0;
	return 0;
}

KInt	KonoeNameSpaceItem::setValue( KChar * pvalue, KInt pindex ) {

	if( pindex >= valuecount )
	    return -1;	// need to allocate first
	if( values[ pindex ] == 0 )
	    values[ pindex ] = new KonoeNameSpaceValue( );
	values[ pindex ] -> set( pvalue );
	return 0;
}

KInt    KonoeNameSpaceItem::setValue( KInt pvalue, KInt pindex ) {

        if( pindex >= valuecount )
            return -1;  // need to allocate first
        if( values[ pindex ] == 0 )
            values[ pindex ] = new KonoeNameSpaceValue( );
        values[ pindex ] -> set( pvalue );
        return 0;
}


KInt	KonoeNameSpaceItem::put(
			KonoeOutputObjectStream * os,
			KChar level, KInt memberid ) {

	KChar	l = level + 1;
	KInt	i, s;
	KonoeNameSpaceValue	** p;

	os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_NAME, name );
	os -> put( l, MID_NAMELENGTH, namelength );
	os -> put( l, MID_VALUECOUNT, valuecount );
	s = 0;
	p = values;
	for( i = 0; i < valuecount; i ++ )
	    s += HEADER_SIZE + ( * p ++ ) -> getStreamSize( );
	os -> putObjectArrayHeader( l, MID_VALUES, valuecount, s );
	p = values;
	for( i = 0; i < valuecount; i ++ )
	    ( * p++ ) -> put( os, l + 1, i );
	return 0;
}

KInt	KonoeNameSpaceItem::getStreamSize( ) {

	KInt	i, s;
	KonoeNameSpaceValue	** p;

	s = N_MEMBERS * HEADER_SIZE;	// header
	s += 2;				// two integer type members
	s += ( namelength / sizeof( KInt ) ) + 1;
	s += ( valuecount / sizeof( KInt ) ) + 1;
	p = values;
	for( i = 0; i < valuecount; i ++ )
	    s += HEADER_SIZE + ( * p ++ ) -> getStreamSize( );
	return s;

}

KInt	KonoeNameSpaceItem::fillPrimitiveMember( KInt memberid,
			KInt datatype, KInt datacount, void * data ) {

	switch( datatype ) {
	case KINT_TYPE :
		switch( memberid ) {
		case MID_NAMELENGTH :
			namelength = * ( KInt * ) data;
			delete [] data;
			break;
		case MID_VALUECOUNT:
			valuecount = * ( KInt * ) data;
			values = new KonoeNameSpaceValue * [ valuecount ];
			fillcount = 0;
			delete [] data;
			break;
		default :
			delete [] data;
			return -1;
		}
		return 0;
	case KCHAR_TYPE :
		switch( memberid ) {
		case MID_NAME :
			name = ( KChar * ) data;
			break;
		default :
			delete [] data;
			return -1;
		}
		return 0;
	default :
		delete [] data;
		return -1;
	}
}

KInt	KonoeNameSpaceItem::fillObjectMember( KInt memberid,
			KonoeStreamableObject * data ) {

	if( memberid != MID_VALUES ) {
		delete data;
		return -1;
	}
	values[ fillcount ++ ] = ( KonoeNameSpaceValue * ) data;
	return 0;
}
