//==================================================================
/*

	KonoeNameSpaceMaster.cc

	Version 0.0.1	11-Nov-1998	H. Sakamoto
	Version 0.0.2	18-Jun-1999	H. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.6	 2-Oct-2000	Y. Shimizu
	Version 0.6.1	11-Nov-2000	H. Sakamoto

*/
//==================================================================

#include	<system/KonoeNameSpaceMaster.hh>
#include        <system/KonoeNameSpaceMessage.hh>
#include        <system/KonoeNameSpaceCM.hh>
#include	<system/KonoeNameSpacePacket.hh>
#include	<system/KonoeHostSystem.hh>
#include	<KonoeCategories.hh>
#include	<stdio.h>
#include	<string.h>


KonoeNameSpaceMaster::KonoeNameSpaceMaster( ) : KonoeNameSpaceDaemon( ) {

	KChar	buffer[256], hostname[256];
	struct  hostent *hp;
	KInt	iport;

	table = new KonoeNameSpaceTable( );
	table -> init();

// get host name
	KonoeHostSystem::getHostName( buffer, 256 );
	hp = KonoeHostSystem::getHostByName( buffer );
        if( hp ){
            strcpy( hostname, hp->h_name );
        } else {
            cerr<<"get hostname error" << endl;
        }

	table -> setValue( "NAMESPACE_MASTER_HOST", hostname );

	iport = NAMESPACESTRINGREQUESTPORT;

	readfds = new KonoeFDSet( );

	stringServerSocket = new KonoeServerSocket( 
					NAMESPACESTRINGSERVERPORT, readfds );
	stringServerSocket -> setSendMode( KonoeServerSocket::SEND_TO_ALL );
	stringRequestSocket = new KonoeServerSocket(NAMESPACESTRINGREQUESTPORT, readfds );
	
	iport = NAMESPACEOBJECTREQUESTPORT;

	requestReceiveSocket = new KonoeObjectServerSocket(
				NAMESPACEOBJECTREQUESTPORT, readfds );

	objectSendSocket = new KonoeObjectServerSocket(
				NAMESPACEOBJECTSERVERPORT, readfds );

	KonoeNameSpaceCM * cm = new KonoeNameSpaceCM( );
	requestReceiveSocket -> installCM( cm );

	objectSendSocket -> setSendMode( KonoeObjectServerSocket::SEND_TO_ALL );

}

KInt	KonoeNameSpaceMaster::go( ) {

	KInt	st, k;
	KChar	buffer[ BUFFERSIZE ];
	KonoeFDClient	* cl;
	KonoeSocketConnection	* connection;
	KonoeObjectSocket	* objSocket;
	KonoeNameSpaceMessage   * message;

	while( 1 ) {

//cout<<"wait" << endl;

	  cl = readfds -> wait( 1000 );
//cout<<"readfds->wait"<<endl;
	  if( cl ) {	// client exists.
//cout<<"cl exist = " << cl << endl;

// string request socket
	    connection = stringRequestSocket -> processReceiveAction( cl );
	    if( connection == ( KonoeSocketConnection * ) -1 ){
	    }
	    else if ( connection ) {
//cout<<"read string req sock" << endl;
	      readStringRequestSocket( connection );
	      continue;
	    }
	    else {
//cout<<"stringRequestSocket returns 0"<<endl;
	    }

// object request socket
	    objSocket = requestReceiveSocket -> processReceiveAction( cl );
	    if( objSocket == ( KonoeObjectSocket * ) -1){  
	    }
	    else if ( objSocket ){ // requestReceiveSocket has data
//cout<<"requestReceiveSocket"<<endl;
		readRequestReceiveSocket( objSocket );
		continue;
	    }
	    else {
//cout<<"requestReceiveSocket connected."<<endl;
	    }

// string server socket
	    connection = stringServerSocket -> processReceiveAction( cl );
            if( connection == ( KonoeSocketConnection * ) -1){
            }
            else if ( connection ){ // stringServerSocket has data
		readStringServerSocket( connection );
        	continue;
            }
	    else {
//cout<<"stringServerSocket returns 0"<<endl;
	    }


// object server socket
	    objSocket = objectSendSocket -> processReceiveAction( cl ); 
            if( objSocket == ( KonoeObjectSocket * ) -1){ 
            }  
            else if ( objSocket ){ // objectServerSocket has data 
//cout<<"objectSendSocket"<<endl;
                readObjectSendSocket( objSocket );
                continue;      
            }
	    else {
//cout<<"objectSendSocket returns 0"<<endl;
	    }


	    
	  }// client exists.
	  else {
//cout<<"Timed out..."<<endl;
	  }
	}// while
	return 0;
}


KInt KonoeNameSpaceMaster::execMessage( KonoeNameSpaceMessage * message ){

        message -> exec( this );
	return 0;
}


KInt	KonoeNameSpaceMaster::readRequestReceiveSocket( KonoeObjectSocket * ch){

	KonoeNameSpaceMessage   * message;
	
	message = ( KonoeNameSpaceMessage * )ch -> get( );
	
	if( message == 0 ){
//cout<<"closing object socket"<<endl;
            requestReceiveSocket -> close( ch );
	}
	else {
//cout<<"exec message" << endl;
	    execMessage( message );
	    objectSendSocket -> put( message );
	    delete message;
	}

	return 0;
}

KInt	KonoeNameSpaceMaster::readStringRequestSocket( 
					KonoeSocketConnection * ch ){
//cout<< "read str req sock is called" << endl;
	KInt	st, k;

	KonoeNameSpacePacket pa;

//cout<< "read socket connection" << endl;
	st = pa.read( ch );
//cout<< "read socket connection end " << endl;
	if( st == KonoeNameSpacePacket::READ_SUCCESS ){
//cout<< "write to str ser sock" << endl;
	    st = pa.write( stringServerSocket );
	    if( st < 0 )
		cerr << "KonoeNameSpaceMaster::readStringRequestSocket."
			" write status = "<<st<<endl;
//cout<< "set to table" << endl;
	    table -> setValue( pa.getEntryName( ), pa.getValue( ),
				pa.getValueType( ), pa.getIndex( ) );
//cout<< "set to table end" << endl;
	}
	else if ( st == KonoeNameSpacePacket::READ_ERROR ){
	    cerr<<"KonoeNameSpaceMaster. read packet error" << endl;
	    cerr<<"clinetName = " << ch -> getClientName( ) << endl;
		return -1;
	}
	else if( st == KonoeNameSpacePacket::ZEROBYTE_READ ){
//cout<< "close ch" << endl;
cout<<"KonoeNameSpacePacket returned ZEROBYTE_READ"<<endl;
		stringRequestSocket -> close( ch );
	}

	return 0;
}


KInt KonoeNameSpaceMaster::readObjectSendSocket( KonoeObjectSocket * ch ){

	KChar	buffer[ BUFFERSIZE ];
	KInt	st;

	st = objectSendSocket -> receiveFromConnection(
			 ch, buffer, BUFFERSIZE );
	if( st > 0 ) {  // slave server should not send
		    cerr << "KonoeNameSpaceMaster::go."
		      << "Unsolicited input from slave. st  = "
			<< st << " : " << buffer << endl;
		return -1;
	}
	return 0;
}

KInt KonoeNameSpaceMaster::readStringServerSocket( KonoeSocketConnection * ch ){

	KChar	buffer[ BUFFERSIZE ];
	KInt	st;

	st = stringServerSocket -> receiveFromConnection(
				ch, buffer, BUFFERSIZE );
	if( st > 0 ) {  // slave server should not send
	    cerr << "KonoeNameSpaceMaster::go."
		  << "Unsolicited input from slave = " 
		    << buffer << endl;
		return -1;
	}
	return 0;
}
