//====================================================================
/*

	KonoeNameSpacePacket.cc

	Version 0.1	10-Sep-1999	Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.1	23-Nov-2000	H. Sakamoto

*/
//====================================================================

#include	<system/KonoeNameSpacePacket.hh>
#include	<system/KonoeInputStream.hh>
#include	<system/KonoeOutputStream.hh>

#include	<stdio.h>
#include	<string.h>

KonoeNameSpacePacket::KonoeNameSpacePacket( ){

	entryname = 0;
	value = 0;
	buffer = 0;

}

KonoeNameSpacePacket::KonoeNameSpacePacket( const KChar * pentryname, 
					KInt pvalue, KInt pindex ){
	init( pentryname, &pvalue, pindex, TYPE_INT );
}

KonoeNameSpacePacket::KonoeNameSpacePacket( const KChar * pentryname, 
				const KChar * pvalue, KInt pindex ){
	init( pentryname, pvalue, pindex, TYPE_CHAR );
}

KonoeNameSpacePacket::KonoeNameSpacePacket( const KChar * pentryname, 
		const void * pvalue, KInt pindex, KInt pvaluetype ){
	init( pentryname, pvalue, pindex, pvaluetype );
}

void KonoeNameSpacePacket::init( const KChar * pentryname, 
		const void * pvalue, KInt pindex, KInt pvaluetype ){

	KInt    valuesize;
        KChar   * p;

	index = pindex;
	valuetype = pvaluetype;

        datasize = 3 * sizeof( KInt );// datasize + type + index
        datasize += strlen( pentryname ) + 1;

        switch( valuetype ){
        case TYPE_INT :
            valuesize = sizeof( KInt );
            break;
        case TYPE_CHAR :
            valuesize = strlen( (KChar*)pvalue ) + 1;
            break;
        default :
            // not yet
	    cerr << "sorry not yet" << endl;
            break;
        }

	datasize += valuesize;

// allocate buffer
	buffer = new KChar[ datasize ];

// fill data 
        p = buffer;
        //*( KInt * ) p  = datasize;
	KonoeOutputStream::hostToNet( &datasize, (KInt*)p );
        p += sizeof( KInt );
        //*( KInt * ) p  = valuetype;
	KonoeOutputStream::hostToNet( &valuetype, (KInt*)p );
        p += sizeof( KInt );
        //*( KInt * ) p  = index;
	KonoeOutputStream::hostToNet( &index, (KInt*)p );
        p += sizeof( KInt );
        strcpy( p, pentryname );
// entryname 
	entryname = p;
        p += strlen( pentryname ) + 1;
        memcpy( p, pvalue, valuesize ); // memory copy
// value
	value = p;

}

KonoeNameSpacePacket::~KonoeNameSpacePacket( ){

	if( buffer ){
	    delete [] buffer;
	}

}

KInt KonoeNameSpacePacket::read( KonoeSocketIO * socket ){

	if( buffer ){

	    delete [] buffer;
	    buffer = 0;
	    entryname = 0;
	    value = 0;
	}

	KInt st;

	//st = socket -> receive( &datasize, sizeof( KInt ) );
	//st = socket -> receive( (KChar*)&datasize, sizeof( KInt ) );

	st = socket -> read( &datasize );


	if( st == 0 ){
	    return ZEROBYTE_READ;
	}
        else if( st != sizeof( KInt ) ){
	    return READ_ERROR;
	}
	else {

// size check 
            if( datasize < MINIMUM_DATA_SIZE ){
                cerr<< "KonoeNameSpacePacket. Error. "
                    << "datasize is < MINIMUM_DATA_SIZE "<< endl; 
	        return READ_ERROR;
            }
// size ok 

	    buffer = new KChar[ datasize ];
	    char	* bufferpointer;
	    int		datacount;

	    KonoeOutputStream::hostToNet( &datasize, (KInt*)buffer );

	    bufferpointer = buffer + sizeof( KInt );
	    datacount = datasize - sizeof( KInt );

            st = socket -> receive( bufferpointer, datacount );

	    if( st == 0 ){
		delete [] buffer;
		buffer = 0;
	        return ZEROBYTE_READ;
	    }
	    else if( st < datacount ) {
		while( st < datacount ) {
		    datacount -= st;
		    bufferpointer += st;
		    st = socket -> receive( bufferpointer, datacount );
		    if( st == 0 ) {
			return ZEROBYTE_READ;
		    }
		}
	    }
            else if( st != datasize - sizeof( KInt ) ){
                cerr << "KonoeNameSpacePacket. Error. readsize error." 
                        << " readsize is "<< st << endl; 
		delete [] buffer;
		buffer = 0;
	        return READ_ERROR;
            }
// message ok


            KChar   *p;

            p = buffer + sizeof( KInt );

            //valuetype = * ( KInt * ) p;

	    KonoeInputStream::netToHost( (KInt*)p, &valuetype );

            p += sizeof( KInt ); 
            //index = * ( KInt * ) p;
	    KonoeInputStream::netToHost( (KInt*)p, &index );

            p += sizeof( KInt ); 
            entryname = p;
            p += strlen( p ) + 1;
	    value = p;

	    return READ_SUCCESS;
	}

}

KInt KonoeNameSpacePacket::write( KonoeSocketIO * socket ){

	KInt	st;

	if( buffer ){
	    st = socket -> send( buffer, datasize );
	    if( st < 0 ) {
		cerr << "KonoeNameSpacepacket::write.";
		perror( 0 );
		return st;
	    }
	    else if ( st == 0 ) {
		cerr << "KonoeNameSpacePacket::write. zero bytes sent" << endl;
		return st;
	    }
	    else if ( st < datasize ) {
		KInt	cnt;
		KChar	* p;

		cnt = datasize - st;
		p = buffer + st;
		st = socket -> send( p, cnt );
		if( st < 0 ) {
		    cerr << "KonoeNameSpacePacket::write.";
		    perror( 0 );
		    return st;
		}
		else if( st == 0 ) {
		    cerr << "KonoeNameSpacePacket::write. send retry failed" << endl;
		    return st;
		}
	    }
	    return st;
	}
	else {
	    cerr<<"KonoeNameSpacePacket. Error" 
		<<" null buffer. nothing sent." << endl;
	    return  -1;
	}
}
