//====================================================================
/*

	KonoeNameSpaceTable.cc

	Version 0.1	04-Dec-1997	H. Sakamoto
	Version 0.2.0	25-Apr-1998	H. Sakamoto
	Version 0.4.0	27-Aug-1998	S. Inaba
	Version 0.4.1	11-Nov-1998	H. Sakamoto
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.1	23-Nov-2000	H. Sakamoto

	----

	To be implemented, but not yet done...

	(2) Other type than KChar.
	(3) Hash table. search method.

*/
//====================================================================

#include	<system/KonoeNameSpaceTable.hh>
#include        <KonoeCategories.hh>
#include	<iostream.h>
#include	<string.h>

extern "C"	KInt	atol( KChar * );

const KChar     * KonoeNameSpaceTable::className = "KonoeNameSpaceTable";

KonoeNameSpaceTable::KonoeNameSpaceTable( KInt tablesize, KInt pkey ) :
        KonoeStreamableObject( KonoeCategories::NAMESPACE,
                        KonoeNameSpaceCM::CID_NAMESPACETABLE ) {

	key = pkey;
	size = tablesize;
	memory = new KonoeSharedMemory( key,
			tablesize * sizeof( KInt ), MEMORY_ACCESS );
	base = ( KInt * ) memory -> getBase( );
	if( base == 0 ) {
	    cerr << "KonoeNameSpaceTable constructor:"
			" Failed to open shared memory for key = "
			<< key << endl;
	    return;
	}

}

KonoeNameSpaceTable::~KonoeNameSpaceTable( ) {

	delete memory;

}

void	KonoeNameSpaceTable::init( ) {

	* ( base + OFS_NAMESPACE_ID ) = memory -> getKey( );
	* ( base + OFS_FREESIZE ) = 
		* ( base + OFS_SPACESIZE ) =
			memory -> getSize( ) / sizeof( KInt )
				- CONTROL_AREA_SIZE;
	* ( base + OFS_NUMENTRIES ) = 0;
	* ( base + OFS_FIRSTENTRY ) = 0;
	* ( base + OFS_LASTENTRY ) = 0;
	* ( base + OFS_FREESPACE ) = CONTROL_AREA_SIZE;
}

void	KonoeNameSpaceTable::dumpSummary( ostream & os ) {

	os <<"Name space table control parameters..." << endl;
	os << "  Namespace key = " << * ( base + OFS_NAMESPACE_ID ) << endl;
	os << "  Entry count = " << * ( base + OFS_NUMENTRIES ) << endl;
	os << "  Total space size = " << * ( base + OFS_SPACESIZE ) << endl;
	os << "  Free space size = " << * ( base + OFS_FREESIZE ) << endl;
	os << "  Next free space = " << * ( base + OFS_FREESPACE ) << endl;
	os << endl;
}

void	KonoeNameSpaceTable::dump( ostream & os ) {

	KInt	i, n, p, q, j, m;

	dumpSummary( os );

	os << "Name space entries..." << endl;

	n = * ( base + OFS_NUMENTRIES );
	p = * ( base + OFS_FIRSTENTRY );
	for( i = 0; i < n; i ++ ) {
	    os << "Entry name = "
		<< ( KChar * ) ( base + p + OFS_ENTRYNAME ) << endl;
	    m = * ( base + p + OFS_VALUECOUNT );
	    q = * ( base + p + OFS_FIRSTVALUE );
	    for( j = 0; j < m; j ++ ) {
		if( * ( base + q + OFS_VALUETYPE ) == TYPE_INT )
		    os << "  " << * ( base + * ( base + q + OFS_VALUE ) ) << endl;
		else if( * ( base + q + OFS_VALUETYPE ) == TYPE_CHAR )
		    os << "  " << ( KChar * ) ( base + * ( base + q + OFS_VALUE ) )
			<< endl;
		 q = * ( base + q + OFS_NEXTVALUE );
	    }
	    p = * ( base + p + OFS_NEXTENTRY );
	}
}

KInt	KonoeNameSpaceTable::addEntry( KChar * entryname ) {

	KInt	i, p, l;
	if( * ( base + OFS_FREESIZE ) < ENTRY_HEADER_SIZE )
	    return -1;	// error. header space not available
	i = * ( base + OFS_FREESPACE );
	if( * ( base + OFS_FIRSTENTRY ) == 0 ) {	// first
	    * ( base + OFS_FIRSTENTRY ) = i;
	    p = 0;
	}
	else {
	    p = * ( base + OFS_LASTENTRY );
	    * ( base + p + OFS_NEXTENTRY ) = i;
	}
	* ( base + OFS_LASTENTRY ) = i;
	( * ( base + OFS_NUMENTRIES ) ) ++;
	* ( base + OFS_FREESPACE ) += ENTRY_HEADER_SIZE;
	* ( base + OFS_FREESIZE ) -= ENTRY_HEADER_SIZE;

	* ( base + i + OFS_PREVENTRY ) = p;
	* ( base + i + OFS_NEXTENTRY ) = 0;
	l = strlen( entryname );
	if( l > 31 ) l = 31;
	strncpy( ( KChar * ) ( base + i + OFS_ENTRYNAME ), entryname, l );
	* ( ( KChar * ) ( base + i + OFS_ENTRYNAME ) + l ) = '\0';
	* ( base + i + OFS_NAMELENGTH ) = l;
	* ( base + i + OFS_VALUECOUNT ) = 0;
	* ( base + i + OFS_FIRSTVALUE ) = 0;
	* ( base + i + OFS_LASTVALUE ) = 0;
	return i;
}

KInt	KonoeNameSpaceTable::findEntry( KChar * entryname ) {

	KInt	i, n, p, l;

	l = strlen( entryname );
	if( l > 31 ) l = 31;

	n = * ( base + OFS_NUMENTRIES );
	p = * ( base + OFS_FIRSTENTRY );
	for( i = 0; i < n; i ++ ) {
	    if( strncmp( entryname,
		 (KChar * ) ( base + p + OFS_ENTRYNAME ),l) == 0 &&
		 l == strlen((KChar *)(base + p + OFS_ENTRYNAME)))
		return p;
	    p = * ( base + p + OFS_NEXTENTRY );
	}
	return -1;
}


KInt	KonoeNameSpaceTable::addValue( KInt offset, void * value, KInt type ) {

	KInt	l, a, v, p;

	switch( type ){
	case TYPE_CHAR :
	    a = strlen((KChar*)value)/sizeof( KInt ) + 1 + VALUE_HEADER_SIZE;
	    break;
	case TYPE_INT :
	    a = 1 + VALUE_HEADER_SIZE;
	    break;
	default : 
	    return -1;
	}
	if( a > * ( base + OFS_FREESIZE ) )
            return -1;  // cannot allocate space
	v = * ( base + OFS_FREESPACE );
        * ( base + OFS_FREESPACE ) += a;
        * ( base + OFS_FREESIZE ) -= a;

	if( * ( base + offset + OFS_VALUECOUNT ) == 0 ) {
            * ( base + offset + OFS_FIRSTVALUE ) = v;
            p = 0;
        }
        else {
            p = * ( base + offset + OFS_LASTVALUE );
            * ( base + * ( base + offset + OFS_LASTVALUE )
                + OFS_NEXTVALUE ) = v;
	}
	* ( base + offset + OFS_LASTVALUE ) = v;
        ( * ( base + offset + OFS_VALUECOUNT ) ) ++;
        * ( base + v + OFS_PREVVALUE ) = p;
        * ( base + v + OFS_NEXTVALUE ) = 0;
        * ( base + v + OFS_ALLOCATIONSIZE ) = a - VALUE_HEADER_SIZE;
        * ( base + v + OFS_VALUETYPE ) = type;
        * ( base + v + OFS_VALUE ) = v + VALUE_HEADER_SIZE;
	switch( type ){
	case( TYPE_INT ) :
            * ( base + v + VALUE_HEADER_SIZE ) =  *(KInt*)value ;
	    break;
	case( TYPE_CHAR ) :
            strcpy((KChar*)( base + v + VALUE_HEADER_SIZE ), (KChar*) value );
	    break;
	default :
	    return -1;
	}
        return v + VALUE_HEADER_SIZE;

}

KInt	KonoeNameSpaceTable::addValue( KInt offset, KChar * value ) {

	return addValue( offset, (void*) value, TYPE_CHAR );
}

KInt    KonoeNameSpaceTable::addValue( KInt offset, KInt value ) {

	return addValue( offset, (void*)&value, TYPE_INT );
}

KInt KonoeNameSpaceTable::addValue( KChar * entryname, void * value, 
					KInt type ){

	KInt offset = findEntry( entryname );
	if( offset == -1 ) {
            offset = addEntry( entryname );
            if( offset < 0 )
                return offset;
        }
	return addValue( offset, value, type );
}

KInt KonoeNameSpaceTable::addValue( KChar * entryname, KChar * value ){

	return addValue( entryname, (void*) value, TYPE_CHAR );
}

KInt KonoeNameSpaceTable::addValue( KChar * entryname, KInt value ){

	return addValue( entryname, (void*)&value, TYPE_INT );
}

KInt KonoeNameSpaceTable::addValue( KChar * entryname, 
				KonoeNameSpaceValue * value ){

	return addValue( entryname, value -> get(), value -> getValueType( ) );

}

KInt	KonoeNameSpaceTable::getValueCount( KInt offset ) {

	return * ( base + offset + OFS_VALUECOUNT );

}

KInt	KonoeNameSpaceTable::getValueCount( KChar * entryname ) {

	KInt	p;

	p = findEntry( entryname );
	if( p == -1 )
	    return -1;
	return getValueCount( p );
}

KInt	KonoeNameSpaceTable::setValue(
		KInt offset, KChar * value, KInt index ) {

	return setValue( offset, (void*)value, TYPE_CHAR, index );
}

KInt    KonoeNameSpaceTable::setValue(
                KInt offset, KInt value, KInt index ) {

	return setValue( offset, (void*)&value, TYPE_INT, index );
}

KInt    KonoeNameSpaceTable::setValue(
                KInt offset, void * value, KInt type, KInt index ) {

        KInt & valuecount = * ( base + offset + OFS_VALUECOUNT );
        KInt    a, l, p;

        if( valuecount > index ) {
	    switch(type){
	    case TYPE_INT :
            	a = 1 + VALUE_HEADER_SIZE;
		break;
	    case TYPE_CHAR :
            	l = strlen( (KChar*)value );
            	a = l /sizeof( KInt ) + 1 + VALUE_HEADER_SIZE;
		break;
	    default :
		return -1;
	    }
            p = get_value_by_index( offset, index );
            if( * ( base + p + OFS_ALLOCATIONSIZE ) >= a - VALUE_HEADER_SIZE ) {
		switch(type){
		case TYPE_INT :
		     *( base + * ( base + p + OFS_VALUE ) )
						 = * ( KInt * ) value;
		     break;
		case TYPE_CHAR :
                     strcpy( ( KChar * ) ( base + * ( base + p + OFS_VALUE ) ),
                        ( KChar *) value );
		     break;	
		default :
		     return -1;
		}
            }
            else {      // need to increase. make garbage
                if( * ( base + OFS_FREESIZE ) < a )
                    return -1;  // no space available
                * ( base + p + OFS_VALUE ) = * ( base + OFS_FREESPACE );
                * ( base + OFS_FREESPACE ) += a;
                * ( base + OFS_FREESIZE ) -= a;
		* ( base + p + OFS_ALLOCATIONSIZE ) = a - VALUE_HEADER_SIZE;
		switch(type){
		case TYPE_INT :
                     *( base + * ( base + p + OFS_VALUE ))
                                                 = * ( KInt * ) value;
                     break; 
                case TYPE_CHAR :
                     strcpy( ( KChar * ) ( base + * ( base + p + OFS_VALUE ) ),
                        ( KChar * ) value );
                     break;              
                default :
                     return -1;
		}
            }
            * ( base + p + OFS_VALUETYPE ) = type;
            return 0;
        }
        else if( valuecount == index ) {
            return addValue( offset, value, type );
        }
        else {
            return -1;
        }
}

KInt	KonoeNameSpaceTable::setValue(KChar * entryname, void * value, 
			KInt type, KInt index ){
	KInt	p;

	 p = findEntry( entryname );
        if( p == -1 ) {
            p = addEntry( entryname );
            if( p == -1 )
                return -1;
        }
	return setValue( p, value, type, index );
}


KInt	KonoeNameSpaceTable::setValue(KChar * entryname, 
				KonoeNameSpaceValue * value, KInt index){

	switch(value->getValueType()){

	case KonoeNameSpaceValue::TYPE_INT :
	//	return setValue(entryname, (KInt*)value->get(), index);

	case KonoeNameSpaceValue::TYPE_CHAR :
		return setValue(entryname, (KChar*)value->get(), index);
	default :
		return -1;
	}
}

KInt    KonoeNameSpaceTable::setValue(KInt offset,
                                KonoeNameSpaceValue * value, KInt index){

        switch(value->getValueType()){

        case KonoeNameSpaceValue::TYPE_INT :
        //      return setValue(offset, (KInt*)value->get(), index);

        case KonoeNameSpaceValue::TYPE_CHAR :
                return setValue(offset, (KChar*)value->get(), index);
        default :
                return -1;
        }
}

KInt	KonoeNameSpaceTable::get_value_by_index( KInt offset, KInt index ) {

	KInt	n, i, p;
	n = * ( base + offset + OFS_VALUECOUNT );
	if( n <= index )
	    return -1;
	p = * ( base + offset + OFS_FIRSTVALUE );
	for( i = 0; i < index; i ++ ) {
	    p = * ( base + p + OFS_NEXTVALUE );
	}
	return p;
}

KonoeNameSpaceValue * KonoeNameSpaceTable::getValue( KChar * entryname, 
							KInt index ) {

	KInt	p, v;

	p = findEntry( entryname );
	if( p == -1 )
	    return 0;
	return getValue( p, index);
}

KonoeNameSpaceValue * KonoeNameSpaceTable::getValue( KInt offset, 
							KInt index ) {

	KInt	p, type;
	p = get_value_by_index( offset, index );
	if( p == -1 )
	    return 0;
	type = *( base + p + OFS_VALUETYPE );
	KonoeNameSpaceValue * value = new KonoeNameSpaceValue();
	switch(type){
	case TYPE_INT:
	    value -> set(  * ( base + * ( base + p + OFS_VALUE ) ) );
	    break;
	case TYPE_CHAR:
	    value -> set( ( KChar * ) ( base + * (base + p + OFS_VALUE) ) );
	    break;
	default:
	    return 0;
	}
	return value;
}


KInt	KonoeNameSpaceTable::setValue(
		KChar * entryname, KChar * value, KInt index ) {

	return setValue( entryname, (void*)value, TYPE_CHAR, index );
}

KInt KonoeNameSpaceTable::setValue(
		KChar * entryname, KInt value, KInt index ){

        return setValue( entryname, (void*)&value, TYPE_INT, index );
}


KInt	KonoeNameSpaceTable::showEntry( ostream & os, KChar * entryname ) {

	KInt	p;
	p = findEntry( entryname );
	if( p == -1 )
	    return -1;
	return show_entry( os, p );
}

KInt	KonoeNameSpaceTable::show_entry( ostream & os, KInt offset ) {

	KInt	i, n;
	KonoeNameSpaceValue * v;

	n = getValueCount( offset );
	for( i = 0; i < n; i ++ ) {
	    if( i != 0 )
		os << ",";
	    v = getValue( offset, i );
	    switch( v -> getValueType() ){
	    case TYPE_INT :
	        os << *(KInt*)getValue( offset, i ) -> get( );
		break;
	    case TYPE_CHAR :
	        os << (KChar*)getValue( offset, i ) -> get( );
		break;
	    default:
		return -1;
	    }
	}
	os << endl;
	return 0;
}

KInt	KonoeNameSpaceTable::show( ostream & os ) {

	KInt	p, n, i;

	n = * ( base + OFS_NUMENTRIES );
	p = * ( base + OFS_FIRSTENTRY );
	for( i = 0; i < n; i ++ ) {
	    os << ( KChar * ) ( base + p + OFS_ENTRYNAME ) <<"=";
	    show_entry( os, p );
	    p = * ( base + p + OFS_NEXTENTRY );
	}
	return 0;    
}

KInt	KonoeNameSpaceTable::getFirstEntry( ) {

	return * ( base + OFS_FIRSTENTRY );

}

KInt	KonoeNameSpaceTable::getNextEntry( KInt offset ) {

	return * ( base + offset + OFS_NEXTENTRY );

}

KonoeNameSpaceItem * KonoeNameSpaceTable::getItem( KInt offset ) {

	KonoeNameSpaceItem	* e;
	KInt	i, n, p;

	e = new KonoeNameSpaceItem( );
	e -> setName( ( KChar * ) ( base + offset + OFS_ENTRYNAME ) );
	e -> allocate( n = * ( base + offset + OFS_VALUECOUNT ) );

	p = * ( base + offset + OFS_FIRSTVALUE );
	for( i = 0; i < n; i ++ ) {
	    e -> setValue( 
		( KChar * ) ( base +  * ( base + p + OFS_VALUE ) ), i );
	    p = * ( base + p + OFS_NEXTVALUE );
	}
	return e;

}

KonoeNameSpaceItem * KonoeNameSpaceTable::getItem( KChar * entryname) {

	KInt    offset = findEntry( entryname );
	if( offset == -1 ) return 0;
	return  getItem( offset );
}

KInt	KonoeNameSpaceTable::setEntry( KonoeNameSpaceItem * e ) {

	KInt	offset, i, n;

	offset = findEntry( e -> getName( ) );
	if( offset == -1 ) {
	    offset = addEntry( e -> getName( ) );
	    if( offset < 0 )
		return offset;
	}
	n = e -> getValueCount( );
	for( i = 0; i < n; i ++ ) {
	    setValue( offset, e -> getValue( i ) , i );
	}
	return 0;
}

KInt    KonoeNameSpaceTable::put(
                        KonoeOutputObjectStream * os,
                        KChar level, KInt memberid ) {

        KChar   l = level + 1;
        KInt    i, s, offset;
	KonoeNameSpaceItem      * item;

        os -> putHeader( this, level, memberid, N_MEMBERS );
	os -> put( l, MID_KEY, key );
        os -> put( l, MID_SIZE, size );
	s = 0;
	i = 0;
        offset = getFirstEntry( );
        while( offset > 0 ) {
            item = getItem( offset );
            if( item == 0 )
                break;
	    s += HEADER_SIZE + item -> getStreamSize( );
	    i++;
	    offset = getNextEntry( offset );
	}
	os -> putObjectArrayHeader( l, MID_ITEMS, i, s );

	offset = getFirstEntry( );
        while( offset > 0 ) {
            item = getItem( offset );
            if( item == 0 )
                break;
            item -> put( os, l+1, MID_ITEMS);
            offset = getNextEntry( offset );
        }

	return 0;
}

KInt    KonoeNameSpaceTable::getStreamSize( ) {

	KInt    i, s, offset;
        KonoeNameSpaceItem      * item;

	s = HEADER_SIZE;
	s += HEADER_SIZE + 1; // key
	s += HEADER_SIZE + 1; // size

        offset = getFirstEntry( );
        while( offset > 0 ) {
            item = getItem( offset );
            if( item == 0 )
                break;
	    s += HEADER_SIZE + item -> getStreamSize( );
            offset = getNextEntry( offset );
        }
        return s;
}

KInt    KonoeNameSpaceTable::fillPrimitiveMember( KInt memberid,
                        KInt datatype, KInt datacount, void * data ) {

        switch( datatype ) {
        case KINT_TYPE :
                switch( memberid ) {
                case MID_KEY :
                        key = * ( KInt * ) data;
                        delete [] data;
                        break;
                case MID_SIZE:
                        size = * ( KInt * ) data;
			delete memory;
			memory = new KonoeSharedMemory( key,
                        		size * sizeof( KInt ), MEMORY_ACCESS );
			base = ( KInt * ) memory -> getBase( );
		        if( base == 0 ) {
     		             cerr << "KonoeNameSpaceTable fillPrimitiveMember:"
                                " Failed to open shared memory for key = "
                        	<< key << endl;
            			return -1;
        		}
                        delete [] data;
                        break;
                default :
                        delete [] data;
                        return -1;
                }
                return 0;
        default :
                delete [] data;
                return -1;
        }    

}


KInt    KonoeNameSpaceTable::fillObjectMember( KInt memberid,
                        KonoeStreamableObject * data ) {

        if( memberid != MID_ITEMS ) {
                delete data;
                return -1;
        }
	setEntry( (KonoeNameSpaceItem*) data );
        return 0;
}

