//====================================================================
/*

	KonoeNameSpaceTable.hh

	Version 0.1	04-Dec-1997	H. Sakamoto
	Version 0.2.0	25-Apr-1998	H. Sakamoto
	Version	0.4.0	27-Aug-1998	S. Inaba
	Version 0.4.1	11-Nov-1998	H. Sakamoto
        Version 0.5.0   21-Sep-1999     Y. Kouno
        Version 0.6.0   20-Aug-2000     Y. Shimizu
	Version 0.6.1	23-Nov-2000	H. Sakamoto

*/
//====================================================================

#ifndef	__KONOENAMESPACETABLE_HH
#define	__KONOENAMESPACETABLE_HH

#include	<system/KonoeSharedMemory.hh>
#include	<system/KonoeNameSpaceItem.hh>
#include        <system/KonoeStreamableObject.hh>
#include	<iostream.h>

class	KonoeNameSpaceTable	: public KonoeStreamableObject {

public :
	enum	{	MEMORY_KEY = 10000,
			MEMORY_ACCESS = 0666,
			DEFAULT_TABLE_SIZE = 0x10000	};
	enum	{	SPACENAMELENGTH = 32,
			ENTRYNAMELENGTH = 32		};
	// control area ( table management information )
	enum	{	OFS_NAMESPACE_ID,
			OFS_SPACENAMELENGTH,
			OFS_SPACENAME,
			OFS_SPACESIZE = SPACENAMELENGTH / sizeof( KInt )
					+ OFS_SPACENAME,
			OFS_FREESIZE,
			OFS_NUMENTRIES,
			OFS_FIRSTENTRY,
			OFS_LASTENTRY,
			OFS_FREESPACE,
			CONTROL_AREA_SIZE	};
	// entry header
	enum	{	OFS_PREVENTRY,
			OFS_NEXTENTRY,
			OFS_NAMEHASH,
			OFS_NAMELENGTH,
			OFS_ENTRYNAME,
			OFS_VALUECOUNT = ENTRYNAMELENGTH / sizeof( KInt )
					+ OFS_ENTRYNAME,
			OFS_FIRSTVALUE,
			OFS_LASTVALUE,
			ENTRY_HEADER_SIZE	};
	// value header
	enum	{	OFS_PREVVALUE,
			OFS_NEXTVALUE,
			OFS_VALUETYPE,
			OFS_ALLOCATIONSIZE,
			OFS_VALUESIZE,
			OFS_VALUE,
			VALUE_HEADER_SIZE	};
	// value type
	enum	{	TYPE_INT,
			TYPE_CHAR		};

        enum    MEMBER_ID       { MID_KEY, MID_SIZE, MID_ITEMS, N_MEMBERS };

	static const KChar      * className;

protected :
	KonoeSharedMemory	* memory;
	KInt	* base;
	KInt	key;
	KInt	size;

	KInt	get_value_by_index( KInt offset, KInt idx );
	KInt	getValueCount( KInt offset );
	KonoeNameSpaceValue * getValue( KInt offset , KInt index = 0 );
	KInt	getFirstEntry( );
	KInt	getNextEntry( KInt offset );
	KInt	show_entry( ostream & os, KInt offset );
	KInt	findEntry( KChar * entryname );
	KInt	addValue( KInt offset, KChar * value );
	KInt	addValue( KInt offset, KInt value );
	KInt	addValue( KInt offset, void * value, KInt type );
	KInt	setValue( KInt offset, KChar * value, KInt index = 0 );
	KInt	setValue( KInt offset, KInt value, KInt index = 0 );
	KInt	setValue( KInt offset, void* value, KInt type, KInt index = 0 );
	KInt	setValue( KInt offset, KonoeNameSpaceValue * value, 
					KInt index = 0 );
	KInt	addEntry( KChar * entryname );
	KonoeNameSpaceItem	* getItem( KInt offset );

public :
	KonoeNameSpaceTable( KInt tablesize = DEFAULT_TABLE_SIZE,
				KInt key = MEMORY_KEY );
	~KonoeNameSpaceTable( );
	void	init( );
	void	dump( ostream & os );
	void	dumpSummary( ostream & os );
	KInt	getValueCount( KChar * entryname );
	KonoeNameSpaceValue * getValue( KChar * entryname , KInt index = 0 );

	KInt    addValue( KChar * entryname, void * value, KInt type );
        KInt    addValue( KChar * entryname, KChar * value );
        KInt    addValue( KChar * entryname, KInt value );
        KInt    addValue( KChar * entryname, KonoeNameSpaceValue * value );

	KInt    setValue( KChar* entryname, void* value, KInt type, 
                                KInt index = 0 );
	KInt	setValue( KChar * entryname, KChar * value, KInt index = 0 );
	KInt	setValue( KChar * entryname, KInt value, KInt index = 0 );
	KInt	setValue( KChar * entryname, KonoeNameSpaceValue * value, 
				KInt index = 0 );
	KInt	showEntry( ostream & os, KChar * entryname );
	KInt	show( ostream & os );

	KonoeNameSpaceItem	* getItem( KChar * entryname );
	KInt	setEntry( KonoeNameSpaceItem * e );

	       // to be streamable

        KInt    put(    KonoeOutputObjectStream * outstream,
                        KChar level, KInt memberid );
        KInt    getStreamSize( );
        KInt    fillPrimitiveMember( KInt memberid, KInt datatype,
                                KInt datacount, void * data );
        KInt    fillObjectMember( KInt memberid, KonoeStreamableObject * );

};

#endif
