//===================================================================
/*

	KonoeObjectServerSocketBuffered.cc

	Version 0.0.1	10-Sep-1999	Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
        Version 0.6.0    6-Nov-2000     Y. Shimizu 
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//===================================================================

#include	<system/KonoeObjectServerSocketBuffered.hh>

#include	<iostream.h>
#include	<string.h>
#include	<stdio.h>

KonoeObjectServerSocketBuffered::KonoeObjectServerSocketBuffered( KInt pport, KonoeFDSet * fds ) {

	for( KInt i = 0; i < MAX_CLIENTS; i ++ )
	    clients[ i ] = 0;

	socket = new KonoeServerSocket( pport );

	errornumber = 0;
	send_mode = SEND_TO_ALL;	// default

	if( fds )
	    readfds = fds;
	else
	    readfds = new KonoeFDSet( );
	readfds -> install( this );
}

KonoeObjectServerSocketBuffered::~KonoeObjectServerSocketBuffered( ) {

	for( KInt i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] )
		readfds -> remove( clients[ i ] );
	    delete clients[ i ];
	}

	readfds -> remove( this );

}

KonoeObjectSocketBuffered * KonoeObjectServerSocketBuffered::getConnection(
					const  KChar * hostname ) {

	KonoeSocketConnection * so;

	for( KInt i = 0; i < MAX_CLIENTS; i ++ ) {
            if( clients[ i ] ){
	        so = (KonoeSocketConnection * )( clients[ i ] -> getSocket( ) );
//cout<< so -> getClientName( ) << endl;
//cout<< hostname << endl;
		if( strcmp( hostname, so -> getClientName( ) ) == 0 ){
		    return clients[ i ];
		}
	    }
        }
	return 0;
}

KInt	KonoeObjectServerSocketBuffered::waitConnection( ) {

	KInt	i, j, st;
	KonoeSocketConnection	* c;
	KonoeCategoryManager	* cm;

	errornumber = 0;
	for( i = 0; i < MAX_CLIENTS; i ++ )
	    if( clients[ i ] == 0 )
		break;
	if( i == MAX_CLIENTS ) {
	    errornumber = E_TABLEFULL;
	    cerr << "KonoeObjectServerSocketBuffered::waitConnection. Client Table Full"
		<< endl;
	    return -1;	// client table full
	}

	c = new KonoeSocketConnection( );
        st = c -> accept( socket -> getFD( ) );
        if( st == -1 ) {
            errornumber = errno;
            perror( 0 );
            return -1;
        }

        clients[ i ] = new KonoeObjectSocketBuffered( c );
cout<<"KonoeObjectSocketBuffered object created."<<endl;
	if( clients[i] ){
	  cm_list -> reset( );
	  while( cm = cm_list -> next( ) ) {
	    clients[ i ] -> installCM( cm );
	  }

          readfds -> install( clients[ i ] );
	}
        return 0;

}

KInt	KonoeObjectServerSocketBuffered::closeConnection( ) {

	KInt	i;

	for( i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] )
		readfds -> remove( clients[ i ] );
	    delete clients[ i ];
	    clients[ i ] = 0;
	}
	return 0;
}

KInt	KonoeObjectServerSocketBuffered::close( KonoeObjectSocketBuffered * so ) {

	KInt	i;

	for( i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] == so ) {
		readfds -> remove( so );
		delete clients[ i ];
		clients[ i ] = 0;
		return 0;
	    }
	}
	errornumber = E_SOCKETNOTMATCH;
	return -1;	// socket connection not found
}

KInt	KonoeObjectServerSocketBuffered::send( KChar * buffer, KInt nbytes ) {

	KInt	i, s, st;

	switch( send_mode ) {
	case SEND_TO_LAST :
	    if( last_received ) {
		s = send_to_client( last_received, buffer, nbytes );
		if( s == -1 )
		    return s;
	    }
	    break;
	case SEND_TO_OTHERS :
	    st = 0;
	    for( i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] && ( clients[ i ] != last_received ) ) {
		    s = send_to_client( clients[ i ], buffer, nbytes );
		    if( s == -1 )
			st = s;
		}
	    return st;
	case SEND_TO_ALL:
	    st = 0;
	    for( i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] ) {
		    s = send_to_client( clients[ i ], buffer, nbytes );
		    if( s == -1 )
			st = s;
		}
	    return st;

	}
	return 0;
}

KInt    KonoeObjectServerSocketBuffered::put( KonoeStreamableObject * obj ){

        KInt    i, s, st;

        switch( send_mode ) {
        case SEND_TO_LAST :
            if( last_received ) {
                s = put_to_client( last_received, obj );
                if( s == -1 )
                    return s;
            }
            break;
        case SEND_TO_OTHERS :
            st = 0;
            for( i = 0; i < MAX_CLIENTS; i ++ )
                if( clients[ i ] && ( clients[ i ] != last_received ) ) {
                    s = put_to_client( clients[ i ], obj );
                    if( s == -1 )
                        st = s;
                }
            return st;
        case SEND_TO_ALL:
            st = 0;
            for( i = 0; i < MAX_CLIENTS; i ++ )
                if( clients[ i ] ) {
                    s = put_to_client( clients[ i ], obj );
                    if( s == -1 )
                        st = s;
                }
            return st;

        }
        return 0;
}

KInt	KonoeObjectServerSocketBuffered::send_to_client( KonoeObjectSocketBuffered * c,
		KChar * buffer, KInt nbytes ) {

	KInt	st;

	st = c -> write( buffer, nbytes );
	if( st == -1 ) {
	    errornumber = errno;
	    cerr << "KonoeServerSocket::send. Error = " << errno << endl;
	    perror( 0 );
	    for( KInt i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] == c ) {
		    delete clients[ i ];
		    clients[ i ] = 0;
		    if( last_received == c )
			last_received = 0;
		    break;
		}
	}
	return st;
}

KInt    KonoeObjectServerSocketBuffered::put_to_client( KonoeObjectSocketBuffered * c,
					KonoeStreamableObject * obj ){

        KInt    st;

        st = c -> put( obj );
        if( st == -1 ) {
            errornumber = errno;
            cerr << "KonoeObjectServerSocketBuffered::put_to_client. Error = "
		 << errno << endl;
            perror( 0 );
            for( KInt i = 0; i < MAX_CLIENTS; i ++ )
                if( clients[ i ] == c ) {
                    delete clients[ i ];
                    clients[ i ] = 0;
                    if( last_received == c )
                        last_received = 0;
                    break;
                }
        }
        return st;
}

KonoeObjectSocketBuffered	* KonoeObjectServerSocketBuffered::wait( KInt milisec ) {

//cout<<"objsersock wait" << endl;

	KInt	st;
	KonoeFDClient	* cl;

	errornumber = 0;

	cl = readfds -> wait( milisec );
	if( cl == this ) {
	    st = waitConnection( );
	    if( st == 0 )
		errornumber = E_CONNECTED;
	    return 0;
	}
	else if( cl ) {
//cout<<"cl exist" << endl;
	    last_received = ( KonoeObjectSocketBuffered * ) cl;
	    return last_received;
	}
	return 0;
}

KonoeObjectSocketBuffered	* KonoeObjectServerSocketBuffered::processReceiveAction(
				KonoeFDClient * client ) {

	KInt	st;

	if( client == this ) {
	    st = waitConnection( );
	    if( st == 0 )
		errornumber = E_CONNECTED;
	    return 0;
	}

	KInt	i;
	for( i = 0; i < MAX_CLIENTS; i ++ )
	    if( clients[ i ] == client ) {
		return ( KonoeObjectSocketBuffered * ) client;
	    }

	return ( KonoeObjectSocketBuffered * ) -1;

}


KInt	KonoeObjectServerSocketBuffered::receive( KChar * buffer, KInt buffersize ) {

	KonoeObjectSocketBuffered	* c;

	errornumber = 0;
	c = wait( WAIT_INTERVAL );
	if( c == 0 ) {
	    if( errornumber != 0 && errornumber != E_CONNECTED )
		return -1;
	    return 0;
	}

	return receiveFromConnection( c, buffer, buffersize );

}

KonoeStreamableObject * KonoeObjectServerSocketBuffered::get( ){

        KonoeObjectSocketBuffered       * c;

        errornumber = 0;
        c = wait( WAIT_INTERVAL );
        if( c == 0 ) {
            if( errornumber != 0 && errornumber != E_CONNECTED )
                return ( KonoeStreamableObject * )-1;
            return 0;
        }

        return getFromConnection( c );

}

KInt	KonoeObjectServerSocketBuffered::receiveFromConnection( KonoeObjectSocketBuffered * c,
		KChar * buffer, KInt buffersize ) {

	KInt	st;

	st = c -> read( buffer, buffersize );
	if( st == 0 ) {
	    for( KInt i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] == c ) {
		    readfds -> remove( c );
		    delete clients[ i ];
		    clients[ i ] = 0;
		    errornumber = E_DISCONNECTED;
		    return 0;	// disconnected = success.
		}
	}
	else if( st == -1 )
	  ;//errornumber = c -> getError( );

	return st;

}

KonoeStreamableObject * KonoeObjectServerSocketBuffered::getFromConnection( 
						KonoeObjectSocketBuffered * c ){

	KonoeStreamableObject * obj;

        obj = c -> get( );
        if( obj  == 0 ) {
            for( KInt i = 0; i < MAX_CLIENTS; i ++ )
                if( clients[ i ] == c ) {
                    readfds -> remove( c );
                    delete clients[ i ];
                    clients[ i ] = 0;
                    errornumber = E_DISCONNECTED;
                    return 0;   // disconnected = success.
                }
        }
        //else if( st == -1 )
        // errornumber = c -> getError( );

        return obj;

}

