//==================================================================
/*

	KonoeObjectSocketBuffered.cc

	Version 0.2.0	24-Apr-1998	H. Sakamoto
	Version 0.4.0	27-Aug-1998	S. Inaba
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	02-Dec-2000	H. Sakamoto
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//==================================================================

#include	<system/KonoeObjectSocketBuffered.hh>
#include	<system/KonoeClientSocket.hh>
#include	<system/KonoeServerSocket.hh>

#include	<string.h>
#include	<iostream.h>

KonoeObjectSocketBuffered::KonoeObjectSocketBuffered( KChar * host, KInt port ) :
	KonoeObjectSocketBase( ), KonoeInputObjectStream( ),
	KonoeOutputObjectStream( ) {

	socket = new KonoeClientSocket( host, port );
	initialize( );

}

KonoeObjectSocketBuffered::KonoeObjectSocketBuffered( KInt port ) :
	KonoeObjectSocketBase( ), KonoeInputObjectStream( ),
	KonoeOutputObjectStream( ) {

	socket = new KonoeServerSocket( port );
	initialize( );

}

KonoeObjectSocketBuffered::KonoeObjectSocketBuffered( KonoeSocketIO * s ) :
	KonoeObjectSocketBase( ), KonoeInputObjectStream( ),
	KonoeOutputObjectStream( ) {

	socket = s;
	initialize( );

}

void	KonoeObjectSocketBuffered::initialize( ) {

	send_buffer_size = SEND_BUFFER_SIZE;
	send_buffer_pointer = 0;
	receive_buffer_size = RECEIVE_BUFFER_SIZE;
	receive_buffer_pointer = receive_buffer_size;
	receive_buffer_bytecount = receive_buffer_size;

}

KonoeObjectSocketBuffered::~KonoeObjectSocketBuffered( ) {

	if( send_buffer_pointer > 0 ) {
	    socket -> send( send_buffer, send_buffer_pointer );
	}
	delete socket;

}

KInt	KonoeObjectSocketBuffered::writeToStream( const KChar * buffer, KInt nbytes ) {


	KInt	i, j, st;
	KChar	* p;
	if( nbytes < send_buffer_size - send_buffer_pointer ) {
	    memcpy( & send_buffer[ send_buffer_pointer ], buffer, nbytes );
	    send_buffer_pointer += nbytes;
	    return nbytes;
	}
	else {
	    i = send_buffer_size - send_buffer_pointer;
	    memcpy( & send_buffer[ send_buffer_pointer ], buffer, i );
	    st = socket -> send( send_buffer, send_buffer_size );
	    if( st < 0 )
		return st;
	    j = nbytes - i;
	    if( j > 0 ) {
		while( j >= send_buffer_size ) {
		    memcpy( send_buffer, buffer + i, send_buffer_size );
		    st = socket -> send( send_buffer, send_buffer_size );
		    if( st < 0 )
			return st;
		    i += send_buffer_size;
		    j -= send_buffer_size;
		}
		memcpy( send_buffer, buffer + i, j );
	    }
	    send_buffer_pointer = j;
	    return nbytes;
	}

}

KInt	KonoeObjectSocketBuffered::flush( ) {

	KInt	st;

	if( send_buffer_pointer == 0 )
	    return 0;
	st = socket -> send( send_buffer, send_buffer_pointer );
	if( st < 0 )
	    return st;
	send_buffer_pointer = 0;
	return st;

}

KInt	KonoeObjectSocketBuffered::readFromStream( KChar * buffer, KInt buffersize ) {

	KInt	i, j, st;
	i = receive_buffer_bytecount - receive_buffer_pointer;
	if( i >= buffersize ) {
	    memcpy( buffer, & receive_buffer[ receive_buffer_pointer ], 
			buffersize );
	    receive_buffer_pointer += buffersize;
	    return buffersize;
	}
	else {
	    if( i > 0 )
		memcpy( buffer, & receive_buffer[ receive_buffer_pointer ],
			i );
	    st = socket -> receive( receive_buffer, receive_buffer_size );
	    if( st < 0 )
		return st;
	    receive_buffer_bytecount = st;
	    j = buffersize - i;
	    if( j > st ) {
		memcpy( buffer + i, receive_buffer, st );
		receive_buffer_pointer = st;
		return i + st;
	    }
	    else {
		memcpy( buffer + i, receive_buffer, j );
		receive_buffer_pointer = j;
		return buffersize;
	    }

	}

}


