//====================================================================
/*

	KonoeInputObjectStream.cc

	Version 0.1.0	21-Oct-1997	H. Sakamoto
	Version 0.1.1	30-Oct-1997	H. Sakamoto
	Version 0.2.0	27-Apr-1998	H. Sakamoto
	Version 0.4.0	01-Sep-1998	S. Inaba	 
	Version 0.4.1	09-Sep-1999	S. Inaba	 
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	02-Dec-2000	H. Sakamoto
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//===================================================================

#include	<system/KonoeInputObjectStream.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<KonoeCategories.hh>
#include	<iostream.h>

KonoeInputObjectStream::KonoeInputObjectStream( ) :
	KonoeObjectStreamBase( ) {

	level = TOP_LEVEL;

}


KonoeStreamableObject	* KonoeInputObjectStream::get(
		KonoeStreamableObject * parent,
		KInt arraymember ) {

	KInt	buffer[ 3 ];
	KInt	header1;
	KInt	header2;
	KInt	datasize;

	KonoeCategoryManager	* cm;
	KInt	categoryid;
	KInt	classid;
	KInt	memberid;
	KInt	elementcount;

	KInt	st, ist;

	KonoeStreamableObject	* p;

	if( read( buffer, 3 ) <= 0 ) {
	    status = ERROR_READFAIL;
	    return 0;	// error reading header
	}

	header1 = buffer[ 0 ];
	header2 = buffer[ 1 ];
	datasize = buffer[ 2 ];

	memberid = ( header2 & MEMBER_ID_MASK ) >> 24;
	if( arraymember != -1 )
		memberid = arraymember;

	elementcount = header2 & DATA_COUNT_MASK;

	categoryid = header1 & CATEGORY_MASK;
	classid = header1 & CLASS_MASK;

	if( categoryid == KonoeCategories::PRIMITIVE ) {
	    switch( classid ) {
	    case KINT_CLASS :
		{
		    KInt * ibuffer;
		    ibuffer = new KInt[ elementcount ];
		    if( read( ibuffer, elementcount ) <= 0 ) {
			status = ERROR_READFAIL;
			delete [] ibuffer;
			return 0;
		    }
		    if( parent ) {
			st = parent -> fillPrimitiveMember( memberid, 
					KonoeStreamableObject::KINT_TYPE,
					elementcount, ibuffer );
			if( st == -1 ) {
			    cerr << "Error in filling member" << endl;
			}
		    }
		    return parent;
		}
	    case KFLOAT_CLASS :
		{
		    KFloat	* v;
		    v = new KFloat[ elementcount ];
		    if( read( v, elementcount ) <= 0 ) {
			status = ERROR_READFAIL;
			delete [] v;
			return 0;
		    }
		    if( parent ) {
			st = parent -> fillPrimitiveMember( memberid,
					KonoeStreamableObject::KFLOAT_TYPE,
					elementcount, v );
			if( st == -1 ) {
			    cerr << "Error in filling member" << endl;
			}
		    }
		    return parent;
		}
	    case KCHAR_CLASS :
		{
		    KChar	* v;
		    KInt	l;
		    l = ( elementcount / sizeof( KInt ) + 1 ) * sizeof( KInt );
		    v = new KChar[ l ];
		    if( read( v, l ) <= 0 ) {
			status = ERROR_READFAIL;
			delete [] v;
			return 0;
		    }
		    if( parent ) {
			st = parent -> fillPrimitiveMember( memberid,
					KonoeStreamableObject::KCHAR_TYPE
					, l, v );
			if( st == -1 ) {
			    cerr << "Error in filling member" << endl;
			}
		    }
		    return parent;
		}
	    case VOID_CLASS :
		{
		    void	* v;
		    KInt	l;
		    l = ( elementcount / sizeof( KInt ) + 1 ) * sizeof( KInt );
		    v = new KChar[ l ];
		    if( read( ( KChar* ) v, l ) <= 0 ){
			status = ERROR_READFAIL;
			delete [] v;
			return 0;
		    }
		    if( parent ){
			st = parent -> fillPrimitiveMember( memberid,
					KonoeStreamableObject::VOID_TYPE,
					elementcount, v);
			if( st == -1 ){
			    cerr << "Error in filling member" << endl;
			}
		    }
		    return parent;
		}
	    default :
		cerr << "Sorry. Not yet implemented" << endl;
		return 0;
	    }
	}
	else if( categoryid == KonoeCategories::OBJECTARRAY ) {
	    if( parent ) {
		level ++;
		for( KInt i = 0; i < elementcount; i ++ ) {
		    if( get( parent, memberid ) == 0 )
			return 0;
		}
		level --;
	    }
	    else
		skip( datasize );
	    return parent;
	}
	else {	// class objects...
	    cm_list -> reset( );	// parse category managers
	    while( cm = cm_list -> next( ) ) {
		if( cm -> match( categoryid ) ) break;
	    }
	    if( cm == 0 ) {
		skip( datasize );
		status = ERROR_CATEGORYUNKNOWN;
		return 0;	// category not found
	    }
	    p = cm -> create( classid ); // ! need version check.
	    level ++;
	    for( KInt i = 0; i < elementcount; i ++ ) {
		if( get( p ) == 0 ) return 0;
	    }
	    level --;
	    if( parent ) {
		st = parent -> fillObjectMember( memberid, p );
		if( st == -1 ) {
		    cerr << "Error in filling member" << endl;
		}
	    }
	    return p;
	}

	return 0;
}

KInt KonoeInputObjectStream::skip( KInt datacount ) {

	KInt	dummy, i;

	for( i = 0; i < datacount; i ++ )
	    read( & dummy, 1 );

	return 0;

}

KInt	KonoeInputObjectStream::analyze( ostream & os, KInt arraymember ) {
	KInt	buffer[ 3 ];
	KInt	header1;
	KInt	header2;
	KInt	datasize;

	KonoeCategoryManager	* cm;
	KInt	categoryid;
	KInt	classid;
	KInt	memberid;
	KInt	elementcount;

	KInt	i, st, ist, indent;

	KonoeStreamableObject	* p;

//checking readcount. this should be included in 'read' method.

	if( read( buffer, 3 ) <= 0 ) {
	    status = ERROR_READFAIL;
	    return -1;	// error reading header
	}

	header1 = buffer[ 0 ];
	header2 = buffer[ 1 ];
	datasize = buffer[ 2 ];

	indent = ( header1 >> 24 ) - 0x41;

	os << hex;
	for( i = 0; i < indent; i ++ ) os << "  ";
	os << "Header 1 = " << header1 << " Header 2 = " << header2
		<< " Datasize = " << datasize << endl;

	memberid = ( header2 & MEMBER_ID_MASK ) >> 24;
	if( arraymember != -1 )
		memberid = arraymember;

	elementcount = header2 & DATA_COUNT_MASK;

	categoryid = header1 & CATEGORY_MASK;
	classid = header1 & CLASS_MASK;

	if( categoryid == KonoeCategories::PRIMITIVE ) {
	    switch( classid ) {
	    case KINT_CLASS :
		{
		    KInt * ibuffer;
		    ibuffer = new KInt[ elementcount ];
		    if( read( ibuffer, elementcount ) <= 0 ) {
			status = ERROR_READFAIL;
			delete [] ibuffer;
			return -1;
		    }
		    //print out ibuffer
		    delete [] ibuffer;
		    return 0;
		}
	    case KFLOAT_CLASS :
		{
		    KFloat	* v;
		    v = new KFloat[ elementcount ];
		    if( read( v, elementcount ) <= 0 ) {
			status = ERROR_READFAIL;
			delete [] v;
			return -1;
		    }
		    // print out v
		    delete [] v;
		    return 0;
		}
	    case KCHAR_CLASS :
		{
		    KChar	* v;
		    KInt	l;
		    l = ( elementcount / sizeof( KInt ) + 1 ) * sizeof( KInt );
		    v = new KChar[ l ];
		    if( read( v, l ) <= 0 ) {
			status = ERROR_READFAIL;
			return 0;
		    }
		    // print out here
		    delete [] v;
		    return 0;
		}
	    default :
		cerr << "Sorry. Not yet implemented" << endl;
		return 0;
	    }
	}
	else if( categoryid == KonoeCategories::OBJECTARRAY ) {
		for( KInt i = 0; i < elementcount; i ++ ) {
		    if( analyze( os, memberid ) == -1 )
			return -1;
		}
		return 0;
	}
	else {	// class objects...

	    cm_list -> reset( );	// parse category managers
	    while( cm = cm_list -> next( ) ) {
		if( cm -> match( categoryid ) ) break;
	    }
	    if( cm == 0 ) {
		status = ERROR_CATEGORYUNKNOWN;
	    }
	    // print something on cm
	    for( KInt i = 0; i < elementcount; i ++ ) {
		if( analyze( os ) == -1 ) return -1;
	    }
	    return 0;
	}
}
