//====================================================================
/*

	KonoeOutputObjectStream.cc

	Version 0.1	15-Oct-1997	H. Sakamoto
	Version 0.2	30-Oct-1997	H. Sakamoto
	Version 0.4.0	01-Sep-1998	S. Inaba
	Version 0.4.1	09-Sep-1999	Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	01-Dec-2000	H. Sakamoto
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//====================================================================

#include	<system/KonoeOutputObjectStream.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<KonoeCategories.hh>
#include	<string.h>

const KInt	KonoeOutputObjectStream::version = KonoeOutputObjectStream::CURRENT_VERSION;

KonoeOutputObjectStream::KonoeOutputObjectStream( ) :
		KonoeObjectStreamBase( ) {
}

KInt	KonoeOutputObjectStream::put(
		KonoeStreamableObject * pobject, KChar plevel, KInt memberid ) {

	return pobject -> put( this, plevel, memberid );

}

KInt	KonoeOutputObjectStream::putHeader(
		KonoeStreamableObject * pobject,
		KChar plevel, KInt pmemberid, KInt pdatacount ) {

	KInt	h1, h2, s, st;

	if( pmemberid > 255 ) return -1;
	if( pdatacount > 0xffffff ) return -1;

	h1 = pobject -> getStreamHeader( plevel );
	st = write( h1 );
	if( st < 0 )
	    return st;
	h2 = ( ( pmemberid & 0xff ) << 24 ) | ( pdatacount & 0xffff );
	st = write( h2 );
	if( st < 0 )
	    return st;
	s = pobject -> getStreamSize( );
	st = write( s );
	return st;

}

KInt	KonoeOutputObjectStream::putIntegerHeader(
		KChar level, KInt memberid, KInt datacount ) {

	KInt	h1, h2, s, st;

	if( memberid > 255 ) return -1;
	if( datacount > 0xffffff ) return -1;

	h1 = ( level << 24 ) | KonoeCategories::PRIMITIVE | KINT_CLASS;
	st = write( h1 );
	if( st < 0 ) return st;
	h2 = ( ( memberid & 0xff ) << 24 ) | datacount;
	st = write( h2 );
	if( st < 0 ) return st;
	s = datacount;	// datacount * sizeof( KInt ) / sizeof( KInt )
	st = write( s );
	return st;
}

KInt	KonoeOutputObjectStream::putFloatHeader(
		KChar level, KInt memberid, KInt datacount ) {

	KInt	h1, h2, s, st;

	if( memberid > 255 ) return -1;
	if( datacount > 0xffffff ) return -1;

	h1 = ( level << 24 ) | KonoeCategories::PRIMITIVE | KFLOAT_CLASS;
	st = write( h1 );
	if( st < 0 ) return st;
	h2 = ( ( memberid & 0xff ) << 24 ) | datacount;
	st = write( h2 );
	if( st < 0 ) return st;
	s = datacount * sizeof( KFloat ) / sizeof( KInt );
	st = write( s );
	return st;
}

KInt	KonoeOutputObjectStream::putStringHeader(
		KChar level, KInt memberid, KInt datacount ) {

	KInt	h1, h2, s, st;

	if( memberid > 255 ) return -1;
	if( datacount > 0xffffff ) return -1;

	h1 = ( level << 24 ) | KonoeCategories::PRIMITIVE | KCHAR_CLASS;
	st = write( h1 );
	if( st < 0 ) return st;
	h2 = ( ( memberid & 0xff ) << 24 ) | datacount;
	st = write( h2 );
	if( st < 0 ) return st;
	s = datacount / sizeof( KInt ) + 1;
	st = write( s );
	return st;
}

KInt    KonoeOutputObjectStream::putVoidHeader(
                KChar level, KInt memberid, KInt datacount ) {

        KInt    h1, h2, s, st;

        if( memberid > 255 ) return -1;
        if( datacount > 0xffffff ) return -1;

        h1 = ( level << 24 ) | KonoeCategories::PRIMITIVE | VOID_CLASS;
        st = write( h1 );
	if( st < 0 ) return st;
        h2 = ( ( memberid & 0xff ) << 24 ) | datacount;
        st = write( h2 );
	if( st < 0 ) return st;
        s = datacount / sizeof( KInt ) + 1;
        st = write( s );
        return st;
}


KInt	KonoeOutputObjectStream::putObjectArrayHeader(
		KChar level, KInt memberid, KInt datacount, KInt datasize ) {

	KInt	h1, h2, st;

	if( memberid > 255 ) return -1;
	if( datacount > 0xffffff ) return -1;

	h1 = ( level << 24 ) | KonoeCategories::OBJECTARRAY;
	st = write( h1 );
	if( st < 0 ) return st;
	h2 = ( ( memberid & 0xff ) << 24 ) | datacount;
	st = write( h2 );
	if( st < 0 ) return st;
	st = write( datasize );
	return st;
}

KInt	KonoeOutputObjectStream::put( KChar level, KInt memberid,
		KInt ivalue ) {

	KInt	s;

	s = putIntegerHeader( level, memberid, 1 );
	if( s < 0 ) return s;
	return write( ivalue );
}

KInt	KonoeOutputObjectStream::put( KChar level, KInt memberid,
		KInt * ivalues, KInt datacount ) {

	KInt	s;

	s = putIntegerHeader( level, memberid, datacount );
	if( s < 0 ) return s;
	return write( ivalues, datacount );

}

KInt	KonoeOutputObjectStream::put( KChar level, KInt memberid,
		KFloat value ) {

	KInt	s;

	s = putFloatHeader( level, memberid, 1 );
	if( s < 0 ) return s;
	return write( value );
}

KInt	KonoeOutputObjectStream::put( KChar level, KInt memberid,
		KFloat * buffer, KInt datasize ) {

	KInt	s;

	s = putFloatHeader( level, memberid, datasize );
	if( s < 0 ) return s;
	return write( buffer, datasize );
}

KInt	KonoeOutputObjectStream::put( KChar level, KInt memberid,
		KChar * string ) {

	KInt	s, l, n, zero = 0;

	l = strlen( string );
	s = putStringHeader( level, memberid, l );
	if( s < 0 ) return s;
	n = l / sizeof( KInt ) + 1;
	n *= sizeof( KInt );
	s = write( string, l );
	if( s < 0 )
	    return s;
	return write( (KChar*)&zero, n - l );
}

KInt    KonoeOutputObjectStream::put( KChar level, KInt memberid,
                void * buffer, KInt datasize ) {

	KInt	s, l, n, zero = 0;

        s = putVoidHeader( level, memberid, datasize );
        if( s < 0 ) return s;
	n = datasize / sizeof( KInt ) + 1;
	n *= sizeof( KInt );
	s = write( (KChar*) buffer, datasize );
	if( s < 0 )
	    return s;
	return write( (KChar*) & zero, n - datasize );
}

