//===================================================================
/*

	KonoeClientSocketOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeClientSocketOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<string.h>

const KChar	* KonoeClientSocketOrderly::classname = "ClientSocket";


KonoeClientSocketOrderly::KonoeClientSocketOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "hostname", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "port", INTEGER, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "connect", N_ARG_CONNECT );
        addCommand( new ConnectUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeClientSocketOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeClientSocketOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeClientSocketOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );
	return 0;
}

KonoeClientSocketOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeClientSocketOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeClientSocket	* target;

	if( ! checkArgument( cmd ) ) return 0;

	//target = ( KonoeClientSocket * ) owner -> getTarget( );
	//target -> dump( );
	os << "KonoeClientSocketOrderly" << endl;

	return 0;
}

KonoeClientSocketOrderly::ConnectUnit::ConnectUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeClientSocketOrderly::ConnectUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeClientSocket *target = ( KonoeClientSocket * ) owner -> getTarget( );
	target -> connect( );

	return 0;
}

KInt	KonoeClientSocketOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OBJECT );
        const KChar *objectname = arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_HOSTNAME );
        const KChar *hostname = arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_PORT );
        KInt port = arg -> toInteger( );

	static KChar nonconst_hostname[256];
	strcpy(nonconst_hostname, hostname);

	target = new KonoeClientSocket( nonconst_hostname, port );

	return 0;

}

KInt	KonoeClientSocketOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeClientSocketOrderly::clone( const KChar * name ) {

	return new KonoeClientSocketOrderly( name );

}
