//===================================================================
/*

	KonoeInputObjSocketOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeInputObjSocketOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<system/KonoeSocketIO.hh>

const KChar	* KonoeInputObjSocketOrderly::classname = "InputObjSocket";


KonoeInputObjSocketOrderly::KonoeInputObjSocketOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "socketio", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "get", N_ARG_GET );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "io_object", STRING, NECESSARY ) );
        addCommand( new GetUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeInputObjSocketOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeInputObjSocketOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeInputObjSocketOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );
	return 0;
}

KonoeInputObjSocketOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeInputObjSocketOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeInputObjSocket	* target;

	if( ! checkArgument( cmd ) ) return 0;

	//target = ( KonoeInputObjSocket * ) owner -> getTarget( );
	//target -> dump( );
	os << "KonoeInputObjSocketOrderly" << endl;

	return 0;
}

KonoeInputObjSocketOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeInputObjSocketOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_GET_IOOBJECT );
        const KChar *io_obj_name = arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * io_obj_orderly = commander -> findObject(io_obj_name);

	if( io_obj_orderly == 0 ){
	  os << io_obj_name << " doesn't exist." << endl;
	  return 0;
	}

	KonoeInputObjSocket *target = ( KonoeInputObjSocket * ) owner -> getTarget( );
	KonoeStreamableObject *io_obj = target -> get( );
	io_obj_orderly -> setTarget( io_obj );

	return 0;
}

KInt	KonoeInputObjSocketOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OBJECT );
        const KChar *objectname = arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_SOCKETIO );
        const KChar *socketio_name = arg -> getValue( );

	KonoeCommander *commander = getOfficer( ) -> getCommander( );
	KonoeOrderly * socketio_orderly = commander -> findObject(socketio_name);

	if( socketio_orderly == 0 ){
	  os << socketio_name << " doesn't exist." << endl;
	  return 0;
	}

	KonoeSocketIO *socketio = ( KonoeSocketIO * ) socketio_orderly -> getTarget( );
	target = new KonoeInputObjSocket( socketio );

	return 0;

}

KInt	KonoeInputObjSocketOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeInputObjSocketOrderly::clone( const KChar * name ) {

	return new KonoeInputObjSocketOrderly( name );

}
