//===================================================================
/*

	KonoeNameSpaceTableOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.6.0	15-Nov-2000	H. Sakamoto
	Version 0.6.1	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeNameSpaceTableOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<string.h>

const KChar	* KonoeNameSpaceTableOrderly::classname = "NameSpaceTable";


KonoeNameSpaceTableOrderly::KonoeNameSpaceTableOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descriptor;
	KonoeArgumentDescriptor	* argd;

	cmd_descriptor = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	cmd_descriptor -> setDescription( "Diplay a name table entry." );
	argd = new KonoeArgumentDescriptor( "name", STRING, OPTIONAL );
	argd -> setDescription( "Name space item to be displayed." );
	cmd_descriptor -> add( argd );
	addCommand( new ShowUnit( cmd_descriptor, this ) );

	cmd_descriptor = new KonoeCommandDescriptor( "get", N_ARG_GET );
	cmd_descriptor -> setDescription( "Display a name space entry value" );
	argd = new KonoeArgumentDescriptor( "name", STRING, NECESSARY );
	argd -> setDescription( "Name of entry to be retrieved." );
	cmd_descriptor -> add( argd );

        addCommand( new GetUnit( cmd_descriptor, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeNameSpaceTableOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeNameSpaceTableOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeNameSpaceTableOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeNameSpaceTable	* target;

	if( ! checkArgument( cmd ) ) return 0;

	os << "KonoeNameSpaceTableOrderly" << endl;

	return 0;
}

KonoeNameSpaceTableOrderly::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeNameSpaceTableOrderly::GetUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_GET_NAME );
        KChar * name = ( KChar * ) arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );

	KonoeNameSpaceTable *target =
		( KonoeNameSpaceTable * ) owner -> getTarget( );

	KChar	* value = ( KChar * ) target -> getValue( name ) -> get( );
	os << name << " = " << value << endl;

	return 0;
}

KInt	KonoeNameSpaceTableOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	os << "NameSpaceTable cannot be created." << endl;

	return 0;

}

KInt	KonoeNameSpaceTableOrderly::deleteObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	os << "NameSpaceTable cannot be deleted." << endl;

	return 0;

}

KonoeOrderly	* KonoeNameSpaceTableOrderly::clone( const KChar * name ) {

	return new KonoeNameSpaceTableOrderly( name );

}
