//===================================================================
/*

	KonoeOutputObjFileOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeOutputObjFileOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<string.h>

const KChar	* KonoeOutputObjFileOrderly::classname = "OutputObjFile";


KonoeOutputObjFileOrderly::KonoeOutputObjFileOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "file", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "put", N_ARG_PUT );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "io_object", STRING, NECESSARY ) );
        addCommand( new PutUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeOutputObjFileOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOutputObjFileOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeOutputObjFileOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );
	return 0;
}

KonoeOutputObjFileOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeOutputObjFileOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeOutputObjFile	* target;

	if( ! checkArgument( cmd ) ) return 0;

	//target = ( KonoeOutputObjFile * ) owner -> getTarget( );
	//target -> dump( );
	os << "KonoeOutputObjFileOrderly" << endl;

	return 0;
}

KonoeOutputObjFileOrderly::PutUnit::PutUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeOutputObjFileOrderly::PutUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_PUT_IOOBJECT );
        const KChar *io_obj_name = arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * io_obj_orderly = commander -> findObject(io_obj_name);
	KonoeStreamableObject *io_obj = ( KonoeStreamableObject * ) io_obj_orderly -> getTarget( );

	if( io_obj_orderly == 0 ){
	  os << io_obj_name << " doesn't exist." << endl;
	  return 0;
	}

	KonoeOutputObjFile *target = ( KonoeOutputObjFile * ) owner -> getTarget( );
	target -> put( io_obj, 'A', 0 );

	return 0;
}

KInt	KonoeOutputObjFileOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OBJECT );
        const KChar *objectname = arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_FILE );
        const KChar *filename = arg -> getValue( );

	static KChar nonconst_filename[256];
	strcpy(nonconst_filename, filename);

	target = new KonoeOutputObjFile( nonconst_filename );

	return 0;

}

KInt	KonoeOutputObjFileOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeOutputObjFileOrderly::clone( const KChar * name ) {

	return new KonoeOutputObjFileOrderly( name );

}

