//===================================================================
/*

	KonoeOutputObjSocketOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeOutputObjSocketOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeStreamableObject.hh>
#include	<system/KonoeSocketIO.hh>


const KChar	* KonoeOutputObjSocketOrderly::classname = "OutputObjSocket";


KonoeOutputObjSocketOrderly::KonoeOutputObjSocketOrderly(
	const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "socketio", STRING, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "put", N_ARG_PUT );
	cmd_descripter -> add(
		new KonoeArgumentDescriptor( "io_object", STRING, NECESSARY ) );
        addCommand( new PutUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeOutputObjSocketOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOutputObjSocketOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeOutputObjSocketOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );
	return 0;
}

KonoeOutputObjSocketOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeOutputObjSocketOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeOutputObjSocket	* target;

	if( ! checkArgument( cmd ) ) return 0;

	//target = ( KonoeOutputObjSocket * ) owner -> getTarget( );
	//target -> dump( );
	os << "KonoeOutputObjSocketOrderly" << endl;

	return 0;
}

KonoeOutputObjSocketOrderly::PutUnit::PutUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeOutputObjSocketOrderly::PutUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_PUT_IOOBJECT );
        const KChar *io_obj_name = arg -> getValue( );

	KonoeCommander *commander = owner -> getOfficer( ) -> getCommander( );
	KonoeOrderly * io_obj_orderly = commander -> findObject(io_obj_name);
	KonoeStreamableObject *io_obj = ( KonoeStreamableObject * ) io_obj_orderly -> getTarget( );

	if( io_obj_orderly == 0 ){
	  os << io_obj_name << " doesn't exist." << endl;
	  return 0;
	}

	KonoeOutputObjSocket *target = ( KonoeOutputObjSocket * ) owner -> getTarget( );
	target -> put( io_obj, 'A', 0 );

	return 0;
}

KInt	KonoeOutputObjSocketOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OBJECT );
        objectname = ( KChar * ) arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_SOCKETIO );
        const KChar *socketio_name = arg -> getValue( );

	KonoeCommander *commander = getOfficer( ) -> getCommander( );
	KonoeOrderly * socketio_orderly = commander -> findObject( socketio_name );

	if( socketio_orderly == 0 ){
	  os << socketio_name << " doesn't exist." << endl;
	  return 0;
	}

	KonoeSocketIO *socketio = ( KonoeSocketIO * ) socketio_orderly -> getTarget( );
	target = new KonoeOutputObjSocket( socketio );

	return 0;

}

KInt	KonoeOutputObjSocketOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeOutputObjSocketOrderly::clone( const KChar * name ) {

	return new KonoeOutputObjSocketOrderly( name );

}
