//===================================================================
/*

	KonoeServerSocketOrderly.cc

	Version 0.0.1	04-Aug-1998	S. Nishida and S. Enomoto
	Version 0.4.0	 6-Sep-1998
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.6.0	21-May-2001	H. Sakamoto

*/
//===================================================================

#include	<system/KonoeServerSocketOrderly.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<system/KonoeStreamableObject.hh>

const KChar	* KonoeServerSocketOrderly::classname = "ServerSocket";


KonoeServerSocketOrderly::KonoeServerSocketOrderly( const KChar * objname ) :
	KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cmd_descripter;

	cmd_descripter = new KonoeCommandDescriptor( "create", N_ARG_CREATE );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "object", STRING, NECESSARY ) );
	cmd_descripter-> add(
		new KonoeArgumentDescriptor( "port", INTEGER, NECESSARY ) );
	addCommand(  new CreateUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	addCommand( new ShowUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "waitConnection", N_ARG_WAITCONNECTION );
        addCommand( new WaitConnectionUnit( cmd_descripter, this ) );

	cmd_descripter = new KonoeCommandDescriptor( "closeConnection", N_ARG_CLOSECONNECTION );
        addCommand( new CloseConnectionUnit( cmd_descripter, this ) );

	addCommand( new KonoeHelpCommand( 0, this ) );

}

KInt	KonoeServerSocketOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeServerSocketOrderly::CreateUnit::CreateUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

}

KInt	KonoeServerSocketOrderly::CreateUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* officer;

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeOrderly	* o = owner -> clone(
		( KChar * ) cmd -> getArgument( ARG_CREATE_OBJECT )
		-> getValue( ) );
	o -> setOfficer( officer = owner -> getOfficer( ) );
	officer -> add( o );
	o -> createObject( cmd );
	return 0;
}

KonoeServerSocketOrderly::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeServerSocketOrderly::ShowUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KonoeServerSocket	* target;

	if( ! checkArgument( cmd ) ) return 0;

	//target = ( KonoeServerSocket * ) owner -> getTarget( );
	//target -> dump( );
	os << "KonoeServerSocketOrderly" << endl;

	return 0;
}

KonoeServerSocketOrderly::WaitConnectionUnit::WaitConnectionUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeServerSocketOrderly::WaitConnectionUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeServerSocket *target = ( KonoeServerSocket * ) owner -> getTarget( );
	target -> waitConnection( );

	return 0;
}


KonoeServerSocketOrderly::CloseConnectionUnit::CloseConnectionUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {
}

KInt	KonoeServerSocketOrderly::CloseConnectionUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;

	KonoeServerSocket *target = ( KonoeServerSocket * ) owner -> getTarget( );
	target -> closeConnection( );

	return 0;
}

KInt	KonoeServerSocketOrderly::createObject( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CREATE_OBJECT );
        const KChar *objectname = arg -> getValue( );

	arg = cmd -> getArgument( ARG_CREATE_PORT );
        KInt port = arg -> toInteger( );

	target = new KonoeServerSocket( port );

	return 0;

}

KInt	KonoeServerSocketOrderly::deleteObject( KonoeCommand * ) {

	delete target;

	return 0;

}

KonoeOrderly	* KonoeServerSocketOrderly::clone( const KChar * name ) {

	return new KonoeServerSocketOrderly( name );

}
