//=================================================================
/*

        KonoeFileStatus.cc

	Version 0.4.0   05-Aug-1999     N. Tanimoto
	Version 0.4.1   03-Sep-1999     Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.5.1	18-Jan-2000	H. Sakamoto
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//=================================================================

#include	<system/KonoeFileStatus.hh>
#include        <pwd.h>
#include        <sys/stat.h>
#include        <stdio.h>
#include        <unistd.h>
#include        <grp.h>
#include        <sys/types.h>
#include        <time.h>
#include        <errno.h>
#include        <system/KonoeTimeData.hh>
#include        <system/KonoeSystemServiceCM.hh>
#include        <KonoeCategories.hh>
#include	<string.h>

const	KChar * KonoeFileStatus::className = "KonoeFileStatus";

KonoeFileStatus::KonoeFileStatus( KChar * pfilename, KInt option ) :
		KonoeStreamableObject( KonoeCategories::SYSTEMSERVICE,
                               KonoeSystemServiceCM::CID_FILESTATUS ) {

	filename = new KChar[ strlen( pfilename ) + 1 ];
	strcpy( filename, pfilename );

	KInt st;
	errno = 0;
	if( option == READ_LINK ){
	    st = stat( pfilename, &buf );
	}
	else {
	    st = lstat( pfilename, &buf ); 
	}
	if ( st == -1 ){
	    if( errno == EACCES ){
                status = E_PERMISSINDENIED;
            }else if( errno == ENOENT ){
                status = E_NOTEXIST;
            }else {
                status = E_OTHER;
                cerr << "KonoeFileStatus::KonoeFileStatus. Open Error" << endl;
                //exit( 0 );
            }
	}else{
	    status = NORMAL;
	}

	groupname = 0;
	ownername = 0;
}

KonoeFileStatus::KonoeFileStatus( ) :
	KonoeStreamableObject( KonoeCategories::SYSTEMSERVICE,
                               KonoeSystemServiceCM::CID_FILESTATUS ) {
	filename = 0;
	ownername = 0;
	groupname = 0;
	status = E_OTHER;
}

KonoeFileStatus:: ~KonoeFileStatus( ){

	delete [] filename;
}

KInt KonoeFileStatus::getStreamSize( ){

        KInt    s;
        s = HEADER_SIZE + ( strlen( filename ) / sizeof( KInt ) ) + 1;
	s +=  HEADER_SIZE + 1;	// status
	if( status == NORMAL ){
            s += HEADER_SIZE + (strlen( getOwnerName() )/ sizeof(KInt) ) + 1;
            s += HEADER_SIZE + (strlen( getGroupName() )/ sizeof(KInt) ) + 1;
	    s += ( HEADER_SIZE + 1 ) * 13; // stat member
	}

        return s;
}

KInt    KonoeFileStatus::put( KonoeOutputObjectStream * os,
                        KChar level, KInt memberid ){

        KChar   l = level + 1;
	if( status == NORMAL ){
            os -> putHeader( this, level, memberid, N_MEMBERS );
            os -> put( l, MID_FILENAME, filename );
            os -> put( l, MID_OWNERNAME, getOwnerName() );
            os -> put( l, MID_GROUPNAME, getGroupName() );
            os -> put( l, MID_DEV, (KInt)buf.st_dev);
            os -> put( l, MID_INODE, (KInt)buf.st_ino);
            os -> put( l, MID_MODE, (KInt)buf.st_mode );
            os -> put( l, MID_NLINK, (KInt)buf.st_nlink );
            os -> put( l, MID_USERID, (KInt)buf.st_uid );
            os -> put( l, MID_GROUPID, (KInt)buf.st_gid );
            os -> put( l, MID_FILESIZE, (KInt)buf.st_size );
            os -> put( l, MID_RDEV, (KInt)buf.st_rdev );
            os -> put( l, MID_BLOCKSIZE, (KInt)buf.st_blksize );
            os -> put( l, MID_BLOCKS, (KInt)buf.st_blocks );
            os -> put( l, MID_MTIME, (KInt)buf.st_mtime );
            os -> put( l, MID_ATIME, (KInt)buf.st_atime );
            os -> put( l, MID_CTIME, (KInt)buf.st_ctime );
            os -> put( l, MID_STATUS, status );
	}
	else {
	    os -> putHeader( this, level, memberid, 2 );
	    os -> put(  l, MID_FILENAME, filename );
            os -> put( l, MID_STATUS, status );
	}

        return 0;
}

KInt    KonoeFileStatus::fillPrimitiveMember( KInt memberid, KInt datatype,
                                KInt datacount, void * data ){

        switch( datatype ) {
        case KCHAR_TYPE :
            switch( memberid ){
            case MID_FILENAME :
                filename = ( KChar * ) data;
                break;
	    case MID_OWNERNAME :
		ownername = ( KChar * ) data;
		break;
	    case MID_GROUPNAME :
		groupname = ( KChar * ) data;
		break;
            default :
                delete [] data;
                return -1;
            }
            return 0;
	case KINT_TYPE :
	    switch( memberid ){
	    case MID_DEV :
		buf.st_dev = *(KInt*) data; 
		break;
	    case MID_INODE :
		buf.st_ino = *(KInt*) data; 
		break;
	    case MID_MODE :
		buf.st_mode = *(KInt*) data; 
		break;
	    case MID_NLINK :
		buf.st_nlink = *(KInt*) data; 
		break;
	    case MID_USERID :
		buf.st_uid = *(KInt*) data; 
		break;
	    case MID_GROUPID :
		buf.st_gid = *(KInt*) data; 
		break;
	    case MID_FILESIZE :
		buf.st_size = *(KInt*) data; 
		break;
	    case MID_RDEV :
		buf.st_rdev = *(KInt*) data; 
		break;
	    case MID_BLOCKSIZE :
		buf.st_blksize = *(KInt*) data; 
		break;
	    case MID_BLOCKS :
		buf.st_blocks = *(KInt*) data; 
		break;
	    case MID_MTIME :
		buf.st_mtime = *(KInt*) data; 
		break;
	    case MID_ATIME :
		buf.st_atime = *(KInt*) data; 
		break;
	    case MID_CTIME :
		buf.st_ctime = *(KInt*) data; 
		break;
	    case MID_STATUS :
		status = *(KInt*) data;
		break;
	    default :
		delete [] data;
		return -1;
	    }
	    delete [] data;
	    return 0;
	default : // datatype error
	    delete [] data;
	    return -1;
	}
}

KChar * KonoeFileStatus::getOwnerName( ){

	if( ownername == 0 ){
	    struct passwd   * pp;
	    pp = getpwuid ( buf.st_uid);
	    return pp -> pw_name;
	}
	else {
	    return ownername;
	}
}

KChar * KonoeFileStatus::getFileName( ){
  return filename;
}

KInt	KonoeFileStatus::getLinkName( KChar * buffer, KInt buffersize ){

  if (( buf.st_mode & S_IFMT ) == S_IFLNK ){
	return readlink( filename, buffer, buffersize );
  }else{
	return -1;
  }
}

KInt	  KonoeFileStatus::getUserID( ){
  return  buf.st_uid;
}

KInt	  KonoeFileStatus::getGroupID( ){
  return buf.st_gid;
}

KChar  *  KonoeFileStatus::getGroupName( ){

	if( groupname == 0 ){
	    struct group    * gp;
	    gp = getgrgid( buf.st_gid);
	    return gp -> gr_name;
	}
	else{
	    return groupname;
	}
}

KInt	  KonoeFileStatus::getLinkCount( ){
  return buf.st_nlink;
}

KInt      KonoeFileStatus::getFileSize( ){
  return buf.st_size ;
}

KBool	  KonoeFileStatus::isDirectory( ){
  if((S_IFMT & buf.st_mode) == S_IFDIR)
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::isCharacterSpecial( ){
  if((S_IFMT & buf.st_mode) == S_IFCHR)
    return 1;
  else 
    return 0;
}

KBool	  KonoeFileStatus::isRegularFile( ){
  if((S_IFMT & buf.st_mode) == S_IFREG)
    return 1;
  else 
    return 0;
}

KBool	  KonoeFileStatus::isBlockSpecial( ){
  if((S_IFMT & buf.st_mode) == S_IFBLK )
    return 1;
  else 
    return 0;
}

KBool	  KonoeFileStatus::isSocket( ){
  if((S_IFMT & buf.st_mode) == S_IFSOCK )
    return 1;
  else 
    return 0;
}

KBool	  KonoeFileStatus::isFIFO( ){
  if((S_IFMT & buf.st_mode) == S_IFIFO )
    return 1;
  else 
    return 0;
}

KBool	  KonoeFileStatus::isSymbolicLink( ){
  if((S_IFMT & buf.st_mode) == S_IFLNK )
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::userCanRead( ){   
  if(S_IREAD & buf.st_mode)
    return 1;
  else 
    return 0;
}
   
KBool     KonoeFileStatus::userCanWrite( ){   
  if(S_IWRITE & buf.st_mode)
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::userCanExecute( ){
  if(S_IEXEC & buf.st_mode)
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::groupCanRead( ){   
  if(S_IREAD & (buf.st_mode << 3 ))
    return 1;
  else 
    return 0;
}
   
KBool     KonoeFileStatus::groupCanWrite( ){   
  if(S_IWRITE & (buf.st_mode << 3 ))
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::groupCanExecute( ){
  if(S_IEXEC & (buf.st_mode << 3 ))
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::othersCanRead( ){   
  if(S_IREAD & (buf.st_mode << 6 ))
    return 1;
  else 
    return 0;
}
   
KBool     KonoeFileStatus::othersCanWrite( ){   
  if(S_IWRITE & (buf.st_mode << 6 ))
    return 1;
  else 
    return 0;
}

KBool     KonoeFileStatus::othersCanExecute( ){
  if(S_IEXEC & (buf.st_mode << 6 ))
    return 1;
  else 
    return 0;
}

KonoeTimeData *  KonoeFileStatus::getTimeOfLastAccess( ){
  KonoeTimeData * time;
  time = new KonoeTimeData;
  time -> set( buf.st_atime );
  return time;
}

KonoeTimeData *  KonoeFileStatus::getTimeOfLastDataModification( ){
  KonoeTimeData * time;
  time = new KonoeTimeData;
  time -> set( buf.st_mtime );
  return time;
}

KonoeTimeData *  KonoeFileStatus::getTimeOfLastFileSatusChange( ){
  KonoeTimeData * time;
  time = new KonoeTimeData;
  time -> set( buf.st_ctime );
  return time;
}
