//===================================================================
/*

	KonoeFileStatus.hh
	
	Version 0.4.0   05-Aug-1999     N. Tanimoto
	Version 0.4.1   03-Sep-1999     Y. Kouno
        Version 0.5.0   21-Sep-1999     Y. Kouno
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//===================================================================

#ifndef __KONOEFILESTATUS_HH
#define __KONOEFILESTATUS_HH

#include        <KonoePrimitives.hh>
#include        <system/KonoeTimeData.hh>
#include        <system/KonoeStreamableObject.hh>
#include	<iostream.h>
#include        <sys/stat.h>

class	KonoeFileStatus : public KonoeStreamableObject {

public :
        static const KChar      * className;

        enum    MEMBER_ID {
		MID_FILENAME, MID_OWNERNAME, MID_GROUPNAME,
                MID_DEV, MID_INODE, MID_MODE,
                MID_NLINK, MID_USERID, MID_GROUPID,
                MID_FILESIZE, MID_RDEV,
                MID_BLOCKSIZE, MID_BLOCKS,
                MID_MTIME, MID_ATIME, MID_CTIME,
		MID_STATUS,
                N_MEMBERS	};
	
	enum	{ READ_LINK, NOT_READ_LINK	};
	enum	{	NORMAL,
			E_NOTEXIST, E_PERMISSINDENIED,
			E_OTHER };

protected :
	KChar	* filename;
	KChar	* ownername;
	KChar	* groupname;
	KInt	status;
        struct stat       buf;
  
public :
        KonoeFileStatus( KChar * filename, KInt option = NOT_READ_LINK );
        KonoeFileStatus( );
        ~KonoeFileStatus( );
	KInt	getStatus( ){ return status; }
	KBool	isNormal( ){ return status == NORMAL; }

        KChar   * getOwnerName( );
        KChar	* getFileName( );
        KInt	getLinkName( KChar * buffer, KInt buffersize );
        KInt	getUserID( );
	KInt	getGroupID( );
	KChar   * getGroupName( );
	KInt	getLinkCount( );
	KInt    getFileSize( );
	KBool	isDirectory( );
	KBool   isCharacterSpecial( );
	KBool	isRegularFile( );
	KBool   isBlockSpecial( );
	KBool   isSocket( );
	KBool   isFIFO( );
	KBool   isSymbolicLink( );
	KBool	userCanRead( );
	KBool   userCanWrite( );
	KBool   userCanExecute( );
	KBool	groupCanRead( );
	KBool   groupCanWrite( );
	KBool   groupCanExecute( );
	KBool	othersCanRead( );
	KBool   othersCanWrite( );
	KBool   othersCanExecute( );
        KonoeTimeData   *  getTimeOfLastAccess( );
	KonoeTimeData   *  getTimeOfLastDataModification( );
        KonoeTimeData   *  getTimeOfLastFileSatusChange( );

        // to be streamable

        KInt    put(    KonoeOutputObjectStream * outstream,
                        KChar level, KInt memberid );
        KInt    getStreamSize( );
        KInt    fillPrimitiveMember( KInt memberid, KInt datatype,
                                KInt datacount, void * data );
        KInt    fillObjectMember( KInt memberid, KonoeStreamableObject * ){
                        return -1; }

};

#endif
