//===================================================================
/*

	KonoeServerSocket.cc

	Version 0.1	17-Jul-1997	H. Sakamoto
	Version 0.2	05-Nov-1997	H. Sakamoto
	Version 0.3	08-Dec-1997	H. Sakamoto
	Version 0.4	06-Jan-1998	H. Sakamoto
	Version 0.1.5	09-Mar-1998	H. Sakamoto
	Version 0.2.0	16-Apr-1998	H. Sakamoto
	Version 0.4.0	27-Aug-1998	S. Inaba
	Version 0.4.1	12-Nov-1998	H. Sakamoto
        Version 0.5.0   21-Sep-1999     Y. Kouno
        Version 0.6.0   21-Sep-2000     Y. Shimizu
        Version 0.6.1    6-Nov-2000     Y. Shimizu
	Version 0.6.2	09-Nov-2000	H. Sakamoto
	Version 0.7.0	28-Aug-2001	S. Morinaka

*/
//===================================================================

#include	<system/KonoeServerSocket.hh>

#include	<iostream.h>
#include	<unistd.h>
#include	<sys/time.h>
#include	<sys/socket.h>
#include	<string.h>
#include	<stdio.h>

/*
#ifndef	socket
extern "C"	{
	int	socket( int, int, int );
//	int	bind( int, struct sockaddr *, int );
	int	listen( int, int );
	void	bzero( char *, int );
	};
#endif
*/
	
KonoeServerSocket::KonoeServerSocket( KInt pport, KonoeFDSet * fds ) {

	wait_interval = WAIT_INTERVAL; // default 1000 ms

	for( KInt i = 0; i < MAX_CLIENTS; i ++ )
	    clients[ i ] = 0;
	status = CLOSED;

	afd = socket( PF_INET, SOCK_STREAM, 0 );
	if( afd == -1 ) {
	    errornumber = errno;
	    perror( 0 );
	    return;
	}
	status = OPENED;
	port = pport;
	set_sockaddr( pport );
	if( bind( afd, ( struct sockaddr * ) & socket_address,
		sizeof( socket_address ) ) == -1 ) {
	    errornumber = errno;
	    perror( 0 );
	    return;
	}
	status = BOUND;
	if( listen( afd, MAX_CLIENTS ) == -1 ) {
	    errornumber = errno;
	    perror( 0 );
	    return;
	}
	status = LISTENING;
	errornumber = 0;
	send_mode = SEND_TO_ALL;	// default

	if( fds )
	    readfds = fds;
	else
	    readfds = new KonoeFDSet( );
	readfds -> install( this );

}

KonoeServerSocket::~KonoeServerSocket( ) {

	for( KInt i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] )
		readfds -> remove( clients[ i ] );
	    delete clients[ i ];
	}

	if( status != CLOSED ) {
	    if( status == LISTENING ) {
		shutdown( afd, 2 );
	    }
	    readfds -> remove( this );
	    ::close( afd );
	}

}

KInt	KonoeServerSocket::waitConnection( ) {

	KInt	i, j, st;

	errornumber = 0;
	for( i = 0; i < MAX_CLIENTS; i ++ )
	    if( clients[ i ] == 0 )
		break;
	if( i == MAX_CLIENTS ) {
	    errornumber = E_TABLEFULL;
	    cerr << "KonoeServerSocket::waitConnection. Client Table Full"
		<< endl;
	    return -1;	// client table full
	}

	clients[ i ] = new KonoeSocketConnection( );

	if( clients[i] ){
	  readfds -> install( clients[ i ] );
	  st = clients[ i ] -> accept( afd );
	  if( st == -1 ) {
	    errornumber = errno;
	    perror( 0 );
	    return -1;
	  }
        }
        return 0;

}

KInt	KonoeServerSocket::closeConnection( ) {

	KInt	i;

	for( i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] )
		readfds -> remove( clients[ i ] );
	    delete clients[ i ];
	    clients[ i ] = 0;
	}
	return 0;
}

KInt	KonoeServerSocket::close( KonoeSocketConnection * so ) {

	KInt	i;

	for( i = 0; i < MAX_CLIENTS; i ++ ) {
	    if( clients[ i ] == so ) {
		readfds -> remove( so );
		delete clients[ i ];
		clients[ i ] = 0;
		return 0;
	    }
	}
	errornumber = E_SOCKETNOTMATCH;
	return -1;	// socket connection not found
}

KInt	KonoeServerSocket::send( const KChar * buffer, KInt nbytes ) {

	KInt	i, s, st;

	switch( send_mode ) {
	case SEND_TO_LAST :
	    if( last_received ) {
		s = send_to_client( last_received, buffer, nbytes );
		if( s == -1 )
		    return s;
	    }
	    break;
	case SEND_TO_OTHERS :
	    st = 0;
	    for( i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] && ( clients[ i ] != last_received ) ) {
		    s = send_to_client( clients[ i ], buffer, nbytes );
		    if( s == -1 )
			st = s;
		}
	    return st;
	case SEND_TO_ALL:
	    st = 0;
	    for( i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] ) {
		    s = send_to_client( clients[ i ], buffer, nbytes );
		    if( s == -1 )
			st = s;
		}
	    return st;

	}
	return 0;
}

KInt	KonoeServerSocket::send_to_client( KonoeSocketConnection * c,
		const KChar * buffer, KInt nbytes ) {

	KInt	st;

	st = c -> send( buffer, nbytes );
	if( st == -1 ) {
	    errornumber = errno;
	    cerr << "KonoeServerSocket::send. Error = " << errno << endl;
	    perror( 0 );
	    for( KInt i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] == c ) {
		    delete clients[ i ];
		    clients[ i ] = 0;
		    if( last_received == c )
			last_received = 0;
		    break;
		}
	}
	return st;
}


KonoeSocketConnection	* KonoeServerSocket::wait( KInt milisec ) {

	struct	timeval	timeout;
	KInt	st;
	KonoeFDClient	* cl;

	errornumber = 0;

	cl = readfds -> wait( milisec );
	if( cl == this ) {
	    st = waitConnection( );
	    if( st == 0 )
		errornumber = E_CONNECTED;
	    return 0;
	}
	else if( cl ) {
	    last_received = ( KonoeSocketConnection * ) cl;
	    return last_received;
	}
	return 0;
}

KonoeSocketConnection	* KonoeServerSocket::processReceiveAction(
				KonoeFDClient * client ) {

	KInt	st;

	if( client == this ) {
	    st = waitConnection( );
	    if( st == 0 )
		errornumber = E_CONNECTED;
	    return 0;
	}

	KInt	i;
	for( i = 0; i < MAX_CLIENTS; i ++ )
	    if( clients[ i ] == client ) {
		return ( KonoeSocketConnection * ) client;
	    }

	return ( KonoeSocketConnection * ) -1;

}


KInt	KonoeServerSocket::receive( KChar * buffer, KInt buffersize ) {

	KonoeSocketConnection	* c;

	errornumber = 0;
	c = wait( wait_interval );
	if( c == 0 ) {
	    if( errornumber != 0 && errornumber != E_CONNECTED )
		return -1;
	    return 0;
	}

	return receiveFromConnection( c, buffer, buffersize );

}

KInt	KonoeServerSocket::receiveFromConnection( KonoeSocketConnection * c,
		KChar * buffer, KInt buffersize ) {

	KInt	st;

	st = c -> receive( buffer, buffersize );
	if( st == 0 ) {
	    for( KInt i = 0; i < MAX_CLIENTS; i ++ )
		if( clients[ i ] == c ) {
		    readfds -> remove( c );
		    delete clients[ i ];
		    clients[ i ] = 0;
		    errornumber = E_DISCONNECTED;
		    return 0;	// disconnected = success.
		}
	}
	else if( st == -1 )
	    errornumber = c -> getError( );
	return st;

}
