//=============================================================
/*

	Konoe2DHistogram.java
	
	Version 0.0.1	03-Jul-1998	K.Ohta
	Version 0.4.0	03-Aug-1998	S. Inaba
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   24-Aug-2000 A. Izumoto

*/
//=============================================================

package	konoe.ui.analysis;

//
//	2 Dimension Histogram
//

import java.io.*;
import konoe.ui.system.*;

public class Konoe2DHistogram implements KonoeStreamableObject,
					KonoeObject {

  public static final String className = "Konoe2DHistogram";

  protected KonoeDataScale xscale;
  protected KonoeDataScale yscale;
  protected KonoeHistogramStorage storage;

  public Konoe2DHistogram( KonoeDataScale pxscale, KonoeDataScale pyscale ){

    xscale = pxscale;
    yscale = pyscale;
    storage = new KonoeHistogramStorage( xscale.getBinCount(),
                                         yscale.getBinCount() ); 

  }

  public Konoe2DHistogram() {

    xscale = null;
    yscale = null;
    storage = null;
  
  }

  public int getXscale() {

    return (int)( xscale.getMaximum() );

 }

  public int getYscale() {

    return (int)( yscale.getMaximum() );

 }

  public int getBinCountx () {

    return xscale.getBinCount();
  }

  public int getBinCounty () {

    return yscale.getBinCount();

  }

  public void clear() {

    if( storage != null )
        storage.clear();
    //else.... should be implemented.

  }

  public void dump() {

    System.out.println ( "2D Histogram " );
    xscale.dump();
    yscale.dump();
    storage.dump();

  }

  public int fill( KonoeFillData fill_data ) {

    storage.addOne( xscale.getBinIndex( fill_data.getX() ),
                    yscale.getBinIndex( fill_data.getY() ) );
    return 0;

  }

  public double getOne ( int pxindex, int pyindex ) {

    return storage.get( pxindex, pyindex );

  }


//  to create me from stremable object

public void read( KonoeInputObjectStream in, KonoeStreamHeader header )	
		throws IOException {
		switch( header.getMemberID( )) {
		case 0: xscale = ( KonoeDataScale )
				( in.readObject( header ));
		break;

		case 1: yscale = ( KonoeDataScale )
				( in.readObject( header ));
		break;
		
		case 2: storage = ( KonoeHistogramStorage )
				( in.readObject( header ));
		break;
		}
	}

public void write( KonoeOutputObjectStream out ) throws IOException {
	}

}
 
