//=============================================================
/*

        KonoeDetectorElement.java

        Version 0.0.1   03-Aug-1998     D. Takagi
        Version 0.4.0   18-Aug-1998     D. Takagi
        Version 0.5.0   28-Sep-1999     T. Hosoi
        Version 0.6.0   01-Feb-2000     T. Hosoi

*/
//=============================================================
package konoe.ui.analysis;

import konoe.ui.system.*;
import java.io.*;

public class KonoeDetectorElement implements KonoeStreamableObject, KonoeObject {

    public static final int MAX_READOUT = 8;
    
    protected KonoeGeometry geometry;
    protected int           identifier;
    protected KonoeDetector detector;
    protected KonoeList     addressList;

    public KonoeDetectorElement() {
	addressList = new KonoeList(MAX_READOUT);
    }

    public KonoeDetectorElement(KonoeGeometry geometry, KonoeDetector detector,
				int identifier) {
	this.geometry   = geometry;
	this.detector   = detector;
	this.identifier = identifier;
	addressList     = new KonoeList(MAX_READOUT);
    }

    public int getID() { return identifier; }
    public KonoeDetector getDetector() { return detector; }
    public void setDetector(KonoeDetector detector) { this.detector = detector; }
    public KonoeGeometry getGeometry() { return geometry; }

    public String toString() {
	String s;
	s =  "KonoeDetectorElement:" + "\n" +
	       "  identifier   = " + identifier + "\n" +
	       "  addressCount = " + addressList.getCount() + "\n";
	s += "\n";
	s += geometry.toString();
	s += "\n";

	for (int i = 0; i < addressList.getCount(); i++) {
	    s += "--- Address " + i + " ---";
	    s += ((KonoeDataAddress)addressList.get(i)).toString();
	}

	return s;
    }

    public void dump() {
	System.out.println("ID:" + identifier);
    }

    // --- for Streamable ---
    public void read(KonoeInputObjectStream in, KonoeStreamHeader header) throws IOException {
        switch (header.getMemberID()) {
          case 0: 
	    geometry = (KonoeGeometry)(in.readObject(header)); 
	    break;

          case 1: 
	    identifier = in.readInt(); 
	    break;

          case 2: 
	    addressList.allocate(in.readInt()); 
	    break;

          case 3: 
	    Object[] array = in.readObjectArray(header);
	    addressList.setEntries(array);
	    break;
	}
    }

    public void write(KonoeOutputObjectStream out) throws IOException {
    }
}
