//=============================================================
/*

	KonoeHistogramStorage.java
	
	Version 0.0.1	03-Jul-1998	K.Ohta
	Version 0.4.0   04-Aug-1998     D. Takagi
    Version 0.6.0   01-Feb-2000 T.Hosoi

*/
//=============================================================

package	konoe.ui.analysis;

import konoe.ui.system.*;
import java.io.*;

//
//     The storage has 2 bins more than required bin count. They
//     contain overflow and underflow values for each dimension.
//

public class KonoeHistogramStorage extends KonoeHistoStorageBase implements Serializable{

    protected static final int TYPE_1D = 0; 
    protected static final int TYPE_2D = 1; 
    protected static final int TYPE_3D = 2; 

    protected int  type;
    protected int  x_bincount = 0;
    protected int  y_bincount = 0;
    protected int  z_bincount = 0;
    protected int  x_size = 0;
    protected int  y_size = 0;
    protected int  z_size = 0;
    protected int  storage_size;
    protected int[]  store;

    public KonoeHistogramStorage() {

	store = null;
	storage_size = 0;
	createStore();

    }

    KonoeHistogramStorage( int xsize ) {

	this(xsize, 0, 0);

    }

    KonoeHistogramStorage( int xsize, int ysize ) {
	
	this(xsize, ysize, 0);

    }

    KonoeHistogramStorage( int xsize, int ysize, int zsize ) {

	x_size = xsize;
	y_size = ysize;
	z_size = zsize;
	x_bincount = xsize + 2;
	y_bincount = ysize + 2;
	z_bincount = zsize + 2;
	createStore();
	
    }
 
    private void createStore () {

        if( z_size != 0 ) {
            type = TYPE_3D;
            storage_size = x_bincount * y_bincount * z_bincount;
        }
        else if( y_size != 0 ) {
            type = TYPE_2D;
            storage_size = x_bincount * y_bincount;
        }
        else {
            type = TYPE_1D;
            storage_size = x_bincount;
        }
	
        store = new int[ storage_size ];
    }


    public int  get_index( int xindex, int yindex, int zindex ) {

        int    k;

        k = 0;
        switch ( type ) {
        case TYPE_3D:
            if( zindex >=0 && zindex < z_size )
                k = zindex * x_bincount * y_bincount;
            else if( zindex == -1 || zindex == -2 )
                k = ( zindex + z_bincount ) * x_bincount * y_bincount;
            else
                return -1;      // failure
        case TYPE_2D:
            if( yindex >= 0 && yindex < y_size )
                k += yindex * x_bincount;
            else if( yindex == -1 || yindex == -2 )
                k += ( yindex + y_bincount ) * x_bincount;
            else
                return -1;      // failure
        case TYPE_1D:
            if( xindex >= 0 && xindex < x_size )
                k += xindex;
            else if( xindex == -1 || xindex == -2 )
                k += xindex + x_bincount;
            else
                return -1;
        }
        return k;

    }


    protected int set( int value, int xindex ) {

        return setStore( value, xindex, 0, 0 );
    }

    protected int set( int value, int xindex, int yindex) {

        return setStore( value, xindex, yindex, 0 );
    }

    protected int set( int value, int xindex, int yindex, int zindex) {

        return setStore( value, xindex, yindex, zindex );
    }
    
    private int setStore( double value, int xindex, int yindex, int zindex ) {
     
	int k;

	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 ) {

	    return k;
	}
        store[ k ] = (int)value;
        return k;
    }

    protected int addOne( int xindex ) {

	return addStore( xindex, 0, 0 );
      
    }

    protected int addOne( int xindex, int yindex ) {

	return addStore( xindex, yindex, 0 );

    }

    protected int addOne( int xindex, int yindex, int zindex ) {
      
	return addStore( xindex, yindex, zindex );

    }
    
    private int addStore( int xindex, int yindex, int zindex ) {

	int k;

	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 ) {

	    return k;
	}
        store[ k ] ++;
        return k;
    }

    protected double get( int xindex ) {
	return getOne( xindex, 0, 0 );
    }

    protected double get( int xindex, int yindex ) {
	return getOne( xindex, yindex, 0 );
    }

    protected double get( int xindex, int yindex, int zindex ) {
	return getOne( xindex, yindex, zindex );
    }

    private double getOne ( int xindex, int yindex, int zindex ) {

	int k;
	if( ( k = get_index( xindex, yindex, zindex ) ) == -1 ) {
	    return 0;
	}
	return (double)store[ k ];
    }
  

    protected void clear() {

        for( int i = 0; i < storage_size; i ++ )
	  store[i] = 0;
    }

  
    public void dump () {

        int    i, j, z;
        int[] p;
        p = store;
        i = 0;
        z = 0;
        while( i < storage_size ) {
            System.out.print( i + ": " );
            j = i + 10;
            if( j > storage_size )
	      j = storage_size;
            for( ; i < j ; i ++ )
	      System.out.print ( p[z++] + " ");
            System.out.println ( " " );
        }
	
    }

    // --- for Streamable ---
    public void read(KonoeInputObjectStream in, KonoeStreamHeader header) throws IOException {
	switch (header.getMemberID()) {
	  case 0: type         = in.readInt(); break;
	  case 1: x_size       = in.readInt(); break;
	  case 2: y_size       = in.readInt(); break;
	  case 3: z_size       = in.readInt(); break;
	  case 4: storage_size = in.readInt(); break;
	  case 5: store        = in.readIntArray(header); break;
	}
    }

    public void write(KonoeOutputObjectStream out) throws IOException {
    }
}
