//=============================================================
/*

        KonoeLinearDataScale.java

        Version 0.0.1   03-Aug-1998     D. Takagi
        Version 0.4.0   18-Aug-1998     D. Takagi
    Version 0.6.0   01-Feb-2000 T.Hosoi

*/
//=============================================================
package konoe.ui.analysis;

import konoe.ui.system.*;
import java.io.*;

public class KonoeLinearDataScale extends KonoeDataScale implements KonoeObject , Serializable{

    public static final String className = "Linear Scale ";
    public static final int OVERFLOW = -1;
    public static final int UNDERFLOW = -2;
    
    protected double minimum;
    protected double maximum;
    protected double binwidth;
    protected int    bincount;

    public KonoeLinearDataScale() {
    }

    public KonoeLinearDataScale(double pminimum, double pmaximum,
                                int pbincount) {
      minimum = pminimum;
      maximum = pmaximum;
      bincount = pbincount;
      binwidth = ( pmaximum - pminimum ) / bincount;
        
    }

    public int    getBinIndex(double value) { 

      int    i;

        i = (int)(( value - minimum ) / binwidth);
        if( i < 0 )
            i = UNDERFLOW;
        else if( i >= bincount )
            i = OVERFLOW;
        return i;
    }

    public double getBinRawer(int binnumber) {

      return minimum + binwidth * binnumber;

    }

    public double getBinWidth(int binnumber) { return binwidth; }
    public int    getBinCount() { return bincount; }
    public double getMinimum() { return minimum; }
    public double getMaximum() { return maximum; }

    public String toString() {
	return "KonoeLinearScalel:" + "\n" +
	       "minimum  = " + minimum + "\n" +
	       "maximum  = " + maximum + "\n" +
               "binwidth = " + binwidth + "\n" +
               "bincount = " + bincount + "\n";
    }

    public void dump() {
	System.out.println(toString());
    }

    //--- for Streambale ---

    public void read(KonoeInputObjectStream in, KonoeStreamHeader header) throws IOException {

	switch (header.getMemberID()) {
	  case 0:
	    minimum = in.readDouble();
	    break;

	  case 1:
	    maximum = in.readDouble();
	    break;

	  case 2:
	    binwidth = in.readDouble();
	    break;

	  case 3:
	    bincount = in.readInt();
	    break;
	}
    }   

    public void write(KonoeOutputObjectStream out) throws IOException {
    }
}
