//====================================================================
/*

	KonoeCommandParser.java

	Version 0.0.1	 3-Jul-1998	H. Sakamoto
	Version 0.4.0	 6-Sep-1998
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi

*/
//====================================================================

package	konoe.ui.command;

import	konoe.ui.command.*;

public class KonoeCommandParser {

  public static final int TOO_MANY_ARGUMENTS = -2;
  public static final int INVALID_SYNTAX = -1;
  public static final int NULL_LINE = 0;
  public static final int COMMAND_PARSED = 1;

  String	commandline;
  String	verb;
  KonoeCommand	command;

  static final int START = 0;
  static final int VERB = 1;
  static final int END_VERB = 2;
  static final int ARGUMENT = 3;
  static final int END_ARGUMENT = 4;
  static final int QUOTEDSTRING = 5;

  int	argumentcount;
  int	parseposition;
  int	st;

  public KonoeCommandParser( ) {


  }

  public KonoeCommand	getCommand( ) {

    return command;

  }

  public int parse( String commandline ) {

    this.commandline = commandline;
    command = new KonoeCommand( );

    int	p, q, r, i, l, sl;
    boolean	eol;

    p = 0;
    st = START;
    eol = false;
    q = p;
    sl = commandline.length( );
    parseposition = 0;
    argumentcount = 0;

    for( p = 0; p < sl; p ++ ) {
	  switch( commandline.charAt( p ) ) {
	  case ' ' :
	  case '\t' :	// white spaces.
	    switch( st ) {
	    case START :
	      break;
	    case VERB :
	      verb = commandline.substring( q, p );
	      st = END_VERB;
	      break;
	    case ARGUMENT :
	      if( add_argument( q, p ) < 0 )
		return TOO_MANY_ARGUMENTS;
	      break;
	    }
	    break;
	  case '"' :	// double quotation.
	    switch( st ) {
	    case QUOTEDSTRING :	// end of quotation
	      if( add_argument( q + 1, p ) < 0 )
		return TOO_MANY_ARGUMENTS;
	      break;
	    case END_VERB :
	    case END_ARGUMENT :
	      st = QUOTEDSTRING;
	      q = p;
	      break;
	    default :
	      return INVALID_SYNTAX;
	    }
	    break;
	  case '\n' :
	    eol = true;
	    break;
	  case '#' :	// comment introducer
	    switch( st ) {
	    case START :
	    case END_VERB :
	    case END_ARGUMENT :
	      eol = true;
	      break;
	    case QUOTEDSTRING :
	      break;
	    default :
	      return INVALID_SYNTAX;
	    }
	    break;
	  default :	// other printable characters
	    switch( st ) {
	    case START :
	      q = p;	// beginning of verb
	      st = VERB;
	      break;
	    case END_ARGUMENT :
	    case END_VERB :
	      q = p;	// head of argument
	      st = ARGUMENT;
	      break;
	    default :
	      break;	// continue.
	    }
	    break;
	  }
	  if( eol )
	    break;
	  parseposition ++;
	}

	switch( st ) {
	case START :
	  return NULL_LINE;
	case VERB :
	  verb = commandline.substring( q, p );
	  st = END_VERB;
	  break;
	case ARGUMENT :
	  if( add_argument( q, p ) < 0 )
	    return TOO_MANY_ARGUMENTS;
	  break;
	case QUOTEDSTRING :
	  return INVALID_SYNTAX;
	default :
	  break;
	}
	st = analyze_verb( );
	if( st == -1 )
	  return INVALID_SYNTAX;
	return COMMAND_PARSED;
  }

  int	add_argument( int strfrom, int strto ) {

    command.addArgument( new KonoeCommandArgument( argumentcount,
	commandline.substring( strfrom, strto ) ) );
    argumentcount ++;
    st = END_ARGUMENT;
    return 0;

  }

  int	analyze_verb( ) {

	int	pdot, pat, pcolon;

	pdot = verb.lastIndexOf( '.' );	// last dot.
	pat = verb.indexOf( '@' );	// first @
	pcolon = verb.indexOf( ':' );	// first :

	if( pcolon != -1 ) {	// contains :
	  if( pat != -1 ) {	// contains @hostname:
	    if( pat < pcolon ) {	// @ must be ahead of :
	      command.setProcessName( verb.substring( 0, pat ) );
	      command.setHostName( verb.substring( pat + 1, pcolon ) );
	    }
	    else
	      return -1;
	  }
	  else {	//processname:objectname.method
	    command.setProcessName( verb.substring( 0, pcolon ) );
	  }
	  if( pdot != -1 ) {
	    command.setObjectName( verb.substring( pcolon + 1, pdot ) );
	    command.setMethodName( verb.substring( pdot + 1 ) );
	  }
	  else {
	    command.setMethodName( verb.substring( pcolon + 1 ) );
	  }
	}
	else	{
	  if( pdot != -1 ) {	// objectname.method
	    command.setObjectName( verb.substring( 0, pdot ) );
	    command.setMethodName( verb.substring( pdot + 1 ) );
	  }
	  else {	// method only.
	    command.setMethodName( verb );
	  }
	}
	return 0;

  }
}
