//===================================================================
/*

	KonoeCommandProcessor.java

	Version 0.0.1	17-Aug-1998	S. Nishida
	Version 0.4.0	29-Sep-1998	S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi

*/
//==================================================================

package konoe.ui.command;

import java.util.*;
import java.io.*;
import konoe.ui.command.*;
import konoe.ui.system.*;

abstract public class KonoeCommandProcessor {

  protected KonoeCommandDescriptor descriptor;
  protected Object owner;   // must be KonoeOrderly

  public KonoeCommandProcessor( KonoeCommandDescriptor cd, Object powner ) {
    
    descriptor = cd;
    owner = powner;
  }

  public String getMethodName( ) {

    return descriptor.getName( );
  }

  public void printHelp( PrintWriter writer, boolean flag ) {

    descriptor.printHelp( writer, flag );
  }

  abstract public int execute( KonoeCommand cmd );

  public boolean checkArgument( KonoeCommand cmd ) {

    KonoeCommandArgument com_arg;
    KonoeArgumentDescriptor arg_des;
    int narg = cmd.getArgumentCount( );
    int des_narg = descriptor.getArgumentCount( );
    int argc;
    boolean typecheck;

    if( narg > des_narg ) {
      System.out.print( "Too many argument(s). Only " +
             descriptor.getArgumentCount( ) +
             " argument(s) is(are) needed." ) ;
      return false;
    }

    for( argc = 0; argc < narg; argc++ ) {   
      com_arg = cmd.getArgument( argc );
      arg_des = descriptor.getArgument( argc );

      switch( arg_des.getType( ) ) {
        case KonoeArgumentDescriptor.INTEGER:
	  typecheck = com_arg.checkInteger( );
	  break;
        case KonoeArgumentDescriptor.FLOATING:
	  typecheck = com_arg.checkFloating( );
	  break;
        default:
	  typecheck = true;
      }
 
      if ( !typecheck ) {
	System.out.print( "Invalid type of argument " );
	arg_des.printHelp( new PrintWriter(System.out,true) );
	return false;   // NG.
      }
    }
    
    if ( narg < des_narg ) {
      arg_des = descriptor.getArgument( narg );
      if ( arg_des.getAppearance( ) == KonoeArgumentDescriptor.NECESSARY ) {
	System.out.print( "Missing argument " );
	arg_des.printHelp( new PrintWriter(System.out,true) );
	return false;   // NG.
      }
    }
    return true;   // OK.
  }

}
