//=====================================================================
/*

	KonoeCommander.java

	Version 0.0.1	10-Jul-1998	H. Sakamoto
	Version 0.0.3	15-Aug-1998	S. Nishida
	Version 0.4.0	29-Sep-1998	S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi

*/
//=====================================================================

package konoe.ui.command;

import java.io.*;
import konoe.ui.command.*;
import konoe.ui.system.*;

public class KonoeCommander extends KonoeOrderly {

  public static final int CMD_SHOW = 0;
  public static final int CMD_EXIT = 1;
  public static final int CMD_HELP = 2;
  public static final int N_COMMANDS = 3;

  public static final int ARG_SHOW_ITEM = 0;
  public static final int N_ARG_SHOW = 1;
  public static final int ARG_EXIT_STATUS = 0;
  public static final int N_ARG_EXIT = 1;
  public static final int ARG_HELP_TOPICS = 0;
  public static final int N_ARG_HELP = 1;

  protected KonoeList officerlist; // member is KonoeOfficer
  KonoeCommandParser parser;
  KonoeOrderly currenttarget;
  KonoeCommand command;

  static String classname = "commander";

  class ExitUnit extends KonoeCommandProcessor {
    public ExitUnit( KonoeCommandDescriptor cd, KonoeOrderly ow ){
      super( cd , ow );
    }
    public int execute( KonoeCommand cmd ) { return -1; }
  }

  public KonoeCommander( int nofficer , String name ) {

    super( N_COMMANDS , name );

    officerlist = new KonoeList( nofficer );
    parser = new KonoeCommandParser( );
    currenttarget = null;

    // installing commands
    KonoeCommandDescriptor cmd_descriptor;

    cmd_descriptor = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
    cmd_descriptor.add(
            new KonoeArgumentDescriptor( "item", KEYWORD, NECESSARY ) );
    addCommand( new KonoeShowCommand( cmd_descriptor, this ) );

    cmd_descriptor = new KonoeCommandDescriptor( "exit", N_ARG_EXIT );
    cmd_descriptor.add(
	  new KonoeArgumentDescriptor( "status", INTEGER, OPTIONAL ) );
    addCommand( new ExitUnit( cmd_descriptor, this ) );

    addCommand( new KonoeHelpCommand( null , this ) );

  }

  KonoeOrderly select_object( ) {

    String name;
    KonoeOfficer aofficer;
    KonoeOrderly target;

    if( command.getHostName( ) != null ) {
      System.out.println( "Sorry. Not implemented. ( process@host:object )" );
      return null;
    }
    if( command.getProcessName( ) != null ) {
      System.out.println( "Sorry. Not implemented. ( process:object )" );
      return null;
    }
    name = command.getObjectName( );
    if( name != null ) {
      if( name.length( ) == 0 ) {	// .method
	return this;
      }
      else {	// object.method
	officerlist.reset( );
	while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null ){
	  if( ( target = aofficer.findObject( name ) ) != null ) {
	    currenttarget = target;
	    return target;
	  }
	  if( ( target = aofficer.hasClass( name ) ) != null ) {
	    currenttarget = target;
	    return target;
	  }
	}
	System.out.println( "Object " + name + " not found." );
	return null;
      }
    }
    else {	// method
      if( currenttarget != null ) {
	return currenttarget;
      }
      else {
	System.out.println( "Target object not selected." );
	return null;
      }
    }
  }

  public int dispatch( String commandline ) {

    int st;
    KonoeOrderly target;

    st = parser.parse( commandline );
    if( st < 0 ) {
      System.out.println( "Error parsing " + commandline );
      st = 0;
    }
    else if( st == KonoeCommandParser.COMMAND_PARSED ) {
      st = 0;
      command = parser.getCommand( );
      // command.dump( new PrintWriter( System.out, true ) );
      target = select_object( );
      if( target != null ) {
	st = target.execute( command );
      }
    }
    command = null;
    return st;
  }

  public KonoeOrderly findObject( String name ) {

    KonoeOfficer  aofficer;
    KonoeOrderly  orderly;

    officerlist.reset( );
    while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null  ) {
      orderly = (KonoeOrderly) aofficer.findObject( name );
      if( orderly != null )  return orderly;
    }
    return null;
  }

  public boolean addOrderly( KonoeOrderly orderly ) {

    KonoeOfficer aofficer;
    String classname = orderly.getClassName( );

    officerlist.reset( );
    while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null  )
      if( ( (KonoeOrderly) aofficer.hasClass( classname ) ) != null )
        if( aofficer.addOrderly( orderly ) >= 0 ) return true;
    return false;
  }

  public String getClassName( ) { return classname; }

  public int showTarget( KonoeCommand cmd ){

    KonoeCommandArgument arg;
    KonoeOfficer aofficer;
    String item;

    arg = cmd.getArgument( ARG_SHOW_ITEM );
    item = arg.getValue( );
    if( item.equals("officer") ) {
      System.out.println( "Officers employed ..." );
      officerlist.reset( );
      while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null )
	System.out.println( aofficer.getObjectName( ) );
    }
    else if( item.equals("object") ) {
      System.out.println( "Objects registered ..." );
      officerlist.reset( );
      while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null )
	aofficer.printObjectList( new PrintWriter( System.out, true ) );
    }
    else if( item.equals("class") ) {
      System.out.println( "Classes manageable ..." );
      officerlist.reset( );
      while( ( aofficer = (KonoeOfficer) officerlist.next( ) ) != null )
	aofficer.printClassList( new PrintWriter( System.out, true ) );
    }
    else {
      System.out.println( "Unknown item." );
      System.out.println( "  item : officer,object,class");
    }
  return 0;
  }

}
