//====================================================================
/*

	KonoeOrderly.java

	Version 0.0.1	 2-Jul-1998	H. Sakamoto
	Version 0.0.2	15-Aug-1998	S. Nishida
	Version 0.4.0	29-Sep-1998	S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi

*/
//====================================================================

package konoe.ui.command;

import java.util.*;
import java.io.*;
import konoe.ui.command.*;
import konoe.ui.system.*;

abstract public class KonoeOrderly {

  public static final int INTEGER = KonoeArgumentDescriptor.INTEGER;
  public static final int FLOATING = KonoeArgumentDescriptor.FLOATING;
  public static final int STRING = KonoeArgumentDescriptor.STRING ;
  public static final int KEYWORD = KonoeArgumentDescriptor.KEYWORD;
  public static final int OPTIONAL = KonoeArgumentDescriptor.OPTIONAL;
  public static final int NECESSARY = KonoeArgumentDescriptor.NECESSARY;

  String objectname;
  KonoeList commandlist; // member is KonoeCommandProcessor
  //  KonoeOfficer officer;
  Object officer;
  int classid;

  public KonoeOrderly( int ptablesize , String name ) {

    if( ptablesize > 0 ) {
      commandlist = new KonoeList( ptablesize );
    } else {
      commandlist = null;
    }
    if( name != null ) {
      objectname = name ;
    }
  }

  public String getObjectName( ) {

    return objectname;
  }

  public int addCommand( KonoeCommandProcessor cmd ) {

    if( commandlist == null )
      return -1;
    return commandlist.add( cmd );
  }

  KonoeCommandProcessor hasMethod( String method ) {

    KonoeCommandProcessor  cd;
    if( commandlist == null ) return null;
    commandlist.reset( );
    while( ( cd = (KonoeCommandProcessor) commandlist.next( ) ) != null  ) {
      if( method.equals( cd.getMethodName( ) ) ) return cd;
    }
    return null;
  }

  public int getCommandCount( ) {

    if( commandlist == null ) return 0;
    return commandlist.getCount( );
  }

  public void setOfficer( Object pofficer ) { officer = pofficer; }
  public Object getOfficer( ) { return officer; }
  public void setClassID( int pclassid ) { classid = pclassid; }
  public int getClassID( ) { return classid; }

  public void printHelp( KonoeCommand command, PrintWriter writer ) {

    KonoeCommandProcessor cd;
    KonoeCommandArgument arg;

    if( objectname != null )
      writer.println( "Object = " + objectname );
    else
      writer.println( "Class = " + this.getClassName( ) );
    if( command.getArgumentCount( ) == 0 ) {
      if( commandlist == null ) {
	writer.println( "    No commands available for this object" );
      } 
      else {
	writer.println( "    Commands available ..." );
	commandlist.reset( );
	while( ( cd = (KonoeCommandProcessor) commandlist.next( ) ) != null ){
	  writer.print( "\t" );
	  cd.printHelp( writer, false );
	}
      }
    } 
    else {
      arg = command.getArgument( 0 );	// first argument
      cd = hasMethod( arg.getValue( ) );
      if( cd == null ) {	// no match
	writer.println( "Command not found = " + arg.getValue( ) );
      } else {
	cd.printHelp( writer, true );	// detailed.
      }
    }
  }

  public abstract String getClassName( );

  public int execute( KonoeCommand cmd ) {

    KonoeCommandProcessor   cd;
    //    ostream & os = cmd.getOutputStream( );

    cd = hasMethod( cmd.getMethodName( ) );
    if( cd != null ) {
      return cd.execute( cmd );
    } else {
      System.out.println( "Method not found. " + cmd.getMethodName( ) );
      return 0;
    }
  }

  public int createObject( KonoeCommand command ) { return 0; }
  public int deleteObject( KonoeCommand command ) { return 0; }
  public int showTarget( KonoeCommand command ) { return 0; }
  public KonoeOrderly clone( String name ) { return null; }
  public Object getTarget( ) { return null; }
  public void setTarget( Object target ) { }
  
}

