//=========================================================
//
//      EventDemo.java
//
//	Version 0.0.1	03-Jul-1998	K.Ohta
//	Version 0.0.2	03-Jul-1998	K.Ohta
//	Version 0.4.0	05-Aug-1998	D.Takagi
//
//=========================================================

import	java.applet.*;
import	java.awt.*;
import	java.io.*;
import  java.util.*;

import  konoe.ui.parts.*;
import  konoe.ui.analysis.*;
import  konoe.ui.graphics.*;
import  konoe.ui.system.*;

public class EventDemo extends Applet implements Runnable {
    KonoeInputObjectSocket in;  

    KonoeEventViewCanvas eventViewer;
    KonoeDrawPanel panel;

    Vector placementlist = new Vector();

    public void init() {
	String host = "localhost";
//	String host = "202.13.212.70";
	int    port = 20000;

	try {
	    in = new KonoeInputObjectSocket(host, port);

	    KonoeStreamHeader header = in.readHeader();
	    // KonoeDetectorList ?
	    if ((header.getCategoryID() == 0x64) && 
		(header.getClassID() == 0x02)) {

		// search every element
		KonoeDetectorList detectorlist = 
		    (KonoeDetectorList)(in.readObject(header));

		for (int i = 0; i < detectorlist.getCount(); i++) {

		    KonoeDetector detector = detectorlist.get(i);
		    int detectorID = detector.getID();

		    for (int j = 0; j < detector.getCount(); j++) {

			KonoeDetectorElement element = detector.get(j);
			addPlacement(element, detectorID);
			
		    }

		}

	    } else {
		System.err.println("Invalid data type: not KonoeDtectorList");
		System.exit(1);
	    }

	} catch (IOException e ) {
	    System.err.println("IOException: ");
	    System.exit(1);
	}

	eventViewer = new KonoeEventViewCanvas ( 
	     new KonoeDimension(950, 700), "Event Viewer", placementlist.size());

	KonoePhysPlacement[] array = new KonoePhysPlacement[placementlist.size()];
	for (int i = 0; i < placementlist.size(); i++) {
	    array[placementlist.size() - i - 1] = 
	        (KonoePhysPlacement)placementlist.elementAt(i);
	}
	eventViewer.setPhysicalVolume(array, 0, array.length);

	setLayout(new GridLayout(1, 2));
	add(new KonoeDrawPanel(eventViewer, "Event Display"));

    }

    public void start( ) {
	eventViewer.paint ( eventViewer.getGraphics() );
	System.out.println( "Applet "+getSize( ).toString( ) );

	new Thread(this).start();
    }

    public void run() {
	try {
	    while (true) {
		// read 1 event
		boolean loop = true;

		while (loop) {
		    KonoeStreamHeader header = in.readHeader();
		    switch (header.getClassID()) {

		      case 0x01: // Konoe3Vector
			Konoe3Vector v = (Konoe3Vector)in.readObject(header);
			if (v.x == 999) {
			    System.out.println("end of events");
			    System.exit(1);
			}
			loop = false;
			break;
			
		      case 0x05: // KonoeDetectorHit
			KonoeDetectorHit hit = 
			  (KonoeDetectorHit)(in.readObject(header));
			int elementID = hit.getElement().getID();
			((KonoePhysPlacement)placementlist.elementAt(elementID)).setVisAtt(255, 0, 0);
			break;
			
		      default:
			loop = false;
		    }
		    
		}
		eventViewer.paint ( eventViewer.getGraphics() );
		for (int i = 0; i < placementlist.size(); i++) {
		    int r, g, b;
		    if (((i / 100) % 2) == 0) {
			r = 0; g = 255; b = 0;
		    } else {
			r = 0; g = 0; b = 255;
		    }
		    ((KonoePhysPlacement)placementlist.elementAt(i)).setVisAtt(r, g, b);
		}
	    }
	    
	} catch (IOException e) {
	    System.err.println("connection closed");
	    System.exit(0);
	}
    }

    protected void addPlacement(KonoeDetectorElement element, int detectorID) {
	KonoeGeometry geometry = element.getGeometry();
	KonoeSolid solid = null;
	Konoe3DVector vector = null;

	if (geometry instanceof KonoeBoxGeometry) {

	    KonoeBoxGeometry box = (KonoeBoxGeometry)geometry;
	    Konoe3Vector displacement = box.getDisplacement();

	    float w = (float)box.getWidth() / 2;
	    float h = (float)box.getHeight() / 2;
	    float d = (float)box.getDepth() / 2;
	    float x = (float)displacement.x;
	    float y = (float)displacement.y;
	    float z = (float)displacement.z;
	    float size = 200;
	    
	    solid = new KonoeDrawableBox("", w * size, h * size, d * size);
	    vector = new Konoe3DVector((x + w) * size, (y + h) * size, (z + d) * size);

	} else {

	    System.err.println("Invalit geometry type");
	    System.exit(1);

	}

	KonoeRotationMatrix rotation = new KonoeRotationMatrix();
	KonoeTransform transform = new KonoeTransform(rotation, vector);
	KonoePhysPlacement placement = new KonoePhysPlacement(transform, solid);
	placement.setForceWireFrame();
	
	int r = 0, g = 0, b = 0;
	if ((detectorID % 2) == 0) {
	    r = 0; g = 255; b = 0;
	} else {
	    r = 0; g = 0; b = 255;
	}

	placement.setVisAtt(r, g, b);
	placementlist.addElement(placement);
    }
    
    public void stop( ) {
      
    }
}

