//==================================================================
/*

	demosend.cc

	Version 0.2.0	30-Apr-1998	H. Sakamoto
	Version 0.3.0    5-Jun-1998     D. Takagi
*/
//==================================================================

#include	<daq/KonoeSimCrate.hh>
#include	<daq/KonoeSimTOFTDC.hh>
#include        <daq/KonoeSimADCDigitizer.hh>
#include	<daq/KonoeSimTDCModule.hh>
#include	<daq/KonoeSimADCModule.hh>

#include	<daq/KonoeReadoutList.hh>
#include	<daq/KonoeSoftwareCrate.hh>
#include	<daq/KonoeSoftCounter.hh>
#include	<daq/KonoeCrateList.hh>
#include	<daq/KonoeTimerXaction.hh>
#include	<analysis/KonoeDataRecordCM.hh>
#include	<analysis/KonoeGeometryCM.hh>
#include	<analysis/KonoeBoxGeometry.hh>
#include	<system/KonoeOutputObjFile.hh>
#include        <system/KonoeServerSocket.hh>
#include        <system/KonoeOutputObjSocket.hh>

#include	<iostream.h>
#include 	<stdio.h>
#include        "DemoEventGenerator.hh"

//------------------------------------------------------------------

// classes

//------------------------------------------------------------------

class	DemoSimulator : public KonoeSimCrate {
        KonoeOutputObjectStream * out;
public :
	DemoSimulator( KInt modulecount, KonoeDetectorList * dl, 
		       KonoeEventGenerator * g, KInt digirizercount );
        KInt setStream( KonoeOutputObjectStream * pout){ out = pout; }
        KInt processHit( KonoeSimDetectorHit * hit );
};

class	GeometryDemo	{
public :
	enum	{ N_EVENT = 20 };
	enum	{ TDC_GAIN = 10, ADC_GAIN =  1000 };

protected :
	KonoeDetectorList	* detectorlist;
	KonoeDetector	        * scifi[8];
	KonoeDetector	        * cal;

	KonoeSoftwareCrate	* softcrate;
	KonoeSoftCounter	* eventnumber;

	KonoeCrateList		* cratelist;
	KonoeReadoutList	* readoutlist;

	DemoEventGenerator	* generator;

	DemoSimulator   	* simulator;

	KonoeTimerXaction	* transaction;
        KonoeOutputObjFile      * outputfile;
// port1
	KonoeOutputObjectStream * dataout;
	KonoeServerSocket	* datasocket;
// port2
	KonoeOutputObjectStream * detectorhitout;
	KonoeServerSocket	* detectorhitsocket;

public :
	GeometryDemo( KInt argc, KChar ** argv );
	~GeometryDemo( );
	KInt	go( );


};

//--------------------------------------------------------------------

// methods

//--------------------------------------------------------------------

DemoSimulator::DemoSimulator( KInt modulecount, 
      KonoeDetectorList * dl, 
      KonoeEventGenerator * g,
      KInt digitizercount ) :
      KonoeSimCrate( modulecount, dl, g, digitizercount ) {
}

KInt    DemoSimulator::processHit( KonoeSimDetectorHit * hit ) {

        KInt    naddress;
        KonoeDetectorElement    * e;
        KonoeDataAddress        * a;
        KonoeSimDigitizer       * d;

        out -> put( hit, 'A', 0 ); 

        e = hit -> getDetectorElement( );
        e -> reset( );
        while( a = e -> next( ) ) {
            d = digitizerlist -> find( a -> getSectionID( ) );
            if( d ) {
                d -> digitize( hit, a, getModule( a ) );
            }
            else {
                cout << "no digitizer found" << endl;
            }
        }

        return 0;
}

// Detector Setting
//
// ->z
// 0m   0.5    1.0    1.5    2.0    2.5    3.0    3.5m
//                                                  _
//              ||     ||            ||     ||     | |
//             H||H   H||H          H||H   H||H    |C|
//             X||Y   X||Y          X||Y   X||Y    |A|
//             1||1   2||2          3||3   4||4    |L|
//              ||     ||            ||     ||     |_|
//
// DetectorID  0  1   2  3          4  5   6  7     8
//

// DataRecord Format
//
//     Sec 0: General
//         Seg 0: Info
//             Ele 0: Event#
//
//     Sec 1-8: SciFi
//         Seg 0-3: TDC0 - TDC3
//             Add 0-31
//
//     Sec 9: Cal TDC0 - TDC3
//         Seg 0-3 TDC
//             Add 0-31
//
//     Sec 10: Cal ADC
//         Seg 0-3 ADC0 - ADC3
//             Add 0-31

GeometryDemo::GeometryDemo( KInt argc, KChar ** argv ){

// serversocket
	KInt port1 = 10000;
	KInt port2 = 20000;
	
	if (argc > 1) sscanf(argv[1], "%d", &port1);
	if (argc > 2) sscanf(argv[2], "%d", &port2);

	datasocket = new KonoeServerSocket( port1 );
	dataout  = new KonoeOutputObjSocket( datasocket  );

	detectorhitsocket = new KonoeServerSocket( port2 );
	detectorhitout =  new KonoeOutputObjSocket( detectorhitsocket );
//
	KInt elementID = 0;

// set up a demo detecter
	detectorlist = new KonoeDetectorList( 9 );
	for ( KInt i = 0; i < 8; i++ ) {
	  scifi[i] = new KonoeDetector( 100, i );
	  detectorlist -> add( scifi[i] );
	}
	cal = new KonoeDetector( 100, 8 );
	detectorlist -> add( cal );

// scintilation fibers
	double z[] = { 1.0, 1.1, 1.5, 1.6, 2.5, 2.6, 3.0, 3.1 };
	KonoeTranslation t;

	KonoeDetectorElement * e;
// scifi ( HX & HY )
	for ( KInt i = 0; i < 8; i ++ ) {
	    for ( KInt j = 0; j < 100; j ++) {
	        KonoeBoxGeometry * b;
	        Konoe3Vector p;
	    
	        if (i % 2 == 0) {
	            // HX ( 0, 2, 4, 6 )
	            b = new KonoeBoxGeometry( 2.0, 0.02, 0.01 );
	            p.set( -1.0, j * 0.02 - 1.0, z[i] );
	        } else {
	            // HY ( 1, 3, 5, 7 )
		    b = new KonoeBoxGeometry( 0.02, 2.0, 0.01 );
		    p.set( j * 0.02 - 1.0, -1.0, z[i] );
		}	    
		t.setDisplacement( p );
		b -> translate( & t);
	    
		scifi[i] -> add( e = new KonoeDetectorElement(
                                          b, scifi[i], elementID++ ) );
		int md = j / 32;
		int ch = j % 32;
		e -> add( new KonoeDataAddress( i + 1, md, ch ) );
	    }
	}

	// cal
	for ( KInt i = 0; i < 10; i ++ ) {
 	    for ( KInt j =  0; j < 10; j ++ ) {
	        KonoeBoxGeometry * b = new KonoeBoxGeometry( 0.2, 0.2, 0.2 );
		Konoe3Vector p( i * 0.2 - 1.0, j * 0.2 - 1.0, 3.5 );
		t.setDisplacement( p );
		b -> translate( & t );

		cal -> add( e = new KonoeDetectorElement( b, cal, elementID++ ));
		int md = ( i * 10 + j ) / 32;
		int ch = ( i * 10 + j ) % 32;

		e -> add( new KonoeDataAddress(  9, md, ch ) );
		e -> add( new KonoeDataAddress( 10, md, ch ) );
	    }
	}

// cout << "detector element  " << elementID << endl;

// set up an event generator

	generator = new DemoEventGenerator( );

// software modules

	softcrate = new KonoeSoftwareCrate( 1 );
	eventnumber = new KonoeSoftCounter( );
	softcrate -> install( eventnumber );

// simulator

	KonoeSimModule *tdc[9][4];
	KonoeSimModule *adc[4];

	simulator = new DemoSimulator( 40, detectorlist, generator, 11 );
	for ( KInt i = 0; i < 9; i ++ ) {
 	    for ( KInt j = 0; j < 4; j ++ ) {
	        tdc[i][j] = new KonoeSimModule( i + 1, j, 32 );
	        simulator -> add( tdc[i][j] );
	    }
	}
	for ( KInt i = 0; i < 4; i ++ ) {
	    adc[i] = new KonoeSimModule( 10, i, 32 );
	    simulator -> add( adc[i] );
	}

// crate list

	cratelist = new KonoeCrateList( 2 );
	cratelist -> add( softcrate );
	cratelist -> add( simulator );

// difitizer

	for ( KInt i = 1; i < 10; i ++ ) {
	    simulator -> addDigitizer( new KonoeSimTOFTDC( i, 0, TDC_GAIN ) );
	}
	simulator -> addDigitizer( new KonoeSimADCDigitizer( 10, 0, ADC_GAIN ) );

// readout procedure
	
	readoutlist = new KonoeReadoutList( 11, 0 );

	KonoeReadoutSection * section;
	KonoeReadoutSegment * segment;

	// general section
	segment = new KonoeReadoutSegment( 1, 0 );
        segment -> add( new KonoeReadoutItem( eventnumber, (KInt)0 ) );
	section = new KonoeReadoutSection( 1, 0 );
        section -> add( segment );
	readoutlist -> add( section );

 	// scifi tdc
	for ( KInt i = 0; i < 8; i ++ ) {
	    section = new KonoeReadoutSection( 4, i + 1);
	    for ( KInt j = 0; j < 4; j ++ ) {
	        segment = new KonoeReadoutSegment( tdc[i][j] );
		section -> add( segment );
	    }
	    readoutlist -> add( section );
	}

	// cal tdc
	section = new KonoeReadoutSection( 4, 9 );
	for ( KInt j = 0; j < 4; j ++ ) {
	    segment = new KonoeReadoutSegment( tdc[8][j] );
	    section -> add( segment );
	}
	readoutlist -> add( section );

	// cal adc
	section = new KonoeReadoutSection( 4, 10 );
	for ( KInt j = 0; j < 4; j ++ ) {
	    segment = new KonoeReadoutSegment( adc[j] );
	    section -> add( segment );
	}
	readoutlist -> add( section );
	
// output
}

GeometryDemo::~GeometryDemo( ) {

	delete detectorlist;
	delete readoutlist;
	delete dataout;
	delete detectorhitout;
}

KInt	GeometryDemo::go( ) {

// connection
	cerr << "waiting for connection No1...";
	while( datasocket -> waitConnection( ) == -1 );
	cerr << " done." << endl;
	
        cerr << "waiting for connection No2...";
	while(  detectorhitsocket -> waitConnection( ) == -1 );
	cerr << " done." << endl;

        KonoeGeometryCM * geometrycm = new KonoeGeometryCM();
        detectorhitout -> installCM( geometrycm );

        detectorlist -> reset();

        cout<< "sending detectorlist... ";
        detectorhitout -> put( detectorlist, 'A', 0 ); 
	cout<< "done " << endl;
//
        KonoeDataRecordCM * datarecordcm = new KonoeDataRecordCM();
	dataout -> installCM( datarecordcm );

        simulator -> setStream( detectorhitout );
//        
	KonoeTimerXaction::initialize( );
	transaction = new KonoeTimerXaction( readoutlist );
	transaction -> setInterval( 2000 );
	transaction -> enable( );

	cratelist -> clear();

        Konoe3Vector * vector = new Konoe3Vector(0, 0, 0);  
	
        for (KInt i = 0; i < 100; i ++ ) {
	    cout << "sending Event# " << i + 1 << endl;
            detectorhitout -> put( vector );
	    cratelist -> update( );
	    KonoeDataRecord * record = transaction -> read( );

	    cout<< "sending record"<<endl;
	    record -> reset();
	    dataout -> put( record, 'A', 0 );
//	    record -> dump(cout);
	    delete record;
	  }

	vector->x = 999;
	dataout -> put( vector ); // event end 

	return 0;

}

//-------------------------------------------------------------------

// entry point

//-------------------------------------------------------------------

int	main( int argc, char ** argv ) {

	GeometryDemo demo( argc, argv );

	return demo.go();

}
