//=========================================
/*
    KonoePrint.java

      Version 0.6.0 05-Dec-2000 A.Izumoto

*/
//=========================================
package konoe.ui.graphics;

import java.awt.*;
import java.awt.print.*;
import javax.swing.*;
import konoe.ui.system.*;

public class KonoePrint implements KonoeObject, Printable{

  public static final String className = "KonoePrint";
  public static final int ORIENTATION_BEST_FIT = 0;
  public static final int ORIENTATION_PORTRAITE = 1;
  public static final int ORIENTATION_LANDSCAPE = 2;
  private Component target = null;
  private boolean scaleUp = false;
  private Thread printingThread = null;
  private int orientation = ORIENTATION_BEST_FIT;


  public KonoePrint(){

  }

  public void printTarget(Component c){

    target = c;
    PrinterJob job = PrinterJob.getPrinterJob();
    job.setPrintable(this);
    boolean dialog = job.printDialog();
    if(dialog){
      RepaintManager rm = RepaintManager.currentManager(target);
      boolean save = rm.isDoubleBufferingEnabled();
      try{
        rm.setDoubleBufferingEnabled(false);
        setPrintingThread(Thread.currentThread());
        job.print();
      }catch(Exception e){
        System.out.println(e);
      }finally{
        rm.setDoubleBufferingEnabled(save);
        setPrintingThread(null);
      }
    }
 
  }

  public void setPrintingThread(Thread t){
    printingThread = t;
  }


  public int print(Graphics g,PageFormat p,int n) throws PrinterException{

    if(n >= 1){
      return Printable.NO_SUCH_PAGE;
    }
    Graphics2D d = (Graphics2D)g;
    d.translate(p.getImageableX(),p.getImageableY());
    Dimension size = target.getSize();
    double pWidth = p.getImageableWidth();
    double pHeight = p.getImageableHeight();
    double portraite = Math.min(pWidth/size.width , pHeight/size.height);
    double landscape = Math.min(pWidth/size.height,pHeight/size.width);
    
    boolean useLand = (orientation == ORIENTATION_BEST_FIT)?
        landscape > portraite : orientation == ORIENTATION_LANDSCAPE;
    if(useLand){
      d.rotate(Math.PI/2);
      d.translate(0,-pWidth);
      d.scale(landscape,landscape);
    }else{
      d.scale(portraite,portraite);
    }
    target.paint(d);
    return Printable.PAGE_EXISTS;

  }

  public void setOrientation(int o){
    
    orientation = o;

  }

  public void setScaleUp(boolean scale){
      
    scaleUp = scale;

  }

  public void dump(){

  }


}

  

    
