//===========================================================
/*

	KonoeTrendControlPanel.java

	Version 0.0.1	19-Jun-1998	H. Sakamoto
    Version 0.5.0   28-Sep-1999 T. Hosoi

*/
//===========================================================

package	konoe.ui.trend;

/**

	Control components for trend view display
	
	@see	KonoeTrendView
 */

import	java.awt.*;
import	java.awt.event.*;
import	java.util.*;
import	konoe.ui.trend.*;

public class KonoeTrendControlPanel extends Panel implements
    ItemListener, AdjustmentListener {

  KonoeTrendData	itemlist;
  int	itemcount;
  KonoeTrendView	trendview;
  Choice	datachoice;
  Label		presenttime;
  Label		presentvalue;
  Label		maximumvalue;
  Label		minimumvalue;
  Scrollbar	maxslider;
  Scrollbar	minslider;
  Checkbox	catchupcheckbox;
  Checkbox	hidecheckbox;
  
  public KonoeTrendControlPanel( KonoeTrendView trendview ) {
  
    super( new GridLayout( 7, 1 ) );
    this.trendview = trendview;
    itemlist = trendview.getItemList( );
    itemcount = itemlist.getItemCount( );
    
    Panel toppanel = new Panel( new GridLayout( 1, 2 ) );
    datachoice = new Choice( );
    for( int i = 0; i < itemcount; i ++ ) {
      datachoice.add( itemlist.getItem( i ).getName( ) );
    }
    datachoice.addItemListener( this );
    toppanel.add( datachoice );
    catchupcheckbox = new Checkbox( "Catch up", true );
    catchupcheckbox.addItemListener( this );
    toppanel.add( catchupcheckbox );
    add( toppanel );
    
    Panel status = new Panel( new GridLayout( 1, 2 ) );
    Date now = new Date( );
    presenttime = new Label( now.toString( ).substring( 0, 19 ) );
    presentvalue = new Label( "0.0000" );
    presentvalue.setAlignment( Label.RIGHT );
    status.add( presenttime );
    status.add( presentvalue );
    add( status );
    
    Panel maximum = new Panel( new GridLayout( 1, 2 ) );
    maximum.add( new Label( "Maximum" ) );
    maximumvalue = new Label( "0.0000" );
    maximumvalue.setAlignment( Label.RIGHT );
    maximum.add( maximumvalue );
    add( maximum );
    maxslider = new Scrollbar( Scrollbar.HORIZONTAL, 0, 1, 0, 100 );
    maxslider.addAdjustmentListener( this );
    add( maxslider );
    
    Panel minimum = new Panel( new GridLayout( 1, 2 ) );
    minimum.add( new Label( "Minimum" ) );
    minimumvalue = new Label( "0.0000" );
    minimumvalue.setAlignment( Label.RIGHT );
    minimum.add( minimumvalue );
    add( minimum );
    minslider = new Scrollbar( Scrollbar.HORIZONTAL, 0, 1, 0, 100 );
    minslider.addAdjustmentListener( this );
    add( minslider );
    
    Panel penselection = new Panel( new GridLayout( 1, 1 ) );
    hidecheckbox = new Checkbox( "Hide", false );
    hidecheckbox.addItemListener( this );
    penselection.add( hidecheckbox );
    add( penselection );
    
    validate( );

  }
  
  public void setup( ) {
  
    KonoeTrendItem	item = trendview.getCurrentItem( );
    int		min, max, low, high;
    min = ( int ) item.getMinimum( );
    max = ( int ) item.getMaximum( );
    low = ( int ) item.getLower( );
    high = ( int ) item.getUpper( );
    minimumvalue.setText( Double.toString( item.getLower( ) ) );
    maximumvalue.setText( Double.toString( item.getUpper( ) ) );
    minslider.setValues( low, 1, min, max );
    maxslider.setValues( high, 1, min, max );
    hidecheckbox.setState( item.getPenIndex( ) < 0 );
    trendview.drawScale( );
  
  }
  
  public void update( ) {
  
    KonoeTrendItem	item = trendview.getCurrentItem( );
    int	index = itemlist.getCurrentIndex( );
    presenttime.setText(
      itemlist.getTime( index ).toString( ).substring( 0, 19 ) );
    presentvalue.setText(
      Double.toString( item.getValue( index ) ).substring( 0, 8 ) );

  }
  
  public void itemStateChanged( ItemEvent e ) {
  
    if( e.getItemSelectable( ) instanceof Checkbox ) {
      Checkbox cb = ( Checkbox ) e.getItemSelectable( );
      if( cb == catchupcheckbox )
        trendview.setCatchup( cb.getState( ) );
      else if( cb == hidecheckbox ) {
        KonoeTrendItem item = trendview.getCurrentItem( );
        item.setPenIndex( - 1 - item.getPenIndex( ) );
        trendview.redraw( );
      }
    }
    else if( e.getItemSelectable( ) instanceof Choice ) {
      Choice ch = ( Choice ) e.getItemSelectable( );
      String lbl = ch.getSelectedItem( );
      for( int i = 0; i < itemcount; i ++ ) {
        if( lbl.equals( itemlist.getItem( i ).getName( ) ) ) {
          trendview.setCurrentItem( i );
          setup( );
        }
      }
    }
  }
  
  public void adjustmentValueChanged( AdjustmentEvent e ) {
  
    if( e.getAdjustable( ) instanceof Scrollbar ) {
    
      Scrollbar sb = ( Scrollbar ) e.getAdjustable( );
      if( sb == minslider ) {
        double v = ( double ) sb.getValue( );
        minimumvalue.setText( Double.toString( v ) );
        trendview.getCurrentItem( ).setLower( v );
        trendview.redraw( );
      }
      else if( sb == maxslider ) {
        double v = ( double ) sb.getValue( );
        maximumvalue.setText( Double.toString( v ) );
        trendview.getCurrentItem( ).setUpper( v );
        trendview.redraw( );
      }
    }
  }

}
