//============================================================
/*

	KonoeTrendData.java
	
	Version 0.0.1	19-Jun-1998	H. Sakamoto
        Version 0.4.0   18-Aug-1998     K.Ohta
    Version 0.5.0   28-Sep-1999 T. Hosoi
*/
//============================================================

package	konoe.ui.trend;
/**

	Manages a set of trend data items.

**/

import	konoe.ui.trend.*;
import	java.util.*;

public class KonoeTrendData {

  static final int SHIFT_FRACTION = 5;
  KonoeTrendItem [] itemlist;
  int	tablesize;
  int	itemcount;
  int	storagesize;
  int	columnpointer;
  int	shiftsize;
  KonoeTrendDataSource	source;
  Date [] time;
  
  public KonoeTrendData( int itemcount, int storagesize,
  	KonoeTrendDataSource source ) {
  
    itemlist = new KonoeTrendItem[ itemcount ];
    tablesize = itemcount;
    this.itemcount = 0;
    this.storagesize = storagesize;
    time = new Date[ storagesize ];
    columnpointer = 0;
    this.source = source;
    shiftsize = storagesize / SHIFT_FRACTION;
  }
  
  public boolean addItem( KonoeTrendItem item ) {
    if( itemcount == tablesize )
      return false;
    itemlist[ itemcount ] = item;
    item.setPenIndex( itemcount );
    itemcount ++;
    return true;
  }
  
  public void setWindow( double lowerend, double upperend ) {
  
    for( int i = 0; i < itemcount; i ++ )
      itemlist[ i ].setWindow( lowerend, upperend );
  }
  
  public boolean fill( ) {

    boolean st = false;
    if( columnpointer >= storagesize ) {
      shift( );
      st = true;
    }
    for( int i = 0; i < itemcount; i ++ ) {
      itemlist[ i ].set( columnpointer,
        source.getTrendData( i ) );
    }
    time[ columnpointer ] = new Date( );
    columnpointer ++;
    return st;
  }
  
  void shift( ) {
  
    for( int i = 0; i < itemcount; i ++ )
      itemlist[ i ].shift( shiftsize );
    for( int j = 0; j < storagesize - shiftsize; j ++ )
      time[ j ] = time[ j + shiftsize ];
    columnpointer -= shiftsize;
    
  }
  
  public int getCurrentIndex( ) {
  
    return columnpointer - 1;
    
  }
  
  public int getItemCount( ) {
  
    return itemcount;
    
  }
  
  public KonoeTrendItem getItem( int index ) {
  
    return itemlist[ index ];
    
  }
  
  public Date getTime( int index ) {
  
    return time[ index ];
    
  }
}
