//===========================================================
/*
	KonoeTrendItem.java

	Version 0.0.1	19-Jun-1998	H. Sakamoto
        Version 0.4.0   18-Aug-1998     K.Ohta
    Version 0.5.0   28-Sep-1999 T. Hosoi
*/
//===========================================================

package	konoe.ui.trend;

/**

	Objects of this class describe properties of
	trend data.

	@author	Hiroshi Sakamoto
	@version	0.0.1	19-Jun-1998

*/

public class KonoeTrendItem {

  double [] data;
  int		storagesize;
  double	lowerend;
  double	upperend;
  double	minimum;
  double	maximum;
  int		pen_index;
  String	name;
  
/**
@param name		Name of trend data.
@param storagesize	Size of data table. Recording capacity.
@param minimum		Absolute minimum value.
@param maximum		Absolute maximum value which this can take.
*/  

  public KonoeTrendItem( String name, int storagesize,
  	double minimum, double maximum ) {


    data = new double[ storagesize ];
    this.storagesize = storagesize;
    this.name = name;
    this.minimum = minimum;
    lowerend = minimum;
    this.maximum = maximum;
    upperend = maximum;
    pen_index = -1;

  }

/**
	specifies the display window.
	
@param	lowerend	lower end value;
@param	upperend	upper end value;
**/

  public void setWindow( double lowerend, double upperend ) {
    this.lowerend = lowerend;
    this.upperend = upperend;
  }
  
/**
	Returns the pen index number given for the item.
**/
  public int getPenIndex( ) {
    return pen_index;
  }
  
  public void setPenIndex( int index ) {
    pen_index = index;
  }
  
  public String getName( ) {
    return name;
  }
  
  public void setName( String pname ) {
    name = pname;
  }
  
  public void shift( int shiftsize ) {
  
    int i, j;
    j = shiftsize;
    for( i = 0; i < storagesize - shiftsize; )
      data[ i++ ] = data[ j++ ];
  }
  
  public void set( int index, double value ) {
    data[ index ] = value;
  }
  
  public double getFraction( int index ) {
  
    double f = ( data[ index ] - lowerend ) /
    		( upperend - lowerend );
    if( f < 0.0 ) f = 0.0;
    if( f > 1.0 ) f = 1.0;
    return f;
    
  }
  
  public double getValue( int index ) {
  
    return data[ index ];
    
  }
  
  public double getMinimum( ) {
  
    return minimum;
    
  }
  
  public double getMaximum( ) {
  
    return maximum;
    
  }
  
  public double getLower( ) {
  
    return lowerend;
    
  }
  
  public void setLower( double v ) {
  
    lowerend = v;
    
  }
  
  public double getUpper( ) {

    return upperend;
    
  }
  
  public void setUpper( double v ) {
  
    upperend = v;
    
  }
}
