//=============================================================
/*

	KonoeAlarmMeter.java
	
	Version 0.0.1	20-Jun-1998	H. Sakamoto
	Version 0.0.2	02-Jul-1998	H. Sakamoto
        Version 0.4.0   18-Aug-1998     K.Ohta
    Version 0.5.0   28-Sep-1999 T. Hosoi
        Version 0.6.0   01-Feb-2000     T. Hosoi
*/
//=============================================================

package	konoe.ui.parts;

/**

	A meter component with upper/lower limit function.

 */
 
import	java.awt.*;
import	konoe.ui.parts.*;

public class KonoeAlarmMeter extends KonoeMeter {

  int		limitmode;
  double	lowerlimit;
  double	upperlimit;

  boolean	laststatus;
  boolean	currentstatus;
  
  Color		normalforeground;
  Color		normalbackground;
  Color		alartforeground;
  Color		alartbackground;
   
// constants

  public static final int WINDOW_LIMIT = 0;
  public static final int HIGH_LIMIT = 1;
  public static final int LOW_LIMIT = 2;
  public static final int NO_LIMIT = 3;
  public static final int N_MODES = 4;
  
  public KonoeAlarmMeter( int length, int width, double fullscale ) {
  
    super( length, width, fullscale );
    
    limitmode = NO_LIMIT;
    
    normalforeground = getForeground( );
    normalbackground = getBackground( );
    alartforeground = normalforeground;
    alartbackground = normalbackground;
    checkValue( );
        
  }
  
  public void paint( Graphics g ) {
  
    checkValue( );
    super.paint( g );
    
  }
  
  public void update( Graphics g ) {
  
    if( checkValue( ) )
      super.paint( g );
    else
      super.update( g );
  }
  
  public void setLimitMode( int mode ) {
  
    limitmode = mode;
    checkValue( );

  }

  public int getLimitMode( ) {
  
    return limitmode;
    
  }
  
  public void setLimit( double ll, double ul ) {
  
    lowerlimit = ll;
    upperlimit = ul;
    
  }
  
  public void setUpperLimit( double v ) {
  
    upperlimit = v;
    
  }
  
  public double getUpperLimit( ) {
  
    return upperlimit;
    
  }
  
  public void setLowerLimit( double v ) {
  
    lowerlimit = v;
    
  }
  
  public double getLowerLimit( ) {
  
    return lowerlimit;
    
  }
  
  public void setColor( Color nf, Color nb ) {
  
    normalforeground = nf;
    normalbackground = nb;
    checkValue( );
    
  }
  
  public void setAlartColor( Color af, Color ab ) {
  
    alartforeground = af;
    alartbackground = ab;
    checkValue( );
    
  }
  
  boolean checkValue( ) {
  
    laststatus = currentstatus;
    switch( limitmode ) {
    case HIGH_LIMIT :
      currentstatus = value > upperlimit;
      break;
    case LOW_LIMIT :
      currentstatus = value < lowerlimit;
      break;
    case WINDOW_LIMIT :
      currentstatus = ( value < lowerlimit ) || ( value > upperlimit );
      break;
    case NO_LIMIT :
      currentstatus = false;
    }
    if( ! laststatus && currentstatus ) {
      foreground = alartforeground;
      background = alartbackground;
      return true;
    }
    else if( laststatus && ! currentstatus ) {
      foreground = normalforeground;
      background = normalbackground;
      return true;
    }
    if( currentstatus ) {
      foreground = alartforeground;
      background = alartbackground;
    }
    else {
      foreground = normalforeground;
      background = normalbackground;
    }
    return false;
  }  

}
