//==========================================================
/*

	KonoeAlarmMeterControl.java
	
	Version 0.0.1	20-Jun-1998	H. Sakamoto
	Version 0.0.2	02-Jul-1998	H. Sakamoto
    Version 0.5.0   28-Sep-1999 T. Hosoi
        Version 0.6.0   01-Feb-2000     T. Hosoi

*/
//==========================================================

package	konoe.ui.parts;

/**

	A control panel for KonoeAlarmMeterPanel
 */

import	java.awt.*;
import	java.awt.event.*;
import	konoe.ui.parts.*;

public class KonoeAlarmMeterControl extends Panel
	implements ItemListener, AdjustmentListener {

  CheckboxGroup	checkboxgroup;
  Checkbox [] checkbox = new Checkbox[ KonoeAlarmMeter.N_MODES ];
  static final String [] checkboxlabels = { "Window",
	"High", "Low", "None" };
  static final String upperlimitlabel = "Upper Limit";
  static final String lowerlimitlabel = "Lower Limit";
  KonoeSliderPanel	lowerlimit;
  KonoeSliderPanel	upperlimit;
  Panel		checkboxpanel;
  Panel		sliderpanel;
  int		minimum;
  int		maximum;
  
  KonoeAlarmMeter	client;
  
  public KonoeAlarmMeterControl( int minimum, int maximum ) {
  
    super( new GridLayout( 2, 1 ) );

// checkboxes

    checkboxpanel = new Panel( new GridLayout( 5, 1 ) );
    checkboxpanel.add( new Label( "Limit Mode" ) );
    checkboxgroup = new CheckboxGroup( );
    for( int i = 0; i < KonoeAlarmMeter.N_MODES; i ++ ) {
      checkboxpanel.add( checkbox[ i ] =
        new Checkbox( checkboxlabels[ i ], checkboxgroup,
        i == KonoeAlarmMeter.WINDOW_LIMIT ) );
      checkbox[ i ].addItemListener( this );
    }
    add( checkboxpanel );

// sliders

    this.maximum = maximum;
    this.minimum = minimum;
    
    upperlimit = new KonoeSliderPanel( upperlimitlabel,
    	maximum, 1, minimum, maximum );
    upperlimit.getScrollbar( ).addAdjustmentListener( this );
    lowerlimit = new KonoeSliderPanel( lowerlimitlabel,
    	minimum, 1, minimum, maximum );
    lowerlimit.getScrollbar( ).addAdjustmentListener( this );
    sliderpanel = new Panel( new GridLayout( 3, 1 ) );
    sliderpanel.add( new Label( "Limit Values" ) );
    sliderpanel.add( upperlimit );
    sliderpanel.add( lowerlimit );
    add( sliderpanel );


// client alarm meter

    client = null;

  }

  public void setClient( KonoeAlarmMeterPanel panel ) {

    setClient( panel.getAlarmMeter( ) );
    
  }
  
  public void setClient( KonoeAlarmMeter meter ) {
  
    client = meter;
    checkboxgroup.setSelectedCheckbox(
    	checkbox[ client.getLimitMode( ) ] );
    upperlimit.setValue( ( int ) client.getUpperLimit( ) );
    lowerlimit.setValue( ( int ) client.getLowerLimit( ) );
    
  }
  
  public void itemStateChanged( ItemEvent e ) {

    if( e.getStateChange( ) == ItemEvent.DESELECTED )
      return;
    if( client == null )
      return;
    Checkbox cb = checkboxgroup.getSelectedCheckbox( );
    String label = cb.getLabel( );
    for( int i = 0; i < KonoeAlarmMeter.N_MODES; i ++ )
      if( label.equals( checkboxlabels[ i ] ) ) {
        client.setLimitMode( i );
        return;
      }      
  }
  
  public void adjustmentValueChanged( AdjustmentEvent e ) {

    if( client == null )
      return;
    Scrollbar	sb = ( Scrollbar ) e.getAdjustable( );
    int	v = e.getValue( );
    if( sb.getName( ).equals( upperlimitlabel ) ) {
      client.setUpperLimit( ( double ) v );
      upperlimit.setValueLabel( v );
    }
    else if( sb.getName( ).equals( lowerlimitlabel ) ) {
      client.setLowerLimit( ( double ) v );
      lowerlimit.setValueLabel( v );
    }
  }
}
