//=============================================================
/*

        KonoeInputObjectStream.java

        Version 0.0.1   03-Aug-1998     D. Takagi
        Version 0.5.0   28-Sep-1999     T. Hosoi
        Version 0.6.0   01-Feb-2000     T. Hosoi

*/
//=============================================================
package konoe.ui.system;

import java.io.*;
import java.net.*;

public abstract class KonoeInputObjectStream implements KonoeObject {
    private KonoeCategoryManager categoryManager;

    public KonoeInputObjectStream() {
	categoryManager = new KonoeDefaultCategoryManager();
    }

    public KonoeInputObjectStream(KonoeCategoryManager categoryManager) {
	this.categoryManager = categoryManager;
    }

    protected abstract DataInputStream in();

    public  abstract void close();

    public  byte[] readBytes(int size) throws IOException {
	byte[] b = new byte[size];
	in().readFully(b, 0, size);
	return b;
    }

    public char   readChar()   throws IOException { return in().readChar();   }
    public int    readInt()    throws IOException { return in().readInt();    }
    public long   readLong()   throws IOException { return in().readLong();   }
    public float  readFloat()  throws IOException { return in().readFloat();  }
    public double readDouble() throws IOException { return in().readDouble(); }

    public char[] readCharArray(KonoeStreamHeader header) throws IOException {
	int dataCount = header.getDataCount();
	char[] array = new char[dataCount]; 
	for (int i = 0; i < dataCount; i++) {
	    array[i] = in().readChar();
	}
	return array;
    }

    public int[] readIntArray(KonoeStreamHeader header) throws IOException { 
	int dataCount = header.getDataCount();
	int[] array = new int[dataCount]; 
	for (int i = 0; i < dataCount; i++) {
	    array[i] = in().readInt();
	}
	return array;
    }

    public long[] readLongArray(KonoeStreamHeader header) throws IOException {
	int dataCount = header.getDataCount();
	long[] array = new long[dataCount]; 
	for (int i = 0; i < dataCount; i++) {
	    array[i] = in().readLong();
	}
	return array;
    }

    public float[] readFloatArray(KonoeStreamHeader header) throws IOException { 
	int dataCount = header.getDataCount();
	float[] array = new float[dataCount]; 
	for (int i = 0; i < dataCount; i++) {
	    array[i] = in().readFloat();
	}
	return array;
    }

    public double[] readDoubleArray(KonoeStreamHeader header) throws IOException { 
	int dataCount = header.getDataCount();
	double[] array = new double[dataCount]; 
	for (int i = 0; i < dataCount; i++) {
	    array[i] = in().readDouble();
	}
	return array;
    }

    public void readHeader(KonoeStreamHeader header) throws IOException {
	byte[] b = readBytes(4 * 3);
	int level        = (int)b[0];
	int categoryID   = (int)b[1];
	int classID      = (int)b[2];
	int classVersion = (int)b[3];  
	int memberID     = (int)b[4];
	int dataCount    = (int)(((b[ 5] & 0xff) << 16) | 
				 ((b[ 6] & 0xff) << 8 ) | 
				 ((b[ 7] & 0xff)));
	int recordSize   = (int)(((b[ 8] & 0xff) << 24) | 
				 ((b[ 9] & 0xff) << 16) | 
				 ((b[10] & 0xff) << 8 ) | 
				 ((b[11] & 0xff)));

	header.setLevel(level);
	header.setCategoryID(categoryID);
	header.setClassID(classID);
	header.setClassVersion(classVersion);
	header.setMemberID(memberID);
	header.setDataCount(dataCount);
	header.setRecordSize(recordSize);
//        System.out.println(header.toString());
    }

    public KonoeStreamHeader readHeader() throws IOException {
	KonoeStreamHeader header = new KonoeStreamHeader();
	readHeader(header);
	return header;
    }

    public KonoeStreamableObject readObject() throws IOException {
	return readObject(readHeader());
    }

    public KonoeStreamableObject readObject(KonoeStreamHeader header) throws IOException {
header.toString();
/*
if(header.getCategoryID() == 0){
  readBytes(header.getDataCount());
return null;
}
*/
	KonoeStreamableObject object = (KonoeStreamableObject)(categoryManager.create(header.getCategoryID(), header.getClassID()));

	int dataCount = header.getDataCount();
	for (int i = 0; i < dataCount; i++) {
	    readHeader(header);
	    object.read(this, header);
	}
	return object;
    }

    public Object[] readObjectArray(KonoeStreamHeader header) throws IOException {
	int dataCount = header.getDataCount();
	Object[] array = new Object[dataCount];

	for (int i = 0; i < dataCount; i++) {
	    readHeader(header);
	    array[i] = readObject(header);
	}
	
	return array;
    }

    public void dump() {
	System.out.println(toString());
    }
}
