//====================================================================
/*

	KonoeCommandArgument.cc

	Version 0.0.1	 6-Jul-1998	H. Sakamoto
	Version 0.0.2	 1-Aug-1998	Sanshiro Enomoto
	Version 0.4.0	 7-Sep-1998	H. Sakamoto and S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi

*/
//====================================================================

#include	<ui/KonoeCommandArgument.hh>
#include	<string.h>
#include	<stdlib.h>
#include	<stdio.h>


KonoeCommandArgument::KonoeCommandArgument( KChar * pname, KChar * pvalue ) {

	name = new KChar[ strlen( pname ) + 1 ];
	strcpy( name, pname );
	value = new KChar[ strlen( pvalue ) + 1 ];
	strcpy( value, pvalue );
	appearance = ASSIGNMENT;

}

KonoeCommandArgument::KonoeCommandArgument( KInt pposition, KChar * pvalue ) {

	name = 0;
	value = new KChar[ strlen( pvalue ) + 1 ];
	strcpy( value, pvalue );
	appearance = POSITIONAL;
	position = pposition;

}

KonoeCommandArgument::KonoeCommandArgument( KChar * pname, KInt ptype ) {

	name = new KChar[ strlen( pname ) + 1 ];
	strcpy( name, pname );
	value = 0;
	appearance = ptype;

}

KonoeCommandArgument::~KonoeCommandArgument( ) {

	delete [] name;
	delete [] value;

}

void	KonoeCommandArgument::dump( ostream & os ) {

	switch( appearance ) {
	case POSITIONAL :
		os << "Positional at " << position << " : Value = " << value;
		break;
	case ASSIGNMENT :
		os << "Assignment " << name << " = " << value;
		break;
	case PLUS_KEYWORD :
		os << "Keyword +" << name;
		break;
	case MINUS_KEYWORD :
		os << "Keyword -" << name;
		break;
	}
	os << endl;
}

KInt	KonoeCommandArgument::checkInteger( ) {

	KChar	* p, c;

	p = value;
	if( * p == '\0' )	// null line
	  return 0;

	if( * p == '0' ) {
	  p ++;
	  if( * p == '\0' )
	    return 1;	// zero
	  if( * p == 'x' || * p == 'X' ) {
	    p ++;
	    while( c = * p ++ ) {
	      if( ( c < '0' || c > '9' ) && ( c < 'a' || c > 'f' )
		&& ( c < 'A' || c > 'F' ) )
		return 0;	// not 
	    }
	    type = HEXADECIMAL;
	  }
	  else {
	    p ++;
	    while( c = * p ++ ) {
	      if( c < '0' || c > '7' )
		return 0;
	    }
	    type = OCTAL;
	  }
	}
	else if( * p >= '1' && * p <= '9' ) {
	  p ++;
	  while( c = * p ++ ) {
	    if( c < '0' || c > '9' )
	      return 0;
	  }
	  type = DECIMAL;
	}
	else
	  return 0;	// not an integer expression;
	return 1;
}

KInt	KonoeCommandArgument::toInteger( ) {


	if( isExpression( ) ) {
	    if( expression == 0 )
		return 0;	// cannot evaluate.
	    return expression -> toInteger( );
	}
	else if( isSymbol( ) ) {
	  KonoeSymbol	* symbol = toSymbol( );
	  if( symbol == 0 )
	    return 0;
	  return symbol -> getIValue( );
	}
	return strtol( value, 0, 0 );

}

KInt	KonoeCommandArgument::isInteger( ) {

	if( isExpression( ) ) {
	    if( expression == 0 )
		return 0;	// cannot evaluate.
	    expression -> evaluate( value );
	    return expression -> isInteger( );
	}
	else if( isSymbol( ) ) {
	  KonoeSymbol	* symbol = toSymbol( );
	  if( symbol == 0 )
	    return 0;
	  return symbol -> getType( ) == INTEGER;
	}
	else
	    return checkInteger( );

}

KInt	KonoeCommandArgument::checkFloating( ) {
	char *endptr;
	strtod( value, &endptr );

	if ((endptr == 0) || (*endptr == '\0')) {
	    return 1;
	}
	else {
	    return 0;
	}
}

KFloat	KonoeCommandArgument::toFloating( ) {

	return strtod( value, (char**) 0 );

}

KInt	KonoeCommandArgument::isSymbol( ) {

	if( * value == '$' )
		return KTrue;
	else
		return KFalse;

}

KonoeSymbol	* KonoeCommandArgument::toSymbol( ) {

	if( expression == 0 )
	  return 0;

	KonoeSymbol	* symbol;
	KChar	* p = value + 1;	// next chara to $

	symbol = expression -> getResolverList( ) -> findSymbol( p );

	return symbol;

}

KInt	KonoeCommandArgument::isExpression( ) {

	if( * value == '(' )
		return KTrue;
	else
		return KFalse;

}

KInt	KonoeCommandArgument::isAlphabetic( ) {

	KChar	c = * value;

	if( ( c >= 'a' && c <= 'z' ) || ( c >= 'A' && c <= 'Z' ) ||
		c == '_' )
	    return KTrue;
	else
	    return KFalse;

}

KInt	KonoeCommandArgument::isString( ) {

	if( isSymbol( ) ) {
	  KonoeSymbol	* symbol = toSymbol( );
	  if( symbol -> getType( ) != STRING )
	    return 0;
	}
	return 1;

}

KChar	* KonoeCommandArgument::toString( ) {

	if( isSymbol( ) ) {
	  KonoeSymbol	* symbol = toSymbol( );
	  switch( symbol -> getType( ) ) {
	  case STRING :
	    return symbol -> getCValue( );
	  default :
	    return 0;
	  }
	}
	else
	  return value;

}
