//====================================================================
/*

	KonoeCommandArgument.hh

	Version 0.0.1	 6-Jul-1998	H. Sakamoto
	Version 0.0.2	 1-Aug-1998	Sanshiro Enomoto
	Version 0.4.0	 5-Sep-1998	H. Sakamoto and S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	25-Nov-2000	H. Sakamoto
	Version 0.6.2	19-May-2001	H. Sakamoto


---------------------------------------------------------------------

	interpretation of an argument.

	Primitive values.
		An integer number. Decimal, Hexadecimal, Octal
		A floating number.
		A character string. Keyword. name etc.

	Symbols
		A commander object.
		Other symbols ( alphanumeric starting with $ )
			should be resolved by a certain service.
		interpreter variables.

	Expressions
		Must be accompanied with ( ).
		Arithmetic and logic operations.
		Constants.
		Symbols.
		
	----
	implementation.

		KonoeSymbolResolver class
			returns the type of the symbol
			returns the value of the symbol
		-> KonoeCommandVariableResolver
		-> KonoeNameSpaceSymbolResolver

		KonoeExpressionAnalyzer class
			analyse an expression.
			evaluate the expression.

*/
//====================================================================

#ifndef	__KONOECOMMANDARGUMENT_HH
#define	__KONOECOMMANDARGUMENT_HH

#include	<KonoePrimitives.hh>
#include	<iostream.h>

#include	<ui/KonoeExpression.hh>

class	KonoeCommandArgument	{

public :
	enum	{	POSITIONAL,	ASSIGNMENT,
			PLUS_KEYWORD,	MINUS_KEYWORD	};
	enum	{	INTEGER,	FLOATING,
			STRING,		QUOTED_STRING,
			SYMBOL,		EXPRESSION	};
	enum	{	DECIMAL,	OCTAL,
			HEXADECIMAL			};
protected :
	KChar	* name;
	KChar	* value;
	KInt	appearance;
	KInt	type;
	KInt	position;
	KonoeExpression	* expression;

public :
	KonoeCommandArgument( KChar * name, KChar * value );
	KonoeCommandArgument( KInt position, KChar * value );
	KonoeCommandArgument( KChar * name, KInt flag );
	~KonoeCommandArgument( );

	const KChar	* getName( ) {
		return ( const KChar * ) name;	}
	const KChar	* getValue( ) {
		return ( const KChar * ) value;	}
	KInt	getAppearance( ) {
		return appearance;		}
	KInt	getType( ) {
		return type;			}
	void	dump( ostream & os );

	KInt	checkInteger( );
	KInt	toInteger( );
	KInt	isInteger( );

	KInt	checkFloating( );
	KFloat	toFloating( );

	KInt	isAlphabetic( );
	KInt	isString( );
	KChar	* toString( );

	KInt		isSymbol( );
	KonoeSymbol	* toSymbol( );

	KInt	isExpression( );

	void	setExpression( KonoeExpression * exp )
			{	expression = exp;	}

};


#endif
