//================================================================
/*

	KonoeCommandBuffer.cc

	Version 0.0.1	17-May-2001	H. Sakamoto

*/
//================================================================

#include	<ui/KonoeCommandBuffer.hh>
#include	<string.h>
#include	<iostream.h>

KonoeCommandBuffer::KonoeCommandBuffer( KInt pbuffersize ) {

	buffersize = pbuffersize;
	buffer = new KChar * [ buffersize ];
	linecount = 0;
	bufferincrease = buffersize;
	lineoffset = 0;

}

KonoeCommandBuffer::~KonoeCommandBuffer( ) {

	for( KInt i = 0; i < linecount; i ++ )
	    delete [] * ( buffer + i );

	delete [] buffer;

}

KChar	* KonoeCommandBuffer::getLine( KInt index ) {

	if( index < lineoffset || index >= linecount + lineoffset )
	    return 0;

	return * ( buffer + index );

}

KInt	KonoeCommandBuffer::putLine( const KChar * string ) {

	if( linecount < buffersize ) {
	    * ( buffer + linecount ) = new KChar [ strlen( string ) + 1 ];
	    strcpy( * ( buffer + linecount ), string );
	    linecount ++;
	    return linecount;
	}

	if( extend_buffer( ) == -1 ) {
	    return 0;
	}

	return putLine( string );

}

KInt	KonoeCommandBuffer::extend_buffer( ) {

	KChar	** newbuffer;
	newbuffer = new KChar * [ buffersize + bufferincrease ];
	if( newbuffer == 0 ) {
	    cerr << "KonoeCommandBuffer::extend_buffer."
			" Can't allocate a buffer." << endl;
	    return -1;
	}

	for( KInt i = 0; i < buffersize; i ++ )
	    * ( newbuffer + i ) = * ( buffer + i );
	delete [] buffer;
	buffer = newbuffer;
	buffersize += bufferincrease;

	return 0;

}

void	KonoeCommandBuffer::pushLine( const KChar * string ) {

	if( linecount < buffersize ) {
	    putLine( string );
	    return;
	}

	delete [] * buffer;	// buffer top.
	KChar	** p = buffer;
	for( KInt i = 0; i < buffersize - 1; i ++ ) {
	    * p = * ( p + 1 );
	    p ++;
	}
	lineoffset ++;
	* p = new KChar [ strlen( string ) + 1 ];
	strcpy( * p, string );
}

KInt	KonoeCommandBuffer::deleteLine( KInt index ) {

	if( index < lineoffset || index >= linecount + lineoffset )
	    return -1;

	KChar	** p;
	KInt	l = index - lineoffset;

	p = ( buffer + l );
	delete [] * p;

	for( KInt i = l; i < linecount - 1; i ++ ) {
	    * p = * ( p + 1 );
	    p ++;
	}
	linecount --;
	return 0;
	
}

KInt	KonoeCommandBuffer::insertLine( KInt index, const KChar * string ) {

	if( index < lineoffset || index >= linecount + lineoffset )
	    return -1;

	if( linecount >= buffersize ) {
	    if( extend_buffer( ) == -1 )
		return -1;
	}

	KInt	l = index - lineoffset;
	KChar	** p = buffer + linecount;
	for( KInt i = linecount; i > l; i ++ ) {
	    * p = * ( p - 1 );
	    p --;
	}
	* p = new KChar [ strlen( string ) + 1 ];
	strcpy( * p, string );

	linecount ++;

	return 0;

}

KInt	KonoeCommandBuffer::replaceLine( KInt index, const KChar * string ) {

	if( index < lineoffset || index >= linecount + lineoffset )
	    return -1;

	KInt	l = index - lineoffset;
	KChar	** p = buffer + l;

	delete [] * p;

	* p = new KChar [ strlen( string ) + 1 ];
	strcpy( * p, string );

	return 0;

}
