//=================================================================
/*

	KonoeCommandBufferOrderly.cc

	Version 0.0.1	24-May-2001	H. Sakamoto

*/
//=================================================================

#include	<ui/KonoeCommandBufferOrderly.hh>
#include	<ui/KonoeHelpCommand.hh>
#include	<ui/KonoeCommandInterpreter.hh>

const KChar	* KonoeCommandBufferOrderly::classname = "CommandBuffer";

KonoeCommandBufferOrderly::KonoeCommandBufferOrderly( const KChar * objname )
	: KonoeOrderly( N_COMMANDS, objname ) {

	setClassID( CLASS_ID );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;

// help command

	addCommand( new KonoeHelpCommand( 0, this ) );

// list command

	cd = new KonoeCommandDescriptor( "list", N_ARG_LIST );
	ad = new KonoeArgumentDescriptor( "from", INTEGER, OPTIONAL );
	ad -> setDescription( "Line number to start listing" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "to", INTEGER, OPTIONAL );
	ad -> setDescription( "Line number to end listing" );
	cd -> add( ad );
	cd -> setDescription( "List contents of the command buffer" );
	addCommand( new ListUnit( cd, this ) );

// delete command

	cd = new KonoeCommandDescriptor( "delele", N_ARG_DELETE );
	ad = new KonoeArgumentDescriptor( "from", INTEGER, NECESSARY );
	ad -> setDescription( "Line number from where lines to be deleted" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "to", INTEGER, OPTIONAL );
	ad -> setDescription( "Line number to where lines to be deleted" );
	cd -> add( ad );
	cd -> setDescription( "Delete lines from the procedure" );
	addCommand( new DeleteUnit( cd, this ) );

// insert commad

	cd = new KonoeCommandDescriptor( "insert", N_ARG_INSERT );
	ad = new KonoeArgumentDescriptor( "at", INTEGER, NECESSARY );
	ad -> setDescription( "Line number where a line is inserted" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "line", STRING, NECESSARY );
	ad -> setDescription( "A command line to be inserted" );
	cd -> add( ad );
	cd -> setDescription( "Insert a line into the command buffer" );
	addCommand( new InsertUnit( cd, this ) );

// replace command

	cd = new KonoeCommandDescriptor( "replace", N_ARG_REPLACE );
	ad = new KonoeArgumentDescriptor( "at", INTEGER, NECESSARY );
	ad -> setDescription( "Line number to be replaced" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "line", STRING, NECESSARY );
	ad -> setDescription( "A command line to be ovrewritten on" );
	cd -> add( ad );
	cd -> setDescription( "Replace a line in the buffer" );
	addCommand( new ReplaceUnit( cd, this ) );


}

KInt	KonoeCommandBufferOrderly::isOrderlyOf( void * address ) {

	return address == ( void * ) target;

}

KonoeOrderly	* KonoeCommandBufferOrderly::clone( const KChar * objname ) {

	return new KonoeCommandBufferOrderly( objname );

}

void	KonoeCommandBufferOrderly::setTarget( void * ptarget ) {

	target = ( KonoeCommandBuffer * ) ptarget;

}

//--------------------------------------------------------------------

KonoeCommandBufferOrderly::ListUnit::ListUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ListUnit" );

}

KInt	KonoeCommandBufferOrderly::ListUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );
	KInt	n_arg = cmd -> getArgumentCount( );
	KonoeCommandArgument	* arg;

	KInt	linefrom = 0, lineto = -1;

	if( n_arg > ARG_LIST_FROM ) {
	    arg = cmd -> getArgument( ARG_LIST_FROM );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
		return -1;
	    }
	    linefrom = arg -> toInteger( );
	}

	if( n_arg > ARG_LIST_TO ) {
	    arg = cmd -> getArgument( ARG_LIST_TO );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
		return -1;
	    }
	    lineto = arg -> toInteger( );
	}

	KonoeCommandBuffer	* buffer = ( KonoeCommandBuffer * )
					owner -> getTarget( );

	if( lineto == -1 ) {
	    lineto = buffer -> getLineCount( ) + buffer -> getLineOffset( );
	}
	if( linefrom < buffer -> getLineOffset( ) )
	    linefrom = buffer -> getLineOffset( );

	for( KInt i = linefrom; i < lineto; i ++ ) {
	    os << i << ": " << buffer -> getLine( i ) << endl;
	}

	return 0;

}

KonoeCommandBufferOrderly::DeleteUnit::DeleteUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "DeleteUnit" );

}

KInt	KonoeCommandBufferOrderly::DeleteUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	if( n_arg < ARG_DELETE_FROM + 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument * arg = cmd -> getArgument( ARG_DELETE_FROM );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	linefrom = arg -> toInteger( );

	KInt	lineto = 0;

	if( n_arg >= ARG_DELETE_TO + 1 ) {

	    arg = cmd -> getArgument( ARG_DELETE_TO );
	    if( ! arg -> isInteger( ) ) {
		printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
		return -1;
	    }

	    lineto = arg -> toInteger( );

	}

	if( lineto < linefrom )
	    lineto = linefrom;

	KonoeCommandBuffer	* buffer = ( KonoeCommandBuffer * )
		owner -> getTarget( );

	if( linefrom < buffer -> getLineOffset( ) ||
	    linefrom >= buffer -> getLineOffset( )
		+ buffer -> getLineCount( ) ) {

	    printErrorHeader( os );
	    os << "Line number " << linefrom << " not found." << endl;
	    return -1;
	}

	if( lineto < buffer -> getLineOffset( ) ||
	    lineto >= buffer -> getLineOffset( )
		+ buffer -> getLineCount( ) ) {

	    printErrorHeader( os );
	    os << "Line number" << lineto << " not found." << endl;
	    return -1;
	}

	for( KInt i = linefrom; i <= lineto; i ++ )
	    buffer -> deleteLine( i );

	return 0;

}

KonoeCommandBufferOrderly::InsertUnit::InsertUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "InsertUnit" );

}

KInt	KonoeCommandBufferOrderly::InsertUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	if( n_arg < ARG_INSERT_LINE + 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument * arg = cmd -> getArgument( ARG_INSERT_AT );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	line = arg -> toInteger( );

	arg = cmd -> getArgument( ARG_INSERT_AT );

	KonoeCommandBuffer	* buffer = ( KonoeCommandBuffer * )
		owner -> getTarget( );

	if( line < buffer -> getLineOffset( ) ||
	    line >= buffer -> getLineOffset( )
		+ buffer -> getLineCount( ) ) {

	    printErrorHeader( os );
	    os << "Line number " << line << " not found." << endl;
	    return -1;
	}

	buffer -> insertLine( line, arg -> getValue( ) );

	return 0;

}

KonoeCommandBufferOrderly::ReplaceUnit::ReplaceUnit( KonoeCommandDescriptor * cd,
	KonoeOrderly * ow ) : KonoeCommandProcessor( cd, ow ) {

	setClassName( "ReplaceUnit" );

}

KInt	KonoeCommandBufferOrderly::ReplaceUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	if( n_arg < ARG_INSERT_LINE + 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument * arg = cmd -> getArgument( ARG_INSERT_AT );

	if( ! arg -> isInteger( ) ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	KInt	line = arg -> toInteger( );

	arg = cmd -> getArgument( ARG_INSERT_AT );

	KonoeCommandBuffer	* buffer = ( KonoeCommandBuffer * )
		owner -> getTarget( );

	if( line < buffer -> getLineOffset( ) ||
	    line >= buffer -> getLineOffset( )
		+ buffer -> getLineCount( ) ) {

	    printErrorHeader( os );
	    os << "Line number " << line << " not found." << endl;
	    return -1;
	}

	buffer -> replaceLine( line, arg -> getValue( ) );

	return 0;

}
