//==================================================================
/*

	KonoeCommandInterpreter.cc

	Version 0.0.1	 3-Sep-1998	H. Sakamoto
	Version 0.4.0	 6-Sep-1998
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	07-Dec-2000	H. Sakamoto
	Version 0.6.2	17-May-2001	H. Sakamoto
	Version 0.6.3	28-May-2001	H. Sakamoto

*/
//==================================================================

#include	<ui/KonoeCommandInterpreter.hh>
#include	<ui/KonoeCommandBufferOrderly.hh>
#include	<fstream.h>
#include	<string.h>
#include	<unistd.h>

//extern "C"	void usleep( unsigned long );

KonoeCommandInterpreter::KonoeCommandInterpreter(
	KonoeCommander * pcommander ) {

	KonoeCommandBufferOrderly	* orderly;

	commander = pcommander;
	commander -> setInterpreter( this );
	commander -> employ( commandofficer = new KonoeCommandOfficer(
			COMMAND_OFFICER_SIZE, "commandofficer" ) );
	orderly = new KonoeCommandBufferOrderly( "history" );
	commandofficer -> add( orderly );
	orderly -> setOfficer( commandofficer );
	orderly -> setInterpreter( this );

	parser = new KonoeCommandParser( );
	variabletable = new KonoeCommandVariableTable(
		"command variable table", VARIABLE_TABLE_SIZE );
	resolverlist = new KonoeSymbolResolverList( RESOLVER_LIST_SIZE );
	resolverlist -> add( variabletable );
	expression = new KonoeExpression( resolverlist );
	historybuffer = new KonoeCommandBuffer( HISTORY_BUFFER_SIZE );
	history_flag = 1;
	orderly -> setTarget( historybuffer );

	proceduremanager = new KonoeProcedureManager( );

	KonoeCommandDescriptor	* cd;
	KonoeArgumentDescriptor	* ad;

// 'let' command

	cd = new KonoeCommandDescriptor( "let", N_ARG_LET );
	ad = new KonoeArgumentDescriptor( "variable",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription(
		"name of variable to be set" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "value",
		KonoeArgumentDescriptor::EXPRESSION,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription(
		"value to be set." );
	cd -> add( ad );
	cd -> setDescription(
		"Sets the value of a variable" );
	commander -> addCommand( new LetUnit( cd, commander, resolverlist ) );

// 'get' command

	cd = new KonoeCommandDescriptor( "get", N_ARG_GET );
	ad = new KonoeArgumentDescriptor( "variable",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::OPTIONAL );
	ad -> setDescription( "name of a variable" );
	cd -> add( ad );
	cd -> setDescription( "Show variable" );
	commander -> addCommand( new GetUnit( cd, commander, resolverlist ) );

// 'if' command

	ifcommand = new KonoeCommandDescriptor( "if", N_ARG_IF );
	ifcommand -> setIdentifier( CMD_IF );
	ad = new KonoeArgumentDescriptor( "expression",
		KonoeArgumentDescriptor::INTEGER,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription( "Integer expression to be tested" );
	ifcommand -> add( ad );
	ifcommand -> setDescription( "Conditional branching" );
	commander -> addCommand(
		new IfUnit( ifcommand, commander, this ) );

// 'else' command

	elsecommand = new KonoeCommandDescriptor( "else", N_ARG_ELSE );
	elsecommand -> setIdentifier( CMD_ELSE );
	elsecommand -> setDescription( "Flip the 'if' condition" );
	commander -> addCommand( new ElseUnit( elsecommand, commander, this ) );

// 'end' command

	endcommand = new KonoeCommandDescriptor( "end", N_ARG_END );
	endcommand -> setIdentifier( CMD_END );
	ad = new KonoeArgumentDescriptor( "block",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription( "Type of block. ex. 'if' or 'while'" );
	endcommand -> add( ad );
	endcommand -> setDescription(
		"End of conditional or repetition block" );
	commander -> addCommand( new EndUnit( endcommand, commander, this ) );

// 'while' command

	whilecommand = new KonoeCommandDescriptor( "while", N_ARG_WHILE );
	whilecommand -> setIdentifier( CMD_WHILE );
	ad = new KonoeArgumentDescriptor( "expression",
		KonoeArgumentDescriptor::INTEGER,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription( "Integer expression to be tested" );
	whilecommand -> add( ad );
	whilecommand -> setDescription(
		"Repeat execution while 'expression' is true." );
	commander -> addCommand(
		new WhileUnit( whilecommand, commander, this ) );

// 'set' command

	cd = new KonoeCommandDescriptor( "set", N_ARG_SET );
	ad = new KonoeArgumentDescriptor( "item",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::OPTIONAL );
	ad -> setDescription( "Item to be set. 'echo'" );
	cd -> add( ad );
	ad = new KonoeArgumentDescriptor( "value",
		KonoeArgumentDescriptor::INTEGER,
		KonoeArgumentDescriptor::OPTIONAL );
	ad -> setDescription( "Value to be set to 'item'" );
	cd -> add( ad );
	cd -> setDescription( "Set control parameters" );
	commander -> addCommand( new SetUnit( cd, commander, this ) );

// 'procedure' command

	cd = new KonoeCommandDescriptor( "procedure", N_ARG_PROCEDURE );
	ad = new KonoeArgumentDescriptor( "name",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription( "Name of the procedure" );
	cd -> add( ad );
	cd -> setDescription( "Start a procedure definition" );
	commander -> addCommand( new ProcedureUnit( cd, commander, this ) );

// 'call' command

	cd = new KonoeCommandDescriptor( "call", N_ARG_CALL );
	ad = new KonoeArgumentDescriptor( "name",
		KonoeArgumentDescriptor::STRING,
		KonoeArgumentDescriptor::NECESSARY );
	ad -> setDescription( "Name of the procedure" );
	cd -> add( ad );
	cd -> setDescription( "Execute a procedure" );
	commander -> addCommand( new CallUnit( cd, commander, this ) );

// flow control variables

	nest = 0;
	if_level = -1;
	saveline_flag = 0;
	whilebuffer = 0;
	while_level = -1;
	source_select = SOURCE_STANDARD_INPUT;
	while_state[ 0 ] = WHILE_FIRST_FALSE;
	procedure_definition = 0;
	call_level = -1;

}

KonoeCommandInterpreter::~KonoeCommandInterpreter( ) {

	if( filename ) delete [] filename;
	delete parser;

}

KInt	KonoeCommandInterpreter::go(
		istream & ifs, ostream & ofs, KChar * prompt ) {

	KInt	st, id;
	KInt	nest_level;
	KonoeCommand	* command;
	KonoeOrderly	* orderly;

	nest_level = nest ++;	// include level

	while( 1 ) {

	  if( source_select == SOURCE_STANDARD_INPUT ) {	// from stdin.

	    if( prompt )
	      ofs << prompt;

	    ifs.getline( commandline, SIZE_COMMANDLINE );

	    if( ifs.eof( ) ) {
	      returncode = 0;
	      nest --;
	      break;
	    }

	    if( ( ! prompt ) & echo_flag )
	      ofs << commandline << endl;

	    if( saveline_flag ) {
	      whilebuffer -> putLine( commandline );
	      saveline_count ++;
	      execline_count ++;
	    }

	    currentline = commandline;

	  }

	  else if ( source_select == SOURCE_PROCEDURE ) {

	    currentline = calllist[ call_level ] -> getLine(
		call_linenumber[ call_level ] ++ );

	    if( echo_flag )
		ofs << currentline << endl;

	    if( saveline_flag ) {
	      whilebuffer -> putLine( currentline );
	      saveline_count ++;
	      execline_count ++;
	    }

	  }

	  else {	// from command buffer

	    currentline = whilebuffer -> getLine( execline_count ++ );
	    if( echo_flag )
		ofs << currentline << endl;

	  }

	  filename = 0;

	  if( history_flag )
	    historybuffer -> pushLine( currentline );

	  if( procedure_definition )
	    definingprocedure -> putLine( currentline );

	  st = parser -> parse( currentline );

	  if( st < 0 ) {	// parse error

	    ofs << "Error parsing " << currentline << endl;
	    returncode = st;

	  }
	  else if( st == KonoeCommandParser::COMMAND_PARSED ) {

	  KInt	flag = 1;

	    command = parser -> getCommand( );
	    command -> setOutputStream( ofs );
	    command -> setExpression( expression );

	    if( while_level >= 0 &&
		while_state[ while_level ] == WHILE_FIRST_FALSE )
		flag = 0;
	    if( if_level >= 0 && ! if_result )
		flag = 0;
	    if( procedure_definition )
		flag = 0;

	    returncode = commander -> dispatch( command, ofs, flag );

	  }
	  else {	// empty line.

	    returncode = 0;

	  }

	  if( nest_level >= nest )
	    break;
	  if( filename ) {	// .include executed.
	    ifstream newifs( filename );
	    returncode = go( newifs, ofs );
	  }
	}
	return returncode;
}

KInt	KonoeCommandInterpreter::doExit( KInt status ) {

	nest --;
	return status;

}

KInt	KonoeCommandInterpreter::doInclude( const KChar * pfilename ) {

	filename = new KChar[ strlen(pfilename)+1 ];
	strcpy(filename,pfilename);
	return 0;

}

KInt	KonoeCommandInterpreter::doIf( KonoeCommand * cmd ) {

	ostream	& os = cmd -> getOutputStream( );

	if( procedure_definition )
	    return 0;

	if( if_level >= IF_NEST_LEVEL - 1 ) {
	    os << "KonoeCommandInterpreter::doIF. Too deep 'if' nesting."
		<< endl;
	    return -1;
	}

	KonoeCommandArgument	* arg =
		cmd -> getArgument( ARG_IF_EXPRESSION );

	if( ! arg -> isInteger( ) )
	    return KonoeCommandProcessor::E_ARGUMENT_NOT_INTEGER;

	KInt	ivalue = arg -> toInteger( );

	if_level ++;
	if_state[ if_level ] = ivalue ? 1 : 0;
	if_result = 1;
	for( KInt i = 0; i <= if_level; i ++ )
	    if_result *= if_state[ i ];
	return 0;

}

KInt	KonoeCommandInterpreter::doElse( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( procedure_definition )
	    return 0;

	if( if_level < 0 ) {
	    os << "KonoeCommandInterpreter::doElse. Not in 'if' clause."
		<< endl;
	    return -1;
	}
	if_state[ if_level ] = if_state[ if_level ] ? 0 : 1;
	if_result = 1;
	for( KInt i = 0; i <= if_level; i ++ )
	    if_result *= if_state[ i ];
	return 0;
}

KInt	KonoeCommandInterpreter::doEndIf( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( procedure_definition )
	    return 0;

	if( if_level < 0 ) {
	    os << "KonoeCommandInterpreter::doElse. Not in 'if' clause."
		<< endl;
	    return -1;
	}
	if_level --;
	if_result = 1;
	for( KInt i = 0; i <= if_level; i ++ )
	    if_result *= if_state[ i ];
	return 0;

}

KInt	KonoeCommandInterpreter::doWhile( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( procedure_definition )
	    return 0;

	if( while_level >= WHILE_NEST_LEVEL -1 ) {
	    os << "KonoeCommandInterpreter::doWhile. "
			"Too deep 'while' nesting." << endl;
	    return -1;
	}

	KInt	ivalue;

	KonoeCommandArgument	* arg =
		cmd -> getArgument( ARG_WHILE_EXPRESSION );

	if( ! arg -> isInteger( ) )
	    return KonoeCommandProcessor::E_ARGUMENT_NOT_INTEGER;

	ivalue = arg -> toInteger( );

	if( while_level == -1 && while_state[ 0 ] == WHILE_FIRST_FALSE ) {
	    whilebuffer = new KonoeCommandBuffer( );
	    whilebuffer -> putLine( currentline );
	    saveline_flag = 1;
	    saveline_count = 0;
	    execline_count = 1;
	    for( KInt i = 1; i < WHILE_NEST_LEVEL; i ++ )
		    while_state[ i ] = WHILE_FIRST_FALSE;
	}

	while_level ++;
	if( while_state[ while_level ] == WHILE_FIRST_FALSE ) {
	    if( while_level == 0 )
		while_begin[ 0 ] = 0;
	    else
		while_begin[ while_level ] = execline_count - 1;
	}

	if( ivalue ) {	// while TRUE.

	    if( while_state[ while_level ] == WHILE_FIRST_FALSE )
		while_state[ while_level ] = WHILE_FIRST_TRUE;
	    else if( while_state[ while_level ] == WHILE_REPEAT_FALSE ) {
		while_state[ while_level ] = WHILE_REPEAT_TRUE;
	    }
	}
	else {		// while FALSE

	    if( while_state[ while_level ] == WHILE_FIRST_FALSE ) {
	    }
	    else {	// repeating. not the first.
		while_state[ while_level ] = WHILE_FIRST_FALSE;
		while_level --;
		if( while_level == -1 ) {	// end of all
		    saveline_flag = 0;
		    delete whilebuffer;
		    if( call_level < 0 )
			source_select = SOURCE_STANDARD_INPUT;
		    else
			source_select = SOURCE_PROCEDURE;
		}
		else {	//
		    if( while_end[ while_level + 1 ] == saveline_count ) {
			saveline_flag = 1;
			if( call_level < 0 )
			    source_select = SOURCE_STANDARD_INPUT;
			else
			    source_select = SOURCE_PROCEDURE;
		    }
		    else {
			execline_count = while_end[ while_level + 1 ] + 1;
		    }
		}
	    }
	}
	return 0;

}

KInt	KonoeCommandInterpreter::doEndWhile( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( procedure_definition )
	    return 0;

	if( while_level < 0 ) {

	    os << "KonoeCommandInterpreter::doEndWhile."
			" Not in a 'while' clause." << endl;
	    return -1;

	}

	if( whilebuffer == 0 ) {

	    os << "KonoeCommandInterpreter::doEndWhile."
		" Command buffer not available" << endl;
	    return -1;

	}

	if( while_state[ while_level ] == WHILE_FIRST_FALSE ||
	    while_state[ while_level ] == WHILE_FIRST_TRUE ) {
	    while_end[ while_level ] = execline_count - 1;
	}

	saveline_flag = 0;
	execline_count = while_begin[ while_level ];
	source_select = SOURCE_WHILE_BUFFER;
	while_state[ while_level ] = WHILE_REPEAT_FALSE;
	while_level --;

	return 0;

}

KInt	KonoeCommandInterpreter::doProcedure( const KChar * procname ) {

	KonoeCommandProcedure	* procedure;

	procedure = proceduremanager -> findProcedure( procname );

	if( procedure )
	    return E_PROCEDURE_EXISTS;	// already exists.

	procedure = new KonoeCommandProcedure( procname );

	if( proceduremanager -> addProcedure( procedure ) < 0 )
	    return E_PROCEDURE_TABLE_FULL;

	if( procedure_definition )
	    return E_NESTED_PROCEDURE;

	KonoeCommandBufferOrderly	* orderly;

	orderly = new KonoeCommandBufferOrderly( procname );
	orderly -> setOfficer( commandofficer );
	commandofficer -> add( orderly );
	orderly -> setTarget( procedure );

	procedure_definition = 1;
	definingprocedure = procedure;

	return 0;

}

KInt	KonoeCommandInterpreter::doEndProcedure( ) {


	if( procedure_definition == 0 && call_level < 0 )
	    return E_NOT_IN_PROCEDURE;

	if( procedure_definition != 0 ) {	// definition phase.

	    procedure_definition = 0;

	}
	else {	// execution phase.

	    call_level --;
	    if( call_level < 0 ) {
		source_select = SOURCE_STANDARD_INPUT;
	    }
	}

	return 0;

}

KInt	KonoeCommandInterpreter::doCall( const KChar * procname ) {

	KonoeCommandProcedure	* proc;

	proc = proceduremanager -> findProcedure( procname );
	if( proc == 0 )
	    return E_PROCEDURE_NOT_FOUND;
	if( call_level == CALL_NEST_LEVEL )
	    return E_TOO_DEEP_CALL_NEST;
	if( call_level < 0 )
	    source_select = SOURCE_PROCEDURE;
	call_level ++;
	call_linenumber[ call_level ] = 0;
	calllist[ call_level ] = proc;

	return 0;

}

void	KonoeCommandInterpreter::setEcho( KInt flag ) {

	echo_flag = flag;

}

void	KonoeCommandInterpreter::setHistory( KInt flag ) {

	history_flag = flag;

}

//-----------------------------------------------------------------

//	LetUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::LetUnit::LetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * powner, KonoeSymbolResolverList * resolver )
	: KonoeCommandProcessor( cd, powner ) {

	resolverlist = resolver;
	expression = new KonoeExpression( resolverlist );
	setClassName( "LetUnit" );

}

KonoeCommandInterpreter::LetUnit::~LetUnit( ) {

	delete expression;

}

KInt	KonoeCommandInterpreter::LetUnit::execute( KonoeCommand * cmd ) {

	KInt	narg, st;
	ostream &	os = cmd -> getOutputStream( );
	KonoeCommandArgument	* avariable, * avalue;
	KChar	* name;
	KonoeSymbolResolver	* resolver;
	KonoeSymbol		* symbol;

	narg = cmd -> getArgumentCount( );
	if( narg < 2 ) {
	    os << "KonoeCommandInterpreter::LetUnit::execute."
			"Argument missing" <<endl;
	    return -1;
	}
	else if( narg > 2 ) {
	    os << "KonoeCommandInterpreter::LetUnit::execute."
			"Too much arguments" << endl;
	    return -1;
	}

	avariable = cmd -> getArgument( ARG_LET_VARIABLE );
	avalue = cmd -> getArgument( ARG_LET_VALUE );

	name = ( KChar * ) avariable -> getValue( );

	resolverlist -> reset( );
	symbol = 0;

	while( resolver = resolverlist -> next( ) ) {

	    if( symbol = resolver -> findSymbol( name ) )
		break;
	}

	if( symbol ) {	// known variable

	}
	else {		// variable to be created. -> to where???
			// temporally, the first resolver is used.

	    symbol = resolverlist -> get( 0 ) ->addSymbol( name );
	}

	if( avalue -> isSymbol( ) ) {
	  KonoeSymbol	* source = avalue -> toSymbol( );
	  if( ! source ) {
	    printErrorHeader( os );
	    os << "Unknown symbol = " << avalue -> getValue( ) << endl;
	    return -1;
	  }
	  if( source -> isInteger( ) )
	    st = symbol -> setIValue( source -> getIValue( ) );
	  else if( source -> isString( ) )
	    st = symbol -> setCValue( source -> getCValue( ) );
	  else if( source -> isFloating( ) )
	    st = symbol -> setFValue( source -> getFValue( ) );
	}
	else {
	  st = expression -> evaluate( ( KChar * ) avalue -> getValue( ) );
	  if( st < 0 ) {
	    printErrorHeader( os );
	    os << "Invalid expression " << avalue -> getValue( ) <<endl;
	    return st;
	  }
	  else {
	    switch( expression -> getType( ) ) {
	    case KonoeExpression::INTEGER :
		st = symbol -> setIValue( expression -> getNumericValue( ) );
		break;
	    case KonoeExpression::STRING :
		st = symbol -> setCValue( ( KChar * ) avalue -> getValue( ) );
		break;
	    default :
		printErrorHeader( os );
		os << " Unsupported type. may be floating" << endl;
		return -1;

	    }
	  }
	}


	return st;

}

//-----------------------------------------------------------------

//	GetUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::GetUnit::GetUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeSymbolResolverList * rs )
		: KonoeCommandProcessor( cd, ow ) {

	resolverlist = rs;

}

KInt	KonoeCommandInterpreter::GetUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) == 0 ) {

	    KonoeSymbolResolver	* resolver;
	    KonoeSymbol	* symbol;

	    KInt	i, n;

	    resolverlist -> reset( );
	    while( resolver = resolverlist -> next( ) ) {
		os << "Resolver = " << resolver -> getResolverName( ) << endl;
		n = resolver -> getSymbolCount( );
		for( i = 0; i < n; i ++ ) {
		    symbol = resolver -> getSymbol( i );
		    symbol -> showName( os );
		    os << " = ";
		    symbol -> showValue( os );
		    os << endl;
		}
	    }
	    
	}
	else {

	    KonoeCommandArgument	* arg;
	    KonoeSymbol	* symbol;
	    KInt	n, i;

	    n = cmd -> getArgumentCount( );
	    for( i = 0; i < n; i ++ ) {

		arg = cmd -> getArgument( i );
		symbol = resolverlist -> findSymbol( arg -> getValue( ) );
		if( symbol ) {
		    symbol -> showName( os );
		    os << " = ";
		    symbol -> showValue( os );
		    os << endl;
		}
		else {
		    os << "KonoeCommandInterpreter::GetUnit::execute."
			<< " Symbol = " << arg -> getValue( )
			<< ": not found." << endl;
		}
	    }

	}

	return 0;
}

//-----------------------------------------------------------------

//	IfUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::IfUnit::IfUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow,
		KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setAlways( 1 );
	setClassName( "IfUnit" );

}

KInt	KonoeCommandInterpreter::IfUnit::execute( KonoeCommand * cmd ) {

	KonoeCommandArgument	* arg;
	KInt	n_arg, st;
	ostream & os = cmd -> getOutputStream( );

	n_arg = cmd -> getArgumentCount( );
	if( n_arg < 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	st = interpreter -> doIf( cmd );

	if( st < 0 ) {
	    printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	    return -1;
	}

	return st;

}

//-----------------------------------------------------------------

//	ElseUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::ElseUnit::ElseUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setAlways( 1 );
	setClassName( "ElseUnit" );

}

KInt	KonoeCommandInterpreter::ElseUnit::execute( KonoeCommand * cmd ) {

	return interpreter -> doElse( cmd );

}

//-----------------------------------------------------------------

//	EndUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::EndUnit::EndUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setClassName( "EndUnit" );
	setAlways( 1 );

}

KInt	KonoeCommandInterpreter::EndUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < 1 ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}
	const KChar	* blocktype = cmd -> getArgument(
			KonoeCommandInterpreter::ARG_END_BLOCKTYPE )
			-> getValue( );
	if( strcmp( blocktype, "if" ) == 0 ) {
	    return interpreter -> doEndIf( cmd );
	}
	else if( strcmp( blocktype, "while" ) == 0 ) {
	    return interpreter -> doEndWhile( cmd );
	}
	else if( strcmp( blocktype, "procedure" ) == 0 ) {
	    KInt st = interpreter -> doEndProcedure( );
	    if( st < 0 ) {
		printErrorHeader( os );
		switch( st ) {
		case E_NOT_IN_PROCEDURE :
		    os << "Not in a procedure definition." << endl;
		    break;
		default :
		    os << "Error code = " << st << " detected." << endl;
		    break;
		}
	    }
	    return st;
	}
	else {
	    os << "KonoeCommandInterpreter::EndUnit::execute."
			"unknown block type :" << blocktype << endl;
	    return -1;
	}

}

//-----------------------------------------------------------------

//	WhileUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::WhileUnit::WhileUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setAlways( 1 );
	setClassName( "WhileUnit" );

}

KInt	KonoeCommandInterpreter::WhileUnit::execute( KonoeCommand * cmd ) {

	KonoeCommandArgument	* arg;
	KInt	n_arg, st;
	ostream & os = cmd -> getOutputStream( );

	n_arg = cmd -> getArgumentCount( );
	if( n_arg < N_ARG_WHILE ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	st = interpreter -> doWhile( cmd );

	if( st < 0 ) {
	    printErrorHeader( os );
	    switch( st ) {

	    default :
		os << "Error code = " << st << " detected." << endl;
		break;
	    }

	}
	return st;	

}



//-----------------------------------------------------------------

//	SetUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::SetUnit::SetUnit(  KonoeCommandDescriptor * cd,
		KonoeOrderly * powner, KonoeCommandInterpreter * ip )
	: KonoeCommandProcessor( cd, powner ) {

	interpreter = ip;

}

KInt	KonoeCommandInterpreter::SetUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	if( n_arg == 0 ) {
	    os << "Echo is ";
	    if( interpreter -> getEchoFlag( ) )
		os << "on";
	    else
		os << "off";
	    os << " History is ";
	    if( interpreter -> getHistoryFlag( ) )
		os << "on";
	    else
		os << "off";
	    os << endl;
	    return 0;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument(
		KonoeCommandInterpreter::ARG_SET_ITEM );

	if( strcmp( arg -> getValue( ), "echo" ) == 0 ) {
	    if( n_arg == 1 ) {
		interpreter -> setEcho( 1 );
	    }
	    else {
		arg = cmd -> getArgument(
			KonoeCommandInterpreter::ARG_SET_VALUE );
		if( strcmp( arg -> getValue( ), "on" ) == 0 )
		    interpreter -> setEcho( 1 );
		else if( strcmp( arg -> getValue( ), "off" ) == 0 )
		    interpreter -> setEcho( 0 );
		else {
		    os << "KonoeCommandInterpreter::SetUnit::execute."
			" Unknown switch = " << arg -> getValue( )
			<< " for echo." << endl;
		    return -1;
		}
	    }
	}
	else if( strcmp( arg -> getValue( ), "noecho" ) == 0 ) {
	    interpreter -> setEcho( 0 );
	}
	else if( strcmp( arg -> getValue( ), "history" ) == 0 ) {
	    if( n_arg == 1 ) {
		interpreter -> setHistory( 1 );
	    }
	    else {
		arg = cmd -> getArgument(
			KonoeCommandInterpreter::ARG_SET_VALUE );
		if( strcmp( arg -> getValue( ), "on" ) == 0 )
		    interpreter -> setHistory( 1 );
		else if( strcmp( arg -> getValue( ), "off" ) == 0 )
		    interpreter -> setHistory( 0 );
		else {
		    os << "KonoeCommandInterpreter::SetUnit::execute."
			" Unknown switch = " << arg -> getValue( )
			<< " for history." << endl;
		    return -1;
		}
	    }
	}
	else if( strcmp( arg -> getValue( ), "nohistory" ) == 0 ) {
	    interpreter -> setHistory( 0 );
	}
	else {
	    os << "KonoeCommandInterpreter::SetUnit::execute."
		"Unknown item for set = " << arg -> getValue( ) << endl;
	    return -1;
	}
	return 0;
}

//-----------------------------------------------------------------

//	ProcedureUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::ProcedureUnit::ProcedureUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow,
	KonoeCommandInterpreter * ip )
	: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setAlways( 1 );
	setClassName( "ProcedureUnit" );

}

KInt	KonoeCommandInterpreter::ProcedureUnit::execute( KonoeCommand * cmd ) {

	ostream	& os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < N_ARG_PROCEDURE ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument
		* arg = cmd -> getArgument( ARG_PROCEDURE_NAME );

	if( ! arg ->isAlphabetic( ) ) {
	    printError( os, E_ARGUMENT_NOT_ALPHA );
	    return -1;
	}

	KInt	st = interpreter -> doProcedure( arg -> getValue( ) );

	if( st < 0 ) {
	    printErrorHeader( os );
	    switch( st ) {
	    case E_PROCEDURE_EXISTS :
		os << "Procedure already exists = "
		<< arg -> getValue( ) << endl;
		break;
	    case E_PROCEDURE_TABLE_FULL :
		os << "Procedure table full" << endl;
		break;
	    case E_NESTED_PROCEDURE :
		os << "Nested procedure not allowed" << endl;
		break;
	    default :
		os << "Error code = " << st << " returned" << endl;
		break;
	    }
	    return st;
	}

	return 0;

}

//-----------------------------------------------------------------

//	CallUnit

//-----------------------------------------------------------------

KonoeCommandInterpreter::CallUnit::CallUnit(
	KonoeCommandDescriptor * cd, KonoeOrderly * ow,
	KonoeCommandInterpreter * ip )
	: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setClassName( "CallUnit" );

}

KInt	KonoeCommandInterpreter::CallUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( cmd -> getArgumentCount( ) < N_ARG_CALL ) {
	    printError( os, E_ARGUMENT_MISSING );
	    return -1;
	}

	KonoeCommandArgument	* arg = cmd -> getArgument( ARG_CALL_NAME );

	if( ! arg -> isAlphabetic( ) ) {
	    printError( os, E_ARGUMENT_NOT_ALPHA, arg -> getValue( ) );
	    return -1;
	}

	KInt	st = interpreter -> doCall( arg -> getValue( ) );

	if( st < 0 ) {
	    printErrorHeader( os );
	    switch( st ) {
	    case E_PROCEDURE_NOT_FOUND :
		os << "Procedure not found = " << arg -> getValue( ) << endl;
		break;
	    case E_TOO_DEEP_CALL_NEST :
		os << "Too deep call nest."<<endl;
		break;
	    default :
		os << "Error code = " << st << " detected." << endl;
		break;
	    }
	}

	return st;

}

