//==================================================================
/*

	KonoeCommandInterpreter.hh

	Version 0.0.1	 3-Sep-1998	H. Sakamoto
	Version 0.4.0	 6-Sep-1998
	Version 0.5.0	28-Sep-1999 T. Hosoi
	Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	07-Dec-2000	H. Sakamoto
	Version 0.6.2	17-May-2001	H. Sakamoto
	Version 0.6.3	28-May-2001	H. Sakamoto

*/
//==================================================================

#ifndef	__KONOECOMMANDINTERPRETER_HH
#define	__KONOECOMMANDINTERPRETER_HH

#include	<ui/KonoeCommander.hh>
#include	<ui/KonoeCommandParser.hh>
#include	<ui/KonoeSymbolResolverList.hh>
#include	<ui/KonoeCommandVariableTable.hh>
#include	<ui/KonoeCommandProcessor.hh>
#include	<ui/KonoeExpression.hh>
#include	<ui/KonoeCommandBuffer.hh>
#include	<ui/KonoeProcedureManager.hh>
#include	<ui/KonoeCommandOfficer.hh>

class	KonoeCommandInterpreter	{

public :
	enum	{	CMD_LET = KonoeCommander::N_COMMANDS,
			CMD_GET,
			CMD_IF,
			CMD_ELSE,
			CMD_END,
			CMD_WHILE,
			CMD_SET,
			CMD_PROCEDURE,
			CMD_CALL,
			N_COMMANDS		};
	enum	{	ARG_LET_VARIABLE,
			ARG_LET_VALUE,
			N_ARG_LET		};
	enum	{	ARG_GET_VARIABLE,
			N_ARG_GET		};
	enum	{	ARG_IF_EXPRESSION,
			N_ARG_IF		};
	enum	{	N_ARG_ELSE		};
	enum	{	ARG_END_BLOCKTYPE,
			N_ARG_END		};
	enum	{	ARG_WHILE_EXPRESSION,
			N_ARG_WHILE		};
	enum	{	ARG_SET_ITEM,
			ARG_SET_VALUE,
			N_ARG_SET		};
	enum	{	ARG_PROCEDURE_NAME,
			N_ARG_PROCEDURE		};
	enum	{	ARG_CALL_NAME,
			N_ARG_CALL		};

protected :

	class	LetUnit		: public KonoeCommandProcessor	{

	protected :
		KonoeSymbolResolverList	* resolverlist;
		KonoeExpression	* expression;

	public :
		LetUnit( KonoeCommandDescriptor * cd, KonoeOrderly * owner,
			KonoeSymbolResolverList * resolver );
		~LetUnit( );
		KInt	execute( KonoeCommand * cmd );

	};

	class	GetUnit		: public KonoeCommandProcessor {

	protected :
		KonoeSymbolResolverList	* resolverlist;

	public :
		GetUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeSymbolResolverList * reslst );
		KInt	execute( KonoeCommand * cmd );
	};

	class	IfUnit		: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		IfUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );
	};

	class	ElseUnit	: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		ElseUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );

	};

	class	EndUnit		: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		EndUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );
	};

	class	WhileUnit	: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		WhileUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );
	};

	class	SetUnit		: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		SetUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );

	};

	class	ProcedureUnit	: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter * interpreter;

	public :
		ProcedureUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );

	};

	class	CallUnit	: public KonoeCommandProcessor {

	protected :
		KonoeCommandInterpreter	* interpreter;

	public :
		CallUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );

	};

public :
	enum	{	SIZE_COMMANDLINE = 512,
			RESOLVER_LIST_SIZE = 2,
			VARIABLE_TABLE_SIZE = 512,
			COMMAND_OFFICER_SIZE = 64,
			IF_NEST_LEVEL = 64,
			WHILE_NEST_LEVEL = 64,
			CALL_NEST_LEVEL = 64		};

	enum	{	WHILE_FIRST_FALSE,
			WHILE_FIRST_TRUE,
			WHILE_REPEAT_FALSE,
			WHILE_REPEAT_TRUE		};

	enum	{	SOURCE_STANDARD_INPUT,
			SOURCE_PROCEDURE,
			SOURCE_WHILE_BUFFER		};

	enum	{	HISTORY_BUFFER_SIZE = 128	};

	enum	{	E_PROCEDURE_EXISTS	= -1,
			E_PROCEDURE_TABLE_FULL	= -2,
			E_NESTED_PROCEDURE	= -3,
			E_NOT_IN_PROCEDURE	= -4,
			E_PROCEDURE_NOT_FOUND	= -5,
			E_TOO_DEEP_CALL_NEST	= -6	};


protected :
	KonoeCommander	* commander;
	KonoeCommandParser	* parser;
	KonoeSymbolResolverList	* resolverlist;
	KonoeCommandVariableTable	* variabletable;
	KonoeExpression	* expression;

	KonoeCommandDescriptor	* ifcommand;
	KonoeCommandDescriptor	* elsecommand;
	KonoeCommandDescriptor	* endcommand;
	KonoeCommandDescriptor	* whilecommand;

	KChar	commandline[ SIZE_COMMANDLINE ];
	KChar	* currentline;
	KInt	echo_flag;
	KInt	returncode;
	KInt	nest;
	KChar	* filename;

	KInt	if_state[ IF_NEST_LEVEL ];
	KInt	if_level;
	KInt	if_result;

	KonoeCommandBuffer	* whilebuffer;
	KInt	saveline_flag;
	KInt	saveline_count;
	KInt	while_begin[ WHILE_NEST_LEVEL ];
	KInt	while_end[ WHILE_NEST_LEVEL ];
	KInt	while_level;
	KInt	source_select;
	KInt	execline_count;
	KInt	while_state[ WHILE_NEST_LEVEL ];

	KonoeCommandBuffer	* historybuffer;
	KInt	history_flag;
	KonoeCommandOfficer	* commandofficer;

	KonoeProcedureManager	* proceduremanager;
	KonoeCommandProcedure	* definingprocedure;
	KInt	procedure_definition;

	KonoeCommandProcedure	* calllist[ CALL_NEST_LEVEL ];
	KInt	call_linenumber[ CALL_NEST_LEVEL ];
	KInt	call_level;

public :
	KonoeCommandInterpreter( KonoeCommander * commander );
	~KonoeCommandInterpreter( );
	KInt	go( istream & ifs, ostream & ofs, KChar * prompt = 0 );
	virtual KInt	doExit( KInt status );
	virtual KInt	doInclude( const KChar * filename );
	virtual KInt	doIf( KonoeCommand * cmd );
	virtual KInt	doElse( KonoeCommand * cmd );
	virtual KInt	doEndIf( KonoeCommand * cmd );
	virtual KInt	doWhile( KonoeCommand * cmd );
	virtual KInt	doEndWhile( KonoeCommand * cmd );
	virtual KInt	doProcedure( const KChar * procname );
	virtual KInt	doEndProcedure( );
	virtual KInt	doCall( const KChar * procname );

	void	setEcho( KInt flag );
	KInt	getEchoFlag( )	{	return echo_flag;	}
	void	setHistory( KInt flag );
	KInt	getHistoryFlag( )	{	return history_flag;	}
	KonoeExpression	* getExpression( )
		{ 	return expression;	}
	KonoeSymbolResolverList	* getResolverList( )
		{	return resolverlist;	}

};

#endif
