//====================================================================
/*

	KonoeCommandParser.cc

	Version 0.0.1	 3-Jul-1998	H. Sakamoto
	Version 0.4.0	 7-Sep-1998
	Version 0.5.0   28-Sep-1999	T. Hosoi
	Version 0.5.1	26-Jan-2000	H. Sakamoto
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	25-Nov-2000	H. Sakamoto

*/
//====================================================================

#include	<ui/KonoeCommandParser.hh>
#include	<string.h>

KonoeCommandParser::KonoeCommandParser( ) {

	verb = 0;
	command = 0;
}

KChar	* KonoeCommandParser::getSubstring( KChar * strfrom,
		KChar * strto ) {

	KInt	l;
	KChar	* p;

	l = strto - strfrom;
	p = new KChar[ l + 1 ];
	strncpy( p, strfrom, l );
	* ( p + l ) = '\0';
	return p;
	
}

KChar	* KonoeCommandParser::copyString( KChar * string ) {

	KChar	* p;

	p = new KChar[ strlen( string ) + 1 ];
	strcpy( p, string );
	return p;

}

void	KonoeCommandParser::clear( ) {

	delete [] verb;
	verb = 0;
	delete command;
	command = 0;

}

KInt	KonoeCommandParser::analyzeVerb( ) {

	KChar	* pdot, * pat, * pcolon, * r;

	pdot = strrchr( verb, '.' );	// last dot.
	pat = strchr( verb, '@' );	// first @
	pcolon = strchr( verb, ':' );	// first :

	if( pcolon ) {	// contains :
	  if( pat ) {	// contains @hostname:
	    if( pat < pcolon ) {	// @ must be ahead of :
	      r = getSubstring( verb, pat );
	      command -> setProcessName( r );
	      delete [] r;
	      r = getSubstring( pat + 1, pcolon );
	      command -> setHostName( r );
	      delete [] r;
	    }
	    else
	      return -1;
	  }
	  else {	//processname:objectname.method
	    r = getSubstring( verb, pcolon );
	    command -> setProcessName( r );
	    delete [] r;
	  }
	  if( pdot ) {
	    r = getSubstring( pcolon + 1, pdot );
	    command -> setObjectName( r );
	    delete [] r;
	    command -> setMethodName( pdot + 1 );
	  }
	  else {
	    command -> setMethodName( pcolon + 1 );
	  }
	}
	else	{
	  if( pdot ) {	// objectname.method
	    r = getSubstring( verb, pdot );
	    command -> setObjectName( r );
	    delete [] r;
	    command -> setMethodName( pdot + 1 );
	  }
	  else {	// method only.
	    command -> setMethodName( verb );
	  }
	}
	return 0;
}

KInt	KonoeCommandParser::add_argument( KChar * strfrom, KChar * strto ) {

	KChar	* r;

	if( argumentcount >= KonoeCommand::MAX_ARGUMENTS )
	  return -1;
	r = getSubstring( strfrom, strto );
	command -> add( new KonoeCommandArgument( argumentcount, r ) );
	delete [] r;
	argumentcount ++;
	st = END_ARGUMENT;
	return 0;

}

KInt	KonoeCommandParser::parse( KChar * pcommandline ) {

	commandline = pcommandline;
	enum	{	isTrue = 1,	isFalse = 0	};

	clear( );
	command = new KonoeCommand( );

	KChar	* p, * q, * r;
	KInt	i, l, eol, parseposition, bracenest;

	p = commandline;	// parsing point
	parseposition = 0;
	st = START;
	eol = isFalse;
	q = p;			// talken header
	argumentcount = 0;

	while( * p ) {
	  switch( * p ) {
	  case ' ' :
	  case '\t' :	// white spaces.
	    switch( st ) {
	    case START :
	      break;
	    case VERB :
	      verb = getSubstring( q, p );
	      st = END_VERB;
	      break;
	    case ARGUMENT :
	      if( add_argument( q, p ) < 0 )
		return TOO_MANY_ARGUMENTS;
	      break;
	    }
	    break;
	  case '"' :	// double quotation.
	    switch( st ) {
	    case QUOTEDSTRING :	// end of quotation
	      if( add_argument( q + 1, p ) < 0 )
		return TOO_MANY_ARGUMENTS;
	      st = END_ARGUMENT;
	      break;
	    case END_VERB :
	    case END_ARGUMENT :
	      st = QUOTEDSTRING;
	      q = p;
	      break;
	    default :
	      return INVALID_SYNTAX;
	    }
	    break;
	  case '(' :	// beginning of a braced argument
	    switch( st ) {
	    case START:
		return INVALID_SYNTAX;
	    case VERB:
		verb = getSubstring( q, p );
		st = IN_BRACE;
		bracenest = 1;
		q = p;
		break;
	    case ARGUMENT :	// argument(...) looks like a function call
		if( add_argument( q, p ) < 0 )
		    return TOO_MANY_ARGUMENTS;
		st = IN_BRACE;
		bracenest = 1;
		q = p;
		break;
	    case END_VERB :
	    case END_ARGUMENT :
		st = IN_BRACE;
		bracenest = 1;
		q = p;
		break;
	    case IN_BRACE :
		bracenest ++;
		break;
	    }
	    break;
	  case ')' :
	    switch( st ) {
	    case IN_BRACE :
		bracenest --;
		if( bracenest == 0 ) {
		    if( add_argument( q, p + 1 ) < 0 )
			return TOO_MANY_ARGUMENTS;
		    st = END_ARGUMENT;
		}
		break;
	    case QUOTEDSTRING :
		break;
	    default :
		return INVALID_SYNTAX;
	    }
	    break;
	  case '\n' :	// newline.
	    eol = isTrue;
	    break;
	  case '#' :	// comment introducer
	    switch( st ) {
	    case START :
	    case END_VERB :
	    case END_ARGUMENT :
	      eol = isTrue;
	      break;
	    case QUOTEDSTRING :
	      break;
	    default :
	      return INVALID_SYNTAX;
	    }
	    break;
	  default :	// other printable characters
	    switch( st ) {
	    case START :
	      q = p;	// beginning of verb
	      st = VERB;
	      break;
	    case END_ARGUMENT :
	    case END_VERB :
	      q = p;	// head of argument
	      st = ARGUMENT;
	      break;
	    default :
	      break;	// continue.
	    }
	    break;
	  }
	  if( eol == isTrue )
	    break;
	  p ++;
	  parseposition ++;
	}
	switch( st ) {
	case START :
	  return NULL_LINE;
	case VERB :
	  verb = getSubstring( q, p );
	  st = END_VERB;
	  break;
	case ARGUMENT :
	  if( add_argument( q, p ) < 0 )
	    return TOO_MANY_ARGUMENTS;
	  break;
	case QUOTEDSTRING :
	case IN_BRACE :
	  return INVALID_SYNTAX;
	default :
	  break;
	}

	st = analyzeVerb( );
	if( st == -1 )
	  return INVALID_SYNTAX;
	return COMMAND_PARSED;
}

