//=================================================================
/*

	KonoeCommandProcessor.cc

	Version 0.0.1	16-Jul-1998	H. Sakamoto
	Version 0.0.2	 2-Aug-1998	Sanshiro Enomoto
	Version 0.4.0	 6-Sep-1998
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	19-May-2001	H. Sakamoto
	Version 0.6.2	28-May-2001	H. Sakamoto

*/
//=================================================================

#include	<ui/KonoeCommandProcessor.hh>
#include	<ui/KonoeCommandArgument.hh>
#include	<ui/KonoeArgumentDescriptor.hh>
#include	<ui/KonoeOrderly.hh>
#include	<KonoeList.hh>
#include	<iostream.h>
#include	<string.h>


KonoeCommandProcessor::KonoeCommandProcessor( KonoeCommandDescriptor * cd,
		KonoeOrderly * powner ) {

	descriptor = cd;
	owner = powner;
	always_executed = 0;

}

KonoeCommandProcessor::~KonoeCommandProcessor( ) {

	delete descriptor;
	delete [] classname;

}

const KChar	* KonoeCommandProcessor::getMethodName( ) {

	return descriptor -> getName( );

}

void	KonoeCommandProcessor::printHelp( ostream & os, KInt flag ) {

	descriptor -> printHelp( os, flag );

}

void	KonoeCommandProcessor::setClassName( const KChar * name ) {

	if( classname ) {
	    delete [] classname;
	}

	classname = new KChar [ strlen( name ) + 1 ];
	strcpy( classname, name );

}

void	KonoeCommandProcessor::printErrorHeader( ostream & os ) {

	os << "Konoe" << owner -> getClassName( ) << "Orderly::"
		<< classname << "::";

}

void	KonoeCommandProcessor::printError(
		ostream & os, KInt code, const KChar * opt ) {

	printErrorHeader( os );
	switch( code ) {
	case E_ARGUMENT_MISSING :
	    os << "Argument missing.";
	    break;
	case E_ARGUMENT_NOT_INTEGER :
	    os << "Invalid argument type. Integer required.";
	    break;
	case E_ARGUMENT_NOT_ALPHA :
	    os << "Invalid argument type. Alphanumeric required.";
	    break;
	case E_INVALID_ARGUMENT :
	    os << "Invalid argument.";
	    break;
	case E_ARGUMENT_OUT_OF_RANGE :
	    os << "Argument value out of range.";
	    break;
	case E_NO_SYMBOL_RESOLVER :
	    os << "No symbol resolver given.";
	    break;
	case E_NO_INTERPRETER :
	    os << "No interpreter available.";
	    break;
	default :
	    os << "Unregistered error code = " << code;
	    break;
	}
	if( opt )
	    os << opt;
	os << endl;

}

KInt	KonoeCommandProcessor::checkArgument( KonoeCommand * cmd ) {
	
	ostream& os = cmd -> getOutputStream( );

	KonoeCommandArgument *arg;
	KonoeArgumentDescriptor *arg_des;

	KInt narg = cmd -> getArgumentCount( );

	if ( narg > descriptor -> getArgumentCount( )) {

	  printErrorHeader( os );
          os << "Too many argument(s). Only " 
            << descriptor -> getArgumentCount( ) 
            << " argument(s) is(are) needed." << endl;
          return 0;   // NG.
        }

	KInt argc;
	KInt typecheck;
	for ( argc = 0; argc < narg; argc++ ) {

	  arg = cmd -> getArgument( argc );
	  arg_des = descriptor -> getArgument( argc );

          switch ( arg_des -> getType( ) ) {
            case KonoeArgumentDescriptor::INTEGER:
              typecheck = arg -> checkInteger( );
	      break;
            case KonoeArgumentDescriptor::FLOATING:
              typecheck = arg -> checkFloating( );
	      break;
            default:
              typecheck = 1;
          }

	  if ( ! typecheck ) {
	    printErrorHeader( os );
            os << "Invalid type of argument "; arg_des -> printHelp( os );
	    return 0;   // NG.
	  }

        }

	if ( narg < descriptor -> getArgumentCount( )) {
	  arg_des = descriptor -> getArgument( narg );
	  if ( arg_des -> getAppearance( ) == KonoeArgumentDescriptor::NECESSARY ) {
	    printErrorHeader( os );
            os << "Missing argument "; arg_des -> printHelp( os );
	    return 0;   // NG.
	  }
        }

	return 1;   // OK.

}
