//=====================================================================
/*

	KonoeCommander.cc

	Version 0.0.1	10-Jul-1998	H. Sakamoto
	Version 0.0.2	 1-Aug-1998	Sanshiro Enomoto
	Version 0.0.3	16-Aug-1998	S. Nishida
	Version 0.4.0	 7-Sep-1998	H. Sakamoto and S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	14-Nov-2000	H. Sakamoto
	Version 0.6.2	17-May-2001	H. Sakamoto
	Version 0.6.3	28-May-2001	H. Sakamoto

*/
//=====================================================================

#include	<ui/KonoeCommander.hh>
#include	<ui/KonoeCommandInterpreter.hh>
#include	<string.h>
#include	<unistd.h>

const KChar	* KonoeCommander::classname = "Commander";

KonoeCommander::KonoeCommander( KInt nofficer, const KChar * name,
	KInt ncommands )
	: KonoeList<KonoeOfficer>( nofficer ),
	KonoeOrderly( ncommands, name ) {

	parser = new KonoeCommandParser( );
	currenttarget = 0;
	interpreter = 0;

// installing commands
	KonoeCommandDescriptor * cmd_descriptor;
	KonoeArgumentDescriptor * arg_descriptor;

	cmd_descriptor = new KonoeCommandDescriptor( "show", N_ARG_SHOW );
	arg_descriptor = new KonoeArgumentDescriptor(
		"item", KEYWORD, NECESSARY );
	arg_descriptor -> setDescription(
		"must be one of 'officers', 'objects' and 'classes'" );
	cmd_descriptor -> add( arg_descriptor );
	cmd_descriptor -> setDescription(
		"Show classes, objects and other items" );
	addCommand( new ShowUnit( cmd_descriptor, this ) );

	cmd_descriptor = new KonoeCommandDescriptor( "exit", N_ARG_EXIT );
	arg_descriptor = new KonoeArgumentDescriptor(
		"status", INTEGER, OPTIONAL );
	arg_descriptor -> setDescription(
		"return status. Any number. " );
	cmd_descriptor -> add( arg_descriptor );
	cmd_descriptor -> setDescription(
		"Terminate execution, or exit from a command file" );
	addCommand( new ExitUnit( cmd_descriptor, this ) );

	cmd_descriptor = new KonoeCommandDescriptor( "help", N_ARG_HELP );
	arg_descriptor = new KonoeArgumentDescriptor(
		"topics", KEYWORD, OPTIONAL );
	arg_descriptor -> setDescription(
		"command ( method ) name" );
	cmd_descriptor -> add( arg_descriptor );
	cmd_descriptor -> setDescription(
		"Show help statements" );
	addCommand( new HelpUnit( cmd_descriptor, this ) );

// 'sleep' command

	cmd_descriptor = new KonoeCommandDescriptor( "sleep", N_ARG_SLEEP );
	arg_descriptor = new KonoeArgumentDescriptor( "interval", INTEGER, NECESSARY );
	arg_descriptor -> setDescription( "Sleeping interval in milliseconds" );
	cmd_descriptor -> add( arg_descriptor );
	cmd_descriptor -> setDescription( "Pause execution for given interval" );
	addCommand( new SleepUnit( cmd_descriptor, this ) );

	// 'print' command

	cmd_descriptor = new KonoeCommandDescriptor( "print", N_ARG_PRINT );
	arg_descriptor = new KonoeArgumentDescriptor( "expression", STRING, OPTIONAL );
	arg_descriptor -> setDescription( "Expression to be printed" );
	cmd_descriptor -> add( arg_descriptor );
	cmd_descriptor -> setDescription( "Print a line" );
        addCommand( new PrintUnit( cmd_descriptor, this ) );

}

KonoeCommander::~KonoeCommander( ) {
}

void KonoeCommander::setInterpreter( KonoeCommandInterpreter * pinterpreter ) {

	interpreter = pinterpreter;

	ExitUnit * exitunit = (ExitUnit *) hasMethod("exit");
	if( exitunit != 0 ) exitunit -> setInterpreter( interpreter );

	KonoeCommandDescriptor * includecommand
		= new KonoeCommandDescriptor( "include", N_ARG_INCLUDE );
	includecommand -> setDescription(
		"Include a command script file" );
	KonoeArgumentDescriptor * ad = new KonoeArgumentDescriptor(
		"file", STRING, NECESSARY );
	ad -> setDescription( "a name of command file" );
	includecommand -> add( ad );
	addCommand( new IncludeUnit( includecommand, this, interpreter ) );

}

KonoeOrderly	* KonoeCommander::selectObject( KonoeCommand * pcommand,
		ostream & os ) {

	KChar	* name;
	KonoeOfficer	* aofficer;
	KonoeOrderly	* target;

	if( pcommand -> getHostName( ) ) {
	  os << "Sorry. Not implemented. ( process@host:object )" << endl;
	  return 0;
	}
	if( pcommand -> getProcessName( ) ) {
	  os << "Sorry. Not implemented. ( process:object )" << endl;
	  return 0;
	}
	name = ( KChar * ) pcommand -> getObjectName( );
	if( name ) {
	  if( strlen( name ) == 0 ) {	// .method
	    return this;
	  }
	  else {	// object.method
	    reset( );	// reset list iterator.
	    while( aofficer = next( ) ) {
	      if( target = aofficer -> findObject( name ) ) {
		currenttarget = target;
		return target;
	      }
	      if( target = aofficer -> hasClass( name ) ) {
		currenttarget = target;
		return target;
	      }
	    }
	    os << "Object " << name << " not found." << endl;
	    return 0;
	  }
	}
	else {	// method
	  if( currenttarget ) {
	    return currenttarget;
	  }
	  else {
	    os << "Target object not selected." << endl;
	    return 0;
	  }
	}
}


KInt	KonoeCommander::dispatch( KChar * commandline, ostream & os ) {

	KInt	st;
	KonoeOrderly	* target;

	st = parser -> parse( commandline );
	if( st < 0 ) {
	    os << "Error parsing " << commandline << endl;
	  st = 0;
	}
	else if( st == KonoeCommandParser::COMMAND_PARSED ) {
	  st = 0;
	  command = parser -> getCommand( );
	  command -> setOutputStream( os );
	  if( target = selectObject( command, os ) ) {
	    st = target -> execute( command );
	  }
	}
	//	delete command;
	return st;

}

KInt	KonoeCommander::dispatch( KonoeCommand * pcommand, ostream & os,
	KInt flag ) {

	KonoeOrderly	* target;
	KInt	st;

	command = pcommand;
	if( target = selectObject( command, os ) ) {
	    st = target -> execute( command, flag );
	}
	return st;

}

KonoeOrderly	* KonoeCommander::findObject( const KChar * name )
{
    KonoeOfficer	* aofficer;
    KonoeOrderly	* orderly;

    reset( );
    while( aofficer = next( ) ) {
      if( orderly = aofficer -> findObject( name ) ) {
	return orderly;
      }
    }

    return 0;
}

KInt	KonoeCommander::addOrderly( KonoeOrderly * orderly ) {

    KonoeOfficer * aofficer;
    const KChar * aclassname = orderly -> getClassName( );

    reset( );
    while( aofficer = next( ) )
      if( aofficer -> hasClass( aclassname ) )
        if( aofficer -> add( orderly ) >= 0 ) {
          orderly -> setOfficer( aofficer );
          return 1;
        }
    return 0;
}

KInt	KonoeCommander::employ( KonoeOfficer * aofficer ) {

	aofficer -> setCommander( this );
	return add( aofficer );

}

void	KonoeCommander::showOfficers( ostream & os ) {

	KonoeOfficer	* aofficer;

	os << "Officers employed ... " << endl;
	reset( );
	while( aofficer = next( ) ) {
	      os << "\t" << aofficer -> getObjectName( ) << endl;
	}

}

void	KonoeCommander::showObjects( ostream & os ) {

	KonoeOfficer	* aofficer;

	os << "Objects registered ..." << endl;
	reset( );
	while( aofficer = next( ) ) {
	      aofficer -> printObjectList( os );
	}

}

void	KonoeCommander::showClasses( ostream & os ) {

	KonoeOfficer	* aofficer;

	os << "Classes manageable ..." << endl;
	reset( );
	while( aofficer = next( ) ) {
	      aofficer ->printClassList( os );
	}

}

//-----------------------------------------------------------------

KonoeCommander::ExitUnit::ExitUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) { 
	interpreter = ip;
	setClassName( "ExitUnit" );

}

void	KonoeCommander::ExitUnit::setInterpreter(
				KonoeCommandInterpreter *ip ) {
	interpreter = ip;
}


KInt	KonoeCommander::ExitUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	if( interpreter == 0 ) {
	    printError( os, E_NO_INTERPRETER );
	    return -1;
	}
	else interpreter -> doExit( 0 );

}

//------------------------------------------------------------------

KonoeCommander::IncludeUnit::IncludeUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow, KonoeCommandInterpreter * ip )
		: KonoeCommandProcessor( cd, ow ) {

	interpreter = ip;
	setClassName( "IncludeUnit" );

}

KInt	KonoeCommander::IncludeUnit::execute( KonoeCommand * cmd ) {

	ostream	& os = cmd -> getOutputStream( );

	if( ! checkArgument( cmd ) ) return 0;
	return interpreter -> doInclude(
		cmd -> getArgument( ARG_INCLUDE_FILE ) -> getValue( ) );
}

//------------------------------------------------------------------

KonoeCommander::HelpUnit::HelpUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow )
		: KonoeCommandProcessor( cd, ow ) {

	setClassName( "HelpUnit" );

}

KInt	KonoeCommander::HelpUnit::execute( KonoeCommand * cmd ) {

	KonoeOfficer	* aofficer;
	KonoeCommander	* commander;
	ostream & os = cmd -> getOutputStream( );

	commander = ( KonoeCommander * ) owner;
	commander -> printHelp( cmd, cout );
	commander -> reset( );
	while( aofficer = commander -> next( ) ) {
	  if( aofficer -> getCommandCount( ) ) {
	    aofficer -> printHelp( cmd, os );
	  }
	}
	return 0;

}

//-----------------------------------------------------------------

KonoeCommander::ShowUnit::ShowUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * ow )
		: KonoeCommandProcessor( cd, ow ) {

	setClassName( "ShowUnit" );

}

KInt	KonoeCommander::ShowUnit::execute( KonoeCommand * cmd ) {

	KInt	n;

	ostream & os = cmd -> getOutputStream( );
	KonoeCommander	* commander;
	commander = ( KonoeCommander * ) owner;

	if( ( n =  cmd -> getArgumentCount( ) ) == 0 ) {
	  commander -> showOfficers( os );
	  commander -> showClasses( os );
	  commander -> showObjects( os );
	  return 0;
	}

	KInt	i;
	KonoeCommandArgument	* arg;
	KonoeOfficer		* aofficer;
	KChar	* item;

	for( i = 0; i < n; i ++ ) {
	  arg = cmd -> getArgument( i );
	  item = ( KChar * ) arg -> getValue( );
	  if( strcmp( item, "officers" ) == 0 )
	    commander -> showOfficers( os );
	  else if( strcmp( item, "objects" ) == 0 )
	    commander -> showObjects( os );
	  else if( strcmp( item, "classes" ) == 0 )
	    commander -> showClasses( os );
	  else {
	    printErrorHeader( os );
	    os << "Unknown item : " << item
	       << " : Choose one of { officers | objects | classes }." << endl;
	  }
	}
	return 0;
}

//---------------------------------------------------------------------

KonoeCommander::SleepUnit::SleepUnit( KonoeCommandDescriptor * cd,
		KonoeOrderly * powner )
	: KonoeCommandProcessor( cd, powner ) {

	setClassName( "SleepUnit" );

}

KInt	KonoeCommander::SleepUnit::execute( KonoeCommand * cmd ) {

	KonoeCommandArgument	* arg;
	KInt	n_arg, st;
	ostream & os = cmd -> getOutputStream( );

	n_arg = cmd -> getArgumentCount( );
	if( n_arg < 1 ) {
	  printError( os, E_ARGUMENT_MISSING );
	  return -1;
	}

	arg = cmd -> getArgument( ARG_SLEEP_INTERVAL );
	if( ! arg -> isInteger( ) ) {
	  printError( os, E_ARGUMENT_NOT_INTEGER, arg -> getValue( ) );
	  return -1;
	}

	usleep( arg -> toInteger( ) * 1000 );	// milli.

	return 0;

}

//------------------------------------------------------------------

KonoeCommander::PrintUnit::PrintUnit( KonoeCommandDescriptor * cd,
				      KonoeOrderly * ow )
	: KonoeCommandProcessor( cd, ow ) {

	setClassName( "PrintUnit" );

}

KInt	KonoeCommander::PrintUnit::execute( KonoeCommand * cmd ) {

	ostream & os = cmd -> getOutputStream( );

	KInt	n_arg = cmd -> getArgumentCount( );

	for( KInt i = 0; i < n_arg; i ++ ) {

	  KonoeCommandArgument	* arg = cmd -> getArgument( i );
	  if( arg -> isInteger( ) )
	    os << arg -> toInteger( );
	  else
	    os << arg -> toString( );
	  if( i != n_arg - 1 )
	    os << ' ';
	}
	os << endl;

}
