//=====================================================================
/*

	KonoeCommander.hh

	Version 0.0.1	10-Jul-1998	H. Sakamoto
	Version 0.0.2	 1-Aug-1998	Sanshiro Enomoto
	Version 0.0.3	16-Aug-1998	S. Nishida
	Version 0.4.0	 6-Sep-1998	H. Sakamoto and S. Nishida
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	07-Dec-2000	H. Sakamoto
	Version 0.6.2	11-May-2001	H. Sakamoto
	Version 0.6.3	28-May-2001	H. Sakamoto


*/
//=====================================================================

#ifndef	__KONOECOMMANDER_HH
#define	__KONOECOMMANDER_HH

#include	<KonoePrimitives.hh>
#include	<KonoeList.hh>
#include	<ui/KonoeOfficer.hh>
#include	<ui/KonoeCommandProcessor.hh>
#include	<ui/KonoeCommandParser.hh>
#include	<ui/KonoeCommand.hh>
#include	<iostream.h>

class	KonoeCommandInterpreter;

class	KonoeCommander	: public KonoeOrderly,
		public KonoeList<KonoeOfficer> {

public :

	enum	{	CMD_SHOW,
			CMD_EXIT,
			CMD_HELP,
			CMD_INCLUDE,
			CMD_SLEEP,
			CMD_PRINT,
			N_COMMANDS		};
	enum	{	ARG_SHOW_ITEM,
			N_ARG_SHOW		};
	enum	{	ARG_EXIT_STATUS,
			N_ARG_EXIT		};
	enum	{	ARG_HELP_TOPICS,
			N_ARG_HELP		};
	enum	{	ARG_INCLUDE_FILE,
			N_ARG_INCLUDE		};
	enum	{	ARG_SLEEP_INTERVAL,
			N_ARG_SLEEP		};
	enum	{	ARG_PRINT_EXPRESSION,
			N_ARG_PRINT		};

protected :
	static const KChar	* classname;

	KonoeCommandInterpreter	* interpreter;

	KonoeCommandParser	* parser;
	KonoeOrderly		* currenttarget;
	KonoeCommand		* command;

	class	ExitUnit	: public KonoeCommandProcessor {
	protected :
		KonoeCommandInterpreter	* interpreter;
	public :
		ExitUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip = 0 );
		void setInterpreter( KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );
	};

	class	IncludeUnit	: public KonoeCommandProcessor {
	protected :
		KonoeCommandInterpreter * interpreter;
	public :
		IncludeUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow,
			KonoeCommandInterpreter * ip );
		KInt	execute( KonoeCommand * cmd );
	};

	class	HelpUnit	: public KonoeCommandProcessor {
	public :
		HelpUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow );
		KInt	execute( KonoeCommand * cmd );
	};

	class	ShowUnit	: public KonoeCommandProcessor {
	public :
		ShowUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow );
		KInt	execute( KonoeCommand * cmd );
	};

	class	SleepUnit	: public KonoeCommandProcessor {
	public :
		SleepUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow );
		KInt	execute( KonoeCommand * cmd );
	};

	class	PrintUnit	: public KonoeCommandProcessor {
	public :
		PrintUnit( KonoeCommandDescriptor * cd, KonoeOrderly * ow );
		KInt	execute( KonoeCommand * cmd );
	};

public :
	KonoeCommander( KInt nofficer, const KChar * name,
			KInt ncommands = N_COMMANDS );
	~KonoeCommander( );
	KInt	dispatch( KChar * commandline, ostream & os );
	KInt	dispatch( KonoeCommand * command, ostream & os, KInt flag );
	KonoeOrderly	* findObject( const KChar * name);
	KInt addOrderly( KonoeOrderly * orderly );
	void	setInterpreter( KonoeCommandInterpreter * interpreter );
	KonoeCommandInterpreter	* getInterpreter( ) {
			return interpreter;	}
	KonoeOrderly	* selectObject( KonoeCommand * command,
					ostream & os );

	KInt	employ( KonoeOfficer * aofficer );

	const KChar	* getClassName( )	{
		return classname;		}

	void	showOfficers( ostream & os );
	void	showObjects( ostream & os );
	void	showClasses( ostream & os );

};

#endif
