//==================================================================
/*
	KonoeExpression.cc

	Version 0.6.0	09-Dec-2000	H. Sakamoto
	Version 0.6.1	15-May-2001	H. Sakamoto

*/
//==================================================================

#include	<ui/KonoeExpression.hh>
#include	<iostream.h>
#include	<stdlib.h>
#include	<string.h>

KonoeExpression::KonoeExpression( KonoeSymbolResolverList * presolver ) {

	resolverlist = presolver;

}

KInt	KonoeExpression::evaluate( KChar * string ) {


	KChar	* p;
	KInt	st;

	nesting = 0;
	KInt	state = INITIAL;

	p = string;
	while( * p ) {

	    if( * p == ' ' || * p == '\t' ) {	// white spaces
		p ++;
	    }
	    else if( * p == '(' ) {	// begin bracket.
		st = evaluate_bracket( p, & p );
		if( st < 0 )
		    return st;
		state = END_TERM;
		first_term[ 0 ] = first_term[ 1 ];
		type[ 0 ] = type[ 1 ];

	    }
	    else if( * p >= '0' && * p <= '9' ) {	// numeric
		st = evaluate_numeric( p, & p );
		if( st < 0 )
		    return st;
		state = END_TERM;
		type[ 0 ] = INTEGER;
		first_term[ 0 ] = numeric_value;
	    }
	    else {	// other. string type.
		state = TERM;
		type[ 0 ] = STRING;
		p ++;
	    }
	}
	if( state == INITIAL ) {
	    st = E_EMPTY_EXPRESSION;
	    return st;
	}
	else if( state == OPERATOR || state == END_OPERATOR ) {
	    st = E_INVALID_SYNTAX;
	    return st;
	}

	return 0;

}

KInt	KonoeExpression::evaluate_bracket( KChar * string, KChar ** tail ) {

	KChar	* p = string, op;
	KInt	st, state;
	if( nesting >= MAX_NESTING )
	    return E_NESTING_OVERFLOW;

	nesting ++;
	state = INITIAL;
	mode[ nesting ] = STANDBY;

	p ++;	// must point to (
	while( * p ) {
	    if( * p == ')' ) {	// find a ')'
		* tail = ++ p;	// next to )
		if( state == INITIAL )
		    return E_EMPTY_EXPRESSION;
		else if( state == OPERATOR || state == END_OPERATOR )
		    return E_INVALID_SYNTAX;
		switch( mode[ nesting ] ) {
		case FIRST_TERM :
		    break;
		case SECOND_TERM :
		    first_term[ nesting ] = do_arithmetic(
			first_term[ nesting ], second_term[ nesting ],
			binary_operator[ nesting ] );
		    mode[ nesting ] = FIRST_TERM;
		    break;
		default :
		    return E_INVALID_SYNTAX;
		}
		return 0;
	    }
	    else if( * p == '(' ) {	// nested (
		if( state == TERM || state == END_TERM ) {
		    * tail = p;
		    return E_INVALID_SYNTAX;
		}
		st = evaluate_bracket( p, & p );
		if( st < 0 )
		    return st;
		nesting --;
		st = update_registers( first_term[ nesting + 1 ] );
		if( st < 0 )
		    return st;
		state = END_TERM;
		
	    }
	    else if( * p >= '0' && * p <= '9' ) {
		if( state == TERM || state == END_TERM )
		    return E_INVALID_SYNTAX;
		st = evaluate_numeric( p, & p );
		if( st < 0 )
		    return st;
		st = update_registers( numeric_value );
		if( st < 0 )
			return st;
		state = END_TERM;
	    }
	    else if( * p == '+' || * p == '-' ) {
		switch( mode[ nesting ] ) {
		case STANDBY :
		    unary_operator[ nesting ] = * p;
		    mode[ nesting ] = UNARY_OPERATOR;
		    break;
		case UNARY_OPERATOR :
		    if( * p == '-' ) {
			if( unary_operator[ nesting ] == '-' )
			    unary_operator[ nesting ] = '+';
			else
			    unary_operator[ nesting ] = '-';
		    }
		    break;
		case FIRST_TERM :
		    binary_operator[ nesting ] = * p;
		    mode[ nesting ] = BINARY_OPERATOR;
		    break;
		case SECOND_TERM :
		    first_term[ nesting ] = do_arithmetic(
			first_term[ nesting ], second_term[ nesting ],
			binary_operator[ nesting ] );
		    binary_operator[ nesting ] = * p;
		    mode[ nesting ] = BINARY_OPERATOR;
		    break;
		default :
		    return E_INVALID_SYNTAX;
		}
		state = END_OPERATOR;
		p ++;
	    }
	    else if( * p == '=' || * p == '!' || * p == '>' || * p == '<'
			|| * p == '&' || * p == '|' || * p == '^' ) {
		KChar	op;
		switch( * p ) {
		case '=' :
		    if( * ( p + 1 ) != '=' )
			return E_INVALID_SYNTAX;
		    op = '=';	// equals
		    p ++;
		    break;
		case '!' :
		    if( * ( p + 1 ) != '=' )
			return E_INVALID_SYNTAX;
		    op = '!';	// not equals
		    p ++;
		    break;
		case '>' :
		    if( * ( p + 1 ) == '=' ) {
			op = ']';	// greater or equal
			p ++;
		    }
		    else if( * ( p + 1 ) == '>' ) {
			op = 'R';	// shift right
			p ++;
		    }
		    else
			op = '>';	// greater
		    break;
		case '<' :
		    if( * ( p + 1 ) == '=' ) {
			op = '[';	// smaller or equal
			p ++;
		    }
		    else if( * ( p + 1 ) == '<' ) {
			op = 'L';	// shift left
			p ++;
		    }
		    else
			op = '<';
		    break;
		case '&' :
		    if( * ( p + 1 ) == '&' ) {
			op = 'A';	// logical and
			p ++;
		    }
		    else
			op = '&';	// bitwise and
		    break;
		case '|' :
		    if( * ( p + 1 ) == '|' ) {
			op = 'O';	// logical or
			p ++;
		    }
		    else
			op = '|';	// bitwise or
		    break;
		default :
		    op = * p;
		    break;
		}
		switch( mode[ nesting ] ) {
		case FIRST_TERM :
		    binary_operator[ nesting ] = op;
		    mode[ nesting ] = BINARY_OPERATOR;
		    break;
		case SECOND_TERM :
		    first_term[ nesting ] = do_arithmetic(
			first_term[ nesting ], second_term[ nesting ],
			binary_operator[ nesting ] );
		    binary_operator[ nesting ] = op;
		    mode[ nesting ] = BINARY_OPERATOR;
		    break;
		default :
		    return E_INVALID_SYNTAX;
		}
		state = END_OPERATOR;
		p ++;
		    
	    }
	    else if( * p == '*' || * p == '/' || * p == '%' ) {
		switch( mode[ nesting ] ) {
		case FIRST_TERM :
		    binary_operator[ nesting ] = * p;
		    mode[ nesting ] = BINARY_OPERATOR;
		    break;
		case SECOND_TERM :
		    second_operator[ nesting ] = * p;
		    mode[ nesting ] = SECOND_OPERATOR;
		    break;
		default :
		    return E_INVALID_SYNTAX;
		}
		state = END_OPERATOR;
		p ++;
	    }
	    else if( ( * p >= 'A' && * p <= 'Z' ) ||
			( * p >= 'a' && * p <= 'z' ) ||
			* p == '_' ) {	// symbol name
		if( state == TERM || state == END_TERM )
		    return E_INVALID_SYNTAX;
		st = evaluate_symbol( p, & p );
		if( st < 0 )
		    return st;
		if( symbol -> getType( ) == INTEGER ) {
		    st = update_registers( symbol -> getIValue( ) );
		    if( st < 0 )
			return st;
		}
		else {
			cout<<"Unsupported symbol type"<<endl;
		    return E_INVALID_SYNTAX;
		}
		state = END_TERM;
	    }
	    else if( ( * p == ' ' || * p == '\t' ) ) {

		p ++;
	    }
	    else {
		return E_INVALID_SYNTAX;
	    }
	}
	return -1;	// unexpected end of line.
}

KInt	KonoeExpression::evaluate_numeric( KChar * string, KChar ** tail ) {

	numeric_value = strtol( string, tail, 0 );
	if( string == * tail )
	    return E_INVALID_SYNTAX;
	return 0;

}

KInt	KonoeExpression::do_arithmetic( KInt first, KInt second, KChar op ) {

	switch( op ) {
	case '+':
		return first + second;
	case '-':
		return first - second;
	case '*':
		return first * second;
	case '/':
		return first / second;
	case '%':
		return first % second;
	case '=':
		return first == second;
	case '!':
		return first != second;
	case '>':
		return first > second;
	case '<':
		return first < second;
	case ']':
		return first >= second;
	case '[':
		return first <= second;
	case 'L':
		return first << second;
	case 'R':
		return first >> second;
	case '&':
		return first & second;
	case '|':
		return first | second;
	case '^':
		return first ^ second;
	case 'A':
		return first && second;
	case 'O':
		return first || second;
	default:
		return 0;
	}
}

KInt	KonoeExpression::update_registers( KInt value ) {

	KChar	op;

	switch( mode[ nesting ] ) {
	case STANDBY:
	    mode[ nesting ] = FIRST_TERM;
	    first_term[ nesting ] = value;
	    type[ nesting ] = INTEGER;
	    break;
	case UNARY_OPERATOR :
	    if( unary_operator[ nesting ] == '-' )
		first_term[ nesting ] = - value;
	    else
		first_term[ nesting ] = value;
	    type[ nesting ] = INTEGER;
	    mode[ nesting ] = FIRST_TERM;
	    break;
	case BINARY_OPERATOR :
	    op = binary_operator[ nesting ];
	    if( op == '*' || op == '/' || op == '%' ) {
		first_term[ nesting ] = do_arithmetic(
			first_term[ nesting ], value, op );
		mode[ nesting ] = FIRST_TERM;
		break;
	    }
	    else {	// + or - etc.
		second_term[ nesting ] = value;
		mode[ nesting ] = SECOND_TERM;
		break;
	    }
	case SECOND_OPERATOR :
	    second_term[ nesting ] = do_arithmetic(
			second_term[ nesting ], value,
			second_operator[ nesting ] );
	    first_term[ nesting ] = do_arithmetic(
			first_term[ nesting ], second_term[ nesting ],
			binary_operator[ nesting ] );
	    mode[ nesting ] = FIRST_TERM;
	    break;
	default :
	    return E_INVALID_SYNTAX;
	}
	return 0;
}

KInt	KonoeExpression::evaluate_symbol( KChar * source, KChar ** tail ) {

	KChar 	* p = source, * q, * symnam;

	q = check_symbol_name( p );
	if( p == q )
	    return E_INVALID_SYNTAX;
	symnam = new KChar[ q - p + 1 ];
	strncpy( symnam, p, q - p );
	symnam[ q - p ] = '\0';

	symbol = resolverlist -> findSymbol( symnam );
	if( symbol == 0 ) {
		cout<<"Symbol not found = "<<symnam<<endl;
	    delete [] symnam;
	    * tail = q;
	    return E_SYMBOL_NOT_FOUND;
	}
	delete [] symnam;
	* tail = q;
	return 0;

}

KChar	* KonoeExpression::check_symbol_name( KChar * string ) {

	KChar * p = string;

	if( ( * p < 'a' || * p > 'z' ) && ( * p < 'A' || * p > 'Z' )
		&& ( * p != '_' ) )
	    return p;	// zero length. not a symbol name.
	p ++;
	while( * p ) {
	    if( ( * p < 'a' || * p > 'z' ) && ( * p < 'A' || * p > 'Z' )
		&& ( * p < '0' || * p > '9' ) && ( * p != '_' ) )
		return p;
	    p ++;
	}
	return p;

}
