//====================================================================
/*

	KonoeOrderly.cc

	Version 0.0.1	02-Jul-1998	H. Sakamoto
	Version 0.4.0	 6-Sep-1998
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
	Version 0.6.1	28-May-2001	H. Sakamoto
	Version 0.6.2	13-Jul-2001	H. Sakamoto

*/
//====================================================================

#include	<ui/KonoeOrderly.hh>
#include	<string.h>

KonoeOrderly::KonoeOrderly( KInt ptablesize, const KChar * name ) {

	if( ptablesize ) {
	  commandlist = new KonoeList<KonoeCommandProcessor>( ptablesize );
	}
	else {
	  commandlist = 0;
	}
	if( name ) {
	  objectname = new KChar[ strlen( name ) + 1 ];
	  strcpy( objectname, name );
	}
}

KonoeOrderly::~KonoeOrderly( ) {

	delete [] objectname;
	delete commandlist;

}

void	KonoeOrderly::setOfficer( KonoeOfficer * pofficer ) {

	officer = pofficer;

}


const KChar	* KonoeOrderly::getObjectName( ) const {

	return ( const KChar * ) objectname;

}

KonoeCommandProcessor	* KonoeOrderly::hasMethod( const KChar * method ) {

	KonoeCommandProcessor	* cd, * cdmatch;

	if( commandlist == 0 )
	  return 0;

	KInt	l = strlen( method );
	cdmatch = 0;

	commandlist -> reset( );
	while( cd = commandlist -> next( ) ) {
	  if( strncmp( method, cd -> getMethodName( ), l ) == 0 ) {
	    if( cdmatch )
	      return 0;	// ambiguous
	    else
	      cdmatch = cd;
	  }
	}
	return cdmatch;
}

KInt	KonoeOrderly::addCommand( KonoeCommandProcessor * cmd ) {

	if( commandlist == 0 )
	  return -1;
	return commandlist -> add( cmd );

}

void	KonoeOrderly::printHelp( KonoeCommand * command, ostream & os ) {

	KonoeCommandProcessor	* cd;
	KonoeCommandArgument	* arg;

	if( objectname )
	  os << "Object = " << objectname << 
	    " of class " << getClassName( ) << endl;
	else
	  os << "Class = " << this -> getClassName( ) << endl;
	if( command -> getArgumentCount( ) == 0 ) {
	  if( commandlist == 0 ) {
	    os << "    No commands available for this object" << endl;
	  }
	  else {
	    os << "    Commands available ..." << endl;
	    commandlist -> reset( );
	    while( cd = commandlist -> next( ) ) {
	      os << "\t";
	      cd -> printHelp( os, 0 );
	    }
	  }
	}
	else {

	    arg = command -> getArgument( 0 );	// first argument
	    cd = hasMethod( ( const KChar * ) arg -> getValue( ) );
	    if( cd == 0 ) {	// no match
	      os << "Command not found = " << arg -> getValue( ) << endl;
	    }
	    else {
		cd -> printHelp( os, 1 );	// detailed.
	    }
	}
}

KInt	KonoeOrderly::getCommandCount( ) {

	if( commandlist == 0 )
	  return 0;
	return commandlist -> getCount( );

}

KInt	KonoeOrderly::execute( KonoeCommand * cmd, KInt flag ) {

	KonoeCommandProcessor	* cd;
	ostream	& os = cmd -> getOutputStream( );

	cd = hasMethod( cmd -> getMethodName( ) );
	if( cd ) {
	  if( flag )
	    return cd -> execute( cmd );
	  else if( cd -> isAlways( ) )
	    return cd -> execute( cmd );
	  else
	    return 0;
	}
	else {
	  os << "Method not found. " << cmd -> getMethodName( ) << endl;
	  return 0;
	}
}

//	dummy members


KInt	KonoeOrderly::createObject( KonoeCommand * ) {

cout<<"KonoeOrderly::createObject called"<<endl;
	return 0;

}

KInt	KonoeOrderly::deleteObject( KonoeCommand * ) {

	return 0;

}

KInt	KonoeOrderly::showTarget( KonoeCommand * ) {

	return 0;

}

KInt	KonoeOrderly::isOrderlyOf( void * ) {

	return 0;

}


KonoeOrderly	* KonoeOrderly::clone( const KChar * ) {

	return 0;

}

void	* KonoeOrderly::getTarget( ) {

	return 0;

}

void	KonoeOrderly::setTarget( void * ptarget ) {

}
