//=====================================================================
/*

    StatusViewer.cc

    Version 0.0.1   16-Dec-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include "StatusViewer.hh"

#include <stdlib.h>

#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclButton.hh>
#include <ui/KonoeTclEntry.hh>
#include <ui/KonoeTclDimension.hh>

#include <analysis/Konoe1DHistogram.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoePhysicalValueCM.hh>

#include <system/KonoeServerSocket.hh>
#include <system/KonoeClientSocket.hh>
#include <system/KonoeOutputObjSocket.hh>
#include <system/KonoeInputObjSocket.hh>
#include <system/KonoeOutputObjFile.hh>

StatusViewer::StatusViewer(KonoeTclWindow* kwin, KChar* kname)
{
  connectButton = new KonoeTclButton(kwin, "connectbutton", "CONNECT");
  disConnectButton = new KonoeTclButton(kwin, "disconnectbutton", "DISCONNECT");
  startButton = new KonoeTclButton(kwin, "startbutton", "START READING");
  stopButton = new KonoeTclButton(kwin, "stopbutton", "STOP READING");

  txthost = new KonoeTclEntry(kwin, "txthost");
  txtport = new KonoeTclEntry(kwin, "txtport");

  addMouseAction( connectButton );
  addMouseAction( startButton );
  addMouseAction( disConnectButton );
  addMouseAction( stopButton );

  connectButton->setEventProcessor( this );
  startButton->setEventProcessor( this );
  disConnectButton->setEventProcessor( this );
  stopButton->setEventProcessor( this );

  txthost->setText("asaix2.cc.it-hiroshima.ac.jp");
  txtport->setText("12000");

  testHisto = new Konoe1DHistogram( new KonoeLinearDataScale(0.0, 100.0, 100) );
  testHisto->clear();

  testPanel = new KonoeTclDrawPanel(kwin, "testpanel1");
  testCanvas = new KonoeTclDrawCanvas(testPanel, "testcanvas", testHisto, 
                                      KonoeTclDimension(200, 200),
                                      "1D Histogram test",
                                      new KonoeLinearDataScale(0.0, 50.0, 100));
  
  testPanel->setCanvas(testCanvas, "");
  testCanvas->draw();
  testCanvas->fillBoth();

}

StatusViewer::~StatusViewer()
{
  delete connectButton;
  delete disConnectButton;
  delete startButton;
  delete stopButton;
  delete txthost;
  delete txtport;
  delete testHisto;
  delete testCanvas;
  delete testPanel;
}

void StatusViewer::connectButtonAction()
{
  KChar host[256];
  KInt  port;

  strcpy(host, txthost->getText());
  port = atoi(txtport->getText());

  socket = new KonoeClientSocket( host, port );
  in = new KonoeInputObjSocket( socket );
  in->installCM( new KonoeHistogramCM() );
  in->installCM( new KonoePhysicalValueCM() );
}

void StatusViewer::disConnectButtonAction()
{
  delete socket;
  delete in;
  cout << "disConnectButtonAction" << endl;
}

void StatusViewer::startButtonAction()
{
  KInt n = 0;
  startflag = 1;

  while(1){
    cout << "--- No. " << ++n << " ---" << endl;

    Konoe1DHistogram* histogram;

    histogram = (Konoe1DHistogram*) in->get();
    if( histogram == 0 ){ cerr << "error" << endl; }

    //histogram->dump(cout);

    testCanvas->setDataSource( histogram );
    testCanvas->clearAll();
    testCanvas->draw();
    testCanvas->update();

    //sleep(1);
    delete histogram;
    if( startflag == 0 ) break;
  }
}

void StatusViewer::stopButtonAction()
{
  startflag = 0;
  cout << "stopButtonAction" << endl;
}

void StatusViewer::mouseAction( KonoeTclButton* kbtn, KInt type, KInt button,
                                Time time, KInt x, KInt y )
{
cout << kbtn->getName() << endl;
  if( type == KMousePress && button == LButton ){
    if( strcmp(kbtn->getName(), "connectbutton") == 0 )
      connectButtonAction();
    else if( strcmp(kbtn->getName(), "startbutton") == 0 )
      startButtonAction();
    else if( strcmp(kbtn->getName(), "disconnectbutton") == 0 )
      disConnectButtonAction();
    else if( strcmp(kbtn->getName(), "stopbutton") == 0 )
      stopButtonAction();
  }
}
