//=====================================================================
/*

    KonoeTclFMCanvas.hh

    Version 0.5.0   08-Sep-1999 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLFMCANVAS_HH
#define __KONOETCLFMCANVAS_HH

#include <ui/KonoeTclCanvas.hh>
#include <ui/KonoeTclActionListener.hh>
#include <ui/KonoeTclMouseAction.hh>
#include <ui/KonoeTclMouseMotion.hh>
#include <ui/KonoeTclWindowAction.hh>
#include <ui/KonoeTclEventProcessor.hh>
#include <KonoeList.hh>

class KonoeTclFMManager;
class KonoeTclWindow;
class KonoeTclImage;
class KonoeTclDimension;
class KonoeNetworkFileManager;
class KonoeDirectoryInfomation;
class KonoeFileInfomation;
class KonoeFileStatus;
class KonoePath;
class KonoeTclFMIconData;

class KonoeTclFMCanvas : public KonoeTclCanvas,
                         public KonoeTclActionListener,
                         public KonoeTclMouseAction,
                         public KonoeTclMouseMotion,
                         public KonoeTclWindowAction,
                         public KonoeTclEventProcessor {

  public:
    KonoeTclFMCanvas(KonoeTclWindow* kwin, KChar* kname, KScrollType scrl);
    virtual ~KonoeTclFMCanvas();

    virtual void setIconXY() = 0;
    virtual void drawIcon() = 0;

    enum KFileMode { KAllFiles, KHiddenFiles };
    void getInfo(KFileMode mode);
    void buildGUI();
    void refresh();
    void selectClear();
    void cd(KChar* _path);
    KInt getIconType(KInt id);
    void setIconType();
    void setCanvasWindowSize();

    void directoryInfoError(KInt st);

    virtual void mouseAction(KonoeTclCanvas* kbt, KInt type, KInt button,
                             Time time, KInt x, KInt y) = 0;
    virtual void mouseMotion(KonoeTclCanvas* kbt,  KInt type, KInt state,
                             Time time, KInt x, KInt y){;}
    virtual void windowAction(KonoeTclCanvas* kbt, XEvent* xe) = 0;

  protected:

    void initFileStatus();
    void selectionFiles(KFileMode mode);
    void createIconImage();
    void createIconXY();
    void deleteIconImage();
    void deleteIconXY();
    void createFileName();
    void deleteFileName();
    void fileNameSort();

    KonoeTclWindow*      parentWindow;
    KInt                 defaultScreenX;
    KInt                 defaultScreenY;
    KonoeTclDimension*   screenXY;
    KonoeTclDimension**  iconXY;
    KInt                 fileCount;
    KBool                repaint;

    KChar                directoryName[256];
    KChar                **fileName;
    KChar                **date;
    KInt                 *fileSize;

    KChar kHost[256];
    KChar kSystemPath[256];
    KonoeTclFMManager*         kfmgr;
    KonoeTclFMIconData*        kicondata;

    KonoeNetworkFileManager*   kNetManager;
    KonoeDirectoryInfomation*  kDirInfo;
    KonoeFileInfomation*       kFileInfo;
    KonoeFileStatus**          kFileStatus;
    KonoePath*                 kPath;

    KonoeTclImage** iconImage;
    KonoeTclImage* comp;
    KonoeTclImage* text;
    KonoeTclImage* dir;
    KonoeTclImage* hist;
    enum iconType { KComputer, KDirectory, KFile, KHistogram };
    KInt* iconImageType;
    KInt selected;
    KInt clickedTime;
    KInt clickedTime_old;
    KInt selectedStartX;
    KInt selectedStartY;
    KInt selectedEndX;
    KInt selectedEndY;
    KonoeList<KInt>* klist;
};

#endif
