//=====================================================================
/*

    KonoeTclFMDetailCanvas.cc

    Version 0.5.0   08-Sep-1999 T. Hosoi

*/
//====================================================================

#include <KonoeTclFMDetailCanvas.hh>
#include <KonoeTclFMManager.hh>
#include <KonoeTclFMEntry.hh>
#include <KonoeTclFMPopupMenu.hh>
#include <KonoeTclFMIconData.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclCanvas.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclImage.hh>

#include <stdlib.h>

KonoeTclFMDetailCanvas::KonoeTclFMDetailCanvas(KonoeTclFMManager* _kfmgr,
                                               KonoeTclWindow* kwin)
  : KonoeTclFMCanvas(kwin, "detailcanvas", KBoth)
{
  parentWindow = kwin;
  //KonoeTclCanvas::createWithVScrollBar(kwin, "detailcanvas");


  defaultScreenX = 350;
  defaultScreenY = 350;
  KonoeTclCanvas::setSize(defaultScreenX, defaultScreenY);
  screenXY = new KonoeTclDimension(0, 0);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());

  KonoeTclMouseAction::addMouseAction( (KonoeTclCanvas*) this );
  KonoeTclWindowAction::addWindowAction( (KonoeTclCanvas*) this );
  KonoeTclUIPrimitive::setEventProcessor( (KonoeTclEventProcessor*) this );

  KonoeTclCanvas::setBackGround("white");

  kfmgr = _kfmgr;
  kfmgr->setCanvas( this );
  kNetManager = kfmgr->getNetworkFileManager();

  KonoeTclFMCanvas::getInfo(KAllFiles);
  drawIcon();
  repaint = true;
  selected = 0;
}

KonoeTclFMDetailCanvas::~KonoeTclFMDetailCanvas()
{ ; }

void KonoeTclFMDetailCanvas::setIconXY()
{
  KInt x = 0;
  KInt y = 0;

  for(KInt i=0; i<fileCount; i++){
    iconXY[i]->setWidth(x);
    iconXY[i]->setHeight(y);
    y += 20;
  }
}

void KonoeTclFMDetailCanvas::drawIcon()
{
  static KInt x;
  static KInt y;

  comp->setData(kicondata->getMiniCompData());
  dir->setData(kicondata->getMiniDirData());
  text->setData(kicondata->getMiniTextData());
  hist->setData(kicondata->getMiniHistData());

  KChar tmpFile[256];
  for(KInt j=0;j<256;j++) tmpFile[j]='\0';

  for(KInt i=0; i<fileCount; i++){
    strcpy(tmpFile, kFileStatus[i]->getFileName());
    strcpy(tmpFile, strrchr(tmpFile, '/'));
    strcpy(fileName[i], strtok(tmpFile, "/"));
    strncpy(tmpFile,fileName[i],20);

    setColor("black");

    iconType t;

    t = (iconType)getIconType(i);
    switch(t){
      case KComputer:
        drawImagenw( iconXY[i]->getWidth(), iconXY[i]->getHeight(), comp );
        break;
      case KDirectory:
        drawImagenw( iconXY[i]->getWidth(), iconXY[i]->getHeight(), dir );
        break;
      case KFile:
        drawImagenw( iconXY[i]->getWidth(), iconXY[i]->getHeight(), text );
        break;
      case KHistogram:
        drawImagenw( iconXY[i]->getWidth(), iconXY[i]->getHeight(), hist );
        break;
      default: ;
    }
    drawString(iconXY[i]->getWidth()+20, iconXY[i]->getHeight(), tmpFile);

//    drawString(iconXY[i]->getWidth()+180, iconXY[i]->getHeight(), 
//               kFileStatus[i]->getFileSize());

    sprintf(tmpFile, "%d", kFileStatus[i]->getFileSize());

    drawString(iconXY[i]->getWidth()+300, iconXY[i]->getHeight(), tmpFile);
    
  }
  if(fileCount > 0)
  setScrollregion(getWidth(), iconXY[fileCount-1]->getHeight());
}

void KonoeTclFMDetailCanvas::checkClickedPoint(KInt mx, KInt my)
{
  KInt xx;
  KInt yy;
  KDouble sxview;
  KDouble syview;
  KInt xregion;
  KInt yregion;

  sxview = getXView();
  syview = getYView();
  xregion = getWidth();
  yregion = iconXY[fileCount-1]->getHeight();

  xx = (KInt)((KDouble)xregion*sxview + (KDouble)mx);
  yy = (KInt)((KDouble)yregion*syview + (KDouble)my);

  for(KInt i=0; i<fileCount; i++){
    if( iconXY[i]->getWidth() < xx && xx <= iconXY[i]->getWidth()+170
      && iconXY[i]->getHeight() < yy && yy <= iconXY[i]->getHeight()+20 ){
          selected = i; break;
    }
    else{
      selected = -1;
    }
  }
}

void KonoeTclFMDetailCanvas::mouseAction(KonoeTclCanvas* kbt, KInt type,
                         KInt button, Time time, KInt x, KInt y)
{
  if(type == KMousePress && button == LButton){
    if( selected != -1){
      //setColor("white");
      //drawRectangle( iconXY[selected]->getWidth()-2,
      //               iconXY[selected]->getHeight()-2,
      //               160, 19);
      deleteItem("selectItem");
    }
    checkClickedPoint(x, y);
    if( abs(clickedTime - time) > 300){ //Single Click
      if( selected != -1){
        setColor("blue");
          drawRectangle( iconXY[selected]->getWidth()-2,
                         iconXY[selected]->getHeight()-2,
                         160, 19, "selectItem");
      }
    }
    else{ //Double Click
      if(selected != -1){
        KonoeTclFMCanvas::cd(fileName[selected]);
        selected = -1;
        refresh();
      }
    }

    clickedTime = time ;
  } // End of KMousePress LButton Action

  // PopupWindow
  else if(type == KMouseRelease && button == RButton){
    if(selected != -1){
      //setColor("white");
      //drawRectangle( iconXY[selected]->getWidth()-2,
      //               iconXY[selected]->getHeight()-2,
      //               160, 19);
      deleteItem("selectItem");
    }
    checkClickedPoint(x, y);
    if(selected != -1){
      setColor("blue");
      drawRectangle( iconXY[selected]->getWidth()-2,
                     iconXY[selected]->getHeight()-2,
                     160, 19, "selectItem");
    }

    KInt xx;
    KInt yy;
    xx = parentWindow->getRootX() + x;
    yy = parentWindow->getRootY() + y;
    kfmgr->setPopupType(KMenu);
    (kfmgr->getPopup())->setPopup(xx+10, yy);
    kfmgr->setFileStatus(kFileStatus[selected]);
  }
}

void KonoeTclFMDetailCanvas::windowAction(KonoeTclCanvas* kbt, XEvent* xe)
{
  if(xe->xdestroywindow.type != KDestroyWindow){
    if(repaint){
      repaint = false;
      KonoeTclFMCanvas::setCanvasWindowSize();
      KonoeTclFMCanvas::refresh();
    }
    else {
      repaint = true;
    }
  }
  else{
    KonoeTclFMCanvas::deleteIconImage();
    KonoeTclFMCanvas::deleteIconXY();
    KonoeTclFMCanvas::deleteFileName();
  }
}
