//====================================================================
/*

    KonoeTclFMIconCanvas.cc

    Version 0.5.0   08-Sep-1999 T. Hosoi

*/
//====================================================================

#include <KonoeTclFMIconCanvas.hh>
#include <KonoeTclFMManager.hh>
#include <KonoeTclFMEntry.hh>
#include <KonoeTclFMPopupMenu.hh>
#include <KonoeTclFMIconData.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclCanvas.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclImage.hh>

#include <system/KonoeFileManagerCM.hh>
#include <system/KonoeFileManageServer.hh>
#include <system/KonoeFileRequest.hh>
#include <system/KonoeDirectoryRequest.hh>
#include <system/KonoePath.hh>
#include <system/KonoeNetworkFileManager.hh>

#include <stdlib.h>
#include <unistd.h>

KonoeTclFMIconCanvas::KonoeTclFMIconCanvas(KonoeTclFMManager* _kfmgr,
                                           KonoeTclWindow* kwin)
  : KonoeTclFMCanvas(kwin, "iconcanvas", KVertical)
{
  parentWindow = kwin;
  //KonoeTclCanvas::createWithVScrollBar(kwin, "canvas");


  defaultScreenX = 350;
  defaultScreenY = 350;
  KonoeTclCanvas::setSize(defaultScreenX, defaultScreenY);
  screenXY = new KonoeTclDimension(0, 0);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());

  KonoeTclMouseAction::addMouseAction( (KonoeTclCanvas*) this );
  KonoeTclMouseMotion::addMouseMotion( (KonoeTclCanvas*) this );
  KonoeTclWindowAction::addWindowAction( (KonoeTclCanvas*) this );
  KonoeTclUIPrimitive::setEventProcessor( (KonoeTclEventProcessor*) this );

  KonoeTclCanvas::setBackGround("white");

  kfmgr = _kfmgr;
  kfmgr->setCanvas( this );
  kNetManager = kfmgr->getNetworkFileManager();

  KonoeTclFMCanvas::getInfo(KAllFiles);
  drawIcon();
  repaint = true;
  selected = 0;
  klist = new KonoeList<KInt>(0);
  klist->clear();
}

KonoeTclFMIconCanvas::~KonoeTclFMIconCanvas()
{
  delete klist;
}

void KonoeTclFMIconCanvas::setIconXY()
{
  KInt x;
  KInt y;

  KInt iconNumperLine;
  iconNumperLine = getIconNumperLine();

  for(KInt i=0; i<fileCount; i++){
    x = 110*(i%iconNumperLine)+50;
    y = 80*(i/iconNumperLine)+30;
    iconXY[i]->setWidth(x);
    iconXY[i]->setHeight(y);
  }
}

void KonoeTclFMIconCanvas::drawIcon()
{
  KChar cmd[256];
  iconType t;

  comp->setData(kicondata->getCompData());
  dir->setData(kicondata->getDirData());
  text->setData(kicondata->getTextData());
  hist->setData(kicondata->getHistData());

  for(KInt i=0; i<fileCount; i++){
    t = (iconType)getIconType(i);
    switch(t){
      case KComputer:
        drawImage( iconXY[i]->getWidth(), iconXY[i]->getHeight(), comp );
        break;
      case KDirectory:
        drawImage( iconXY[i]->getWidth(), iconXY[i]->getHeight(), dir );
        break;
      case KFile:
        drawImage( iconXY[i]->getWidth(), iconXY[i]->getHeight(), text );
        break;
      case KHistogram:
        drawImage( iconXY[i]->getWidth(), iconXY[i]->getHeight(), hist );
        break;
      default: ;
    }

    setColor("black");

    KChar tmpFile[256];
    for(KInt j=0;j<256;j++) tmpFile[j]='\0';

    strcpy(tmpFile, kFileStatus[i]->getFileName());
    strcpy(tmpFile, strrchr(tmpFile, '/'));
    strcpy(fileName[i], strtok(tmpFile, "/"));

    drawText( iconXY[i]->getWidth(), iconXY[i]->getHeight()+30,
              fileName[i], 100 );
  }
  if(fileCount > 0)
  setScrollregion(getWidth(), iconXY[fileCount-1]->getHeight()+50);
}

KInt KonoeTclFMIconCanvas::getIconNumperLine()
{
  KInt screenX;
  screenX = getScreenSize().getWidth();

  KInt iconNumperLine;
  iconNumperLine = (screenX-90)/110;

  return iconNumperLine+1;
}

void KonoeTclFMIconCanvas::checkClickedPoint(KInt mx, KInt my)
{

  KonoeTclDimension* realDim = toRealPosition(mx, my);
  KInt xx = realDim->getWidth();
  KInt yy = realDim->getHeight();
  delete realDim;

  for(KInt i=0; i<fileCount; i++){
    if( iconXY[i]->getWidth()-16 < xx && xx < iconXY[i]->getWidth()+16
    && iconXY[i]->getHeight()-16 < yy && yy < iconXY[i]->getHeight()+35 ){
          selected = i; break;
    }
    else{
      selected = -1;
    }
  }
}

KonoeTclDimension* KonoeTclFMIconCanvas::toRealPosition(KInt x, KInt y)
{
  KInt xx;
  KInt yy;
  KDouble sxview;
  KDouble syview;
  KInt xregion;
  KInt yregion;

  sxview = getXView();
  syview = getYView();
  xregion = getWidth();
  yregion = iconXY[fileCount-1]->getHeight() + 35;

  xx = (KInt)((KDouble)xregion*sxview + (KDouble)x);
  yy = (KInt)((KDouble)yregion*syview + (KDouble)y);

  return new KonoeTclDimension(xx, yy);
}

void KonoeTclFMIconCanvas::setSelectedIconList
                              (KInt x1, KInt y1, KInt x2, KInt y2)
{
  klist->allocate(fileCount);
  if(x1>x2){
    KInt tmpX = x1;
    x1 = x2;
    x2 = tmpX;
  }
  if(y1>y2){
    KInt tmpY = y1;
    y1 = y2;
    y2 = tmpY;
  }
  for(KInt i=0; i<fileCount; i++){
    KInt ix = iconXY[i]->getWidth();
    KInt iy = iconXY[i]->getHeight();
    if(x1 <= ix && ix <= x2 && y1<= iy && iy <= y2){
      klist->add(new KInt(i));
    }
  }
  for(KInt i=0; i<klist->getCount(); i++){
    setColor("blue");
    drawRectangle( iconXY[*(klist->get(i))]->getWidth()-18,
                     iconXY[*(klist->get(i))]->getHeight()-18,
                     36, 36, "selectItem");
    cout << *(klist->get(i)) << endl;
  }
}

void KonoeTclFMIconCanvas::mouseAction(KonoeTclCanvas* kbt, KInt type,
                                       KInt button, Time time, KInt x, KInt y)
{
  if(type == KMousePress && button == LButton){
    if(klist->getCount()!=0){
      for(KInt i=0; i<klist->getCount(); i++){
      deleteItem("selectItem");
      }
    }
    if(selected != -1){
    
      deleteItem("selectItem");
    }
    checkClickedPoint(x, y);
    cout << "selected icon number : " << selected << endl;
    if( abs(clickedTime - time) > 300){ //Single Click
      if(selected != -1){
        setColor("blue");
        drawRectangle( iconXY[selected]->getWidth()-18,
                       iconXY[selected]->getHeight()-18,
                       36, 36, "selectItem");
      }
    }
    else{ //Double Click
      if(selected != -1){
        KonoeTclFMCanvas::cd(fileName[selected]);
cout << "bbbbbbbbbbbbbbbbb" << endl;
        selected = -1;
        KonoeTclFMCanvas::refresh();
cout << "ccccccccccccccccc" << endl;
      }
    }

    clickedTime_old = clickedTime;
    clickedTime = time ;
  }

  // PopupWindow
  else if(type == KMouseRelease && button == RButton){
    if(selected != -1){
      deleteItem("selectItem");
    }
    checkClickedPoint(x, y);
    if(selected != -1){
      setColor("blue");
      drawRectangle( iconXY[selected]->getWidth()-18,
                     iconXY[selected]->getHeight()-18,
                     36, 36, "selectItem");
 
      KInt xx;
      KInt yy;
      xx = parentWindow->getRootX() + x;
      yy = parentWindow->getRootY() + y;
      kfmgr->setPopupType(KMenu);
      (kfmgr->getPopup())->setPopup(xx+10, yy);
      kfmgr->setFileStatus(kFileStatus[selected]);
    }
  }
}

void KonoeTclFMIconCanvas::mouseMotion(KonoeTclCanvas* kbt,  KInt type, 
                                       KInt state, Time time, KInt x, KInt y)
{
  static KInt startXp;
  static KInt startYp;
  static KInt mflag;
  KInt endXp;
  KInt endYp;

  KonoeTclDimension* realDim = toRealPosition(x, y);
  endXp = realDim->getWidth();
  endYp = realDim->getHeight();
  delete realDim;

  if(type ==  KMousePress){ //Not Drag & Drop
    startXp = endXp;
    startYp = endYp;
    selectedStartX = startXp;
    selectedStartY = startYp;
    if( abs(clickedTime_old - time) <= 300) mflag = 0; //Double Click
    else mflag = 1; //Single Click
  }
  if(type == KMouseRelease){
    selectedEndX = endXp;
    selectedEndY = endYp;
    mflag = 0;
    deleteItem("renge");
    setSelectedIconList(selectedStartX, selectedStartY,
                        selectedEndX, selectedEndY);
  }
  if(type == KMouseMove && mflag == 1){
    deleteItem("renge");
    setColor("red");
    drawRectangle(startXp, startYp, 
                  endXp-startXp, endYp-startYp, "renge");

    //Scroll To Bottom (Top)
    KonoeTclDimension dim = KonoeTclCanvas::getSize();
    if(y > dim.getHeight() - 5) KonoeTclCanvas::scrollOneUnitToBottom();
    else if(y < 5)               KonoeTclCanvas::scrollOneUnitToTop();
  }
}

void KonoeTclFMIconCanvas::windowAction(KonoeTclCanvas* kbt, XEvent* xe)
{
  if(xe->xdestroywindow.type != KDestroyWindow){
    if(repaint){
      repaint = false;
      KonoeTclFMCanvas::setCanvasWindowSize();
      KonoeTclFMCanvas::refresh();
    }
    else {
      repaint = true;
    }
  }
  else{
    KonoeTclFMCanvas::deleteIconImage();
    KonoeTclFMCanvas::deleteIconXY();
    KonoeTclFMCanvas::deleteFileName();
  }
}

