//=====================================================================
/*

    KonoeTclFMInfoPanel.cc

    Version 0.5.0   28-Oct-1999 T. Hosoi

*/
//====================================================================

#include <KonoeTclFMInfoPanel.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclFrame.hh>
#include <ui/KonoeTclLabel.hh>
#include <ui/KonoeTclButton.hh>
#include <KonoeTclFMManager.hh>
#include <system/KonoeFileStatus.hh>
#include <KonoeTclFMFileStatus.hh>
#include <system/KonoeTimeData.hh>

KonoeTclFMInfoPanel::KonoeTclFMInfoPanel(KonoeTclFMManager* kfmgr,
                                         KonoeTclWindow* kwin)
{
  subwin = new KonoeTclSubWindow(kwin, "fileinformationpanel",
                                 "File Information");
  //subwin->create(kwin, "fileinformationpanel");

  //setupInformation();

  okbutton = new KonoeTclButton(subwin, "okbutton_infopanel", "OK");
  //okbutton->create(subwin, "okbutton_infopanel");

  addMouseAction( (KonoeTclButton*) okbutton );
  okbutton->setEventProcessor( (KonoeTclEventProcessor*) this );
}

KonoeTclFMInfoPanel::~KonoeTclFMInfoPanel()
{
//    delete subwin;
//    delete frm;
 
    delete lname;
    delete lsize;
    delete lowner;
    delete lgroup;
    delete lpermission;
    delete ltype;
    delete lmodification;
    delete lstatuschange;

    delete rname;
    delete rsize;
    delete rowner;
    delete rgroup;
    delete rpermission;
    delete rtype;
    delete rmodification;
    delete rstatuschange;

    delete okbutton;
}

void KonoeTclFMInfoPanel::mouseAction( KonoeTclButton* kbtn, KInt type,
                      KInt button, Time time, KInt x, KInt y )
{ 
  if(type == KMouseRelease && button == LButton){
    subwin->destroy();
  }
}

void KonoeTclFMInfoPanel::setupInformation(KonoeFileStatus* kfs)
{
  frm = new KonoeTclFrame(subwin, "frm_infopanel");
//  frm->create(subwin, "frm_infopanel");

  lname = new KonoeTclLabel(frm, "lname", "Name",0,0);
//  lname->create(frm, "lname",0,0);
  lsize = new KonoeTclLabel(frm, "lsize", "Size",0,1);
//  lsize->create(frm, "lsize",0,1);
  lowner = new KonoeTclLabel(frm, "lowner", "Owner", 0,2);
//  lowner->create(frm, "lowner",0,2);
  lgroup = new KonoeTclLabel(frm, "lgroup", "Group",0,3);
//  lgroup->create(frm, "lgroup",0,3);
  lpermission = new KonoeTclLabel(frm, "lpermission", "Access Permissions",0,4);
//  lpermission->create(frm, "lpermission",0,4);
  ltype = new KonoeTclLabel(frm, "ltype", "Type", 0,5);
//  ltype->create(frm, "ltype",0,5);
  lmodification = new KonoeTclLabel(frm, "lmodification", "Last Modification",
                                    0,6);
//  lmodification->create(frm, "lmodification",0,6);
  lstatuschange = new KonoeTclLabel(frm, "lstatuschange", "Last Status Change",                                     0,7);
//  lstatuschange->create(frm, "lstatuschange",0,7);

  rname = new KonoeTclLabel(frm, "rname", getFileName(kfs),1,0);
//  rname->create(frm, "rname",1,0);
  rsize = new KonoeTclLabel(frm, "rsize", getSize(kfs),1,1);
//  rsize->create(frm, "rsize",1,1);
  rowner = new KonoeTclLabel(frm, "rowner", getOwnerName(kfs),1,2);
//  rowner->create(frm, "rowner",1,2);
  rgroup = new KonoeTclLabel(frm, "rgroup", getGroupName(kfs),1,3);
//  rgroup->create(frm, "rgroup",1,3);
  rpermission = new KonoeTclLabel(frm, "rpermission", getAccessPermission(kfs),
                                  1,4);
//  rpermission->create(frm, "rpermission",1,4);
  rtype = new KonoeTclLabel(frm, "rtype", getFileType(kfs),1,5);
//  rtype->create(frm, "rtype",1,5);
  rmodification = new KonoeTclLabel(frm, "rmodification",
                                    getLastDataModification(kfs),1,6);
//  rmodification->create(frm, "rmodification",1,6);
  rstatuschange = new KonoeTclLabel(frm, "rstatuschange", 
                                    getLastFileStatusChange(kfs),1,7);
//  rstatuschange->create(frm, "rstatuschange",1,7);

  frm->setFrame();
}

KChar* KonoeTclFMInfoPanel::getFileName(KonoeFileStatus* kfs)
{
  return kfs->getFileName();
}

KChar* KonoeTclFMInfoPanel::getSize(KonoeFileStatus* kfs)
{
  sprintf(fileSize, "%ld", kfs->getFileSize());
  return fileSize;
}

KChar* KonoeTclFMInfoPanel::getOwnerName(KonoeFileStatus* kfs)
{
  return kfs->getOwnerName();
}

KChar* KonoeTclFMInfoPanel::getGroupName(KonoeFileStatus* kfs)
{
  return kfs->getGroupName();
}

KChar* KonoeTclFMInfoPanel::getAccessPermission(KonoeFileStatus* kfs)
{
  KInt u_status = 0;
  KInt g_status = 0;
  KInt o_status = 0;

  if(kfs->userCanRead( ))      u_status+=4;
  if(kfs->userCanWrite( ))     u_status+=2;
  if(kfs->userCanExecute( ))   u_status+=1;
  if(kfs->groupCanRead( ))     g_status+=4;
  if(kfs->groupCanWrite( ))    g_status+=2;
  if(kfs->groupCanExecute( ))  g_status+=1;
  if(kfs->othersCanRead( ))    o_status+=4;
  if(kfs->othersCanWrite( ))   o_status+=2;
  if(kfs->othersCanExecute( )) o_status+=1;
  sprintf(accessPermission, "%ld%ld%ld", u_status, g_status, o_status);

  return accessPermission;
}

KChar* KonoeTclFMInfoPanel::getFileType(KonoeFileStatus* kfs)
{
  if(kfs->isDirectory( )) return "Directory";
  if(kfs->isRegularFile( )) return "RegularFile";
  if(kfs->isCharacterSpecial( )) return "CharacterSpecial";
  if(kfs->isBlockSpecial( )) return "BlockSpecial";
  if(kfs->isSocket( )) return "Socket";
  if(kfs->isFIFO( )) return "FIFO";
  if(kfs->isSymbolicLink( )) return "SymbolicLink";
  else return "??????????";
}

KChar* KonoeTclFMInfoPanel::getLastDataModification(KonoeFileStatus* kfs)
{
  KonoeTimeData* kt = kfs->getTimeOfLastDataModification( );
  sprintf(modTime, "%4ld/%2ld/%2ld %2ld:%2ld:%2ld",
          kt->getYear(), kt->getMonth(), kt->getDay(),
          kt->getHour(), kt->getMinute(), kt->getSecond() );
  return modTime;
}

KChar* KonoeTclFMInfoPanel::getLastFileStatusChange(KonoeFileStatus* kfs)
{
  KonoeTimeData* kt = kfs->getTimeOfLastFileSatusChange( );
  sprintf(changeTime, "%4ld/%2ld/%2ld %2ld:%2ld:%2ld",
          kt->getYear(), kt->getMonth(), kt->getDay(),
          kt->getHour(), kt->getMinute(), kt->getSecond() );
  return changeTime;
}
