//=====================================================================
/*

    KonoeTclFMModePanel.cc

    Version 0.5.0   26-Oct-1999 T. Hosoi

*/
//====================================================================

#include <KonoeTclFMModePanel.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclFrame.hh>
#include <ui/KonoeTclLabel.hh>
#include <ui/KonoeTclButton.hh>
#include <ui/KonoeTclCheckButton.hh>
#include <KonoeTclFMManager.hh>
#include <system/KonoeFileStatus.hh>

KonoeTclFMModePanel::KonoeTclFMModePanel(KonoeTclFMManager* _kfmgr,
                                         KonoeTclWindow* _kwin)
{
  subwin = new KonoeTclSubWindow(_kwin, "mp_subwin", "Change mode panel");
//  subwin->create(_kwin, "mp_subwin");

  setupUserPanel();
  setupGroupPanel();
  setupOtherPanel();

  okbutton = new KonoeTclButton(subwin, "okbutton_mp", "OK");
//  okbutton->create(subwin, "okbutton_mp");

  addMouseAction( (KonoeTclButton*) okbutton );
  okbutton->setEventProcessor( (KonoeTclEventProcessor*) this );
}

KonoeTclFMModePanel::~KonoeTclFMModePanel()
{
  delete okbutton;
  delete frm_u;
  delete frm_g;
  delete frm_o;
  delete lbl_u;
  delete lbl_g;
  delete lbl_o;
  delete chkbtn_ur;
  delete chkbtn_uw;
  delete chkbtn_ux;
  delete chkbtn_gr;
  delete chkbtn_gw;
  delete chkbtn_gx;
  delete chkbtn_or;
  delete chkbtn_ow;
  delete chkbtn_ox;
}

void KonoeTclFMModePanel::setupMode(KonoeFileStatus* kfs)
{
  KInt u_status = 0;
  KInt g_status = 0;
  KInt o_status = 0;

  if(kfs->userCanRead( ))      u_status+=4;
  if(kfs->userCanWrite( ))     u_status+=2;
  if(kfs->userCanExecute( ))   u_status+=1;
  if(kfs->groupCanRead( ))     g_status+=4;
  if(kfs->groupCanWrite( ))    g_status+=2;
  if(kfs->groupCanExecute( ))  g_status+=1;
  if(kfs->othersCanRead( ))    o_status+=4;
  if(kfs->othersCanWrite( ))   o_status+=2;
  if(kfs->othersCanExecute( )) o_status+=1;

  setMode(u_status, g_status, o_status);
}

void KonoeTclFMModePanel::mouseAction( KonoeTclButton* kbtn, KInt type,
                                       KInt button, Time time, KInt x, KInt y )
{
  if(type == KMouseRelease && button == LButton){
      subwin->destroy();
  }
}

void KonoeTclFMModePanel::setupUserPanel()
{
  frm_u = new KonoeTclFrame(subwin, "frm_u");
//  frm_u->create(subwin, "frm_u");

  lbl_u = new KonoeTclLabel(frm_u, "lbl_u", "User",0,0);
//  lbl_u->create(frm_u, "lbl_u",0,0);

  chkbtn_ur = new KonoeTclCheckButton(frm_u, "chkbtn_ur", "r", 0,1);
//  chkbtn_ur->create(frm_u, "chkbtn_ur",0,1);
  chkbtn_uw = new KonoeTclCheckButton(frm_u, "chkbtn_uw", "w", 1,1);
//  chkbtn_uw->create(frm_u, "chkbtn_uw",1,1);
  chkbtn_ux = new KonoeTclCheckButton(frm_u, "chkbtn_ux", "x",2,1);
//  chkbtn_ux->create(frm_u, "chkbtn_ux",2,1);

  frm_u->setFrame();
}

void KonoeTclFMModePanel::setupGroupPanel()
{
  frm_g = new KonoeTclFrame(subwin, "frm_g");
//  frm_g->create(subwin, "frm_g");

  lbl_g = new KonoeTclLabel(frm_g, "lbl_u", "Group",0,0);
//  lbl_g->create(frm_g, "lbl_u",0,0);

  chkbtn_gr = new KonoeTclCheckButton(frm_g, "chkbtn_gr", "r",0,1);
//  chkbtn_gr->create(frm_g, "chkbtn_gr",0,1);
  chkbtn_gw = new KonoeTclCheckButton(frm_g, "chkbtn_gw", "w",1,1);
//  chkbtn_gw->create(frm_g, "chkbtn_gw",1,1);

  chkbtn_gx = new KonoeTclCheckButton(frm_g, "chkbtn_gx", "x",2,1);
//  chkbtn_gx->create(frm_g, "chkbtn_gx",2,1);

  frm_g->setFrame();
}

void KonoeTclFMModePanel::setupOtherPanel()
{
  frm_o = new KonoeTclFrame(subwin, "frm_o");
//  frm_o->create(subwin, "frm_o");

  lbl_o = new KonoeTclLabel(frm_o, "lbl_o", "Other",0,0);
//  lbl_o->create(frm_o, "lbl_o",0,0);

  chkbtn_or = new KonoeTclCheckButton(frm_o, "chkbtn_or", "r",0,1);
//  chkbtn_or->create(frm_o, "chkbtn_or",0,1);
  chkbtn_ow = new KonoeTclCheckButton(frm_o, "chkbtn_ow", "w",1,1);
//  chkbtn_ow->create(frm_o, "chkbtn_ow",1,1);
  chkbtn_ox = new KonoeTclCheckButton(frm_o, "chkbtn_ox", "x",2,1);
//  chkbtn_ox->create(frm_o, "chkbtn_ox",2,1);

  frm_o->setFrame();
}

void KonoeTclFMModePanel::setMode(KInt u, KInt g, KInt o)
{
  if(u >= 4){
    chkbtn_ur->setSelect();
    u -= 4;
  }
  if(u >= 2){
    chkbtn_uw->setSelect();
    u -= 2;
  }
  if(u >= 1){
    chkbtn_ux->setSelect();
  }

  if(g >= 4){
    chkbtn_gr->setSelect();
    g -= 4;
  }
  if(g >= 2){
    chkbtn_gw->setSelect();
    g -= 2;
  }
  if(g >= 1){
    chkbtn_gx->setSelect();
  }

  if(o >= 4){
    chkbtn_or->setSelect();
    o -= 4;
  }
  if(o >= 2){
    chkbtn_ow->setSelect();
    o -= 2;
  }
  if(o >= 1){
    chkbtn_ox->setSelect();
  }
}
