//=====================================================================
/*

    KonoeTcl2DrawCanvas.cc

    Version 0.0.1   26-Nov-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTcl2DrawCanvas.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclPhysPlacement.hh>
#include <ui/KonoeTclSpaceManager.hh>
#include <ui/KonoeTclRotationMatrix.hh>
#include <ui/KonoeTclDrawableBox.hh>
#include <ui/KonoeTclDrawableHorizonLine.hh>
#include <ui/KonoeTclDrawableVerticalLine.hh>
#include <ui/KonoeTclDrawableXLine.hh>
#include <ui/KonoeTcl3DVector.hh>
#include <ui/KonoeTclTransform.hh>
#include <analysis/Konoe2DHistogram.hh>
#include <analysis/KonoeDataScale.hh>
#include <analysis/KonoeLinearDataScale.hh>

KonoeTcl2DrawCanvas::KonoeTcl2DrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                         Konoe2DHistogram* pHisto,
                         KonoeTclDimension dimension, KChar* pName )
  : KonoeTclCanvas(kwin, kname)
{
  dim = dimension;
  histogram = pHisto;
  strcpy(myName, pName);
  forceWireFrame = KFalse;
  scale = KTrue;

  yDataScale = 0;

  drawObjTotalx = histogram->getBinCountx();
  drawObjTotaly = histogram->getBinCounty();

  objTotal = drawObjTotalx*drawObjTotaly+1;

  myPhys = new KonoeTclPhysPlacement*[ objTotal ];

  for(KInt i=0; i<objTotal; i++)
    myPhys[i] = new KonoeTclPhysPlacement;

  barwidthx = dimension.getHeight() / histogram->getBinCountx();
  barwidthy = dimension.getWidth() / histogram->getBinCounty();

  setVerticalScale( new KonoeLinearDataScale(0.0, 100.0, 100) );

  createDrawParts();
}

KonoeTcl2DrawCanvas::KonoeTcl2DrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                       Konoe2DHistogram* pHisto,
                       KonoeTclDimension dimension, KChar* pName,
                       KonoeDataScale* pScale )
  : KonoeTclCanvas(kwin, kname)
{
  dim = dimension;
  histogram = pHisto;
  strcpy(myName, pName);
  forceWireFrame = KFalse;
  scale = KTrue;

  yDataScale = 0;

  drawObjTotalx =  histogram->getBinCountx();
  drawObjTotaly =  histogram->getBinCounty();

  objTotal = drawObjTotalx*drawObjTotaly+1;

  myPhys = new KonoeTclPhysPlacement*[ objTotal ];

  for(KInt i=0; i<objTotal; i++)
    myPhys[i] = new KonoeTclPhysPlacement;

  barwidthx = dimension.getHeight() / pHisto->getBinCountx();
  barwidthy = dimension.getWidth() / pHisto->getBinCounty();

  setVerticalScale( pScale );

  myXLine = new KonoeTclPhysPlacement();
  myYLine = new KonoeTclPhysPlacement();
  myZLine = new KonoeTclPhysPlacement();

  createDrawParts();
}

KonoeTcl2DrawCanvas::~KonoeTcl2DrawCanvas()
{
  for(KInt i=0; i < objTotal; i++){
    delete myPhys[i];
  }
  delete [] myPhys;
  delete yDataScale;
}

void KonoeTcl2DrawCanvas::setSpaceManager( KonoeTclSpaceManager* pSpace )
{
  spMan = pSpace;
  createDrawParts();
}

void KonoeTcl2DrawCanvas::setSize( KInt width, KInt height )
{
  KonoeTclCanvas::setSize( width, height );
  draw();
}

void KonoeTcl2DrawCanvas::setMagnification(KFloat value)
{
  spMan->setMagnification(value);
}

void KonoeTcl2DrawCanvas::setCenterEye(KInt eyeX, KInt eyeY)
{
  spMan->setCenterEye(eyeX, eyeY);
}

void KonoeTcl2DrawCanvas::setViewLotation(KFloat value)
{
  spMan->setViewLotation(value);
}

void KonoeTcl2DrawCanvas::setViewAngle(KFloat value)
{
  spMan->setViewAngle(value);
}

void KonoeTcl2DrawCanvas::draw()
{
  KInt a;

  KonoeTclCanvas::clearAll();

  if(forceWireFrame)
  for(a=drawObjTotalx*drawObjTotaly; a>=1; a--){
    myPhys[a]->setForceWireFrame();
    myPhys[a]->draw(this);
  }

  else
  for(a=drawObjTotalx*drawObjTotaly; a>=1; a--){
    myPhys[a]->unsetForceWireFrame();
    myPhys[a]->draw(this);
  }

  if(scale) drawScale();
  //setBackGround( "black" );
}

void KonoeTcl2DrawCanvas::drawScale()
{
  KonoeTclRotationMatrix rm;

  KonoeTclDrawableHorizonLine* aLine1 =
                    new KonoeTclDrawableHorizonLine( "aHorizonLine",
                                    histogram->getYscale()*magnificationX/2 );
  KonoeTclDrawableXLine* aLine2 =
                  new KonoeTclDrawableXLine( "aXLine",
                                  histogram->getXscale()*magnificationY/2);
  KonoeTclDrawableVerticalLine* aLine3 =
                  new KonoeTclDrawableVerticalLine( "aVerticalLine",
                                  yDataScale->getMaximum()*magnification/2 );

  KonoeTcl3DVector aVector1
                  ( histogram->getYscale()
                                       *magnificationX/2-(KFloat)barwidthy/2,                                          0,
                                      -(KFloat)barwidthx/2  );
  KonoeTcl3DVector aVector2
                  ( -(KFloat)barwidthy/2,
                                      0,
                                      histogram->getXscale()
                                       *magnificationY/2-(KFloat)barwidthy/2);
  KonoeTcl3DVector aVector3
                  ( -(KFloat)barwidthx/2, 
                                    yDataScale->getMaximum()*magnification/2,
                                      -(KFloat)barwidthy/2 );

  myXLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector1),
                                  aLine1, spMan );
  myYLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector2),
                                  aLine2, spMan);
  myZLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector3),
                                  aLine3, spMan);

  myXLine->replace ( KonoeTcl2Dimension ( ) );
      myXLine->setVisAtt( 255, 0, 0 );
  myYLine->replace ( KonoeTcl2Dimension ( ) );
      myYLine->setVisAtt( 0, 255, 0 );
  myZLine->replace ( KonoeTcl2Dimension ( ) );
      myZLine->setVisAtt( 0, 0, 255 );


  myXLine->draw(this);
  myYLine->draw(this);
  myZLine->draw(this);
  delete myXLine;
  delete myYLine;
  delete myZLine;

  KonoeTclDimension nowDim;
  nowDim = spMan->reLocation ( KonoeTcl3DVector( 0 ,0, 0 ) );
  setColor("white");
  drawText( nowDim.getWidth() - 25, nowDim.getHeight() + 15, "0" );

  nowDim =  spMan->reLocation ( KonoeTcl3DVector
                      ( histogram->getYscale()*magnificationX ,0, 0 ) );

  KChar XMaxValueStr[64];
  sprintf(XMaxValueStr, "%.1f", (KFloat)histogram->getYscale()) ;
  setColor("white");
  drawText( nowDim.getWidth() - 25, nowDim.getHeight() + 15,
              XMaxValueStr);

  nowDim =  spMan->reLocation ( KonoeTcl3DVector
                      ( 0 , 0, histogram->getXscale()*magnificationY ) );

  KChar YMaxValueStr[64];
  sprintf( YMaxValueStr, "%.1f", (KFloat)histogram->getXscale()) ;

  setColor("white");
  drawText( nowDim.getWidth() - 25, nowDim.getHeight() + 15,
              YMaxValueStr);

  nowDim =  spMan->reLocation ( KonoeTcl3DVector
                      ( 0 , yDataScale->getMaximum()*magnification, 1 ) );

  KChar ZMaxValueStr[64];
  sprintf(ZMaxValueStr, "%.1f", (KFloat)yDataScale->getMaximum()) ;

  setColor("white");
  drawText(nowDim.getWidth() - 25, nowDim.getHeight() + 15,
              ZMaxValueStr);

  KInt i;
  for( i  = 1; i <= 4; i ++){
    KonoeTcl3DVector xGuideVec4
         ( -(KFloat)barwidthx/2,
                             0,
                             -(KFloat)barwidthy/2 + histogram->getXscale()*1/4
                              *i*magnificationY );
    KonoeTclDrawableVerticalLine* xGuideLine4 =
                    new KonoeTclDrawableVerticalLine( "xGuideLine4", 3);
    KonoeTclPhysPlacement* xGuideLinePhys4 =
           new KonoeTclPhysPlacement ( KonoeTclTransform ( rm, xGuideVec4),
                                                       xGuideLine4, spMan);
    xGuideLinePhys4->replace ( KonoeTcl2Dimension() );
    xGuideLinePhys4->setVisAtt( 0, 255, 0 );
    xGuideLinePhys4->draw(this);

    delete xGuideLinePhys4;
  }

  for( i=1; i <= 4; i++){
    KonoeTcl3DVector yGuideVec5
         ( -(KFloat)barwidthx/2+histogram->getXscale()
                                  *1/4*i*magnificationX,
                             0,
                             -(KFloat)barwidthy/2 );
    KonoeTclDrawableVerticalLine*  yGuideLine5 =
                    new KonoeTclDrawableVerticalLine( "y GuideLine5", 3);
    KonoeTclPhysPlacement* yGuideLinePhys5 =
           new KonoeTclPhysPlacement (KonoeTclTransform(rm, yGuideVec5),
                                   yGuideLine5, spMan);
    yGuideLinePhys5->replace( KonoeTcl2Dimension() );
    yGuideLinePhys5->setVisAtt( 0, 255, 0 );
    yGuideLinePhys5->draw(this);

    delete yGuideLinePhys5;
  }

  for( i= 1; i<= 4; i++){
    KonoeTcl3DVector zyGuideVec6
         ( -(KFloat)barwidthy/2,
                            yDataScale->getMaximum()*1/4*i*magnification,
                            -(KFloat)barwidthx/2  );

    KonoeTclDrawableHorizonLine*  zyGuideLine6 =
                    new KonoeTclDrawableHorizonLine( "zy GuideLine6", 3);

    KonoeTclDrawableXLine*  zxGuideLine6 =
                    new KonoeTclDrawableXLine( "zx GuideLine6", 3);

    KonoeTclPhysPlacement*  zyGuideLinePhys6 =
           new KonoeTclPhysPlacement( KonoeTclTransform( rm, zyGuideVec6),
                                   zyGuideLine6, spMan);

    KonoeTclPhysPlacement*  zxGuideLinePhys6 =
           new KonoeTclPhysPlacement ( KonoeTclTransform( rm, zyGuideVec6),
                                    zxGuideLine6, spMan);

    zyGuideLinePhys6->replace ( KonoeTcl2Dimension ( ) );
    zyGuideLinePhys6->setVisAtt( 0, 255, 0 );
    zyGuideLinePhys6->draw(this);
    zxGuideLinePhys6->replace ( KonoeTcl2Dimension ( ) );
    zxGuideLinePhys6->setVisAtt( 0, 255, 0 );
    zxGuideLinePhys6->draw(this);

    delete zyGuideLinePhys6;
    delete zxGuideLinePhys6;
  }

}

void KonoeTcl2DrawCanvas::setVerticalScale( KonoeDataScale* pScale )
{
  if(yDataScale) delete yDataScale;
  yDataScale = pScale;
  magnification = (KFloat)dim.getHeight()/pScale->getMaximum();
  magnificationX = (KFloat)dim.getHeight()/histogram->getYscale();
  magnificationY = (KFloat)dim.getWidth()/histogram->getXscale();
}

void KonoeTcl2DrawCanvas::createDrawParts()
{
  KonoeTclRotationMatrix rm;
  KInt c = 1;

  for(KInt i=0; i<drawObjTotalx; i++){
    for(KInt j=0; j<drawObjTotaly; j++){

      KonoeTclDrawableBox* aBox
           = new KonoeTclDrawableBox ( "aBox", barwidthx/2,
                      histogram->getOne(i,j)*magnification/2 , barwidthy/2);

      KonoeTcl3DVector aVector4 
         ( i*barwidthx, histogram->getOne(i,j)*magnification/2, j*barwidthy);

      myPhys[c] =
           new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector4), 
                                     aBox, spMan );

      myPhys[c]->replace ( KonoeTcl2Dimension() );

      KInt RR;

      RR = (KInt)( 255*histogram->getOne(i,j)/yDataScale->getMaximum() );
      if ( RR >= 255 ) RR = 255;
      if ( RR <= 0   ) RR = 0;
      KInt BB = 255-RR;

      myPhys[c]->setVisAtt ( RR, 0, BB  );

      //c = c + 1;
      c++;

    }
  }
}

void KonoeTcl2DrawCanvas::setDataSource(Konoe2DHistogram* pHisto)
{
   delete histogram;
   histogram = pHisto;

   drawObjTotalx =  histogram->getBinCountx();
   drawObjTotaly =  histogram->getBinCounty();


   KInt i;

   if(myPhys != NULL){
     for(i=0; i<objTotal; i++){
       delete myPhys[i];
     }
     delete [] myPhys;
   }

   objTotal = drawObjTotalx*drawObjTotaly+1;

   myPhys = new KonoeTclPhysPlacement*[ objTotal ];
   myPhys[0] = new KonoeTclPhysPlacement;

   barwidthx = dim.getHeight() / pHisto->getBinCountx();
   barwidthy = dim.getWidth() / pHisto->getBinCounty();

   createDrawParts();
}

void KonoeTcl2DrawCanvas::setVisAtt(KInt n, KInt RR, KInt GG, KInt BB)
{
  if( RR < 0 ) RR = 0;
  if( GG < 0 ) GG = 0;
  if( BB < 0 ) BB = 0;
  if( 255 < RR ) RR = 255;
  if( 255 < GG ) GG = 255;
  if( 255 < BB ) BB = 255;
  myPhys[n+1]->setVisAtt(RR, GG, BB);
}

void KonoeTcl2DrawCanvas::update()
{
  KonoeTclCanvas::update();
}

void KonoeTcl2DrawCanvas::setForceWireFrame()
{
  forceWireFrame = KTrue;
}

void KonoeTcl2DrawCanvas::unsetForceWireFrame()
{
  forceWireFrame = KFalse;
}

void KonoeTcl2DrawCanvas::setScale()
{
  scale = KTrue;
}

void KonoeTcl2DrawCanvas::unsetScale()
{
  scale = KFalse;
}
