//=====================================================================
/*

    KonoeTcl2DrawCanvasController.cc

    Version 0.0.1   02-Dec-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTcl2DrawCanvasController.hh>
#include <ui/KonoeTcl2DrawCanvas.hh>
#include <ui/KonoeTclSpaceManager.hh>

#include <analysis/KonoeLinearDataScale.hh>

#include <iostream.h>
#include <stdlib.h>

KonoeTcl2DrawCanvasController::KonoeTcl2DrawCanvasController
                                             (KonoeTcl2DrawCanvas* pwin)
{
  KChar tmpstr[256];
  KonoeTclWindow* kwin = pwin->getTopLevelWindow();
  subWindow = new KonoeTclSubWindow(kwin, tmpnam(tmpstr), "Control Window");

  subWindow->setEventProcessor(this);
  addWindowAction(subWindow);

  dc2 = pwin;

  initWidget();

  if(dc2) writeProperty();
}

KonoeTcl2DrawCanvasController::~KonoeTcl2DrawCanvasController()
{
  delete viewLotationLabel;
  delete viewLotationSlider;
  delete viewAngleLabel;
  delete viewAngleSlider;

  delete verticalScaleLabel;
  delete verticalScaleEntry;

  delete viewingModeLabel;
  delete wireFrameRadio;
  delete surfaceRadio;

  delete targetPointLabel;
  delete xCoordLabel;
  delete xCoordEntry;
  delete yCoordLabel;
  delete yCoordEntry;

  delete magnificationLabel;
  delete magnificationEntry;

  delete displayAxesLabel;
  delete displayAxesYesRadio;
  delete displayAxesNoRadio;

  delete postscriptButton;
  delete postscriptEntry;
  delete setupButton;

  KInt i;
  for(i=0; i<FrameNum; i++)
    delete frame[i];
  delete [] frame;

  subWindow->destroy();
  delete subWindow;
}

void KonoeTcl2DrawCanvasController::initWidget()
{
  KInt i;
  KChar fname[256];
  frame = new KonoeTclFrame*[FrameNum];

  for( i=0; i<FrameNum; i++){
    sprintf(fname, "frame%d" ,i);
    frame[i] = new KonoeTclFrame(subWindow, fname, 0, i);
  }

  KInt fNum = 0;

//==================== Frame 1  ====================
  viewLotationLabel =
      new KonoeTclLabel(frame[fNum], "viewLotationLabel", "ViewLotation", 0, 0);
  viewLotationSlider =
        new KonoeTclSlider(frame[fNum], "viewLotationSlider2D", "", 1, 0);
  viewAngleLabel =
      new KonoeTclLabel(frame[0], "viewAngleLabel", "ViewAngle", 0, 1);
  viewAngleSlider =
        new KonoeTclSlider(frame[0], "viewAngleSlider2D", "", 1, 1);

  viewLotationSlider->setMinMax(0, 90);
  viewAngleSlider->setMinMax(0, 90);
  viewLotationSlider->setBigIncrement(1);
  viewAngleSlider->setBigIncrement(1);
  viewLotationSlider->setShowValue(KTrue);
  viewAngleSlider->setShowValue(KTrue);
  viewAngleSlider->setValue(45);
  viewLotationSlider->setValue(45);

  fNum++;


//==================== Frame 2 ====================
  verticalScaleLabel =
      new KonoeTclLabel(frame[fNum], "verticalScaleLabel",
                                     "Vertical Scale Max", 0, 0);
  verticalScaleEntry =
      new KonoeTclEntry(frame[fNum], "verticalScaleEntry2D", 1, 0);

  verticalScaleEntry->setBackGround("white");
  verticalScaleEntry->setSize(15);

  verticalScaleEntry->setEventProcessor(this);
  addKeyAction(verticalScaleEntry);

  fNum++;


//==================== Frame 3  ====================
  canvasSizeLabel =
      new KonoeTclLabel(frame[fNum], "canvasSizeLabel", "Canvas Size", 0, 0);
  canvasSizeLabel->setGridRowspan(2);
  canvasSizeLabel->setGridSticky("news");

  canvasSizeXEntry =
      new KonoeTclEntry(frame[fNum], "canvasSizeXEntry", 1, 0);
  canvasSizeYEntry =
      new KonoeTclEntry(frame[fNum], "canvasSizeYEntry", 2, 0);

  canvasSizeXEntry->setBackGround("white");
  canvasSizeYEntry->setBackGround("white");

  canvasSizeXEntry->setSize(10);
  canvasSizeYEntry->setSize(10);

  canvasSizeXEntry->setEventProcessor(this);
  addKeyAction(canvasSizeXEntry);
  canvasSizeYEntry->setEventProcessor(this);
  addKeyAction(canvasSizeYEntry);

  fNum++;


//==================== Frame 4  ====================
  viewingModeLabel =
      new KonoeTclLabel(frame[fNum], "viewingModeLabel", "Viewing Mode", 0, 0);
  viewingModeLabel->setGridRowspan(2);
  viewingModeLabel->setGridSticky("news");
  wireFrameRadio =
      new KonoeTclRadioButton(frame[fNum], "wireFrameRadio2D", "WireFrame",1,0);
  surfaceRadio=
      new KonoeTclRadioButton(frame[fNum], "surfaceRadio2D", "Surface", 2, 0);

  wireFrameRadio->setGroup("wframeview2d");
  surfaceRadio->setGroup("wframeview2d");
  surfaceRadio->setSelect();

  fNum++;


//==================== Frame 5  ====================
  targetPointLabel =
      new KonoeTclLabel(frame[fNum], "targetPointLabel",
                                     "Targetting Point", 0, 0);
  targetPointLabel->setGridRowspan(3);
  targetPointLabel->setGridSticky("news");

  xCoordLabel =
      new KonoeTclLabel(frame[fNum], "xCoordLabel", "X coord", 1, 0);
  xCoordEntry =
      new KonoeTclEntry(frame[fNum], "xCoordEntry2D", 2, 0);
  yCoordLabel =
      new KonoeTclLabel(frame[fNum], "yCoordLabel", "Y coord", 1, 1);
  yCoordEntry =
      new KonoeTclEntry(frame[fNum], "yCoordEntry2D", 2, 1);

  xCoordEntry->setBackGround("white");
  yCoordEntry->setBackGround("white");
  xCoordEntry->setSize(15);
  yCoordEntry->setSize(15);
  xCoordEntry->setText("450");
  yCoordEntry->setText("450");

  xCoordEntry->setEventProcessor(this);
  addKeyAction(xCoordEntry);
  yCoordEntry->setEventProcessor(this);
  addKeyAction(yCoordEntry);

  fNum++;


//==================== Frame 6  ====================
  magnificationLabel =
      new KonoeTclLabel(frame[fNum], "magnificationLabel", "Magnification",0,0);
  magnificationEntry =
      new KonoeTclEntry(frame[fNum], "magnificationEntry2D", 1, 0);

  magnificationEntry->setBackGround("white");
  magnificationEntry->setText("1.0");

  magnificationEntry->setEventProcessor(this);
  addKeyAction(magnificationEntry);

  fNum++;


//==================== Frame 7  ====================

  displayAxesLabel =
      new KonoeTclLabel(frame[fNum], "dispalyAxesLabel", "Display Axes", 0, 0);
  displayAxesYesRadio =
      new KonoeTclRadioButton(frame[fNum], "displayAxesYesRadio2D", "Yes",1, 0);
  displayAxesNoRadio =
      new KonoeTclRadioButton(frame[fNum], "displayAxesNoRadio2D", "No", 2, 0);

  displayAxesYesRadio->setGroup("dcaxes2d");
  displayAxesNoRadio->setGroup("dcaxes2d");

  displayAxesNoRadio->setSelect();

  fNum++;


//==================== Frame 8  ====================
  postscriptButton = new KonoeTclButton
                      (frame[fNum], "postscriptButton", "Save to ...", 0, 0);
  postscriptEntry = new KonoeTclEntry
                      (frame[fNum], "postscriptEntry2D", 1, 0);

  postscriptEntry->setBackGround("white");

  addMouseAction(postscriptButton);
  postscriptButton->setEventProcessor(this);

  postscriptEntry->setEventProcessor(this);
  addKeyAction(postscriptEntry);

  fNum++;


//==================== Frame 9  ====================
  setupButton = new KonoeTclButton(frame[fNum], "setupButton", "Setup", 0, 0);
  addMouseAction(setupButton);
  setupButton->setEventProcessor(this);

  fNum++;


  for( i=0; i<fNum; i++){
    frame[i]->setBorderWidth(1);
    frame[i]->setRelief(KGroove);
    frame[i]->setGridSticky("news");
    frame[i]->setGridIpady(5);
  }

}

void KonoeTcl2DrawCanvasController::writeProperty()
{
  KChar value[256];
  KonoeTclDimension dim;
  KInt v;
  KonoeTclSpaceManager* spMan = dc2->getSpaceManager();

//==================== 1  ====================
  viewAngleSlider->setValue( spMan->getViewAngle()*180./M_PI );
  viewLotationSlider->setValue( spMan->getViewLotation()*180./M_PI );

//==================== 2  ====================
  sprintf( value, "%.lf", dc2->getVerticalScale()->getMaximum() );
  verticalScaleEntry->setText( value );

//==================== 3  ====================
  if( dc2->isForceWire() )
    wireFrameRadio->setSelect();
  else
    surfaceRadio->setSelect();

//==================== 4  ====================
  dim = dc2->getSize();
  sprintf(value, "%d", dim.getWidth());
  canvasSizeXEntry->setText(value);
  sprintf(value, "%d", dim.getHeight());
  canvasSizeYEntry->setText(value);

//==================== 5  ====================
  sprintf( value, "%ld", spMan->getCenterEyeX() );
  xCoordEntry->setText( value );
  sprintf( value, "%ld", spMan->getCenterEyeY() );
  yCoordEntry->setText( value );

//==================== 6  ====================
  sprintf( value, "%.1lf", spMan->getMagnification() );
  magnificationEntry->setText( value );

//==================== 7  ====================
  if( dc2->isScale() )
    displayAxesYesRadio->setSelect();
  else
    displayAxesNoRadio->setSelect();

//==================== 8  ====================
  postscriptEntry->setText("filename.ps");
}


void KonoeTcl2DrawCanvasController::mouseAction
       (KonoeTclButton* bt, KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
  if( 0<=x && x<bt->getWidth() && 0<=y && y<=bt->getHeight()){
    if(strcmp(bt->getName(), "setupButton") == 0)
      setButtonAction();

    if(strcmp(bt->getName(), "postscriptButton") == 0)
      postscriptAction();
  }
  }
}


void KonoeTcl2DrawCanvasController::setButtonAction()
{
  KonoeTclSpaceManager* spMan = dc2->getSpaceManager();
  KChar viewLotation[16];
  KChar viewAngle[16];
  KChar verticalScale[16];
  KChar magnification[16];
  KChar eyeX[16];
  KChar eyeY[16];

  sprintf(viewLotation, "%d", viewLotationSlider->getIndex());
  sprintf(viewAngle, "%d", viewAngleSlider->getIndex());
  strcpy(verticalScale, verticalScaleEntry->getText());
  strcpy(magnification, magnificationEntry->getText());
  strcpy(eyeX, xCoordEntry->getText());
  strcpy(eyeY, yCoordEntry->getText());

  spMan->setViewLotation(atof(viewLotation)*DEG);
  spMan->setViewAngle(atof(viewAngle)*DEG);

  KFloat max = atof(verticalScale);
  KInt   bin = atoi(verticalScale);

  if( max < 0. ) max = 0.;
  if( bin < 1 )  bin = 1; 

  dc2->setVerticalScale( new KonoeLinearDataScale(0.0, max, bin) );


  KFloat mag = atof(magnification);
  if( mag < 0. ) mag = 1.;
  spMan->setMagnification( mag );

  KInt x = atoi(eyeX);
  KInt y = atoi(eyeY);

  if(x < 0) x = 0;
  if(y < 0) y = 0;

  spMan->setCenterEye(x, y);

  if(strcmp("wireFrameRadio2D",
             wireFrameRadio->getValue()) == 0) dc2->setForceWireFrame();
  else dc2->unsetForceWireFrame();

  if(strcmp("displayAxesYesRadio2D",
             displayAxesYesRadio->getValue()) == 0) dc2->setScale();
  else dc2->unsetScale();

  x = atoi(canvasSizeXEntry->getText());
  y = atoi(canvasSizeYEntry->getText());
  if( 0<x && 0<y )
  dc2->setSize( x, y );
 
  dc2->draw();
  dc2->update();
}


void KonoeTcl2DrawCanvasController::postscriptAction()
{
  dc2->toPostScript( postscriptEntry->getText() );
}


void KonoeTcl2DrawCanvasController::keyAction(KonoeTclEntry* kbt, KInt type,
                                                    KeySym keysym, KInt state)
{
  if(type == KKeyRelease){
  if(keysym == XK_KP_Enter || keysym == XK_Return){
    if(strcmp(kbt->getName(), "postscriptEntry") == 0)
      postscriptAction();
    else
      setButtonAction();
  }
  }
}

void KonoeTcl2DrawCanvasController::windowAction
                            (KonoeTclWindow* kcv, XEvent* xev)
{
  if(xev->type == KDestroyWindow){
//    this->~KonoeTcl2DrawCanvasController();
  }
}
