//=====================================================================
/*

    KonoeTclCalculator.cc

    Version 0.0.1   09-Nov-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclCalculator.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclScale.hh>
#include <ui/KonoeTclScaleVertical.hh>
#include <ui/KonoeTclDimension.hh>
#include <stdlib.h>
#include <math.h>

KonoeTclCalculator::KonoeTclCalculator()
{
  ;
}

KonoeTclCalculator::~KonoeTclCalculator()
{
  ;
}

KFloat KonoeTclCalculator::getMax(KonoeTclDrawCanvas* canv)
{
  KInt bin;
  bin = canv->getHistogram()->getBinCount();
  KFloat* data = new KFloat[bin];

  KInt i;
  for(i=0; i<bin; i++){
    data[i] = canv->getHistogram()->getOne(i);
  }

  KFloat maxData = 0.;

  for(i=0; i<bin; i++){
    if(maxData < data[i])
      maxData = data[i];
  }

  delete [] data;

  return maxData;
}

KInt KonoeTclCalculator::getMaxBinNumber(KonoeTclDrawCanvas* canv)
{
  KInt bin;
  bin = canv->getHistogram()->getBinCount();
  KFloat* data = new KFloat[bin];

  KInt i;
  for(i=0; i<bin; i++){
    data[i] = canv->getHistogram()->getOne(i);
  }

  KFloat maxData = 0.;

  KInt maxIndex;

  for(i=0; i<bin; i++){
    if(maxData < data[i]){
      maxData = data[i];
      maxIndex = i;
    }
  }

  delete [] data;

  return maxIndex;
}

KFloat KonoeTclCalculator::calcMean
                    (KonoeTclDrawCanvas* canv, KInt startX, KInt endX)
{
  KonoeTclDimension d1 = getClickPoint(canv, startX, 0);
  KonoeTclDimension d2 = getClickPoint(canv, endX, 0);
  KonoeTclDimension dim;

  binCount = abs(d1.getWidth() - d2.getWidth());

  if(d1.getWidth() < d2.getWidth())
    dim = d1;
  else
    dim = d2;

  KFloat* data = new KFloat[binCount];
  ///binData = new KInt[binCount];

  KInt i;
  KInt j;

  for(i=0, j=dim.getWidth(); i<binCount; i++, j++)
    data[i] = canv->getHistogram()->getOne(j);

  totalData = 0.;
  for(j=dim.getWidth(); j<dim.getWidth()+binCount; j++)
    totalData += canv->getHistogram()->getOne(j);

  KFloat mean = 0.;
  for(i=0; i<binCount; i++)
    mean += data[i]*i;

  mean /= totalData;
  mean += dim.getWidth() + canv->getHistogram()->getScale()->getMinimum();

/*********************
  for(i=0, j=dim.getWidth(); i<binCount; i++, j++){
    data[i] = canv->getHistogram()->getOne(j);
    binData[i] = j;
  }

  totalData = 0.;
  for(i=0, j=dim.getWidth(); i<canv->getBinCount(); i++, j++)
    totalData += canv->getHistogram()->getOne(i);

  KFloat mean = 0.;
  for(i=0; i<binCount; i++)
    mean += data[i]*binData[i];

  mean /= totalData;
************************/

  delete [] data;

  return mean;
}

KFloat KonoeTclCalculator::calcVariance(KonoeTclDrawCanvas* canv,
                                                 KInt startX, KInt endX)
{
  KFloat mean = calcMean(canv, startX, endX);
  KInt sX;

  KFloat* data = new KFloat[binCount];

  if( startX < endX )
    sX = getClickPoint(canv, startX, 0).getWidth();
  else
    sX = getClickPoint(canv, endX, 0).getWidth();

  KFloat vX = sX + canv->getHistogram()->getScale()->getMinimum();

  KInt i;
  KInt j;
  KFloat variance = 0.;
  KFloat t;

  for(i=0, j=sX; i<binCount; i++, j++)
    data[i] = canv->getHistogram()->getOne(j);

  for(i=0; i<binCount; i++)
  {
    variance += data[i]*i*i;
  }

  variance /= totalData;
  variance -= (mean-vX)*(mean-vX);

/*************************
  for(i=0; i<binCount; i++)
  {
    //t = data[i]*binData[i] - mean;
    //t = data[i]*j*j;
    variance += data[i]*binData[i]*binData[i];
  }

  variance /= totalData;
  variance -= mean*mean;
***********************/

  delete [] data;

  return variance;
}

KFloat KonoeTclCalculator::calcStandardDeviation(KonoeTclDrawCanvas* canv,
                                                    KInt startX, KInt endX)
{
  KFloat variance = calcVariance(canv, startX, endX);

  return sqrt(variance);
}

KInt KonoeTclCalculator::calcBinTotal(KonoeTclDrawCanvas* canv,
                                                    KInt startX, KInt endX)
{
  KInt sX = getClickPoint(canv, startX, 0).getWidth();
  KInt eX = getClickPoint(canv, endX, 0).getWidth();
  
  KInt bin = abs(sX - eX);

  KInt tmpX;
  if( eX < sX ){
    tmpX = sX;
    sX = eX;
    eX = tmpX;
  } 

  KFloat totalData = 0.;

  KInt i;
  for(i=sX; i<sX+bin; i++){
    totalData += canv->getHistogram()->getOne(i);
  }

  return (KInt)(totalData+0.5);
}

KonoeTclDimension KonoeTclCalculator::
                   getClickPoint(KonoeTclDrawCanvas* drawCanvas, KInt x, KInt y)
{
  KInt origX = drawCanvas->getDimension().getOrigX();
  KInt width = drawCanvas->getBinCount();
  KInt realWidth = drawCanvas->getScale()->getDimension().getWidth();
  KInt realX = width*(x-origX)/realWidth;

  KInt origY = drawCanvas->getDimension().getOrigY();
  KInt realY = (KInt)drawCanvas->getHistogram()->getOne(realX);

  if(realX >= width) return KonoeTclDimension(drawCanvas->getBinCount(), 0);
  else if(realX <= 0) return KonoeTclDimension(0, 0);

  return KonoeTclDimension(realX, realY);
}
