//=====================================================================
/*

    KonoeTclDrawCanvas.cc

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <unistd.h>
#include <iostream.h>
#include <ui/KonoeTclDrawCanvas.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/KonoeDataScale.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/Konoe1DHistogramBand.hh>
#include <system/KonoeOutputObjFile.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclDrawableRect.hh>
#include <ui/KonoeTclScale.hh>
#include <ui/KonoeTclScaleVertical.hh>
#include <ui/KonoeTcl1DHistogram.hh>
#include <ui/KonoeTclCalculator.hh>

KonoeTclDrawCanvas::KonoeTclDrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                                        Konoe1DHistogram* pHisto,
                                        KonoeTclDimension dimension,
                                        KChar* pName )
  : KonoeTclCanvas(kwin, kname)
{
  isBox = KTrue;
  isFill = KFalse;
  isLine = KFalse;
  createdWindow = KFalse;
  autoVertical = KTrue;
  isStatus = KTrue;
  size = dimension;
  histogram = pHisto;
  KonoeLinearDataScale* scale =
       new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                 histogram->getScale()->getMaximum(),
                                 histogram->getScale()->getBinCount() );
  Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
  defaultHistogram = band->getNewHisto( histogram );
  strcpy(myName, pName);
  strcpy(histoColor, "black");

  drawObjTotal = histogram->getBinCount();
  yDataScale = 0;

  setVerticalScale( new KonoeLinearDataScale( 0.0, 100.0, 100 ) );

  KInt i;
  drawRect = new KonoeTclDrawableRect*[drawObjTotal];
  for(i = 0; i < drawObjTotal; i++){
    drawRect[i] = new KonoeTclDrawableRect();
  }

  xScale = 0;
  yScale = 0;
  xPos = 0;
  yPos = 0;

  xMin = 0;
  xMax = 0;
  xBin = 0;
  xMark = 0;
  xBigMark = 0;

  createDrawParts();
  setSize(size.getWidth(), size.getHeight());

  draw();
}

KonoeTclDrawCanvas::KonoeTclDrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                                        Konoe1DHistogram* pHisto,
                                        KonoeTclDimension dimension,
                                        KChar* pName, KonoeDataScale* pScale )
  : KonoeTclCanvas(kwin, kname)
{
  isBox = KTrue;
  isFill = KFalse;
  isLine = KFalse;
  createdWindow = KFalse;
  autoVertical = KTrue;
  isStatus = KTrue;
  size = dimension;
  histogram = pHisto;
  KonoeLinearDataScale* scale =
       new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                 histogram->getScale()->getMaximum(),
                                 histogram->getScale()->getBinCount() );
  Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
  defaultHistogram = band->getNewHisto( histogram );
  strcpy(myName, pName);
  strcpy(histoColor, "black");

  drawObjTotal = histogram->getBinCount();
  yDataScale = 0;

  setVerticalScale( pScale );

  KInt i;
  drawRect = new KonoeTclDrawableRect*[drawObjTotal];
  for(i = 0; i < drawObjTotal; i++){
    drawRect[i] = new KonoeTclDrawableRect();
  }

  xScale = 0;
  yScale = 0;
  xPos = 0;
  yPos = 0;

  xMin = 0;
  xMax = 0;
  xBin = 0;
  xMark = 0;
  xBigMark = 0;

  createDrawParts();
  setSize(size.getWidth(), size.getHeight());

  draw();
}

KonoeTclDrawCanvas::~KonoeTclDrawCanvas()
{
//  drawObjTotal = histogram->getBinCount();
  KInt i;
  for(i=0; i<drawObjTotal; i++) delete drawRect[i];
  delete [] drawRect;
  delete xScale;
  delete yScale;
  delete yDataScale;
  if(histogram!=NULL)
    delete histogram;
  delete defaultHistogram;
}

void KonoeTclDrawCanvas::setDimension( KonoeTclDimension dimension )
{
  size = dimension;

  createDrawParts();
}

void KonoeTclDrawCanvas::setSize( KInt width, KInt height )
{
  KonoeTclCanvas::setSize( width, height );
//  draw();
}

KInt KonoeTclDrawCanvas::setDataSource( Konoe1DHistogram* pHisto )
{
  if( histogram ){
    delete histogram;
    histogram = 0;
  }
  histogram = pHisto;

  if( !histogram )  return 0;

  if( defaultHistogram->getScale()->getBinCount() <=
                        histogram->getScale()->getBinCount() ){
    delete defaultHistogram;
    KonoeLinearDataScale* scale =
         new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                   histogram->getScale()->getMaximum(),
                                   histogram->getScale()->getBinCount() );
    Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
    defaultHistogram = band->getNewHisto( histogram );
  }

  KInt	i;
  if(drawRect != NULL){ 
    for(i=0; i<drawObjTotal; i++){
      delete drawRect[i];
    } 
    delete drawRect;
  }

  drawObjTotal = histogram->getBinCount();

  drawRect = new KonoeTclDrawableRect*[drawObjTotal];
  for(i=0; i<drawObjTotal; i++){
      drawRect[i] = NULL;
  } 

  for(i = 0; i < drawObjTotal; i++ ){
    drawRect[i] = new KonoeTclDrawableRect();
  }

  KFloat maxData=0.;

  for(i=0; i<histogram->getBinCount(); i++){
    if(maxData < histogram->getOne(i))
      maxData = histogram->getOne(i);
  }

  if(autoVertical)
  if(maxData > yDataScale->getMaximum()){
    KFloat min = yDataScale->getMinimum();
    KFloat max = yDataScale->getMaximum();
    KInt bin = yDataScale->getBinCount();
    delete yDataScale;
    yDataScale = new KonoeLinearDataScale(min, max*2., bin);
  }

  createDrawParts();

  return 1;
}

void KonoeTclDrawCanvas::setHorizontalScale(KInt min, KInt max, KInt bin)
{
  xMin = min;
  xMax = max;
  xBin = bin;
}

void KonoeTclDrawCanvas::setMark(KInt mark)
{
  xMark = mark;
}

void KonoeTclDrawCanvas::setBigMark(KInt bigMark)
{
  xBigMark = bigMark;
}

void KonoeTclDrawCanvas::setVerticalScale( KonoeDataScale* pScale )
{
  if(yDataScale) delete yDataScale;
  yDataScale = pScale;
}

void KonoeTclDrawCanvas::setAutoVertical()
{
  if( autoVertical ) autoVertical = KFalse;
  else               autoVertical = KTrue;
}

void KonoeTclDrawCanvas::setStatus()
{
  if( isStatus ) isStatus = KFalse;
  if( isStatus ) isStatus = KTrue;
}

void KonoeTclDrawCanvas::createDrawParts()
{
  delete xScale;
  delete yScale;
  xScale = new KonoeTclScale( histogram->getScale(), size);
  yScale = new KonoeTclScaleVertical( yDataScale, size );

  KInt i;

  delete [] xPos;
  delete [] yPos;
  xPos = new KInt[drawObjTotal*2+2];
  yPos = new KInt[drawObjTotal*2+2];
  xPos[0] = size.getOrigX();
  yPos[0] = size.getOrigY()+size.getHeight();

  for ( i = 0; i < drawObjTotal; i++ ) {
    drawRect[i]->setInfo( yScale->getCoordinate(histogram->getOne(i)) );
    drawRect[i]->setPlacement( size.getOrigX()+i*size.getWidth()/drawObjTotal,
                               size.getHeight() + size.getOrigY(),
                               size.getWidth()/drawObjTotal );

    KInt j = i*2;
    xPos[j+1] = size.getOrigX()+i*size.getWidth()/drawObjTotal;
    xPos[j+2] = xPos[j+1] + size.getWidth()/drawObjTotal;
    yPos[j+1] = size.getHeight() + size.getOrigY()
                         - yScale->getCoordinate(histogram->getOne(i));
    yPos[j+2] = yPos[j+1];
  }
  xPos[drawObjTotal*2+1] = size.getOrigX()+size.getWidth();
  yPos[drawObjTotal*2+1] = yPos[0];
}

KInt KonoeTclDrawCanvas::getBinWidth()
{
  //return (xScale->getMaximum() - xScale->getMinimum())/(KFloat)drawObjTotal;
  return (KInt)(size.getWidth()/(KFloat)drawObjTotal/2. + 0.5);
}

void KonoeTclDrawCanvas::setHistoColor(KChar* hcol)
{
  strcpy(histoColor, hcol);
}

void KonoeTclDrawCanvas::setHistogramName(KChar* pName)
{
  strcpy(myName, pName);
}

void KonoeTclDrawCanvas::enableBox()
{
  isBox = KTrue;
  isFill = KFalse;
  isLine = KFalse;
}

void KonoeTclDrawCanvas::enableFill()
{
  isBox = KFalse;
  isFill = KTrue;
  isLine = KFalse;
}

void KonoeTclDrawCanvas::enableLine()
{
  isBox = KFalse;
  isFill = KFalse;
  isLine = KTrue;
}

void KonoeTclDrawCanvas::toFile(KChar* fName)
{
  KonoeOutputObjFile* oFile = new KonoeOutputObjFile( fName );
  oFile->put( defaultHistogram );
  delete oFile;
}

void KonoeTclDrawCanvas::draw()
{
  this->clearAll();
  KonoeTclGraphics* g;
  g = this;


  if( 0 < xMax && 0 < xBin ){
    KonoeLinearDataScale* scale =
        new KonoeLinearDataScale( xMin, xMax, xBin );
    Konoe1DHistogramBand* band =
        new Konoe1DHistogramBand( scale );
    Konoe1DHistogram* hist = band->getNewHisto(defaultHistogram);
    this->setDataSource(hist);
  } 

  g->setColor(this->getHistoColor());
  KInt i;
  if(isBox){
    for( i = 0; i < drawObjTotal; i++) drawRect[i]->drawMe( g );
  }

  else if(isFill){
    for( i = 0; i < drawObjTotal; i++) drawRect[i]->drawMe2( g );
    setColor("black");
    if( histogram->getScale()->getBinCount()*2 <= size.getWidth() )
      drawPolygon(xPos, yPos, drawObjTotal*2+2);
  }

  else if(isLine){
    drawByLine();
  }

  g->setColor("black");
  g->drawRectangle( size.getOrigX(), size.getOrigY(),
                    size.getWidth(), size.getHeight() );
  //g->setFont("....");
  g->drawText( size.getWidth()/2 + size.getOrigX(),
                 size.getHeight() + size.getOrigY() + 40, myName);

  if( 0 < xMark && 0 < xBigMark ){
    xScale->setMarkTotal( xMark );
    xScale->setBigMarkTotal( xBigMark );
  }
  xScale->drawMe(g);
  yScale->drawMe(g);

  if( isStatus )
    drawStatus();
}

void KonoeTclDrawCanvas::drawByLine()
{
  KonoeTclGraphics* g;
  g = this;

  g->setColor(this->getHistoColor());
  histoData = new KInt[drawObjTotal];

  if( isStatus )
    drawStatus();

  KInt i;
  for(i=0; i<drawObjTotal; i++)
    histoData[i] = (KInt)histogram->getOne(i);

  KInt xpos1;
  KInt ypos1;
  KInt xpos2;
  KInt ypos2;
  for(i=0; i<drawObjTotal-1; i++){
    xpos1 = size.getOrigX() + i*size.getWidth()/drawObjTotal + getBinWidth();
    ypos1 = size.getOrigY() + size.getHeight()
                             - yScale->getCoordinate(histoData[i]);
    xpos2 = xpos1 + size.getWidth()/drawObjTotal;
    ypos2 = size.getOrigY() + size.getHeight()
                             - yScale->getCoordinate(histoData[i+1]);
    g->drawLine(xpos1, ypos1, xpos2, ypos2);
  }
  delete [] histoData;
}

void KonoeTclDrawCanvas::drawStatus()
{
  KonoeTclGraphics* g;
  g = this;

  g->setColor("black");

  if( 150 < size.getWidth() && 85 < size.getHeight() ){
    KonoeTclCalculator calc;
    KInt binTotal = calc.calcBinTotal( this, 0, 4096 ); //getWidth() );
    KInt max = (KInt)calc.getMax( this );
    KInt maxBinNum = calc.getMaxBinNumber( this );
    KFloat mean = calc.calcMean( this, 0, 4096 ); //getWidth() );
    KFloat variance = calc.calcVariance( this, 0, 4096 ); //getWidth() );
    KFloat sigma = calc.calcStandardDeviation( this, 0, 4096 ); //getWidth() );

    KInt startX = size.getOrigX() + size.getWidth() - 150;
    KInt startY = size.getOrigY();
    KChar value[256];

    g->drawRectangle( startX, startY, 150, 85 );

    sprintf(value, "  Entry");
    g->drawString( startX, startY+5, value );
    sprintf(value, "%d", binTotal);
    g->drawString( startX+70, startY+5, value );

    sprintf(value, "  Max \\[bin\\]");
    g->drawString( startX, startY+20, value );
    sprintf(value, "%d \\[%d\\]", max, maxBinNum);
    g->drawString( startX+70, startY+20, value );

    sprintf(value, "  Mean");
    g->drawString( startX, startY+35, value );
    sprintf(value, "%.2lf", mean);
    g->drawString( startX+70, startY+35, value );

    sprintf(value, "  Variance");
    g->drawString( startX, startY+50, value );
    sprintf(value, "%.2lf", variance);
    g->drawString( startX+70, startY+50, value );

    sprintf(value, "  Sigma");
    g->drawString( startX, startY+65, value );
    sprintf(value, "%.2lf", sigma);
    g->drawString( startX+70, startY+65, value );
  }
}
