//=====================================================================
/*

    KonoeTclDrawCanvas.hh

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLDRAWCANVAS_HH
#define __KONOETCLDRAWCANVAS_HH

#include <ui/KonoeTclCanvas.hh>

#include <analysis/Konoe1DHistogram.hh>
#include <ui/KonoeTclScaleVertical.hh>
class KonoeDataScale;
class KonoeTclWindow;
class KonoeTclDimension;
class KonoeTclGraphics;
class KonoeTclDrawableRect;
//class KonoeTclScale;
//class KonoeTclScaleVertical;

class KonoeTclDrawCanvas : public KonoeTclCanvas {

  public:
    KonoeTclDrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                        Konoe1DHistogram* pHisto, 
                        KonoeTclDimension dimension,
                        KChar* pName );
    KonoeTclDrawCanvas( KonoeTclWindow* kwin, KChar* kname,
                        Konoe1DHistogram* pHisto, 
                        KonoeTclDimension dimension,
                        KChar* pName, KonoeDataScale* pScale );
    ~KonoeTclDrawCanvas();

    void setVerticalScale( KonoeDataScale* pScale );
    void setDimension( KonoeTclDimension dimension );
    void setSize( KInt width, KInt height ); //canvas size
    KInt setDataSource( Konoe1DHistogram* pHisto );
    Konoe1DHistogram* getHistogram() { return histogram; }
    Konoe1DHistogram* getDefaultHistogram() { return defaultHistogram; }
    KInt getBinCount() { return drawObjTotal; }
    KInt getBinWidth();
    KonoeTclDimension getDimension() { return size; } //draw rect size
    KonoeTclScaleVertical* getVerticalScale() { return yScale; }
    KonoeTclScale* getScale() { return xScale; }
    KonoeDataScale* getDataScale() { return yDataScale; }
    void draw();
    void drawStatus();
    void enableBox();
    void enableFill();
    void enableLine();
    KChar* getHistoColor() { return histoColor; }
    KChar* getTitleName() { return myName; }
    void setHistoColor(KChar* hcol);
    void setHistogramName(KChar* pName);

    void setHorizontalScale(KInt min, KInt max, KInt bin);
    void setMark(KInt mark);
    void setBigMark(KInt bigMark);
    void setAutoVertical();
    void setStatus();
    KBool isCreated() { return createdWindow; }
    void setCreated() { createdWindow = KTrue; }

    void toFile(KChar* fName);

  private:
    void drawByLine();
    void createDrawParts();
    void clearVerticalLine();
    Konoe1DHistogram* defaultHistogram;
    Konoe1DHistogram* histogram;
    KonoeTclDimension size;
    KonoeTclDrawableRect** drawRect;
    KInt drawObjTotal;
    KChar myName[64];
    KonoeTclScale* xScale;
    KonoeTclScaleVertical* yScale;
    KonoeDataScale* yDataScale;
    KInt* histoData;
    KBool isBox;
    KBool isFill;
    KBool isLine;
    KChar histoColor[256];
    KInt* xPos;
    KInt* yPos;

    KBool createdWindow;
    KInt xMin;
    KInt xMax;
    KInt xBin;
    KInt xMark;
    KInt xBigMark;
    KBool autoVertical;
    KBool isStatus;
};

#endif
