//=====================================================================
/*

    KonoeTclDrawCanvasController.cc

    Version 0.0.1   07-Nov-2000 T. Hosoi

*/
//====================================================================

#include "KonoeTclDrawCanvasController.hh"
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclScaleVertical.hh>
#include <analysis/KonoeLinearDataScale.hh>
#include <analysis/Konoe1DHistogramBand.hh>

#include <stdlib.h>

KonoeTclDrawCanvasController::KonoeTclDrawCanvasController(
                                          KonoeTclDrawCanvas* canv)
{
  KInt i;
  for(i=0; i<256; i++)
    fullPath[i] = '\0';

  KonoeTclWindow* kwin = canv->getTopLevelWindow();
  subWindow = new KonoeTclSubWindow(kwin, "controlWindow", "Control Window");
  defaultFrame = new KonoeTclFrame(subWindow, "defaultFrame");
  drawCanvas = canv;

  createParts();
  defaultFrame->setFrame();

  if(drawCanvas != NULL)
    writeProperty();
}

KonoeTclDrawCanvasController::KonoeTclDrawCanvasController(
                                          KonoeTclWindow* kwin)
{
  KInt i;
  for(i=0; i<256; i++)
    fullPath[i] = '\0';

  subWindow = new KonoeTclSubWindow(kwin, "hControlWindow", "Control Window");
  defaultFrame = new KonoeTclFrame(subWindow, "defaultFrame");
  drawCanvas = 0;

  createParts();
  defaultFrame->setFrame();
}

KonoeTclDrawCanvasController::~KonoeTclDrawCanvasController()
{
    delete verticalScaleLabel;
    delete verticalMaxLabel;
    delete horizontalScaleLabel;
    delete horizontalMinLabel;
    delete horizontalMaxLabel;
    delete horizontalBinCountLabel;
    delete histCanvasSizeLabel;
    delete horizontalMarkTotalLabel;
    delete horizontalBigMarkTotalLabel;
    delete verticalMarkTotalLabel;
    delete verticalBigMarkTotalLabel;
    delete drawTypeBoxLabel;
    delete setHistColorLabel;

    delete verticalScaleEntry;
    delete horizontalMinEntry;
    delete horizontalMaxEntry;
    delete horizontalBinCountEntry;
    delete histCanvasSizeEntryX;
    delete histCanvasSizeEntryY;
    delete horizontalMarkTotalEntry;
    delete horizontalBigMarkTotalEntry;
    delete verticalMarkTotalEntry;
    delete verticalBigMarkTotalEntry;
    delete setHistColorEntry;
    delete saveAsEntry;
    delete saveAsXWDEntry;
    delete saveAsDataEntry;

    delete drawTypeBoxRadio;
    delete drawTypeFillRadio;
    delete drawTypeLineRadio;

    delete saveAsButton;
    delete saveAsXWDButton;
    delete saveAsDataButton;
    delete setButton;
    delete verticalScaleFrame;
    delete horizontalScaleFrame;
    delete drawTypeBoxFrame;
    delete defaultFrame;
    subWindow->destroy();
    delete subWindow;
    subWindow = 0;
}

void KonoeTclDrawCanvasController::createParts()
{
//---------- 1 ----------
  verticalScaleLabel
    = new KonoeTclLabel(defaultFrame,"verticalScaleLabel","Vertical Scale",0,0);

  verticalScaleFrame =
           new KonoeTclFrame(defaultFrame, "verticalScaleFrame", 1, 0);

  verticalMaxLabel
    = new KonoeTclLabel(verticalScaleFrame, "verticalMinLabel",
                        "Max", 0,0);

  verticalScaleEntry
    = new KonoeTclEntry(verticalScaleFrame, "verticalScaleEntry", 1,0);

  verticalScaleLabel->setRelief(KRidge);
  verticalScaleLabel->setGridSticky("news");
  verticalScaleEntry->setBackGround("white");
  verticalScaleEntry->setSize(15);

  verticalScaleFrame->setGridColumnspan(2);
  verticalScaleFrame->setRelief(KRidge);
  verticalScaleFrame->setGridSticky("news");
  verticalScaleFrame->setBorderWidth(1);

  verticalScaleEntry->setEventProcessor(this);
  addKeyAction(verticalScaleEntry);

//---------- 2 ----------
  horizontalScaleLabel
    = new KonoeTclLabel(defaultFrame,"horizontalScaleLabel",
                                             "Horizontal Scale",0,1);

  horizontalScaleFrame =
             new KonoeTclFrame(defaultFrame, "horizontalScaleFrame", 1, 1);


  horizontalMinLabel
    = new KonoeTclLabel(horizontalScaleFrame, "horizontalMinLabel",
                        "Min", 0,0);

  horizontalMaxLabel
    = new KonoeTclLabel(horizontalScaleFrame, "horizontalMaxLabel",
                        "Max", 0,1);

  horizontalBinCountLabel
    = new KonoeTclLabel(horizontalScaleFrame, "horizontalBinCountLabel",
                        "Bin", 0,2);

  horizontalScaleLabel->setRelief(KRidge);
  horizontalScaleLabel->setGridSticky("news");

  horizontalMinEntry
    = new KonoeTclEntry(horizontalScaleFrame, "horizontalMinEntry", 1,0);
  horizontalMinEntry->setBackGround("white");
  horizontalMinEntry->setSize(15);

  horizontalMaxEntry
    = new KonoeTclEntry(horizontalScaleFrame, "horizontalMaxEntry", 1,1);
  horizontalMaxEntry->setBackGround("white");
  horizontalMaxEntry->setSize(15);

  horizontalBinCountEntry
    = new KonoeTclEntry(horizontalScaleFrame, "horizontalBinCountEntry", 1,2);
  horizontalBinCountEntry->setBackGround("white");
  horizontalBinCountEntry->setSize(15);


  horizontalScaleFrame->setGridColumnspan(2);
  horizontalScaleFrame->setRelief(KRidge);
  horizontalScaleFrame->setBorderWidth(1);
  horizontalScaleFrame->setGridSticky("news");

  horizontalMinEntry->setEventProcessor(this);
  addKeyAction(horizontalMinEntry);
  horizontalMaxEntry->setEventProcessor(this);
  addKeyAction(horizontalMaxEntry);
  horizontalBinCountEntry->setEventProcessor(this);
  addKeyAction(horizontalBinCountEntry);

//---------- 3 ----------
  histCanvasSizeLabel
    = new KonoeTclLabel(defaultFrame, "histCanvasSizeLabel", "Canvas Size",0,2);

  histCanvasSizeEntryX
     = new KonoeTclEntry(defaultFrame, "histCanvasSizeEntryX",1,2);
  histCanvasSizeEntryX->setBackGround("white");
  histCanvasSizeEntryX->setSize(10);

  histCanvasSizeEntryY
     = new KonoeTclEntry(defaultFrame, "histCanvasSizeEntryY",2,2);
  histCanvasSizeEntryY->setBackGround("white");
  histCanvasSizeEntryY->setSize(10);

  histCanvasSizeEntryX->setEventProcessor(this);
  addKeyAction(histCanvasSizeEntryX);
  histCanvasSizeEntryY->setEventProcessor(this);
  addKeyAction(histCanvasSizeEntryY);

  histCanvasSizeLabel->setRelief(KRidge);
  histCanvasSizeLabel->setGridSticky("news");

//---------- 4 ----------
  horizontalMarkTotalLabel
    = new KonoeTclLabel(defaultFrame, "horizontalMarkTotalLabel",
                                      "Horizontal Mark Total",0,3);

  horizontalMarkTotalEntry
    = new KonoeTclEntry(defaultFrame, "horizontalMarkTotalEntry",1,3);
  horizontalMarkTotalEntry->setBackGround("white");
  horizontalMarkTotalEntry->setGridColumnspan(2);
  horizontalMarkTotalEntry->setGridSticky("news");

  horizontalMarkTotalLabel->setRelief(KRidge);
  horizontalMarkTotalLabel->setGridSticky("news");

  horizontalMarkTotalEntry->setEventProcessor(this);
  addKeyAction(horizontalMarkTotalEntry);

//---------- 5 ----------
  horizontalBigMarkTotalLabel =
    new KonoeTclLabel(defaultFrame, "horizontalBigMarkTotalLabel",
                                    "Horizontal Big Mark Total",0,4);

  horizontalBigMarkTotalEntry
    = new KonoeTclEntry(defaultFrame, "horizontalBigMarkTotalEntry",1,4);
  horizontalBigMarkTotalEntry->setGridColumnspan(2);
  horizontalBigMarkTotalEntry->setGridSticky("news");
  horizontalBigMarkTotalEntry->setBackGround("white");

  horizontalBigMarkTotalLabel->setRelief(KRidge);
  horizontalBigMarkTotalLabel->setGridSticky("news");

  horizontalBigMarkTotalEntry->setEventProcessor(this);
  addKeyAction(horizontalBigMarkTotalEntry);

//---------- 6 ----------
  verticalMarkTotalLabel
    = new KonoeTclLabel(defaultFrame, "verticalMarkTotalLabel",
                                      "Vertical Mark Total",0,5);

  verticalMarkTotalEntry
    = new KonoeTclEntry(defaultFrame, "verticalMarkTotalEntry",1,5); 
  verticalMarkTotalEntry->setBackGround("white");
  verticalMarkTotalEntry->setGridColumnspan(2);
  verticalMarkTotalEntry->setGridSticky("news");

  verticalMarkTotalLabel->setRelief(KRidge);
  verticalMarkTotalLabel->setGridSticky("news");

  verticalMarkTotalEntry->setEventProcessor(this);
  addKeyAction(verticalMarkTotalEntry);

//---------- 7 ----------
  verticalBigMarkTotalLabel
    = new KonoeTclLabel(defaultFrame, "verticalBigMarkTotalLabel",
                                      "Vertical Big Mark Total",0,6);

  verticalBigMarkTotalEntry
    = new KonoeTclEntry(defaultFrame, "verticalBigMarkTotalEntry",1,6);
  verticalBigMarkTotalEntry->setBackGround("white");
  verticalBigMarkTotalEntry->setGridColumnspan(2);
  verticalBigMarkTotalEntry->setGridSticky("news");

  verticalBigMarkTotalLabel->setRelief(KRidge);
  verticalBigMarkTotalLabel->setGridSticky("news");

  verticalBigMarkTotalEntry->setEventProcessor(this);
  addKeyAction(verticalBigMarkTotalEntry);

//---------- 8 ----------
  drawTypeBoxLabel
    = new KonoeTclLabel(defaultFrame, "drawTypeBoxLabel", "Draw Type",0,7);

  drawTypeBoxFrame
    = new KonoeTclFrame(defaultFrame, "drawTypeBoxFrame", 1, 7);

  drawTypeBoxRadio
    = new KonoeTclRadioButton(drawTypeBoxFrame, "drawTypeBoxRadio","Box",1,0);
  drawTypeBoxRadio->setGroup("enableBox");
  drawTypeBoxRadio->setGridSticky("news");

  drawTypeFillRadio
    = new KonoeTclRadioButton(drawTypeBoxFrame, "drawTypeFillRadio","Fill",2,0);
  drawTypeFillRadio->setGroup("enableBox");
  drawTypeFillRadio->setGridSticky("news");

  drawTypeLineRadio
    = new KonoeTclRadioButton(drawTypeBoxFrame, "drawTypeLineRadio","Line",3,0);
  drawTypeLineRadio->setGroup("enableBox");
  drawTypeLineRadio->setGridSticky("news");

  drawTypeBoxLabel->setRelief(KRidge);
  drawTypeBoxLabel->setGridSticky("news");

  drawTypeBoxFrame->setGridColumnspan(2);
  drawTypeBoxFrame->setRelief(KRidge);
  drawTypeBoxFrame->setGridSticky("news");
  drawTypeBoxFrame->setBorderWidth(1);


//---------- 9 ----------
  setHistColorLabel
    = new KonoeTclLabel(defaultFrame, "setHistColorLabel",
                                      "Histogram Color",0,8);

  setHistColorEntry
    = new KonoeTclEntry(defaultFrame, "setHistColorEntry",1,8);
  setHistColorEntry->setBackGround("white");
  setHistColorEntry->setGridColumnspan(2);
  setHistColorEntry->setGridSticky("news");

  setHistColorLabel->setRelief(KRidge);
  setHistColorLabel->setGridSticky("news");

  setHistColorEntry->setEventProcessor(this);
  addKeyAction(setHistColorEntry);

//---------- 10 ----------
  saveAsButton
    = new KonoeTclButton(defaultFrame, "saveAsButton",
                                       "Save as PostScript",0,9);
  saveAsButton->setEventProcessor(this);
  addMouseAction(saveAsButton);
  saveAsEntry
    = new KonoeTclEntry(defaultFrame, "saveAsEntry",1,9);
  saveAsEntry->setBackGround("white");
  saveAsEntry->setGridColumnspan(2);
  saveAsEntry->setGridSticky("news");
  saveAsButton
    = new KonoeTclButton(defaultFrame, "saveAsButton",
                                       "Save as PostScript",0,9);
  saveAsButton->setEventProcessor(this);
  addMouseAction(saveAsButton);
  addKeyAction(saveAsButton);

  saveAsEntry->setEventProcessor(this);
  addKeyAction(saveAsEntry);

//---------- 11 ----------
  saveAsXWDButton
    = new KonoeTclButton(defaultFrame, "saveAsXWDButton",
                                       "Save as xwd",0,10);
  saveAsXWDButton->setEventProcessor(this);

  saveAsXWDEntry
    = new KonoeTclEntry(defaultFrame, "saveAsXWDEntry",1,10);
  saveAsXWDEntry->setBackGround("white");
  saveAsXWDEntry->setGridColumnspan(2);
  saveAsXWDEntry->setGridSticky("news");

  addMouseAction(saveAsXWDButton);
  addKeyAction(saveAsXWDButton);

  saveAsXWDEntry->setEventProcessor(this);
  addKeyAction(saveAsXWDEntry);

//---------- 12 ----------
  saveAsDataButton
    = new KonoeTclButton(defaultFrame, "saveAsDataButton",
                                       "Save as Data File",0,11);
  saveAsDataButton->setEventProcessor(this);

  saveAsDataEntry
    = new KonoeTclEntry(defaultFrame, "saveAsDataEntry",1,11);
  saveAsDataEntry->setBackGround("white");
  saveAsDataEntry->setGridColumnspan(2);
  saveAsDataEntry->setGridSticky("news");

  addMouseAction(saveAsDataButton);
  addKeyAction(saveAsDataButton);

  saveAsDataEntry->setEventProcessor(this);
  addKeyAction(saveAsDataEntry);

//---------- 12 ----------
  setButton
    = new KonoeTclButton(defaultFrame, "setButton",
                                       "Set Property",0,12);
  setButton->setEventProcessor(this);
  addMouseAction(setButton);
  addKeyAction(setButton);
  setButton->setGridColumnspan(3);
  setButton->setGridSticky("news");
  setButton->setGridPadx(11);
  setButton->setGridPady(11);

}

void KonoeTclDrawCanvasController::writeProperty()
{
  KChar value[256];
  KonoeTclDimension dim;
  KInt v;
  KInt min;
  KInt max;
  KInt bin;

//---------- 1 ----------
  max = (KInt)drawCanvas->getVerticalScale()->getMaximum();
  sprintf(value, "%d", max);
  verticalScaleEntry->setText(value);


//---------- 2 ----------
  min = (KInt)drawCanvas->getScale()->getMinimum();
  max = (KInt)drawCanvas->getScale()->getMaximum();
  bin = drawCanvas->getScale()->getBinCount();
  sprintf(value, "%d", min);
  horizontalMinEntry->setText(value);
  sprintf(value, "%d", max);
  horizontalMaxEntry->setText(value);
  sprintf(value, "%d", bin);
  horizontalBinCountEntry->setText(value);


//---------- 3 ----------
  dim = drawCanvas->getSize();
  sprintf(value, "%d", dim.getWidth());
  histCanvasSizeEntryX->setText(value);
  sprintf(value, "%d", dim.getHeight());
  histCanvasSizeEntryY->setText(value);

//---------- 4 ----------
  v = drawCanvas->getScale()->getMarkTotal();
  sprintf(value, "%d", v);
  horizontalMarkTotalEntry->setText(value);

//---------- 5 ----------
  v = drawCanvas->getScale()->getBigMarkTotal();
  sprintf(value, "%d", v);
  horizontalBigMarkTotalEntry->setText(value);

//---------- 6 ----------
  v = drawCanvas->getVerticalScale()->getMarkTotal();
  sprintf(value, "%d", v);
  verticalMarkTotalEntry->setText(value);

//---------- 7 ----------
  v = drawCanvas->getVerticalScale()->getBigMarkTotal();
  sprintf(value, "%d", v);
  verticalBigMarkTotalEntry->setText(value);

//---------- 8 ----------
  drawTypeBoxRadio->setSelect();

//---------- 9 ----------
  strcpy(value, drawCanvas->getHistoColor());
  setHistColorEntry->setText(value);

//---------- 10 ----------
  strcpy(value, "histogram.ps");
  saveAsEntry->setText(value);

//---------- 11 ----------
  strcpy(value, "histogram.xwd");
  saveAsXWDEntry->setText(value);

//---------- 12 ----------
  strcpy(value, "histogram.obj");
  saveAsDataEntry->setText(value);
}

void KonoeTclDrawCanvasController::setHistogramDrawSize()
{
  KInt x = atoi(histCanvasSizeEntryX->getText()) - 70;
  KInt y = atoi(histCanvasSizeEntryY->getText()) - 70;
  if( x < 1 ){
    x = 1;
    histCanvasSizeEntryX->setText("1");
  }
  if( y < 1 ){
    y = 1;
    histCanvasSizeEntryY->setText("1");
  }
  KonoeTclDimension d(x,y);
  drawCanvas->setDimension(d);
}

void KonoeTclDrawCanvasController::setHistogramCanvasSize()
{
  KInt x = atoi(histCanvasSizeEntryX->getText());
  KInt y = atoi(histCanvasSizeEntryY->getText());
  drawCanvas->setSize(x, y);
}

void KonoeTclDrawCanvasController::setVerticalScale()
{
  KFloat min = drawCanvas->getVerticalScale()->getMinimum();
  KFloat max = atoi(verticalScaleEntry->getText());
  KInt bin = drawCanvas->getVerticalScale()->getBinCount();

  if(max < 1){
    max = 1;
    verticalScaleEntry->setText("1");
  }

  drawCanvas->setVerticalScale(new KonoeLinearDataScale(min, max, bin));
}

void KonoeTclDrawCanvasController::setHorizontalScale()
{
  KInt min = atoi(horizontalMinEntry->getText());
  KInt max = atoi(horizontalMaxEntry->getText());
  KInt bin = atoi(horizontalBinCountEntry->getText());

  if( min < 0 ){
    min = 0;
    horizontalMinEntry->setText("0");
  }
  if( max < 1 ){
    max = 1;
    horizontalMaxEntry->setText("1");
  }
  if( bin < 1 ){
    bin = 1;
    horizontalBinCountEntry->setText("1");
  }

  if( max - min < bin ){
    bin = max - min;
    KChar cbin[16];
    sprintf(cbin, "%d", bin);
    horizontalBinCountEntry->setText(cbin);
  }

  KonoeLinearDataScale* scale =
        new KonoeLinearDataScale( min, max, bin );
  Konoe1DHistogramBand* band =
        new Konoe1DHistogramBand( scale );
  Konoe1DHistogram* hist = band->getNewHisto(drawCanvas->getDefaultHistogram());
  drawCanvas->setDataSource(hist);

  drawCanvas->setHorizontalScale( min, max, bin );
}
  
void KonoeTclDrawCanvasController::setHorizontalMarkTotal()
{
  KInt n = atoi(horizontalMarkTotalEntry->getText());
  if(n < 1){
    n = 1;
    horizontalMarkTotalEntry->setText("1");
  }
  drawCanvas->getScale()->setMarkTotal(n);
  drawCanvas->setMark(n);
}

void KonoeTclDrawCanvasController::setHorizontalBigMarkTotal()
{
  KInt n = atoi(horizontalBigMarkTotalEntry->getText());
  if( n < 1 ){
    n = 1;
    horizontalBigMarkTotalEntry->setText("1");
  }
  drawCanvas->getScale()->setBigMarkTotal(n);
  drawCanvas->setBigMark(n);
}

void KonoeTclDrawCanvasController::setVerticalMarkTotal()
{
  KInt n = atoi(verticalMarkTotalEntry->getText());
  if( n < 1 ){
    n = 1;
    verticalMarkTotalEntry->setText("1");
  }
  drawCanvas->getVerticalScale()->setMarkTotal(n);
}

void KonoeTclDrawCanvasController::setVerticalBigMarkTotal()
{
  KInt n = atoi(verticalBigMarkTotalEntry->getText());
  if( n < 1 ){
    n = 1;
    verticalBigMarkTotalEntry->setText("1");
  }
  drawCanvas->getVerticalScale()->setBigMarkTotal(n);
}

void KonoeTclDrawCanvasController::setDrawTypeBox()
{
  strcpy(cmd, drawTypeBoxRadio->getValue());
  if(strcmp(cmd, drawTypeBoxRadio->getName()) == 0)
    drawCanvas->enableBox();
  else if(strcmp(cmd, drawTypeFillRadio->getName()) == 0)
    drawCanvas->enableFill();
  else if(strcmp(cmd, drawTypeLineRadio->getName()) == 0)
    drawCanvas->enableLine();
}

void KonoeTclDrawCanvasController::setHistoColor()
{
  strcpy(cmd, setHistColorEntry->getText());
  drawCanvas->setHistoColor(cmd);
}

void KonoeTclDrawCanvasController::setSaveFileFullPath(KChar* pFullPath)
{
  strcpy(fullPath, pFullPath);
}

void KonoeTclDrawCanvasController::mouseAction(KonoeTclButton* kbtn,
                           KInt type, KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if(strcmp(kbtn->getName(),"saveAsButton")==0) saveAsButtonAction();
      if(strcmp(kbtn->getName(),"saveAsXWDButton")==0) saveAsXWDButtonAction();
      if(strcmp(kbtn->getName(),"saveAsDataButton")==0)
                                                    saveAsDataButtonAction();
      if(strcmp(kbtn->getName(),"setButton")==0)    setButtonAction();
    }
  }
}

void KonoeTclDrawCanvasController::keyAction(KonoeTclEntry* kbt, KInt type,
                                                     KeySym keysym, KInt state)
{
  if(type == KKeyRelease){
    if(keysym == XK_KP_Enter || keysym == XK_Return){
      if(strcmp(kbt->getName(), "saveAsButton") == 0)
        saveAsButtonAction();
      else if(strcmp(kbt->getName(), "saveAsXWDButton") == 0)
        saveAsXWDButtonAction();
      else if(strcmp(kbt->getName(), "saveAsDataButon") == 0)
        saveAsDataButtonAction();
      else
        setButtonAction();
    }
  }
}

void KonoeTclDrawCanvasController::keyAction(KonoeTclButton* kbt, KInt type,
                                                     KeySym keysym, KInt state)
{
  if(type == KKeyPress){
    if(keysym == XK_KP_Enter || keysym == XK_Return){
      kbt->setRelief(KSunken);
    }
  }
  else if(type == KKeyRelease){
    if(keysym == XK_KP_Enter || keysym == XK_Return){
      if(strcmp(kbt->getName(), "saveAsEntry") == 0)
        saveAsButtonAction();
      else if(strcmp(kbt->getName(), "saveAsXWDEntry") == 0)
        saveAsXWDButtonAction();
      else if(strcmp(kbt->getName(), "saveAsDataEntry") == 0)
        saveAsDataButtonAction();
      else
        setButtonAction();
    }
      kbt->setRelief(KRaised);
  }
}

KInt KonoeTclDrawCanvasController::saveAsButtonAction()
{
  if( strlen(fullPath) > 0 )
    sprintf(cmd, "%s/%s", fullPath, saveAsEntry->getText());
  else
    strcpy(cmd, saveAsEntry->getText());
  if(drawCanvas!=NULL)
    drawCanvas->toPostScript(cmd);
  return 0;
}

KInt KonoeTclDrawCanvasController::saveAsXWDButtonAction()
{
  if( strlen(fullPath) > 0 )
    sprintf(cmd, "%s/%s", fullPath, saveAsXWDEntry->getText());
  else
    strcpy(cmd, saveAsEntry->getText());

  if(drawCanvas!=NULL){
    subWindow->hide(subWindow);
    subWindow->after(1000);
    drawCanvas->toXWD(cmd);
    subWindow->show(subWindow);
  }
  return 0;
}

KInt KonoeTclDrawCanvasController::saveAsDataButtonAction()
{
  if( strlen(fullPath) > 0 )
    sprintf(cmd, "%s/%s", fullPath, saveAsDataEntry->getText());
  else
    strcpy(cmd, saveAsEntry->getText());

  if(drawCanvas!=NULL){
    drawCanvas->toFile(cmd);
  }
  return 0;
}

KInt KonoeTclDrawCanvasController::setButtonAction()
{
  if(drawCanvas!=NULL){
    setVerticalScale();
    setHistogramDrawSize();
    setHistogramCanvasSize();
    setHorizontalScale();
    setHorizontalMarkTotal();
    setHorizontalBigMarkTotal();
    setVerticalMarkTotal();
    setVerticalBigMarkTotal();
    setDrawTypeBox();
    setHistoColor();
    drawCanvas->draw();
    drawCanvas->update();
  }

  return 0;
}
