//=====================================================================
/*

    KonoeTclDrawCanvasMouseListener.cc

    Version 0.0.1   09-Nov-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclDrawCanvasMouseListener.hh>
#include <ui/KonoeTclSubWindow.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawCanvasController.hh>
#include <ui/KonoeTclBalloon.hh>
#include <ui/KonoeTclCalculator.hh>
#include <ui/KonoeTcl1DHistogram.hh>
#include <ui/KonoeTclDataScale.hh>

#include <system/KonoeTime.hh>
#include <analysis/Konoe1DHistogramBand.hh>

#include <stdlib.h>

KonoeTclDrawCanvasMouseListener::KonoeTclDrawCanvasMouseListener()
{
  balloon = new KonoeTclBalloon;
  drawCanvasController = 0;
  s = 0;
  p = 0;
  c = 0;
  isCreateDrawCanvas = KFalse;
  isCreateControlPanel = KFalse;

  KInt i;
  for(i=0; i<256; i++)
    fullPath[i] = '\0';
}

KonoeTclDrawCanvasMouseListener::~KonoeTclDrawCanvasMouseListener()
{ ; }

void KonoeTclDrawCanvasMouseListener::setSaveFileFullPath(KChar* pFullPath)
{
  strcpy(fullPath , pFullPath);
}

void KonoeTclDrawCanvasMouseListener::mouseAction
        (KonoeTclCanvas* kbt, KInt type, KInt button, Time time, KInt x, KInt y)
{
  KonoeTclDrawCanvas* canv = (KonoeTclDrawCanvas*)kbt;
  drawCanvas = canv;

  KChar backColor[256];
  strcpy(backColor, canv->getBackGround());

  if(type == KMouseRelease && button == LButton){
    static KInt clickedTime = 0;
    
    if(abs(clickedTime - time) > 300){ //Single Click
      KChar message[256];
      KonoeTclDimension d = getClickPoint(x,y);
      KFloat xPoint = canv->getScale()->getMinimum()+d.getWidth();
      //                                           +canv->getBinWidth()/2.;
      //KFloat xPoint = d.getWidth();
      KFloat yPoint = d.getHeight();
      KFloat maxData;
      KInt maxIndex;
      KFloat mean;
      KFloat variance;
      KFloat standardDeviation;
      KInt entry;

      maxData = getMax();
      maxIndex = getMaxBinNumber();
      mean = calcMean(canv->getDimension().getOrigX(), canv->getWidth());
      variance
          = calcVariance(canv->getDimension().getOrigX(), canv->getWidth());
      standardDeviation = calcStandardDeviation
                            (canv->getDimension().getOrigX(), canv->getWidth());
      entry
          = calcBinTotal(canv->getDimension().getOrigX(), canv->getWidth());


      sprintf(message, "bin , value = %.0lf , %.2lf\nmax \\[bin\\] = %.2lf\\[%d\\]\nentry = %d\nmean = %.2lf\nvariance = %.2lf\nsigma = %.2lf"
                     , xPoint, yPoint, maxData, maxIndex, entry
                     , mean, variance, standardDeviation);

      balloon->setBalloon(kbt, message);
    }

    else if(abs(clickedTime - time) <= 300 //Double Click
              && canv->isCreated() == KFalse){
      KChar titleName[256];
      strcpy(titleName, drawCanvas->getTitleName());
      KonoeTclWindow* w = drawCanvas->getTopLevelWindow();

      if(!isCreateDrawCanvas){
      if(!s){
        isCreateDrawCanvas = KTrue;
        s = new KonoeTclSubWindow(w, "subWinOfCanv", titleName);
        p = new KonoeTclDrawPanel(s, "dp");
        Konoe1DHistogram* histogram = drawCanvas->getDefaultHistogram();
        KonoeLinearDataScale* scale =
             new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                       histogram->getScale()->getMaximum(),
                                       histogram->getScale()->getBinCount() );
        Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
        c = new KonoeTclDrawCanvas(p,"dc",
                           band->getNewHisto( histogram ), 
                           KonoeTclDimension(400,400), titleName,
                           new KonoeTclDataScale(drawCanvas->getDataScale()) );
        c->setBackGround(backColor);
        c->setCreated();
      }
      else{
        isCreateDrawCanvas = KTrue;
        delete c;
        delete p;
        s->destroy();
        delete s;
        s = new KonoeTclSubWindow(w, "subWinOfCanv", titleName);
        p = new KonoeTclDrawPanel(s, "dp");
        Konoe1DHistogram* histogram = drawCanvas->getDefaultHistogram();
        KonoeLinearDataScale* scale =
             new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                       histogram->getScale()->getMaximum(),
                                       histogram->getScale()->getBinCount() );
        Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
        c = new KonoeTclDrawCanvas(p,"dc",
                           band->getNewHisto( histogram ), 
                           KonoeTclDimension(400,400), titleName,
                           new KonoeTclDataScale(drawCanvas->getDataScale()) );
        c->setBackGround(backColor);
        c->setCreated();
      }
      p->setCanvas(c,"");

      KonoeTime ktime;
      KChar* timetxt = ktime.getTimeInfo();
      KChar histName[256];
      sprintf(histName, "%s (%s)", c->getTitleName(), timetxt);
      c->setHistogramName(histName);

      c->setAutoVertical();
      c->draw();
      c->update();
      isCreateDrawCanvas = KFalse;

      c->setEventProcessor(this);
      addMouseAction(c);
      addMouseMotion(c);
      }
    }
    clickedTime = time;
  }

  if(type == KMouseRelease && button == RButton){
    if(!drawCanvasController){
      drawCanvasController = new KonoeTclDrawCanvasController(canv);
      drawCanvasController->setSaveFileFullPath(fullPath);
    }
    else{
      delete drawCanvasController;
      drawCanvasController = new KonoeTclDrawCanvasController(canv);
      drawCanvasController->setSaveFileFullPath(fullPath);
    }
  }
}

void KonoeTclDrawCanvasMouseListener::mouseMotion
        (KonoeTclCanvas* kbt, KInt type, KInt state, Time time, KInt x, KInt y)
{
  static KInt startXp;
  static KInt startYp;
  KInt endXp;
  KInt endYp;
  KonoeTclDrawCanvas* canv = (KonoeTclDrawCanvas*) kbt;
  drawCanvas = canv;
  KChar message[256];
  KInt binTotal;

  endXp = x;
  endYp = y;

  if(type == KMousePress){
    canv->deleteItem("renge");
    startXp = endXp;
    startYp = endYp;
  }
  else if(type == KMouseRelease){
    if(abs(endXp-startXp) < 4)
      canv->deleteItem("renge");
    else{
//      sprintf(message, "selected BinCount = %d\nMean = %.2lf",
//                        calcSelectedBinCount(startXp, endXp),
//                        calcMean(startXp, endXp));
      sprintf(message, "selected BinCount = %d\nstart bin = %d\nend bin = %d\nentry = %d\nmean = %.2lf\nvariance = %.2lf\nsigma = %.2lf",
                       calcSelectedBinCount(startXp, endXp),
                       getClickPoint(startXp, 0).getWidth(),
                       getClickPoint(endXp, 0).getWidth(),
                       calcBinTotal(startXp, endXp),
                       calcMean(startXp, endXp),
                       calcVariance(startXp, endXp),
                       calcStandardDeviation(startXp, endXp));
      balloon->setBalloon(kbt, message);
    }
  }
  else if(type == KMouseMove && state == MouseLButton){
    canv->deleteItem("renge");
    canv->setColor("blue");
    canv->drawRectangle(startXp, startYp,
                  endXp-startXp, 1, "renge");
    canv->drawLine(startXp, startYp-5, startXp, startYp+5, "renge");
    canv->drawLine(endXp, startYp-5, endXp, startYp+5, "renge");
  }
}

KonoeTclDimension KonoeTclDrawCanvasMouseListener::getClickPoint(KInt x, KInt y)
{
  KonoeTclCalculator calc;
  return calc.getClickPoint(drawCanvas, x, y);
}

KInt KonoeTclDrawCanvasMouseListener::calcSelectedBinCount
                                            (KInt startX, KInt endX)
{
  KonoeTclDimension d1 = getClickPoint(startX, 0);
  KonoeTclDimension d2 = getClickPoint(endX, 0);
  KInt binCount;

  binCount = abs(d1.getWidth() - d2.getWidth());

  return binCount;
}

KFloat KonoeTclDrawCanvasMouseListener::getMax()
{
  KInt binCount;
  binCount = drawCanvas->getHistogram()->getBinCount();
  KFloat* data = new KFloat[binCount];

  KInt i;
  for(i=0; i<binCount; i++){
    data[i] = drawCanvas->getHistogram()->getOne(i);
  }

  KonoeTclCalculator calc;
  KFloat mdata = calc.getMax(drawCanvas);
  delete [] data;

  return mdata;
}

KInt KonoeTclDrawCanvasMouseListener::getMaxBinNumber()
{
  KInt binCount;
  binCount = drawCanvas->getHistogram()->getBinCount();
  KFloat* data = new KFloat[binCount];

  KInt i;
  for(i=0; i<binCount; i++){
    data[i] = drawCanvas->getHistogram()->getOne(i);
  }

  KonoeTclCalculator calc;
  KInt mdata = calc.getMaxBinNumber(drawCanvas);
  delete [] data;

  return mdata;
}

KFloat KonoeTclDrawCanvasMouseListener::calcMean(KInt startX, KInt endX)
{
  KonoeTclCalculator calc;
  KFloat mdata = calc.calcMean(drawCanvas, startX, endX);

  return mdata;
}

KFloat KonoeTclDrawCanvasMouseListener::calcVariance(KInt startX, KInt endX)
{
  KonoeTclCalculator calc;
  KFloat variance = calc.calcVariance(drawCanvas, startX, endX);

  return variance;
}

KFloat KonoeTclDrawCanvasMouseListener::calcStandardDeviation
                                                 (KInt startX, KInt endX)
{
  KonoeTclCalculator calc;
  KFloat standardDeviation =
                calc.calcStandardDeviation(drawCanvas, startX, endX);
  return standardDeviation;
}

KInt KonoeTclDrawCanvasMouseListener::calcBinTotal(KInt startX, KInt endX)
{
  KonoeTclCalculator calc;
  KInt binTotal = calc.calcBinTotal(drawCanvas, startX, endX);
  return binTotal;
}
