//=====================================================================
/*

    KonoeTclEventHitCanvas.cc

    Version 0.6.0   06-Sep-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclEventHitCanvas.hh>
#include <ui/KonoeTclEventViewCanvas.hh>
#include <ui/KonoeTclPhysPlacement.hh>
#include <ui/KonoeTclSpaceManager.hh>
#include <ui/KonoeTcl3DVector.hh>
#include <ui/KonoeTclDrawableBox.hh>
#include <ui/KonoeTclRotationMatrix.hh>
#include <ui/KonoeTclDrawableString.hh>

const KInt STRING_SIZE = 256;

KonoeTclEventHitCanvas::KonoeTclEventHitCanvas()
{
  totalOfPhys = 0;
  detectorElement = 0;
  physicalVolume = 0;
  stringVolume = 0;
  stringSolid = 0;
  str = 0;
  rotX = 0.;
  rotY = 0.;
  slowDraw = KFalse;
}

KonoeTclEventHitCanvas::KonoeTclEventHitCanvas(
                               KonoeTclEventViewCanvas* peventViewCanvas)
{
  totalOfPhys = 0;
  detectorElement = 0;
  physicalVolume = 0;
  stringVolume = 0;
  stringSolid = 0;
  str = 0;
  rotX = 0.;
  rotY = 0.;
  slowDraw = KFalse;
  setEventViewCanvas(peventViewCanvas);
}

KonoeTclEventHitCanvas::~KonoeTclEventHitCanvas()
{
  if(totalOfPhys){
    KInt i;
    for(i=0; i<totalOfPhys; i++){
      delete physicalVolume[i];
      delete stringVolume[i];
//      delete stringSolid[i];
      delete str[i];
    }
      delete [] detectorElement;
      delete [] physicalVolume;
      delete [] stringVolume;
//      delete [] stringSolid;
      delete [] str;
  }
}

void KonoeTclEventHitCanvas::setEventViewCanvas(
                        KonoeTclEventViewCanvas* peventViewCanvas)
{
  eventViewCanvas = peventViewCanvas;
  spMan = eventViewCanvas->getSpaceManager();

  KInt i;
  for(i=0; i<totalOfPhys; i++){
    delete physicalVolume[i];
    delete stringVolume[i];
  }

  if(eventViewCanvas->getTotalPhysicalVolume() != totalOfPhys){
    this->initInstance();
  }

  for(i=0;i<totalOfPhys;i++){
    detectorElement[i] = eventViewCanvas->getPhysicalVolume(i);
  }
  drawNumber = eventViewCanvas->getDrawNumber();

  this->createGeometry();
}

void KonoeTclEventHitCanvas::initInstance()
{
    totalOfPhys = eventViewCanvas->getTotalPhysicalVolume();
    detectorElement = new KonoeTclPhysPlacement*[totalOfPhys];

    physicalVolume = new KonoeTclPhysPlacement*[totalOfPhys];
    solid = new KonoeTclSolid*[totalOfPhys];

    stringVolume = new KonoeTclPhysPlacement*[totalOfPhys];
    stringSolid = new KonoeTclDrawableString*[totalOfPhys];

    str = new KChar*[totalOfPhys];
    KInt i;
    for(i=0; i<totalOfPhys; i++)
      str[i] = new KChar[STRING_SIZE];

    hitColorR = new KInt[totalOfPhys];
    hitColorG = new KInt[totalOfPhys];
    hitColorB = new KInt[totalOfPhys];

    for(i=0; i<totalOfPhys; i++){
      hitColorR[i] = 217;
      hitColorG[i] = 217;
      hitColorB[i] = 217;
    }
}

void KonoeTclEventHitCanvas::createGeometry()
{
  KInt i;

  for(i=0; i<totalOfPhys; i++){
    //if(Box)
    this->createBoxPhysicalVolume(i);
    //else if(...){}
    this->createStringPhysicalVolume(i);
  }
}

void KonoeTclEventHitCanvas::createBoxPhysicalVolume(KInt i)
{
  KChar geometryName[256];
  KInt width;
  KInt height;
  KInt depth; 

  sprintf(geometryName, "hitgeometry_%d", i);
  KonoeTclDrawableBox* box
                       = (KonoeTclDrawableBox*)detectorElement[i]->getSolid();
  width = (KInt)box->getBoxSize().getX();
  height = (KInt)box->getBoxSize().getY();
  depth = 1;

  solid[i] = new KonoeTclDrawableBox(geometryName, width, height, depth);

  KInt posX;
  KInt posY;
  KInt posZ;

  KonoeTclTransform posTrans = detectorElement[i]->getTransform();
  posX = (KInt)posTrans.getX();
  posY = (KInt)posTrans.getY();
  posZ = (KInt)(posTrans.getZ() - box->getBoxSize().getZ());

  KonoeTclRotationMatrix rm;
  rm.rotateZ(0);

  KonoeTclTransform trans(rm, KonoeTcl3DVector(posX, posY, posZ));

  physicalVolume[i] = new KonoeTclPhysPlacement(trans, solid[i]);
  physicalVolume[i]->replace(KonoeTcl2Dimension());
  physicalVolume[i]->setSpaceManager(spMan);
}

void KonoeTclEventHitCanvas::createStringPhysicalVolume(KInt i)
{
  stringSolid[i] = new KonoeTclDrawableString("");

  KonoeTclTransform trans = physicalVolume[i]->getTransform();

  stringVolume[i] = new KonoeTclPhysPlacement(trans, stringSolid[i]);
  stringVolume[i]->replace(KonoeTcl2Dimension());
  stringVolume[i]->setSpaceManager(spMan);
}

void KonoeTclEventHitCanvas::setString(KChar** pstr, KChar* fColor,
                                        KChar* fName, KInt fSize)
{
  KInt i;
  for(i=0; i<totalOfPhys; i++){
    strcpy(str[i], pstr[i]);
    stringSolid[i]->setString(str[i], fColor, fName, fSize);
  }

  isDrawString = KTrue;
  for(i=0; i<totalOfPhys; i++){
    stringVolume[drawNumber[i]]->draw(eventViewCanvas);
  }
}

void KonoeTclEventHitCanvas::setVisAtt(KInt i, KInt RR, KInt GG, KInt BB)
{
  hitColorR[i] = RR;
  hitColorG[i] = GG;
  hitColorB[i] = BB;
}

void KonoeTclEventHitCanvas::rotateX(KFloat rx)
{
  rotX = rx;
}

void KonoeTclEventHitCanvas::rotateY(KFloat ry)
{
  rotY = ry;
}

void KonoeTclEventHitCanvas::draw()
{
  drawNumber = eventViewCanvas->getDrawNumber();

  KInt i;
  for(i=0; i<totalOfPhys; i++){
    physicalVolume[drawNumber[i]]->setVisAtt( hitColorR[drawNumber[i]],
                                              hitColorG[drawNumber[i]],
                                              hitColorB[drawNumber[i]]);
    physicalVolume[drawNumber[i]]->rotateX( rotX );
    stringVolume[drawNumber[i]]->rotateX( rotX );
    physicalVolume[drawNumber[i]]->rotateY( rotY );
    stringVolume[drawNumber[i]]->rotateY( rotY );
    physicalVolume[drawNumber[i]]->draw(eventViewCanvas);
    stringVolume[drawNumber[i]]->draw(eventViewCanvas);
    if(slowDraw) eventViewCanvas->update();
//physicalVolume[drawNumber[i]]->dump();
  }
}
