//=====================================================================
/*

    KonoeTclEventViewCanvas.cc

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclImage.hh>
#include <ui/KonoeTclEventViewCanvas.hh>
#include <ui/KonoeTclEventHitCanvas.hh>
#include <ui/KonoeTclPhysPlacement.hh>
#include <ui/KonoeTclSpaceManager.hh>
#include <ui/KonoeTclDimension.hh>

#include <ui/KonoeTclRotationMatrix.hh>
#include <ui/KonoeTclDrawableBox.hh>
#include <ui/KonoeTclDrawableHorizonLine.hh>
#include <ui/KonoeTclDrawableVerticalLine.hh>
#include <ui/KonoeTclDrawableXLine.hh>
#include <ui/KonoeTclSolid.hh>
#include <ui/KonoeTcl3DVector.hh>
#include <analysis/KonoeDetectorList.hh>

#include <analysis/KonoeDetector.hh>
#include <analysis/KonoeDetectorElement.hh>
#include <analysis/KonoeBoxGeometry.hh>

#include <math.h>

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif

KonoeTclEventViewCanvas::KonoeTclEventViewCanvas( KonoeTclWindow* kwin,
                                                  KChar* kname,
                                                  KonoeTclDimension dimension,
                                                  KChar* pTitle, KInt pTotal )
  : KonoeTclCanvas(kwin, kname)
{
  totalOfPhys = 0;
  enableGeometry = 1;
  dim = dimension;
  KonoeTclCanvas::setSize(dim.getWidth(), dim.getHeight());
  strcpy( myName, pTitle );
//  spMan = new KonoeTclSpaceManager;
  setBackGround("black");

  totalOfPhys = pTotal;
  drawNumber = new KInt[totalOfPhys];

  myPhysicalVolume = new KonoeTclPhysPlacement*[totalOfPhys];
  hitCanvas = new KonoeTclEventHitCanvas();
  konoeLogoImage = new KonoeTclImage("konoeLogoImage");
  setImage();

  forceWireFrame = KFalse;
  scale = KFalse;
  isHitCanvas = KTrue;
  slowDraw = KFalse;
}

KonoeTclEventViewCanvas::KonoeTclEventViewCanvas( KonoeTclWindow* kwin,
                                                  KChar* kname,
                                                  KInt canvasSizeX,
                                                  KInt canvasSizeY,
                                                  Konoe3Vector pworldSize,
                                                  KonoeDetectorList* klist,
                                                  KChar* pTitle )
  : KonoeTclCanvas(kwin, kname)
{
  totalOfPhys = 0;
  enableGeometry = 1;
  dim = KonoeTclDimension(canvasSizeX, canvasSizeY);
  KonoeTclCanvas::setSize(dim.getWidth(), dim.getHeight());
  strcpy( myName, pTitle );
  spMan = new KonoeTclSpaceManager;
  setBackGround("black");

  worldSizeX = pworldSize.x;
  worldSizeY = pworldSize.y;
  worldSizeZ = pworldSize.z;

  drawNumber = new KInt[totalOfPhys];
  detectorPosition = new KonoeTcl3DVector[totalOfPhys];

  myPhysicalVolume = new KonoeTclPhysPlacement*[totalOfPhys];
  konoeLogoImage = new KonoeTclImage("konoeLogoImage");
  setImage();

  KFloat realCanvasWidth = worldSizeX*cos(45.*M_PI/540.)
                            + worldSizeZ*cos(45.*M_PI/540.);
  KFloat realCanvasHeight = worldSizeY
       + sqrt(worldSizeX*worldSizeX + worldSizeZ*worldSizeZ)*sin(45.*M_PI/540.);

  if(canvasSizeX > canvasSizeY)
    toRealCanvasSize = (KFloat)canvasSizeX / realCanvasWidth;
  else if(canvasSizeX < canvasSizeY)
    toRealCanvasSize = (KFloat)canvasSizeY / realCanvasHeight;
  else
    toRealCanvasSize = (KFloat)canvasSizeX / realCanvasWidth;

  setDetectorList(klist);

  hitCanvas = new KonoeTclEventHitCanvas();

  forceWireFrame = KFalse;
  scale = KFalse;
  isHitCanvas = KTrue;
  slowDraw = KFalse;
}

KonoeTclEventViewCanvas::~KonoeTclEventViewCanvas()
{ ; }

void KonoeTclEventViewCanvas::setPhysicalVolume( KonoeTclPhysPlacement** pPhys,
                                                 KInt startID, KInt endID )
{
  KInt copyNo = 0;
  KInt i;
  for( i=startID; i<=endID; i++ ){
    myPhysicalVolume[i] = pPhys[ copyNo ];
    myPhysicalVolume[i]->setSpaceManager(spMan);
    copyNo++;
  }
  //locateGenerator();

  //hitCanvas = new KonoeTclEventHitCanvas(this);
  hitCanvas->setEventViewCanvas(this);
}

void KonoeTclEventViewCanvas::setDetectorList( KonoeDetectorList* klist )
{
  detectorList = klist;

  detectorList->reset();

  KInt i;
  KInt n;
  for(i=0; i<detectorList->getCount(); i++){
    n = detectorList->next()->getCount();
    totalOfPhys += n;
  }
  this->createGeometry();
}

void KonoeTclEventViewCanvas::createGeometry()
{
  KonoeDetector* kDetector;
  KonoeDetectorElement* kDetectorElement;

// need delete?
  solid = new KonoeTclSolid*[totalOfPhys];
  detectorPosition = new KonoeTcl3DVector[totalOfPhys];
  myPhysicalVolume = new KonoeTclPhysPlacement*[totalOfPhys];

  KInt i;
  KInt j;
  KInt n = 0;
  KInt numOfDetector;
  KFloat xPos;
  KFloat yPos;
  KFloat zPos;

  detectorList->reset();
  numOfDetector = detectorList->getCount();

  for(i=0; i<numOfDetector; i++){
    kDetector = detectorList->next();
    kDetector->reset();
    for(j=0; j<kDetector->getCount(); j++){
      kDetectorElement = kDetector->next();
      xPos = kDetectorElement->getGeometry()->getDisplacement().x;
      yPos = kDetectorElement->getGeometry()->getDisplacement().y;
      zPos = kDetectorElement->getGeometry()->getDisplacement().z;

/*magnification*/
      xPos *= toRealCanvasSize;
      yPos *= toRealCanvasSize;
      zPos *= toRealCanvasSize;
      detectorPosition[n] = KonoeTcl3DVector(xPos, yPos, zPos);
      //if(Box)
      this->createBoxGeometry(kDetectorElement, n);
      //else if(...)
      //create...Geometry
      n++;
    }
  }
}

void KonoeTclEventViewCanvas::createBoxGeometry(
                  KonoeDetectorElement* kDetectorElement, KInt n)
{
  KChar geometryName[256];
  KFloat width;
  KFloat height;
  KFloat depth;
  width = ((KonoeBoxGeometry*)kDetectorElement->getGeometry())->getWidth();
  height = ((KonoeBoxGeometry*)kDetectorElement->getGeometry())->getHeight();
  depth = ((KonoeBoxGeometry*)kDetectorElement->getGeometry())->getDepth();

  width *= toRealCanvasSize;
  height *= toRealCanvasSize;
  depth *= toRealCanvasSize;

/*magnification*/

  sprintf(geometryName, "box_%d", kDetectorElement->getID());
  solid[n] = new KonoeTclDrawableBox(geometryName,
                                     (KInt)width, (KInt)height, (KInt)depth);

  KonoeTclRotationMatrix rm;
  this->createPhysicalVolume(KonoeTclTransform(rm, detectorPosition[n]),
                                                                solid[n], n);
}

void KonoeTclEventViewCanvas::createPhysicalVolume(KonoeTclTransform trans,
                                                   KonoeTclSolid* pSolid,
                                                   KInt n)
{
  myPhysicalVolume[n] = new KonoeTclPhysPlacement(trans, pSolid);
  myPhysicalVolume[n]->replace(KonoeTcl2Dimension());
  myPhysicalVolume[n]->setSpaceManager(spMan);
}

void KonoeTclEventViewCanvas::locateGenerator()
{
  KDouble* tmpDistanceToEye;
  KInt tmpNum;
  KDouble tmpDistance;
  KInt i;
  KInt j;

  if(drawNumber != NULL) delete [] drawNumber;
  drawNumber = new KInt[totalOfPhys];
  tmpDistanceToEye = new KDouble[totalOfPhys];

  for( i=0; i<totalOfPhys; i++){
    drawNumber[i] = i;
    tmpDistanceToEye[i] = myPhysicalVolume[i]->getDistanceToEye();
  }

  for( i=0; i<totalOfPhys-1; i++ ){
    for( j=i+1; j<totalOfPhys; j++ ){
      if( tmpDistanceToEye[i] <= tmpDistanceToEye[j] ){
        tmpNum = drawNumber[i];
        drawNumber[i] = drawNumber[j];
        drawNumber[j] = tmpNum;

        tmpDistance = tmpDistanceToEye[i];
        tmpDistanceToEye[i] = tmpDistanceToEye[j];
        tmpDistanceToEye[j] = tmpDistance;
      }
    }
  }
  delete [] tmpDistanceToEye;
}

void KonoeTclEventViewCanvas::setDimension( KonoeTclDimension dimension )
{
  dim = dimension;
  setSize(dim.getWidth(), dim.getHeight());
}

void KonoeTclEventViewCanvas::setSize( KInt width, KInt height )
{
  KonoeTclCanvas::setSize( width, height );
  draw();
}

void KonoeTclEventViewCanvas::setViewLotation(KFloat value)
{
  spMan->setViewLotation(value);
}

void KonoeTclEventViewCanvas::setViewAngle(KFloat value)
{
  spMan->setViewAngle(value);
}

void KonoeTclEventViewCanvas::rotateX(KFloat rotX)
{
  KInt i;
  for( i=0; i<totalOfPhys; i++ ){
    myPhysicalVolume[i]->rotateX( rotX );
    hitCanvas->rotateX( rotX );
  }
}

void KonoeTclEventViewCanvas::rotateY(KFloat rotY)
{
  KInt i;
  for( i=0; i<totalOfPhys; i++ ){
    myPhysicalVolume[i]->rotateY( rotY );
    hitCanvas->rotateY( rotY );
  }
}

void KonoeTclEventViewCanvas::draw()
{
  this->locateGenerator();

  KonoeTclGraphics* g = this;

  g->setColor(KonoeTclCore::bgcolor);
  KonoeTclCanvas::clearAll();
  this->drawImagenw(10, 10, konoeLogoImage);
  g->drawString( dim.getWidth()/2 + dim.getOrigX()/2,
                 dim.getHeight() + dim.getOrigY() + 30, myName );

  if(enableGeometry){
    KInt i;
    if(forceWireFrame) 
    for( i=0; i<totalOfPhys; i++ ){
      myPhysicalVolume[drawNumber[i]]->setForceWireFrame();
      myPhysicalVolume[drawNumber[i]]->draw(g);
      if(slowDraw)
        this->update();
    }
    else
    for( i=0; i<totalOfPhys; i++ ){
      myPhysicalVolume[drawNumber[i]]->unsetForceWireFrame();
      myPhysicalVolume[drawNumber[i]]->draw(g);
      if(slowDraw)
        this->update();
    }
  }
  
  if(scale) drawScale();

  if(isHitCanvas){
//static KInt s = 0;
//if(!s){
    hitCanvas->setEventViewCanvas(this);
//s=1;
//}
    hitCanvas->draw();
  }
}

void KonoeTclEventViewCanvas::setSpaceManager(KonoeTclSpaceManager* pSpace)
{
  spMan = pSpace;
  KInt i;
  for(i=0; i<totalOfPhys; i++)
    myPhysicalVolume[i]->setSpaceManager(spMan);
}

void KonoeTclEventViewCanvas::setMagnification(KFloat value)
{
  spMan->setMagnification(value);
}

void KonoeTclEventViewCanvas::setCenterEye(KInt eyeX, KInt eyeY)
{
  spMan->setCenterEye(eyeX, eyeY);
}

void KonoeTclEventViewCanvas::setEventHitString(KChar** pstr, KChar* fColor,
                                                KChar* fName, KInt fSize)
{
  hitCanvas->setString(pstr, fColor, fName, fSize);
}

void KonoeTclEventViewCanvas::setEventHitVisAtt
                             (KInt i, KInt RR, KInt GG, KInt BB)
{
  hitCanvas->setVisAtt(i, RR, GG, BB);
}

void KonoeTclEventViewCanvas::setSlowDraw()
{
  slowDraw = KTrue;
  hitCanvas->setSlowDraw();
}

void KonoeTclEventViewCanvas::unsetSlowDraw()
{
  slowDraw = KFalse;
  hitCanvas->unsetSlowDraw();
}

void KonoeTclEventViewCanvas::drawScale()
{
  KonoeTclRotationMatrix rm;
  KonoeTclDrawableHorizonLine* aLine1 =
                    new KonoeTclDrawableHorizonLine( "aHorizonLine",
                                    100 );
  KonoeTclDrawableXLine* aLine2 =
                  new KonoeTclDrawableXLine( "aXLine",
                                  100);
  KonoeTclDrawableVerticalLine* aLine3 =
                  new KonoeTclDrawableVerticalLine( "aVerticalLine",
                                  100 );
  KonoeTcl3DVector aVector1
                  ( 100, 0, 0 );
  KonoeTcl3DVector aVector2
                  ( 0, 0, 100 );
  KonoeTcl3DVector aVector3
                  ( 0, 100, 1 );
  KonoeTclPhysPlacement* myXLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector1),
                                  aLine1, spMan );
  KonoeTclPhysPlacement* myYLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector2),
                                  aLine2, spMan);
  KonoeTclPhysPlacement* myZLine =
         new KonoeTclPhysPlacement( KonoeTclTransform(rm, aVector3),
                                  aLine3, spMan);
  myXLine->replace ( KonoeTcl2Dimension ( ) );
      myXLine->setVisAtt( 255, 0, 0 );
  myYLine->replace ( KonoeTcl2Dimension ( ) );
      myYLine->setVisAtt( 0, 255, 0 );
  myZLine->replace ( KonoeTcl2Dimension ( ) );
      myZLine->setVisAtt( 0, 0, 255 );


  myXLine->draw(this);
  myYLine->draw(this);
  myZLine->draw(this);
  delete myXLine;
  delete myYLine;
  delete myZLine;
}

void KonoeTclEventViewCanvas::drawRunNumber(KInt num)
{
  KonoeTclGraphics* g = this;
  KChar rNum[32];
  KInt x;
  KInt y;
  KonoeTclDimension d = getSize();
  x = 10; //d.getWidth() - 250;
  y = d.getHeight() - 50;
  sprintf(rNum, "Run Number   : %ld", num);
  KonoeTclCanvas::drawString(x, y, rNum, KW);
}

void KonoeTclEventViewCanvas::drawEventNumber(KInt num)
{
  KonoeTclGraphics* g = this;
  KChar eNum[32];
  KInt x;
  KInt y;
  KonoeTclDimension d = getSize();
  x = 10; //d.getWidth() - 250;
  y = d.getHeight() - 35;
  sprintf(eNum, "Event Number : %ld", num);
  KonoeTclCanvas::drawString(x, y, eNum, KW);
}

void KonoeTclEventViewCanvas::drawHitPoint(KFloat px, KFloat py, KFloat pz)
{
  KonoeTclGraphics* g = this;
  KChar hPoint[256];
  KInt x;
  KInt y;
  KonoeTclDimension d = getSize();
  x = 10; //d.getWidth() - 250;
  y = d.getHeight() - 20;
  sprintf(hPoint, "Hit Point( %.2lf, %.2lf, %.2lf )", px, py, pz);
  KonoeTclCanvas::drawString(x, y, hPoint, KW);
}

void KonoeTclEventViewCanvas::drawHitTime(KChar* ktime)
{
  KonoeTclGraphics* g = this;
  KChar hTime[256];
  KInt x;
  KInt y;
  KonoeTclDimension d = getSize();
  x = d.getWidth() - 250;
  y = 45;
  sprintf(hTime, "Hit Time : %s", ktime);
  KonoeTclCanvas::drawString(x, y, hTime, KW);
}

void KonoeTclEventViewCanvas::setImage()
{
  konoeLogoImage->setData(
"R0lGODlhyABKAPcAAAAAADMzM2ZmZpmZmczMzP///5lmZmYzM8yZmZkzM8xmZswzMzMAAGYA"
"AJkAAMwAAP8AAMwzAMxmM5kzAJlmM8yZZmYzAMxmAMyZM5lmAMyZAJmZZmZmM8zMmZmZM8zM"
"ZszMMzMzAGZmAJmZAMzMAMz/AJnMAMz/M5nMM2aZAMz/Zpn/AGaZM5nMZjNmAGbMAJn/M8z/"
"mWb/AGbMMzOZAJn/ZjPMAGb/MzP/AGaZZjNmM5nMmTOZM2bMZjPMMwAzAABmAACZAADMAAD/"
"ADP/M2b/Zpn/mcz/zAD/MwDMMzP/ZjPMZgCZM2b/mQD/ZjOZZmbMmQBmMwDMZjP/mZn/zAD/"
"mTPMmQCZZmb/zADMmTP/zAD/zGaZmTNmZpnMzDOZmWbMzDPMzAAzMwBmZgCZmQDMzAD//zP/"
"/2b//5n//8z///D//wDM/wCZzDPM/zOZzABmmWbM/wCZ/zNmmWaZzAAzZgBmzDOZ/5nM/wBm"
"/zNmzAAzmWaZ/wAzzDNm/wAz/2ZmmTMzZpmZzDMzmWZmzDMzzAAAMwAAZgAAmQAAzAAA/zMz"
"/2Zm/5mZ/8zM/+jo//Dw/9jQ/zMA/+DY/zMAzGYz/9DA/+jg/8iw/9jI/7iY/2YzzDMAmYhI"
"/5hg/5lm/6h4/7CI/8Cg/9C4/+DQ//Do/4A4/4hA/5BQ/6Bo/7CA/7iQ/8io/2YA/4Aw/4g4"
"/5BI/5hY/6Bg/6hw/7iI/8CY/9jA/4Ao/4gw/5BA/5hQ/6ho/7B4/8ig/9Cw/+jY/4Ao8Igo"
"/5A4/5hI/6BY/7Bw/7iA/8CQ/9i4/+DI/5Aw/5hA/6BQ/6hg/7h4/8CI/8iY/9Co/7Bo/2Yz"
"mZlmzDMAZmYAzJkz/8yZ/9Cg/9iw/+DA/+jQ//Dg/5kA//DY/5kzzGYAmcxm/5kAzMwz/8wA"
"/5lmmWYzZsyZzJkzmcxmzMwzzDMAM2YAZpkAmcwAzP/o///w//8AzMwAmcwzmZkAZv8AmZkz"
"ZsxmmWYAM8wAZv8AZswzZpkAM8wAM/8AMyH5BAAAAAAALAAAAADIAEoAAAj/AAsIHEiwoMGD"
"CBMqXMiwocOHECNKnEixosWLGDNq3Mixo8ePIEOKHEmypMmTKFOqXMmypcuXMGPKnEmzps2b"
"OHPq3Mmzp8+fQIMKHUq0qNGjOyHBg1RKKVOnTZcKjPd0adSn8R4y9dWray9bWRnGa1pqLFOz"
"DaGqhbfG4VhkvVh1HVXqYamreNdW3ds0bMd4vGLpGoxqsK7ChmX5ikfLMOLDg4n1akiql6xh"
"uIC5uuVKWKxapBbWkmXYsa5hrBZe2lX6saxoDC9Fm6XrFTDOwV7pOoaM4ShixCATNj28uPBh"
"kz9eQibrNrBgzm/hihXtF6QCyXjZvuUc165Vv/wi//z1DFYwV69m8XImK3Nm1OILlvrVa1h0"
"zcCG1U0YjxStXN1R98t+CHXTiy6b4ZKKMbzssh0wr9RC4EHx/OILMa44B510r3To4SuZcccd"
"dK4kI1Ipu5wnHTC43CJLNwYVI510siAT30GXHAMLd7jMYqJApbAC4C3S0XJjQaQgeB50LabG"
"kC+vnMcLPAsFGYt7soziFzKwsCjdLhOO12WLmwEDyza/cJNmmrXwokuLmbnCzHUisfJchufl"
"8stBqrCYSzRHGmSLLOdBSMtBq8CpmZEL2eKel8DoEuZBpegCDDN7KkQKLyH2OCkyIbIIZpXD"
"EOneLbvQaVCOXQbj4kjxyP+CJ56qHJSMealw4xCULLJ46EG/dHnLebg4mRApuGRoqnSwLQTJ"
"m8YidOuIroxqUDyxzKiZMaTiyRkwySU0LS7NhvSsiiwWa9CtwMgyKaXaDcuiKoEWYEyGmQUT"
"S6YI0XJqqLHUC6QuevK3CoAqNvOuQL7c1mIwudiiUCn2xQlMwQuB6orEIgV7J3QR/yJymr6Y"
"R0xoQIqsppq/XNLNajxqlsslFF7yCy2FFloLf7K8YkyyH7tSCzc0IwQqKAWU0o3KIl9yyc+a"
"QfcKxwYp7cuKys6ikC8trhjMmWuyzHSatrwyTDIjp1n0RtF4vZkuvtgity29dLkvQZCQMrct"
"aF//x8uK0uHSW0HxTEIKKb/Ucudzt+iCskGkvEJMKc0Vyp0uoySz8DG47FwAJL/MPWAvnUrn"
"OUHxXIJ46NSeJwxY/fY64yzI7C334dzgToossiRj++MZxZOKw87R4rvcdeNCy6SXiL6fLcI4"
"zJ0s4lWYTDLcZNXLshnmwkr2gvZYANfEt/hMJvwSVCnGBfS3DSmF6zKssrDASNAvpCQDP3Zu"
"swjajRTDDWdwcQzb2SJ9AzHefI63jbVlBBLMUFZmIqY3usECF9w6SDf4trZfAKhF6fKF+ibB"
"t8dtr3TAMIYtgCcQXgCjN9hyBZM4owtkaG5Vw9CF/QTyi2QwhRi9KtQx/9RHim3YAnz82wye"
"2rXC+HisUPkxoA8rlJBScINvqspI2SCFi1xww3eswAwvFpYmVQGmTCAkhl96uI1M6GogvSDe"
"jFRhCzQR5BLDmBrD4kStAiKwANFwRQYHgr94iAIXZHoOLPjVQ99ZZyDJcBtnZFHH/cFRhvi6"
"BTGON7fQ1OJ0lNJfR6AxPy/NInS0uKCPEFKKS4gHGZmUjiua1R/RiQdnmWQRL+R2Qx7qZiCl"
"iIWrOlXDXk5lFq8YHCFFlouYnedXhZubDwnii1k9h4lyc+AxusOdof2OG7Gg2njeqBFs4WeY"
"QzOGbXLhiz9+rix3HIYMZSeMPeXNiHybUKWo5f8lOsqNX7UgIEFY0SmH1eKAeNOFMBxYgJbt"
"IpauwIUIC4dPOxJkF4tbEiVvF5ZKLWmAvaig6NxEpYktjCKPkKeruseNVdgmGJ2b1Fjis00V"
"rWgXAuFGJkQnnzGZylfS3A9GR6E+8+SSksZMxitShToL2QaE76nLJPDJt/gQyprVCqpAthgq"
"WORub8g4xiuolxLyzWoYq4gemXgRH6YcRBbfKhQuJlMKTk7zfqXC123+NDc7/uIViyzIM7wl"
"wz+hD47AGOL9JkELxs1TkEnjpC0YWgAE3WlYnetrpkj3U1wUI3S2QEY0GpOZaJnEhUvKEMFu"
"gydYTGimgnqpt14xoCv/ys2iA83XrNi5t9AoVRaEI0Ve7+SKXd4QW8JgYeiGqyKJQsK2vIwP"
"srjIoq90MiuzkF2LiMGLWcxiF6cIkRdTEg8EcSdDwqhFLuQKIaq59SB/0y2LZPHcnfJyUseI"
"a5l2MYq9mahP5UpZQBdnprjtqVKwYGji5AudYVxCp3v7Iyx5RCRXDCNu18XjsvAzomvKkKwo"
"IcUHh3WLXCSjcpuBTnKUdqRKsXZxn/1d1WKR0YjS4nfdIIbgCuKybuzIW7igow9hWQz5cGMW"
"P8XkyeoozRtlF12ZmYXtsocJi7UoF7RwBi8CA6LbhOskV8PkfElxr8XdghcPRhtCkLUiqPpC"
"/6R1/GMyhIFG7vB1b7nTTZjCU4CaJig/vtgGN35m2ksko2KOTaxkj1g15uLLutcVxUd9BVpb"
"aKMXGJIo4R4xksYySTPH+IVYKdxFDLMQkqXDJIalmUWB1ILCK6rR76IRZPlk6hIfvI10VJgJ"
"hSIwTTv6Vr5WQdVsCsqa56lhX1dYCiC6DRrQlZsvhOFa1E1ihx+pVCKDkVxS+CIXBRVkdOEb"
"UQ25Ihe1623VdByzWwgD0nPjhgtFeD9++auUjfPFt2Px2kuQLoicEQY0bJeMGxVjnl3DxSoW"
"nb9cWDMXhzNgLooMzMONxFHc9KrvsgvC8zQj3ZQtwJOPajvcDkTEXP9ElQHrCIvkFuTahASQ"
"XpXX2EEK5MG1qHOGhtFfPFfNUjJc0TCYrNm25cwVzTC07ZBxCkCBLn/YG0mYLccL0D7jqd/i"
"LaMNMosZxprgrS5AHDcsDF8s+nqSqx4Cy7zS/FxQmTy8hOISrshVy42ckAw6nggI53w2Npeh"
"XjQyiJEJm3ES2x9pT9Ru8YpegPbbzlTe7Q6C5Mf673fxabaYk0WLSnfSX6KwdUF+QWevcSaZ"
"o49HQJ1Z3HTPDfEFcKFuXUHfaDNbfm0nF5zRNJ/2gS4Z2/hRSPDo4YvhT27IyK7egeFPYw5k"
"5I6F9woNwrUlalJkJSfFLjR9R9hzSo7A2CT/gUBXgFeHiFYEl27p82XiutqOmY/CxSIlW/Cq"
"HU5gGBnF/Iapix7O7W85k29yA3v+Ij2ZAW8m9znN4CUy9AqwYV/SRArUhnh8hiR0Vj67xC8v"
"Mz6nUigqpG4FoQr5gh+H4nn5FEi3oSLG8AsQOH0HwQ1yIRLQIEG3YQzNMze9MCRx4j2ThySK"
"ghui4F9hojF30jkNVWwHVDY4pT7uFHuXhUhf0UsDkjTmEVe34E/xNnpDgknFkBWLFh7Dc3Sd"
"934I8W1EFRKWAlVTE22j8CZyVAy1U3+o0xzP1lvicQnEQGHAwG8F0HcF1yf0NhA2Iy2vsEQ8"
"yGxWNBBshy9YeHeo/wMK2oILtBVZQlhewlZi+uNfNxIPo5AL1QYSmSAMeqgncJYM29Q1edJz"
"lLWIl3ddBHEMmeQKuqAr8fCFl/Am4lQARJMQXacszeBfFicQBCVfH3hbDjQKupUbHGOCe4IM"
"L1Vh/SdZpxYPqpAhX+YRJ0Rhs9ANksUNnJUh+FJALng/zTQruwCCwoh1+YEyN3hb2ROKutBq"
"FWgrT0VAy4ZbzaYZI4I5TQZJIwYhgwMJ3ZgV5tcriXUJxYZE7WMJivcK2yASztY1KdSO/+Rj"
"1BULcXgQ5FMor8AKWcgwUYIvvXNy/nUdxRAp4tFKC7EL3PEKdodFBJEMIxZR4jiO04IvF/9G"
"SEJYAN1gKbghQ2MYYQNhCw7iMLqAfxYxCqIYfz5Skh4Ea4hUQHelPg/FOLgga9NXCrwCJ/QC"
"TJyEJqqXLA52P7BXEGUTDLLQd6d2b3eiC9HAS+1jCxfEOO4yEAKpWQKRjR7GC65XQqQADakg"
"W88BShzBiTLnWO5WC66HOFf1adwRC72QgEBSDK8gbLFwDL2gb4q3XaxQUjzUV6TQDbWwHT3C"
"SEiJUWwFgZIJCf9xVrSQmbaATHcCC9CAbb9QUfADCdMWi0wCC7zDO8QgDF0GZYElHxfRJqmQ"
"L1hzJ2O1C76ADLwARDQIJ8FwCs55JPEQRh6GSF0miZIRJu6HPIH/UVC3AAu8AD8hF1siVGnO"
"RxC2oGPAoBmSKIm3oRuhVhB3eXeZwAvNADSflmLWFypQRSS7kHkn5RC2EA2sEA0K2qAMygrH"
"YAzGQAu1Aw0LyqANygrPQAsSugrIkJ4F4G+qkAvD4Im5EAsWug1hR1F1pFMO6qALegkq2RDI"
"UBe1yFFVNFrDAAueCAvDQAvRMAqeeT9B1Q0vuqAXCqETOqGviaFJyqDpUzjz+BHx0IJTiSQy"
"phC/gAy+MAqBdqBFdF8MITISAVp4x0q2oG/9BaKUGGdiIY1ImTRFdGodgZB2iBC2156Qo1UU"
"MlW3U5b4qWYRUQp604SjZ1+SCUw6FXxs/0pILUintlZEmQCoG3F2YScQfYdQCuE+lXQk+fNP"
"SElRkzAR+DOk/BGm+UQpofM+l3otftiqSXN84wYS0WRA7kSoUnSg7eOn+pNFhOo7ydCoU8Gr"
"ukopruQWJPRP+tRIwfoQmSpoZbEG8UAV+JOJeDl8uZM7usMNsNcNpKCt3/qtU7gQ1Xo9upMm"
"ofkQD4Y7MioRxcpjX2Su36omh3OsDrGu4Xo43+o03+A06JqvufNIsDIVBMsfBRsRFZI29ioR"
"YRGn5WQzYuOwCNGwA0GxBWuxSJGxGruxHNuxHvuxIBuyN0EkuiYisqQMkiicubCysOCjO8oM"
"ghELxOAJv9kMu/9ws97VXVsmoaqwpBv6DLVwDEErCrXgFXHBoPrWpb5gCenWRtejP9zADZPA"
"MjbTDVZ7F/DwCDMFCVTBtZCgFHehNErzCEsjMogTtfmTCcBnX8gwCtrApfqGCaywoF0BDdDw"
"SdBwDKtwDM+wChxqDKqwHltWDA1SDLuQCrvAO4kLHLEQC8xgGDu6DLlwCsKQC8LZIYhEJCYr"
"EDPyL72iDK8gDMJAuTxKosOQQ48LHLJADLKQuIdruN5FuFvmDBIaChO6Cs+gCccgtLVwt0bb"
"oGqKDMLLZE6rP7gjNlYro2HrtdPKtQXAFs0LD2ShNFbbryoDrsmQCcFnRG3btl6qDZj/wKBG"
"Cw1Eu7tBewy0sKE86wyBywvFUAzfNQs22wyrCxz2Oxg7erqlS7mYiwvBAFOQIhBydH6I5ArK"
"wL+T27Itexowa78127rfVQygIMFbpgo9O6E/u7u8Cw1ekQ1zGw36pg3PWTvDyzf6Y7xqcgnf"
"8Asvc7VXwbVt8Tm+RxVjIb13kbxpgz+Hcz22MwpcOgqjEA3hywoe3Asc/ElCq7e0sMR/275b"
"5l3edbO7QL/0Cxy6EByncbq5ILmWCyLogUjJ6AqcWzqdG1EfsrKeeLqlIbMyy7qta7OzQLjw"
"274SyqRLrLfHoAnk+0lygaS+wAra4KU+LE1q+636M7Uj8zJh/1sWSkHDMjwV1zG9rWS1LSMy"
"U5s7T8vDoSW8SasNSCoXRtwLSKzBz6C+gMsgW+a+USzF9Fu/xCAY+KsLPsqjp5ALoQshtqGP"
"vcK51GUq3Bm6lbuyJYoKOaQLzPDKbpwKNou4uxDHshu4FsykGry7d1u0vSAKc8sKvgDCXiq8"
"g1y82ZuvVdsy3bDIX9s+FVsAbTGt07rIl0C2v7DCZos9T8tk3izCSSu3chENRWu3o7y3S8yz"
"gJvKsXu4iSsLzYDMugDLWVy6Xdy/7mE5nAs4ZBJRKSu6lovGalzM9ksM9KvMzRzBsxu4E9qz"
"tOC3eCy0dtsV2RAXCxrCw+t62Xs9Uf8bsFVbzo9wF8yLzvj5vF8btslrM2Gjr2prRNsgvMjw"
"tvqmzdl8zeXbuxq8xM8AuE5MuBEsxQgNnI8LyzuKCiY6uaLrHkBDYWPcy8mCC6aA0cHcssOA"
"GI2ruqvbujgbx+pRwSaNwc+gwXfru13x0iCstD5s1CZ8wmvSNOVcznfhyPEQw6iTNDN1w2Sr"
"wr9wyTusvW2EfECsbyCsoHRrxKJgvrvrtxvKobTLCxMswfE7xcosC7HA2sHxuGqMxisbuqGb"
"uTPCHQLcWfICxrT9Ciu7DGzt1lfsxq2rzKk9uKpAuxIq1bQA2p8kChx8tH/tC2+rDaE1N9q7"
"w+LaNOQctkv/Ma0UIq1fOxZK02NCjT+YfD3BR8JIndkN6hVEK8rny7emTNUE/b5SvNpx/cqP"
"exj6uwyUK7q24cUFhQtlrS0Jwp2Xy7KzXMwLbb/0a9CxS9ep3LMm3dx5vQpBS74s3QvAC9jD"
"uw3FC7CTIDLJ2w3SS8PO27Dr7LWLPMkqU+JQ97QQ6MNKm7RETMSivNehrcQ8m9ztC8Xfdbit"
"7MqC0db/jcbAPJyzcgu8PKAwZca4ULnB3KM5NAwxC9eKO9d1ncrsi8EnncSf9El9HQ3ZIMR/"
"7cOjgE/AGglQK7VN4zJiK73fTScsnhVNwRRi+zIis8K6w8Pae90+jM8YGhddUQvQ/53EfTva"
"f4vKExzHzYzVwCmzMIu/sADcttwhwgkMylBuI1jWUMVaKEvbk8uyp/uyV3zMrNsMiIu40hDH"
"qcwLVM3EYa63Gx7dHvzSSdu9Tau25hq1k73Cybu8db7Yxj7DfVEK8NANZNuvZYu216O2c0PC"
"IizCGFrEh867u9vcHGq70LwezvxdqfCb+23MsYDkJVq6lfshlkMmvLwscZUZl1vlLsvRrxzX"
"CC3FzgwK7ssgdSzVe2u+vsvBfSzEIezDJCxNv44/K+wyPYbYNAzeXNu8XavnZGu1fI6uX4TJ"
"O5UJyHDUma3ZTS0KRuzP883EPJvKEgy/Q36zwMm4g8EMSf8uufy76cqQcmPcZs9x1gMu2zy6"
"ozEfGfhuszj76nEMCloGuByavny74WRu6B+szfqW1CQs4uGMPQEL59/Qry/eF8zrtUtB5+Xt"
"NDYj41Jb2Qy0yW1r7XQbDU5dzburCaVMC7br7VrWXaCwyvo9s8HB0Dvq0AIOIf8iIk+ua51i"
"wMKJ0T9Poqfx4LqA73KNs+9b19EM5hoctL1rzUer6885yOkGrCdsOIjj7MOO4n0RD1r7tS6u"
"59S7NDbzDdkKdfZ1PLWj5prNFW5ft6JMyoxex+uR96d9s4gLnKvr97KsxSs7uqILIWXSuQfe"
"KecBuqJ7Cr6t0cWcuoyb1cZt9O90CwoXbMcobb5kTvBub/BT7823ZcL5c7wl3q8PXwo5rdM0"
"LL0/DdTzQckjc/YzPu1pChC+tPkaFQ1TtGisei2EVqvhsWOrntGiZcyiKmeqeG0sNmvXR1kh"
"mxGTRYyYLpSwYOVamSuXsFOvZOLCBazmrZoFAgIAOw==");
}
