//=====================================================================
/*

    KonoeTclEventViewCanvas.hh

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLEVENTVIEWCANVAS_HH
#define __KONOETCLEVENTVIEWCANVAS_HH

#include <ui/KonoeTclCanvas.hh>

#include <ui/KonoeTclDimension.hh>
class KonoeTclEventHitCanvas;
class KonoeTclPhysPlacement;
class KonoeTclSpaceManager;
class KonoeTclWindow;
class KonoeDetectorList;
class KonoeTclSolid;
class Konoe3Vector;
class KonoeTcl3DVector;
class KonoeDetectorList;
class KonoeDetector;
class KonoeDetectorElement;
class KonoeTclRotationMatrix;
class KonoeTclTransform;
class KonoeTclImage;

class KonoeTclEventViewCanvas : public KonoeTclCanvas {

  public:
    KonoeTclEventViewCanvas( KonoeTclWindow* kwin, KChar* kname,
                             KonoeTclDimension dimension, KChar* pTitle, 
                             KInt pTotal );
    KonoeTclEventViewCanvas( KonoeTclWindow* kwin, KChar* kname,
                             KInt canvasSizeX,
                             KInt canvasSizeY,
                             Konoe3Vector pworldSize,
                             KonoeDetectorList* klist, KChar* pTitle );
    ~KonoeTclEventViewCanvas();

    void setPhysicalVolume( KonoeTclPhysPlacement** pPhys,
                            KInt startID, KInt endID );
    KonoeTclPhysPlacement* getPhysicalVolume(int i) {
      return myPhysicalVolume[i];
    }
    KInt getTotalPhysicalVolume() { return totalOfPhys; }
    void setDimension( KonoeTclDimension dimension );
    void setSize( KInt width, KInt height );
    KonoeTclDimension getSize() { return dim; }
    void draw();
    void setSpaceManager(KonoeTclSpaceManager* pSpace);
    inline KonoeTclSpaceManager* getSpaceManager() { return spMan; }
    void setMagnification(KFloat value);
    void setCenterEye(KInt eyeX, KInt eyeY);
    void setViewLotation(KFloat value);
    void setViewAngle(KFloat value);
    void drawScale();
    void drawEventNumber(KInt num);
    void drawRunNumber(KInt num);
    void drawHitPoint(KFloat px, KFloat py, KFloat pz);
    void drawHitTime(KChar* ktime);
    KInt* getDrawNumber() { return drawNumber; }
    KFloat getRealCanvasSize() { return toRealCanvasSize; }
    void showGeometry(){ enableGeometry = 1; }
    void hideGeometry(){ enableGeometry = 0; }
    void setForceWireFrame() { forceWireFrame = KTrue; }
    void setScale() { scale = KTrue; }
    void unsetForceWireFrame() { forceWireFrame = KFalse; }
    void unsetScale() { scale = KFalse; }
    void setEventHitCanvas() { isHitCanvas = KTrue; }
    void unsetEventHitCanvas() { isHitCanvas = KFalse; }
    void setEventHitString(KChar** str, KChar* fColor="black",
                           KChar* fName="times", KInt=12);
    void setEventHitVisAtt(KInt i, KInt RR, KInt GG, KInt BB);
    void rotateX( KFloat rotX );
    void rotateY( KFloat rotY );
    void setSlowDraw();
    void unsetSlowDraw();

  private:
    void locateGenerator();
    void createGeometry();
    void createBoxGeometry(KonoeDetectorElement* kElement,
                            KInt n);
    void createPhysicalVolume(KonoeTclTransform trans,
                              KonoeTclSolid* pSolid,
                              KInt n);
    void setDetectorList( KonoeDetectorList* klist );
    void setImage();

  private:
    KonoeTclEventHitCanvas* hitCanvas;
    KonoeDetectorList* detectorList;
    KonoeTclSolid** solid;
    KonoeTclPhysPlacement** myPhysicalVolume;
    KonoeTclSpaceManager* spMan;
    KonoeTclImage* konoeLogoImage;
    KFloat worldSizeX;
    KFloat worldSizeY;
    KFloat worldSizeZ;
    KFloat toRealCanvasSize;
    KonoeTcl3DVector* detectorPosition;
    KInt totalOfPhys;
    KInt* drawNumber;
    KonoeTclDimension dim;
    KChar myName[64];
    KInt enableGeometry;

    KBool forceWireFrame;
    KBool scale;
    KBool isHitCanvas;
    KBool slowDraw;
};

#endif
