//=====================================================================
/*

    KonoeTclOfflineHistogram.cc

    Version 0.0.1   18-Dec-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclOfflineHistogram.hh>
#include <ui/KonoeTcl1DHistogram.hh>
#include <ui/KonoeTclDrawCanvas.hh>
#include <ui/KonoeTclDrawPanel.hh>
#include <ui/KonoeTclDrawCanvasController.hh>
#include <ui/KonoeTclDrawCanvasMouseListener.hh> 

#include <analysis/KonoeHistogramCM.hh>
#include <analysis/KonoePhysicalValueCM.hh>
#include <analysis/Konoe1DHistogram.hh>
#include <analysis/Konoe1DHistogramBand.hh>
#include <analysis/Konoe1DHistogramList.hh>
#include <analysis/KonoeLinearDataScale.hh>

#include <system/KonoeInputObjFile.hh>

KonoeTclOfflineHistogram::KonoeTclOfflineHistogram(
                                  KonoeTclWindow* pwin, KChar* pfName)
{
  kwin = pwin;
  readFromFile(pfName);
}

KonoeTclOfflineHistogram::~KonoeTclOfflineHistogram()
{
  delete drawCanvas;
  delete drawPanel;
}

KInt KonoeTclOfflineHistogram::readFromFile(KChar* pfName)
{
  strcpy(fileName, pfName);
  KonoeInputObjFile* iFile = new KonoeInputObjFile( fileName );

  iFile->installCM( new KonoeHistogramCM );
  iFile->installCM( new KonoePhysicalValueCM );

  object = iFile->get();

  if( !object ) return 0;


  switch( object->getClassID() )
  {
    case KonoeHistogramCM::CID_1DHISTOGRAM:
      init1DPanel();
      break;
    case KonoeHistogramCM::CID_1DHISTOGRAMLIST:
      init1DListPanel();
      break;
    default:
      ;
  }

  delete iFile;
}

void KonoeTclOfflineHistogram::init1DPanel()
{
  KChar aName[256];

  hist1d = (Konoe1DHistogram*) object;

  drawPanel = new KonoeTclDrawPanel(kwin, tmpnam(aName));
  drawCanvas = new KonoeTclDrawCanvas(drawPanel, tmpnam(aName),
                               hist1d,
                               KonoeTclDimension(400, 400),
                               "1D Histogram Panel",
                               new KonoeLinearDataScale(0.0, 100.0, 100));
  drawPanel->setCanvas(drawCanvas, "");

  drawCanvas->setBackGround("white");
  drawCanvas->setAutoVertical();

  mouseListener = new KonoeTclDrawCanvasMouseListener;
  drawCanvas->setEventProcessor(mouseListener);
  mouseListener->addMouseMotion(drawCanvas);
  mouseListener->addMouseAction(drawCanvas);
}

void KonoeTclOfflineHistogram::init1DListPanel()
{
  KChar aName[256];

  hist1dlist = (Konoe1DHistogramList*) object;

  hist1dlist->reset();
  histNum = hist1dlist->getCount();
  currentHist = 0;

  Konoe1DHistogram* histogram = hist1dlist->get( currentHist );
  KonoeLinearDataScale* scale =
       new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                 histogram->getScale()->getMaximum(),
                                 histogram->getScale()->getBinCount() );
  Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
  hist1d = band->getNewHisto( histogram );

  drawPanel = new KonoeTclDrawPanel(kwin, tmpnam(aName));
  drawCanvas = new KonoeTclDrawCanvas(drawPanel, tmpnam(aName),
                                  hist1d,
                                  KonoeTclDimension(400, 400),
                                  "1D Histogram Panel -- No.1",
                                  new KonoeLinearDataScale(0.0, 100.0, 100));

  drawPanel->setCanvas(drawCanvas, "");

  drawCanvas->setBackGround("white");
  drawCanvas->setAutoVertical();

  mouseListener = new KonoeTclDrawCanvasMouseListener;
  drawCanvas->setEventProcessor(mouseListener);
  mouseListener->addMouseAction(drawCanvas);
  mouseListener->addMouseMotion(drawCanvas);

  controlFrame = new KonoeTclFrame(kwin, "controlFrame");
  
  currentHistLabel = new KonoeTclLabel(controlFrame, "currentHistLabel", "",
                                       0, 0);
  prevButton = new KonoeTclButton(controlFrame, "prevButton", "Prev", 1, 0);
  nextButton = new KonoeTclButton(controlFrame, "nextButton", "Next", 2, 0);

  controlFrame->setFrame();

  prevButton->setEventProcessor(this);
  nextButton->setEventProcessor(this);

  addMouseAction(prevButton);
  addMouseAction(nextButton);
}

void KonoeTclOfflineHistogram::reDrawHistogram()
{
  Konoe1DHistogram* histogram = hist1dlist->get( currentHist );
  KonoeLinearDataScale* scale =
       new KonoeLinearDataScale( histogram->getScale()->getMinimum(),
                                 histogram->getScale()->getMaximum(),
                                 histogram->getScale()->getBinCount() );
  Konoe1DHistogramBand* band = new Konoe1DHistogramBand( scale );
  hist1d = band->getNewHisto( histogram );

  KChar hName[256];
  sprintf(hName, "1D Histogram %d/%d", currentHist+1, histNum);

  drawCanvas->setHistogramName( hName );

  drawCanvas->setDataSource( hist1d );
  drawCanvas->draw();
  drawCanvas->update();
}

void KonoeTclOfflineHistogram::setSaveFileFullPath(KChar* fullPath)
{
  mouseListener->setSaveFileFullPath(fullPath);
}

void KonoeTclOfflineHistogram::mouseAction(KonoeTclButton* kbtn, KInt type,
                                        KInt button, Time time, KInt x, KInt y)
{
  if(type == KMouseRelease && button == LButton){
    if( 0<=x && x<kbtn->getWidth() && 0<=y && y<=kbtn->getHeight()){
      if(strcmp(kbtn->getName(), "prevButton") == 0) prevAction();
      if(strcmp(kbtn->getName(), "nextButton") == 0) nextAction();
    }
  }
}

void KonoeTclOfflineHistogram::prevAction()
{
  currentHist--;

  if( currentHist <= 0 ){
    currentHist = histNum - 1;
  }

  reDrawHistogram();
}

void KonoeTclOfflineHistogram::nextAction()
{
  currentHist++;

  if( currentHist == histNum ){
    currentHist = 0;
  }

  reDrawHistogram();
}
