//=====================================================================
/*

    KonoeTclPhysPlacement.cc

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclPhysPlacement.hh>
#include <ui/KonoeTclTransform.hh>
#include <ui/KonoeTclSolid.hh>
#include <ui/KonoeTclSpaceManager.hh>
#include <ui/KonoeTcl2Dimension.hh>
#include <ui/KonoeTcl3DVector.hh>

KonoeTclPhysPlacement::KonoeTclPhysPlacement()
{
  RR = 0;
  GG = 0;
  BB = 255;

  //myTrans = NULL;
  mySolid = NULL;
}

KonoeTclPhysPlacement::KonoeTclPhysPlacement( KonoeTclTransform trans,
                                              KonoeTclSolid* solid )
{
  RR = 0;
  GG = 0;
  BB = 255;

  myTrans = trans;
  mySolid = solid;

  distanceToEye = myTrans.distanceToPoint( KonoeTcl3DVector(-9999,9999,0) );
}

KonoeTclPhysPlacement::KonoeTclPhysPlacement( KonoeTclTransform trans, 
                              KonoeTclSolid* solid, KonoeTclSpaceManager* pMan )
{
  RR = 0;
  GG = 0;
  BB = 255;

  myTrans = trans;
  mySolid = solid;
  setSpaceManager( pMan );

  distanceToEye = myTrans.distanceToPoint( KonoeTcl3DVector(-9999,9999,0) );
}

KonoeTclPhysPlacement::~KonoeTclPhysPlacement()
{
  if(mySolid != NULL) delete mySolid;
}

void KonoeTclPhysPlacement::setVisAtt( KInt rr, KInt gg, KInt bb )
{
  RR = rr;
  GG = gg;
  BB = bb;
}

void KonoeTclPhysPlacement::setSpaceManager(KonoeTclSpaceManager* pMan)
{
  spMan = pMan;
  mySolid->setSpaceManager( spMan );
}


void KonoeTclPhysPlacement::replace( KonoeTcl2Dimension nowView )
{
  mySolid->realDrawReplace( nowView );
}

void KonoeTclPhysPlacement::rotateX(KFloat rotX)
{
  mySolid->rotateX( rotX );
}

void KonoeTclPhysPlacement::rotateY(KFloat rotY)
{
  mySolid->rotateY( rotY );
}

void KonoeTclPhysPlacement::dump()
{
  cout << "R : G : B = " << RR << " : " << GG << " : " << BB << endl;
  myTrans.dump();
  mySolid->dump();
}

void KonoeTclPhysPlacement::draw( KonoeTclGraphics* g )
{
if(mySolid!=NULL){
  if(0 <= spMan->getViewLotation() + mySolid->getRotateY() &&
          spMan->getViewLotation() + mySolid->getRotateY() < 1.57)
    distanceToEye = myTrans.distanceToPoint( KonoeTcl3DVector(-9999,9999,0) );

  else if( 1.57 <= spMan->getViewLotation() + mySolid->getRotateY() &&
                   spMan->getViewLotation() + mySolid->getRotateY() < 3.14)
    distanceToEye = myTrans.distanceToPoint( KonoeTcl3DVector(9999,9999,0) );

  mySolid->draw2( myTrans, g, RR, GG, BB );
  mySolid->draw( myTrans, g, RR, GG, BB );
}
}
