//=====================================================================
/*

    KonoeTclSpaceManager.cc

    Version 0.0.1   26-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <ui/KonoeTclSpaceManager.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTcl3DVector.hh>
#include <math.h>
#include <iostream.h>

#ifndef M_PI
#define M_PI        3.14159265358979323846
#endif

KonoeTclSpaceManager::KonoeTclSpaceManager()
{
  viewLotation = 30.;
  viewAngle = 20.;
//  originXX = 350;
//  originYY = 400;
  originXX = 0;
  originYY = 0;
  axisLengthReal = 800;
  centerPositionX = axisLengthReal/2;
  centerPositionY = axisLengthReal/2;
  centerEyeX = 100;
  centerEyeY = 300;
  reTargetX = 0;
  reTargetY = 0;
  magnification = 1.0;

  viewLotation = viewLotation * M_PI / 540.0;
  viewAngle = viewAngle * M_PI / 540.0;

  offsetScale();
}

KonoeTclSpaceManager::~KonoeTclSpaceManager()
{ ; }

void KonoeTclSpaceManager::setViewLotation( KFloat diff )
{
//  viewLotation = viewLotation + diff;
  viewLotation = diff;

//  if( viewLotation > 1.57 ) viewLotation = 1.57;
//  if( viewLotation < 0 ) viewLotation = 0;
}

void KonoeTclSpaceManager::setViewAngle( KFloat diff )
{
//  viewAngle = viewAngle + diff;
  viewAngle = diff;
  
  if( viewAngle > 1.57 ) viewAngle = 1.57;
  if( viewAngle < 0 ) viewAngle = 0;
}

KonoeTclDimension KonoeTclSpaceManager::reLocation( KonoeTcl3DVector pVec )
{
  KFloat targetX = (KFloat) pVec.getZ();
  KFloat targetY = (KFloat) pVec.getX();
  KFloat targetZ = (KFloat) pVec.getY();

  KFloat X1;
  KFloat Y1;
  KFloat X2;
  KFloat Y2;
  KFloat originX=0.;
  KFloat originY=0.;
  KFloat axisLength=0.;

  originX =  (KFloat) originXX;
  originY =  (KFloat) originYY;
  axisLength =  (KFloat) axisLengthReal;

  X1 = originX + axisLength - (originX + axisLength*cos(viewLotation));
  Y1 = originY - (originY - axisLength*sin(viewLotation)*sin(viewAngle));
  X2 = originX - (originX - axisLength*sin(viewLotation));
  Y2 = (originY - axisLength*cos(viewLotation)*sin(viewAngle)) - (originY - axisLength);

  KFloat replaceRateX = X1*(targetX / axisLength);
  KFloat replaceRateY = Y1*(targetX / axisLength);
  
  KFloat replaceRateX2 = X2*(targetY / axisLength);
  KFloat replaceRateY2 = Y2*(targetY / axisLength);

  KFloat drawableX = (targetX + originX) - replaceRateX - replaceRateX2;
  KFloat drawableY = (originY - targetY) - replaceRateY + replaceRateY2;
  KFloat tmpTargetZ = targetZ*cos(viewAngle);

  reTargetX = (KInt)(drawableX + 0.5);
  reTargetY = (KInt)(drawableY + 0.5);

  if( pVec.getZ() != 0 ){
    reTargetY = (KInt)(drawableY - tmpTargetZ + 0.5);
  }

  KInt toCenterPosX = (KInt)(-centerEyeX*magnification + centerEyeX);
  KInt toCenterPosY = (KInt)(-centerEyeY*magnification + centerEyeY);

  targetX = (KFloat)(reTargetX - offsetX)*magnification + toCenterPosX;
  targetY = (KFloat)(reTargetY - offsetY)*magnification + toCenterPosY;
         

  KonoeTclDimension target( (KInt)(targetX + 0.5),
                            (KInt)(targetY + 0.5) );

  return target;
}

void KonoeTclSpaceManager::offsetScale()
{
  reLocation( KonoeTcl3DVector(centerPositionX, centerPositionY, 0.) );
  offsetX = reTargetX - centerEyeX;
  offsetY = reTargetY - centerEyeY;
}

void KonoeTclSpaceManager::setMagnification(KFloat value)
{
  magnification = value;
}

void KonoeTclSpaceManager::setCenterEye(KInt eyeX, KInt eyeY)
{
  centerEyeX = eyeX;
  centerEyeY = eyeY;
  offsetX = -eyeX;
  offsetY = -eyeY;
}
