//=====================================================================
/*

    KonoeTclButton.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <tcl.h>
#include <string.h>
#include <ui/KonoeTclButton.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclButton::KonoeTclButton(KonoeTclWindow* kwin, KChar* kname, KChar* cap)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(Button);
  create(kwin, kname);
  isEnable = KTrue;
}

KonoeTclButton::KonoeTclButton(KonoeTclWindow* kwin, KChar* kname, KChar* cap,
                               KInt col, KInt row)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(Button);
  create(kwin, kname, col, row);
  isEnable = KTrue;
}

KonoeTclButton::~KonoeTclButton()
{ ; }

void KonoeTclButton::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);
  KChar cmd[256];

  sprintf(cmd, "button %s -text \"%s\"", path, caption);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);

  KonoeTclUIPrimitive::setRelief(KRaised);
}

void KonoeTclButton::create(KonoeTclWindow* kwin, KChar* kname,
                            KInt col, KInt row)
{
  createProcess(kwin, kname);
  KChar cmd[256];

  sprintf(cmd, "button %s -text \"%s\"", path, caption);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);

  KonoeTclUIPrimitive::setRelief(KRaised);
}

void KonoeTclButton::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();
  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
  }
}

void KonoeTclButton::setPosition(KInt sx, KInt sy)
{
  KChar cmd[256];
  sprintf(cmd, "place %s -x %ld -y %ld", path, sx, sy);
  configNoUpdate(cmd);
}

void KonoeTclButton::setCaption(KChar* cap)
{
  strcpy(caption, cap);
  KChar cmd[256];
  sprintf(cmd, "%s configure -text \"%s\"", path, caption);
  configNoUpdate(cmd);
}

void KonoeTclButton::setWidth(KInt wid)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %d", path, wid);
  configNoUpdate(cmd);
}

void KonoeTclButton::setHeight(KInt hei)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -height %d", path, hei);
  configNoUpdate(cmd);
}

void KonoeTclButton::enable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state normal", path);
  configNoUpdate(cmd);
  isEnable = KTrue;
}

void KonoeTclButton::disable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state disabled", path);
  configNoUpdate(cmd);
  isEnable = KFalse;
}

void KonoeTclButton::setSize(KInt wid, KInt hei)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %d -height %d", path, wid, hei);
  configNoUpdate(cmd);
}

void KonoeTclButton::mouseActionUI(KonoeTclButton* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseAction(bt, btype, button, time,x, y);
    kEventProcessor->mouseAction(bt, xev);
  }
}

void KonoeTclButton::mouseMotionUI(KonoeTclButton* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
    kEventProcessor->mouseMotion(bt, xev);
  }
}

void KonoeTclButton::keyActionUI(KonoeTclButton* kw, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  if(isEnable){
    kEventProcessor->keyAction(kw, btype, keysym, state);
    kEventProcessor->keyAction(kw, xev);
  }
}

void KonoeTclButton::windowActionUI(KonoeTclButton* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}
