//=====================================================================
/*

    KonoeTclCanvas.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
	Version 0.5.1	26-Jan-2000	H. Sakamoto
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <string.h>
#include <stdlib.h>
#include <ui/KonoeTclCanvas.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclEventProcessor.hh>
#include <ui/KonoeTclImage.hh>
#include <ui/KonoeTclDimension.hh>


KonoeTclCanvas::KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname)
{ 
  parentWindow = kwin;
  setWidgetType(Canvas); 
  create(kwin, kname);
}

KonoeTclCanvas::KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname,
                               KInt col, KInt row)
{ 
  parentWindow = kwin;
  setWidgetType(Canvas); 
  create(kwin, kname, col, row);
}

KonoeTclCanvas::KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname,
                               KScrollType scrl)
{ 
  parentWindow = kwin;
  setWidgetType(Canvas); 
  KChar frameName[256];
  sprintf(frameName, "kscrolled%s", kname);
  scrollFrame = new KonoeTclFrame(kwin, frameName);

  switch(scrl){
    case KBoth:
      createWithScrollBar(scrollFrame, kname);
      break;
    case KHorizontal:
      createWithHScrollBar(scrollFrame, kname);
      break;
    case KVertical:
      createWithVScrollBar(scrollFrame, kname);
      break;
    default:
      createWithScrollBar(scrollFrame, kname);
  }

  scrollFrame->setFrame();
  scrollFrame->fillBoth();
  scrollFrame->setExpand();
  parentWindow = kwin;
}

KonoeTclCanvas::KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname,
                               KScrollType scrl, KInt col, KInt row)
{ 
  parentWindow = kwin;
  setWidgetType(Canvas); 
  switch(scrl){
    case KBoth:
      createWithScrollBar(kwin, kname, col, row);
      break;
    case KHorizontal:
      createWithHScrollBar(kwin, kname, col, row);
      break;
    case KVertical:
      createWithVScrollBar(kwin, kname, col, row);
      break;
    default:
      createWithScrollBar(kwin, kname, col, row);
  }
}
KonoeTclCanvas::~KonoeTclCanvas()
{
}

void KonoeTclCanvas::create(KonoeTclWindow* kwin, KChar* kname)
{
  withScroll = isFalse;
  createProcess(kwin, kname);

  KChar cmd[256];
  sprintf(cmd, "canvas %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::create(KonoeTclWindow* kwin, KChar* kname,
                            KInt col, KInt row)
{
  withScroll = isFalse;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "canvas %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "canvas %s -xscrollcommand \"%s set\" \
               -yscrollcommand \"%s set\"", path, scrllx, scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "scrollbar %s -command \"%s xview\" -orient horizontal",
          scrllx, path);
  configNoUpdate(cmd);

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrlly, path);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side bottom -fill x", scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname,
                                         KInt col, KInt row)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "canvas %s -xscrollcommand \"%s set\" \
          -yscrollcommand \"%s set\"", path, scrllx, scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "scrollbar %s -command \"%s xview\" -orient horizontal",
          scrllx, path);
  configNoUpdate(cmd);

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrlly, path);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side bottom -fill x", scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithHScrollBar(KonoeTclWindow* kwin, KChar* kname)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s xview\" -orient horizontal",
          scrllx, path);
  configNoUpdate(cmd);

  sprintf(cmd, "canvas %s -xscrollcommand \"%s set\"", path, scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side bottom -fill x", scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithHScrollBar(KonoeTclWindow* kwin, KChar* kname,
                                         KInt col, KInt row)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s xview\" -orient horizontal",
          scrllx, path);

  configNoUpdate(cmd);

  sprintf(cmd, "canvas %s -xscrollcommand \"%s set\"", path, scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side bottom -fill x", scrllx);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithVScrollBar(KonoeTclWindow* kwin, KChar* kname)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrlly, path);
  configNoUpdate(cmd);

  sprintf(cmd, "canvas %s -yscrollcommand \"%s set\"", path, scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createWithVScrollBar(KonoeTclWindow* kwin, KChar* kname,
                                         KInt col, KInt row)
{
  withScroll = isTrue;
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrlly, path);
  configNoUpdate(cmd);

  sprintf(cmd, "canvas %s -yscrollcommand \"%s set\"", path, scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrlly);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCanvas::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();

  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
    sprintf(scrllx, ".scrllx_%s", name);
    sprintf(scrlly, ".scrlly_%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
    sprintf(scrllx, "%s.scrllx_%s", kwin->getPath(), name);
    sprintf(scrlly, "%s.scrlly_%s", kwin->getPath(), name);
  }
}

void KonoeTclCanvas::setSize(KInt width, KInt height)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %ld -height %ld", path, width, height);
  //configNoUpdate(cmd);
  configCommand(cmd);

  dim.setWidth( width );
  dim.setHeight( height );
}

void KonoeTclCanvas::setScrollregion(KInt sx, KInt sy)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -scrollregion \"0 0 %ld %ld\"", path, sx, sy);
  configCommand(cmd);
}

KDouble KonoeTclCanvas::getXView()
{
  KChar cmd[256];
  KChar valuename[8];
  KChar charvalue[8];
  KDouble  numvalue;

  strcpy(valuename, "xfrac");

  sprintf(cmd, "global %s",valuename);
  configNoUpdate(cmd);
  sprintf(cmd, "set xfrac [lindex [%s xview] 0]", path);
  configNoUpdate(cmd);
  //configCommand(cmd);

  strcpy(charvalue,Tcl_GetVar(mgr->getInterp(), valuename, TCL_GLOBAL_ONLY));
  numvalue = atof(charvalue);

  return numvalue;
}

KDouble KonoeTclCanvas::getYView()
{
  KChar cmd[256];
  KChar valuename[8];
  KChar charvalue[8];
  KDouble  numvalue;

  strcpy(valuename, "yfrac");

  sprintf(cmd, "global %s",valuename);
  configNoUpdate(cmd);
  sprintf(cmd, "set yfrac [lindex [%s yview] 0]", path);
  configNoUpdate(cmd);
  //configCommand(cmd);

  strcpy(charvalue,Tcl_GetVar(mgr->getInterp(), valuename, TCL_GLOBAL_ONLY));
  numvalue = atof(charvalue);

  return numvalue;
}

void KonoeTclCanvas::setVScrollBarPosition(KInt n)
{
  KChar cmd[256];
  sprintf(cmd, ".scrlly_%s yview scroll %ld", name, n);
  configCommand(cmd);
}

void KonoeTclCanvas::setHScrollBarPosition(KInt n)
{
  KChar cmd[256];
  sprintf(cmd, ".scrllx_%s xview scroll %ld", name, n);
  configCommand(cmd);
}

void KonoeTclCanvas::clearAll()
{
  KChar cmd[256];
  sprintf(cmd, "%s delete all", path);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::setPosition(KInt x, KInt y)
{
  KChar cmd[256];
  sprintf(cmd, "place %s -x %ld -y %ld -anchor nw", path, x, y);
  configCommand(cmd);
}

void KonoeTclCanvas::setForget()
{
  KChar cmd[256];
  sprintf(cmd, "pack forget %s", path);
  configCommand(cmd);

  if(withScroll){
    sprintf(cmd, "pack forget %s", scrllx);
    configCommand(cmd);
    sprintf(cmd, "pack forget %s", scrlly);
    configCommand(cmd);
  }
}

void KonoeTclCanvas::deleteItem(KChar* iname)
{
  KChar cmd[256];
  sprintf(cmd, "%s delete %s", path, iname);
  configNoUpdate(cmd);
}



void KonoeTclCanvas::mouseActionUI(KonoeTclCanvas* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(kEventProcessor != NULL){
    kEventProcessor->mouseAction(bt, btype, button, time,x, y);
    kEventProcessor->mouseAction(bt, xev);
  }
}

void KonoeTclCanvas::mouseMotionUI(KonoeTclCanvas* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
  kEventProcessor->mouseMotion(bt, xev);
}

void KonoeTclCanvas::keyActionUI(KonoeTclCanvas* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  kEventProcessor->keyAction(bt, btype, keysym, state);
  kEventProcessor->keyAction(bt, xev);
}

void KonoeTclCanvas::windowActionUI(KonoeTclCanvas* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclCanvas::drawLine(KInt x1, KInt y1, KInt x2, KInt y2)
{
  KChar cmd[256];
  sprintf(cmd,"%s create line %ld %ld %ld %ld -fill %s",
          path, x1, y1, x2, y2, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawLine(KInt x1, KInt y1, KInt x2, KInt y2, KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create line %ld %ld %ld %ld -fill %s -tags %s",
          path, x1, y1, x2, y2, color, tag);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawLine(KInt x1, KInt y1, KInt x2, KInt y2,KInt _width)
{
  KChar cmd[256];
  sprintf(cmd,"%s create line %ld %ld %ld %ld -fill %s -width %ld",
          path, x1, y1, x2, y2, color, _width);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawLine(KInt x1, KInt y1, KInt x2, KInt y2,KInt _width,                                 KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create line %ld %ld %ld %ld -fill %s -width %ld -tags %s",
          path, x1, y1, x2, y2, color, _width, tag);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawRectangle(KInt x, KInt y, KInt w, KInt h)
{
  KChar cmd[256];
  sprintf(cmd,"%s create rectangle %ld %ld %ld %ld -outline %s",
          path, x, y, x+w, y+h, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawRectangle(KInt x, KInt y, KInt w, KInt h, KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create rectangle %ld %ld %ld %ld -outline %s -tags %s",
          path, x, y, x+w, y+h, color, tag);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::setRectBack(KChar* tag,KChar* col)
{
  KChar cmd[256];
  sprintf(cmd, "%s itemconfigure %s -outline %s", path, tag, col);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillRectangle(KInt x, KInt y, KInt w, KInt h)
{
  KChar cmd[256];
  sprintf(cmd,"%s create rectangle %ld %ld %ld %ld -fill %s -outline %s",
          path, x, y, x+w, y+h, color, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillRectangle(KInt x, KInt y, KInt w, KInt h,
                                                               KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create rectangle %ld %ld %ld %ld -fill %s -outline %s -tags %s",
          path, x, y, x+w, y+h, color, color, tag);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::setFillRectBack(KChar* tag,KChar* col)
{
  KChar cmd[256];
  sprintf(cmd, "%s itemconfigure %s -fill %s", path, tag, col);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawOval(KInt x1, KInt y1, KInt x2, KInt y2)
{
  KChar cmd[256];
  sprintf(cmd,"%s create oval %ld %ld %ld %ld -outline %s",
          path, x1, y1, x1+x2, y1+y2, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillOval(KInt x1, KInt y1, KInt x2, KInt y2)
{
  KChar cmd[256];
  sprintf(cmd,"%s create oval %ld %ld %ld %ld -fill %s -outline %s",
          path, x1, y1, x1+x2, y1+y2, color, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawArc(KInt x1, KInt y1, KInt x2, KInt y2)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -outline %s",
          path, x1, y1, x2, y2, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawArc(KInt x1, KInt y1, KInt x2, KInt y2,
                             KInt start, KInt extent)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -start %ld -extent %ld \
               -outline %s", path, x1, y1, x2, y2, start, extent, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawArc(KInt x1, KInt y1, KInt x2, KInt y2,
                             KInt start, KInt extent, KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -start %ld -extent %ld \
               -outline %s -tags %s", path, x1, y1, x2, y2, start, extent,                     color, tag);
  configNoUpdate(cmd);
}


void KonoeTclCanvas::drawArc(KInt x1, KInt y1, KInt x2, KInt y2,
                             KInt start, KInt extent, KInt _width)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -start %ld -extent %ld \
               -outline %s -width %ld", path, x1, y1, x2, y2, start, extent,                    color, _width);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -fill %s -outline %s",
          path, x1, y1, x2, y2, color, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2,
                                 KInt start, KInt extent)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -start %ld -extent %ld\
              -fill %s", path, x1, y1, x2, y2, start, extent, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2,
                                 KInt start, KInt extent,KChar* tag)
{
  KChar cmd[256];
  sprintf(cmd,"%s create arc %ld %ld %ld %ld -start %ld -extent %ld\
               -fill %s -outline %s -tags %s", path, x1, y1, x2, y2, start,
               extent, color, color, tag);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawText(KInt x, KInt y, KChar* text)
{
  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s", 
          path, x, y, text, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawText(KInt x, KInt y, KChar* text, KChar* fname,
            KInt size, KInt weight, KInt slant, KInt underline, KInt overstrike)
{
  KChar cWeight[16];
  KChar cSlant[16];
  KChar cUnderline[16];
  KChar cOverstrike[16];

  if(weight) strcpy(cWeight, "bold");
  else       strcpy(cWeight, "normal");
  if(slant) strcpy(cSlant, "italic");
  else      strcpy(cSlant, "roman");
  if(underline) strcpy(cUnderline, "underline");
  else          strcpy(cUnderline, "");
  if(overstrike) strcpy(cOverstrike, "overstrike");
  else           strcpy(cOverstrike, "");

  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s -font { {%s} %d { %s %s %s %s }}"
  , path, x, y, text, color, fname, size, cWeight, cUnderline, cOverstrike);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawText(KInt x, KInt y, KChar* text, KInt width)
{
  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s -width %ld", 
          path, x, y, text, color, width);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawString(KInt x, KInt y, KChar* text)
{
  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s -anchor nw", 
          path, x, y, text, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawString(KInt x, KInt y, KChar* text, KChar* fname,
            KInt size, KInt weight, KInt slant, KInt underline, KInt overstrike)
{
  KChar cWeight[16];
  KChar cSlant[16];
  KChar cUnderline[16];
  KChar cOverstrike[16];

  if(weight) strcpy(cWeight, "bold");
  else       strcpy(cWeight, "normal");
  if(slant) strcpy(cSlant, "italic");
  else      strcpy(cSlant, "roman");
  if(underline) strcpy(cUnderline, "underline");
  else          strcpy(cUnderline, "");
  if(overstrike) strcpy(cOverstrike, "overstrike");
  else           strcpy(cOverstrike, "");
  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s -anchor nw -font { {%s} %d { %s %s %s %s }}"
  , path, x, y, text, color, fname, size, cWeight, cUnderline, cOverstrike);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawString(KInt x, KInt y, KChar* text, KAnchor anchor)
{
  KChar anc[16];
  switch(anchor){
    case KN: strcpy(anc, "n");
    case KNE: strcpy(anc, "ne");
    case KNW: strcpy(anc, "nw");
    case KS: strcpy(anc, "s");
    case KSE: strcpy(anc, "se");
    case KSW: strcpy(anc, "sw");
    case KE: strcpy(anc, "e");
    case KW: strcpy(anc, "w");
    case KC: strcpy(anc, "c");
    default: strcpy(anc, "nw");
  }
  KChar cmd[256];
  sprintf(cmd,"%s create text %ld %ld -text \"%s\" -fill %s -anchor nw", 
          path, x, y, text, color);
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawPolygon(KInt xPoints[], KInt yPoints[], KInt nPoints)
{
  for(KInt i=0; i<nPoints; i++){
    if(i < nPoints-1)
      drawLine(xPoints[i], yPoints[i], xPoints[i+1], yPoints[i+1]);
    else
      drawLine(xPoints[i], yPoints[i], xPoints[0], yPoints[0]);
  }
}

void KonoeTclCanvas::drawFillPolygon(KInt xPoints[], KInt yPoints[],
                                                              KInt nPoints)
{
  KChar* cmd = new KChar[16*nPoints+256];
//  KChar cmd[16*nPoints+256];
  KChar* points = new KChar[16*nPoints];
//  KChar points[16*nPoints];
  KChar itochar[16];
  KInt	i;

//  for(i=0; i<16*nPoints+256; i++) cmd[i] = '\0';
//  for(i=0; i<16*nPoints; i++) points[i] = '\0';
  cmd[0] = '\0';
  points[0] = '\0';
  
  for(i=0; i < nPoints; i++){
    sprintf(itochar,"%ld %ld ", xPoints[i], yPoints[i]);
    strcat(points, itochar);
  }

  sprintf(cmd,"%s create polygon %s -fill %s -outline %s",
          path, points, color, color);
  configNoUpdate(cmd);

  delete [] cmd;
  delete [] points;
}

void KonoeTclCanvas::drawImage(KInt x, KInt y, KonoeTclImage* image)
{
  KChar cmd[256];
  sprintf(cmd, "%s create image %ld %ld -image %s",
          path, x, y, image->getName() );
  configNoUpdate(cmd);
}

void KonoeTclCanvas::drawImagenw(KInt x, KInt y, KonoeTclImage* image)
{
  KChar cmd[256];
  sprintf(cmd, "%s create image %ld %ld -image %s -anchor nw",
          path, x, y, image->getName() );
  configNoUpdate(cmd);
}

void KonoeTclCanvas::toPostScript(KChar* fName)
{
  KChar cmd[256];
  sprintf(cmd, "%s postscript -file \"%s\"", path, fName);
  configCommand(cmd);
}

void KonoeTclCanvas::toPostScript(KChar* fName, KInt startX, KInt startY,
                                  KInt canvasWidth, KInt canvasHeight)
{
  KChar cmd[256];
  sprintf(cmd, "%s postscript -file \"%s\" -x %d -y %d -width %d -height %d",
                path, fName, startX, startY, canvasWidth, canvasHeight);
  configCommand(cmd);
}

void KonoeTclCanvas::toXWD(KChar* fName)
{
  KChar cmd[256];
  KInt windowId;

  windowId = this->getTopLevelXWindow();
  sprintf(cmd, "xwd -out %s -id %d", fName, windowId);

  system(cmd);
}

void KonoeTclCanvas::scrollOneUnitToBottom()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll +1 unit", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOneUnitToTop()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll -1 unit", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOneUnitToRight()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll +1 unit", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOneUnitToLeft()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll -1 unit", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOnePageToBottom()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll +1 page", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOnePageToTop()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll -1 page", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOnePageToRight()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll +1 page", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollOnePageToLeft()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll -1 page", path);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollXTo(KFloat value)
{
  KChar cmd[256];
  sprintf(cmd, "%s xview moveto %f", path, value);
  configCommand(cmd);
}

void KonoeTclCanvas::scrollYTo(KFloat value)
{
  KChar cmd[256];
  sprintf(cmd, "%s yview moveto %f", path, value);
  configCommand(cmd);
}

KFloat KonoeTclCanvas::getCurrentXPosition()
{
  KChar cmd[256];
  sprintf(cmd, "set scrlPos [%s xview]", path);
  configNoUpdate(cmd);

  KChar valueName[32];
  KChar value[64];
  strcpy(valueName, "scrlPos");

  strcpy(value, Tcl_GetVar(mgr->getInterp(), valueName, TCL_GLOBAL_ONLY));

  return atof(value);
}

KFloat KonoeTclCanvas::getCurrentYPosition()
{
  KChar cmd[256];
  sprintf(cmd, "set scrlPos [%s yview]", path);
  configNoUpdate(cmd);

  KChar valueName[32];
  KChar value[64];
  strcpy(valueName, "scrlPos");

  strcpy(value, Tcl_GetVar(mgr->getInterp(), valueName, TCL_GLOBAL_ONLY));

  return atof(value);
}
