//=====================================================================
/*

    KonoeTclCanvas.hh

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.5.1   25-Jan-2000 H. Sakamoto
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLCANVAS_HH
#define __KONOETCLCANVAS_HH

#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclGraphics.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclFrame.hh>

class KonoeTclImage;

class KonoeTclCanvas : virtual public KonoeTclGraphics {
  public:
    KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname);
    KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname, KInt col, KInt row);
    KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname, KScrollType scrl);
    KonoeTclCanvas(KonoeTclWindow* kwin, KChar* kname, KScrollType scrl,
                   KInt col, KInt row);
    
    virtual ~KonoeTclCanvas();


    virtual void setSize(KInt width, KInt height);

    void mouseActionUI(KonoeTclCanvas* kcvs, XEvent* xe);
    void mouseMotionUI(KonoeTclCanvas* kcvs, XEvent* xe);
    void keyActionUI(KonoeTclCanvas* kcvs, XEvent* xe);
    void windowActionUI(KonoeTclCanvas* kcvs, XEvent* xe);

    void setScrollregion(KInt sx, KInt sy);
    KDouble getXView();
    KDouble getYView();
    void setVScrollBarPosition(KInt n);
    void setHScrollBarPosition(KInt n);
    void clearAll();

    KonoeTclDimension getSize() { return dim; }

    void setPosition(KInt x, KInt y);
    void setForget();
    void deleteItem(KChar* iname);

    void drawLine(KInt x1, KInt y1, KInt x2, KInt y2);
    void drawLine(KInt x1, KInt y1, KInt x2, KInt y2, KChar* tag);
    void drawLine(KInt x1, KInt y1, KInt x2, KInt y2, KInt _width);
    void drawLine(KInt x1, KInt y1, KInt x2, KInt y2, KInt _width, KChar* tag);
    void drawRectangle(KInt x, KInt y, KInt w, KInt h);
    void drawRectangle(KInt x, KInt y, KInt w, KInt h, KChar* tag);
    void setRectBack(KChar* tag, KChar* col);
    void drawFillRectangle(KInt x, KInt y, KInt w, KInt h);
    void drawFillRectangle(KInt x, KInt y, KInt w, KInt h, KChar* tag);
    void setFillRectBack(KChar* tag, KChar* col);
    void drawOval(KInt x1, KInt y1, KInt x2, KInt y2);
    void drawFillOval(KInt x1, KInt y1, KInt x2, KInt y2);
    void drawArc(KInt x1, KInt y1, KInt x2, KInt y2);
    void drawArc(KInt x1, KInt y1, KInt x2, KInt y2, KInt start, KInt extent);
    void drawArc(KInt x1, KInt y1, KInt x2, KInt y2, KInt start, KInt extent,                    KChar* tag);
    void drawArc(KInt x1, KInt y1, KInt x2, KInt y2, KInt start, KInt extent,
                 KInt _width);
    void drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2);
    void drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2,
                     KInt start, KInt extent);
    void drawFillArc(KInt x1, KInt y1, KInt x2, KInt y2,
                     KInt start, KInt extent, KChar* tag);
    void drawText(KInt x, KInt y, KChar* text);
    void drawText(KInt x, KInt y, KChar* text, KChar* fname,
           KInt size, KInt weight, KInt slant, KInt underline, KInt overstrike);
    void drawText(KInt x, KInt y, KChar* text, KInt width);
    void drawString(KInt x, KInt y, KChar* text);
    void drawString(KInt x, KInt y, KChar* text, KChar* fname,
           KInt size, KInt weight, KInt slant, KInt underline, KInt overstrike);
    void drawString(KInt x, KInt y, KChar* text, KAnchor anchor);
    void drawPolygon(KInt xPoints[], KInt yPoints[], KInt nPoints);
    void drawFillPolygon(KInt xPoints[], KInt yPoints[], KInt nPoints);
    void drawImage(KInt x, KInt y, KonoeTclImage* image);
    void drawImagenw(KInt x, KInt y, KonoeTclImage* image);
    void toPostScript(KChar* fName);
    void toPostScript(KChar* fName, KInt startX, KInt startY,
                                    KInt canvasWidth, KInt canvasHeight);
    void toXWD(KChar* fName);

    void scrollOneUnitToBottom();
    void scrollOneUnitToTop();
    void scrollOneUnitToRight();
    void scrollOneUnitToLeft();
    void scrollOnePageToBottom();
    void scrollOnePageToTop();
    void scrollOnePageToRight();
    void scrollOnePageToLeft();
    void scrollXTo(KFloat value);
    void scrollYTo(KFloat value);
    KFloat getCurrentXPosition();
    KFloat getCurrentYPosition();

  private:

	enum	{	isFalse = 0,	isTrue = 1	};

    void create(KonoeTclWindow* kwin, KChar* kname);
    void create(KonoeTclWindow* kwin, KChar* kname, KInt col, KInt row);
    void createWithScrollBar(KonoeTclWindow* kwin, KChar* kname);
    void createWithScrollBar(KonoeTclWindow* kwin, KChar* kname,
                             KInt col, KInt row);
    void createWithHScrollBar(KonoeTclWindow* kwin, KChar* kname);
    void createWithHScrollBar(KonoeTclWindow* kwin, KChar* kname,
                              KInt col, KInt row);
    void createWithVScrollBar(KonoeTclWindow* kwin, KChar* kname);
    void createWithVScrollBar(KonoeTclWindow* kwin, KChar* kname,
                              KInt col, KInt row);
    void createProcess(KonoeTclWindow* kwin, KChar* kname);
    KChar scrllx[256];
    KChar scrlly[256];
    KBool withScroll;
    KonoeTclDimension dim;
    KonoeTclFrame* scrollFrame;
};

#endif
