//=====================================================================
/*

    KonoeTclCheckButton.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <string.h>
#include <stdlib.h>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclCheckButton.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclCheckButton::KonoeTclCheckButton(KonoeTclWindow* kwin, KChar* kname,
                                         KChar* cap)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(CheckButton);
  create(kwin, kname);
  isEnable = KTrue;
}

KonoeTclCheckButton::KonoeTclCheckButton(KonoeTclWindow* kwin, KChar* kname,
                                         KChar* cap, KInt col, KInt row)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(CheckButton);
  create(kwin, kname, col, row);
  isEnable = KTrue;
}

KonoeTclCheckButton::~KonoeTclCheckButton()
{ ; }

void KonoeTclCheckButton::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "checkbutton %s -text \"%s\"", path, caption);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCheckButton::create(KonoeTclWindow* kwin, KChar* kname,
                                 KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "checkbutton %s -text \"%s\"", path, caption);
  configNoUpdate(cmd);
  
  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclCheckButton::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();
  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
  }
}

void KonoeTclCheckButton::setCaption(KChar* cap)
{
  KChar cmd[256];
  strcpy(caption, cap);
  sprintf(cmd, "%s configure -text \"%s\"",path, caption);
  configNoUpdate(cmd);
}

void KonoeTclCheckButton::setSelect()
{
  KChar cmd[256];
  sprintf(cmd, "%s select", path);
  configNoUpdate(cmd);  
}

void KonoeTclCheckButton::deSelect()
{
  KChar cmd[256];
  sprintf(cmd, "%s deselect", path);
  configNoUpdate(cmd);  
}

void KonoeTclCheckButton::toggle()
{
  KChar cmd[256];
  sprintf(cmd, "%s toggle", path);
  configNoUpdate(cmd);  
}

void KonoeTclCheckButton::invoke()
{
  KChar cmd[256];
  sprintf(cmd, "%s invoke", path);
  configNoUpdate(cmd);  
}

KChar* KonoeTclCheckButton::getValue()
{
  return Tcl_GetVar(mgr->getInterp(), name, TCL_GLOBAL_ONLY);
}

KInt KonoeTclCheckButton::isChecked()
{
  return atoi(Tcl_GetVar(mgr->getInterp(), name, TCL_GLOBAL_ONLY));
}

void KonoeTclCheckButton::enable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state normal", path);
  configNoUpdate(cmd);
  isEnable = KTrue;
}

void KonoeTclCheckButton::disable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state disabled", path);
  configNoUpdate(cmd);
  isEnable = KFalse;
}

void KonoeTclCheckButton::mouseActionUI(KonoeTclCheckButton* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseAction(bt, btype, button, time,x, y);
    kEventProcessor->mouseAction(bt, xev);
  }
}

void KonoeTclCheckButton::mouseMotionUI(KonoeTclCheckButton* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
    kEventProcessor->mouseMotion(bt, xev);
  }
}

void KonoeTclCheckButton::windowActionUI(KonoeTclCheckButton* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclCheckButton::keyActionUI(KonoeTclCheckButton* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  if(isEnable){
    kEventProcessor->keyAction(bt, btype, keysym, state);
    kEventProcessor->keyAction(bt, xev);
  }
}
