//=====================================================================
/*

    KonoeTclCore.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <tk.h>
#include <string.h>
#include <stdlib.h>
#include <ui/KonoeTclCore.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclDimension.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclCore::KonoeTclCore()
{
  strcpy(type, "top");
  strcpy(tcl_tmpvalue, "tcl_tmpvalue");
}

KonoeTclCore::~KonoeTclCore()
{
}

void KonoeTclCore::configCommand(KChar* cmd)
{
  Tcl_Eval(mgr->getInterp(), cmd);
  Tcl_Eval(mgr->getInterp(), "update");
}

void KonoeTclCore::setConfigure(KChar* cmd)
{
  KInt n = strlen(cmd) + 1;
  KChar* comd = new KChar[n];
  sprintf(comd, "%s configure %s", path, cmd);
  configCommand(comd);
}

void KonoeTclCore::configNoUpdate(KChar* cmd)
{
  Tcl_Eval(mgr->getInterp(), cmd);
}

void KonoeTclCore::update()
{
  Tcl_Eval(mgr->getInterp(), "update");
}

void KonoeTclCore::after(KInt milisec)
{
  KChar cmd[256];
  sprintf(cmd, "after %d", milisec);
  configCommand(cmd);
}

void KonoeTclCore::setManager()
{
  mgr = KonoeTclManager::getManager();
}

void KonoeTclCore::setWindowMaxSize( KInt mx, KInt my )
{
  KChar cmd[256];
  sprintf( cmd, "wm maxsize %s %ld %ld", path, mx, my );
  configCommand(cmd);
}

void KonoeTclCore::setWindowMaxSize( KonoeTclDimension* kd )
{
  KChar cmd[256];
  sprintf( cmd, "wm maxsize %s %ld %ld", path,
                 kd->getWidth(), kd->getHeight() );
  configCommand(cmd);
}

void KonoeTclCore::setWindowMinSize( KInt mx, KInt my )
{
  KChar cmd[256];
  sprintf( cmd, "wm mixsize %s %ld %ld", path, mx, my );
  configCommand(cmd);
}

void KonoeTclCore::setWindowMinSize( KonoeTclDimension* kd )
{
  KChar cmd[256];
  sprintf( cmd, "wm maxsize %s %ld %ld", path,
                 kd->getWidth(), kd->getHeight() );
  configCommand(cmd);
}

KInt KonoeTclCore::getRootX()
{
  KInt rx;
  KChar cmd[256];
  KChar tcl_root_x[12];
  KChar rootX[8];

  strcpy(tcl_root_x, "tcl_root_x");

  sprintf(cmd, "set %s [ winfo rootx %s ]", tcl_root_x, path);
  configCommand(cmd);

  strcpy(rootX,
         Tcl_GetVar(mgr->getInterp(), tcl_root_x, TCL_GLOBAL_ONLY));

  rx = atoi( rootX );

  return rx;
}

KInt KonoeTclCore::getRootY()
{
  KInt ry;
  KChar cmd[256];
  KChar tcl_root_y[12];
  KChar rootY[8];

  strcpy(tcl_root_y, "tcl_root_y");

  sprintf(cmd, "set %s [ winfo rooty %s ]", tcl_root_y, path);
  configCommand(cmd);

  strcpy(rootY,
         Tcl_GetVar(mgr->getInterp(), tcl_root_y, TCL_GLOBAL_ONLY));

  ry = atoi( rootY );

  return ry;
}

void KonoeTclCore::transformColor(KChar* color16, KInt rr, KInt gg, KInt bb)
{
  if(rr<0) rr = 0;
  else if(rr>255) rr = 255;
  if(gg<0) gg = 0;
  else if(gg>255) gg = 255;
  if(bb<0) bb = 0;
  else if(bb>255) bb = 255;

  if(rr < 16 && gg < 16 && bb < 16) sprintf(color16, "#0%x0%x0%x", rr, gg, bb);
  else if( rr < 16 && gg < 16 ) sprintf(color16, "#0%x0%x%x", rr, gg, bb);
  else if( rr < 16 && bb < 16 ) sprintf(color16, "#0%x%x0%x", rr, gg, bb);
  else if( gg < 16 && bb < 16 ) sprintf(color16, "#%x0%x0%x", rr, gg, bb);
  else if( rr < 16 ) sprintf(color16, "#0%x%x%x", rr, gg, bb);
  else if( gg < 16 ) sprintf(color16, "#%x0%x%x", rr, gg, bb);
  else if( bb < 16 ) sprintf(color16, "#%x%x0%x", rr, gg, bb);
  else sprintf(color16, "#%x%x%x", rr, gg, bb);
}

KChar* KonoeTclCore::getBackGround(){
  KChar* bgcolor = "widgetBackGroundCOLOR";
  KChar cmd[256];
  sprintf(cmd, "set %s [%s cget -background]", bgcolor, path);
  configCommand(cmd);
  return Tcl_GetVar(mgr->getInterp(), bgcolor, TCL_GLOBAL_ONLY);
}

Window KonoeTclCore::getTopLevelXWindow()
{
  Tk_Window tkwin = this->getWindow();
  while(!isTopLevel(tkwin)){
    tkwin = this->getParent(tkwin);
  }
  return this->getXWindow(tkwin);
}

KonoeTclWindow* KonoeTclCore::getTopLevelWindow()
{
  if(isTopLevel(win)) return NULL;

  KonoeTclWindow* w = this->getParentWindow();
  while(w->getParentWindow()){
    w = w->getParentWindow();
  }

  return w;
}

void KonoeTclCore::setLayoutTypeWithPack(KChar* typ)
{
  strcpy(type, typ);
  KChar cmd[256];
  sprintf(cmd, "pack %s -side %s", path, type);
  configCommand(cmd);
}

void KonoeTclCore::setEventProcessor(KonoeTclEventProcessor* ep)
{
  kEventProcessor = ep;
}
