//=====================================================================
/*

    KonoeTclCore.hh

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLCORE_HH
#define __KONOETCLCORE_HH

#include <string.h>
#include <tcl.h>
#include <tk.h>
#include <KonoePrimitives.hh>

class KonoeTclManager;
class KonoeTclWindow;
class KonoeTclEventProcessor;
class KonoeTclDimension;

class KonoeTclCore {
  public:
    KonoeTclCore();
    virtual ~KonoeTclCore();

    void configCommand(KChar* );
    void configNoUpdate(KChar* );
    void update();
    void after(KInt milisec);
    void setConfigure(KChar* );
    void tclCommand(KChar* );
    inline Tk_Window& getWindow(){ return win; }
    inline Tk_Window& getParent(){ return Tk_Parent(win); }
    inline Tk_Window& getParent(Tk_Window tkwin){ return Tk_Parent(tkwin); }
    inline KonoeTclWindow* getParentWindow(){ return parentWindow; }
    inline KChar* getPath(){ return path; }
    KChar* getParentPath()
    {
      if(getParent()!=NULL) return Tk_PathName(getParent());
      else return NULL;
    }
    inline KChar* getName(){ return name; }
    inline KInt  getWidth() const { return Tk_Width(win); }
    inline KInt  getHeight() const { return Tk_Height(win); }
    inline Display* getDisplay() { return Tk_Display(win); }
    inline Screen* getScreen() { return Tk_Screen(win); }
    inline KInt  getScreenNumber() const { return Tk_ScreenNumber(win); }
    inline KInt  getXPos() const { return Tk_X(win); }
    inline KInt  getYPos() const { return Tk_Y(win); }
    inline Window getXWindow() { return Tk_WindowId(win); }
    inline Window getXWindow(Tk_Window tkwin) { return Tk_WindowId(tkwin); }
    Window getTopLevelXWindow();
    KonoeTclWindow* getTopLevelWindow();

    void setManager();
    KonoeTclManager* getManager() { return mgr; }

    void setWindowMaxSize( KInt mx, KInt my );
    void setWindowMaxSize( KonoeTclDimension* kd );
    void setWindowMinSize( KInt mx, KInt my );
    void setWindowMinSize( KonoeTclDimension* kd );

    KInt getRootX();
    KInt getRootY();

    void setEventProcessor(KonoeTclEventProcessor* ep);

    KChar* getBackGround();

    virtual void setForeGround(KChar* col)
    {
      KChar cmd[256];
      if(strcmp(fgcolor, col) != 0) 
        strcpy(fgcolor, col);
      sprintf(cmd, "%s configure -fg %s", path, fgcolor);
      configNoUpdate(cmd);
    }

    virtual void setForeGround(KInt rr, KInt gg, KInt bb)
    {
      transformColor(fgcolor, rr, gg, bb);
      setForeGround(fgcolor);
    }

    virtual void setBackGround(KChar* col)
    {
      KChar cmd[256];
      if(strcmp(bgcolor, col) != 0) 
        strcpy(bgcolor, col);
      sprintf(cmd, "%s configure -background %s", path, bgcolor);
      configNoUpdate(cmd);
    }

    virtual void setBackGround(KInt rr, KInt gg, KInt bb)
    {
      transformColor(bgcolor, rr, gg, bb);
      setBackGround(bgcolor);
    }

    virtual void setLayoutType(KChar* typ)
    {
      // Layout type : top bottom left right
      //KChar cmd[256];
      strcpy(type, typ);
      //sprintf(cmd, "pack %s -side %s", path, type);
      //configCommand(cmd);
    }
    void setLayoutTypeWithPack(KChar* typ);

    virtual void setActiveFore(KChar* col)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -activeforeground %s", path, col);
      configNoUpdate(cmd);
    }

    virtual void setActiveBack(KChar* col)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -activebackground %s", path, col);
      configNoUpdate(cmd);
    }

    virtual void setDisableFore(KChar* col)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -disableforeground %s", path, col);
      configNoUpdate(cmd);
    }

    virtual void setHiLightBack(KChar* col)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -highlightbackground %s", path, col);
      configNoUpdate(cmd);
    }

    virtual void setHiLightCol(KChar* col)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -highlightcolor %s", path, col);
      configNoUpdate(cmd);
    }

    virtual void setHiLightThick(KChar* num)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -highlightthickness %s", path, num);
      configNoUpdate(cmd);
    }

    virtual void setBorderWidth(KInt num)
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -borderwidth %ld", path, num);
      configNoUpdate(cmd);
    }

    virtual void setTakeFocus()
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -takefocus 1", path);
      configNoUpdate(cmd);
    }

    virtual void unsetTakeFocus()
    {
      KChar cmd[256];
      sprintf(cmd, "%s configure -takefocus 0", path);
      configNoUpdate(cmd);
    }


    inline KInt isTopLevel(Tk_Window kwin){ return Tk_IsTopLevel(kwin); }

  protected:
    void transformColor(KChar* color16, KInt rr, KInt gg, KInt bb);
    Tk_Window win;
    KChar path[256];
    KChar name[32];
    KonoeTclManager* mgr;
    KonoeTclWindow* parentWindow;
    KChar type[32];
    KChar bgcolor[32];
    KChar fgcolor[32];
    KChar tcl_tmpvalue[256];

    KonoeTclEventProcessor* kEventProcessor;
};

#endif
