//=====================================================================
/*

    KonoeTclEntry.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <string.h>
#include <ui/KonoeTclEntry.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclEntry::KonoeTclEntry(KonoeTclWindow* kwin, KChar* kname)
{
  parentWindow = kwin;
  setWidgetType(Entry);
  create(kwin, kname);
  isEnable = KTrue;
}

KonoeTclEntry::KonoeTclEntry(KonoeTclWindow* kwin, KChar* kname,
                             KInt col, KInt row)
{
  parentWindow = kwin;
  setWidgetType(Entry);
  create(kwin, kname, col, row);
  isEnable = KTrue;
}

KonoeTclEntry::~KonoeTclEntry()
{ ; }

void KonoeTclEntry::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "entry %s -textvariable %s", path, name);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclEntry::create(KonoeTclWindow* kwin, KChar* kname,
                           KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "entry %s -textvariable %s", path, name);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclEntry::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();
  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
  }
}

void KonoeTclEntry::mouseActionUI(KonoeTclEntry* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseAction(bt, btype, button, time,x, y);
    kEventProcessor->mouseAction(bt, xev);
  }
}

void KonoeTclEntry::mouseMotionUI(KonoeTclEntry* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
    kEventProcessor->mouseMotion(bt, xev);
  }
}

void KonoeTclEntry::keyActionUI(KonoeTclEntry* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  if(isEnable){
    kEventProcessor->keyAction(bt, btype, keysym, state);
    kEventProcessor->keyAction(bt, xev);
  }
}

void KonoeTclEntry::windowActionUI(KonoeTclEntry* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclEntry::setText(KChar* txt)
{
  Tcl_SetVar(mgr->getInterp(), name, txt, TCL_GLOBAL_ONLY);
}

void KonoeTclEntry::setSize(KInt n)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %d",path,n);
  configNoUpdate(cmd);
}

void KonoeTclEntry::enable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state normal", path);
  configNoUpdate(cmd);
  isEnable = KTrue;
}

void KonoeTclEntry::disable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state disable", path);
  configNoUpdate(cmd);
  isEnable = KFalse;
}

void KonoeTclEntry::selectAll()
{
  KChar cmd[256];
  sprintf(cmd, "%s selection range 0 end", path);
  configCommand(cmd);
}

KChar* KonoeTclEntry::getText()
{
  return Tcl_GetVar(mgr->getInterp(), name, TCL_GLOBAL_ONLY);
}
