//=====================================================================
/*

    KonoeTclListBox.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <ui/KonoeTclListBox.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclListBox::KonoeTclListBox(KonoeTclWindow* kwin, KChar* kname)
{
  parentWindow = kwin;
  setWidgetType(ListBox); 
  create(kwin, kname);
}

KonoeTclListBox::KonoeTclListBox(KonoeTclWindow* kwin, KChar* kname,
                                 KInt col, KInt row)
{
  parentWindow = kwin;
  setWidgetType(ListBox); 
  create(kwin, kname, col, row);
}

KonoeTclListBox::KonoeTclListBox(KonoeTclWindow* kwin, KChar* kname,
                                 KScrollType scrl)
{
  parentWindow = kwin;
  setWidgetType(ListBox); 
  KChar frameName[256];
  sprintf(frameName, "kscrolled%s", kname);
  scrollFrame = new KonoeTclFrame(kwin, frameName);

  switch(scrl){
    case KBoth:
    case KHorizontal:
    case KVertical:
      createWithScrollBar(scrollFrame, kname);
      break;
    default:
      scrollFrame = new KonoeTclFrame(kwin, tmpnam(frameName));
      createWithScrollBar(scrollFrame, kname);
  }

  scrollFrame->setFrame();
  scrollFrame->fillBoth();
  scrollFrame->setExpand();
}

KonoeTclListBox::KonoeTclListBox(KonoeTclWindow* kwin, KChar* kname,
                                 KScrollType scrl, KInt col, KInt row)
{
  parentWindow = kwin;
  setWidgetType(ListBox); 
  switch(scrl){
    case KBoth:
      createWithScrollBar(kwin, kname, col, row);
      break;
    case KHorizontal:
      createWithScrollBar(kwin, kname, col, row);
      break;
    case KVertical:
      createWithScrollBar(kwin, kname, col, row);
      break;
    default:
      createWithScrollBar(kwin, kname, col, row);
  }
}

KonoeTclListBox::~KonoeTclListBox()
{ ; }

void KonoeTclListBox::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "listbox %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclListBox::create(KonoeTclWindow* kwin, KChar* kname,
                             KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "listbox %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclListBox::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrll, path);
  configNoUpdate(cmd);

  sprintf(cmd, "listbox %s -yscrollcommand \"%s set\"", path, scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclListBox::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname,
                                          KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrll, path);
  configNoUpdate(cmd);

  sprintf(cmd, "listbox %s -yscrollcommand \"%s set\"", path, scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclListBox::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();

  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
    sprintf(scrll, ".%sscr", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
    sprintf(scrll, "%s.%sscr", kwin->getPath(),  name);
  }
}

void KonoeTclListBox::mouseActionUI(KonoeTclListBox* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseAction(bt, btype, button, time,x, y);
  kEventProcessor->mouseAction(bt, xev);
}

void KonoeTclListBox::mouseMotionUI(KonoeTclListBox* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
  kEventProcessor->mouseMotion(bt, xev);
}

void KonoeTclListBox::keyActionUI(KonoeTclListBox* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  kEventProcessor->keyAction(bt, btype, keysym, state);
  kEventProcessor->keyAction(bt, xev);
}

void KonoeTclListBox::windowActionUI(KonoeTclListBox* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclListBox::setSize(KInt w, KInt h)
{
  KChar cmd[256];

  sprintf(cmd, "%s configure -height %ld", path, h);
  configCommand(cmd);
}

void KonoeTclListBox::insert(KChar* kitem)
{
  KChar cmd[256];

  sprintf(cmd, "%s insert end \"%s\"", path, kitem);
  configNoUpdate(cmd);
}

void KonoeTclListBox::insert(KInt nPos, KChar* kitem)
{
  KChar cmd[256];
  sprintf(cmd, "%s insert %ld \"%s\"", path, nPos, kitem);
  configNoUpdate(cmd);
}

void KonoeTclListBox::deleteItem(KInt nPos)
{
  KChar cmd[256];
  sprintf(cmd, "%s delete %d %d", path, nPos, nPos);
  configNoUpdate(cmd);
}

void KonoeTclListBox::deleteItem(KInt firstPos, KInt lastPos)
{
  KChar cmd[256];
  sprintf(cmd, "%s delete %d %d", path, firstPos, lastPos);
  configNoUpdate(cmd);
}

void KonoeTclListBox::setMode(KChar* mode)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -selectmode %s", path, mode);
  configNoUpdate(cmd);
}

void KonoeTclListBox::setMode(KInt mode)
{
  KChar cmode[32];

  switch( mode ){
    case KSingle:
      strcpy(cmode, "single");
      break;
    case KBrowse:
      strcpy(cmode, "browse");
      break;
    case KMultiple:
      strcpy(cmode, "multiple");
      break;
    case KExtended:
      strcpy(cmode, "extended");
      break;
    default:
      strcpy(cmode, "single");
  }

  KChar cmd[256];
  sprintf(cmd, "%s configure -selectmode %s", path, cmode);
  configNoUpdate(cmd);
}

void KonoeTclListBox::setSelectBackGround(KChar* col)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -selectbackground %s", path, col);
  configNoUpdate(cmd);
}

void KonoeTclListBox::setSelectForeGround(KChar* col)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -selectforeground %s", path, col);
  configNoUpdate(cmd);
}

void KonoeTclListBox::exportSelection()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -exportselection", path);
  configNoUpdate(cmd);
}

KInt KonoeTclListBox::getTotalIndex()
{
  KChar cmd[256];
  KChar listId[32];
  sprintf(listId, "%sNum", name);
  sprintf(cmd, "set %s [%s size]", listId, path);
  configCommand(cmd);
  KInt total;
  total = atoi(Tcl_GetVar(mgr->getInterp(), listId, TCL_GLOBAL_ONLY));
  return total;
}

KChar* KonoeTclListBox::getSelectedIndex()
{
  KChar cmd[256];
  KChar listId[32];
  sprintf(listId, "%sId", name);
  sprintf(cmd, "set %s [%s curselection]", listId, path);
  configCommand(cmd);
  return Tcl_GetVar(mgr->getInterp(), listId, TCL_GLOBAL_ONLY);
}

