//=====================================================================
/*

    KonoeTclRadioButton.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <string.h>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclRadioButton.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclRadioButton::KonoeTclRadioButton(KonoeTclWindow* kwin, KChar* kname,
                                         KChar* cap)
{ 
  parentWindow = kwin;
  strcpy(caption, cap);
  strcpy(group, "defaultG");
  setWidgetType(RadioButton);
  create(kwin, kname);
  isEnable = KTrue;
}

KonoeTclRadioButton::KonoeTclRadioButton(KonoeTclWindow* kwin, KChar* kname,
                                         KChar* cap, KInt col, KInt row)
{ 
  parentWindow = kwin;
  strcpy(caption, cap);
  strcpy(group, "defaultG");
  setWidgetType(RadioButton);
  create(kwin, kname, col, row);
  isEnable = KTrue;
}

KonoeTclRadioButton::~KonoeTclRadioButton()
{ ; }

void KonoeTclRadioButton::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "radiobutton %s -text \"%s\" -variable %s -value %s",
          path, caption, group, name);
  configNoUpdate(cmd);

  sprintf(cmd, "global %s", group);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclRadioButton::create(KonoeTclWindow* kwin, KChar* kname,
                                 KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "radiobutton %s -text \"%s\" -variable %s -value %s",
          path, caption, group, name);
  configNoUpdate(cmd);

  sprintf(cmd, "global %s", group);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclRadioButton::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();
  strcpy(name, kname);

  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
  }
}

void KonoeTclRadioButton::mouseActionUI(KonoeTclRadioButton* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseAction(bt, btype, button, time,x, y);
    kEventProcessor->mouseAction(bt, xev);
  }
}

void KonoeTclRadioButton::mouseMotionUI(KonoeTclRadioButton* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  if(isEnable){
    kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
    kEventProcessor->mouseMotion(bt, xev);
  }
}

void KonoeTclRadioButton::keyActionUI(KonoeTclRadioButton* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;
  if(isEnable){
    kEventProcessor->keyAction(bt, btype, keysym, state);
    kEventProcessor->keyAction(bt, xev);
  }
}

void KonoeTclRadioButton::windowActionUI(KonoeTclRadioButton* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclRadioButton::setCaption(KChar* cap)
{
  strcpy(caption, cap);

  KChar cmd[256];
  sprintf(cmd, "%s configure -text \"%s\"", path, cap);
  configNoUpdate(cmd);
}

void KonoeTclRadioButton::setGroup(KChar* grp)
{
  strcpy(group, grp);
  
  KChar cmd[256];
  sprintf(cmd, "%s configure -variable %s", path, grp);
  configNoUpdate(cmd);
}

void KonoeTclRadioButton::setSelect()
{
  KChar cmd[256];
  sprintf(cmd, "%s select", path);
  configNoUpdate(cmd);
}

KChar* KonoeTclRadioButton::getValue()
{
  return Tcl_GetVar(mgr->getInterp(), group, TCL_GLOBAL_ONLY);
}

void KonoeTclRadioButton::enable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state normal", path);
  configNoUpdate(cmd);
  isEnable = KTrue;
}

void KonoeTclRadioButton::disable()
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -state disabled", path);
  configNoUpdate(cmd);
  isEnable = KFalse;
}
