//=====================================================================
/*

    KonoeTclSlider.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclSlider.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclSlider::KonoeTclSlider(KonoeTclWindow* kwin, KChar* kname, KChar* cap)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(Slider);
  create(kwin, kname);
}

KonoeTclSlider::KonoeTclSlider(KonoeTclWindow* kwin, KChar* kname, KChar* cap,
                               KInt col, KInt row)
{
  parentWindow = kwin;
  strcpy(caption, cap);
  setWidgetType(Slider);
  create(kwin, kname, col, row);
}

KonoeTclSlider::~KonoeTclSlider()
{ ; }

void KonoeTclSlider::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scale %s -label \"%s\" -variable %s -from 0 -to 100 \
          -orient horizontal" , path, caption, name);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclSlider::create(KonoeTclWindow* kwin, KChar* kname,
                            KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scale %s -label \"%s\" -variable %s -from 0 -to 100 \
          -orient horizontal" , path, caption, name);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclSlider::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();
  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
  }
}

void KonoeTclSlider::mouseActionUI(KonoeTclSlider* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseAction(bt, btype, button, time,x, y);
  kEventProcessor->mouseAction(bt, xev);
}

void KonoeTclSlider::mouseMotionUI(KonoeTclSlider* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
  kEventProcessor->mouseMotion(bt, xev);
}

void KonoeTclSlider::keyActionUI(KonoeTclSlider* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  kEventProcessor->keyAction(bt, btype, keysym, state);
  kEventProcessor->keyAction(bt, xev);
}

void KonoeTclSlider::windowActionUI(KonoeTclSlider* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclSlider::setCaption(KChar* cap)
{
  strcpy(caption, cap);
}

void KonoeTclSlider::setMinMax(KInt mn, KInt mx)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -from %ld -to %ld", path, mn, mx);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setLength(KInt l)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -length %ld", path, l);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setShowValue(KInt b)
{
  KChar cmd[256];
  if(b) 
    sprintf(cmd, "%s configure -showvalue yes", path);
  else
    sprintf(cmd, "%s configure -showvalue no", path);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setResolution(KInt v)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -resolution %ld", path, v);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setTickInterval(KInt v)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -tickInterval %ld", path, v);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setBigIncrement(KInt v)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -bigIncrement %ld", path, v);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setWidth(KInt w)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %ld", path, w);
  configNoUpdate(cmd);
}

void KonoeTclSlider::setValue(KInt v)
{
  KChar cmd[256];
  sprintf(cmd, "%s set %ld", path, v);
  configNoUpdate(cmd);
}

KInt KonoeTclSlider::getIndex()
{
  KInt index = 0;
  index = atoi(Tcl_GetVar(mgr->getInterp(), name, TCL_GLOBAL_ONLY));
  return index;
}
