//=====================================================================
/*

    KonoeTclText.cc

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi
    Version 0.6.2   05-Nov-2000 K. Sakai, Y. Ikuta

*/
//====================================================================

#include <string.h>
#include <stdlib.h>
#include <ui/KonoeTclText.hh>
#include <ui/KonoeTclManager.hh>
#include <ui/KonoeTclEventProcessor.hh>

KonoeTclText::KonoeTclText(KonoeTclWindow* kwin, KChar* kname)
{
  parentWindow = kwin;
  setWidgetType(Text);
  create(kwin, kname);
}

KonoeTclText::KonoeTclText(KonoeTclWindow* kwin, KChar* kname,
                           KInt col, KInt row)
{
  parentWindow = kwin;
  setWidgetType(Text);
  create(kwin, kname, col, row);
}
 
KonoeTclText::KonoeTclText(KonoeTclWindow* kwin, KChar* kname, KScrollType scrl)
{
  parentWindow = kwin;
  setWidgetType(Text);
  KChar frameName[256];
  sprintf(frameName, "kscrolled%s", kname);
  scrollFrame = new KonoeTclFrame(kwin, frameName);

  switch(scrl){
    case KBoth:
      createWithScrollBar(scrollFrame, kname);
      break;
    case KHorizontal:
      createWithScrollBar(scrollFrame, kname);
      break;
    case KVertical:
      createWithScrollBar(scrollFrame, kname);
      break;
    default:
      createWithScrollBar(scrollFrame, kname);
  }

  scrollFrame->setFrame();
  scrollFrame->fillBoth();
  scrollFrame->setExpand();
}

KonoeTclText::KonoeTclText(KonoeTclWindow* kwin, KChar* kname, KScrollType scrl,
                           KInt col, KInt row)
{
  parentWindow = kwin;
  setWidgetType(Text);
  switch(scrl){
    case KBoth:
      createWithScrollBar(kwin, kname, col, row);
      break;
    case KHorizontal:
      createWithScrollBar(kwin, kname, col, row);
      break;
    case KVertical:
      createWithScrollBar(kwin, kname, col, row);
      break;
    default:
      createWithScrollBar(kwin, kname, col, row);
  }
}
KonoeTclText::~KonoeTclText()
{ ; }

void KonoeTclText::create(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "text %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclText::create(KonoeTclWindow* kwin, KChar* kname,
                          KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "text %s", path);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclText::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrll, path);
  configNoUpdate(cmd);

  sprintf(cmd, "text %s -yscrollcommand \"%s set\"", path, scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side %s", path, type);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclText::createWithScrollBar(KonoeTclWindow* kwin, KChar* kname,
                                       KInt col, KInt row)
{
  createProcess(kwin, kname);

  KChar cmd[256];

  sprintf(cmd, "scrollbar %s -command \"%s yview\"", scrll, path);
  configNoUpdate(cmd);

  sprintf(cmd, "text %s -yscrollcommand \"%s set\"", path, scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "pack %s -side right -fill y", scrll);
  configNoUpdate(cmd);

  sprintf(cmd, "grid %s -column %ld -row %ld", path, col, row);
  configNoUpdate(cmd);
  //configCommand(cmd);

  win = Tk_NameToWindow(mgr->getInterp(), path, kwin->getWindow());
  setGraphics(win);
}

void KonoeTclText::createProcess(KonoeTclWindow* kwin, KChar* kname)
{
  setManager();

  strcpy(name, kname);
  if(strcmp(kwin->getPath(), ".") == 0){
    sprintf(path, ".%s", name);
    sprintf(scrll, ".scrll_%s", name);
  }
  else{
    sprintf(path, "%s.%s", kwin->getPath(), name);
    sprintf(scrll, "%s.scrll_%s", kwin->getPath(), name);
  }
}

void KonoeTclText::mouseActionUI(KonoeTclText* bt, XEvent* xev)
{
  KInt btype;
  KInt button;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  button = xev->xbutton.button;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseAction(bt, btype, button, time,x, y);
  kEventProcessor->mouseAction(bt, xev);
}

void KonoeTclText::mouseMotionUI(KonoeTclText* bt, XEvent* xev)
{
  KInt btype;
  KInt state;
  Time time;
  KInt x;
  KInt y;

  btype = xev->xbutton.type;
  state = xev->xbutton.state;
  time = xev->xbutton.time;
  x = xev->xbutton.x;
  y = xev->xbutton.y;

  kEventProcessor->mouseMotion(bt, btype, state, time,x, y);
  kEventProcessor->mouseMotion(bt, xev);
}

void KonoeTclText::keyActionUI(KonoeTclText* bt, XEvent* xev)
{
  KChar buffer[20];
  KInt bufsize = 20;
  KeySym keysym;
  KInt btype;
  KInt state;

  XLookupString(&(xev->xkey), buffer, bufsize, &keysym, (XComposeStatus*) NULL);
  btype = xev->xkey.type;
  state = xev->xkey.state;

  kEventProcessor->keyAction(bt, btype, keysym, state);
  kEventProcessor->keyAction(bt, xev);
}

void KonoeTclText::windowActionUI(KonoeTclText* bt, XEvent* xev)
{
  kEventProcessor->windowAction(bt, xev);
}

void KonoeTclText::setSize(KInt width, KInt height)
{
  KChar cmd[256];
  sprintf(cmd, "%s configure -width %ld -height %ld", path, width, height);
  configCommand(cmd);
}

void KonoeTclText::insert(KChar* contents)
{
  KInt n = strlen(contents);
  KChar* cmd;
  cmd = new KChar[n+256];

  sprintf(cmd, "%s insert end \"%s\"", path, contents);
  configNoUpdate(cmd);

  delete [] cmd;
}

void KonoeTclText::insert(KChar* contents,KChar* _tag)
{
  KInt n = strlen(contents);
  KChar* cmd;
  cmd = new KChar[n+256];

  sprintf(cmd, "%s insert end \"%s\" %s", path, contents, _tag);
  configNoUpdate(cmd);

  delete [] cmd;
}

KDouble KonoeTclText::index(KChar* _pos)
{
  KChar cmd[256];
  KChar valuename[8];
  KChar charvalue[8];
  KDouble numvalue;
      
  strcpy(valuename, "numline");
      
  sprintf(cmd, "global %s", valuename);
  configNoUpdate(cmd); 
  sprintf(cmd, "set numline [%s index %s]", path, _pos);
  configNoUpdate(cmd);
  strcpy(charvalue,Tcl_GetVar(mgr->getInterp(), valuename, TCL_GLOBAL_ONLY));
  numvalue = atof(charvalue);
  return numvalue;
}

void KonoeTclText::clear()
{
  KChar cmd[256];
  sprintf(cmd, "%s delete 1.0 end", path);
  configNoUpdate(cmd);
}

void KonoeTclText::tagClear(KChar* _tag)
{
  KChar cmd[256];
  sprintf(cmd, "%s tag delete %s", path, _tag);
  configNoUpdate(cmd);
}

KChar* KonoeTclText::getText()
{
  KChar cmd[256];
  KChar txt[256];
  sprintf(txt, "txt%s", name);
      
  sprintf(cmd, "global %s", txt);
  configNoUpdate(cmd); 
  sprintf(cmd, "set txt%s [%s get 1.0 end]", name, path);
  configNoUpdate(cmd);
  return Tcl_GetVar(mgr->getInterp(), txt, TCL_GLOBAL_ONLY);
}

void KonoeTclText::deleteText(KInt startID, KInt endID)
{
  KChar cmd[256];
  sprintf(cmd, "%s delete %d.0 %d.0", path, startID, endID);
  configNoUpdate(cmd);
}

void KonoeTclText::setBackGround(KChar* _tag,KChar* _color)
{
  KChar cmd[256];
  sprintf(cmd,"%s tag configure %s -background %s", path, _tag, _color);
  configNoUpdate(cmd);
} 

void KonoeTclText::setBackGround(KChar* col)
{
  KChar cmd[256];
  sprintf(cmd,"%s configure -background %s", path, col);
  configNoUpdate(cmd);
} 

void KonoeTclText::setForeGround(KChar* _tag,KChar* _color)
{
  KChar cmd[256];
  sprintf(cmd,"%s tag configure %s -foreground %s", path, _tag, _color);
  configNoUpdate(cmd);
}

void KonoeTclText::setBorderWidth(KChar* _tag,KInt pix)
{
  KChar cmd[256];
  sprintf(cmd,"%s tag configure %s -borderwideh %d", path, _tag, pix);
  configNoUpdate(cmd);
}

void KonoeTclText::setTagFont(KChar* _tag,KChar* _font)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -font %s", path, _tag, _font);
 configNoUpdate(cmd);
}

void KonoeTclText::setJustify(KChar* _tag,KChar* _justify)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -justify %s", path, _tag, _justify);
 configNoUpdate(cmd);
}

void KonoeTclText::setLmargin1(KChar* _tag,KInt pix)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -lmargin1 %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setLmargin2(KChar* _tag,KInt pix)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -lmargin2 %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setOffset(KChar* _tag,KInt pix)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -offset %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setOverStrike(KChar* _tag,KInt boolean)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -overstrike %d", path, _tag, boolean);
 configNoUpdate(cmd);
}

void KonoeTclText::setRelief(KChar* _tag,KChar* _relief)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -relief %s", path, _tag, _relief);
 configNoUpdate(cmd);
}

void KonoeTclText::setRmargin(KChar* _tag,KInt pix)
{
 KChar cmd[256];
 sprintf(cmd,"%s tag configure %s -rmargin %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setSpacing1(KChar* _tag,KInt pix)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -spacing1 %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setSpacing2(KChar* _tag,KInt pix)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -spacing2 %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setSpacing3(KChar* _tag,KInt pix)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -spacing3 %d", path, _tag, pix);
 configNoUpdate(cmd);
}

void KonoeTclText::setTabs(KChar* _tag,KChar* tablist)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -tabs %s", path, _tag, tablist);
 configNoUpdate(cmd);
}

void KonoeTclText::setUnderLine(KChar* _tag,KInt boolean)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -underline %d", path, _tag, boolean);
 configNoUpdate(cmd);
}

void KonoeTclText::setWrap(KChar* _tag,KChar* mode)
{
 KChar cmd [256];
 sprintf(cmd,"%s tag configure %s -wrap %s", path, _tag, mode);
 configNoUpdate(cmd);
}

void KonoeTclText::scrollOneUnitToBottom()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll +1 unit", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOneUnitToTop()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll -1 unit", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOneUnitToRight()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll +1 unit", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOneUnitToLeft()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll -1 unit", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOnePageToBottom()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll +1 page", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOnePageToTop()
{
  KChar cmd[256];
  sprintf(cmd, "%s yview scroll -1 page", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOnePageToRight()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll +1 page", path);
  configCommand(cmd);
}

void KonoeTclText::scrollOnePageToLeft()
{
  KChar cmd[256];
  sprintf(cmd, "%s xview scroll -1 page", path);
  configCommand(cmd);
}

void KonoeTclText::scrollXTo(KFloat value)
{
  KChar cmd[256];
  sprintf(cmd, "%s xview moveto %f", path, value);
  configCommand(cmd);
}

void KonoeTclText::scrollYTo(KFloat value)
{
  KChar cmd[256];
  sprintf(cmd, "%s yview moveto %f", path, value);
  configCommand(cmd);
}

KFloat KonoeTclText::getCurrentXPosition()
{
  KChar cmd[256];
  sprintf(cmd, "set scrlPos [%s xview]", path);
  configNoUpdate(cmd);

  KChar valueName[32];
  KChar value[64];
  strcpy(valueName, "scrlPos");

  strcpy(value, Tcl_GetVar(mgr->getInterp(), valueName, TCL_GLOBAL_ONLY));

  return atof(value);
}

KFloat KonoeTclText::getCurrentYPosition()
{
  KChar cmd[256];
  sprintf(cmd, "set scrlPos [%s yview]", path);
  configNoUpdate(cmd);

  KChar valueName[32];
  KChar value[64];
  strcpy(valueName, "scrlPos");

  strcpy(value, Tcl_GetVar(mgr->getInterp(), valueName, TCL_GLOBAL_ONLY));

  return atof(value);
}

void KonoeTclText::printText()
{
  KChar cmd[256];
  sprintf(cmd, "exec lpr << [ %s get 1.0 end ]", path);
  configNoUpdate(cmd);
}

KInt KonoeTclText::search(KChar* search_string)
{
  KChar cmd[256];
  KChar valuename[8];
  KChar charvalue[8];
  KChar strvalue[256];
  KChar character[256];
  KInt line = 1;
  KInt line_number = 0;      
  strcpy(valuename, "numline");
  strcpy(strvalue, "string");
      
  sprintf(cmd, "global %s", valuename);
  configNoUpdate(cmd); 
  sprintf(cmd, "global %s", strvalue);
  configNoUpdate(cmd); 
  sprintf(cmd, "set numline [%s index end]", path);
  configNoUpdate(cmd);
  strcpy(charvalue,Tcl_GetVar(mgr->getInterp(), valuename, TCL_GLOBAL_ONLY));
  KDouble index_end = atoi(charvalue);
  for (line = 0; line < index_end; line++) {
  sprintf(cmd, "set string [%s get %d.0 \"%d.0 lineend\"]", path, line, line);
  configNoUpdate(cmd);
  strcpy(character,Tcl_GetVar(mgr->getInterp(), strvalue, TCL_GLOBAL_ONLY));
  if(strstr(character,search_string)!=NULL){
  line_number = line; 
  break;
  }
  }
  return line_number;
}
