//=====================================================================
/*

    KonoeTclUIPrimitive.hh

    Version 0.0.1   05-Aug-1999 T. Hosoi
    Version 0.5.0   28-Sep-1999 T. Hosoi
    Version 0.6.0   01-Feb-2000 T. Hosoi
    Version 0.6.1   05-Apr-2000 T. Hosoi

*/
//====================================================================

#ifndef __KONOETCLUIPRIMITIVE_HH
#define __KONOETCLUIPRIMITIVE_HH

#include <ui/KonoeTclCore.hh>
#include <ui/KonoeTclWindow.hh>
#include <ui/KonoeTclDimension.hh>
#include <iostream.h>
#include <X11/Xutil.h>

class KonoeTclEventProcessor;
class KonoeTclListener;
class KonoeTclMouseListener;
class KonoeTclMouseMotionListener;
class KonoeTclKeyListener;
class KonoeTclWindowListener;

enum {KFlat, KGroove, KRaised, KRidge, KSolid, KSunken};
enum {KPutTop, KPutBottom, KPutLeft, KPutRight, KPutCenter};
enum KScrollType {KBoth=0, KHorizontal, KVertical};
enum KAnchor {KN=0, KNE, KNW, KS, KSE, KSW, KE, KW, KC};

class KonoeTclUIPrimitive : virtual public KonoeTclCore{
  public:
    KonoeTclUIPrimitive();
    virtual ~KonoeTclUIPrimitive();

  protected:
    enum widgetType { Button=0, Canvas=1, CheckButton, Entry, Label, ListBox,
                      Menu, MenuButton, Message, RadioButton, Slider, Text };

  public:
    void mouseEvent(KonoeTclUIPrimitive* wid, XEvent* xe);
    void mouseMotionEvent(KonoeTclUIPrimitive* wid, XEvent* xe);
    void keyEvent(KonoeTclUIPrimitive* wid, XEvent* xe);
    void windowEvent(KonoeTclUIPrimitive* wid, XEvent* xe);

    void addMouseListener( KonoeTclMouseListener* l );
    void addMouseMotionListener( KonoeTclMouseMotionListener* l );
    void addKeyListener( KonoeTclKeyListener* l );
    void addWindowListener( KonoeTclWindowListener* l );

  public:
    KInt getWidgetType() const { return t; }
    void setWidgetType( widgetType i ) { t = i; }
    KonoeTclDimension getScreenSize();

//    void setEventProcessor( KonoeTclEventProcessor* ep );
    void timerUI();
    void timerUI(KonoeTclUIPrimitive* uip);

    void fillX();
    void fillY();
    void fillBoth();
    void setExpand();
    void setRelief(KInt reliefType);
    void resetRelief();
    void setBorderWidth(KInt bord);
    void setSide(KInt side);
    void setIpadx(KInt amount);
    void setIpady(KInt amount);
    void setPadx(KInt amount);
    void setPady(KInt amount);
    void setAnchor(KAnchor anchorType);
    virtual void setWidth(KInt w);
    virtual void setHeight(KInt h);
    
    virtual void setForget();

  public:
    void setGridColumn(KInt n);
    void setGridColumnspan(KInt n);
    void setGridIn(KChar* pPath);
    void setGridIpadx(KInt amount);
    void setGridIpady(KInt amount);
    void setGridPadx(KInt amount);
    void setGridPady(KInt amount);
    void setGridRow(KInt n);
    void setGridRowspan(KInt n);
    void setGridSticky(KChar* style);

    void setGridColMinsize(KInt id, KInt value);
    void setGridColWeight(KInt id, KInt value);
    void setGridColPad(KInt id, KInt value);
    void setGridRowMinsize(KInt id, KInt value);
    void setGridRowWeight(KInt id, KInt value);
    void setGridRowPad(KInt id, KInt value);

    void setGridForget();
    KonoeTclDimension getGridLocation(KInt x, KInt y);
    KonoeTclDimension getGridSize();

    void setFont(KChar* name, KInt size, KInt weight, KInt slant,
                 KInt underline, KInt overstrike);
    void getAllFontName();
    void setCursor(KChar* cursorname);
    // Cursor Name
    // arrow based_arrow_down based_arrow_up boat bogosity bottom_left_corner
    // bottom_right_corner bottom_side bottom_tee box_spiral center_ptr
    // circle clock cofee_mug cross cross_reverse crosshair diamond_cross
    // dot dotbox double_arrow draft_large draft_small draped_box exchange
    // fluer gobbler gumby hand1 hand2 heart icon iron_cross left_ptr
    // left_side left_tee leftbutton ll_angle lr_angle man middlebutton
    // mouse pencil pirate plus question right_ptr right_side right_tee
    // rightbutton rtl_logo sailboat sb_down_arrow sb_h_double_arrow
    // sb_left_arrow sb_right_arrow sb_up_arrow sb_v_double_arrow shuttle
    // sizing spider spraycan star target tcross top_left_arrow top_left_corner
    // top_right_corner top_side top_tee trek ul_angle umbrella ur_angle
    // watch X_cursor xterm
    //

  protected:
    widgetType t;
    KInt reliefType;
    KonoeTclDimension screenSize;
//    KonoeTclEventProcessor* kEventProcessor;
    KonoeTclDimension dim;

    void* listener;
    KBool isEnable;
};

#endif
